/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCApplication.h
// Interface definition of the PnP TerrainCreator application object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	05. October 2004
// ----------------------------------------------------------------------------------
// change history:
//  24. Nov. 2004 (Ralf Westphal)
//		Added new methods for progress bar
//  15. Jun. 2005 (Ralf Westphal)
//		Added default storage folder parameters
//  26. Mai  2007 (Ralf Westphal)
//		Made most methods const and added AddLogText() method.
//  30. Mai  2007 (Ralf Westphal)
//		Restructured applications progress bar and made strings const
//		(sorry for breaking compatibility)
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIAPPLICATION_H_INCLUDED)
#define PNPTC_CIAPPLICATION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"



/// @brief Main application access class.
///
/// Access to the PnPTC application to update the progress
/// bar or to browse for files/folders.
class PNPTCPLUGIN_CALLBACK CIPnPTCApplication01
{
public:

	// --------------------------------------------------------------------------------------
	// methods to browse for a file/folder
	// --------------------------------------------------------------------------------------

	/// @brief Browse for a directory.
	///
	/// Shows a dialog where the user can browse for a folder
	/// to save anything to.
	/// @param cDefaultFolder The folder to be initially opened as default storage folder.
	/// @return Returns a pointer to a null terminated string containing
	/// the folder selected by the user. In case of an error or if no folder
	/// has been selected <i>NULL</i> is returned.
	virtual const char*		SaveFolderDlg(const char* cDefaultFolder=NULL) const=0;

	/// @brief Browse for a file.
	///
	/// Shows a dialog where the user can browse for a file
	/// to save anything to.
	/// @param cFileType Textual desription of the file type to browse for.
	/// @param cFileExt File extension of the file type to browse for.
	/// @param cDefaultFolder The folder to be initially opened as default storage folder.
	/// @return Returns a pointer to a null terminated string containing
	/// the file selected by the user. In case of an error or if no file
	/// has been selected <i>NULL</i> is returned.
	virtual const char*		SaveFileDlg(const char* cFileType,const char* cFileExt,const char* cDefaultFolder=NULL) const=0;

	/// @brief Browse for a file.
	///
	/// Shows a dialog where the user can browse for a file
	/// to load anything from.
	/// @param cFileType Textual desription of the file type to browse for.
	/// Multiple file types can be separated with a ;
	/// @param cFileExt File extension of the file type to browse for.
	/// Multiple file extensions can be separated with a ;<BR>
	/// The number of file types and file extensions passed to this method
	/// should match.
	/// @param cDefaultFolder The folder to be initially opened as default loading folder.
	/// @return Returns a pointer to a null terminated string containing
	/// the file selected by the user. In case of an error or if no file
	/// has been selected <i>NULL</i> is returned.
	virtual const char*		OpenFileDlg(const char* cFileType,const char* cFileExt,const char* cDefaultFolder=NULL) const=0;


	// --------------------------------------------------------------------------------------
	// methods to display/update progress bar
	// --------------------------------------------------------------------------------------

	/// @brief Displays the application's progress bar.
	///
	/// Displays the application's progress bar.
	/// @param cTaskName Name of the task to be displayed for this progress bar.
	/// @param nIncrements Number of increments to be displayed between 0% and 100%.
	/// @return Returns the task ID, by which this progress bar instance can be accessed.
	virtual unsigned int	DisplayProgressBar(const char* cTaskName,unsigned long nIncrements=100) const=0;

	/// @brief Sets the current value of the application's progress bar.
	///
	/// Updates the application's progress bar for a given task ID
	/// to the specified percentage value.
	/// @param nTaskID The task ID (as returned by DisplayProgressBar()) for which 
	/// the progress bar should be set.
	/// @param nPercentage Percentage [0..100] of the progress bar to be displayed.
	/// @param cText (optional) Additional text to be display in the progress window.
	virtual void			SetProgressBarPos(unsigned int nTaskID,unsigned int nPercentage,const char* cText=NULL) const=0;

	/// @brief Increments the progress bar.
	///
	/// The progress bar is incremented by one position. The progress bar can
	/// be incremented until the number of increments set by the DisplayProgressBar()
	/// method are reached. Further incrementing has no affect.
	/// @param nTaskID The task ID (as returned by DisplayProgressBar()) for which 
	/// the progress bar should be set.
	/// @param cText (optional) Additional text to be display in the progress window.
	virtual void			IncrementProgressBarPos(unsigned int nTaskID,const char* cText=NULL) const=0;

	/// @brief Hides the application's progress bar.
	///
	/// Hides the application's progress bar.
	/// @param nTaskID The task ID (as returned by DisplayProgressBar()) for which 
	/// the progress bar should be hidden.
	virtual void			HideProgressBar(unsigned int nTaskID) const=0;


	// --------------------------------------------------------------------------------------
	// additional methods
	// --------------------------------------------------------------------------------------

	/// @brief Returns the rendering device used by the application.
	///
	/// Returns a pointer to the rendering device, which is currently
	/// used by the application object.
	/// @return Returns a pointer to an &quot;IDirect3DDevice9&quot; object.
	virtual void*			GetRenderDevice() const=0;


	// --------------------------------------------------------------------------------------
	// logging methods
	// --------------------------------------------------------------------------------------

	/// @brief Adds an entry to the log file.
	/// 
	/// Adds an antry to the applications log file.
	/// @param strLogText The log text to be added.
	/// @param strFunction The function from which the text was added.
	/// @param strCategory Log text category (Error, Warning, Info, Debug).
	virtual void			AddLogText(const char* strLogText,const char* strFunction, const char* strCategory) const=0;


}; // class


#endif // !defined(PNPTC_CIAPPLICATION_H_INCLUDED)