/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCBrush.h
// Interface definition of an object containing all heightmap elements
// that are within the current brush.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//	20. Dec. 2004 (Ralf Westphal)
//		Added the methods Inside and DistanceSquare
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIBRUSH_H_INCLUDED)
#define PNPTC_CIBRUSH_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


/// @brief Contains the currently selected elements of a terrain.
///
/// An instance of this interface is passed to your
/// plugin to give you information about the terrain
/// elements that have to be affected by the call of
/// your plugin.<br>
/// It contains all terrain elements currently selected
/// by the brush used by the user and some other informations
/// like the bounding box extends of the brush.<br>
/// The elements are always ordered with their y and x index.
class PNPTCPLUGIN_CALLBACK CIPnPTCBrush01
{
public:

	
	/// @brief Returns number of brush elements.
	/// 
	/// Returns the number of elements contained in the brush.
	/// @return Number of terrain elements in the brush.
	virtual int					GetNumberOfElements()=0;


	/// @brief Returns one element of the brush.
	/// 
	/// @param iIndex Element index [0..GetNumberOfElements()-1] to be returned.
	/// @param posReturn The returned brush element describing the heightmap position.
	/// @return <i>true</i> if the element could be returned, <i>false</i> if not.
	virtual bool				GetElement(int iIndex,PnPTCHMapPosition* posReturn)=0;


	/// @brief Returns brush's minimum.
	/// 
	/// Returns the minimum heightmap position of the brush's bounding box.
	/// @return Minimum position of brush's bounding box.
	virtual PnPTCHMapPosition	GetMinPosition()=0;


	/// @brief Returns brush's maximum.
	/// 
	/// Returns the maximum heightmap position of the brush's bounding box.
	/// @return Maximum position of brush's bounding box.
	virtual PnPTCHMapPosition	GetMaxPosition()=0;


	/// @brief Tests a heightmap element if it is inside the brush.
	///
	/// Tests whether or not a given heightmap element is inside the brush.
	/// @param pos Heightmap element to be tested.
	/// @return Returns <i>true</i> if the heightmap element is within the brush,
	/// otherwise <i>false</i>.
	virtual bool				Inside(PnPTCHMapPosition& pos)=0;


	/// @brief Returns the distance of one heightmap element to the brush.
	///
	/// Computes and returnes the distance of one heightmap element
	/// to the brush. For the computation the brush elemet nearest to
	/// the tested heightmap element will be used.
	/// For performance reasons the root value is not computed; 
	/// so the square value of the original distance is returned.
	/// @param posTest Heightmap element that's distance has to be computed.
	/// @param posBrushElement (output) Heightmap element within the brush that
	/// was nearest one to the tested element. This parameter can be set to
	/// <i>NULL</i> if the brush element is not required.
	/// @return Returns the squared distance of the given heightmap element
	/// to the brush. The returned distance is normalized to the size of one
	/// heightmap element and therefore not necessarily metrical. 
	/// For heightmap elements within the brush <i>0</i> is returned.
	virtual float				DistanceSquare(PnPTCHMapPosition& posTest,PnPTCHMapPosition* posBrushElement)=0;

}; // class



#endif // !defined(PNPTC_CIBRUSH_H_INCLUDED)