/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCHeightmap.h
// Interface definition for heightmap access.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. July. 2004
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIHEIGHTMAP_H_INCLUDED)
#define PNPTC_CIHEIGHTMAP_H_INCLUDED
#pragma once

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


/// @brief Interface to access heightmap data.
///
/// Using this interface it is possible to read and
/// write heightmap data of sector detail heightmaps
/// as well as of the terrain's definition heightmap.<br>
/// <b>Sector's detail map:</b><br>
/// When applied for a sector's detail map the indices
/// of accessible heightmap elements might exceed
/// the sector's border size, as it is possible to 
/// access up to one surrounding sector in every
/// direction from the current sector. In this case
/// the minimum indices can be less than zero, as the
/// indices of the current sector always start at (0,0).<br>
/// <b>Terrain's definition map:</b><br>
/// The indices returned by the methods GetMinXIndex(),
/// GetMaxXIndex(), GetMinYIndex() and GetMinYIndex() correspond
/// to the heightmap elements of the sector definitions 
/// currently loaded into memory. When accessing heightmap 
/// elements outside this range the corresponding sector 
/// definitions will be automatically reloaded. So the
/// whole terrain can be accessed.
class PNPTCPLUGIN_CALLBACK CIPnPTCHeightmap01
{
public:

	/// @brief Returns the minimum x-index.
	///
	/// This method returns the minimum x-index of
	/// accessible heightmap elements.<br>
	/// <b>This index might be negativ</b> when
	/// the heightmap object represents a sector's
	/// detail heightmap and surrounding sectors
	/// can be accessed from it.
	/// @return Minimum heightmap x-index.
	virtual int		GetMinXIndex()=0;

	/// @brief Returns the minimum y-index.
	///
	/// This method returns the minimum y-index of
	/// accessible heightmap elements.<br>
	/// <b>This index might be negativ</b> when
	/// the heightmap object represents a sector's
	/// detail heightmap and surrounding sectors
	/// can be accessed from it.
	/// @return Minimum heightmap y-index.
	virtual	int		GetMinYIndex()=0;

	/// @brief Returns the maximum x-index.
	///
	/// This method returns the maximum x-index of
	/// accessible heightmap elements.
	/// @return Maximum heightmap x-index.
	virtual int		GetMaxXIndex()=0;

	/// @brief Returns the maximum y-index.
	///
	/// This method returns the maximum y-index of
	/// accessible heightmap elements.
	/// @return Maximum heightmap y-index.
	virtual int		GetMaxYIndex()=0;

	/// @brief Returns the height at a given position of the heightmap.
	///
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @return Returns the height at this position.<br>
	/// If the heightmap is not yet initialized or the x 
	/// or y parameters are outside the range of accessible
	/// heightmap elements <i>-1</i> is returned.
	virtual float	GetHeight(int iX,int iY)=0;

	/// @brief Sets the height at a given position of the heightmap.
	/// 
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @param fHeight New height to be set at this position.
	/// @return Returns <i>true</i> if the new height could
	/// be set successfully.<br>Returns <i>false</i> in case of
	/// an error. For example if the values of x or y
	/// are out of range or the heightmap element is locked.
	virtual bool	SetHeight(int iX,int iY,float fHeight)=0;

	/// @brief Returns the horizontal scaling factor of the heightmap.
	/// 
	/// This method returns the scaling factor for the single heightmap
	/// elements (resp. their distances) with respect to the height values.
	/// @return Returns the horizontal scaling factor.
	virtual float	GetHorizontalScaleFactor()=0;
};

#pragma pack(pop)

#endif // !defined(PNPTC_CIHEIGHTMAP_H_INCLUDED)