/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCObject.h
// Interface definition to access an object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. January 2005
// ----------------------------------------------------------------------------------
// change history:
//  19. Feb. 2007 (Ralf Westphal)
//		Made methods const and added GetID.
//  26. Mai. 2007 (Ralf Westphal)
//		Added GetObjectCategory() and added format, scale, uprotation, and
//		flipaxis to the save-to methods. Renamed GetName() to GetObjectName()
//  08. Sep. 2007 (Ralf Westphal)
//		Added GetObjectScale()
//  21. Sep. 2007 (Ralf Westphal)
//		Added GetComments()
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIOBJECT_H_INCLUDED)
#define PNPTC_CIOBJECT_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcpropertyvalue.h"

#pragma pack(push)
#pragma pack(1)



/// @brief Callback accessor class for objects.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to an object.
class PNPTCPLUGIN_CALLBACK CIPnPTCObject01
{
public:

	// --------------------------------------------------------------------------------------------
	// base object and file methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the internal name of the object.
	///
	/// Returns the internal name of the object as it is used
	/// within the main program.
	/// @return Returns the internal name of the object.
	virtual const char*					GetObjectName() const=0;

	/// @brief Returns the category of the object.
	///
	/// Returns the category of the object as it is used
	/// within the main program.
	/// @return Returns the category of the object.
	virtual const char*					GetObjectCategory() const=0;

	/// @brief Returns the ID/index of the object.
	///
	/// Returns the ID/index of the object as it is used
	/// within the main program and object manager plugin.
	/// @return Returns the ID/index of the object.
	virtual WORD						GetID() const=0;

	/// @brief Returns the scale factor for this object.
	///
	/// Returns the scale factor for this object.
	/// @return Returns the scale factor for this object.
	virtual float						GetObjectScale() const=0;

	/// @brief Returns a global object comment.
	///
	/// Returns a comment text which was specified for
	/// for this type of object globally in the object manager.
	/// @return Comment text for this object.
	virtual const char*					GetComments() const=0;

	/// @brief Saves the object to a file.
	///
	/// Saves the object to a file.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the object should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool						SaveToFile(const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL) const=0;
	
	/// @brief Saves the object to memory.
	///
	/// Saves the object file to the memory.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool						SaveToMem(const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0) const=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this object.
	/// @return Returns the object's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class


#pragma pack(pop)


#endif // !defined(PNPTC_CIOBJECT_H_INCLUDED)