/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCPropertyValue.h
// Interface definition for user defined property definition access.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	05. Feb. 2008
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIPROPERTYVALUE_H_INCLUDED)
#define PNPTC_CIPROPERTYVALUE_H_INCLUDED


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


/// @brief Interface to access property values.
///
/// An interface class via which user defined 
/// property values can be accessed. One property
/// value is characterized by its name and data type.
/// Unique identification of such properties is 
/// possible via a GUID.
class PNPTCPLUGIN_CALLBACK CIPnPTCPropertyValue01
{

public:

	// ------------------------------------------------------------
	// public data types
	// ------------------------------------------------------------

	/// The available property value types.
	typedef enum VALUE_TYPE
	{
		vtUnknown		= -1,		//!< The property type is unknown => this indicates an error.
		vtCategory		= 0,		//!< The property has no value, it is just a categorization property.
		vtParent		= 1,		//!< The property has no value, it is just the parent property. The value is held by the childs.
		vtString		= 2,		//!< The property holds a string value.
		vtInteger		= 3,		//!< The property holds an integer value.
		vtFloat			= 4,		//!< The property holds a real value.
		vtBoolean		= 5,		//!< The property holds a boolean value.
		vtList			= 6,		//!< The property holds the selection of a list. The identifier of the selected list entry can be retrieved as integer (GetIntegerValue()), while the selected string can be retrieved by GetStringValue().
		vtNumTypes					//!< The number of available types.
	} VALUE_TYPE;


	// ------------------------------------------------------------
	// access to the property data
	// ------------------------------------------------------------

	/// @brief Returns the name of the property.
	///
	/// Returns the name of the property.
	/// @return The name of the property.
	virtual const char*				GetName() const=0;

	/// @brief Returns the GUID of the property.
	///
	/// Returns the globaly uniquie UID of this property.
	/// @return The GUID of the property.
	virtual GUID					GetGUID() const=0;

	/// @brief Returns the value type of this property.
	///
	/// Returns the value type of this property.
	/// @return The value type of this property.
	virtual VALUE_TYPE				GetType() const=0;

	/// @brief Returns the name of the property.
	///
	/// Returns the string value of this property.
	/// @return The property's string value.
	virtual const char*				GetStringValue() const=0;

	/// @brief Returns the integer value of this property.
	///
	/// Returns the integer value of this property.
	/// @return The property's integer value.
	virtual int						GetIntegerValue() const=0;

	/// @brief Returns the boolean value of this property.
	///
	/// Returns the boolean value of this property.
	/// @return The property's boolean value.
	virtual bool					GetBooleanValue() const=0;

	/// @brief Returns the real value of this property.
	///
	/// Returns the real value of this property.
	/// @return The property's real value.
	virtual float					GetRealValue() const=0;

	/// @brief Sets the string value of this property.
	///
	/// Sets the string value of this property.
	/// @param strValue The property's string value.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the value could be set successfully.
	virtual bool					SetStringValue(const char* strValue)=0;

	/// @brief Sets the integer value of this property.
	///
	/// Sets the integer value of this property.
	/// @param iValue The property's integer value.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the value could be set successfully.
	virtual bool					SetIntegerValue(int iValue)=0;

	/// @brief Sets the boolean value of this property.
	///
	/// Sets the boolean value of this property.
	/// @param bValue The property's boolean value.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the value could be set successfully.
	virtual bool					SetBooleanValue(bool bValue)=0;

	/// @brief Sets the real value of this property.
	///
	/// Sets the real value of this property.
	/// @param fValue The property's real value.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the value could be set successfully.
	virtual bool					SetRealValue(float fValue)=0;


	// ------------------------------------------------------------
	// access to sub properties
	// ------------------------------------------------------------

	/// @brief Returns the number of sub properties.
	///
	/// Returns the number of sub properties of this property.
	/// @return Number of sub properties.
	virtual unsigned int			GetNumSubProperties() const=0;

	/// @brief Returns one sub property.
	///
	/// Returns one sub property of this property.
	/// @param nSubProperty Index of the sub property to be returned
	/// in the range [0..GetNumSubProperties()-1].
	virtual CIPnPTCPropertyValue01*	GetSubProperty(unsigned int nSubProperty) const=0;

}; // class CIPnPTCPropertyValue01




/// @brief A class, which manages all property values assigned to a given object.
///
/// A class, which manages all userdefined property values of a
/// PnP TerrainCreator data object. A property container
/// holds a number of property values as instances of the class
/// CIPnPTCPropertyValue01. The top level properties in the
/// hierarchy of all property values of the container can
/// be accessed directly via this interface.
class PNPTCPLUGIN_CALLBACK CIPnPTCPropertyContainer01
{

public:

	// ------------------------------------------------------------
	// access to top level properties in this container
	// ------------------------------------------------------------

	/// @brief Returns the number of properties.
	///
	/// Returns the number of properties of this property container.
	/// @return Number of properties.
	virtual unsigned int			GetNumProperties() const=0;

	/// @brief Returns one property.
	///
	/// Returns one property of this property container.
	/// @param nProperty Index of the property to be returned
	/// in the range [0..GetNumProperties()-1].
	/// @return Returns the property for this index if successful
	/// or <i>NULL</i> in case of an error.
	virtual CIPnPTCPropertyValue01*	GetProperty(unsigned int nProperty) const=0;

}; // class CIPnPTCPropertyContainer01


#pragma pack(pop)


#endif // !defined(PNPTC_CIPROPERTYVALUE_H_INCLUDED)
