/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCRenderSettings.h
// Interface definition of a terrain render settings object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	24. Apr. 2006
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIRENDERSETTINGS_H_INCLUDED)
#define PNPTC_CIRENDERSETTINGS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"

#pragma pack(push)
#pragma pack(1)



/// @brief Terrain render settings access class.
///
/// Using this class the basic render settings
/// for this terrain and installation can be 
/// accessed.
class PNPTCPLUGIN_CALLBACK CIPnPTCRenderSettings01
{
public:

	/// @brief Returns whether or not lighting is active.
	///
	/// Returns whether or not the dynamic terrain lighting
	/// is activated
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the dynamic terrain lighting is activated.
	virtual bool					GetDynamicLightingActivated()=0;

	/// @brief Returns day light color.
	///
	/// Returns the color which is used by the light source
	/// during day time.
	/// @return Returns the color of the day's light source.
	virtual PNPTC_RGB				GetDayLightColor()=0;

	/// @brief Returns day light ambient color.
	///
	/// Returns the ambient color which is used by the light source
	/// during day time.
	/// @return Returns the ambient color of the day's light source.
	virtual PNPTC_RGB				GetDayLightAmbientColor()=0;

	/// @brief Returns night light color.
	///
	/// Returns the color which is used by the light source
	/// during night time.
	/// @return Returns the color of the night's light source.
	virtual PNPTC_RGB				GetNightLightColor()=0;

	/// @brief Returns night light ambient color.
	///
	/// Returns the ambient color which is used by the light source
	/// during night time.
	/// @return Returns the ambient color of the night's light source.
	virtual PNPTC_RGB				GetNightLightAmbientColor()=0;

	/// @brief Returns the maximum viewing distance for the terrain.
	///
	/// Returns the maximum viewing distance for terrain surface
	/// rendering in meters.
	/// @return Terrain viewing distance in meters.
	virtual float					GetMaxViewDistanceTerrain()=0;

	/// @brief Returns the maximum viewing distance for the object placements.
	///
	/// Returns the maximum viewing distance for object placements
	/// rendering in meters.
	/// @return Object placements viewing distance in meters.
	virtual float					GetMaxViewDistanceObjects()=0;

	/// @brief Returns the maximum viewing distance for the tree placements.
	///
	/// Returns the maximum viewing distance for tree placements
	/// rendering in meters.
	/// @return Tree placements viewing distance in meters.
	virtual float					GetMaxViewDistanceTrees()=0;

	/// @brief Returns the maximum viewing distance for the grasses.
	///
	/// Returns the maximum viewing distance for geass
	/// rendering in meters.
	/// @return Grasses viewing distance in meters.
	virtual float					GetMaxViewDistanceGrasses()=0;

	/// @brief Returns the maximum number of object placements to be rendered.
	///
	/// Returns the maximum number of object placements to be rendered.
	/// @return Maximum number of object placements to be rendered.
	/// Returns <i>-1</i> if the number is not limited.
	virtual int						GetMaxNumRenderedObjects()=0;

	/// @brief Returns the maximum number of tree placements to be rendered.
	///
	/// Returns the maximum number of tree placements to be rendered.
	/// @return Maximum number of tree placements to be rendered.
	/// Returns <i>-1</i> if the number is not limited.
	virtual int						GetMaxNumRenderedTrees()=0;

	/// @brief Returns the maximum number of grasses to be rendered.
	///
	/// Returns the maximum number of grasses to be rendered.
	/// @return Maximum number of grasses to be rendered.
	/// Returns <i>-1</i> if the number is not limited.
	virtual int						GetMaxNumRenderedGrasses()=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_CIRENDERSETTINGS_H_INCLUDED)