/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSector.h
// Interface definition of a sector object enabling access
// to the sectors data fields.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//	24. Nov. 2004 (Ralf Westphal)
//		Added vegetation
//  24. Nov. 2004 (Ralf Westphal)
//		Added GetApplicationObject(), GetPosX(), GetPosY() and GetSectorDefinition()
//	24. Feb. 2005 (Ralf Westphal)
//		Added GetInterpolatedHeight(), GetNormal() and GetInterpolatedNormal()
//	25. Mar. 2005 (Ralf Westphal)
//		Added GetQuadOrientation
//  20. Jun. 2005 (Ralf Westphal)
//		Added GetNeighbourSector
//  23. Aug. 2005 (Ralf Westphal)
//		Added GetLightMap
//  28. Mai. 2006 (Ralf Westphal)
//		Added GetQuadVisibility
//  03. Sep. 2006 (Ralf Westphal)
//      Added GetSectorWater()
//  29. Dec. 2007 (Ralf Westphal)
//		Added SetQuadVisibility() and SetQuadOrientation()
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTOR_H_INCLUDED)
#define PNPTC_CISECTOR_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/// includes
#include "./cipnptcheightmap.h"
#include "./cipnptcsectortextures.h"
#include "./cipnptcsectorobjects.h"
#include "./cipnptcsectorsounds.h"
#include "./cipnptcsectorvegetation.h"
#include "./cipnptcapplication.h"
#include "./cipnptclightmap.h"
#include "./cipnptcsectorwater.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


/// indices to access neighbour sectors
typedef enum NEIGHBOUR
{
	NB_TOPLEFT		= 0,
	NB_TOP			= 1,
	NB_TOPRIGHT		= 2,
	NB_RIGHT		= 3,
	NB_BOTTOMRIGHT	= 4,
	NB_BOTTOM		= 5,
	NB_BOTTOMLEFT	= 6,
	NB_LEFT			= 7
} NEIGHBOUR;



// forward definition
class CIPnPTCSectorDefinition01;

/// @brief Callback accessor class for sector data.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access (read and write) to
/// the sector's data fields.
class PNPTCPLUGIN_CALLBACK CIPnPTCSector01 : public CIPnPTCHeightmap01
{
public:

	/// @brief Returns the sector's border size.
	///
	/// <b>Remark:</b> The sectors are square!
	/// @return Returns the sector's border size.
	virtual unsigned int				GetBorderSize()=0;
	
	/// @brief Stores the sector's data
	///
	/// After successfull execution of generator plugins
	/// you have to save the sector manually.
	/// @return Returns whether or not the sector could
	/// be saved successfully.
	virtual bool						Save()=0;

	/// @brief Returns the color for a given height.
	///
	/// The color code for a given height is computed
	/// and returned with respect to the color scaling
	/// of the sector.
	/// @param fHeight Height to be converted into a color.
	/// @return Returns the color value for the given height.
	virtual PNPTC_RGB					GetHeightColor(float fHeight)=0;

	/// @brief Returns the minimum current height of the sector.
	///
	/// @return Minimum height of all sector heightmap elements.
	virtual float						GetMinHeight()=0;

	/// @brief Returns the maximum current height of the sector.
	///
	/// @return Maximum height of all sector heightmap elements.
	virtual float						GetMaxHeight()=0;

	/// @brief Returns the sector texture layer.
	///
	/// @return Reference to texture layer object.
	virtual	CIPnPTCSectorTextures01*	GetSectorTextures()=0;

	/// @brief Returns the sector's object placements.
	///
	/// @return Reference to the sector's object placements.
	virtual	CIPnPTCSectorObjects01*		GetSectorObjects()=0;

	/// @brief Returns the sector's sound placements.
	///
	/// @return Reference to the sector's sound placements.
	virtual	CIPnPTCSectorSounds01*		GetSectorSounds()=0;

	/// @brief Returns the sector's vegetation data.
	///
	/// @return Reference to the sector's vegetation object.
	virtual	CIPnPTCSectorVegetation01*	GetSectorVegetation()=0;

	/// @brief Returns the sector's water data.
	///
	/// @return Reference to the sector's water object.
	virtual	CIPnPTCSectorWater01*		GetSectorWater()=0;

	/// @brief Returns the PnP TC application object.
	///
	/// @return Returns a reference to the application object.
	virtual	CIPnPTCApplication01*		GetApplicationObject()=0;

	/// @brief Returns the x position of this sector.
	///
	/// @return Returns the x position of this sector on the terrain
	/// in the range of [0..CIPnPTCTerrain01::GetNumSectorsX()-1].
	virtual int							GetPosX()=0;

	/// @brief Returns the y position of this sector.
	///
	/// @return Returns the y position of this sector on the terrain
	/// in the range of [0..CIPnPTCTerrain01::GetNumSectorsY()-1].
	virtual int							GetPosY()=0;

	/// @brief Returns the sector definition map.
	///
	/// @return Returns a pointer to the sector's definition
	/// map object.
	virtual CIPnPTCSectorDefinition01*	GetSectorDefinition()=0;

	/// @brief Returns the height at a given coordinate.
	///
	/// Computes the height at a given coordinate (not heightmap element)
	/// by interpolating between the neighboring heightmap elements.
	/// @param fX X coordinate in meters.
	/// @param fY Y coordinate in meters.
	/// @return Returns the interpolated height at the given coordinate.
	virtual	float						GetInterpolatedHeight(float fX,float fY)=0;
	
	/// @brief Returns the surface normal of a heightmap element.
	///
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @return Returns the surface normal of the given heightmap element.
	virtual	PnPTC3DVec					GetNormal(int iX,int iY)=0;

	/// @brief Returns the surface normal at a given coordinate.
	///
	/// Computes the surface normal at a given coordinate (not heightmap element)
	/// by interpolating between the neighboring heightmap elements.
	/// @param fX X coordinate in meters.
	/// @param fY Y coordinate in meters.
	/// @return Returns the surface normal at the given coordinate.
	virtual	PnPTC3DVec					GetInterpolatedNormal(float fX,float fY)=0;

	/// @brief Returns the orientation of one terrain quad.
	///
	/// Returns the orientation of on terrain sector quad. The orientation
	/// of a quad determines how it is divided into two triangles.
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @return Quad orientation. Possible values are PNPTC_QUADOR_BL2TR and
	/// PNPTC_QUADOR_TL2BR. They define the direction of the diagonal, dividing
	/// the quad into triangles.
	virtual int							GetQuadOrientation(int iX,int iY)=0;

	/// @brief Sets the orientation of one terrain quad.
	///
	/// Sets the orientation of on terrain sector quad. The orientation
	/// of a quad determines how it is divided into two triangles.
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @param iOrientation Quad orientation. Possible values are PNPTC_QUADOR_BL2TR and
	/// PNPTC_QUADOR_TL2BR. They define the direction of the diagonal, dividing
	/// the quad into triangles.
	virtual void						SetQuadOrientation(int iX,int iY,int iOrientation)=0;

	/// @brief Returns one neighbour sector of this sector.
	///
	/// @param enNeighbour Index of the neighbour sector to be returned.
	/// @return Returns a reference to the neighbour sector. Returns <i>NULL</i>
	/// if the sector does not exist or if it is not opened.
	virtual CIPnPTCSector01*			GetNeighbourSector(NEIGHBOUR enNeighbour)=0;

	/// @brief Returns a sector's light map.
	///
	/// Returns an object pointer for the light map textures of this sector.
	/// @param strLightMapName Name of the light map to be returned.
	/// @return Returns a pointer to the requested light map object if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCLightMap01*			GetLightMap(const char* strLightMapName)=0;

	/// @brief Returns the visibility of one terrain quad.
	///
	/// Returns the visibility of on terrain sector quad. 
	/// @param iX x-position on the heightmap within the range
	/// [GetMinXIndex()..GetMaxXIndex()].
	/// @param iY y-position on the heightmap within the range
	/// [GetMinYIndex()..GetMaxYIndex()].
	/// @return Returns the terrain quad's visibility. <i>true</i>
	/// if the quad is visible, otherwise <i>false</i>.
	virtual bool						GetQuadVisibility(int iX, int iY)=0;

	/// @brief Sets the visibility of one terrain quad.
	///
	/// Sets the visibility state of quad's triangles.
	/// @param iX x coordinate of the quad to be set.
	/// @param iY y coordinate of the quad to be set.
	/// @param bVisible the visibility state of the triangle.
	virtual void						SetQuadVisibility(int iX, int iY,bool bVisible)=0;

	/// @brief Opens surrounding sectors.
	///
	/// This method opens all neighbouring sectors. If the sector
	/// object on which this method is called was not opened with 
	/// the GetSectorDetail method, the method call will fail. So,
	/// this method will not succeed on sectors which have been
	/// retrieved by the GetNeighbourSector method.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the neighbouring sectors could be opened.
	virtual bool						OpenSurroundingSectors()=0;

	/// @brief Opens one neighbouring sector.
	///
	/// This method opens one neighbouring sector. If the sector
	/// object on which this method is called was not opened with 
	/// the GetSectorDetail method, the method call will fail. So,
	/// this method will not succeed on sectors which have been
	/// retrieved by the GetNeighbourSector method.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the neighbouring sector could be opened.
	virtual bool						OpenSurroundingSector(NEIGHBOUR enNeighbour)=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this sector.
	/// @return Returns the sector's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class CIPnPTCSector01


#pragma pack(pop)


#endif // !defined(PNPTC_CISECTOR_H_INCLUDED)