/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorDefinition.h
// Interface definition of a sector definition object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	01. Jun. 2004
// ----------------------------------------------------------------------------------
// change history:
//  24. Nov. 2004 (Ralf Westphal)
//		Added GetApplicationObject()
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2006 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTORDEFINITION_H_INCLUDED)
#define PNPTC_CISECTORDEFINITION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"

// includes
#include "./CIPnPTCSector.h"
#include "./cipnptcapplication.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


/// @brief Sector definition interface.
///
/// Interface class to get acces to the definitions
/// of one terrain sector.
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorDefinition01
{
public:
	/// @brief Returns sector definitions bordersize.
	///
	/// Return the sectors definition heightmap bordersize.<br>
	/// <b>Remark:</b> The sectors are square!
	/// @return Returns the sectors definition bordersize.
	virtual unsigned int				GetSectorDefinitionBorderSize()=0;

	/// @brief Returns the height at one position
	///
	/// Returns the height at a given position of the
	/// sector's definition heightmap.
	/// @param nX x-position on the sectors map starting at 0. 
	/// @param nY y-position on the sectors map starting at 0. 
	/// @return Returns the height at this position.<br>
	/// If the heightmap is not yet initialized or the x 
	/// or y parameters are outside range <i>-1</i> is returned.
	virtual float						GetHeight(unsigned int nX,unsigned int nY)=0;

	/// @brief Sets the height at one position.
	///
	/// Sets the height at a given position of the
	/// sector's definition heightmap.
	/// @param nX x-position on the sectors map starting at 0. 
	/// @param nY y-position on the sectors map starting at 0. 
	/// @param fHeight New height at this position
	/// @return Returns <i>true</i> if the new height could
	/// be set successfully.<br>Returns <i>false</i> in case of
	/// an error. For example if the values of x or y
	/// are out of range.
	virtual bool						SetHeight(unsigned int nX,unsigned int nY,float fHeight)=0;

	/// @brief Returns the x position of this sector.
	///
	/// @return Returns the x position of this sector on the terrain
	/// in the range of [0..CIPnPTCTerrain01::GetNumSectorsX()-1].
	virtual int							GetPosX()=0;

	/// @brief Returns the y position of this sector.
	///
	/// @return Returns the y position of this sector on the terrain
	/// in the range of [0..CIPnPTCTerrain01::GetNumSectorsY()-1].
	virtual int							GetPosY()=0;

	/// @brief Returns one neighbour sector of this sector.
	///
	/// @param enNeighbour Index of the neighbour sector to be returned.
	/// @return Returns a reference to the neighbour sector. Returns <i>NULL</i>
	/// in case of an error.
	virtual CIPnPTCSectorDefinition01*	GetNeighbourSector(NEIGHBOUR enNeighbour)=0;

	/// @brief Returns the sector's detail.
	///
	/// Returns the sector's detail object.
	/// @return Returns a pointer to the sector object. Returns <i>NULL</i>
	/// in case of an error.<br>
	/// <b>Remarks:</b><br>
	/// This is a temporary pointer: Don't store or delete it, as the
	/// memory will be managed from main application and might therefore
	/// no longer be valid when your plugin is called the next time.<br>
	/// <b>If you make two subsequent calls to GetSector (from different
	/// instances of CIPnPTCSectorDefinition01), the object of the
	/// first call will be destroyed by the second call</b>. So you can only
	/// access one sector at a time. This is done because of the large amount
	/// of memory that will be needed if you would try to open all sectors at
	/// the same time.<br>
	/// If the detail map of a sector does not exist, this method
	/// will create a new detail map and initialize its heightmap
	/// with the default linear interpolator.
	virtual CIPnPTCSector01*			GetSectorDetail()=0;

	/// @brief Returns the existence of a detail map.
	///
	/// Returns whether or not the detail sector map exists.
	/// @return <i>true</i> if the sector's detail map exists, 
	/// <i>false</i> if not.
	virtual bool						HasSectorDetail()=0;

	/// @brief Returns the random heightmap generation parameters.
	/// 
	/// Returns a pointer to a memory area in which the parameters
	/// for the random heightmap generation for a given generator plugin
	/// are stored.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Return parameter pointing to the memory containing the 
	/// generation parameters. If the method failes a NULL pointer is returned.<br>
	/// <b>Remarks</b>: The pointer returned is a temporary pointer. You 
	/// should copy the contents of it directly after the call.
	/// @param dwMemSize Returns the amount of bytes stored in the memory block.
	/// If the method fails <i>0</i> is returned.
	/// @return Returns <i>true</i> if the parameters could be retrieved successfully.
	/// Returns <i>false</i> in case of an error, for example there are no
	/// parameters available for the given plugin.
	virtual bool						GetRandomHeightmapParameters(GUID uidPlugin,void** pMem,DWORD* dwMemSize)=0;

	/// @brief Set the random heightmap generation parameters.
	/// 
	/// Sets the random heightmap generation parameters for a given generator plugin.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Memory block containing the generation parameters.
	/// @param dwMemSize Size of the memory block containing the generation parameters.
	/// @return Returns <i>true</i> if the parameters could be set successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool						SetRandomHeightmapParameters(GUID uidPlugin,void* pMem,DWORD dwMemSize)=0;

	/// @brief Returns the random texturing parameters.
	/// 
	/// Returns a pointer to a memory area in which the parameters
	/// for the random terrain texturing for a given generator plugin
	/// are stored.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Return parameter pointing to the memory containing the 
	/// generation parameters. If the method failes a NULL pointer is returned.<br>
	/// <b>Remarks</b>: The pointer returned is a temporary pointer. You 
	/// should copy the contents of it directly after the call.
	/// @param dwMemSize Returns the amount of bytes stored in the memory block.
	/// If the method fails <i>0</i> is returned.
	/// @return Returns <i>true</i> if the parameters could be retrieved successfully.
	/// Returns <i>false</i> in case of an error, for example there are no
	/// parameters available for the given plugin.
	virtual bool						GetRandomTexturingParameters(GUID uidPlugin,void** pMem,DWORD* dwMemSize)=0;

	/// @brief Set the random texturing parameters.
	/// 
	/// Sets the random terrain texturing parameters for a given generator plugin.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Memory block containing the generation parameters.
	/// @param dwMemSize Size of the memory block containing the generation parameters.
	/// @return Returns <i>true</i> if the parameters could be set successfully.
	/// Returns <i>false</i>in case of an error.
	virtual bool						SetRandomTexturingParameters(GUID uidPlugin,void* pMem,DWORD dwMemSize)=0;

	/// @brief Returns the random vegetation parameters.
	/// 
	/// Returns a pointer to a memory area in which the parameters
	/// for the random terrain vegetation distribution for a given 
	/// generator plugin are stored.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Return parameter pointing to the memory containing the 
	/// generation parameters. If the method failes a NULL pointer is returned.<br>
	/// <b>Remarks</b>: The pointer returned is a temporary pointer. You 
	/// should copy the contents of it directly after the call.
	/// @param dwMemSize Returns the amount of bytes stored in the memory block.
	/// If the method fails <i>0</i> is returned.
	/// @return Returns <i>true</i> if the parameters could be retrieved successfully.
	/// Returns <i>false</i> in case of an error, for example there are no
	/// parameters available for the given plugin.
	virtual bool						GetRandomVegetationParameters(GUID uidPlugin,void** pMem,DWORD* dwMemSize)=0;

	/// @brief Set the random vegetation parameters.
	/// 
	/// Sets the random terrain vegetation distribution parameters for a 
	/// given generator plugin.
	/// @param uidPlugin GUID of the generator plugin.
	/// @param pMem Memory block containing the generation parameters.
	/// @param dwMemSize Size of the memory block containing the generation parameters.
	/// @return Returns <i>true</i> if the parameters could be set successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool						SetRandomVegetationParameters(GUID uidPlugin,void* pMem,DWORD dwMemSize)=0;

	/// @brief Returns the PnP TC application object.
	///
	/// @return Returns a reference to the application object.
	virtual	CIPnPTCApplication01*		GetApplicationObject()=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this sector.
	/// @return Returns the sector's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;


}; // class CIPnPTCSectorDefinition01


#pragma pack(pop)


#endif // !defined(PNPTC_CISECTORDEFINITION_H_INCLUDED)