/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorObjects.h
// Interface definition to access the objects placed on one sector.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	10. October 2004
// ----------------------------------------------------------------------------------
// change history:
//  03. Jan. 2005 (Ralf Westphal)
//		Added methods GetNumUsedObjects, GetUsedObject, GetObject
//  30. Nov. 2005 (Ralf Westphal)
//		Added object placement comments: GetObjectComments
//  26. Mai. 2006 (Ralf Westphal)
//		Added object flipping: GetFlipAxis
//  18. Feb. 2007 (Ralf Westphal)
//		Added GetCaption, GetObjectEngineGUID, and GetObjectCategory and
//		renamed GetObjectComments to GetComments
//  19. Feb. 2007 (Ralf Westphal)
//		Now all member variables can only be returned by getter methods.
//		Changed the retrieval of used objects.
//		(Sorry for breaking compatibility)
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTOROBJECTS_H_INCLUDED)
#define PNPTC_CISECTOROBJECTS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcobject.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


// forward definitions of objects used by CIPnPTCSectorObjects01
class CIPnPTCObjectPlacement01;




// ---------------------------------------------------------------------------------------

/// @brief Callback accessor class for sector object data.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to the objects placed on
/// on sector.
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorObjects01 
{
public:

	// --------------------------------------------------------------------------------------------
	// System/terrain informations
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the GUID of the object rendering engine.
	///
	/// Returns the GUID of the object rendering engine used to place
	/// and render objects.
	/// @return GUID of object rendering engine.
	virtual GUID						GetObjectEngineGUID() const=0;

	/// @brief Returns the global scaling factor.
	///
	/// Returns the scaling factor that is applied to
	/// all object placements.
	/// @return Global object scaling factor.
	virtual float						GetGlobalScale() const=0;

	/// @brief Returns the global up rotation.
	///
	/// Returns the global up rotation that is applied to
	/// all object placements in order to align the object's 
	/// up axis accordingly to the terrain's up axis.
	/// @return Global up rotation as a value defined by UPROTATION_NONE,
	/// UPROTATION_ROTX90, or UPROTATION_ROTX270.
	virtual int							GetGlobalUpRotation() const=0;

	/// @brief Returns the objects' flip axis.
	///
	/// Returns the axis at which all object vertices are flipped in order
	/// to convert between right and left handed coordinate system usage.
	/// @return Global flip axis as a value defined by FLIP_NONE,
	/// FLIP_XAXIS, FLIP_YAXIS, or FLIP_ZAXIS.
	virtual int							GetFlipAxis() const=0;

	/// @brief Returns the number of objects placed on one sector.
	///
	/// Returns the number of objects placed on one sector.
	/// @return Returns the number of objects.
	virtual unsigned int				GetNumObjectPlacements() const=0;
	
	/// @brief Returns one object placement.
	///
	/// Returns one object placement.
	/// @param nIndex Index [0..GetNumObjectPlacements()-1] of the
	/// sector's object placement to be returned.
	/// @return Object placement. In case of an error <i>NULL</i>
	/// is returned.
	virtual CIPnPTCObjectPlacement01*	GetObjectPlacement(unsigned int nIndex) const=0;

	/// @brief Returns the number of different objects used by one sector.
	///
	/// Returns the number of different objects that are currently 
	/// used by one sector.
	/// @return Returns the number of different objects.
	virtual unsigned int				GetNumUsedObjects() const=0;
	
	/// @brief Returns one object which is used by the terrain sector.
	///
	/// Returns one object that is currently used by the terrain sector.
	/// @param nIndex Index [0..GetNumUsedObjects()-1] of the
	/// terrain sector object, that has to be returned.
	/// @return The object used by the terrain sector.
	/// In case of an error <i>NULL</i> is returned.
	virtual CIPnPTCObject01*			GetUsedObject(unsigned int nIndex) const=0;

	/// @brief Returns object.
	///
	/// Returns a reference to an object by which all relevant
	/// object data can be accessed.
	/// @param wID Internal ID of the object to be returned.
	/// @return Pointer to the object or <i>NULL</i> in case of an error.
	virtual CIPnPTCObject01*			GetObject(WORD wID) const=0;

}; // class




// ---------------------------------------------------------------------------------------

/// @brief Object placement data.
///
/// This interface contains all data of one object placement.<br>
/// <b>Remarks:</b> The object rotation is performed by the
/// RPY definition of DirectX: A rotation around the z-axis, followed 
/// by rotation around the x-axis, followed by rotation around the y-axis.
class PNPTCPLUGIN_CALLBACK CIPnPTCObjectPlacement01
{
public:

	// -----------------------------------------------------------
	// public variables and methods
	// -----------------------------------------------------------

	/// @brief Returns the id of the object placement.
	///
	/// Returns the unique identifier of this object placement.
	/// @return ID of the object placement.
	virtual GUID					GetID() const=0;		

	/// @brief Returns the position of the object placement.
	///
	/// Returns the position of this object placement.
	/// @return Position of the object placement.
	virtual const PnPTC3DPosition&	GetPos() const=0;

	/// @brief Returns the scaling of the object placement.
	///
	/// Returns the scaling of this object placement in x direction.
	/// @return Scaling of object placement in x direction.
	virtual float					GetScaleX() const=0;

	/// @brief Returns the scaling of the object placement.
	///
	/// Returns the scaling of this object placement in y direction.
	/// @return Scaling of object placement in y direction.
	virtual float					GetScaleY() const=0;

	/// @brief Returns the scaling of the object placement.
	///
	/// Returns the scaling of this object placement in z direction.
	/// @return Scaling of object placement in z direction.
	virtual float					GetScaleZ() const=0;

	/// @brief Returns the orientation of the object placement.
	///
	/// Returns the rotation of this object placement.
	/// around the x axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of object placement about the x axis.
	/// The angle is given in radians.
	virtual float					GetRotX() const=0;

	/// @brief Returns the orientation of the object placement.
	///
	/// Returns the rotation of this object placement.
	/// around the y axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of object placement about the y axis.
	/// The angle is given in radians.
	virtual float					GetRotY() const=0;

	/// @brief Returns the orientation of the object placement.
	///
	/// Returns the rotation of this object placement.
	/// around the z axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of object placement about the z axis.
	/// The angle is given in radians.
	virtual float					GetRotZ() const=0;

	/// @brief Returns the name/id of the object.
	///
	/// Returns the name/id of the object which is used for
	/// this object placement. This is the ID used by the
	/// object manager to uniquely identify a type of object.
	/// @return A zero terminated string containing the object's name.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*				GetObjectName() const=0;

	/// @brief Returns the category of the object.
	///
	/// Returns the category string of the object which is used for
	/// this object placement. This is the category used by the
	/// object manager. Subcategories are divided by an '\' in the string.
	/// @return A zero terminated string containing the object's category.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*				GetObjectCategory() const=0;

	/// @brief Returns the object for this object placement.
	///
	/// Returns a pointer to the object, which is used for this object placement.
	/// @return Returns a pointer to the object placement's object if
	/// successful or <i>NULL</i> in case of an error.
	virtual CIPnPTCObject01*		GetObject() const=0;

	/// @brief Returns the caption of the object placement.
	///
	/// Returns the caption of the object placement.
	/// @return A zero terminated string containing the object placement's caption.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*				GetCaption() const=0;

	/// @brief Returns comments for that object placement.
	///
	/// Returns a user defined comment/description string for
	/// this object placement.
	/// @return A zero terminated string containing the object placements's comments.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*				GetComments() const=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this object placement.
	/// @return Returns the object placement's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class


#pragma pack(pop)


#endif // !defined(PNPTC_CISECTOROBJECTS_H_INCLUDED)