/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorSounds.h
// Interface definition to access the environmental sector sounds.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	14. October 2004
// ----------------------------------------------------------------------------------
// change history:
//  03. Jan. 2005 (Ralf Westphal)
//		Added methods GetNumUsedSounds, GetUsedSound, GetSound
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
//  19. Jul. 2008 (Ralf Westphal)
//		Sound placement, sound fucntion, and playlist entry now use getter 
//		methods instead of variables.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTORSOUNDS_H_INCLUDED)
#define PNPTC_CISECTORSOUNDS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcsound.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


// forward definitions of objects used by CIPnPTCSectorSounds01
class CIPnPTCSoundPlacement01;
class CIPnPTCSoundFunction01;
class CIPnPTCSoundDependencyFunction01;
class CIPnPTCPlaylistEntry01;

/// Definitions for playback types.
#define	PNPTC_PLAYBACK_SEQUENTIAL	0	//!< Sequential playback of all sounds from the playlist.
#define PNPTC_PLAYBACK_RANDOM		1	//!< Random playback of sounds.
#define	PNPTC_PLAYBACK_SIMULTANEOUS	2	//!< Simultaneous playback of more or all sounds from the playlist

/// Definitions for playback transitions.
#define PNPTC_TRANSITION_UNDELAYED	0	//!< Plays the next sound undelayed after the first sound ended.
#define PNPTC_TRANSITION_RANDOM		1	//!< Plays the next sound with a random delay time after the first sound ended.


/// @brief Description of one sound used by the terrain sector.
///
/// Data structure containing all information needed to
/// identify one sound used by the terrain sector.
PNPTCPLUGIN_CALLBACK typedef struct PnPTC_UsedSound
{
	char	Name[100];	//!< Sound name used by the sound manager.
	WORD	ID;			//!< Internal ID/Index of the sound used by the sound manager.
} PnPTC_UsedSound;


// ---------------------------------------------------------------------------------------

/// @brief Callback accessor class for environmental sector sounds.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to the sector's environmental sounds.
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorSounds01
{
public:

	/// @brief Returns the number of environmental sound placements.
	///
	/// @return Returns number of environmental sound placements.
	virtual unsigned int				GetNumSoundPlacements() const=0;
	
	/// @brief Returns one sound placement.
	///
	/// Returns the a data structure containing all sound placement data.
	/// @param nIndex Index [0..GetNumSoundPlacements()-1] of the
	/// sound placement to be returned.
	/// @return The environmental sound placement data structure.
	/// In case of an error the returned data structure contains no data.
	virtual CIPnPTCSoundPlacement01*	GetSoundPlacement(unsigned int nIndex)=0;

	/// @brief Returns the number of different sounds used by one sector.
	///
	/// Returns the number of different sounds that are currently 
	/// used by one sector.
	/// @return Returns the number of different sounds.
	virtual unsigned int				GetNumUsedSounds()=0;
	
	/// @brief Returns the name and ID of one sound.
	///
	/// Returns the description of one sound that is currently
	/// used by the terrain sector.
	/// @param nIndex Index [0..GetNumUsedSounds()-1] of the
	/// terrain sector sound, thats description has to be returned.
	/// @return The sound's description data structure.
	/// In case of an error the returned data structure contains no data.
	virtual PnPTC_UsedSound				GetUsedSound(unsigned int nIndex)=0;

	/// @brief Returns sound object.
	///
	/// Returns a reference to a sound object by which all relevant
	/// sound data can be accessed.
	/// @param wID Internal ID of the sound to be returned.
	/// @return Pointer to the sound or <i>NULL</i> in case of an error.
	virtual CIPnPTCSound01*				GetSound(WORD wID)=0;

}; // class CIPnPTCSectorSounds01



// ---------------------------------------------------------------------------------------

/// @brief Class containing all data of one sound placement.
class PNPTCPLUGIN_CALLBACK CIPnPTCSoundPlacement01
{
public:

	// -----------------------------------------------------------
	// public methods to access base data of sound placement
	// -----------------------------------------------------------

	/// @brief Returns the id of the sound placement.
	///
	/// Returns the unique identifier of this sound placement.
	/// @return ID of the sound placement.
	virtual GUID						GetID() const=0;		

	/// @brief Returns the position of the sound placement.
	///
	/// Returns the position of this sound placement.
	/// @return Position of the sound placement.
	virtual const PnPTC3DPosition&		GetPos() const=0;

	/// @brief Returns the sound placement volume.
	///
	/// Returns the sound placement volume.
	/// @return The volume of the sound placement in
	/// the range [0..1].
	virtual float						GetVolume() const=0;

	/// @brief Returns the sound volume function.
	///
	/// Returns the sound volume function.
	/// @return The volume function.
	virtual CIPnPTCSoundFunction01*		GetVolumeFunction()=0;

	/// @brief Returns the full volume sound radius.
	///
	/// Returns the radius in which the sound is played at its full volume.
	/// @return Returns the full volume sound radius in meters.
	virtual float						GetFullVolumeRadius() const=0;

	/// @brief Returns the full volume function.
	///
	/// Returns the full volume function.
	/// @return The full volume function.
	virtual CIPnPTCSoundFunction01*		GetFullVolumeRadiusFunction()=0;

	/// @brief Returns the maximum hearing radius of the sound.
	///
	/// Returns the maximum radius at which the sound can be heard.
	/// (Actually this is the radius beyound which the sound
	/// isn't anymore attenuated.
	/// @return Returns the maximum hearing radius sound radius in meters.
	virtual float						GetVolumeMaxRadius() const=0;

	/// @brief Returns the maximum hearing radius function.
	///
	/// Returns the maximum hearing radius function.
	/// @return The maximum hearing radius function.
	virtual CIPnPTCSoundFunction01*		GetVolumeMaxRadiusFunction()=0;


	// -----------------------------------------------------------
	// sound playlist
	// -----------------------------------------------------------

	/// @brief Returns the number of sounds in the playlist.
	///
	/// @return Returns the number of sound in the playlist.
	virtual unsigned int				GetNumSounds() const=0;

	/// @brief Returns one sound from the playlist.
	///
	/// @param nIndex Sound index [0..GetNumSounds()-1] to be returned.
	/// @return Returns a pointer to the playlist entry. Returns
	/// <i>NULL</i> in case of an error.
	virtual CIPnPTCPlaylistEntry01*		GetPlaylistEntry(unsigned int nIndex)=0;

	/// @brief Returns the sounds playback type.
	///
	/// Returns the sounds playback type.
	/// @return The sounds playback type. The possible values are
	/// PNPTC_PLAYBACK_SEQUENTIAL, PNPTC_PLAYBACK_RANDOM, and PNPTC_PLAYBACK_SIMULTANEOUS
	virtual int							GetPlaybackType() const=0;

	/// @brief Returns the maximum number of simultaneous playbacks.
	///
	/// Returns the maximum number of sounds that can be played simultaneous 
	/// when one of the following playback modes is selected: 
	/// PNPTC_PLAYBACK_RANDOM, PNPTC_PLAYBACK_SIMULTANEOUS
	/// @return The maximum number of simultaneous playbacks.
	virtual unsigned int				GetMaxSimultaneous() const=0;

	/// @brief Returns the function for max simultaneous playbacks.
	///
	/// Returns the function for the maximum number of simultaneous played sounds.
	/// @return The function for the maximum number of simultaneous played sounds.
	virtual CIPnPTCSoundFunction01*		GetMaxSimultaneousFunction()=0;

	/// @brief Returns the transition between two sound playbacks.
	///
	/// Returns the Playback transitions between two sound playbacks.
	/// @return The Playback transitions between two sound playbacks.
	/// Transition effect between two sound playbacks as given by the following definitions:
	/// PNPTC_TRANSITION_UNDELAYED, PNPTC_TRANSITION_RANDOM
	virtual int							GetPlaybackTransition() const=0;

	/// @brief Returns the minimum delay between two sound playbacks.
	///
	/// Returns the minimum delay between two sound playbacks.
	/// @return The minimum delay between two sound playbacks in seconds.
	virtual unsigned int				GetMinDelay() const=0;

	/// @brief Returns the function for the minimum delay between two sound playbacks.
	///
	/// Returns the function for the minimum delay between two sound playbacks.
	/// @return The function for the minimum delay between two sound playbacks.
	virtual CIPnPTCSoundFunction01*		GetMinDelayFunction()=0;

	/// @brief Returns the maximum delay between two sound playbacks.
	///
	/// Returns the maximum delay between two sound playbacks.
	/// @return The maximum delay between two sound playbacks in seconds.
	virtual unsigned int				GetMaxDelay() const=0;

	/// @brief Returns the function for the maximum delay between two sound playbacks.
	///
	/// Returns the function for the maximum delay between two sound playbacks.
	/// @return The function for the maximum delay between two sound playbacks.
	virtual CIPnPTCSoundFunction01*		GetMaxDelayFunction()=0;


	// -----------------------------------------------------------
	// random placement border
	// -----------------------------------------------------------

	/// @brief Returns the number of border points for the random sound placement area.
	///
	/// @return Number of random placement area border points.
	virtual unsigned int				GetNumRandomBorderPoints() const=0;

	/// @brief Returns one border point of the random placement area.
	///
	/// The methods returns one point of the border line defining the random
	/// sound placement area.<BR>
	/// <B>Remarks:</b> The border points have to be connected as a polyline
	/// from index <i>0</i> to <i>GetNumRandomBorderPoints()-1</i> and back to <i>0</i>.
	/// @param nIndex Index [0..GetNumRandomBorderPoints()-1] of the border point to be returned.
	/// @return Returns the border point with the given index.
	virtual	PnPTC2DVec					GetBorderPoint(unsigned int nIndex)=0;

	// -----------------------------------------------------------
	// additional methods
	// -----------------------------------------------------------

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this sound placement.
	/// @return Returns the sound placement's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class CIPnPTCSoundPlacement01




// ---------------------------------------------------------------------------------------

/// @brief Class containing all data of one playlist entry.
class PNPTCPLUGIN_CALLBACK CIPnPTCPlaylistEntry01
{
public:

	// -----------------------------------------------------------
	// public variables and methods
	// -----------------------------------------------------------

	/// @brief Returns the sound function for the volume and ranges.
	///
	/// Returns the sound function for the volume and range of this sound 
	/// entry with respect to the values defined for the sound placement.
	virtual CIPnPTCSoundFunction01*		GetVolumeRangeFunction()=0;

	/// @brief Returns the probability of this sound.
	///
	/// Returns the playback probability of this sound in the list of 
	/// sound entries of this sound placement.
	/// @return The playback probability of this sound
	/// in the range [0..1].
	virtual float						GetProbability() const=0;

	/// @brief Returns the probability function of this sound.
	///
	/// Returns the probability function of this sound.
	/// @returns The probability function of this sound.
	virtual CIPnPTCSoundFunction01*		GetProbabilityFunction()=0;

	/// @brief Returns the name/id of the sound.
	///
	/// @return A zero terminated string containing the sound's name.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*					GetSoundName() const=0;

}; // class CIPnPTCPlaylistEntry01



// ---------------------------------------------------------------------------------------

/// @brief Class containing sound function definitions.
class PNPTCPLUGIN_CALLBACK CIPnPTCSoundFunction01
{
public:

	// -----------------------------------------------------------
	// public variables and methods
	// -----------------------------------------------------------

	/// @brief Returns whether or not the function is set.
	///
	/// @return Returns <i>true</i> if any part of the sound function
	/// is set, otherwise <i>false</i>.
	virtual bool								IsFunctionSet() const=0;

	/// @brief Returns the textual dependency function.
	///
	/// @return A zero terminated string containing the userdefined
	/// textual sound dependency function.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*							GetTextFunction() const=0;

	/// @brief Returns the day time sound dependency function.
	///
	/// Returns the day time sound dependency function.
	/// @return The day time sound dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetDayTimeDependencyFunction()=0;

	/// @brief Returns the light sound dependency function.
	///
	/// Returns the light sound dependency function.
	/// @return The light sound dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetLightDependencyFunction()=0;

	/// @brief Returns the temperature sound dependency function.
	///
	/// Returns the temperature sound dependency function.
	/// @return The temperature sound dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetTemperatureDependencyFunction()=0;

	/// @brief Returns the season sound dependency function.
	///
	/// Returns the day season dependency function.
	/// @return The day season dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetSeasonDependencyFunction()=0;

	/// @brief Returns the wind sound dependency function.
	///
	/// Returns the wind sound dependency function.
	/// @return The wind sound dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetWindDependencyFunction()=0;

	/// @brief Returns the precipitation sound dependency function.
	///
	/// Returns the precipitation sound dependency function.
	/// @return The precipitation sound dependency function.
	virtual CIPnPTCSoundDependencyFunction01*	GetPrecipitationDependencyFunction()=0;

}; // class CIPnPTCSoundFunction01




// ---------------------------------------------------------------------------------------

/// control point of a sound dependency function.
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_SNDDEP_VALUE
{
	float	fPos;	//!< position [0..1] of function's control point.
	float	fValue;	//!< value [0..1] of function's control point.
} PNPTC_SNDDEP_VALUE;


/// @brief Class containing a sound dependency function.
class PNPTCPLUGIN_CALLBACK CIPnPTCSoundDependencyFunction01
{
public:

	// -----------------------------------------------------------
	// public methods
	// -----------------------------------------------------------

	/// @brief Returns the number of control points.
	///
	/// Returns the number of control points defining
	/// this sound dependency function.
	/// @return Number of defined control points. If the dependency
	/// function is not set <i>0</i> is returned.
	virtual unsigned int		GetNumControlPoints() const=0;

	/// @brief Returns one control point.
	///
	/// Returns one of the control points defining
	/// this sound dependency function.
	/// @param nIndex Index [0..GetNumControlPoints()-1] of the
	/// control point to be returned.
	/// @return Returns the data of the control point. In case of an
	/// error all entries of the returned data structure have the
	/// value <i>-1</i>.
	virtual PNPTC_SNDDEP_VALUE	GetControlPoint(unsigned int nIndex) const=0;

}; // class CIPnPTCSoundDependencyFunction01

#pragma pack(pop)

#endif // !defined(PNPTC_CISECTORSOUNDS_H_INCLUDED)