/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorTextures.h
// Interface definition to access the texture layer of one sector.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	10. October 2004
// ----------------------------------------------------------------------------------
// change history:
//  03. Jan. 2005 (Ralf Westphal)
//		Added methods GetNumUsedTextures, GetUsedTexture, GetTexture
//  25. Mar. 2005 (Ralf Westphal)
//		Added methods GetTexturedColor, GetTextureUV, GetTextureMap
//  24. Jan. 2007 (Ralf Westphal)
//		Added methods SetTexture
//  02. Jan. 2009 (Ralf Westphal)
//		Made getter methods const
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTORTEXTURES_H_INCLUDED)
#define PNPTC_CISECTORTEXTURES_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface 
#include "ipnptcplugin.h"
#include "cipnptctexture.h"
#include "cipnptctexturemap.h"

#pragma pack(push)
#pragma pack(1)


/// @brief Description of one texture used by the terrain sector.
///
/// Data structure containing all information needed to
/// identify one texture used by the terrain sector.
PNPTCPLUGIN_CALLBACK typedef struct PnPTC_UsedTexture
{
	char	Name[100];	//!< Texture name used by the texture manager.
	WORD	ID;			//!< Internal ID/Index of the texture used by the texture manager. This is the ID returned by calls to CIPnPTCSectorTextures01::GetTexture().
} PnPTC_UsedTexture;


/// @brief Callback accessor class for sector texture data.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access (read and write) to
/// the sector's texture layer.
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorTextures01
{
public:

	/// @brief Returns the number of different textures used by one sector.
	///
	/// Returns the number of different textures that are currently 
	/// used by one sector.
	/// @return Returns the number of different textures.
	virtual unsigned int			GetNumUsedTextures() const=0;
	
	/// @brief Returns the name and ID of one texture.
	///
	/// Returns the description of one texture that is currently
	/// used by the terrain sector.
	/// @param nIndex Index [0..GetNumUsedTextures()-1] of the
	/// terrain sector texture, thats description has to be returned.
	/// @return The texture's description data structure.
	/// In case of an error the returned data structure contains no data.
	virtual PnPTC_UsedTexture		GetUsedTexture(unsigned int nIndex) const=0;

	/// @brief Returns the texture of one terrain element.
	///
	/// Returns the ID of a texture at a given sector position.
	/// @param iX X position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param iY Y position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @return Texture index corresponding to PnPTC_UsedTexture::ID. Returns <i>-1</i> in
	/// case of an error.
	virtual int						GetTexture(int iX,int iY) const=0;

	/// @brief Returns texture object.
	///
	/// Returns a reference to a texture object by which all relevant
	/// texture data can be accessed.
	/// @param wID Internal ID of the texture to be returned.
	/// @return Pointer to the texture object or <i>NULL</i> in case of an error.
	virtual CIPnPTCTexture01*		GetTexture(WORD wID) const=0;

	/// @brief Sets the texture of one terrain element.
	///
	/// Sets the texture of one terrain element.
	/// @param iX X position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param iY Y position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param wTextureID Index of the texture to be applied to the terrain element.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// texture could be applied successfully.
	virtual bool					SetTexture(int iX,int iY,WORD wTextureID)=0;

	/// @brief Sets the texture of one terrain element.
	///
	/// Sets the texture of one terrain element.
	/// @param iX X position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param iY Y position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param strTexture Name of the texture to be applied to the terrain element.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// texture could be applied successfully.
	virtual bool					SetTexture(int iX,int iY,const char* strTexture)=0;

	/// @brief Returns the color of one heightmap element defined by its texture.
	///
	/// Returns the color of one heightmap element as it is defined by the
	/// texture assigned to it.
	/// @param iX X position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param iY Y position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @return Textured color of the given heightmap element.
	virtual PNPTC_RGB				GetTexturedColor(int iX,int iY) const=0;

	/// @brief Returns the UV coordinates of a heightmap position.
	///
	/// Returns the texture UV coordinates used by the detail textures
	/// at a given heightmap position.
	/// @param iX X position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @param iY Y position of the sector element [0..CIPnPTCSector01::GetBorderSize()-1].
	/// @return Returns the detail texture's UV coordinates.
	virtual PnPTCUV					GetTextureUV(int iX,int iY) const=0;

	/// @brief Returns the sector's texturemap.
	///
	/// Returns the sector's texturemap, which is a set of overlay
	/// textures created from the detail textures.
	/// @return Returns a pointer to the sector's texturemap.
	virtual CIPnPTCTextureMap01*	GetTextureMap()=0;

}; // class


#pragma pack(pop)


#endif // !defined(PNPTC_CISECTORTEXTURES_H_INCLUDED)