/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorVegetation.h
// Interface definition to access the vegetation layer of one sector.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	28. November 2004
// ----------------------------------------------------------------------------------
// change history:
//  03. Jan. 2005 (Ralf Westphal)
//		Added methods GetNumUsedTrees, GetUsedTree, GetTree
//  30. Nov. 2005 (Ralf Westphal)
//		Added tree placement comments: GetObjectComments
//  26. Mai. 2006 (Ralf Westphal)
//		Added tree flipping: GetGlobalFlipAxis
//  23. Aug. 2006 (Ralf Westphal)
//		Added separate scaling, flipping, and rotation values for trees
//		and grasses: GetGlobalTreeScale, GetGlobalTreeUpRotation,
//		GetGlobalTreeFlipAxis, GetGlobalGrassScale, 
//		GetGlobalGrassUpRotation, GetGlobalGrassFlipAxis
//  18. Feb. 2007 (Ralf Westphal)
//		Added GetCaption, GetVegetationEngineGUID, and GetTreeCategory and
//		renamed GetTreeComments to GetComments and GetTreePlacementExtra...
//		to GetExtra...
//  19. Feb. 2007 (Ralf Westphal)
//		Now all member variables can only be returned by getter methods.
//		Changed the retrieval of used trees.
//		(Sorry for breaking compatibility)
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTORVEGETATION_H_INCLUDED)
#define PNPTC_CISECTORVEGETATION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcvegetation.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


// forward definitions
class CIPnPTCTreePlacement01;



/// @brief Placement information of grass
///
/// Description of the grass 
typedef struct GRASS_PLACEMENT
{
	const char*		cGrassType;		//<! the type/name of the grass. Zeroterminated string. <i>NULL</i> if no grass placement
	float			fDensity;		//<! density of the grass [0..1]. <i>0</i> if no grass placement.
} GRASS_PLACEMENT;



/// @brief Vegetation of one sector
///
/// All trees and grassed placed on a sector are accessible 
/// through this interface.<br>
/// <b>Trees:</b>
/// Get the total number of tree placements using
/// the GetNumTreePlacements method and iterate
/// through all trees using the GetTreePlacement
/// method. <br>
/// You can get vegetation engine specific tree placement
/// information using the GetTreePlacementExtraData
/// method. But make sure you can interprete the
/// received data by checking for supported
/// vegetation engines with GetVegetationEngineGUID.<br>
/// Using AddTreePlacement and SetTreePlacementExtraData
/// you can generate tree placements for your random/automated
/// vegetation distribution plugins. Call RemoveAllTreePlacements
/// before starting tree placement to make sure that the
/// sector is empty.<br>
/// <b>Grass:</b>
/// Use the GetGrassPlacement method to get information of 
/// grass placements for every square element of the sector.
/// There can be up to 4 different grass types simultanously 
/// on the same square element.<br>
/// Using AddGrassPlacement you can generate grass placements for 
/// your random/automated vegetation distribution plugins. Call 
/// RemoveAllGrassPlacements before starting grass placement
/// to make sure that the sector is empty.<br>
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorVegetation01
{
public:

	// --------------------------------------------------------------------------------------------
	// System/terrain informations
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the GUID of the vegetation engine.
	///
	/// Returns the GUID of the vegetation engine used to place
	/// and render trees and grass.
	/// @return GUID of vegetation engine.
	virtual GUID					GetVegetationEngineGUID() const=0;

	/// @brief Returns the global tree scaling factor.
	///
	/// Returns the scaling factor that is applied to
	/// all tree placements.
	/// @return Global scaling factor.
	virtual float					GetGlobalTreeScale() const=0;

	/// @brief Returns the global tree up rotation.
	///
	/// Returns the global up rotation that is applied to
	/// all tree placements in order to align their
	/// up axis accordingly to the terrain's up axis.
	/// @return Global up rotation as a value defined by UPROTATION_NONE,
	/// UPROTATION_ROTX90, or UPROTATION_ROTX270.
	virtual int						GetGlobalTreeUpRotation() const=0;

	/// @brief Returns the trees' flip axis.
	///
	/// Returns the axis at which all tree object vertices are flipped in order
	/// to convert between right and left handed coordinate system usage.
	/// @return Global flip axis as a value defined by FLIP_NONE,
	/// FLIP_XAXIS, FLIP_YAXIS, or FLIP_ZAXIS.
	virtual int						GetGlobalTreeFlipAxis() const=0;

	/// @brief Returns the global grass scaling factor.
	///
	/// Returns the scaling factor that is applied to
	/// all grass placements.
	/// @return Global scaling factor.
	virtual float					GetGlobalGrassScale() const=0;

	/// @brief Returns the global up rotation of grasses.
	///
	/// Returns the global up rotation that is applied to
	/// all grass placements in order to align their
	/// up axis accordingly to the terrain's up axis.
	/// @return Global up rotation as a value defined by UPROTATION_NONE,
	/// UPROTATION_ROTX90, or UPROTATION_ROTX270.
	virtual int						GetGlobalGrassUpRotation() const=0;

	/// @brief Returns the grasses flip axis.
	///
	/// Returns the axis at which all grass object vertices are flipped in order
	/// to convert between right and left handed coordinate system usage.
	/// @return Global flip axis as a value defined by FLIP_NONE,
	/// FLIP_XAXIS, FLIP_YAXIS, or FLIP_ZAXIS.
	virtual int						GetGlobalGrassFlipAxis() const=0;


	// --------------------------------------------------------------------------------------------
	// Access to sector's tree placements
	// --------------------------------------------------------------------------------------------

	/// @brief Returns number of trees.
	///
	/// Returns the number of trees placed on this sector.
	/// @return Number of trees.
	virtual unsigned long			GetNumTreePlacements() const=0;

	/// @brief Returns one tree placement.
	///
	/// Returns the placement information of one tree.
	/// @param ulIndex Index of the tree placement to be retrieved.
	/// @return Returns the desription of the tree placement if successfull.
	/// Returns <i>NULL</i> in case of an error. The pointer returned by
	/// this method is valid as long as the corresponding CIPnPTCSector01 object
	/// is valid.
	virtual CIPnPTCTreePlacement01*	GetTreePlacement(unsigned long ulIndex) const=0;

	/// @brief Removes one tree.
	///
	/// Removes one tree of this sector.
	/// @param ulIndex Index of the tree placement to be removed.
	/// @return Returns <i>true</i> if removal was successfull,
	/// <i>false</i> in case of an error.
	virtual bool					RemoveTreePlacement(unsigned long ulIndex)=0;

	/// @brief Removes all trees.
	///
	/// Removes all trees of this sector.
	/// @return Returns <i>true</i> if removal was successfull,
	/// <i>false</i> in case of an error.
	virtual bool					RemoveAllTreePlacements()=0;

	/// @brief Adds one tree.
	///
	/// Adds one tree to the sector.
	/// @param cTreeName Name/ID of the tree type to be added.
	/// @return Returns a pointer to the newly created placed tree.
	/// Returns <i>NULL</i> in case of an error.<br>
	/// The returned tree placement object can be used to set the
	/// parameters of the tree placement.
	virtual CIPnPTCTreePlacement01*	AddTreePlacement(const char* cTreeName)=0;

	/// @brief Returns the number of different trees used by one sector.
	///
	/// Returns the number of different trees that are currently 
	/// used by one sector.
	/// @return Returns the number of different trees.
	virtual unsigned int			GetNumUsedTrees() const=0;
	
	/// @brief Returns the name and ID of one tree.
	///
	/// Returns the description of one tree that is currently
	/// used by the terrain sector.
	/// @param nIndex Index [0..GetNumUsedTrees()-1] of the
	/// terrain sector tree, thats description has to be returned.
	/// @return The tree used by the terrain sector.
	/// In case of an error <i>NULL</i> is returned.
	virtual CIPnPTCTree01*			GetUsedTree(unsigned int nIndex) const=0;

	/// @brief Returns tree object.
	///
	/// Returns a reference to a tree object by which all relevant
	/// tree data can be accessed.
	/// @param wID Internal ID of the tree to be returned.
	/// @return Pointer to the tree or <i>NULL</i> in case of an error.
	virtual CIPnPTCTree01*			GetTree(WORD wID) const=0;


	// --------------------------------------------------------------------------------------------
	// Access to sector's grass placements
	// --------------------------------------------------------------------------------------------

	/// @brief Returns one grass placement
	///
	/// Returns the grass informations of one square sector element.
	/// @param uiX X index of sector element.
	/// @param uiY Y index of sector element.
	/// @param ucGrassIndex Index of the grass placement on the given sector element.
	/// There can be up to 4 different grasses [0..3] placed on the same sector element.
	/// @return Returns a pointer to the selected grass placement. If the desired grass 
	/// placement is unset or in case of an error, the GRASS_PLACEMENT elements
	/// are set to zero.
	virtual GRASS_PLACEMENT			GetGrassPlacement(unsigned int uiX,unsigned int uiY, unsigned char ucGrassIndex) const=0;

	/// @brief Removes all grass.
	///
	/// Removes all grass placements from one sector.
	/// @return Returns <i>true</i> if removal was successfull,
	/// <i>false</i> in case of an error.
	virtual bool					RemoveAllGrassPlacements()=0;

	/// @brief Removes one grass placement
	///
	/// Removes one grass placement from the sector
	/// @param uiX X index of sector element.
	/// @param uiY Y index of sector element.
	/// @param ucGrassIndex Index of the grass placement on the given sector element.
	/// There can be up to 4 different grasses [0..3] placed on the same sector element.
	virtual void					RemoveGrassPlacement(unsigned int uiX,unsigned int uiY, unsigned char ucGrassIndex)=0;

	/// @brief Adds/Set one grass placement.
	///
	/// Adds/sets one grass placement to/of the sector.
	/// @param uiX X index of sector element.
	/// @param uiY Y index of sector element.
	/// @param ucGrassIndex Index of the grass placement on the given sector element.
	/// There can be up to 4 different grasses [0..3] placed on the same sector element.
	/// @param gpData Data of the grass placement.
	/// @return Returns <i>true</i> if the grass placement could be set successfull.
	/// Returns <i>false</i> in case of an error - for example sector or grass index out of range.
	virtual bool					AddGrassPlacement(unsigned int uiX,unsigned int uiY, unsigned char ucGrassIndex,GRASS_PLACEMENT& gpData)=0;

}; // class CIPnPTCSectorVegetation01



// ---------------------------------------------------------------------------------------

/// @brief Tree placement data.
///
/// This interface contains all data of one tree placement.<br>
class PNPTCPLUGIN_CALLBACK CIPnPTCTreePlacement01
{
public:

	// -----------------------------------------------------------
	// access to tree placement data
	// -----------------------------------------------------------

	/// @brief Returns the id of the tree placement.
	///
	/// Returns the unique identifier of this tree placement.
	/// @return ID of the tree placement.
	virtual GUID						GetID() const=0;		

	/// @brief Returns the position of the tree placement.
	///
	/// Returns the position of this tree placement.
	/// @return Position of the tree placement.
	virtual const PnPTC3DPosition&		GetPos() const=0;

	/// @brief Sets a new position of the tree placement.
	///
	/// Sets a new position of the tree placement.
	/// @param fPosX New x coordinate of the tree in metrical
	/// units with respect to the sector's origin.
	/// @param fPosY New y coordinate of the tree in metrical
	/// units with respect to the sector's origin.
	/// @param fHeight Height over ground of the tree in metrical units.
	virtual void						SetPos(float fPosX,float fPosY,float fHeight)=0;

	/// @brief Sets a new position of the tree placement.
	///
	/// Sets a new position of the tree placement.
	/// @param pos New position to be set.
	virtual void						SetPos(const PnPTC3DPosition& pos)=0;

	/// @brief Returns the scaling of the tree placement.
	///
	/// Returns the scaling of this tree placement in x direction.
	/// @return Scaling of tree placement in x direction.
	virtual float						GetScaleX() const=0;

	/// @brief Returns the scaling of the tree placement.
	///
	/// Returns the scaling of this tree placement in y direction.
	/// @return Scaling of tree placement in y direction.
	virtual float						GetScaleY() const=0;

	/// @brief Returns the scaling of the tree placement.
	///
	/// Returns the scaling of this tree placement in z direction.
	/// @return Scaling of tree placement in z direction.
	virtual float						GetScaleZ() const=0;

	/// @brief Sets the scaling of the tree.
	/// 
	/// Sets the tree scaling.
	/// @param fScaleX Scaling in x direction.
	/// @param fScaleY Scaling in y direction.
	/// @param fScaleZ Scaling in z direction.
	virtual void						SetScale(float fScaleX,float fScaleY,float fScaleZ)=0;

	/// @brief Returns the orientation of the tree placement.
	///
	/// Returns the rotation of this tree placement.
	/// around the x axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of tree placement about the x axis.
	/// The angle is given in radians.
	virtual float						GetRotX() const=0;

	/// @brief Returns the orientation of the tree placement.
	///
	/// Returns the rotation of this tree placement.
	/// around the y axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of tree placement about the y axis.
	/// The angle is given in radians.
	virtual float						GetRotY() const=0;

	/// @brief Returns the orientation of the tree placement.
	///
	/// Returns the rotation of this tree placement.
	/// around the z axis. The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// @return rotation of tree placement about the z axis.
	/// The angle is given in radians.
	virtual float						GetRotZ() const=0;

	/// @brief Sets the tree placements orientation.
	///
	/// Sets the orientation of the tree placement.
	/// The rotations are performed in the 
	/// following order RotZ -> RotX -> RotY.
	/// The angles are given in radians.
	/// @param fRotX Rotation about x axis
	/// @param fRotY Rotation about y axis.
	/// @param fRotZ Rotation about z axis.
	virtual void						SetRot(float fRotX,float fRotY,float fRotZ)=0;

	/// @brief Returns the name/id of the tree.
	///
	/// Returns the name of the tree.
	/// @return A zero terminated string containing the tree's name.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*					GetTreeName() const=0;

	/// @brief Returns the category of the tree.
	///
	/// Returns the category string of the tree which is used for
	/// this tree placement. This is the category used by the
	/// vegetation manager. Subcategories are divided by an '\' in the string.
	/// @return A zero terminated string containing the tree's category.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*					GetTreeCategory() const=0;

	/// @brief Returns the tree object for this tree placement.
	///
	/// Returns a pointer to the tree object, which is used for this tree placement.
	/// @return Returns a pointer to the tree placement's tree object if
	/// successful or <i>NULL</i> in case of an error.
	virtual CIPnPTCTree01*				GetTree() const=0;

	/// @brief Returns additional tree data.
	///
	/// Returns additional, tree engine specific data of one tree placement.
	/// @param pMem Return parameter: Pointer to a memory region containing the extra data.
	/// This pointer might be temporary.
	/// @param ulMemSize Return parameter: Containing the size of the memory region.
	/// @return Returns <i>true</i> if the extra data could be retrieved successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool						GetExtraData(const void** pMem,unsigned long* ulMemSize) const=0;

	/// @brief Sets additional tree data.
	///
	/// Sets additional, tree engine specific data of one tree placement.
	/// @param pMem Pointer to a memory region containing the extra data.
	/// @param ulMemSize The size of the memory region.
	/// @return Returns <i>true</i> if the extra data could be set successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool						SetExtraData(void* pMem,unsigned long ulMemSize)=0;

	/// @brief Returns comments for that tree placement.
	///
	/// Returns a user defined comment/description string for this tree placement.
	/// @return A zero terminated string containing the tree placements's comments.
	/// <i>NULL</i> in case of an error.<BR>
	/// <b>Remarks:</b>This might be a temporary pointer, so don't store
	/// it for later use.
	virtual const char*					GetComments() const=0;

	/// @brief Sets comments for that tree placement.
	///
	/// Sets user defined comment/description string for this tree placement.
	/// @param strComments A zero terminated string containing the tree placements's comments.
	virtual void						SetComments(const char* strComments)=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this tree placement.
	/// @return Returns the tree placement's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class CIPnPTCTreePlacement01

#pragma pack(pop)

#endif // !defined(PNPTC_CISECTORVEGETATION_H_INCLUDED)