/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSkySettings.h
// Interface definition of a terrain sky settings object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	24. Apr. 2006
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISKYSETTINGS_H_INCLUDED)
#define PNPTC_CISKYSETTINGS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"
#include "cipnptctexture.h"

#pragma pack(push)
#pragma pack(1)


// Possible sky rendering systems
#define PNPTC_SKYSYSTEM_NOSKY			0		//!< no sky rendering system.
#define PNPTC_SKYSYSTEM_BOX				1		//!< a sky box is used for sky rendering.
#define PNPTC_SKYSYSTEM_DOME			2		//!< a sky dome is used for sky rendering.
#define PNPTC_SKYSYSTEM_SPHERE			3		//!< a sky sphere is used for sky rendering.

// Possible sky rendering systems
#define PNPTC_SKYTEX_ORDER_UNDEFINED	-1		//!< sky box textures ordered is undefined.
#define PNPTC_SKYTEX_ORDER_CW			0		//!< sky box textures are ordered clockwise.
#define PNPTC_SKYTEX_ORDER_CCW			1		//!< sky box textures are ordered counter clockwise.

// Possible sky rendering systems
#define PNPTC_SKYTEX_VERTPOS_UNDEFINED	-1		//!< the vertical textures position is undefined.
#define PNPTC_SKYTEX_VERTPOS_LEFT		0		//!< the vertical textures of a sky box are connected to the left texture.
#define PNPTC_SKYTEX_VERTPOS_FRONT		1		//!< the vertical textures of a sky box are connected to the front texture.
#define PNPTC_SKYTEX_VERTPOS_RIGHT		2		//!< the vertical textures of a sky box are connected to the right texture.
#define PNPTC_SKYTEX_VERTPOS_BACK		3		//!< the vertical textures of a sky box are connected to the back texture.


/// @brief Terrain sky settings access class.
///
/// Using this class the sky settings for this 
/// terrain can be accessed.
class PNPTCPLUGIN_CALLBACK CIPnPTCSkySettings01
{
public:

	/// @brief Returns the sky background color.
	///
	/// Returns the background color of the sky for regions
	/// in which no sky texture is rendered.
	/// @return Sky background color.
	virtual PNPTC_RGB				GetBackgroundColor()=0;

	/// @brief Returns whether or not fog is activated.
	///
	/// Returns whether or not the distance range for
	/// is defined for this terrain.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// this terrain uses distance range fog.
	virtual bool					GetFogActivated()=0;

	/// @brief Returns the fog color.
	///
	/// Returns the color for the distance range fog.
	/// @return Fog color.
	virtual PNPTC_RGB				GetFogColor()=0;

	/// @brief Returns the fog start distance.
	///
	/// Returns the distance from the viewer where to
	/// begin to blend in fog.
	/// @return Distance in meters where to start fog.
	virtual float					GetFogMinDistance()=0;

	/// @brief Returns the fog end distance.
	///
	/// Returns the distance from the viewer where to
	/// completely blend into the fog color.
	/// @return Distance in meters where to end the fog.
	virtual float					GetFogMaxDistance()=0;

	/// @brief Returns the sky rendering system.
	///
	/// Returns the sky rendering system which is used
	/// for this terrain.
	/// @return Sky rendering system. This can be one of the following defines:
	/// PNPTC_SKYSYSTEM_NOSKY, PNPTC_SKYSYSTEM_BOX, PNPTC_SKYSYSTEM_DOME, and
	/// PNPTC_SKYSYSTEM_SPHERE.
	virtual int						GetSkySystem()=0;

	/// @brief Returns the name of the sky texture.
	///
	/// Returns the name of the sky texture, resp. the
	/// name of the sky texture set.
	/// @return Sky texture's name
	virtual const char*				GetSkyTextureName()=0;

	/// @brief Returns the number of sky textures.
	///
	/// Returns the number of sky textures for the sky system
	/// @return Number of sky textures.
	virtual int						GetNumSkyTextures()=0;

	/// @brief Returns one of the sky textures.
	///
	/// @param iTexture Index of the sky texture to be returned
	/// [0..GetNumSkyTextures()-1].
	/// @return Returns a pointer to the sky texture. 
	/// Returns <i>NULL</i> in case of an error.
	virtual CIPnPTCTexture01*		GetSkyTexture(int iTexture)=0;

	/// @brief Returns the order of sky textures.
	///
	/// Returns the order in which the sky textures of a 
	/// sky box are aligned. Please see the main manual for
	/// further descriptions.
	/// @return Order of sky box textures. Possible values are defined by:
	/// PNPTC_SKYTEX_ORDER_CW, PNPTC_SKYTEX_ORDER_CCW, and
	/// PNPTC_SKYTEX_ORDER_UNDEFINED.
	virtual int						GetSkyTexturesOrder()=0;

	/// @brief Returns the position of the vertical sky textures.
	///
	/// Returns the position where the vertical textures of a sky box
	/// are connected to the 4 horizontal textures.
	/// @return Position of vertical sky box textures. Possible values are defined by:
	/// PNPTC_SKYTEX_VERTPOS_LEFT, PNPTC_SKYTEX_VERTPOS_FRONT,
	/// PNPTC_SKYTEX_VERTPOS_BACK, PNPTC_SKYTEX_VERTPOS_RIGHT, and
	/// PNPTC_SKYTEX_VERTPOS_UNDEFINED.
	virtual int						GetSkyTexturesVerticalPos()=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_CISKYSETTINGS_H_INCLUDED)