/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCTerrain.h
// Interface definition of a terrain object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	01. Jun. 2004
// ----------------------------------------------------------------------------------
// change history:
//  24. Nov. 2004 (Ralf Westphal)
//		Added GetApplicationObject() and GetSectorDetailBorderSize()
//  01. Aug. 2005 (Ralf Westphal)
//		Added GetNumTextures() and GetTexture()
//  23. Aug. 2005 (Ralf Westphal)
//		Added GetNumLightMapDefinitions() and GetLightMapDefinition()
//  30. Sep. 2005 (Ralf Westphal)
//		Added GetTextureSize(), GetOverlayTexture() and GetOverlayTextureSize()
//  24. Apr. 2006 (Ralf Westphal)
//		Added GetTextureMode(), GetNumTexturemapPixelsPerElement(),
//		GetRenderSettings(), GetSkySettings()
//  29. Aug. 2006 (Ralf Westphal)
//		Added GetOverlayTextureAddressMode()
//  03. Sep. 2006 (Ralf Westphal)
//		Added GetTerrainWater()
//  29. Dec. 2006 (Ralf Westphal)
//		Added GetNumTrees(), GetTree(), GetNumGrasses(), GetGrass()
//  04. Feb. 2007 (Ralf Westphal)
//		Added GetDetailHorizontalScaleFactor(), Save()
//  25. Mai. 2007 (Ralf Westphal)
//		Added GetName(), GetNumObjects(), and GetObject()
//  21. Aug. 2008 (Ralf Westphal)
//		Added GetProjectPath()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CITERRAIN_H_INCLUDED)
#define PNPTC_CITERRAIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"

// includes
#include "./CIPnPTCSectorDefinition.h"
#include "./CIPnPTCSector.h"
#include "./cipnptcapplication.h"
#include "./cipnptclightmapdefinition.h"
#include "./cipnptcrendersettings.h"
#include "./cipnptcskysettings.h"
#include "./cipnptcterrainwater.h"

#pragma pack(push)
#pragma pack(1)


// Possible terrain texturing modes
#define PNPTC_TEXTUREMODE_FLAT			0		//!< no blend effect just one texture per quad and hard transitions
#define PNPTC_TEXTUREMODE_BLEND			1		//!< linear blended textures (splatting)
#define PNPTC_TEXTUREMODE_MAP			2		//!< precomputed texture map bitmap

// Possible texture addressing modes
#define	PNPTC_TEXTUREADDRESSMODE_REPEAT	1		//!< textures are repeated / wrapped
#define PNPTC_TEXTUREADDRESSMODE_MIRROR	2		//!< textures are mirrored


/// @brief Terrain access class.
///
/// Using this class all elements of a terrain
/// can be accessed.
class PNPTCPLUGIN_CALLBACK CIPnPTCTerrain01
{
public:

	/// @brief Number of sectors in x direction.
	///
	/// Returns the number of terrain sectors in x (left to right) direction.
	/// @return Number of terrain sectors in x direction.
	virtual unsigned int					GetNumSectorsX()=0;

	/// @brief Number of sectors in y direction.
	///
	/// Returns the number of terrain sectors in y (bottom to top) direction.
	/// @return Number of terrain sectors in y direction.
	virtual unsigned int					GetNumSectorsY()=0;

	/// @brief Returns the border size of a definition sector.
	///
	/// Returns the border size of a sector's definition heightmap.
	/// @return Sector definition border size.
	virtual unsigned int					GetSectorDefinitionBorderSize()=0;

	/// @brief Returns one sector definition.
	///
	/// Returns one sector definition object.
	/// @param nX Index of the sector definition to be returned in x direction.
	/// In the range [0..GetNumSectorsX()-1].
	/// @param nY Index of the sector definition to be returned in y direction.
	/// In the range [0..GetNumSectorsY()-1].
	/// @return Returns a pointer to the sector definition object. Returns <i>NULL</i>
	/// in case of an error.<br>
	/// <b>Remarks:</b><br>
	/// This is a temporary pointer: Don't store or delete it, as the
	/// memory will be managed from main application and might therefore
	/// no longer be valid when your plugin is called the next time.<br>
	/// <b>If you make two subsequent calls to GetSectorDefinition, the object of the
	/// first call will be destroyed by the second call</b>. So you can only
	/// access one sector definition at a time.
	virtual CIPnPTCSectorDefinition01*		GetSectorDefinition(unsigned int nX,unsigned int nY)=0;

	/// @brief Returns one sector.
	///
	/// Returns one sector object.
	/// @param nX Index of the sector to be returned in x direction.
	/// In the range [0..GetNumSectorsX()-1].
	/// @param nY Index of the sector to be returned in y direction.
	/// In the range [0..GetNumSectorsY()-1].
	/// @return Returns a pointer to the sector object. Returns <i>NULL</i>
	/// in case of an error.<br>
	/// <b>Remarks:</b><br>
	/// This is a temporary pointer: Don't store or delete it, as the
	/// memory will be managed from main application and might therefore
	/// no longer be valid when your plugin is called the next time.<br>
	/// <b>If you make two subsequent calls to GetSector, the object of the
	/// first call will be destroyed by the second call</b>. So you can only
	/// access one sector at a time. This is done because of the large amount
	/// of memory that will be needed if you would try to open all sectors at
	/// the same time.
	virtual CIPnPTCSector01*				GetSectorDetail(unsigned int nX,unsigned int nY)=0;

	/// @brief Initializes the terrain.
	///
	/// Initializes the whole terrain.<br>
	/// <b>Remarks:</b> Only call this method from an import filter plugin,
	/// as the terrain will in any other case always already be initialized.<br>
	/// Calls to this method for an already initialized terrain will not result
	/// in damage or error but just return <i>false</i>.
	/// @param nSectorsX Number of terrain sectors in x direction.
	/// @param nSectorsY Number of terrain sectors in y direction.
	/// @param nSectorDefinitionBorderSize Bordersize (in heightmap elements) of one sector definition map.<br>
	/// Must be one of these : 8, 16, 32, 64, 128
	/// @param nSectorDetailBorderSize Bordersize (in heightmap elements) of one sector detail map.<br>
	/// Must be one of these : 256, 512, 1024, 2048
	/// @param fMeterFactor The size of one detail heightmap element in meters.<br>
	/// Must be in the range ]0..10]
	/// @return Returns <i>0</i> if the terrain could be successfully initialized.<br>
	/// in case of an error the following <b>error codes</b> are returned.<br>
	/// <b> 1 : </b>Terrain already initialized.<br>
	/// <b> 2 : </b>Invalid number of sectors.<br>
	/// <b> 3 : </b>Invalid sector definition border size.<br>
	/// <b> 4 : </b>Invalid sector detail border size.<br>
	/// <b> 5 : </b>Invalid meter factor.<br>
	/// <b> 6 : </b>Not enough memory to create terrain.<br>
	/// <b>-1 : </b>Unknown/undocumented error.<br>
	virtual int								InitTerrain(unsigned int nSectorsX,unsigned int nSectorsY,unsigned int nSectorDefinitionBorderSize=64,unsigned int nSectorDetailBorderSize=1024,float fMeterFactor=1)=0;

	/// @brief Sets definition map height.
	///
	/// Sets the height of one element of the terrains definition map.
	/// @param nX Definition heightmap x index in the range [0..GetNumSectorsX()*GetSectorDefinitionBorderSize()-1].
	/// @param nY Definition heightmap y index in the range [0..GetNumSectorsY()*GetSectorDefinitionBorderSize()-1].
	/// @param fHeight New height value to be set.
	/// @return Returns <i>true</i> if the height could be set. Returns <i>false</i>
	/// if the height could not be set (nX or nY out of range).
	virtual bool							SetDefMapHeight(unsigned int nX,unsigned int nY,float fHeight)=0;

	/// @brief Sets definition map height.
	///
	/// Sets the height of one element of the terrains definition map.
	/// @param nX Definition heightmap x index in the range [0..GetNumSectorsX()*GetSectorDefinitionBorderSize()-1].
	/// @param nY Definition heightmap y index in the range [0..GetNumSectorsY()*GetSectorDefinitionBorderSize()-1].
	/// @return Returns the height at the given definition heightmap index. Returns <i>-1</i>
	/// if the height could not be returned (nX or nY out of range).
	virtual float							GetDefMapHeight(unsigned int nX,unsigned int nY)=0;

	/// @brief Returns the color for a given height.
	///
	/// The color code for a given height is computed
	/// and returned with respect to the color scaling
	/// of the terrain.
	/// @param fHeight Height to be converted into a color.
	/// @return Returns the color value for the given height.
	virtual PNPTC_RGB						GetHeightColor(float fHeight)=0;

	/// @brief Returns the minimum available height of the terrain.
	///
	/// @return Minimum available height.
	virtual float							GetMinHeight()=0;

	/// @brief Returns the maximum available height of the terrain.
	///
	/// @return Maximum available height.
	virtual float							GetMaxHeight()=0;

	/// @brief Returns the PnP TC application object.
	///
	/// @return Returns a reference to the application object.
	virtual	CIPnPTCApplication01*			GetApplicationObject()=0;

	/// @brief Returns the border size of a sector.
	///
	/// Returns the border size of a sector's detail heightmap.
	/// @return Sector detail border size.
	virtual unsigned int					GetSectorDetailBorderSize()=0;

	/// @brief Returns the number of textures used by the terrain.
	///
	/// Returns the complete number of textures that are available
	/// from the texture manager for this terrain project.
	/// @return Number of textures.
	virtual unsigned int					GetNumTextures()=0;

	/// @brief Returns one texture.
	///
	/// Return a pointer to one surface texture.
	/// @param nTexture Index of the texture to be returned within
	/// the range [0..GetNumTextures()-1].
	/// @return Returns a pointer to the requested texture if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCTexture01*				GetTexture(unsigned int nTexture)=0;

	/// @brief Returns the size of the textures.
	///
	/// Returns the size in heightmap elements of the (blended) detail textures.
	/// @return (Blended) detail texture size.
	virtual int								GetTextureSize()=0;

	/// @brief Returns the number of light map definitions.
	///
	/// Returns the number of light map definitions available
	/// for this terrain.
	/// @return Number of light map definitions.
	virtual unsigned int					GetNumLightMapDefinitions()=0;

	/// @brief Returns one light map definition.
	///
	/// Returns a pointer to one light map definition object.
	/// @param nLightMapDefinition index of the light map definition object
	/// to be returned within the range [0..GetNumLightMapDefinitions()-1].
	/// @return Returns a pointer to the light map definition object if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCLightMapDefinition01*	GetLightMapDefinition(unsigned int nLightMapDefinition)=0;

	/// @brief Returns the overlay texture.
	///
	/// Return a pointer to the overlay texture.
	/// @return Returns a pointer to the overlay texture if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCTexture01*				GetOverlayTexture()=0;

	/// @brief Returns the size of the overlay texture.
	///
	/// Returns the size in heightmap elements of the overlay textures.
	/// @return Overlay texture size.
	virtual int								GetOverlayTextureSize()=0;

	/// @brief Returns the addressing mode of the overlay texture.
	///
	/// Returns the addressing mode which is used to render the 
	/// overlay texture.
	/// @return Overlay texture's adressing mode. Possible values are
	/// PNPTC_TEXTUREADDRESSMODE_REPEAT and PNPTC_TEXTUREADDRESSMODE_MIRROR.
	virtual int								GetOverlayTextureAddressMode()=0;

	/// @brief Returns the texturing mode used by this terrain.
	///
	/// Returns the texturing mode used by this terrain.
	/// @return The terrain's texturing mode. Possible return values are
	/// PNPTC_TEXTUREMODE_FLAT, PNPTC_TEXTUREMODE_BLEND, and PNPTC_TEXTUREMODE_MAP.
	virtual int								GetTextureMode()=0;

	/// @brief Returns the number of texture pixels per heightmap element.
	///
	/// This method returns the number of pixels which are used
	/// to cover one heightmap element in the texture maps.
	/// @return Number of texturemap pixels per heightmap element.
	virtual int								GetNumTexturemapPixelsPerElement()=0;

	/// @brief Returns the terrain's rendering settings.
	///
	/// Returns a pointer to an object by which the terrain's
	/// rendering settings can be accessed.
	/// @return Pointer to an object to access the terrain's
	/// rendering settings.
	virtual CIPnPTCRenderSettings01*		GetRenderSettings()=0;

	/// @brief Returns the terrain's sky settings.
	///
	/// Returns a pointer to an object by which the terrain's
	/// sky settings can be accessed.
	/// @return Pointer to an object to access the terrain's
	/// sky settings.
	virtual CIPnPTCSkySettings01*			GetSkySettings()=0;

	/// @brief Returns the terrain's water definition.
	///
	/// Returns a pointer to an object by which the terrain's
	/// water definition can be accessed.
	/// @return Pointer to an object to access the terrain's
	/// water definition.
	virtual	CIPnPTCTerrainWater01*			GetTerrainWater()=0;

	/// @brief Returns the number of trees used by the terrain.
	///
	/// Returns the complete number of trees that are available
	/// from the vegetation manager for this terrain project.
	/// @return Number of trees.
	virtual unsigned int					GetNumTrees() const=0;

	/// @brief Returns one tree object.
	///
	/// Return a pointer to one tree object.
	/// @param nTree Index of the tree to be returned within
	/// the range [0..GetNumTrees()-1].
	/// @return Returns a pointer to the requested tree if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCTree01*					GetTree(unsigned int nTree) const=0;

	/// @brief Returns the number of grasses used by the terrain.
	///
	/// Returns the complete number of grasses that are available
	/// from the vegetation manager for this terrain project.
	/// @return Number of grasses.
	virtual unsigned int					GetNumGrasses() const=0;

	/// @brief Returns one grass object.
	///
	/// Return a pointer to one grass object.
	/// @param nGrass Index of the grass to be returned within
	/// the range [0..GetNumGrasses()-1].
	/// @return Returns a pointer to the requested grass if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCGrass01*					GetGrass(unsigned int nGrass) const=0;

	/// @brief Returns the horizontal scaling factor of the detail heightmap.
	/// 
	/// This method returns the scaling factor for the single heightmap
	/// elements (resp. their distances) with respect to the height values.
	/// @return Returns the horizontal scaling factor.
	virtual float							GetDetailHorizontalScaleFactor() const=0;

	/// @brief Stores the terrain's data
	///
	/// After successfull execution of importer or generator plugins
	/// you have to save the terrain manually.
	/// @return Returns whether or not the terrain could
	/// be saved successfully.
	virtual bool							Save()=0;

	/// @brief Returns the number of objects used by the terrain.
	///
	/// Returns the complete number of objects that are available
	/// from the object manager for this terrain project.
	/// @return Number of objects.
	virtual unsigned int					GetNumObjects() const=0;

	/// @brief Returns one object.
	///
	/// Return a pointer to one object.
	/// @param nObj Index of the object to be returned within
	/// the range [0..GetNumObjects()-1].
	/// @return Returns a pointer to the requested object if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCObject01*				GetObject(unsigned int nObj) const=0;

	/// @brief Returns the name of the terrain.
	///
	/// Returns the terrain's name.
	/// @return The terrain's name.
	virtual const char*						GetName() const=0;

	/// @brief Returns the path of this terrain project.
	///
	/// Returns the path of this terrain project.
	/// @return The path in which the terrain project is stored.
	virtual const char*						GetProjectPath() const=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_CITERRAIN_H_INCLUDED)