/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCTexture.h
// Interface definition to access a texture.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. January 2005
// ----------------------------------------------------------------------------------
// change history:
//  25. Mar. 2005 (Ralf Westphal)
//		Added save texture format and options
//  02. Oct. 2005 (Ralf Westphal)
//		Added GetNormalMap()
//  04. Nov. 2006 (Ralf Westphal)
//      Getter methods are now const
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
//  04. May 2008 (Ralf Westphal)
//		Added GetComments()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CITEXTURE_H_INCLUDED)
#define PNPTC_CITEXTURE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)


// -----------------------------------------------------------------------------------

// supported file formats for texture storage
#define	PNPTC_TEXFMT_DEFAULT	-1			//!< the default texture format. this is the same format as the one from which the texture was loaded or PNG for new textures
#define	PNPTC_TEXFMT_BMP		0			//!< windows bitmap format
#define	PNPTC_TEXFMT_PNG		1			//!< png file format
#define	PNPTC_TEXFMT_DDS		2			//!< dds (direct x) file format



// -----------------------------------------------------------------------------------

/// @brief Callback accessor class for texture.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to a textures.
class PNPTCPLUGIN_CALLBACK CIPnPTCTexture01
{
public:

	// --------------------------------------------------------------------------------------------
	// base texture and file methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the internal name of the texture.
	///
	/// Returns the internal name of the texture as it is used
	/// within the main program.
	/// @return Returns the internal name of the textures.
	virtual const char*					GetName() const=0;


	// --------------------------------------------------------------------------------------------
	// methods to save the texture
	// --------------------------------------------------------------------------------------------

	/// @brief Saves the texture to a file.
	///
	/// Saves the texture to a file.
	/// @param cFileName the path and filename to that the texture should be saved.
	/// @param iFormat Save file format (PNPTC_TEXFMT_BMP, PNPTC_TEXFMT_PNG ...).
	/// @param dwFlags Additional save flags (not yet supported (must be 0)).
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool						SaveToFile(char* cFileName,int iFormat=PNPTC_TEXFMT_DEFAULT,DWORD dwFlags=0) const=0;
	
	/// @brief Saves the texture to memory.
	///
	/// Saves the texture file to the memory.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param iFormat Save file format (PNPTC_TEXFMT_BMP, PNPTC_TEXFMT_PNG ...).
	/// @param dwFlags Additional save flags (not yet supported (must be 0)).
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool						SaveToMem(void** pMem,DWORD* dwSize,int iFormat=PNPTC_TEXFMT_DEFAULT,DWORD dwFlags=0) const=0;


	// --------------------------------------------------------------------------------------------
	// methods to access the texture data
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the number of mip maps.
	///
	/// Returns the number of mip maps available in this texture.
	virtual int							GetNumMipLevels() const=0;

	/// @brief Creates the complete mip map chain.
	/// 
	/// (Re-)creates the mip map chain of the texture. At first all
	/// existing mip maps will be deleted. Afterwards they will be 
	/// recreated from the base (0) level.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// mip maps could be created.
	virtual bool						CreateMipMapChain()=0;

	/// @brief Returns the width of the texture
	///
	/// Returns the width of the texture at a given mip map level.
	/// @param iLevel Mip map level [0..GetNumMipLevels()-1] thats 
	/// width has to be returned.
	/// @return Width of the texture at that level or <i>0</i> in
	/// case of an error
	virtual int							GetWidth(int iLevel=0) const=0;

	/// @brief Returns the heigth of the texture
	///
	/// Returns the heigth of the texture at a given mip map level.
	/// @param iLevel Mip map level [0..GetNumMipLevels()-1] thats 
	/// heigth has to be returned.
	/// @return Height of the texture at that level or <i>0</i> in
	/// case of an error
	virtual int							GetHeight(int iLevel=0) const=0;

	/// @brief Returns the contents of one texture pixel.
	///
	/// Returns the color information of on texture pixel.<BR>
	/// <B>Not yet implemented</B>
	/// @param iX X coordinate of the pixel to be returned.
	/// @param iY Y coordinate of the pixel to be returned.
	/// @param iLevel Mip map level of the pixel to be returned.
	/// @return Returns the RGB values of the pixel.
	virtual	PNPTC_RGB&					GetPixel(int iX,int iY,int iLevel=0) const=0;

	/// @brief Sets one texture pixel
	///
	/// Sets the color of one texture pixel.<BR>
	/// <B>Not yet implemented</B>
	/// @param iX X coordinate of the pixel to be set.
	/// @param iY Y coordinate of the pixel to be set.
	/// @param color Color to be set.
	/// @param iLevel Mip map level of the pixel to be set.
	virtual void						SetPixel(int iX,int iY,PNPTC_RGB& color,int iLevel=0)=0;


	// --------------------------------------------------------------------------------------------
	// additional methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the normal map of the texture.
	///
	/// The normal map for this texture is returned.
	/// <B>Currently only returns a uniform normal map</B>
	/// @return Returns a pointer to the normal map texture
	/// if successfull or <i>NULL</i> in case of an error.
	/// If this texture itself is a normal map, <i>NULL</i>
	/// is returned.
	virtual CIPnPTCTexture01*			GetNormalMap() const=0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this texture.
	/// @return Returns the texture's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

	/// @brief Returns a global texture comment.
	///
	/// Returns a comment text which was specified for
	/// for this type of texture globally in the texture manager.
	/// @return Comment text for this texture.
	virtual const char*					GetComments() const=0;

}; // class


#pragma pack(pop)


#endif // !defined(PNPTC_CITEXTURE_H_INCLUDED)