/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCVegetation.h
// Interface definition to access vegetation objects.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. January 2005
// ----------------------------------------------------------------------------------
// change history:
//  29. Dec. 2006 (Ralf Westphal)
//		Added CIPnPTCGrass01(), made CIPnPTCTree01 methods const
//  19. Feb. 2007 (Ralf Westphal)
//		Added GetID.
//  26. Mai. 2007 (Ralf Westphal)
//		Added GetTreeCategory() and GetGrassCategory() and added format, 
//		scale, uprotation, and flipaxis to the save-to methods.
//		Renamed GetName() to GetTreeName() and GetName() to GetGrassName().
//  08. Sep. 2007 (Ralf Westphal)
//		Added GetTreeScale() and GetGrassScale()
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
//  04. May 2008 (Ralf Westphal)
//		Added GetComments()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIVEGETATION_H_INCLUDED)
#define PNPTC_CIVEGETATION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)



/// @brief Callback accessor class for trees.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to a tree object.
class PNPTCPLUGIN_CALLBACK CIPnPTCTree01
{
public:

	// --------------------------------------------------------------------------------------------
	// base tree and file methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the internal name of the tree.
	///
	/// Returns the internal name of the tree as it is used
	/// within the main program.
	/// @return Returns the internal name of the tree.
	virtual const char*					GetTreeName() const=0;

	/// @brief Returns the category of the tree.
	///
	/// Returns the category of the tree as it is used
	/// within the main program.
	/// @return Returns the category of the tree.
	virtual const char*					GetTreeCategory() const=0;

	/// @brief Returns the ID/index of the tree.
	///
	/// Returns the ID/index of the tree as it is used
	/// within the main program and vegetation manager plugin.
	/// @return Returns the ID/index of the tree.
	virtual WORD						GetID() const=0;

	/// @brief Returns the scale factor for this tree.
	///
	/// Returns the scale factor for this tree.
	/// @return Returns the scale factor for this tree.
	virtual float						GetTreeScale() const=0;


	// --------------------------------------------------------------------------------------------
	// methods to save the tree
	// --------------------------------------------------------------------------------------------

	/// @brief Saves the tree to a file.
	///
	/// Saves the tree to a file.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the tree should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool						SaveToFile(const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL) const=0;
	
	/// @brief Saves the tree to memory.
	///
	/// Saves the tree file to the memory.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool						SaveToMem(const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0) const=0;


	// --------------------------------------------------------------------------------------------
	// additional methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this tree.
	/// @return Returns the tree's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

	/// @brief Returns a global tree comment.
	///
	/// Returns a comment text which was specified for
	/// for this type of tree globally in the vegetation manager.
	/// @return Comment text for this tree.
	virtual const char*					GetComments() const=0;

}; // class CIPnPTCTree01



/// @brief Callback accessor class for grasses.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to a grass object.
class PNPTCPLUGIN_CALLBACK CIPnPTCGrass01
{
public:

	// --------------------------------------------------------------------------------------------
	// base grass and file methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the internal name of the grass.
	///
	/// Returns the internal name of the grass as it is used
	/// within the main program.
	/// @return Returns the internal name of the grass.
	virtual const char*					GetGrassName() const=0;

	/// @brief Returns the category of the grass.
	///
	/// Returns the category of the grass as it is used
	/// within the main program.
	/// @return Returns the category of the grass.
	virtual const char*					GetGrassCategory() const=0;

	/// @brief Returns the ID/index of the grass.
	///
	/// Returns the ID/index of the grass as it is used
	/// within the main program and vegetation manager plugin.
	/// @return Returns the ID/index of the grass.
	virtual WORD						GetID() const=0;

	/// @brief Returns the scale factor for this grass.
	///
	/// Returns the scale factor for this grass.
	/// @return Returns the scale factor for this grass.
	virtual float						GetGrassScale() const=0;


	// --------------------------------------------------------------------------------------------
	// methods to save the grass
	// --------------------------------------------------------------------------------------------

	/// @brief Saves the grass to a file.
	///
	/// Saves the grass to a file.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the grass should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool						SaveToFile(const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL) const=0;
	
	/// @brief Saves the grass to memory.
	///
	/// Saves the grass file to the memory.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool						SaveToMem(const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0) const=0;


	// --------------------------------------------------------------------------------------------
	// additional methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this grass.
	/// @return Returns the grass's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

	/// @brief Returns a global grass comment.
	///
	/// Returns a comment text which was specified for
	/// for this type of grass globally in the vegetation manager.
	/// @return Comment text for this grass.
	virtual const char*					GetComments() const=0;

}; // class CIPnPTCGrass01


#pragma pack(pop)


#endif // !defined(PNPTC_CIVEGETATION_H_INCLUDED)