/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCWaterDefinition.h
// Interface definition of an object containing all settings
// for a water surface.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. Sep. 2006
// ----------------------------------------------------------------------------------
// change history:
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIWATERDEFINITION_H_INCLUDED)
#define PNPTC_CIWATERDEFINITION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptctexture.h"
#include "./cipnptcpropertyvalue.h"


/// @brief Contains the settings for a water surface.
///
/// An instance of this interface is passed to your
/// plugin to give you information about the settings
/// of a water surface definition.
class PNPTCPLUGIN_CALLBACK CIPnPTCWaterDefinition01
{
public:

	/// @brief Returns the name of this water definition.
	/// 
	/// Returns the name of this water definition.
	/// @return Name string of the water definition
	virtual const char*					GetName() const = 0;

	/// @brief Sets the name of this water definition.
	/// 
	/// Sets the name of this water definition.
	/// @param cName New name of the water definition.
	/// Whitespaces on the left and right of the name
	/// string are trimmed.
	virtual void						SetName(const char* cName) = 0;

	/// @brief Returns the GUID of the water definition.
	/// 
	/// Returns the GUID of the water definition.
	/// @return Water definition's GUID.
	virtual GUID						GetGUID() const = 0;

	/// @brief Returns the color used to display the water in 2D mode.
	/// 
	/// Returns the color used to display the water in 2D mode.
	/// @return Color value for 2D water display.
	virtual PNPTC_RGB					Get2DColor() const = 0;

	/// @brief Sets the color used to display the water in 2D mode.
	/// 
	/// Sets the color used to display the water in 2D mode.
	/// @param col Color value for 2D water display.
	virtual void						Set2DColor(const PNPTC_RGB& col) = 0;

	/// @brief Returns the color used to render the water in 3D mode.
	/// 
	/// Returns the color used to render the water in 3D mode.
	/// @return Color value for 3D water rendering.
	virtual PNPTC_RGBA					Get3DColor() const = 0;

	/// @brief Sets the color used to render the water in 3D mode.
	/// 
	/// Sets the color used to render the water in 3D mode.
	/// @param col Color value for 3D water rendering.
	virtual void						Set3DColor(const PNPTC_RGBA& col) = 0;

	/// @brief Returns whether or not texture alpha is used for rendering.
	/// 
	/// Returns whether or not texture alpha is used for rendering.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// texture alpha is used for rendering.
	virtual bool						GetUseTextureAlpha() const = 0;

	/// @brief Sets whether or not texture alpha is used for rendering.
	/// 
	/// Sets whether or not texture alpha is used for rendering.
	/// @param bUseTextureAlpha <i>true</i> or <i>false</i>, whether or not
	/// texture alpha is used for rendering.
	virtual void						SetUseTextureAlpha(bool bUseTextureAlpha) = 0;

	/// @brief Returns the name of the water's texture.
	/// 
	/// Returns the name of the water's texture.
	/// @return A string containing the name of the water's texture.
	virtual const char*					GetTextureName() const = 0;

	/// @brief Returns a pointer to the water texture object.
	/// 
	/// Returns a pointer to the water texture object.
	/// @return Pointer to the water texture object if successfull,
	/// otherwise <i>NULL</i>.
	virtual CIPnPTCTexture01*			GetTexture() = 0;

	/// @brief Sets the name of the water texture.
	/// 
	/// Sets the name of the texture which should be used for
	/// water rendering in 3D.
	/// @param cTextureName Name of the new water surface texture.
	/// @return Returns <i>true</i>, if the texture could be set
	/// successfully, otherwise <i>false</i>.
	virtual bool						SetTexture(const char* cTextureName) = 0;

	/// @brief Returns the texture's scaling factor.
	/// 
	/// Returns the textures scaling factor. This value defines
	/// how often the water texture is repeated per heightmap element
	/// resp. over how many heightmap elements the texture is spanned.
	/// @return Water texture scaling factor. Positive values define
	/// the number of heightmap elements, over which the texture is spanned.
	/// Negative values indicate, that the texture is repeated several times
	/// per heightmap element (the absolute value will be the number of
	/// texture repetitions). A return value of <i>0</i> indicates an error.
	virtual int							GetTextureScaling() const = 0;

	/// @brief Sets the texture's scaling factor.
	/// 
	/// Sets the textures scaling factor. This value defines
	/// how often the water texture is repeated per heightmap element
	/// resp. over how many heightmap elements the texture is spanned.
	/// @param iScaling Water texture scaling factor. Positive values define
	/// the number of heightmap elements, over which the texture is spanned
	/// (possible values are 1, 2, 4, 8, 16, 32, 64, 128).
	/// Negative values indicate, that the texture is repeated several times
	/// per heightmap element (the absolute value will be the number of
	/// texture repetitions, possible values are -1, -2, -3, -4, -5, -6, -7, -8).
	virtual void						SetTextureScaling(int iScaling) = 0;

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this water definition.
	/// @return Returns the water definition's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

}; // class



#endif // !defined(PNPTC_CIWATERDEFINITION_H_INCLUDED)