/////////////////////////////////////////////////////////////////////////////////////
// IPnPTC3DObjectPlugIn.h
// Interface definition of the 3d object engine plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. Aug. 2004
// ----------------------------------------------------------------------------------
// change history:
//  19. Feb. 2007 (Ralf Westphal)
//		Redesigned category management and made chars const.
//		added SaveObjectToFile, SaveObjectToMem, DeleteObject, AddObject, and 
//		GetSupportedAddObjectFileExtensions.
//		Added color value to intersection test.
//      (Sorry for breaking compatibility)
//  26. Mai. 2007 (Ralf Westphal)
//		Added format, scale, uprotation, and flipaxis parameter to the 
//		save-to methods.
//  01. Aug. 2007 (Ralf Westphal)
//      Added GetDefaultGlobalComments, GetDefaultPlacementComments.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IOBJECTPLUGIN_H_INCLUDED)
#define PNPTC_IOBJECTPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


/// Interface definition for the 3d object PlugIn API.<br>
/// The implementation of this interface should do the following things:<br>
/// <b>1.</b> Enum all displayable objects from a given directory in the method
/// Init and store textual IDs for these objects. For example the filenames.<br>
/// The single objects should be accessable from the PnP-TC via these textual IDs.
/// PnP-TC will create 3D instances for the rendering from these objects and store its
/// textual IDs.<br>
/// <b>2.</b> Create and manage an arbitrary number of 3D objects (called 3D 
/// object instances) created from the managed objects via the method Create3DInstance.
class PNPTCPLUGIN_API IPnPTC3DObjectPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int					GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID		GetGUID()=0;

	/// Initialization of object interface.<br>
	/// This method is called directly after a terrain document
	/// is loaded or the object plugin for an already loaded
	/// terrain is changed.<br>
	/// You should scan the given object path for all object files
	/// supported by your plugin and create a list of these files.
	/// @param cObjectPath Directory where the object files for 
	/// this terrain are located. Usually this is a directory
	/// with the name <i>Objects</i> directly under the main
	/// terrain file location.
	/// @param pDevice Reference to the rendering device, that
	/// will render the terrain and the objects.<br>
	/// <b>This will be a pointer to an object of the class 
	/// &quot;IDirect3DDevice9&quot;</b>
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and the object engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool		Init(const char* cObjectPath,void* pDevice)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin.You should reload/rescan the
	/// objects from the object directory within this method.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull and the object engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool		Reset()=0;


	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// Performs any cleanups needed before this class is gonna
	/// be destroyed.<br>
	/// This method is called when a terrain document is to
	/// be closed or if the plugin used for a terrain was changed.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be shure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void		Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Access methods to the objects managed by this plugin
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the number of available objects.
	///
	/// @return Number of objects.
	virtual int			GetObjectCount()=0;

	/// @brief Returns object's name.
	///
	/// Returns the name (which is the ID) of a object.<br>
	/// You should make sure that every object managed by your plugin
	/// has an unique name.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object.
	/// @return Return the null terminated character id-string of the object if 
	/// successfull. Return <i>null</i> in case of an error.
	virtual const char*	GetObjectName(int iObject)=0;

	/// @brief Returns the object category.
	///
	/// Returns the category and sub category of an object.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object.
	/// @return Returns the category string of the object. Subcategories are
	/// divided by a '\' (example "maincategory\subcategory\subcategory2").
	/// In case of an error or if the category of the object is not preset, 
	/// <i>NULL</i> is returned.<br>
	virtual const char*	GetObjectCategory(unsigned int iObject)=0;

	/// @brief Returns object's index.
	///
	/// Gets the index of a given object id within the list of objects managed 
	/// by this plugin.
	/// @return Returns the zero based index of the object. Returns a value < 0
	/// to indicate that the given object id is not managed by the plugin.
	virtual int			GetObjectIndex(const char* cObjectName)=0;


	// --------------------------------------------------------------------------------------------
	// methods for 3D placement/instatiations of objects
	// --------------------------------------------------------------------------------------------

	/// @brief Creates a new 3d object instance.
	///
	/// Creates a new 3d object instance for rendering
	/// for a given object ID.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// for which a 3D instance should be created.
	/// @return Returns an ID (>0) for the create 3D object instance.
	/// Returns <i>0</i> in case of an error.
	virtual int			Create3DInstace(int iObject)=0;

	/// @brief Deletes one 3d object instance.
	///
	/// @param iInstance Instance ID of the object to be deleted as returned
	/// from Create3DInstace.
	virtual void		Delete3DInstance(int iInstance)=0;

	/// @brief Deletes all 3d object instances.
	///
	/// Deletes all 3d object instances.
	/// This method is called when the detail edit mode of
	/// a sector is to be closed.
	virtual void		DeleteAll3DInstances()=0;

	/// @brief Sets 3d objects instance transformation.
	///
	/// @param iInstance Instance ID of the object thats transformation matrix 
	/// is to be set as returned from Create3DInstace.
	/// @param matTransform 4x4 transformation matrix containing rotation, scaling
	/// and translation in row major format.
	virtual void		Set3DInstanceTransformation(int iInstance,float matTransform[16])=0;

	/// @brief Returns information of one 3d object instance.
	///
	/// @param iInstance Instance ID of the object to return its data.
	/// @param iFaces (output) Number of faces of this object instance.
	/// @param iVertices (output) Number of vertices of this object instance.
	/// @return <i>true</i> if the object's data could be returned successfully,
	/// <i>false</i> in case of an error.
	virtual bool		GetObjectInfo(int iInstance,int* iFaces,int* iVertices)=0;

	/// @brief Renders a 3d object.
	///
	/// Renders a given 3d object instance.
	/// @param iInstance 3d object instance ID to be rendered.
	/// @return <i>true</i> if the object could be rendered successfully,
	/// <i>false</i> in case of an error.
	virtual bool		Render(int iInstance)=0;

	/// @brief Get object's bounding box.
	///
	/// This method should return an object's bounding
	/// box in world coordinates using the current object
	/// transformation.
	/// @param iInstance Instance ID of the object thats bounding box has 
	/// to be returned.
	/// @param vecVertex1 (output) Coordinates of one corner of the object's
	/// bounding box in world coordinates.
	/// @param vecVertex2 (output) Coordinates of the opposite corner of the object's
	/// bounding box in world coordinates.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the bounding
	/// box could be returned successfully.
	virtual bool		GetBoundingBox(int iInstance,float vecVertex1[3], float vecVertex2[3])=0;

	/// @brief: Returns whether or not an object is hit by a mouse click.
	///
	/// This method should test a given ray against intersections with 
	/// one 3d object instance.
	/// @param iInstance Instance ID of the object that has to be checked.
	/// @param vecStart Starting point of a ray in world coordinates.
	/// @param vecDir Direction of the ray in world coordinates.
	/// @param pRGBA (output) Pointer to a color value. If a valid pointer
	/// is passed with the method call, the caller expects a color value including 
	/// the alpha component from the position where the object was hit as return value.
	/// @return Returns <i>true</i> if the object is hit by the given ray.
	/// Returns <i>false</i> if not.<br>
	/// <b>Remarks:</b> This method will only be called if a previous intersection
	/// test performed by PnP-TC against the object's bounding box ( as returned
	/// by GetBoundingBox) was successfull.
	/// So you should simply return <i>true</i> if you don't want to implement
	/// a detailed intersection test here.
	virtual bool		Intersect(int iInstance,float vecStart[3],float vecDir[3],PNPTC_RGBA* pRGBA)=0;


	// --------------------------------------------------------------------------------------------
	// Object management
	// --------------------------------------------------------------------------------------------

	/// @brief Deletes one object.
	///
	/// Deletes one object which is managed by this plugin.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// which should be deleted.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// object could be deleted.
	virtual bool		DeleteObject(int iObject)=0;

	/// @brief Returns supported file extensions.
	/// 
	/// Returns the file extensions which are allowed for adding object
	/// files to the list of objects managed by this plugin.
	/// @return Returns a string containing the supported file extensions.
	/// If more than one file extension is supported, the single extensions
	/// should be seperated by a ';' (for example "*.obj;*.3ds;*.b3d").
	virtual const char*	GetSupportedAddObjectFileExtensions()=0;

	/// @brief Returns the default global object comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of object in the object manager.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// for which the default comment text should be returned.
	/// @return Default comment text for this object.
	virtual const char*	GetDefaultGlobalComments(int iObject) const=0;

	/// @brief Returns the default object placement comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of object for the object placements.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// for which the default comment text should be returned.
	/// @return Default comment text for this object.
	virtual const char*	GetDefaultPlacementComments(int iObject) const=0;

	/// @brief Adds one object.
	///
	/// Adds one object which should be loaded from a given file.
	/// @param cFileName The file name of the object to be added.
	/// @param cCategory The category (including subcategories separated by '\')
	/// into which this object should be placed.
	/// @return Returns the index of the newly added object if successfull, 
	/// or <i>-1</i> in case of an error.
	virtual int			AddObject(const char* cFileName,const char* cCategory)=0;

	/// @brief Saves the object to a file.
	///
	/// Saves the object to a file.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the object should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool		SaveObjectToFile(int iObject,const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL)=0;
	
	/// @brief Saves the object to memory.
	///
	/// Saves the object file to the memory.
	/// @param iObject The index in the range [0...GetObjectCount()-1] of the object
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool		SaveObjectToMem(int iObject,const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0)=0;

}; // class

#pragma pack(pop)

#endif // !defined(PNPTC_IOBJECTPLUGIN_H_INCLUDED)
