/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCHMapFilterPlugIn.h
// Interface definition of the height map filter plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//  26. Feb. 2004 (Ralf Westphal)
//		Added ButtonState to Apply method
//  18. Jul. 2004 (Ralf Westphal)
//		Added heightmap base class
//  21. Nov. 2007 (Ralf Westphal)
//		Added fPressure to support pressure sensitive graphics tablets.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IFILTERPLUGIN_H_INCLUDED)
#define PNPTC_IHMAPFILTERPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"

// include callback object interfaces
#include "./cipnptcheightmap.h"
#include "./cipnptcbrush.h"


// event of filter application
#define PNPTC_FILTEREVENT_START		0		//!< start of filter application
#define PNPTC_FILTEREVENT_MOVE		1		//!< movement of filter during application
#define PNPTC_FILTEREVENT_END		2		//!< filter application finished



/// @brief Interface definition for the heightmap filter PlugIn API.
/// 
/// With an implementation of this interface the height values of a terrain
/// can be modified within a given brush region.<br>
/// The implementation of this interface should do the following things:<br>
/// <b>1.</b> Do some initializations if necessary within the Init() method.<br>
/// <b>2.</b> Apply the filter to the heightmap of a sector. Therefore the
/// sector heights at the positions specified within the CIPnPTCBrush01 object
/// should be read from the CIPnPTCHeightmap01 object using the CIPnPTCHeightmap01::GetHeight 
/// method, filtered and rewritten to the sector using 
/// the CIPnPTCHeightmap01::SetHeight method.
class PNPTCPLUGIN_API IPnPTCHMapFilterPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Returns the UID of this plugin.<br>
	/// Create an unique identifier for your plugin using
	/// the guidgen-utility of VisualC or any other creation
	/// utility.<br>
	/// @return The unique identifier of this plugin.
	virtual GUID	GetGUID()=0;

	/// @brief Plugin initialization.
	///
	/// Initialization of filter interface.<br>
	/// This method is called directly after the start of
	/// the main application.<br>
	/// Do every one time initializations that have to be done
	/// within this method.
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and the filter is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Init()=0;


	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// @brief Plugin cleanup.
	///
	/// Performs any cleanups needed before this class will be destroyed.<br>
	/// This method is called when the main application is terminated.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be sure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void	Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Methods to handle settings windows creation
	// --------------------------------------------------------------------------------------------

	/// @brief Creation of a filter settings window.
	/// 
	/// Creates a subwindow of the main PnP-TC window to make
	/// userdefined settings for the filter possible.<br>
	/// <b>Remember</b> to internally store references to the 
	/// created windows in a list (or don't store them at
	/// all), as there is exactly one instance of your
	/// plugin for the whole lifetime of a PnP-TC session,
	/// while there can be several terrain windows
	/// opened at the same time resulting in the same number
	/// of instances of your settings window.
	/// @param hParent Window handle to the parent (the PnP-TC
	/// main window) window.
	/// @return Returns the window handle for the newly
	/// created settings window. Returns <i>NULL</i> if
	/// no window was created.
	virtual	HWND	CreateSettingsWindow(HWND hParent)=0;

	/// @brief Destruction of a settings window
	///
	/// Destroys one instance of the plugin's settings window.
	/// @param hWnd Window handle of the settings window
	/// to be destroyed.
	virtual void	DestroySettingsWindow(HWND hWnd)=0;


	// --------------------------------------------------------------------------------------------
	// Applying the filter
	// --------------------------------------------------------------------------------------------

	/// @brief applying the filter.
	///
	/// Apply the filter on the given sector for the given brush.
	/// @param Heightmap The heightmap at which the filter should be applied.
	/// You can read values from the whole range of accessible
	/// heightmap elements but you can only write to those contained in
	/// the brush, passed to this method.
	/// @param Brush The selected heightmap positions to that the filter
	/// should be applied
	/// @param hWnd Handle to the window with the filter specific settings.
	/// If there is no window associated with this filter or the window
	/// could not be created, this parameter is <i>NULL</i>.
	/// @param ButtonState Current state of mouse buttons and keyboard.
	/// @param FilterEvent Filter event of this filter execution
	/// (PNPTC_FILTEREVENT_START, PNPTC_FILTEREVENT_MOVE, PNPTC_FILTEREVENT_END)<br>
	/// If filter event is PNPTC_FILTEREVENT_END, then the Brush parameter
	/// contains no elements. The end event is only executed to inform the
	/// plugin of a finished filter application and so allows it to do some
	/// post processing actions.
	/// @param fPressure The pressure/force with which the filter should be applied.
	/// The pressure value is used to support pressure sensitive graphics tablets
	/// and is in the range [0..2] with <i>0</i> beeing the lowest and <i>2</i> the 
	/// highest pressure force. If no graphics tablet is used - ordinary mouse or 
	/// keyboard - this value is always <i>1</i>.
	/// @return Returns <i>true</i> if the filter could be applied
	/// successfully. <i>false</i> in case of an error.
	virtual bool	Apply(CIPnPTCHeightmap01& Heightmap,CIPnPTCBrush01& Brush,HWND hWnd,WORD ButtonState,int FilterEvent,float fPressure)=0;

}; // class

#endif // !defined(PNPTC_IFILTERPLUGIN_H_INCLUDED)