/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCHMapGenerationPlugIn.h
// Interface definition of the height map generation plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	01. Jun. 2004
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IHMAPGENERATIONPLUGIN_H_INCLUDED)
#define PNPTC_IHMAPGENERATIONPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"

// includes
#include "CIPnPTCSector.h"
#include "CIPnPTCSectorDefinition.h"
#include "CIPnPTCTerrain.h"


/// @brief Interface to automated terrain generation
///
/// Interface definition for the API of automated, randomized
/// heightmap generation.<br>
class PNPTCPLUGIN_API IPnPTCHMapGenerationPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// Returns the UID of this plugin.<br>
	/// Create an unique identifier for your plugin using
	/// the guidgen-utility of VisualC or any other creation
	/// utility.<br>
	/// @return The unique identifier of this plugin.
	virtual GUID	GetGUID()=0;

	/// Returns the Version of the implemented plugin API.<br>
	/// Don't override this methode, as it should always return
	/// the predefined value.<br>
	/// @return Implemented API version.
	int				GetVersion() {return 1;};


	// --------------------------------------------------------------------------------------------
	// PlugIns capabilities
	// --------------------------------------------------------------------------------------------

	/// @brief Returns randomization capabilities.
	///
	/// Returns whether or not this plugin is able to create
	/// randomized terrains.
	/// @return <i>true</i> if this plugin can create randomized terrains.
	/// <i>false</i> if it can not.
	virtual bool	SupportsRandomization()=0;

	/// @brief Returns detail generation capabilities.
	///
	/// Returns whether or not this plugin is able to create
	/// a sectors detailled height map.
	/// @return <i>true</i> if this plugin can create detailled height maps.
	/// <i>false</i> if it can not.
	virtual bool	SupportsDetailGeneration()=0;


	// --------------------------------------------------------------------------------------------
	// Methods to handle settings windows creation
	// --------------------------------------------------------------------------------------------

	/// @brief Creation of a settings window.
	/// 
	/// Creates a subwindow of the main PnP-TC window to enter
	/// heightmap generation settings.<br>
	/// @param hParent Window handle to the parent (the PnP-TC
	/// main window) window.
	/// @return Returns the window handle for the newly
	/// created settings window. Returns <i>NULL</i> if
	/// no window was created.
	virtual	HWND	CreateSettingsWindow(HWND hParent)=0;

	/// @brief Destruction of a settings window.
	///
	/// Destroys the plugin's settings window.
	/// @param hWnd Window handle of the settings window to be destroyed.
	virtual void	DestroySettingsWindow(HWND hWnd)=0;

	/// @brief Validates inputs in settings window.
	///
	/// Validates user input in this settings window.
	/// This method is called from the PnP-TC right before
	/// the heightmap generation settings will be saved.
	/// @param hWnd Window handle of the settings window to be validated.
	/// @return Returns <i>true</i> if the user's settings are correct
	/// an can therefore be saved. Returns <i>false</i> in case of
	/// an error.
	virtual bool	ValidateSettingsWindow(HWND hWnd)=0;

	/// @brief Clear the settings window.
	///
	/// Clears all entries of this settings window.
	/// @param hWnd Window handle of the settings window to be cleared.
	virtual void	ClearSettingsWindow(HWND hWnd)=0;

	/// @brief Displays random generation settings.
	///
	/// Displays random generation settings within a settings window.
	/// @param hWnd Window handle of the settings window to be displayed.
	/// @param pMem Pointer to the PlugIns data to be displayed.
	/// @param dwMemSize Size of the memory pointed to by pMem.
	/// @return Returns whether or not the settings could be displayed correctly.
	virtual bool	DisplaySettings(HWND hWnd,void* pMem,DWORD dwMemSize)=0;

	/// @brief Returns the user's inputs from a random generation settings window.
	///
	/// @param hWnd Window handle of the settings window.
	/// @param pMem Return parameter pointing to the memory containing the 
	/// generation parameters. If the method failes a NULL pointer is returned.<br>
	/// <b>Remarks</b>: The pointer returned is a temporary pointer. You 
	/// should copy the contents of it directly after the call.
	/// @param dwMemSize Returns the amount of bytes stored in the memory block.
	/// If the method fails <i>0</i> is returned.
	/// @return Returns <i>true</i> if the parameters could be retrieved successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool	UpdateSettings(HWND hWnd,void** pMem,DWORD* dwMemSize)=0;


	// --------------------------------------------------------------------------------------------
	// Heightmap generation
	// --------------------------------------------------------------------------------------------

	/// @brief Generates the heightmap.
	/// 
	/// An implementation of this method should automatically
	/// generate the complete definition heightmap of a terrain.<br>
	/// The sector detail heightmaps should be generated too, if supported.
	/// @param Terrain Reference to the main terrain object.
	/// @param bGenerateDetail <i>true</i> if the detailled sector
	/// heightmaps should be created. <i>false</i> if not.
	/// @return Returns <i>true</i> if creation was successfull.
	/// Returns <i>false</i> in case of an error.
	virtual bool	GenerateTerrainHeightMap(CIPnPTCTerrain01& Terrain, bool bGenerateDetail)=0;

	/// @brief Generates the sectors heightmap.
	/// 
	/// An implementation of this method should automatically
	/// generate the definition heightmap of one terrain sector.<br>
	/// The sectors detail heightmap should be generated too, if supported.
	/// @param Sector Reference to the sector object thats heightmap has to be created.
	/// @param bGenerateDetail <i>true</i> if the detailed sector
	/// heightmaps should be created. <i>false</i> if not.
	/// @return Returns <i>true</i> if creation was successfull.
	/// Returns <i>false</i> in case of an error.
	virtual bool	GenerateSectorHeightMap(CIPnPTCSectorDefinition01& Sector, bool bGenerateDetail)=0;

	/// @brief Generates sectors detail height map.
	/// 
	/// Generates the sectors detailed heightmap from the
	/// sector definition heightmap.<br>
	/// <b>Remarks:</b> This functionality is not yet suppoted.
	/// @param Sector The sector thats detailmap has to be created.
	/// @return Returns <i>true</i> if the heightmap could be created.
	/// Returns <i>false</i> in case of error.
	virtual bool	GenerateDetailHeightMap(CIPnPTCSectorDefinition01& Sector)=0;

}; // class



#endif // !defined(PNPTC_IHMAPGENERATIONPLUGIN_H_INCLUDED)