/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCInputDevicePlugIn.h
// Interface definition for input device plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Mai 2008
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IINPUTDEVICEPLUGIN_H_INCLUDED)
#define PNPTC_IINPUTDEVICEPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


// forward declarations
class CIPnPTCApplication01;


/// @brief Terrain access class.
///
/// Access to the PnPTC application to update the progress
/// bar or to browse for files/folders.
class PNPTCPLUGIN_CALLBACK CIPnPTCInputDeviceContext01
{
public:

	// --------------------------------------------------------------------------------------------
	// Callback methods for the input device
	// --------------------------------------------------------------------------------------------

	/// @brief The plugin's status window has changed.
	///
	/// Called, whenever the plugin's status window has
	/// changed and needs to be updated by the main application.
	virtual void		OnStatusWindowChanged()=0;

	/// @brief Moves the viewport.
	///
	/// @param fTx The translational movement component in x direction.
	/// @param fTy The translational movement component in y direction.
	/// @param fTz The translational movement component in z direction.
	/// @param fRx The rotational movement component around the x axis.
	/// @param fRy The rotational movement component around the y axis.
	/// @param fRz The rotational movement component around the z axis.
	virtual void		MoveViewport(float fTx,float fTy,float fTz,float fRx,float fRy,float fRz)=0;

	/// @brief Moves the current selection.
	///
	/// @param fTx The translational movement component in x direction.
	/// @param fTy The translational movement component in y direction.
	/// @param fTz The translational movement component in z direction.
	/// @param fRx The rotational movement component around the x axis.
	/// @param fRy The rotational movement component around the y axis.
	/// @param fRz The rotational movement component around the z axis.
	virtual void		MoveSelection(float fTx,float fTy,float fTz,float fRx,float fRy,float fRz)=0;

}; // class CIPnPTCInputDeviceContext01





/// @brief Interface definition for the texture PlugIn API.
/// 
/// An implementation of this interface enables the TerrainCreator to manage surface textures.<br>
/// The implementation of this interface should do the following things:<br>
/// Enum all textures from a given directory in the method
/// Init() and store textual IDs for these textures. For example the filenames.<br>
/// Each single texture should be accessible from the PnP-TC via these textual IDs.<br>
class PNPTCPLUGIN_API IPnPTCInputDevicePlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int						GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID			GetGUID()=0;

	/// @brief Plugin initialization.
	///
	/// Initialization of input device interface.<br>
	/// This method is called directly after the program was started.<br>
	/// You should perform all necessary initializations in this method.
	/// @param pApplication The active PnPTC application.
	/// @param pContext A context, in which this plugin runs and which
	/// can be used to post back data asynchronously to the application.
	/// @param hwnd Handle to the main window.
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull. Returns <i>false</i> in case of an error.
	virtual bool			Init(CIPnPTCApplication01* pApplication,CIPnPTCInputDeviceContext01* pContext,HWND hwnd)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull. Returns <i>false</i> in case of an error.
	virtual bool			Reset()=0;

	/// @brief Whether or not this input device is available.
	///
	/// Returns whether or not this input device is available
	/// and can be used for input processing.
	/// @return Returns <i>true</i> if the device is available
	/// or <i>false</i> if not.
	virtual bool			GetIsAvailable()=0;


	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// @brief Plugin cleanup.
	///
	/// Performs any cleanups needed before this class will be destroyed.<br>
	/// This method is called when program is to be closed.
	virtual void			Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Input processing
	// --------------------------------------------------------------------------------------------

	/// @brief Whether or not this plugin should be processed during mouse events.
	///
	/// @return Returns <i>true</i> if this input plugin should
	/// be processed during mouse events or <i>false</i> if not.
	virtual bool			GetProcessInMouseEvents()=0;

	/// @brief Processes the input device.
	///
	/// @return Whether or not the processing was successful.
	virtual bool			Process()=0;

	/// @brief Information about the applications input focus.
	///
	/// Informs the plugin, when the main application receives
	/// or looses the input focus.
	/// @param bFocus <i>true</i> if the main application receives
	/// the focus, or <i>false</i> if it loses it.
	virtual void			SetFocus(bool bFocus)=0;

	/// @brief Returns the position of a 2D pointing device.
	///
	/// @return The 2D pointer position.
	virtual PnPTC2DVec		Get2DPosition() const=0;

	/// @brief Returns the pressure of a tablet input device.
	///
	/// @return The pressure with which a pen was pressed on
	/// a tablet input device in the range [0..1].
	virtual float			GetPressure() const=0;


	// --------------------------------------------------------------------------------------------
	// The status window
	// --------------------------------------------------------------------------------------------

	/// @brief Whether or not this plugin has a status window.
	///
	/// @return Returns <i>true</i> if this plugin has a status window
	/// or <i>false</i> if not.
	virtual bool			GetHasStatusWindow() const=0;

	/// @brief Returns the bitmap of the status window.
	///
	/// @return The bitmap to be displayed as status window.
	virtual HBITMAP			GetStatusWindowBitmap() const=0;

	/// @brief Returns the number of context menue entires.
	///
	/// @return The number of context menue entires.
	virtual unsigned int	GetNumContextMenuItems() const=0;

	/// @brief Returns the text of a context menu item
	///
	/// @param nItem The index of the context menu item for which
	/// the text should be returned in the range [0..GetNumContextMenuItems()].
	/// @return Returns the text of the context menu item.
	virtual const char*		GetContextMenuTest(unsigned int nItem) const=0;

	/// @brief Returns the enable state of a context menu item
	///
	/// @param nItem The index of the context menu item for which
	/// the enable state should be returned in the range [0..GetNumContextMenuItems()].
	/// @return Returns <i>true</i> if the context menu item is enabled, or 
	/// <i>false</i> if not.
	virtual bool			GetContextMenuEnableState(unsigned int nItem) const=0;

	/// @brief Returns the checked state of a context menu item
	///
	/// @param nItem The index of the context menu item for which
	/// the checked state should be returned in the range [0..GetNumContextMenuItems()].
	/// @return Returns <i>true</i> if the context menu item is checked, or 
	/// <i>false</i> if not.
	virtual bool			GetContextMenuCheckedState(unsigned int nItem) const=0;

	/// @brief Callback, when a context menu item was clicked.
	///
	/// @param nItem The index of the context menu item, which was clicked.
	virtual void			OnContextMenuItemClicked(unsigned int nItem)=0;


	// --------------------------------------------------------------------------------------------
	// The settings window
	// --------------------------------------------------------------------------------------------

	/// @brief Whether or not this plugin has a settings window.
	///
	/// @return Returns <i>true</i> if this plugin has a settings window
	/// or <i>false</i> if not.
	virtual bool			GetHasSettingsWindow() const=0;

	/// @brief Opens the plugin's settings window.
	virtual void			ShowSettingsWindow()=0;


}; // class IPnPTCInputDevicePlugIn01

#pragma pack(pop)

#endif // !defined(PNPTC_IINPUTDEVICEPLUGIN_H_INCLUDED)