/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCPhysicsPlugIn.h
// Interface definition of the physic engine plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	15. May 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IPHYSICSPLUGIN_H_INCLUDED)
#define PNPTC_IPHYSICSPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcsector.h"


#pragma pack(push)
#pragma pack(1)



/// Interface definition for the physics engine PlugIn API.<br>
/// The implementation of this interface should do the following things:<br>
/// <b>1.</b> Enum all displayable tree and grass objects from a given directory in the method
/// Init and store textual IDs for these objects. For example the filenames.<br>
/// The single objects should be accessable from the PnP-TC via these textual IDs.
/// PnP-TC will create 3D instances for the rendering from these trees and store their
/// textual IDs.<br>
/// <b>2.</b> Create and manage an arbitrary number of tree placements (called 3D 
/// tree instances) created from the managed tree objects via the method Create3DInstance.
class PNPTCPLUGIN_API IPnPTCPhysicsPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID	GetGUID()=0;

	/// Initialization of physics engine interface.<br>
	/// This method is called directly after a terrain document
	/// is loaded or the phyiscs plugin for an already loaded
	/// terrain is changed.<br>
	/// You should do all required initializations for your plugin 
	/// within this method.
	/// @param pDevice Reference to the rendering device, that
	/// will render all simulation objects.<br>
	/// <b>This will be a pointer to an object of the class 
	/// &quot;IDirect3DDevice9&quot;</b>
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and the physics engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Init(void* pDevice)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin.You should reload/reinitialize
	/// all required resources within this method.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull and the physics engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Reset()=0;


	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// Performs any cleanups needed before this class is gonna
	/// be destroyed.<br>
	/// This method is called when a terrain document is to
	/// be closed or if the plugin used for a terrain was changed.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be sure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void	Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Physic simulation
	// --------------------------------------------------------------------------------------------

	/// @brief Initializes the simulation.
	/// 
	/// Initializes the simulation by loading and placing 
	/// all simulated models.
	/// @param Sector Currently simulated sector.
	/// @param fScaleFactor3D Scaling factor used for the 3D rendering.
	/// @param fPos Current position vector of viewer.
	/// @param fDir Current direction vector of viewer.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// simulation could be initialized.
	virtual bool		InitSimulation(CIPnPTCSector01& Sector,float fScaleFactor3D,float fPos[3],float fDir[3])=0;

	/// @brief Proceed simulation.
	///
	/// Proceeds the physics simulation for the next time step
	/// by checking the user input and computing new positions for
	/// simulated objects.
	/// @param dTime time elapsed since last call of method in seconds.
	/// @param Sector Currently opened sector.
	/// @param fViewPos (output) Returns the viewer position from which
	/// the proceed physic sim can be seen best.
	/// @param fViewDir (output) Returns the viewing direction.
	/// @param fViewUp (output) Returns the up vector for the desired view.
	virtual void		ProceedSimulation(double dTime,CIPnPTCSector01& Sector,float fViewPos[3],float fViewDir[3],float fViewUp[3])=0;

	/// @brief Renders all graphic objects.
	///
	/// Renders all graphical objects visible within the physic simulation.
	/// @param matView 4x4 transformation matrix containing the current view 
	/// matrix in row major format.
	/// @param iWidth Width of the rendering screen.
	/// @param iHeight Height of the rendering screen.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// rendering was successfull.
	virtual bool		Render(float matView[16],int iWidth,int iHeight)=0;


	// --------------------------------------------------------------------------------------------
	// Methods for settings window
	// --------------------------------------------------------------------------------------------

	/// @brief Creation of a window to display PlugIn's settings.
	/// 
	/// Creates a subwindow of the PnP-TC 3d settings window to enter
	/// additional PlugIn specific settings.<br>
	/// @param hParent Window handle to the parent (the PnP-TC
	/// 3d settings window) window.
	/// @return Returns the window handle for the newly
	/// created window. Returns <i>NULL</i> if no window was created.
	virtual	HWND		CreateSettingsWindow(HWND hParent)=0;

	/// @brief Destruction of the settings window.
	///
	/// Destroys the plugin's settings window.
	/// @param hWnd Window handle of the window to be destroyed.
	virtual void		DestroySettingsWindow(HWND hWnd)=0;

	/// @brief validates and saves settings window.
	///
	/// This method validates and saves the contents of the
	/// PlugIn's settings window. It is called directly before
	/// the window is to be destroyed.
	/// @param hWnd Window handle of the window to be saved.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the data was valid and could be saved.
	virtual bool		ValidateAndSaveSettingsWindow(HWND hWnd)=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_IPHYSICSPLUGIN_H_INCLUDED)
