/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCPlugIn.h
// Interface definition of the plugin base class
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//  23. Feb. 2004 (Ralf Westphal)
//		Added states of buttons and keys 
//  24. Feb. 2005 (Ralf Westphal)
//		Added PnPTC3DVec type 
//	25. Mar. 2005 (Ralf Westphal)
//		Added PnPTCUV type
//  03. Sep. 2006 (Ralf Westphal)
//      Added PNPTC_RGBA
//  29. Oct. 2006 (Ralf Westphal)
//		Added color format conversion operators.
//		Added PNPTC_ARGB and PNPTC_BGRA
//  01. Feb. 2007 (Ralf Westphal)
//      Added operators (*,+) to color data types.
//  16. Mar. 2007 (Ralf Westphal)
//      Added comparison operators (==,!=) to color data types.
//  26. Mai  2007 (Ralf Westphal)
//      Moved vectors and colors to seperate files and move
//      UPROTATION_xxx and FLIP_xxx to this file.
//	18. Nov  2007 (Ralf Westphal)
//		Added IPnPTCPlugIn::GetName()
//  18. Mai  2008 (Ralf Westphal)
//		Added PNPTC_PLUGIN_APIID_INPUT to support input devices.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_PLUGIN_H_INCLUDED)
#define PNPTC_PLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/// needed for GUID
#include <windows.h>


// Make sure the symbol PNPTCPLUGIN_IMPORT is not defined when you compile your plugin!
#ifdef PNPTCPLUGIN_IMPORT		
#define PNPTCPLUGIN_API			__declspec(dllimport)
#define PNPTCPLUGIN_CALLBACK	__declspec(dllexport)
#else
#define PNPTCPLUGIN_API			__declspec(dllexport)
#define PNPTCPLUGIN_CALLBACK	__declspec(dllimport)
#endif


// include some base types
#include "IPnPTCVectors.h"
#include "IPnPTCColors.h"



#pragma pack(push)
#pragma pack(1)


// PlugIn API ID used by the PnP-TC
#define PNPTC_PLUGIN_APIID_SOUND		10		//!< Sound plugin api
#define PNPTC_PLUGIN_APIID_HMAPFILTER	11		//!< Heightmap filter functions
#define PNPTC_PLUGIN_APIID_VEGETATION	12		//!< tree and vegetation engine
#define PNPTC_PLUGIN_APIID_RANDOMHMP	13		//!< randomizer for heightmaps
#define PNPTC_PLUGIN_APIID_RANDOMTEX	14		//!< randomizer for surface textures (not yet implemented)
#define PNPTC_PLUGIN_APIID_RANDOMVEG	15		//!< randomizer for vegetation (not yet implemented)
#define PNPTC_PLUGIN_APIID_3DOBJECT		16		//!< 3D objects
#define PNPTC_PLUGIN_APIID_IMPORT		17		//!< Importfilter for terrains (not yet implemented)
#define PNPTC_PLUGIN_APIID_EXPORT		18		//!< Exportfilter for terrains and all other things
#define	PNPTC_PLUGIN_APIID_FALLOFF		19		//!< falloff effects for self defined workspaces
#define PNPTC_PLUGIN_APIID_TEXTURE		20		//!< Surface textures
#define PNPTC_PLUGIN_APIID_RENDER		21		//!< 3D rendering engine (not yet implemented)
#define PNPTC_PLUGIN_APIID_PHYSICS		22		//!< Physic simulation engine
#define PNPTC_PLUGIN_APIID_INPUT		23		//!< Input device plugin


// ---------------------------------------------------------------------------------
// some base types and definitions
// ---------------------------------------------------------------------------------


// States of buttons and keys
#define PNPTC_PLUGIN_LEFTMOUSE			1		//!< Left mouse button is pressed
#define PNPTC_PLUGIN_MIDDLEMOUSE		2		//!< Middle mouse button is pressed
#define PNPTC_PLUGIN_RIGHTMOUSE			4		//!< Right mouse button is pressed
#define PNPTC_PLUGIN_ALT				8		//!< Alt key is pressed
#define PNPTC_PLUGIN_CTRL				16		//!< Ctrl key is pressed
#define PNPTC_PLUGIN_SHIFT				32		//!< Shift key is pressed

// Possible orientations of one heightmap quad diagonal
#define PNPTC_QUADOR_BL2TR				0		//!< diagonal of a quad goes from the bottom-left to the top-right
#define PNPTC_QUADOR_TL2BR				1		//!< diagonal of a quad goes from the top-left to the bottom-right


/// Definitions for global up rotations. This rotation is applied
/// to all tree, grass, or object placements before all other placement specific rotations
/// are performed in order to align the tree's up axis accordingly to the
/// terrain's up axis.
#ifndef UPROTATION_NONE
#define UPROTATION_NONE					0		//!< no rotation is applied to the trees.
#define	UPROTATION_ROTX90				1		//!< rotation of 90 deg about x axis.
#define	UPROTATION_ROTX270				2		//!< rotation of -90 deg about x axis.
#define	UPROTATION_ROTX180				3		//!< rotation of 180 deg about x axis.
#endif

/// Definitions for tree flipping. The vertices of a tree, grass, or object are
/// flipped along this axis in order to convert between right and
/// left handed coordinate system usage.
#ifndef FLIP_NONE
#define FLIP_NONE						0		//!< trees are not flipped.
#define	FLIP_XAXIS						1		//!< the x values of the trees are flipped.
#define	FLIP_YAXIS						2		//!< the y values of the trees are flipped.
#define	FLIP_ZAXIS						3		//!< the z values of the trees are flipped.
#endif

// ---------------------------------------------------------------------------------
/// @brief Heightmap position
///
/// This data structure contains the coordinates
/// of one single heightmap element.
PNPTCPLUGIN_CALLBACK typedef struct PnPTCHMapPosition
{
	int x;				//!< x-position of heightmap.
	int y;				//!< y-position of heightmap.
} PnPTCHMapPosition;

// ---------------------------------------------------------------------------------
/// @brief A 3D position
///
/// 3D positions are described by a x and z position and
/// a height over ground.
PNPTCPLUGIN_CALLBACK typedef struct PnPTC3DPosition
{
	PnPTC2DVec	coord;	//!< position on heightmap. 
	float		height;	//!< height over ground.
} PnPTC3DPosition;

// ---------------------------------------------------------------------------------
/// @brief texture UV coordinate.
///
/// Data structure containing texture UV coordinates.
PNPTCPLUGIN_CALLBACK typedef struct PnPTCUV
{
	float u;			//!< u texture coordinate.
	float v;			//!< v texture coordinate.
} PnPTCUV;






// ---------------------------------------------------------------------------------
// base plugin class
// ---------------------------------------------------------------------------------


/// @brief Base PnP-TC PlugIn interface class.
///
/// This is the base class of all specialized PlugIn classes.
/// It contains base functionality needed by the TerrainCreator's
/// PlugIn-Manager when scanning for available plugins,
class PNPTCPLUGIN_API IPnPTCPlugIn
{
public:


	/// @brief Returns the plugin's guid.
	/// 
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID			GetGUID()=0;


	/// @brief Returns the plugin's version.
	/// 
	/// Returns the version of the implemented plugin.
	/// @return Implemented API version.
	virtual int				GetVersion()=0;

	/// @brief Returns the name of the plugin.
	/// 
	/// Returns the name of the plugin.
	/// @return PlugIn name.
	virtual const char*		GetName()=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_PLUGIN_H_INCLUDED)