/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCSoundPlugIn.h
// Interface definition of the sound engine plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//  04. Apr. 2004 (Ralf Westphal)
//		Added the support for default categories
//  13. May  2004 (Ralf Westphal)
//		Support for environment simulation added
//  04. May 2008 (Ralf Westphal)
//      Added GetDefaultGlobalComments, GetDefaultPlacementComments.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_ISOUNDPLUGIN_H_INCLUDED)
#define PNPTC_ISOUNDPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


// forward definitions
class CIPnPTCEnvSim01;


/// @brief Description of 3D sound instances.
///
/// This struct contains the data for a 3d
/// instantiation of a sound.
PNPTCPLUGIN_API typedef struct PnPTC3DSoundInstanceData
{
	float	fPos[3];				//!< 3D position (x,y,z) of the sound in meters (left hand system y=up)
	float	fVolume;				//!< volume of the sound from 0=silence to 1=maximum
	float	fMinDistance;			//!< Distance from the listener at which the sound begins to be attenuated
	float	fMaxDistance;			//!< Distance from the listener beyond which the sound is no longer attenuated
} PnPTC3DSoundInstanceData;


/// @brief Interface definition for the sound PlugIn API.
/// 
/// An implementation of this interface enables the TerrainCreator to manage sounds.<br>
/// The implementation of this interface should do the following things:<br>
/// <b>1.</b> Enum all playable sounds from a given directory in the method
/// Init() and store textual IDs for these sounds. For example the filenames.<br>
/// Each single sound should be accessible from the PnP-TC via these textual IDs.
/// PnP-TC will play these sounds and create 3D instances by it.<br>
/// <b>2.</b> Create and manage an arbitrary number of 3D sound objects (called 3D 
/// sound instances) created from the managed sounds via the method Create3DInstace().
class PNPTCPLUGIN_API IPnPTCSoundPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID	GetGUID()=0;

	/// @brief Plugin initialization.
	///
	/// Initialization of sound interface.<br>
	/// This method is called directly after a terrain document
	/// is loaded or the sound plugin for an already loaded
	/// terrain is changed.<br>
	/// You should scan the given sound path for all sound files
	/// supported by your plugin and create a list of these files.
	/// @param cSoundPath Directory where the sound files for 
	/// this terrain are located. Usually this is a directory
	/// with the name <i>SOUNDS</i> directly under the main
	/// terrain file location.
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and the sound engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Init(char* cSoundPath)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin.You should reload/rescan the
	/// sounds from the sound directory within this method.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull and the sound-engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Reset()=0;

	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// @brief Plugin cleanup.
	///
	/// Performs any cleanups needed before this class will be destroyed.<br>
	/// This method is called when a terrain document is to
	/// be closed or if the plugin used for a terrain was changed.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be sure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void	Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Access methods to the sounds managed by this plugin
	// --------------------------------------------------------------------------------------------

	/// @brief Returns number of available sounds.
	///
	/// @return Number of sounds.
	virtual int		GetSoundsCount()=0;

	/// @brief Returns the name/id of one sound.
	///
	/// Returns the name (which is the ID) of a sound.<br>
	/// You should make sure that every sound managed by your plugin
	/// has an unique name.<br>
	/// <b>The sound IDs/names should not be case sensitive!</b>
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound.
	/// @return Return the null terminated character id-string of the sound if 
	/// successfull. Return <i>null</i> in case of an error.
	virtual char*	GetSoundName(unsigned int iSound)=0;

	/// @brief Returns default sound category
	///
	/// Returns the category and sub category of a sound.
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound.
	/// @param pcCategory Return parameter: Category name of the sound. If the category
	/// of the sound is not preset, <i>NULL</i> is returned.<br>
	/// The only supported categories so far are "Environment" and "Ground".
	/// @param pcSubCategory Return parameter: Sub category name of the sound. If the sub category
	/// of the sound is not preset, <i>NULL</i> is returned.
	virtual void	GetSoundCategory(unsigned int iSound,char** pcCategory,char** pcSubCategory)=0;

	/// @brief Returns the index for a given sound name/id.
	///
	/// Gets the index of a given sound id within the list of sounds managed 
	/// by this plugin.
	/// @return Returns the zero based index of the sound. Returns a value < 0
	/// to indicate that the given sound id is not managed by the plugin.
	virtual int		GetSoundIndex(char* cSoundName)=0;

	/// @brief Plays a sound.
	///
	/// Plays the sound with the given index directly without any 3d effects.
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound.
	/// @return Returns <i>true</i> if the sound could be played successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool	PlaySound(unsigned int iSound)=0;

	/// @brief Stops playing a sound
	///
	/// Stops the current playback of all sound played with PlaySound().
	virtual void	StopSound()=0;


	// --------------------------------------------------------------------------------------------
	// Instantiating methods for 3D placement of sounds
	// --------------------------------------------------------------------------------------------

	/// @brief Sets 3D sound parameters.
	///
	/// Sets general 3D sound parameters.<br>
	/// This method is called when the sector sound editor mode is activated.
	/// @param fDistanceFactor This is the number of meters in a vector unit.
	/// @param fDopplerFactor Factor to be multiplied for the doppler effect. The default
	/// value is 1.0 which corresponds to the real world doppler effect.
	/// @param fRollOffFactor Determines the rate of attenuation over distance. The default
	/// value is 1.0 which corresponds to the real world attenuation.
	virtual void			Set3DSoundParameters(float fDistanceFactor,float fDopplerFactor,float fRollOffFactor)=0;

	/// @brief Sets listeners attributes.
	///
	/// Sets the current position, orientation and speed of the listener.<br>
	/// <b>Keep in mind:</b> The up vector for all 3D positions is y and a left hand coordinate
	/// system is used.
	/// @param fPos Current position (x,y,z) of the listener given in meters.
	/// @param fFront Front vector of the listener.
	/// @param fUp Up vector of the listener.
	/// @param fSpeed Direction of movement and speed of the listener given in meters per second.
	virtual void			Set3DListener(float fPos[3],float fFront[3],float fUp[3],float fSpeed[3])=0;

	/// @brief Updates all sound data.
	///
	/// This method is usually called once per frame.
	/// perform any updates on your 3d sound instances here.
	virtual void			Update()=0;

	/// @brief Set the reference to the environment simulation.
	///
	/// Set reference to the environment simulation implemented
	/// within the main application. You need this reference to
	/// inform the main application as soon as a 3d sound instance
	/// ended its playback
	virtual void			SetEnvSimulation(CIPnPTCEnvSim01* pEnvSim)=0;

	/// @brief Creates a 3d sound instance.
	///
	/// Creates a new 3D sound instance for a given sound.
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound
	/// for which a 3D instance should be created.
	/// @return Returns an ID (index) (>=0) for the create 3D sound instance.
	/// Returns <i>-1</i> in case of an error.
	virtual int				Create3DInstace(unsigned int iSound)=0;

	/// @brief Sets 3d sound instance parameters.
	///
	/// Sets the parameters of a 3D sound instance
	/// @param iInstance Instance ID of the sound thats parameters are to be set
	/// as returned from Create3DInstance().
	/// @param Params Structure containing all 3D sound instance parameters.
	/// @return Returns <i>true</i> if the 3D sound instance parameters
	/// could be set. Returns <i>false</i> in case of an error.
	virtual bool			Set3DInstanceParameters(unsigned int iInstance,PnPTC3DSoundInstanceData& Params)=0;

	/// @brief Plays a 3d sound instance.
	///
	/// @param iInstance Instance ID of the sound to be played as returned
	/// from Create3DInstance().
	/// @param pParams Structure containing all 3D sound instance parameters.
	/// You can pass NULL if the instance parameters should not be set
	/// within this method.
	/// @return Returns <i>true</i> if the 3D sound instance
	/// could be played. Returns <i>false</i> in case of an error.
	virtual bool			Play3DInstance(unsigned int iInstance,PnPTC3DSoundInstanceData* pParams)=0;

	/// @brief Stops playing a 3D sound instance.
	///
	/// @param iInstance Instance ID of the sound to be stopped playing as returned
	/// from Create3DInstance.
	virtual void			Stop3DInstance(unsigned int iInstance)=0;
	
	/// @brief Deletes a 3D sound instance.
	/// 
	/// @param iInstance Instance ID of the sound to be deleted as returned
	/// from Create3DInstance().
	virtual void			Delete3DInstance(unsigned int iInstance)=0;

	/// @brief Deletes all 3D sound instances.
	///
	/// Deletes all 3D sound instances.<br>
	/// This method is called when the detail edit mode of
	/// a sector will be closed.
	virtual void			DeleteAll3DInstances()=0;


	// --------------------------------------------------------------------------------------------
	// default comments
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the default global sound comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of sound in the sound manager.
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound
	/// for which the default comment text should be returned.
	/// @return Default comment text for this sound.
	virtual const char*		GetDefaultGlobalComments(int iSound) const=0;

	/// @brief Returns the default sound placement comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of sound for the sound placements.
	/// @param iSound The index in the range [0...GetSoundsCount()-1] of the sound
	/// for which the default comment text should be returned.
	/// @return Default comment text for this sound.
	virtual const char*		GetDefaultPlacementComments(int iSound) const=0;

}; // class IPnPTCSoundPlugIn01

#pragma pack(pop)

#endif // !defined(PNPTC_ISOUNDPLUGIN_H_INCLUDED)