/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCTexturePlugIn.h
// Interface definition of the surface texture manager plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	08. Mai 2004
// ----------------------------------------------------------------------------------
// change history:
//  17. Mar. 2007 (Ralf Westphal)
//		Redesigned category management and made chars const.
//		added SaveTextureToFile, SaveTextureToMem, DeleteTexture, 
//		AddTexture, and GetSupportedAddTextureFileExtensions.
//		(Sorry for breaking compatibility)
//  04. May 2008 (Ralf Westphal)
//      Added GetDefaultGlobalComments.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_ITEXTUREPLUGIN_H_INCLUDED)
#define PNPTC_ITEXTUREPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)



/// @brief Interface definition for the texture PlugIn API.
/// 
/// An implementation of this interface enables the TerrainCreator to manage surface textures.<br>
/// The implementation of this interface should do the following things:<br>
/// Enum all textures from a given directory in the method
/// Init() and store textual IDs for these textures. For example the filenames.<br>
/// Each single texture should be accessible from the PnP-TC via these textual IDs.<br>
class PNPTCPLUGIN_API IPnPTCTexturePlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID	GetGUID()=0;

	/// @brief Plugin initialization.
	///
	/// Initialization of texture interface.<br>
	/// This method is called directly after a terrain document
	/// is loaded or the texture plugin for an already loaded
	/// terrain is changed.<br>
	/// You should scan the given texture path for all texture files
	/// supported by your plugin and create a list of these files.
	/// @param cTexturePath Directory where the texture files for 
	/// this terrain are located. Usually this is a directory
	/// with the name <i>TEXTURES</i> directly under the main
	/// terrain file location.
	/// @param pDevice Reference to the rendering device, that
	/// will render the terrain and display the textures.<br>
	/// <b>The will be pointer to an object of the class 
	/// &quot;IDirect3DDevice9&quot;</b>
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and your texture manager is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Init(const char* cTexturePath,void* pDevice)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin. You should reload/rescan the
	/// textures from the texture directory within this method.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull and your texture manager is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool	Reset()=0;

	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// @brief Plugin cleanup.
	///
	/// Performs any cleanups needed before this class will be destroyed.<br>
	/// This method is called when a terrain document is to
	/// be closed or if the plugin used for a terrain was changed.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be sure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void	Exit()=0;


	// --------------------------------------------------------------------------------------------
	// Access methods to the textures managed by this plugin
	// --------------------------------------------------------------------------------------------

	/// @brief Returns number of available textures.
	///
	/// @return Number of textures.
	virtual unsigned int		GetTextureCount() const=0;

	/// @brief Returns the name/id of one texture.
	///
	/// Returns the name (which is the ID) of a texture.<br>
	/// You should make sure that every texture managed by your plugin
	/// has an unique name.<br>
	/// <b>The texture IDs/names should not be case sensitive!</b>
	/// @param nTexture The index in the range [0...GetTextureCount()-1] of the texture.
	/// @return Return the null terminated character id-string of the sound if 
	/// successfull. Return <i>null</i> in case of an error.
	virtual const char*			GetTextureName(unsigned int nTexture) const=0;

	/// @brief Returns default texture category
	///
	/// Returns the default category and sub category of a texture.
	/// @param nTexture The index in the range [0...GetTextureCount()-1] of the texture.
	/// @return Returns the category string of the texture. Subcategories are
	/// divided by a '\' (example "maincategory\subcategory\subcategory2").
	/// In case of an error or if the category of the texture is not preset, 
	/// <i>NULL</i> is returned.<br>
	virtual const char*			GetTextureCategory(unsigned int nTexture) const=0;

	/// @brief Returns the index for a given texture name/id.
	///
	/// Gets the index of a given texture id within the list of the textures managed 
	/// by this plugin.
	/// @return Returns the zero based index of the texture. Returns a value < 0
	/// to indicate that the given texture id is not managed by the plugin.
	virtual int					GetTextureIndex(const char* cTextureName) const=0;


	// --------------------------------------------------------------------------------------------
	// Access methods to the 3d textures objects
	// --------------------------------------------------------------------------------------------

	/// @brief Returns a pointer to the texture object.
	///
	/// Returns a pointer to the texture object for a given texture index to be
	/// displayed by the graphics engine.
	/// @param nTexture The index in the range [0...GetTextureCount()-1] of the texture.
	/// @return Returns the pointer to the texture object. <b>By now this has to be a
	/// pointer to an object of the class &quot;IDirect3DTexture9&quot;</b><br>
	/// Returns <i>NULL</i> in case of an error.
	virtual void*				GetTexture(unsigned int nTexture)=0;

	/// @brief Releases one texture object.
	///
	/// Releases one texture object when it is no longer needed.
	/// @param pTexture Pointer to the texture object.
	virtual void				ReleaseTexture(void* pTexture)=0;

	/// @brief Releases all texture objects.
	///
	/// Releases all texture objects when they are no longer needed.
	/// This method might not be called in cases when the texture manager
	/// plugin is changed (for example), so make sure to release all
	/// open textures when your plugin is exited (within the Exit() method).
	virtual void				ReleaseAllTextures()=0;


	// --------------------------------------------------------------------------------------------
	// Texture management
	// --------------------------------------------------------------------------------------------

	/// @brief Deletes one texture.
	///
	/// Deletes one texture which is managed by this plugin.
	/// @param nTexture The index in the range [0...GetTextureCount()-1] of the 
	/// texture which should be deleted.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// texture could be deleted.
	virtual bool				DeleteTexture(unsigned int nTexture)=0;

	/// @brief Returns supported file extensions.
	/// 
	/// Returns the file extensions which are allowed for adding texture
	/// files to the list of textures managed by this plugin.
	/// @return Returns a string containing the supported file extensions.
	/// If more than one file extension is supported, the single extensions
	/// should be seperated by a ';' (for example "*.bmp;*.png;*.jpg").
	virtual const char*			GetSupportedAddTextureFileExtensions() const=0;

	/// @brief Adds one texture.
	///
	/// Adds one texture which should be loaded from a given file.
	/// @param cFileName The file name of the texture to be added.
	/// @param cCategory The category (including subcategories separated by '\')
	/// into which this texture should be placed.
	/// @return Returns the index of the newly added texture if successfull, 
	/// or <i>-1</i> in case of an error.
	virtual int					AddTexture(const char* cFileName,const char* cCategory)=0;

	/// @brief Saves the texture to a file.
	///
	/// Saves the texture to a file.
	/// @param nTexture The index in the range [0...GetTextureCount()-1] of the texture
	/// which should be saved.
	/// @param cPathName the path (not including filename) to that
	/// the texture should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the texture was saved. The memory 
	/// to store the file name in the char array will be created during 
	/// the call of SaveTextureToFile. The caller is responsible to delete this 
	/// mem (using free) if it is no longer needed.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool				SaveTextureToFile(unsigned int nTexture,const char* cPathName,char** cFileName)=0;
	
	/// @brief Saves the texture to memory.
	///
	/// Saves the texture file to the memory.
	/// @param nTexture The texture in the range [0...GetTextureCount()-1] of the texture
	/// which should be saved.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveTextureToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool				SaveTextureToMem(unsigned int nTexture,void** pMem,DWORD* dwSize)=0;


	// --------------------------------------------------------------------------------------------
	// default comments
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the default global texture comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of texture in the texture manager.
	/// @param iTexture The index in the range [0...GetSoundsCount()-1] of the texture
	/// for which the default comment text should be returned.
	/// @return Default comment text for this texture.
	virtual const char*			GetDefaultGlobalComments(int iTexture) const=0;

}; // class IPnPTCTexturePlugIn01

#pragma pack(pop)

#endif // !defined(PNPTC_ITEXTUREPLUGIN_H_INCLUDED)