/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCVegetationPlugIn.h
// Interface definition of the vegetation engine engine plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	08. Nov. 2004
// ----------------------------------------------------------------------------------
// change history:
//  15. Oct. 2005 (Ralf Westphal) 	
//		Added grass placements generation method GenerateGrassPlacements()
//  19. Feb. 2007 (Ralf Westphal)
//		Redesigned category management and made chars const.
//		added SaveTreeToFile, SaveTreeToMem, DeleteTree, AddTree, 
//		SaveGrassToFile, SaveGrassToMem, DeleteGrass, AddGrass, 
//		GetSupportedAddTreeFileExtensions, and GetSupportedAddGrassFileExtensions
//		Added color value to intersection test.
//		(Sorry for breaking compatibility)
//  26. Mai. 2007 (Ralf Westphal)
//		Added format parameter to the save-to methods.
//  04. May 2008 (Ralf Westphal)
//      Added GetTreeDefaultGlobalComments, GetTreeDefaultPlacementComments,
//		GetGrassDefaultGlobalComments.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IVEGETATIONPLUGIN_H_INCLUDED)
#define PNPTC_IVEGETATIONPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)


// forward definitions
class CIPnPTCVegetationRenderer01;


/// Interface definition for the vegetation PlugIn API.<br>
/// The implementation of this interface should do the following things:<br>
/// <b>1.</b> Enum all displayable tree and grass objects from a given directory in the method
/// Init and store textual IDs for these objects. For example the filenames.<br>
/// The single objects should be accessable from the PnP-TC via these textual IDs.
/// PnP-TC will create 3D instances for the rendering from these trees and store their
/// textual IDs.<br>
/// <b>2.</b> Create and manage an arbitrary number of tree placements (called 3D 
/// tree instances) created from the managed tree objects via the method Create3DInstance.
class PNPTCPLUGIN_API IPnPTCVegetationPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID		GetGUID()=0;

	/// Initialization of vegetation interface.<br>
	/// This method is called directly after a terrain document
	/// is loaded or the vegetation plugin for an already loaded
	/// terrain is changed.<br>
	/// You should scan the given tree and grass path for all files
	/// supported by your plugin and create a list of these files.
	/// @param cTreePath Directory where the tree files for 
	/// this terrain are located. Usually this is a directory
	/// with the name <i>Vegetation\\Trees</i> directly under the main
	/// terrain file location.
	/// @param cGrassPath Directory where the grass files for 
	/// this terrain are located. Usually this is a directory
	/// with the name <i>Vegetation\\Grass</i> directly under the main
	/// terrain file location.
	/// @param pDevice Reference to the rendering device, that
	/// will render the terrain and the vegetation.<br>
	/// <b>This will be a pointer to an object of the class 
	/// &quot;IDirect3DDevice9&quot;</b>
	/// @return Returns <i>true</i> if the initialization was 
	/// successfull and the vegetation engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool		Init(const char* cTreePath,const char* cGrassPath,void* pDevice)=0;

	/// @brief Resets the data of the plugin.
	///
	/// Resets the contents of this plugin.You should reload/rescan the
	/// trees and grass files from the directories within this method.
	/// @return Returns <i>true</i> if the resetting was 
	/// successfull and the vegetation engine is available.<br>
	/// Returns <i>false</i> in case of an error.
	virtual bool		Reset()=0;


	// --------------------------------------------------------------------------------------------
	// termination methods
	// --------------------------------------------------------------------------------------------

	/// Performs any cleanups needed before this class is gonna
	/// be destroyed.<br>
	/// This method is called when a terrain document is to
	/// be closed or if the plugin used for a terrain was changed.<br>
	/// Free all memory you created during the work of your plugin 
	/// within this method call. You can be shure that no further calls
	/// to your plugin will be made after the call of this method.
	virtual void		Exit()=0;



	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------
	// tree specific functions
	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------



	// --------------------------------------------------------------------------------------------
	// Access methods to the trees managed by this plugin
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the number of available trees.
	///
	/// @return Number of trees.
	virtual unsigned int	GetTreeCount() const=0;

	/// @brief Returns trees's name.
	///
	/// Returns the name (which is the ID) of a tree.<br>
	/// You should make sure that every tree managed by your plugin
	/// has an unique name.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree.
	/// @return Return the null terminated character id-string of the tree if 
	/// successfull. Return <i>null</i> in case of an error.
	virtual const char*		GetTreeName(int iTree) const=0;

	/// @brief Returns default tree category
	///
	/// Returns the default category and sub category of a tree.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree.
	/// @return Returns the category string of the tree. Subcategories are
	/// divided by a '\' (example "maincategory\subcategory\subcategory2").
	/// In case of an error or if the category of the tree is not preset, 
	/// <i>NULL</i> is returned.<br>
	virtual const char*		GetTreeCategory(unsigned int iTree) const=0;

	/// @brief Returns trees's index.
	///
	/// Gets the index of a given tree id within the list of trees managed 
	/// by this plugin.
	/// @param cTreeName name/id of the tree, whichs index should be returned.
	/// @return Returns the zero based index of the tree. Returns a value < 0
	/// to indicate that the given tree id is not managed by the plugin.
	virtual int				GetTreeIndex(const char* cTreeName) const=0;


	// --------------------------------------------------------------------------------------------
	// methods for 3D tree placement/instatiations
	// --------------------------------------------------------------------------------------------

	/// @brief Creates a new 3d tree instance.
	///
	/// Creates a new 3d tree instance for rendering
	/// for a given tree ID.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// for which a 3D instance should be created.
	/// @return Returns an ID (>0) for the create 3D tree instance.
	/// Returns <i>0</i> in case of an error.
	virtual int			Create3DTreeInstace(int iTree)=0;

	/// @brief Deletes one 3d tree instance.
	///
	/// @param iInstance Instance ID of the tree to be deleted as returned
	/// from Create3DTreeInstace.
	virtual void		Delete3DTreeInstance(int iInstance)=0;

	/// @brief Deletes all 3d tree instances.
	///
	/// Deletes all 3d tree instances.
	/// This method is called when the detail edit mode of
	/// a sector is to be closed.
	virtual void		DeleteAll3DTreeInstances()=0;

	/// @brief Sets 3d tree instance transformation.
	///
	/// @param iInstance Instance ID of the tree thats transformation matrix 
	/// is to be set as returned from Create3DTreeInstace.
	/// @param matTransform 4x4 transformation matrix containing rotation, scaling
	/// and translation in row major format.
	virtual void		Set3DTreeInstanceTransformation(int iInstance,float matTransform[16])=0;

	/// @brief Returns information of one 3d tree instance.
	///
	/// @param iInstance Instance ID of the tree to return its data.
	/// @param iFaces (output) Number of faces of this tree instance when rendered.
	/// @param iVertices (output) Number of vertices of this tree instance when rendered.
	/// @return <i>true</i> if the tree's data could be returned successfully,
	/// <i>false</i> in case of an error.
	virtual bool		GetTreeInfo(int iInstance,int* iFaces,int* iVertices)=0;

	/// @brief Returns the number of render passes.
	///
	/// Returns the number of render passes required to render trees.
	/// @return Number of required tree render passes.
	virtual int			GetNumberTreeRenderPasses()=0;

	/// @brief Prepare everything you need to render the tree.
	///
	/// This method is called every frame before the first tree is to be rendered.
	/// @param matView 4x4 transformation matrix containing the current view matrix
	/// in row major format.
	/// @return <i>true</i> if the initialization for rendering was successfull,
	/// <i>false</i> in case of an error.
	virtual bool		StartRenderTrees(float matView[16])=0;

	/// @brief Renders a 3d tree.
	///
	/// Renders a given 3d tree instance.
	/// @param iRenderPass number of the current render pass [0..GetNumberTreeRenderPasses()-1].
	/// If all passes have to be rendered with one call, this parameter is set to <i>-1</i>.
	/// @param iInstance 3d tree instance ID to be rendered.
	/// @param pExtra Additional PlugIn specific data of tree placement.
	/// @param dwMemSize Size of the trees extra data memory in bytes.
	/// @return <i>true</i> if the tree could be rendered successfully,
	/// <i>false</i> in case of an error.
	virtual bool		RenderTree(int iRenderPass,int iInstance,void* pExtra=NULL,DWORD dwMemSize=0)=0;

	/// @brief Cleanup everything you set for rendering.
	///
	/// This method is called every frame after the last tree was rendered.
	/// @return <i>true</i> if the cleanup of rendereing was successfull,
	/// <i>false</i> in case of an error.
	virtual bool		EndRenderTrees()=0;

	/// @brief Get tree's bounding box.
	///
	/// This method should return a tree's bounding
	/// box in world coordinates using the current tree
	/// transformation.
	/// @param iInstance Instance ID of the tree thats bounding box has 
	/// to be returned.
	/// @param vecVertex1 (output) Coordinates of one corner of the tree's
	/// bounding box in world coordinates.
	/// @param vecVertex2 (output) Coordinates of the opposite corner of the tree's
	/// bounding box in world coordinates.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the bounding
	/// box could be returned successfully.
	virtual bool		GetTreeBoundingBox(int iInstance,float vecVertex1[3], float vecVertex2[3])=0;

	/// @brief: Returns whether or not a tree is hit by a mouse click.
	///
	/// This method should test a given ray against intersections with 
	/// one 3d tree instance.
	/// @param iInstance Instance ID of the tree that has to be checked.
	/// @param vecStart Starting point of a ray in world coordinates.
	/// @param vecDir Direction of the ray in world coordinates.
	/// @param pRGBA (output) Pointer to a color value. If a valid pointer
	/// is passed with the method call, the caller expects a color value including 
	/// the alpha component from the position where the tree was hit as return value.
	/// @return Returns <i>true</i> if the tree is hit by the given ray.
	/// Returns <i>false</i> if not.<br>
	/// <b>Remarks:</b> This method will only be called if a previous intersection
	/// test performed by PnP-TC against the tree's bounding box ( as returned
	/// by GetTreeBoundingBox) was successfull.
	/// So you should simply return <i>true</i> if you don't want to implement
	/// a detailed intersection test here.
	virtual bool		IntersectTree(int iInstance,float vecStart[3],float vecDir[3],PNPTC_RGBA* pRGBA)=0;

	/// @brief: Returns whether or not a tree can be rotated.
	///
	/// This method returns whether or not the underlaying tree rendering
	/// engine supports to rotation of trees.
	/// @return Returns <i>true</i> if the trees can be rotated.
	/// Returns <i>false</i> if not.<br>
	virtual bool		AllowTreeRotation()=0;


	// --------------------------------------------------------------------------------------------
	// Tree management
	// --------------------------------------------------------------------------------------------

	/// @brief Deletes one tree.
	///
	/// Deletes one tree which is managed by this plugin.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// which should be deleted.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// tree could be deleted.
	virtual bool		DeleteTree(int iTree)=0;

	/// @brief Returns supported file extensions.
	/// 
	/// Returns the file extensions which are allowed for adding tree
	/// files to the list of trees managed by this plugin.
	/// @return Returns a string containing the supported file extensions.
	/// If more than one file extension is supported, the single extensions
	/// should be seperated by a ';' (for example "*.obj;*.3ds;*.b3d").
	virtual const char*	GetSupportedAddTreeFileExtensions()=0;

	/// @brief Adds one tree.
	///
	/// Adds one tree which should be loaded from a given file.
	/// @param cFileName The file name of the tree to be added.
	/// @param cCategory The category (including subcategories separated by '\')
	/// into which this tree should be placed.
	/// @return Returns the index of the newly added tree if successfull, 
	/// or <i>-1</i> in case of an error.
	virtual int			AddTree(const char* cFileName,const char* cCategory)=0;

	/// @brief Saves the tree to a file.
	///
	/// Saves the tree to a file.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the tree should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool		SaveTreeToFile(int iTree,const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL)=0;
	
	/// @brief Saves the tree to memory.
	///
	/// Saves the tree file to the memory.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveTreeToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool		SaveTreeToMem(int iTree,const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0)=0;


	// --------------------------------------------------------------------------------------------
	// Methods to handle tree placements extra data display it on a dialog
	// --------------------------------------------------------------------------------------------

	/// @brief Creation of a window to display extra data.
	/// 
	/// Creates a subwindow of the main PnP-TC window to enter
	/// additional data for one tree placement.<br>
	/// @param hParent Window handle to the parent (the PnP-TC
	/// main window) window.
	/// @param bToolWindow Whether or not this window should be displayed
	/// as tool window. <i>true</i> if it is a tool window, in which case
	/// you should only display the most important data on a small window.
	/// <i>false</i> if it is a window to be displayed on the main 
	/// placement settings dialog, where you can display all extra data
	/// on a large window.
	/// @return Returns the window handle for the newly
	/// created window. Returns <i>NULL</i> if
	/// no window was created.
	virtual	HWND		CreateExtraTreeDataWindow(HWND hParent,bool bToolWindow)=0;

	/// @brief Destruction of an extra data window.
	///
	/// Destroys one plugin's extra data window.
	/// @param hWnd Window handle of the window to be destroyed.
	virtual void		DestroyExtraTreeDataWindow(HWND hWnd)=0;

	/// @brief Validates inputs in an extra data window.
	///
	/// Validates user input in this extra data window.
	/// This method is called from the PnP-TC right before
	/// the tree placement settings will be saved.
	/// @param hWnd Window handle of the extra data window to be validated.
	/// @param bToolWindow Whether or not this window is a tool window.
	/// <i>true</i> if it is a tool window, in which case you should only
	/// display and validate the most important data on a small window.
	/// <i>false</i> if it is a window displayed on the main 
	/// placement settings dialog, where you can display all extra data
	/// on a large window.
	/// @return Returns <i>true</i> if the user's inputs are correct
	/// an can therefore be saved. Returns <i>false</i> in case of
	/// an error.
	virtual bool		ValidateExtraTreeDataWindow(HWND hWnd,bool bToolWindow)=0;

	/// @brief Clear the extra data window.
	///
	/// Clears all entries of this extra data window.
	/// @param hWnd Window handle of the extra data window to be cleared.
	/// @param bToolWindow Whether or not this window is a tool window.
	/// <i>true</i> if it is a tool window, in which case you should only
	/// display and clear the most important data on a small window.
	/// <i>false</i> if it is a window displayed on the main 
	/// placement settings dialog, where you can display all extra data
	/// on a large window.
	virtual void		ClearExtraTreeDataWindow(HWND hWnd,bool bToolWindow)=0;

	/// @brief Displays extra tree data.
	///
	/// Displays tree placement's extra data within a window.
	/// @param hWnd Window handle of the extra data window to be displayed.
	/// @param bToolWindow Whether or not this window is a tool window. 
	/// <i>true</i> if it is a tool window, in which case
	/// you should only display the most important data on a small window.
	/// <i>false</i> if it is a window displayed on the main 
	/// placement settings dialog, where you can display all extra data
	/// on a large window.
	/// @param pMem Pointer to the PlugIns tree extra data to be displayed.
	/// @param dwMemSize Size of the memory pointed to by pMem.
	/// @return Returns whether or not the data could be displayed correctly.
	virtual bool		DisplayExtraTreeData(HWND hWnd,bool bToolWindow,void* pMem,DWORD dwMemSize)=0;

	/// @brief Returns the user's inputs from a extra data window.
	///
	/// @param hWnd Window handle of the extra data window.
	/// @param bToolWindow Whether or not this window is a tool window. 
	/// <i>true</i> if it is a tool window, in which case
	/// you should only update the most important data on a small window.
	/// <i>false</i> if it is a window displayed on the main 
	/// placement settings dialog, where you can update all extra data.
	/// @param pMem Return parameter pointing to the memory containing the 
	/// extra data. If the method failes a NULL pointer is returned.<br>
	/// <b>Remarks</b>: The pointer returned is a temporary pointer. You 
	/// should copy the contents of it directly after the call.
	/// @param dwMemSize Returns the amount of bytes stored in the memory block.
	/// If the method fails <i>0</i> is returned.
	/// @return Returns <i>true</i> if the data could be retrieved successfully.
	/// Returns <i>false</i> in case of an error.
	virtual bool		UpdateExtraTreeData(HWND hWnd,bool bToolWindow,void** pMem,DWORD* dwMemSize)=0;


	// --------------------------------------------------------------------------------------------
	// default comments
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the default global tree comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of tree in the vegetation manager.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// for which the default comment text should be returned.
	/// @return Default comment text for this tree.
	virtual const char*		GetTreeDefaultGlobalComments(int iTree) const=0;

	/// @brief Returns the default tree placement comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of tree for the tree placements.
	/// @param iTree The index in the range [0...GetTreeCount()-1] of the tree
	/// for which the default comment text should be returned.
	/// @return Default comment text for this tree.
	virtual const char*		GetTreeDefaultPlacementComments(int iTree) const=0;



	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------
	// general plugin specific functions
	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------



	// --------------------------------------------------------------------------------------------
	// Methods to manage wind
	// --------------------------------------------------------------------------------------------

	/// @brief Sets the wind parameters.
	/// 
	/// Sets all parameters required for wind computations.
	/// @param vecDirection Direction of wind.
	/// @param dSpeed Wind speed in m/s.
	virtual void		SetWindParameters(const PnPTC3DVec& vecDirection,const double& dSpeed)=0;


	// --------------------------------------------------------------------------------------------
	// Methods for settings window
	// --------------------------------------------------------------------------------------------

	/// @brief Creation of a window to display PlugIn's settings.
	/// 
	/// Creates a subwindow of the PnP-TC 3d settings window to enter
	/// additional PlugIn specific settings.<br>
	/// @param hParent Window handle to the parent (the PnP-TC
	/// 3d settings window) window.
	/// @return Returns the window handle for the newly
	/// created window. Returns <i>NULL</i> if no window was created.
	virtual	HWND		CreateSettingsWindow(HWND hParent)=0;

	/// @brief Destruction of the settings window.
	///
	/// Destroys the plugin's settings window.
	/// @param hWnd Window handle of the window to be destroyed.
	virtual void		DestroySettingsWindow(HWND hWnd)=0;

	/// @brief validates and saves settings window.
	///
	/// This method validates and saves the contents of the
	/// PlugIn's settings window. It is called directly before
	/// the window is to be destroyed.
	/// @param hWnd Window handle of the window to be saved.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the data was valid and could be saved.
	virtual bool		ValidateAndSaveSettingsWindow(HWND hWnd)=0;



	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------
	// grass specific functions
	// --------------------------------------------------------------------------------------------
	// --------------------------------------------------------------------------------------------



	// --------------------------------------------------------------------------------------------
	// Access methods to the grasses managed by this plugin
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the number of available grasses.
	///
	/// @return Number of grasses.
	virtual unsigned int	GetGrassCount() const=0;

	/// @brief Returns grasse's name.
	///
	/// Returns the name (which is the ID) of a grass.<br>
	/// You should make sure that every grass managed by your plugin
	/// has an unique name.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass.
	/// @return Return the null terminated character id-string of the tree if 
	/// successfull. Return <i>null</i> in case of an error.
	virtual const char*		GetGrassName(int iGrass) const=0;

	/// @brief Returns default grass category
	///
	/// Returns the default category and sub category of a grass.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass.
	/// @return Returns the category string of the grass. Subcategories are
	/// divided by a '\' (example "maincategory\subcategory\subcategory2").
	/// In case of an error or if the category of the grass is not preset, 
	/// <i>NULL</i> is returned.<br>
	virtual const char*		GetGrassCategory(unsigned int iGrass) const=0;

	/// @brief Returns grass's index.
	///
	/// Gets the index of a given grass id within the list of grasses managed 
	/// by this plugin.
	/// @param cGrassName name/id of the grass, whichs index should be returned.
	/// @return Returns the zero based index of the grass. Returns a value < 0
	/// to indicate that the given grass id is not managed by the plugin.
	virtual int				GetGrassIndex(const char* cGrassName) const=0;


	// --------------------------------------------------------------------------------------------
	// methods for 3D grass placement/instatiations
	// --------------------------------------------------------------------------------------------

	/// @brief Lets the plugin randomly generate grass placements.
	///
	/// The distribution of single grass placements on a given terrain
	/// area might depend on the underlaying grass rendering method
	/// and the grass object itself. Therefore the vegetation manager
	/// PlugIn is responsible for generating grass placements on its own.
	/// The main application will call this method whenever a terrain
	/// region comes inside the view and grass has to be rendered within
	/// this region. Inside this method the PlugIn should generate randomly
	/// a number of grass placements suitable to the passed density value
	/// around the passed center point inside a squared region using the
	/// also passed border size. The single placements are returned to
	/// the main application by calling the AddGrassPlacement
	/// method on the passed CIPnPTCVegetationRenderer01 object.
	/// The main application will later call the Create3DGrassInstace method.
	/// @param vecCenter Center position around which the grass placements
	/// will be created.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// for which placements should be created.
	/// @param byDensity Density in percentage (0..100) of the grass placements.
	/// @param fRegionBorderSize Border size of a squared region around the center
	/// coordinate in which grass placements can be created.
	/// @param pVegetationRenderer Pointer to the vegetation rendering object
	/// which is responsible for creating and destroying grass placements.
	/// @return Returns the number of generated grass placements.
	virtual int			GenerateGrassPlacements(const PnPTC2DVec& vecCenter,int iGrass,unsigned char byDensity,float fRegionBorderSize,CIPnPTCVegetationRenderer01* pVegetationRenderer)=0;

	/// @brief Creates a new 3d grass instance.
	///
	/// Creates a new 3d grass instance for rendering
	/// for a given grass ID.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// for which a 3D instance should be created.
	/// @return Returns an ID (>0) for the create 3D grass instance.
	/// Returns <i>0</i> in case of an error.
	virtual int			Create3DGrassInstace(int iGrass)=0;

	/// @brief Deletes one 3d grass instance.
	///
	/// @param iInstance Instance ID of the grass to be deleted as returned
	/// from Create3DGrassInstace.
	virtual void		Delete3DGrassInstance(int iInstance)=0;

	/// @brief Deletes all 3d grass instances.
	///
	/// Deletes all 3d grass instances.
	/// This method is called when the detail edit mode of
	/// a sector is to be closed.
	virtual void		DeleteAll3DGrassInstances()=0;

	/// @brief Sets 3d grass instance transformation.
	///
	/// @param iInstance Instance ID of the grass thats transformation matrix 
	/// is to be set as returned from Create3DGrassInstace.
	/// @param matTransform 4x4 transformation matrix containing rotation, scaling
	/// and translation in row major format.
	virtual void		Set3DGrassInstanceTransformation(int iInstance,float matTransform[16])=0;

	/// @brief Returns information of one 3d grass instance.
	///
	/// @param iInstance Instance ID of the grass to return its data.
	/// @param iFaces (output) Number of faces of this grass instance when rendered.
	/// @param iVertices (output) Number of vertices of this grass instance when rendered.
	/// @return <i>true</i> if the grasse's data could be returned successfully,
	/// <i>false</i> in case of an error.
	virtual bool		GetGrassInfo(int iInstance,int* iFaces,int* iVertices)=0;

	/// @brief Returns the number of render passes.
	///
	/// Returns the number of render passes required to render grasses.
	/// @return Number of required grass render passes.
	virtual int			GetNumberGrassRenderPasses()=0;

	/// @brief Prepare everything you need to render the grass.
	///
	/// This method is called every frame before the first grass is to be rendered.
	/// @param matView 4x4 transformation matrix containing the current view matrix
	/// in row major format.
	/// @return <i>true</i> if the initialization for rendering was successfull,
	/// <i>false</i> in case of an error.
	virtual bool		StartRenderGrasses(float matView[16])=0;

	/// @brief Renders a 3d grass.
	///
	/// Renders a given 3d grass instance.
	/// @param iRenderPass number of the current render pass [0..GetNumberGrassRenderPasses()-1].
	/// If all passes have to be rendered with one call, this parameter is set to <i>-1</i>.
	/// @param iInstance 3d grass instance ID to be rendered.
	/// @return <i>true</i> if the grass could be rendered successfully,
	/// <i>false</i> in case of an error.
	virtual bool		RenderGrass(int iRenderPass,int iInstance)=0;

	/// @brief Cleanup everything you set for rendering.
	///
	/// This method is called every frame after the last grass was rendered.
	/// @return <i>true</i> if the cleanup of rendereing was successfull,
	/// <i>false</i> in case of an error.
	virtual bool		EndRenderGrasses()=0;

	/// @brief Get grasse's bounding box.
	///
	/// This method should return a grasse's bounding
	/// box in world coordinates using the current grass
	/// transformation.
	/// @param iInstance Instance ID of the grass thats bounding box has 
	/// to be returned.
	/// @param vecVertex1 (output) Coordinates of one corner of the grasse's
	/// bounding box in world coordinates.
	/// @param vecVertex2 (output) Coordinates of the opposite corner of the grasse's
	/// bounding box in world coordinates.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the bounding
	/// box could be returned successfully.
	virtual bool		GetGrassBoundingBox(int iInstance,float vecVertex1[3], float vecVertex2[3])=0;


	// --------------------------------------------------------------------------------------------
	// Grass management
	// --------------------------------------------------------------------------------------------

	/// @brief Deletes one grass.
	///
	/// Deletes one grass which is managed by this plugin.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// which should be deleted.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// grass could be deleted.
	virtual bool		DeleteGrass(int iGrass)=0;

	/// @brief Returns supported file extensions.
	/// 
	/// Returns the file extensions which are allowed for adding grass
	/// files to the list of grasses managed by this plugin.
	/// @return Returns a string containing the supported file extensions.
	/// If more than one file extension is supported, the single extensions
	/// should be seperated by a ';' (for example "*.obj;*.3ds;*.b3d").
	virtual const char*	GetSupportedAddGrassFileExtensions()=0;

	/// @brief Adds one grass.
	///
	/// Adds one grass which should be loaded from a given file.
	/// @param cFileName The file name of the grass to be added.
	/// @param cCategory The category (including subcategories separated by '\')
	/// into which this grass should be placed.
	/// @return Returns the index of the newly added grass if successfull, 
	/// or <i>-1</i> in case of an error.
	virtual int			AddGrass(const char* cFileName,const char* cCategory)=0;

	/// @brief Saves the grass to a file.
	///
	/// Saves the grass to a file.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param cPathName the path (not including filename) to that
	/// the grass should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the object was saved. The memory 
	/// to store the file name should be large enough. This pointer can be
	/// <i>NULL</i> in which case, the file name is not returned to the caller.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool		SaveGrassToFile(int iGrass,const char* cFormat,const char* cPathName,char* cFileName=NULL,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL)=0;
	
	/// @brief Saves the grass to memory.
	///
	/// Saves the grass file to the memory.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// which should be saved.
	/// @param cFormat The file format into which the object should be exported.
	/// This is the file extension of the file format.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveGrassToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool		SaveGrassToMem(int iGrass,const char* cFormat,void** pMem,DWORD* dwSize,float fScale=1,int iUpRotation=0,int iFlipAxis=0)=0;


	// --------------------------------------------------------------------------------------------
	// default comments
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the default global grass comment.
	///
	/// Returns a comment text which is used as default value
	/// for a given type of grass in the vegetation manager.
	/// @param iGrass The index in the range [0...GetGrassCount()-1] of the grass
	/// for which the default comment text should be returned.
	/// @return Default comment text for this grass.
	virtual const char*		GetGrassDefaultGlobalComments(int iGrass) const=0;

}; // class IPnPTCVegetationPlugIn01

#pragma pack(pop)


#endif // !defined(PNPTC_IVEGETATIONPLUGIN_H_INCLUDED)
