/////////////////////////////////////////////////////////////////////////////////////
// 3DObject.cpp
// Object render object implementation.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	 21. Jul. 2006 :	Added possibility to render single subsets of a mesh.
//   24. Sep. 2006 :	Added 3ds loader
//   05. Mar. 2007 :	Now supports unlimited number of subcategories.
//   01. Jul. 2007 :	Added support for dae (collada) file formats.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "../PnPTCUtils/utils.h"
#include ".\3dobject.h"
#include "./3DObjectData.h"
#include "./3DObjectSerializerOBJ.h"
#include "./3DObjectSerializerDAE.h"
#include "./3DObjectSerializer3DS.h"
#include "./3DObjectSerializerB3D.h"
#include "./3DObjectSerializerMDL.h"
#include "./3DObjectSerializerMD2.h"
#include "./3DObjectSerializerMD3.h"
#include "./3DObjectSerializerOgre.h"
#include "./3DObjectRendererD3DXMESH.h"
#include "./3DObjectRenderer.h"




C3DObject::C3DObject(IDirect3DDevice9* pD3DDevice,const std::string& strFileName,const std::string& strPath,const std::string& strCategory,CTempFileManager* pTempFileManager,bool bNameWithoutExtension/*=false*/)
	: m_pD3DDevice(pD3DDevice)
	, m_strFileName(strFileName)
	, m_strName(strFileName)
	, m_strPath(strPath)
	, m_strCategory(strCategory)
	, m_pRenderer(NULL)
	, m_pTempFileManager(pTempFileManager)
{
	// object name without file extension?
	if (bNameWithoutExtension)
	{
		std::string strExtension;
		RemoveFileExtension(strFileName,this->m_strName,strExtension);
	}

	// extract file extension
	std::string strDummy;
	RemoveFileExtension(this->m_strFileName,strDummy,this->m_strFileExtension);
}

C3DObject::~C3DObject(void)
{
	// free everything
	this->Free();
}

bool C3DObject::Load()
{
	// unload previous renderer first
	this->Free();

	// get file name
	std::string	strFileName=this->m_strPath+this->m_strFileName;

	// decide which renderer to take
	if (EqualNoCase(this->m_strFileExtension,"x"))
	{
		// create D3DXMESH renderer
		this->m_pRenderer=new C3DObjectRendererD3DXMESH(this->m_pD3DDevice);
		if (!this->m_pRenderer || 
			!static_cast<C3DObjectRendererD3DXMESH*>(this->m_pRenderer)->Load(strFileName))
		{
			return false;
		}
		else
		{
			// successfully loaded DX mesh
			return true;
		}
	}
	else
	{
		// create default renderer
		this->m_pRenderer=new C3DObjectRenderer(this->m_pD3DDevice);
		if (this->m_pRenderer)
		{
			// get serializer
			I3DObjectSerializer* pSerializer=NULL;
			if (EqualNoCase(this->m_strFileExtension,"obj"))
			{
				pSerializer=new C3DObjectSerializerOBJ();
			}
			else if (EqualNoCase(this->m_strFileExtension,"3ds"))
			{
				pSerializer=new C3DObjectSerializer3DS();
			}
			else if (EqualNoCase(this->m_strFileExtension,"b3d"))
			{
				pSerializer=new C3DObjectSerializerB3D();
			}
			else if (EqualNoCase(this->m_strFileExtension,"mesh.xml"))
			{
				pSerializer=new C3DObjectSerializerOGRE();
			}
			else if (EqualNoCase(this->m_strFileExtension,"dae"))
			{
				pSerializer=new C3DObjectSerializerDAE();
			}
			else if (EqualNoCase(this->m_strFileExtension,"mdl"))
			{
				pSerializer=new C3DObjectSerializerMDL(this->m_pTempFileManager);
			}
			else if (EqualNoCase(this->m_strFileExtension,"md2"))
			{
				pSerializer=new C3DObjectSerializerMD2(this->m_pTempFileManager);
			}
			else if (EqualNoCase(this->m_strFileExtension,"md3"))
			{
				pSerializer=new C3DObjectSerializerMD3(this->m_pTempFileManager);
			}

			// serializer available?
			if (pSerializer)
			{
				// load, clean.up, and return result
				bool bReturn=static_cast<C3DObjectRenderer*>(this->m_pRenderer)->Load(pSerializer,strFileName);
				SAFE_DELETE(pSerializer);
				return bReturn;
			}
			else
			{
				// unsupported file format
				return false;
			}
		}
		else
		{
			// could not create renderer
			return false;
		}
	}


	// error, shouldn't get here
	return false;
}

void C3DObject::Free()
{
	// delete object renderer
	SAFE_DELETE(this->m_pRenderer);
}

int C3DObject::AddReference()
{
	// is this the first reference
	if (this->GetReferenceCount()==0)
	{
		// load object
		if (!this->Load())
		{
			// error loading
			return 0;
		}
	}

	// increment reference counter
	return CReferenceCount::AddReference();
}

int C3DObject::ReleaseReference()
{
	// is this the last reference?
	if (this->GetReferenceCount()==1)
	{
		// unload it
		this->Free();
	}

	// release reference
	return CReferenceCount::ReleaseReference();
}
