/////////////////////////////////////////////////////////////////////////////////////
// 3DObject.h
// Object render object definition.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	 21. Jul. 2006 :	Added possibility to render single subsets of a mesh.
//   05. Mar. 2007 :	Now supports unlimited number of subcategories.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECT_H_INCLUDED)
#define PNPTC_3DOBJECT_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// direct-x includes
#include <d3dx9.h>

// base class includes
#include "../pnptcutils/referencecount.h"


// forward definitions
class C3DObject;
class C3DObjectData;
class I3DObjectRenderer;
class CTempFileManager;


/// 3d object data
class C3DObject : public CReferenceCount
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Construction of a new object.
	/// @param pD3DDevice The rendering device for which the mesh files should 
	/// be loaded and rendered.
	/// @param strFileName Name of the object file (without path information).
	/// @param strPath Path name where the object file is located.
	/// @param strCategory Category name of this object.
	/// @param pTempFileManager A temporary file manager.
	/// @param bNameWithoutExtension Whether or not the file extension should
	/// be included in the object name.
	C3DObject(IDirect3DDevice9* pD3DDevice,const std::string& strFileName,const std::string& strPath,const std::string& strCategory,CTempFileManager* pTempFileManager,bool bNameWithoutExtension=false);

	/// Destruction of the object.
	~C3DObject(void);


	// ---------------------------------------------------------------------
	// initialization and cleanup
	// ---------------------------------------------------------------------

	/// Load object from file.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// object could be loaded successfully.
	bool						Load();

	/// Frees everything.
	void						Free();


	// ---------------------------------------------------------------------
	// object parameter methods
	// ---------------------------------------------------------------------

	/// Returns the name of the object.
	/// @return Returns the object's name.
	inline const std::string&	GetName() const {return this->m_strName;};

	/// Returns the file name of this object.
	/// @return Returns the object's name.
	inline const std::string&	GetFileName() const {return this->m_strFileName;};	

	/// Returns the category of the object.
	/// @return Returns the object's category.
	inline const std::string&	GetCategory() const {return this->m_strCategory;};


	// ---------------------------------------------------------------------
	// 3D rendering methods
	// ---------------------------------------------------------------------

	/// Returns a pointer to the renderer used to render
	/// this 3d object.
	/// @return Pointer to the object renderer.
	inline I3DObjectRenderer*	GetRenderer() const {return this->m_pRenderer;};

	/// Increments the reference counter for this 3D object.
	/// If this is the first reference to the object, the mesh file
	/// will be loaded automatically.
	/// @return Returns the current reference counter value after adding a reference.
	/// In case of an error, <i>0</i> is returned.
	int							AddReference();

	/// Decrements the reference counter for this 3D object.
	/// If this was the last reference to the object, it
	/// will be unloaded automatically.
	/// @return Returns the current reference counter value after release of a reference.
	int							ReleaseReference();


private:

	// ---------------------------------------------------------------------
	// Member variables
	// ---------------------------------------------------------------------

	/// Object's file name.
	std::string			m_strFileName;

	/// Object's name.
	std::string			m_strName;

	/// Object's category.
	std::string			m_strCategory;


	// ---------------------------------------------------------------------
	// Helper variables
	// ---------------------------------------------------------------------
	
	/// A temporary file manager.
	CTempFileManager*	m_pTempFileManager;
	
		
	// ---------------------------------------------------------------------
	// Object's base data
	// ---------------------------------------------------------------------

	/// path to object file.
	std::string			m_strPath;

	/// file extension.
	std::string			m_strFileExtension;

	/// Rendering device used for this object.
	IDirect3DDevice9*	m_pD3DDevice;


	// ---------------------------------------------------------------------
	// Object rendering
	// ---------------------------------------------------------------------

	/// The object renderer.
	I3DObjectRenderer*	m_pRenderer;
	

};


#endif // !defined(PNPTC_3DOBJECT_H_INCLUDED)