/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectData.cpp
// Data structures required for 3d-object/mesh data.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	31. Dec. 2005 : Added GetMaterialIndex method.
//  24. Sep. 2006 : Added methods to set vertex and face data.
//  02. Oct. 2006 : Added bounding box methods, ComputeNormals and ReCreateMeshes method.
//  05. Oct. 2006 : Added user data.
//  04. Jun. 2007 : Added local mesh transformation.
//  22. Jul. 2007 : Added collision geometries.
//  22. Sep. 2007 : Added reference frames.
//  12. Mai  2008 : Normal (re)computation can now be limited to a region.
//  11. Aug. 2008 : Added precise hit test.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include ".\3dobjectdata.h"
#include <stack>
#include <map>
#include <d3dx9.h>
#include "../pnptcutils/utils.h"
#include "../PnPTCUtils/pnpse/pnpseBoundingBox.h"

// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// C3DObjectData
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------


C3DObjectData::C3DObjectData(void)
{
}

C3DObjectData::~C3DObjectData(void)
{
	// remove all joints
	for (unsigned long l=0;l<this->m_vecJoints.size();l++)
	{
		if (this->m_vecJoints[l])
		{
			delete this->m_vecJoints[l];
		}
	}
	this->m_vecJoints.clear();

	// remove all meshe instances
	for (unsigned long l=0;l<this->m_vecMeshInstances.size();l++)
	{
		if (this->m_vecMeshInstances[l])
		{
			delete this->m_vecMeshInstances[l];
		}
	}
	this->m_vecMeshInstances.clear();

	// remove all meshes
	for (unsigned long l=0;l<this->m_vecMeshes.size();l++)
	{
		if (this->m_vecMeshes[l])
		{
			delete this->m_vecMeshes[l];
		}
	}
	this->m_vecMeshes.clear();

	// remove all collision geometries
	for (unsigned long l=0;l<this->m_vecCollisionGeometries.size();l++)
	{
		if (this->m_vecCollisionGeometries[l])
		{
			delete this->m_vecCollisionGeometries[l];
		}
	}
	this->m_vecCollisionGeometries.clear();
	
	// clear all other lists
	this->m_vecMaterials.clear();
	this->m_vecReferenceFrames.clear();
}

int C3DObjectData::AddMaterial(const PNPTC_MATERIAL& mat)
{
	// is this material already in list
	int iMat=this->GetMaterialIndex(mat.strName);
	if (iMat>=0)
	{
		return iMat;
	}

	// add material to list
	this->m_vecMaterials.push_back(mat);
	// return material index
	return (int)this->m_vecMaterials.size()-1;
}

int C3DObjectData::AddMesh(C3DMesh* pMesh,bool bAddMeshInstance/*=true*/)
{
	// add mesh to list
	this->m_vecMeshes.push_back(pMesh);

	// create instance
	if (bAddMeshInstance)
	{
		D3DXMATRIX matIdent;
		D3DXMatrixIdentity(&matIdent);
		this->AddMeshInstance(pMesh,"",matIdent);
	}

	// return mesh index
	return (int)this->m_vecMeshes.size()-1;
}

size_t C3DObjectData::GetNumMeshes() const
{
	// return number of meshes
	return this->m_vecMeshes.size();
}

C3DMesh* C3DObjectData::GetMesh(int iMesh)
{
	// check range
	if (iMesh<0 || iMesh>=(int)this->m_vecMeshes.size())
	{
		return NULL;
	}

	// return mesh
	return this->m_vecMeshes[iMesh];
}

const C3DMesh* C3DObjectData::GetMesh(int iMesh) const
{
	// check range
	if (iMesh<0 || iMesh>=(int)this->m_vecMeshes.size())
	{
		return NULL;
	}

	// return mesh
	return this->m_vecMeshes[iMesh];
}

bool C3DObjectData::AddMeshInstance(C3DMesh* pMesh,const std::string& strName,const D3DXMATRIX& matLocalTransform)
{
	// test if mesh belongs to the object
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh]==pMesh)
		{
			// check name
			std::string strInstanceName=strName;
			if (strInstanceName.empty())
			{
				strInstanceName=pMesh->GetName();
				if (strInstanceName.empty())
				{
					strInstanceName="unnamed";
				}
				strInstanceName+=ToString(static_cast<int>(this->m_vecMeshInstances.size())+1);
			}

			// add instance
			this->m_vecMeshInstances.push_back(new C3DMeshInstance(strInstanceName,pMesh,matLocalTransform));
			return true;
		}
	}

	// mesh no found
	return false;
}

size_t C3DObjectData::GetNumMeshInstances() const
{
	// return number of meshe instances
	return this->m_vecMeshInstances.size();
}

C3DMeshInstance* C3DObjectData::GetMeshInstance(int iMeshInstance)
{
	// check range
	if (iMeshInstance<0 || iMeshInstance>=(int)this->m_vecMeshInstances.size())
	{
		return NULL;
	}

	// return mesh
	return this->m_vecMeshInstances[iMeshInstance];
}

const C3DMeshInstance* C3DObjectData::GetMeshInstance(int iMeshInstance) const
{
	// check range
	if (iMeshInstance<0 || iMeshInstance>=(int)this->m_vecMeshInstances.size())
	{
		return NULL;
	}

	// return mesh
	return this->m_vecMeshInstances[iMeshInstance];
}

size_t C3DObjectData::GetNumMaterials() const
{
	// return number of materials
	return this->m_vecMaterials.size();
}

PNPTC_MATERIAL* C3DObjectData::GetMaterial(int iMaterial)
{
	// check index
	if (iMaterial>=0 && iMaterial<(int)this->m_vecMaterials.size())
	{
		// return pointer to material
		return &this->m_vecMaterials[iMaterial];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

const PNPTC_MATERIAL* C3DObjectData::GetMaterial(int iMaterial) const
{
	// check index
	if (iMaterial>=0 && iMaterial<(int)this->m_vecMaterials.size())
	{
		// return pointer to material
		return &this->m_vecMaterials[iMaterial];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

int C3DObjectData::GetMaterialIndex(const std::string& strMaterialName) const
{
	// search through materials
	for (unsigned int n=0;n<this->m_vecMaterials.size();n++)
	{
		if (_stricmp(strMaterialName.c_str(),this->m_vecMaterials[n].strName.c_str())==0)
		{
			return n;
		}
	}

	// material not found
	return -1;
}

void C3DObjectData::ComputeBoundingBox()
{
	// compute bounding boxes of all meshes
	for (unsigned int nMesh=0;nMesh<static_cast<unsigned int>(this->m_vecMeshes.size());nMesh++)
	{
		// compute bounding box of that mesh
		this->m_vecMeshes[nMesh]->ComputeBoundingBox();
	}

	// initialize bounding box
	this->m_BoundingBox.vMin.x=-1;
	this->m_BoundingBox.vMin.y=-1;
	this->m_BoundingBox.vMin.z=-1;
	this->m_BoundingBox.vMax.x=1;
	this->m_BoundingBox.vMax.y=1;
	this->m_BoundingBox.vMax.z=1;

	// go through all mesh instances
	bool bFirstBB=true;
	for (unsigned int nMeshInstance=0;nMeshInstance<static_cast<unsigned int>(this->m_vecMeshInstances.size());nMeshInstance++)
	{
		// get mesh instance
		C3DMeshInstance* pMeshInstance=this->m_vecMeshInstances[nMeshInstance];
		if (pMeshInstance)
		{
			const C3DMesh* pMesh=pMeshInstance->GetMesh();
			if (pMesh)
			{
				// get local transformation of mesh instance
				D3DXMATRIX matLocalTransform=pMeshInstance->GetLocalTransform();

				// transform coordinates of bounding box
				const PNPTC_AABBOX& bbox=pMesh->GetBoundingBox();
				D3DXVECTOR3 v3Min(bbox.vMin.x,bbox.vMin.y,bbox.vMin.z);
				D3DXVECTOR3 v3Max(bbox.vMax.x,bbox.vMax.y,bbox.vMax.z);
				D3DXVECTOR3 verticesSource[8];
				verticesSource[0]=D3DXVECTOR3(bbox.vMin.x,bbox.vMin.y,bbox.vMin.z);
				verticesSource[1]=D3DXVECTOR3(bbox.vMax.x,bbox.vMin.y,bbox.vMin.z);
				verticesSource[2]=D3DXVECTOR3(bbox.vMin.x,bbox.vMax.y,bbox.vMin.z);
				verticesSource[3]=D3DXVECTOR3(bbox.vMin.x,bbox.vMin.y,bbox.vMax.z);
				verticesSource[4]=D3DXVECTOR3(bbox.vMax.x,bbox.vMax.y,bbox.vMin.z);
				verticesSource[5]=D3DXVECTOR3(bbox.vMax.x,bbox.vMin.y,bbox.vMax.z);
				verticesSource[6]=D3DXVECTOR3(bbox.vMin.x,bbox.vMax.y,bbox.vMax.z);
				verticesSource[7]=D3DXVECTOR3(bbox.vMax.x,bbox.vMax.y,bbox.vMax.z);
				D3DXVECTOR3 verticesDest[8];
				D3DXVec3TransformCoord(&verticesDest[0],&verticesSource[0],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[1],&verticesSource[1],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[2],&verticesSource[2],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[3],&verticesSource[3],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[4],&verticesSource[4],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[5],&verticesSource[5],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[6],&verticesSource[6],&matLocalTransform);
				D3DXVec3TransformCoord(&verticesDest[7],&verticesSource[7],&matLocalTransform);

				// create bounding box of this mesh instance
				pnpse::CBoundingBox bboxTransformed;
				bboxTransformed.SetBorderPoints(minimum(minimum(minimum(minimum(minimum(minimum(minimum(verticesDest[0].x,verticesDest[1].x),verticesDest[2].x),verticesDest[3].x),verticesDest[4].x),verticesDest[5].x),verticesDest[6].x),verticesDest[7].x),
												minimum(minimum(minimum(minimum(minimum(minimum(minimum(verticesDest[0].y,verticesDest[1].y),verticesDest[2].y),verticesDest[3].y),verticesDest[4].y),verticesDest[5].y),verticesDest[6].y),verticesDest[7].y),
												min(minimum(minimum(minimum(minimum(minimum(minimum(verticesDest[0].z,verticesDest[1].z),verticesDest[2].z),verticesDest[3].z),verticesDest[4].z),verticesDest[5].z),verticesDest[6].z),verticesDest[7].z),
												maximum(maximum(maximum(maximum(maximum(maximum(maximum(verticesDest[0].x,verticesDest[1].x),verticesDest[2].x),verticesDest[3].x),verticesDest[4].x),verticesDest[5].x),verticesDest[6].x),verticesDest[7].x),
												maximum(maximum(maximum(maximum(maximum(maximum(maximum(verticesDest[0].y,verticesDest[1].y),verticesDest[2].y),verticesDest[3].y),verticesDest[4].y),verticesDest[5].y),verticesDest[6].y),verticesDest[7].y),
												maximum(maximum(maximum(maximum(maximum(maximum(maximum(verticesDest[0].z,verticesDest[1].z),verticesDest[2].z),verticesDest[3].z),verticesDest[4].z),verticesDest[5].z),verticesDest[6].z),verticesDest[7].z));

				// update object bounding box
				if (bFirstBB)
				{
					// this is the first mesh instance => initialize bounding box
					this->m_BoundingBox.vMin.x=bboxTransformed.GetMin().x;
					this->m_BoundingBox.vMin.y=bboxTransformed.GetMin().y;
					this->m_BoundingBox.vMin.z=bboxTransformed.GetMin().z;
					this->m_BoundingBox.vMax.x=bboxTransformed.GetMax().x;
					this->m_BoundingBox.vMax.y=bboxTransformed.GetMax().y;
					this->m_BoundingBox.vMax.z=bboxTransformed.GetMax().z;

					// don't initialize again
					bFirstBB=false;
				}
				else
				{
					// update bounding box for all following mesh instances
					this->m_BoundingBox.vMin.x=minimum(bboxTransformed.GetMin().x,this->m_BoundingBox.vMin.x);
					this->m_BoundingBox.vMin.y=minimum(bboxTransformed.GetMin().y,this->m_BoundingBox.vMin.y);
					this->m_BoundingBox.vMin.z=minimum(bboxTransformed.GetMin().z,this->m_BoundingBox.vMin.z);
					this->m_BoundingBox.vMax.x=maximum(bboxTransformed.GetMax().x,this->m_BoundingBox.vMax.x);
					this->m_BoundingBox.vMax.y=maximum(bboxTransformed.GetMax().y,this->m_BoundingBox.vMax.y);
					this->m_BoundingBox.vMax.z=maximum(bboxTransformed.GetMax().z,this->m_BoundingBox.vMax.z);
				}
			}
		}
	}
}

bool C3DObjectData::Intersect(const pnpse::CRay& ray) const
{
	// go through all mesh instances
	for (unsigned int nMeshInstance=0;nMeshInstance<static_cast<unsigned int>(this->m_vecMeshInstances.size());nMeshInstance++)
	{
		// try to hit that mesh instance
		if (this->m_vecMeshInstances[nMeshInstance] && 
			this->m_vecMeshInstances[nMeshInstance]->Intersect(ray))
		{
			// hit it
			return true;
		}
	}

	// nothing was hit
	return false;
}

bool C3DObjectData::ReCreateMeshes(unsigned int nMaxNumVertices,unsigned int nMaxNumFaces)
{
	// get number of initially available meshes
	unsigned int nMeshes=static_cast<unsigned int>(this->GetNumMeshes());

	// go through all meshes and test their parameters
	if (nMeshes>0)
	{
		for (int iMesh=static_cast<int>(nMeshes)-1;iMesh>=0;iMesh--)
		{
			// get that mesh
			C3DMesh* pMesh=this->GetMesh(iMesh);
			if (pMesh)
			{
				// check parameters
				if (pMesh->GetNumVertices()>nMaxNumVertices || pMesh->GetNumFaces()>nMaxNumFaces)
				{
					// ok, this mesh has to be recreated
					std::vector<bool> vecVertexCopied;
					std::vector<DWORD> vecNewVertexIndex;
					vecVertexCopied.resize(pMesh->GetNumVertices(),false);
					vecNewVertexIndex.resize(pMesh->GetNumVertices(),0);

					// create a new mesh
					C3DMesh* pNewMesh=new C3DMesh(this,pMesh->HasVertexColor(),pMesh->HasVertexNormals(),static_cast<int>(pMesh->GetNumMappingCoords()));
					if (!pNewMesh) return false;
					this->m_vecMeshes.push_back(pNewMesh);

					// go through all face sets
					for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
					{
						// get that face set
						C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
						if (pFaceSet)
						{
							// create new face set
							C3DFaceSet* pNewFaceSet=new C3DFaceSet(pNewMesh,pFaceSet->GetMaterial());
							if (!pNewFaceSet) return false;
							pNewMesh->AddFaceSet(pNewFaceSet);

							// copy all faces from the old to the new face set
							for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
							{
								// copy this face
								PNPTC_FACE face=pFaceSet->GetFace(nFace);

								// test how many new vertices we have to create
								unsigned int nNumNewIndices=0;
								if (!vecVertexCopied[face.dwVert[0]]) nNumNewIndices++;
								if (!vecVertexCopied[face.dwVert[1]]) nNumNewIndices++;
								if (!vecVertexCopied[face.dwVert[2]]) nNumNewIndices++;

								// test if we can create the new vertices
								if (nNumNewIndices>0)
								{
									if (pNewMesh->GetNumVertices()+nNumNewIndices>nMaxNumVertices)
									{
										// start with a new mesh
										pNewMesh=new C3DMesh(this,pMesh->HasVertexColor(),pMesh->HasVertexNormals(),static_cast<int>(pMesh->GetNumMappingCoords()));
										if (!pNewMesh) return false;
										this->m_vecMeshes.push_back(pNewMesh);
										pNewFaceSet=new C3DFaceSet(pNewMesh,pFaceSet->GetMaterial());
										if (!pNewFaceSet) return false;
										pNewMesh->AddFaceSet(pNewFaceSet);
										vecVertexCopied.resize(0,false);
										vecNewVertexIndex.resize(0,0);											
										vecVertexCopied.resize(pMesh->GetNumVertices(),false);
										vecNewVertexIndex.resize(pMesh->GetNumVertices(),0);		
									}

									// create vertices
									if (!vecVertexCopied[face.dwVert[0]])
									{
										vecNewVertexIndex[face.dwVert[0]]=pNewMesh->AddVertex(pMesh->GetVertex(face.dwVert[0]));
										vecVertexCopied[face.dwVert[0]]=true;
									}
									if (!vecVertexCopied[face.dwVert[1]])
									{
										vecNewVertexIndex[face.dwVert[1]]=pNewMesh->AddVertex(pMesh->GetVertex(face.dwVert[1]));
										vecVertexCopied[face.dwVert[1]]=true;
									}
									if (!vecVertexCopied[face.dwVert[2]])
									{
										vecNewVertexIndex[face.dwVert[2]]=pNewMesh->AddVertex(pMesh->GetVertex(face.dwVert[2]));
										vecVertexCopied[face.dwVert[2]]=true;
									}
								}

								// get the vertex index in the new mesh
								face.dwVert[0]=vecNewVertexIndex[face.dwVert[0]];
								face.dwVert[1]=vecNewVertexIndex[face.dwVert[1]];
								face.dwVert[2]=vecNewVertexIndex[face.dwVert[2]];
								
								// add face to new face set
								pNewFaceSet->AddFace(face);

								// check if we reached the face limit
								if (pNewMesh->GetNumFaces()>=nMaxNumFaces)
								{
									// start with a new mesh
									pNewMesh=new C3DMesh(this,pMesh->HasVertexColor(),pMesh->HasVertexNormals(),static_cast<int>(pMesh->GetNumMappingCoords()));
									if (!pNewMesh) return false;
									this->m_vecMeshes.push_back(pNewMesh);
									pNewFaceSet=new C3DFaceSet(pNewMesh,pFaceSet->GetMaterial());
									if (!pNewFaceSet) return false;
									pNewMesh->AddFaceSet(pNewFaceSet);
									vecVertexCopied.resize(0,false);
									vecNewVertexIndex.resize(0,0);											
									vecVertexCopied.resize(pMesh->GetNumVertices(),false);
									vecNewVertexIndex.resize(pMesh->GetNumVertices(),0);											
								}
							}
						}
						else
						{
							// could not read face set
							return false;
						}
					}
				}

				// delete this mesh
				delete pMesh;
				this->m_vecMeshes.erase(this->m_vecMeshes.begin()+iMesh);
			}
			else
			{
				// could not read face
				return false;
			}
		}
	}

	// ok
	return true;
}

size_t C3DObjectData::GetNumFaces() const
{
	// compute overall number of faces from all meshes
	size_t nNumFaces=0;
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh])
		{
			nNumFaces+=this->m_vecMeshes[nMesh]->GetNumFaces();
		}
	}

	// return result
	return nNumFaces;
}

size_t C3DObjectData::GetNumVertices() const
{
	// compute overall number of faces from all meshes
	size_t nNumVertices=0;
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh])
		{
			nNumVertices+=this->m_vecMeshes[nMesh]->GetNumVertices();
		}
	}

	// return result
	return nNumVertices;
}

bool C3DObjectData::ComputeNormals(bool bReCompute/*=false*/,PNPTC_AABBOX* pBBoxRegion/*=NULL*/)
{
	// compute normals of all meshes
	bool bReturn=true;
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh])
		{
			bReturn&=this->m_vecMeshes[nMesh]->ComputeNormals(bReCompute,pBBoxRegion);
		}
	}

	// return result
	return bReturn;
}

void C3DObjectData::RemoveUnusedVertices()
{
	// compute unused vertices of all meshes
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh])
		{
			this->m_vecMeshes[nMesh]->RemoveUnusedVertices();
		}
	}
}

void C3DObjectData::CombineMeshes()
{
	// go through all meshes
	for (unsigned int nMesh1=0;nMesh1<this->GetNumMeshes();nMesh1++)
	{
		C3DMesh* pMesh1=this->GetMesh(nMesh1);
		if (pMesh1)
		{
			// try to find a second mesh with the same parameters
			for (unsigned int nMesh2=nMesh1+1;nMesh2<this->GetNumMeshes();)
			{
				C3DMesh* pMesh2=this->GetMesh(nMesh2);
				if (pMesh2)
				{
					// test parameters
					if (pMesh1->GetNumMappingCoords() == pMesh2->GetNumMappingCoords() &&
						pMesh1->HasVertexColor()      == pMesh2->HasVertexColor()      &&
						pMesh1->HasVertexNormals()    == pMesh2->HasVertexNormals()    )
					{
						// copy all vertices
						unsigned int nVertexOffset=static_cast<unsigned int>(pMesh1->GetNumVertices());
						for (unsigned int nVertex=0;nVertex<pMesh2->GetNumVertices();nVertex++)
						{
							pMesh1->AddVertex(pMesh2->GetVertex(nVertex));
						}

						// copy all face sets
						for (unsigned int nFaceSet=0;nFaceSet<pMesh2->GetNumFaceSets();nFaceSet++)
						{
							C3DFaceSet* pFaceSetSource=pMesh2->GetFaceSet(nFaceSet);
							if (pFaceSetSource)
							{
								C3DFaceSet* pFaceSetDest=new C3DFaceSet(pMesh1,pFaceSetSource->GetMaterial());
								if (pFaceSetDest)
								{
									pMesh1->AddFaceSet(pFaceSetDest);
									pFaceSetDest->ReserveFaceMem(static_cast<DWORD>(pFaceSetSource->GetNumFaces()));
									for (unsigned int nFace=0;nFace<pFaceSetSource->GetNumFaces();nFace++)
									{
										PNPTC_FACE face=pFaceSetSource->GetFace(nFace);
										face.dwVert[0]+=nVertexOffset;
										face.dwVert[1]+=nVertexOffset;
										face.dwVert[2]+=nVertexOffset;
										pFaceSetDest->AddFace(face);
									}
								}
							}
						}

						// delete that mesh
						delete this->m_vecMeshes[nMesh2];
						this->m_vecMeshes.erase(this->m_vecMeshes.begin()+nMesh2);
					}
					else
					{
						// not the same parameters => test next mesh
						nMesh2++;
					}
				}
				else
				{
					// test next mesh
					nMesh2++;
				}
			}
		}
	}
}

void C3DObjectData::CombineFaceSets()
{
	// combine face sets of all meshes
	for (unsigned int nMesh=0;nMesh<this->m_vecMeshes.size();nMesh++)
	{
		if (this->m_vecMeshes[nMesh])
		{
			this->m_vecMeshes[nMesh]->CombineFaceSets();
		}
	}
}

bool C3DObjectData::AddCollisionGeometry(C3DCollisionGeometry* pCollisionGeometry)
{
	// check pointer
	if (pCollisionGeometry)
	{
		// add geometry
		this->m_vecCollisionGeometries.push_back(pCollisionGeometry);

		// ok
		return true;
	}
	else
	{
		// invalid geometry
		return false;
	}
}

size_t C3DObjectData::GetNumCollisionGeometries() const
{
	// return number of collision geometries
	return this->m_vecCollisionGeometries.size();
}

C3DCollisionGeometry* C3DObjectData::GetCollisionGeometry(int iCollisionGeometry)
{
	// check index
	if (iCollisionGeometry>=0 && iCollisionGeometry<static_cast<int>(this->m_vecCollisionGeometries.size()))
	{
		return this->m_vecCollisionGeometries[iCollisionGeometry];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

const C3DCollisionGeometry* C3DObjectData::GetCollisionGeometry(int iCollisionGeometry) const
{
	// check index
	if (iCollisionGeometry>=0 && iCollisionGeometry<static_cast<int>(this->m_vecCollisionGeometries.size()))
	{
		return this->m_vecCollisionGeometries[iCollisionGeometry];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

bool C3DObjectData::AddJoint(C3DJoint* pJoint)
{
	// add to list
	this->m_vecJoints.push_back(pJoint);
	return true;
}

size_t C3DObjectData::GetNumJoints() const
{
	// return number of joints
	return this->m_vecJoints.size();
}

C3DJoint* C3DObjectData::GetJoint(unsigned int nJoint) const
{
	// check index
	if (nJoint<this->GetNumJoints())
	{
		// return joint
		return this->m_vecJoints[nJoint];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

bool C3DObjectData::AddReferenceFrame(const std::string& strName,const D3DXMATRIX& matPose)
{
	// add reference frame
	ReferenceFrame frm;
	frm.strName=strName;
	frm.matPos=matPose;
	this->m_vecReferenceFrames.push_back(frm);
	return true;
}

size_t C3DObjectData::GetNumReferenceFrames() const
{
	// get number of reference frames
	return this->m_vecReferenceFrames.size();
}

const C3DObjectData::ReferenceFrame* C3DObjectData::GetReferenceFrame(unsigned int nReferenceFrame) const
{
	// check index
	if (nReferenceFrame<this->m_vecReferenceFrames.size())
	{
		// return that reference frame
		return &this->m_vecReferenceFrames[nReferenceFrame];
	}
	else
	{
		// invalid index
		return NULL;
	}
}





// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// C3DMeshInstance
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------





C3DMeshInstance::C3DMeshInstance(const std::string& strName,C3DMesh* pMesh,const D3DXMATRIX& matLocalTransform)
	: m_strName(strName)
	, m_pMesh(pMesh)
	, m_matLocalTransform(matLocalTransform)
	, m_pCollisionGeometry(NULL)
{
}

C3DMeshInstance::~C3DMeshInstance()
{
}

bool C3DMeshInstance::Intersect(const pnpse::CRay& ray) const
{
	// is the mesh available
	if (!this->m_pMesh)
	{
		return false;
	}

	// convert ray into local coordinate system of that instance
	D3DXVECTOR3 v3Start(ray.GetStartPoint().x,ray.GetStartPoint().y,ray.GetStartPoint().z);
	D3DXVECTOR3 v3Dir(ray.GetDirection().x,ray.GetDirection().y,ray.GetDirection().z);
	D3DXMATRIX matInvLocalTransform;
	D3DXMatrixInverse(&matInvLocalTransform,NULL,&this->m_matLocalTransform);
	D3DXVec3TransformCoord(&v3Start,&v3Start,&matInvLocalTransform);
	matInvLocalTransform._41=0;
	matInvLocalTransform._42=0;
	matInvLocalTransform._43=0;
	D3DXVec3TransformCoord(&v3Dir,&v3Dir,&matInvLocalTransform);
	pnpse::CRay rayLocal;
	rayLocal.MakeRayFromPointDirection(pnpse::CVector3(v3Start.x,v3Start.y,v3Start.z),pnpse::CVector3(v3Dir.x,v3Dir.y,v3Dir.z));

	// try to hit the mesh
	return this->m_pMesh->Intersect(rayLocal);
}





// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// C3DMesh
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------


C3DMesh::C3DMesh(C3DObjectData* pObj,bool bColor,bool bNormal,int iUVCoordinates)
	: m_pObject(pObj)
	, m_bColor(bColor)
	, m_bNormal(bNormal)
	, m_iUVCoordinates(max(min(static_cast<int>(iUVCoordinates),3),0))
	, m_pUserData(NULL)
{
}

C3DMesh* C3DMesh::Clone(C3DObjectData* pObj) const
{
	// create new mesh
	C3DMesh* pNewMesh=new C3DMesh(pObj,this->m_bColor,this->m_bNormal,this->m_iUVCoordinates);
	if (pNewMesh)
	{
		// copy further parameters
		pNewMesh->m_strName=this->m_strName;
		pNewMesh->m_BoundingBox=this->m_BoundingBox;
		pNewMesh->m_pUserData=this->m_pUserData;

		// copy all face sets
		for (unsigned int nFaceSet=0;nFaceSet<this->m_vecFaceSets.size();nFaceSet++)
		{
			if (this->m_vecFaceSets[nFaceSet])
			{
				C3DFaceSet* pNewFaceSet=this->m_vecFaceSets[nFaceSet]->Clone(pNewMesh);
				if (pNewFaceSet)
				{
					pNewMesh->m_vecFaceSets.push_back(pNewFaceSet);
				}
			}
		}

		// copy vertices
		pNewMesh->m_vecVertices=this->m_vecVertices;
	}

	// return newly created mesh
	return pNewMesh;
}

C3DMesh::~C3DMesh(void)
{
	// clear vertices list
	this->m_vecVertices.clear();

	// remove all face setc
	for (unsigned long l=0;l<this->m_vecFaceSets.size();l++)
	{
		if (this->m_vecFaceSets[l])
		{
			delete this->m_vecFaceSets[l];
		}
	}
	this->m_vecFaceSets.clear();
}

void C3DMesh::SetName(const std::string& strName)
{
	// set name
	this->m_strName=strName;
}

bool C3DMesh::ReserveVertexMem(DWORD dwNumVertices)
{
	// reserve mem
	this->m_vecVertices.reserve(dwNumVertices);
	return true;
}

bool C3DMesh::SetNumVertices(size_t nVertices)
{
	// set number of vertices
	this->m_vecVertices.resize(nVertices);
	return true;
}

int C3DMesh::AddVertex(const PNPTC_VERTEX& vert)
{
	// add vertex to list
	this->m_vecVertices.push_back(vert);
	// return vertex index
	return (int)this->m_vecVertices.size()-1;
}

int C3DMesh::AddFaceSet(C3DFaceSet* faceset)
{
	// add mesh to list
	this->m_vecFaceSets.push_back(faceset);
	// return mesh index
	return (int)this->m_vecFaceSets.size()-1;
}

size_t C3DMesh::GetNumFaces() const
{
	// summarize all faces of all face sets
	size_t nFaces=0;
	for (int iFaceSet=0;iFaceSet<(int)this->m_vecFaceSets.size();iFaceSet++)
	{
		if (this->m_vecFaceSets[iFaceSet])
		{
			nFaces+=this->m_vecFaceSets[iFaceSet]->GetNumFaces();
		}
	}

	// return number of faces
	return nFaces;
}

size_t C3DMesh::GetNumVertices() const
{
	// return number of vertices
	return this->m_vecVertices.size();
}

size_t C3DMesh::GetNumMappingCoords() const
{
	// return number of uv coordinate pairs
	return (size_t)this->m_iUVCoordinates;
}

void C3DMesh::SetNumMappingCoords(size_t nMappingCoords)
{
	this->m_iUVCoordinates=max(min(static_cast<int>(nMappingCoords),3),0);
}

bool C3DMesh::HasVertexColor() const
{
	// return vertex colors
	return this->m_bColor;
}

bool C3DMesh::HasVertexNormals() const
{
	// return vertex normals
	return this->m_bNormal;
}

PNPTC_VERTEX& C3DMesh::GetVertex(unsigned int nVertex)
{
	if (nVertex<this->m_vecVertices.size())
	{
		// return vertex
		return this->m_vecVertices[nVertex];
	}
	else
	{
		// invalid index
		static PNPTC_VERTEX vert;
		memset(&vert,0,sizeof(PNPTC_VERTEX));
		return vert;
	}
}

const PNPTC_VERTEX& C3DMesh::GetVertex(unsigned int nVertex) const
{
	if (nVertex<this->m_vecVertices.size())
	{
		// return vertex
		return this->m_vecVertices[nVertex];
	}
	else
	{
		// invalid index
		static PNPTC_VERTEX vert;
		memset(&vert,0,sizeof(PNPTC_VERTEX));
		return vert;
	}
}

size_t C3DMesh::GetNumFaceSets() const
{
	// return number of face sets
	return this->m_vecFaceSets.size();
}

C3DFaceSet* C3DMesh::GetFaceSet(unsigned int nFaceSet)
{
	// check index
	if (nFaceSet<this->m_vecFaceSets.size())
	{
		return this->m_vecFaceSets[nFaceSet];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

const C3DFaceSet* C3DMesh::GetFaceSet(unsigned int nFaceSet) const
{
	// check index
	if (nFaceSet<this->m_vecFaceSets.size())
	{
		return this->m_vecFaceSets[nFaceSet];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

void C3DMesh::SetVertexPos(DWORD dwVertex,float fPos[3])
{
	// check index
	if (dwVertex<static_cast<DWORD>(this->m_vecVertices.size()))
	{
		this->m_vecVertices[dwVertex].vPos.x=fPos[0];
		this->m_vecVertices[dwVertex].vPos.y=fPos[1];
		this->m_vecVertices[dwVertex].vPos.z=fPos[2];
	}
}

void C3DMesh::SetVertexUV(DWORD dwVertex,unsigned int nUVCoordinate,float fUV[2])
{
	// check index
	if (dwVertex<static_cast<DWORD>(this->m_vecVertices.size()))
	{
		this->m_vecVertices[dwVertex].uv[min(nUVCoordinate,2)].u=fUV[0];
		this->m_vecVertices[dwVertex].uv[min(nUVCoordinate,2)].v=fUV[1];
	}
}

void C3DMesh::SetVertexNormal(DWORD dwVertex,float fNormal[3])
{
	// check index
	if (dwVertex<static_cast<DWORD>(this->m_vecVertices.size()))
	{
		this->m_vecVertices[dwVertex].vNormal.x=fNormal[0];
		this->m_vecVertices[dwVertex].vNormal.y=fNormal[1];
		this->m_vecVertices[dwVertex].vNormal.z=fNormal[2];
	}
}

void C3DMesh::SetVertexColor(DWORD dwVertex,const PNPTC_RGBA& rgbaColor)
{
	// check index
	if (dwVertex<static_cast<DWORD>(this->m_vecVertices.size()))
	{
		this->m_vecVertices[dwVertex].colDiffuse=rgbaColor;
	}
}

void C3DMesh::ComputeBoundingBox()
{
	// compute bounding box over all vertices
	for (unsigned int nVertex=0;nVertex<static_cast<unsigned int>(this->m_vecVertices.size());nVertex++)
	{
		if (nVertex==0)
		{
			// this is the first vertex => initialize bounding box
			this->m_BoundingBox.vMin.x=this->m_vecVertices[nVertex].vPos.x;
			this->m_BoundingBox.vMin.y=this->m_vecVertices[nVertex].vPos.y;
			this->m_BoundingBox.vMin.z=this->m_vecVertices[nVertex].vPos.z;
			this->m_BoundingBox.vMax.x=this->m_vecVertices[nVertex].vPos.x;
			this->m_BoundingBox.vMax.y=this->m_vecVertices[nVertex].vPos.y;
			this->m_BoundingBox.vMax.z=this->m_vecVertices[nVertex].vPos.z;
		}
		else
		{
			// update bounding box for all following vertices
			this->m_BoundingBox.vMin.x=minimum(this->m_vecVertices[nVertex].vPos.x,this->m_BoundingBox.vMin.x);
			this->m_BoundingBox.vMin.y=minimum(this->m_vecVertices[nVertex].vPos.y,this->m_BoundingBox.vMin.y);
			this->m_BoundingBox.vMin.z=minimum(this->m_vecVertices[nVertex].vPos.z,this->m_BoundingBox.vMin.z);
			this->m_BoundingBox.vMax.x=maximum(this->m_vecVertices[nVertex].vPos.x,this->m_BoundingBox.vMax.x);
			this->m_BoundingBox.vMax.y=maximum(this->m_vecVertices[nVertex].vPos.y,this->m_BoundingBox.vMax.y);
			this->m_BoundingBox.vMax.z=maximum(this->m_vecVertices[nVertex].vPos.z,this->m_BoundingBox.vMax.z);
		}
	}
}

bool C3DMesh::Intersect(const pnpse::CRay& ray) const
{
	// go through all face sets
	for (unsigned int nFaceSet=0;nFaceSet<this->GetNumFaceSets();nFaceSet++)
	{
		const C3DFaceSet* pFaceSet=this->GetFaceSet(nFaceSet);
		if (pFaceSet && pFaceSet->Intersect(ray))
		{
			// hit it
			return true;
		}
	}

	// nothing was hit
	return false;
}

bool C3DMesh::ComputeNormals(bool bReCompute/*=false*/,PNPTC_AABBOX* pBBoxRegion/*=NULL*/)
{
	// check if normals are already available
	if (this->m_bNormal && !bReCompute)
	{
		return true;
	}

	// create temporary mem for normals
	std::vector<D3DXVECTOR3> vecNormals;
	vecNormals.resize(this->GetNumVertices(),D3DXVECTOR3(0,0,0));

	// go through all face sets
	for (unsigned int nFaceSet=0;nFaceSet<this->GetNumFaceSets();nFaceSet++)
	{
		C3DFaceSet* pFaceSet=this->GetFaceSet(nFaceSet);
		if (pFaceSet)
		{
			for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
			{
				// get the three vertices
				PNPTC_FACE& face=pFaceSet->GetFace(nFace);
				D3DXVECTOR3 vec1((float*)&this->m_vecVertices[face.dwVert[0]].vPos);
				D3DXVECTOR3 vec2((float*)&this->m_vecVertices[face.dwVert[1]].vPos);
				D3DXVECTOR3 vec3((float*)&this->m_vecVertices[face.dwVert[2]].vPos);
				
				// compute normal of the face
				D3DXVECTOR3 vec12=-vec1+vec2;
				D3DXVECTOR3 vec13=-vec1+vec3;
				D3DXVECTOR3 vecFaceNormal;
				D3DXVec3Cross(&vecFaceNormal,&vec12,&vec13);

				// add to normals of all vertices
				vecNormals[face.dwVert[0]]+=vecFaceNormal;
				vecNormals[face.dwVert[1]]+=vecFaceNormal;
				vecNormals[face.dwVert[2]]+=vecFaceNormal;
			}
		}
	}

	// normalize and set all normals
	for (unsigned int nVertex=0;nVertex<this->m_vecVertices.size();nVertex++)
	{
		// test vertex position
		if (!pBBoxRegion || pBBoxRegion->Inside(this->m_vecVertices[nVertex].vPos))
		{
			// normalize it
			D3DXVECTOR3 vecTemp;
			D3DXVec3Normalize(&vecTemp,&vecNormals[nVertex]);

			// set value
			this->m_vecVertices[nVertex].vNormal.x=vecNormals[nVertex].x;
			this->m_vecVertices[nVertex].vNormal.y=vecNormals[nVertex].y;
			this->m_vecVertices[nVertex].vNormal.z=vecNormals[nVertex].z;
		}
	}

	// set normals as beeing activated
	this->m_bNormal=true;

	// ok
	return true;
}

void C3DMesh::RemoveUnusedVertices()
{
	// create an array to find out which vertices are used
	std::vector<bool> vecUsedVertices;
	std::vector<int> vecVertexOffsets;
	vecUsedVertices.resize(this->GetNumVertices(),false);
	vecVertexOffsets.resize(this->GetNumVertices(),0);

	// go through all face sets
	for (unsigned int nFaceSet=0;nFaceSet<this->GetNumFaceSets();nFaceSet++)
	{
		const C3DFaceSet* pFaceSet=this->GetFaceSet(nFaceSet);
		if (pFaceSet)
		{
			// go through all faces
			for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
			{
				const PNPTC_FACE& face=pFaceSet->GetFace(nFace);
				vecUsedVertices[face.dwVert[0]]=true;
				vecUsedVertices[face.dwVert[1]]=true;
				vecUsedVertices[face.dwVert[2]]=true;
			}
		}
	}

	// compute offset for all vertex indices
	bool bFoundUnusedVertex=false;
	int iCurrentOffset=0;
	for (unsigned int nVertex=0;nVertex<vecUsedVertices.size();nVertex++)
	{
		// store offset for this vertex
		vecVertexOffsets[nVertex]=iCurrentOffset;

		// check if vertex is unused
		if (!vecUsedVertices[nVertex])
		{
			// this is an unused vertex
			bFoundUnusedVertex=true;
			iCurrentOffset++;
		}
	}

	// update faces and delete vertices
	if (bFoundUnusedVertex)
	{
		// go through all face sets
		for (unsigned int nFaceSet=0;nFaceSet<this->GetNumFaceSets();nFaceSet++)
		{
			C3DFaceSet* pFaceSet=this->GetFaceSet(nFaceSet);
			if (pFaceSet)
			{
				// go through all faces
				for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
				{
					PNPTC_FACE& face=pFaceSet->GetFace(nFace);
					face.dwVert[0]=face.dwVert[0]-vecVertexOffsets[face.dwVert[0]];
					face.dwVert[1]=face.dwVert[1]-vecVertexOffsets[face.dwVert[1]];
					face.dwVert[2]=face.dwVert[2]-vecVertexOffsets[face.dwVert[2]];
				}
			}
		}

		// delete those vertices
		for (int nVertex=static_cast<int>(vecUsedVertices.size())-1;nVertex>=0;nVertex--)
		{
			if (!vecUsedVertices[nVertex])
			{
				// delete this vertex
				this->m_vecVertices.erase(this->m_vecVertices.begin()+nVertex);
			}
		}
	}
}

void C3DMesh::CombineFaceSets()
{
	// go through all face sets
	for (unsigned int nFaceSet1=0;nFaceSet1<this->GetNumFaceSets();nFaceSet1++)
	{
		C3DFaceSet* pFaceSet1=this->GetFaceSet(nFaceSet1);
		if (pFaceSet1)
		{
			// try to find a second face set with the same material
			for (unsigned int nFaceSet2=nFaceSet1+1;nFaceSet2<this->GetNumFaceSets();)
			{
				C3DFaceSet* pFaceSet2=this->GetFaceSet(nFaceSet2);
				if (pFaceSet2)
				{
					// test material
					if (pFaceSet1->GetMaterial()==pFaceSet2->GetMaterial())
					{
						// copy all faces
						for (unsigned int nFace=0;nFace<pFaceSet2->GetNumFaces();nFace++)
						{
							pFaceSet1->AddFace(pFaceSet2->GetFace(nFace));
						}

						// delete that face set
						delete this->m_vecFaceSets[nFaceSet2];
						this->m_vecFaceSets.erase(this->m_vecFaceSets.begin()+nFaceSet2);
					}
					else
					{
						// not the same material => test next face set
						nFaceSet2++;
					}
				}
				else
				{
					// test next face set
					nFaceSet2++;
				}
			}
		}
	}
}


// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// C3DFaceSet
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------



C3DFaceSet::C3DFaceSet(C3DMesh* pMesh,int iMaterial)
	: m_pMesh(pMesh)
	, m_pUserData(NULL)
	, m_iMaterial(iMaterial)
	, m_pHullHierarchy(NULL)
{
}

C3DFaceSet::~C3DFaceSet(void)
{
	// clear hull hierarchy
	SAFE_DELETE(this->m_pHullHierarchy);

	// clear face list
	this->m_vecFaces.clear();
}

C3DFaceSet* C3DFaceSet::Clone(C3DMesh* pMesh) const
{
	// create a new face set
	C3DFaceSet* pNewFaceSet=new C3DFaceSet(pMesh,this->GetMaterial());
	if (pNewFaceSet)
	{
		// set user data
		pNewFaceSet->SetUserDataPtr(this->GetUserDataPtr());

		// copy faces
		pNewFaceSet->m_vecFaces=this->m_vecFaces;
	}

	// return new face set
	return pNewFaceSet;
}

bool C3DFaceSet::ReserveFaceMem(DWORD dwNumFaces)
{
	// reserve mem
	this->m_vecFaces.reserve(dwNumFaces);
	return true;
}

bool C3DFaceSet::SetNumFaces(size_t nFaces)
{
	// set number of faces
	this->m_vecFaces.resize(nFaces);
	return true;
}

int C3DFaceSet::AddFace(const PNPTC_FACE& face)
{
	// check for degenerated face
	if (face.dwVert[0]==face.dwVert[1] || 
		face.dwVert[0]==face.dwVert[2] ||
		face.dwVert[1]==face.dwVert[2] )
	{
		return -1;
	}

	// add face to list
	this->m_vecFaces.push_back(face);
	// return face index
	return (int)this->m_vecFaces.size()-1;
}

size_t C3DFaceSet::GetNumFaces() const
{
	// return number of faces
	return this->m_vecFaces.size();
}

void C3DFaceSet::SetMaterial(int iMaterial)
{
	// set material index
	this->m_iMaterial=iMaterial;
}

int C3DFaceSet::GetMaterial() const
{
	// return material index
	return this->m_iMaterial;
}

PNPTC_FACE& C3DFaceSet::GetFace(unsigned int nFace)
{
	// check index
	if (nFace<this->m_vecFaces.size())
	{
		// return face
		return this->m_vecFaces[nFace];
	}
	else
	{
		// invalid index
		static PNPTC_FACE face;
		memset(&face,0,sizeof(PNPTC_FACE));
		return face;
	}
}

const PNPTC_FACE& C3DFaceSet::GetFace(unsigned int nFace) const
{
	// check index
	if (nFace<this->m_vecFaces.size())
	{
		// return face
		return this->m_vecFaces[nFace];
	}
	else
	{
		// invalid index
		static PNPTC_FACE face;
		memset(&face,0,sizeof(PNPTC_FACE));
		return face;
	}
}

void C3DFaceSet::SetFace(unsigned int nFace,const PNPTC_FACE& face)
{
	// check index
	if (nFace<this->m_vecFaces.size())
	{
		// set face
		this->m_vecFaces[nFace].dwVert[0]=face.dwVert[0];
		this->m_vecFaces[nFace].dwVert[1]=face.dwVert[1];
		this->m_vecFaces[nFace].dwVert[2]=face.dwVert[2];
	}
}

bool C3DFaceSet::Intersect(const pnpse::CRay& ray) const
{
	// check mesh
	if (!this->m_pMesh)
	{
		return false;
	}

	// check if hull hierarchy exists
	if (!this->m_pHullHierarchy)
	{
		// create new base node
		this->m_pHullHierarchy=new HullHierarchyNode<pnpse::CBoundingBox>();
		if (this->m_pHullHierarchy)
		{
			// initialize triangles
			this->m_pHullHierarchy->m_vecTriangles.reserve(this->m_vecFaces.size());
			for (int i=0;i<static_cast<int>(this->m_vecFaces.size());++i)
			{
				this->m_pHullHierarchy->m_vecTriangles.push_back(i);
			}

			// compute bounding box
			this->ComputeHullHierarchyBoundingBox(this->m_pHullHierarchy);

			// build the hierarchy
			this->BuildHullHierarchy(this->m_pHullHierarchy,0,20,20);
		}
	}

	// hit test against hull hierarchy
	if (this->m_pHullHierarchy)
	{
		// test the hull hierarchy
		std::vector<const HullHierarchyNode<pnpse::CBoundingBox>*> vecHitNodes;
		if (this->m_pHullHierarchy->HitHierarchy(ray,vecHitNodes))
		{
			// test all nodes
			for (std::vector<const HullHierarchyNode<pnpse::CBoundingBox>*>::const_iterator itNode=vecHitNodes.begin();itNode!=vecHitNodes.end();++itNode)
			{
				// test all triangles
				for (std::vector<int>::const_iterator itTriangle=(*itNode)->m_vecTriangles.begin();itTriangle!=(*itNode)->m_vecTriangles.end();++itTriangle)
				{
					// test triangle intersection
					PNPTC_FACE face=this->GetFace(*itTriangle);
					D3DXVECTOR3 v3Vert1((float*)&this->m_pMesh->GetVertex(face.dwVert[0]).vPos);
					D3DXVECTOR3 v3Vert2((float*)&this->m_pMesh->GetVertex(face.dwVert[1]).vPos);
					D3DXVECTOR3 v3Vert3((float*)&this->m_pMesh->GetVertex(face.dwVert[2]).vPos);
					static float fU;
					static float fV;
					static float fHitDist;
					if (D3DXIntersectTri(&v3Vert1,&v3Vert2,&v3Vert3,&D3DXVECTOR3((float*)&ray.GetStartPoint()),&D3DXVECTOR3((float*)&ray.GetDirection()),&fU,&fV,&fHitDist))
					{
						return true;
					}
				}				
			}
		}
		else
		{
			// nothing was hit
			return false;
		}
	}

	// not hit
	return false;
}

void C3DFaceSet::ComputeHullHierarchyBoundingBox(HullHierarchyNode<pnpse::CBoundingBox>* pNode) const
{
	// go through all faces
	for (unsigned int nTriangle=0;nTriangle<pNode->m_vecTriangles.size();++nTriangle)
	{
		// get face
		const PNPTC_FACE face=this->GetFace(pNode->m_vecTriangles[nTriangle]);
		for (unsigned int nVertex=0;nVertex<3;++nVertex)
		{
			// get vertex
			const PnPTC3DVec& v3Pos=this->m_pMesh->GetVertex(face.dwVert[nVertex]).vPos;
			if (nTriangle==0 && nVertex==0)
			{				
				pNode->m_Hull.SetBorderPoints(v3Pos.x,v3Pos.y,v3Pos.z,v3Pos.x,v3Pos.y,v3Pos.z);
			}
			else
			{
				pNode->m_Hull.AddPoint(v3Pos.x,v3Pos.y,v3Pos.z);
			}
		}
	}
}

void C3DFaceSet::BuildHullHierarchy(HullHierarchyNode<pnpse::CBoundingBox>* pNode,int iLevel,int iMinTriangleNumber,int iMaxLevelCount) const
{
	// get center of current bounding box
	pnpse::CVector3 v3Center(pNode->m_Hull.GetCenter());

	// test if we reached the minimum number of faces
	if ((iMaxLevelCount<=0 || iLevel<iMaxLevelCount) && 
		static_cast<int>(pNode->m_vecTriangles.size())>iMinTriangleNumber)
	{
		// no => divide
		int iAxis=iLevel%3;
		pNode->m_pLeftChild=new HullHierarchyNode<pnpse::CBoundingBox>();
		pNode->m_pRightChild=new HullHierarchyNode<pnpse::CBoundingBox>();
		pNode->m_pLeftChild->m_vecTriangles.reserve(pNode->m_vecTriangles.size()*2/3);
		pNode->m_pRightChild->m_vecTriangles.reserve(pNode->m_vecTriangles.size()*2/3);

		// initialize bounding boxes
		if (iAxis==0)
		{
			pNode->m_pLeftChild->m_Hull.SetBorderPoints(pNode->m_Hull.GetMin().x,pNode->m_Hull.GetMin().y,pNode->m_Hull.GetMin().z,
														v3Center.x,pNode->m_Hull.GetMax().y,pNode->m_Hull.GetMax().z);
			pNode->m_pRightChild->m_Hull.SetBorderPoints(v3Center.x,pNode->m_Hull.GetMin().y,pNode->m_Hull.GetMin().z,
														 pNode->m_Hull.GetMax().x,pNode->m_Hull.GetMax().y,pNode->m_Hull.GetMax().z);
		}
		else if (iAxis==1)
		{
			pNode->m_pLeftChild->m_Hull.SetBorderPoints(pNode->m_Hull.GetMin().x,pNode->m_Hull.GetMin().y,pNode->m_Hull.GetMin().z,
														pNode->m_Hull.GetMax().x,v3Center.y,pNode->m_Hull.GetMax().z);
			pNode->m_pRightChild->m_Hull.SetBorderPoints(pNode->m_Hull.GetMin().x,v3Center.y,pNode->m_Hull.GetMin().z,
														 pNode->m_Hull.GetMax().x,pNode->m_Hull.GetMax().y,pNode->m_Hull.GetMax().z);
		}
		else if (iAxis==2)
		{
			pNode->m_pLeftChild->m_Hull.SetBorderPoints(pNode->m_Hull.GetMin().x,pNode->m_Hull.GetMin().y,pNode->m_Hull.GetMin().z,
														pNode->m_Hull.GetMax().x,pNode->m_Hull.GetMax().y,v3Center.z);
			pNode->m_pRightChild->m_Hull.SetBorderPoints(pNode->m_Hull.GetMin().x,pNode->m_Hull.GetMin().y,v3Center.z,
														 pNode->m_Hull.GetMax().x,pNode->m_Hull.GetMax().y,pNode->m_Hull.GetMax().z);
		}

		// add all triangles to the sub nodes
		for (unsigned int nTriangle=0;nTriangle<pNode->m_vecTriangles.size();++nTriangle)
		{
			// get face
			const PNPTC_FACE face=this->GetFace(pNode->m_vecTriangles[nTriangle]);
			bool bAdded=false;
			for (unsigned int nVertex=0;nVertex<3;++nVertex)
			{
				// get vertex
				PnPTC3DVec v3Pos=this->m_pMesh->GetVertex(face.dwVert[nVertex]).vPos;
				if (pNode->m_pLeftChild->m_Hull.Inside(pnpse::CVector3(v3Pos.x,v3Pos.y,v3Pos.z)))
				{
					pNode->m_pLeftChild->m_vecTriangles.push_back(pNode->m_vecTriangles[nTriangle]);
					bAdded=true;
					break;
				}
			}
			if (!bAdded)
			{
				pNode->m_pRightChild->m_vecTriangles.push_back(pNode->m_vecTriangles[nTriangle]);
			}
		}

		// start recursive building
		this->BuildHullHierarchy(pNode->m_pLeftChild,iLevel+1,iMinTriangleNumber,iMaxLevelCount);
		this->BuildHullHierarchy(pNode->m_pRightChild,iLevel+1,iMinTriangleNumber,iMaxLevelCount);

		// update bounding box
		pNode->m_Hull=pNode->m_pLeftChild->m_Hull;
		pNode->m_Hull.AddPoint(pNode->m_pRightChild->m_Hull.GetMin());
		pNode->m_Hull.AddPoint(pNode->m_pRightChild->m_Hull.GetMax());

		// clear list of nodes
		pNode->m_vecTriangles.clear();
		pNode->m_vecTriangles.reserve(0);
	}
	else
	{
		// compute bounding box of this node
		this->ComputeHullHierarchyBoundingBox(pNode);
	}
}






// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionShape
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------




C3DCollisionShape::C3DCollisionShape()
	: m_fSkinWidth(0.025f)
	, m_bCollidable(true)
	, m_bIntersectionTestDataInitialized(false)
{
	// initialize local transform
	D3DXMatrixIdentity(&this->m_matLocalTransform);
}

bool C3DCollisionShape::GetIntersectionCentroid(C3DCollisionShape& Shape2,const D3DXMATRIX& matTransform1,const D3DXMATRIX& matTransform2,D3DXVECTOR3& v3Centroid,float fIncreaseSize/*=0*/)
{
	// initialize intersection test data first
	if (!this->m_bIntersectionTestDataInitialized)
	{
		this->InitializeIntersectionTestData(matTransform1,fIncreaseSize);
		this->m_bIntersectionTestDataInitialized=true;
	}
	if (!Shape2.m_bIntersectionTestDataInitialized)
	{
		Shape2.InitializeIntersectionTestData(matTransform2,fIncreaseSize);
		Shape2.m_bIntersectionTestDataInitialized=true;
	}

	// get bounding boxes
	PNPTC_AABBOX bbox1=this->GetBoundingBox();
	PNPTC_AABBOX bbox2=Shape2.GetBoundingBox();

	// get range in which we search for intersection
	PNPTC_AABBOX bbox;
	bbox.vMin.x=maximum(bbox1.vMin.x,bbox2.vMin.x);
	bbox.vMin.y=maximum(bbox1.vMin.y,bbox2.vMin.y);
	bbox.vMin.z=maximum(bbox1.vMin.z,bbox2.vMin.z);
	bbox.vMax.x=minimum(bbox1.vMax.x,bbox2.vMax.x);
	bbox.vMax.y=minimum(bbox1.vMax.y,bbox2.vMax.y);
	bbox.vMax.z=minimum(bbox1.vMax.z,bbox2.vMax.z);

	// check if overlapping
	if (bbox.vMin.x>=bbox.vMax.x || bbox.vMin.y>=bbox.vMax.y || bbox.vMin.z>=bbox.vMax.z)
	{
		return false;
	}

	// compute centroid
	v3Centroid=D3DXVECTOR3(0,0,0);
	unsigned int nVoxelsOverlapping=0;
	D3DXVECTOR3 v3StepSize(0.5f,0.5f,0.5f);
	D3DXVECTOR3 v3TestPosition;
	for (v3TestPosition.z=bbox.vMin.z;v3TestPosition.z<=bbox.vMax.z;v3TestPosition.z+=v3StepSize.z)
	{
		for (v3TestPosition.y=bbox.vMin.y;v3TestPosition.y<=bbox.vMax.y;v3TestPosition.y+=v3StepSize.y)
		{
			for (v3TestPosition.x=bbox.vMin.x;v3TestPosition.x<=bbox.vMax.x;v3TestPosition.x+=v3StepSize.x)
			{
				if (this->Inside(v3TestPosition) && Shape2.Inside(v3TestPosition))
				{
					// this is an overlapping voxel
					v3Centroid+=v3TestPosition;
					nVoxelsOverlapping++;
				}
			}
		}
	}

	// now compute centroid if any voxel was overlapping
	if (nVoxelsOverlapping!=0)
	{
		// compute centroid
		v3Centroid.x/=static_cast<float>(nVoxelsOverlapping);
		v3Centroid.y/=static_cast<float>(nVoxelsOverlapping);
		v3Centroid.z/=static_cast<float>(nVoxelsOverlapping);

		// shapes intersect
		return true;
	}
	else
	{
		// no overlapping voxels
		return false;
	}
}

bool C3DCollisionShape::Inside(const D3DXVECTOR3& v3Position) const
{
	// just check against bounding box
	// TODO : needs to be implemented for convex hull and capsule test
	if (v3Position.x>=this->m_BoundingBox.vMin.x && v3Position.x<=this->m_BoundingBox.vMax.x &&
		v3Position.y>=this->m_BoundingBox.vMin.y && v3Position.y<=this->m_BoundingBox.vMax.y &&
		v3Position.z>=this->m_BoundingBox.vMin.z && v3Position.z<=this->m_BoundingBox.vMax.z)
	{
		// inside bounding box
		return true;
	}
	else
	{
		// outside bounding box
		return false;
	}
}




// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionSphere
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------




void C3DCollisionSphere::InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize/*=0*/)
{
	// compute the intersection radius
	this->m_fIntersectionSquaredRadius=SQR(this->m_fRadius+fIncreaseSize);

	// compute center of sphere
	D3DXMATRIX matShapePose;
	D3DXMatrixMultiply(&matShapePose,&this->GetLocalTransform(),&matTransform);
	this->m_v3IntersectionCenter.x=matShapePose._41;
	this->m_v3IntersectionCenter.y=matShapePose._42;
	this->m_v3IntersectionCenter.z=matShapePose._43;
}

bool C3DCollisionSphere::Inside(const D3DXVECTOR3& v3Position) const
{
	// compute and test squared distance to center
	float fSquaredDistance=SQR(v3Position.x-this->m_v3IntersectionCenter.x)+SQR(v3Position.y-this->m_v3IntersectionCenter.y)+SQR(v3Position.z-this->m_v3IntersectionCenter.z);
	return (this->m_fIntersectionSquaredRadius<=fSquaredDistance);
}






// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionCapsule
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------




void C3DCollisionCapsule::InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize/*=0*/)
{
	// compute center of capsule
	D3DXMATRIX matShapePose;
	D3DXMatrixMultiply(&matShapePose,&this->GetLocalTransform(),&matTransform);

	// compute the 10 control points of the capsule
	D3DXVECTOR3 v3ControlPoints[10];
	v3ControlPoints[0]=D3DXVECTOR3( this->m_fRadius+fIncreaseSize , this->m_fHeight*0.5f+fIncreaseSize , 0);
	v3ControlPoints[1]=D3DXVECTOR3(-this->m_fRadius-fIncreaseSize , this->m_fHeight*0.5f+fIncreaseSize , 0);
	v3ControlPoints[2]=D3DXVECTOR3(-this->m_fRadius-fIncreaseSize ,-this->m_fHeight*0.5f-fIncreaseSize , 0);
	v3ControlPoints[3]=D3DXVECTOR3( this->m_fRadius+fIncreaseSize ,-this->m_fHeight*0.5f-fIncreaseSize , 0);
	v3ControlPoints[4]=D3DXVECTOR3( 0                             , this->m_fHeight*0.5f+fIncreaseSize , this->m_fRadius+fIncreaseSize);
	v3ControlPoints[5]=D3DXVECTOR3( 0                             , this->m_fHeight*0.5f+fIncreaseSize ,-this->m_fRadius-fIncreaseSize);
	v3ControlPoints[6]=D3DXVECTOR3( 0                             ,-this->m_fHeight*0.5f-fIncreaseSize ,-this->m_fRadius-fIncreaseSize);
	v3ControlPoints[7]=D3DXVECTOR3( 0                             ,-this->m_fHeight*0.5f-fIncreaseSize , this->m_fRadius+fIncreaseSize);
	v3ControlPoints[8]=D3DXVECTOR3( 0 , this->m_fHeight*0.5f+fIncreaseSize+this->m_fRadius , 0);
	v3ControlPoints[9]=D3DXVECTOR3( 0 ,-this->m_fHeight*0.5f-fIncreaseSize-this->m_fRadius , 0);

	// now compute bounding box of those control points
	for (unsigned int n=0;n<10;++n)
	{
		// transform vertex
		D3DXVec3TransformCoord(&v3ControlPoints[n],&v3ControlPoints[n],&matShapePose);

		// update bounding box
		if (n==0)
		{
			// initialize bounding box
			this->m_BoundingBox.vMin.x=v3ControlPoints[n].x;
			this->m_BoundingBox.vMin.y=v3ControlPoints[n].y;
			this->m_BoundingBox.vMin.z=v3ControlPoints[n].z;
			this->m_BoundingBox.vMax.x=v3ControlPoints[n].x;
			this->m_BoundingBox.vMax.y=v3ControlPoints[n].y;
			this->m_BoundingBox.vMax.z=v3ControlPoints[n].z;
		}
		else
		{
			// update bounding box
			this->m_BoundingBox.vMin.x=minimum(this->m_BoundingBox.vMin.x,v3ControlPoints[n].x);
			this->m_BoundingBox.vMin.y=minimum(this->m_BoundingBox.vMin.y,v3ControlPoints[n].y);
			this->m_BoundingBox.vMin.z=minimum(this->m_BoundingBox.vMin.z,v3ControlPoints[n].z);
			this->m_BoundingBox.vMax.x=maximum(this->m_BoundingBox.vMax.x,v3ControlPoints[n].x);
			this->m_BoundingBox.vMax.y=maximum(this->m_BoundingBox.vMax.y,v3ControlPoints[n].y);
			this->m_BoundingBox.vMax.z=maximum(this->m_BoundingBox.vMax.z,v3ControlPoints[n].z);
		}
	}
}




// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionBox
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------



C3DCollisionBox::C3DCollisionBox(float fSizeX,float fSizeY,float fSizeZ)
	: m_fSizeX(fSizeX)
	, m_fSizeY(fSizeY)
	, m_fSizeZ(fSizeZ)
	, m_v3IntersectionHalfSize(fSizeX,fSizeY,fSizeZ)
{
	// further initializations
	D3DXMatrixIdentity(&this->m_matIntersectionInversePose);
}

void C3DCollisionBox::InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize/*=0*/)
{
	// compute center of box
	D3DXMATRIX matShapePose;
	D3DXMatrixMultiply(&matShapePose,&this->GetLocalTransform(),&matTransform);

	// update intersection size
	this->m_v3IntersectionHalfSize.x=this->m_fSizeX*0.5f+fIncreaseSize;
	this->m_v3IntersectionHalfSize.y=this->m_fSizeY*0.5f+fIncreaseSize;
	this->m_v3IntersectionHalfSize.z=this->m_fSizeZ*0.5f+fIncreaseSize;

	// compute the 8 control points of the box
	D3DXVECTOR3 v3ControlPoints[8];
	v3ControlPoints[0]=D3DXVECTOR3( this->m_v3IntersectionHalfSize.x , this->m_v3IntersectionHalfSize.y , this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[1]=D3DXVECTOR3(-this->m_v3IntersectionHalfSize.x , this->m_v3IntersectionHalfSize.y , this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[2]=D3DXVECTOR3(-this->m_v3IntersectionHalfSize.x ,-this->m_v3IntersectionHalfSize.y , this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[3]=D3DXVECTOR3(-this->m_v3IntersectionHalfSize.x , this->m_v3IntersectionHalfSize.y ,-this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[4]=D3DXVECTOR3(-this->m_v3IntersectionHalfSize.x ,-this->m_v3IntersectionHalfSize.y ,-this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[5]=D3DXVECTOR3( this->m_v3IntersectionHalfSize.x ,-this->m_v3IntersectionHalfSize.y , this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[6]=D3DXVECTOR3( this->m_v3IntersectionHalfSize.x , this->m_v3IntersectionHalfSize.y ,-this->m_v3IntersectionHalfSize.z);
	v3ControlPoints[7]=D3DXVECTOR3( this->m_v3IntersectionHalfSize.x ,-this->m_v3IntersectionHalfSize.y ,-this->m_v3IntersectionHalfSize.z);

	// now compute bounding box of those control points
	for (unsigned int n=0;n<8;++n)
	{
		// transform vertex
		D3DXVec3TransformCoord(&v3ControlPoints[n],&v3ControlPoints[n],&matShapePose);

		// update bounding box
		if (n==0)
		{
			// initialize bounding box
			this->m_BoundingBox.vMin.x=v3ControlPoints[n].x;
			this->m_BoundingBox.vMin.y=v3ControlPoints[n].y;
			this->m_BoundingBox.vMin.z=v3ControlPoints[n].z;
			this->m_BoundingBox.vMax.x=v3ControlPoints[n].x;
			this->m_BoundingBox.vMax.y=v3ControlPoints[n].y;
			this->m_BoundingBox.vMax.z=v3ControlPoints[n].z;
		}
		else
		{
			// update bounding box
			this->m_BoundingBox.vMin.x=minimum(this->m_BoundingBox.vMin.x,v3ControlPoints[n].x);
			this->m_BoundingBox.vMin.y=minimum(this->m_BoundingBox.vMin.y,v3ControlPoints[n].y);
			this->m_BoundingBox.vMin.z=minimum(this->m_BoundingBox.vMin.z,v3ControlPoints[n].z);
			this->m_BoundingBox.vMax.x=maximum(this->m_BoundingBox.vMax.x,v3ControlPoints[n].x);
			this->m_BoundingBox.vMax.y=maximum(this->m_BoundingBox.vMax.y,v3ControlPoints[n].y);
			this->m_BoundingBox.vMax.z=maximum(this->m_BoundingBox.vMax.z,v3ControlPoints[n].z);
		}
	}

	// compute inverse transformation matrix for faster inside test.
	D3DXMatrixInverse(&this->m_matIntersectionInversePose,NULL,&matShapePose);
}

bool C3DCollisionBox::Inside(const D3DXVECTOR3& v3Position) const
{
	// transform position into local coordinate system
	D3DXVECTOR3 v3LocalPosition;
	D3DXVec3TransformCoord(&v3LocalPosition,&v3Position,&this->m_matIntersectionInversePose);

	// check local position
	return (fabs(v3LocalPosition.x)<=this->m_v3IntersectionHalfSize.x) &&
		   (fabs(v3LocalPosition.y)<=this->m_v3IntersectionHalfSize.y) &&
		   (fabs(v3LocalPosition.z)<=this->m_v3IntersectionHalfSize.z);
}







// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionConvexHull
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------





C3DCollisionConvexHull::C3DCollisionConvexHull()
{
}

const PnPTC3DVec* C3DCollisionConvexHull::GetVertex(unsigned int nVertex) const
{
	// check vertex index
	if (nVertex<this->m_vecVertices.size())
	{
		return &this->m_vecVertices[nVertex];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

bool C3DCollisionConvexHull::AddVertex(const PnPTC3DVec& v3Add)
{
	// add vertex to list
	this->m_vecVertices.push_back(v3Add);
	return true;
}

void C3DCollisionConvexHull::InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize/*=0*/)
{
	// compute center of convex hull
	D3DXMATRIX matShapePose;
	D3DXMatrixMultiply(&matShapePose,&this->GetLocalTransform(),&matTransform);

	// clear list first
	this->m_vecIntersectionVertices.clear();

	// now fill list with all transformed vertices
	D3DXVECTOR3 v3Centroid(0,0,0);
	for (std::vector<PnPTC3DVec>::iterator itVertex=this->m_vecVertices.begin();itVertex!=this->m_vecVertices.end();++itVertex)
	{
		// add transformed vertex
		D3DXVECTOR3 v3Vertex(itVertex->x,itVertex->y,itVertex->z);
		D3DXVec3TransformCoord(&v3Vertex,&v3Vertex,&matShapePose);
		this->m_vecIntersectionVertices.push_back(v3Vertex);

		// update centroid
		v3Centroid+=v3Vertex;
	}

	// compute centroid
	if (this->m_vecVertices.size()>0)
	{
		v3Centroid.x/=static_cast<float>(this->m_vecVertices.size());
		v3Centroid.y/=static_cast<float>(this->m_vecVertices.size());
		v3Centroid.z/=static_cast<float>(this->m_vecVertices.size());
	}

	// resize convex hull and compute bounding box
	for (std::vector<D3DXVECTOR3>::iterator itVertex=this->m_vecIntersectionVertices.begin();itVertex!=this->m_vecIntersectionVertices.end();++itVertex)
	{
		// compute vector from centroid to vertex
		D3DXVECTOR3 v3Centroid2Vertex=(*itVertex)-v3Centroid;
		
		// compute scale factor
		float fLength=D3DXVec3Length(&v3Centroid2Vertex);
		if (fLength!=0)
		{
			v3Centroid2Vertex*=(fLength+fIncreaseSize)/fLength;
		}

		// set new vertex position
		*itVertex=v3Centroid2Vertex+v3Centroid;

		// update bounding box
		if (itVertex==this->m_vecIntersectionVertices.begin())
		{
			// initialize bounding box
			this->m_BoundingBox.vMin.x=itVertex->x;
			this->m_BoundingBox.vMin.y=itVertex->y;
			this->m_BoundingBox.vMin.z=itVertex->z;
			this->m_BoundingBox.vMax.x=itVertex->x;
			this->m_BoundingBox.vMax.y=itVertex->y;
			this->m_BoundingBox.vMax.z=itVertex->z;
		}
		else
		{
			// update bounding box
			this->m_BoundingBox.vMin.x=minimum(this->m_BoundingBox.vMin.x,itVertex->x);
			this->m_BoundingBox.vMin.y=minimum(this->m_BoundingBox.vMin.y,itVertex->y);
			this->m_BoundingBox.vMin.z=minimum(this->m_BoundingBox.vMin.z,itVertex->z);
			this->m_BoundingBox.vMax.x=maximum(this->m_BoundingBox.vMax.x,itVertex->x);
			this->m_BoundingBox.vMax.y=maximum(this->m_BoundingBox.vMax.y,itVertex->y);
			this->m_BoundingBox.vMax.z=maximum(this->m_BoundingBox.vMax.z,itVertex->z);
		}
	}
}






// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DCollisionGeometry
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------





C3DCollisionGeometry::C3DCollisionGeometry(C3DObjectData* pObj)
	: m_pObject(pObj)
	, m_bDynamic(true)
//	, m_pMesh(NULL)
	, m_bCollidable(true)
	, m_fMass(0)
{
	D3DXMatrixIdentity(&this->m_matLocalTransform);
}

C3DCollisionGeometry::~C3DCollisionGeometry(void)
{
	// delete all collision shapes
	for (unsigned int nShape=0;nShape<static_cast<unsigned int>(this->m_vecShapes.size());++nShape)
	{
		if (this->m_vecShapes[nShape])
		{
			delete this->m_vecShapes[nShape];
		}
	}
	this->m_vecShapes.clear();
}

bool C3DCollisionGeometry::AddShape(C3DCollisionShape* pShape)
{
	// add shape to vector
	this->m_vecShapes.push_back(pShape);
	return true;
}

C3DCollisionShape* C3DCollisionGeometry::GetShape(unsigned int nShapeIndex) const
{
	// check index
	if (nShapeIndex<this->m_vecShapes.size())
	{
		return this->m_vecShapes[nShapeIndex];
	}
	else
	{
		// invalid index
		return NULL;
	}
}





// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// C3DJoint
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------



C3DJoint::C3DJoint()
	: m_fMaxForce(-1)
	, m_fMaxTorque(-1)
{
}

C3DJoint::~C3DJoint()
{
}

void C3DJoint::AddCollisionGeometry(const C3DCollisionGeometry* pGeometry,const D3DXVECTOR3& v3Position)
{
	// add it to lists
	this->m_vecAttachments.push_back(v3Position);
	this->m_vecGeometries.push_back(pGeometry);
}

size_t C3DJoint::GetNumCollisionGeometries() const
{
	// return number of geometries
	return maximum(this->m_vecAttachments.size(),this->m_vecGeometries.size());
}

const C3DCollisionGeometry* C3DJoint::GetCollisionGeometry(unsigned int nGeometry) const
{
	// check index
	if (nGeometry<this->GetNumCollisionGeometries())
	{
		// return collision geometry
		return this->m_vecGeometries[nGeometry];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

D3DXVECTOR3 C3DJoint::GetAttachmentPosition(unsigned int nGeometry) const
{
	// check index
	if (nGeometry<this->GetNumCollisionGeometries())
	{
		// return attachment position
		return this->m_vecAttachments[nGeometry];
	}
	else
	{
		// invalid index
		return D3DXVECTOR3(0,0,0);
	}
}

void C3DJoint::SetAttachmentPosition(unsigned int nGeometry,const D3DXVECTOR3& v3AttachmentPosition)
{
	// check index
	if (nGeometry<this->GetNumCollisionGeometries())
	{
		// set the attachment position
		this->m_vecAttachments[nGeometry]=v3AttachmentPosition;
	}
}






// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// PNPTC_MATERIAL
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------









PNPTC_MATERIAL::PNPTC_MATERIAL()
	: iTextures(0)
	, bUseVertexColors(false)
	, bAlphaBlend(false)
{
	colDiffuse.r=colDiffuse.g=colDiffuse.b=colDiffuse.a=255;
	colAmbient.r=colAmbient.g=colAmbient.b=colAmbient.a=255;
}








// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------
// PNPTC_AABBOX
// -----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------






bool PNPTC_AABBOX::Inside(const PnPTC3DVec& vTest) const
{
	if (vTest.x<this->vMin.x || vTest.y<this->vMin.x || vTest.z<this->vMin.z ||
		vTest.x>this->vMax.x || vTest.y>this->vMax.x || vTest.z>this->vMax.z)
	{
		// outside the box
		return false;
	}
	else
	{
		// inside the box
		return true;
	}
}

