/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectData.h
// Data structures required for 3d-object/mesh data.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	31. Dec. 2005 : Added GetMaterialIndex method.
//  24. Sep. 2006 : Added methods to set vertex and face data, added 3ds loader.
//  02. Oct. 2006 : Added bounding box methods, ComputeNormals and ReCreateMeshes method.
//  05. Oct. 2006 : Added user data.
//  07. Oct. 2006 : Added C3DMesh::SetHasVertexNormals and C3DMesh::SetHasVertexColors.
//  10. Oct. 2006 : Achieved VS2005 compatibility, made compilation easier.
//  04. Jun. 2007 : Added local mesh transformation.
//  01. Jul. 2007 : Added mesh instancing.
//  22. Jul. 2007 : Added collision geometries.
//  22. Sep. 2007 : Added reference frames.
//  12. Mai  2008 : Normal (re)computation can now be limited to a region.
//  11. Aug. 2008 : Added precise hit test.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECTDATA_H_INCLUDED)
#define PNPTC_3DOBJECTDATA_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <string>
#include <vector>
#include "../../Includes/IPnPTCPlugIn.h"
#include <d3dx9.h>
#include "..\PnPTCUtils\pnpse\pnpseBoundingBox.h"
#include "..\PnPTCUtils\pnpse\pnpseRay.h"


/// material data
typedef struct	PNPTC_MATERIAL
{
	// members
	std::string	strName;			//!< material name
	std::string	strTexture[3];		//!< name of up to three texture files
	PNPTC_RGBA	colDiffuse;			//!< diffuse color of material
	PNPTC_RGBA	colAmbient;			//!< ambient color of material
	int			iTextures;			//!< number of textures used
	bool		bUseVertexColors;	//!< whether or not vertex colors should be used instead of material diffuse/ambient color
	bool		bAlphaBlend;		//!< whether or not to activate alpha blending

	// methods
	PNPTC_MATERIAL();				//!< construction of a new material

} PNPTC_MATERIAL;

/// vertex data
typedef struct PNPTC_VERTEX
{
	PnPTC3DVec	vPos;				//!< position vector of vertex
	PNPTC_RGBA	colDiffuse;			//!< diffuse vertex color
	PnPTC3DVec	vNormal;			//!< normal vector of vertex
	PnPTCUV		uv[3];				//!< up to three uv coordinates
} PNPTC_VERTEX;

/// face (triangle) data
typedef struct PNPTC_FACE
{
	DWORD		dwVert[3];			//!< the three vertices of a face.
} PNPTC_FACE;

/// 3D axis aligned bounding box
typedef struct PNPTC_AABBOX
{
	PnPTC3DVec	vMin;				//!< minimum of bounding box
	PnPTC3DVec	vMax;				//!< maximum of bounding box

	/// Tests, whether or not a given vertex
	/// is within the bounding box region.
	/// @param vTest The vertex to be tested.
	/// @return Returns <i>true</i> if the vertex
	/// is within the bounding box region (including
	/// the bounding box border), or <i>false</i>
	/// is it is outside.
	bool		Inside(const PnPTC3DVec& vTest) const;

} PNPTC_AABBOX;

/// A simple binary tree data structure to
/// store a hull body hierarchy for faster raycast.
template <class T>
class HullHierarchyNode
{
public:

	// ---------------------------------------------
	// construction / destruction
	// ---------------------------------------------

	/// Constructs a new default material definition.
	HullHierarchyNode()
		: m_pLeftChild(NULL)
		, m_pRightChild(NULL)
	{};

	~HullHierarchyNode()
	{
		SAFE_DELETE(this->m_pLeftChild);
		SAFE_DELETE(this->m_pRightChild);
	};


	// ---------------------------------------------
	// public methods
	// ---------------------------------------------

	/// Tests whether or not a ray hits the node.
	/// @param rayTest Ray to be tested for hitting the node.
	/// @return Returns <i>true</i> if the ray hits the node,
	/// otherwise <i>false</i>.
	bool				Hit(const pnpse::CRay& rayTest) const {return T.Hit(rayTest);};

	/// Returns the leaf node in the hierarchy, which was hit by a given ray
	/// @param rayTest Ray to be tested for hitting the node.
	/// @param vecHitNodes A list of leaf nodes hit by the ray.
	/// @return Returns <i>true</i> if the ray hits a leaf node,
	/// otherwise <i>false</i>.
	bool				HitHierarchy(const pnpse::CRay& rayTest,std::vector<const HullHierarchyNode<T>*>& vecHitNodes) const
	{
		// test if this node was hit
		if (this->m_Hull.Hit(rayTest))
		{
			// test if it is a leaf node
			if (this->m_pLeftChild && this->m_pRightChild)
			{
				// test both child nodes
				return  this->m_pLeftChild->HitHierarchy(rayTest,vecHitNodes) | 
						this->m_pRightChild->HitHierarchy(rayTest,vecHitNodes);
			}
			else
			{
				// leaf node
				vecHitNodes.push_back(this);
				return true;
			}
		}
		else
		{
			// not hit 
			return false;
		}
	};


	// ---------------------------------------------
	// public member variables
	// ---------------------------------------------

	/// The left child in the hull hierarchy.
	HullHierarchyNode*	m_pLeftChild;

	/// The right child in the hull hierarchy.
	HullHierarchyNode*	m_pRightChild;

	/// The bounding/hull object of this node.
	T					m_Hull;

	/// The list of all triangles in this node, if it is a leaf
	std::vector<int>	m_vecTriangles;

}; // class HullHierarchyNode




// forward definitions
class C3DFaceSet;
class C3DMesh;
class C3DObjectData;
class C3DCollisionGeometry;


// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// object's face set
class C3DFaceSet
{
	friend class C3DObjectData;

public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a new face set.
	/// @param pMesh Mesh to which the face set belongs.
	/// @param iMaterial Material with which the face set is to be rendered.
	C3DFaceSet(C3DMesh* pMesh,int iMaterial);

	/// Destruction of the face set.
	~C3DFaceSet(void);

	/// Clones this face set and creates a new face set.
	/// @param pMesh The mesh to which the cloned mesh will belong.
	/// @return Returns the newly created/cloned face set if successful, otherwise <i>NULL</i>.
	C3DFaceSet*	Clone(C3DMesh* pMesh) const;


	// ----------------------------------------------------------------
	// creating the faces
	// ----------------------------------------------------------------

	/// Reservers memory for faces.
	/// @param dwNumFaces number of faces for which memory
	/// should be reserved.
	bool				ReserveFaceMem(DWORD dwNumFaces);

	/// Adds one face (triangel) to the mesh's face list
	/// @param face Face to be added.
	/// @return Returns the index of the added face.
	/// Returns <i>-1</i> in case of an error.
	int					AddFace(const PNPTC_FACE& face);


	// ----------------------------------------------------------------
	// face set's material
	// ----------------------------------------------------------------

	/// Sets a new index for the material used by the face set.
	/// @param iMaterial New material index.
	void				SetMaterial(int iMaterial);

	/// Returns the index for the material used by the face set.
	/// @return Material index.
	int					GetMaterial() const;


	// ----------------------------------------------------------------
	// queary and set face information
	// ----------------------------------------------------------------

	/// Returns the number of faces in the face set
	/// @return Number of faces.
	size_t				GetNumFaces() const;

	/// Sets the number of faces in the face set.
	/// @param nFaces Number of faces of the face set.
	bool				SetNumFaces(size_t nFaces);

	/// Returns the data of one face.
	/// @param nFace face index to be returned.
	/// @return Data of the requested face.
	PNPTC_FACE&			GetFace(unsigned int nFace);

	/// Returns the data of one face.
	/// @param nFace face index to be returned.
	/// @return Data of the requested face.
	const PNPTC_FACE&	GetFace(unsigned int nFace) const;

	/// Sets the data of one face.
	/// @param nFace Face index to be set.
	/// @param face Face data to be set.
	void				SetFace(unsigned int nFace,const PNPTC_FACE& face);


	// ----------------------------------------------------------------
	// user data
	// ----------------------------------------------------------------

	/// Get the pointer to user data.
	/// @return Returns the pointer to the user data.
	inline void*		GetUserDataPtr() const {return this->m_pUserData;};

	/// Sets the pointer to the user data.
	/// @param pUserData User data pointer.
	inline void			SetUserDataPtr(void* pUserData) {this->m_pUserData=pUserData;};


	// ----------------------------------------------------------------
	// intersection test methods
	// ----------------------------------------------------------------

	/// Tests intersection with this face set in the face set's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the face set.
	/// @return Returns the <i>true</i> if the face set was intersected
	/// by the ray, otherwise <i>false</i>.
	bool				Intersect(const pnpse::CRay& ray) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// parent mesh of this face set.
	C3DMesh*											m_pMesh;

	/// material index to be used.
	int													m_iMaterial;

	/// list of all faces.
	std::vector<PNPTC_FACE>								m_vecFaces;

	/// Pointer to associated user data.
	void*												m_pUserData;

	/// The hull body hierarchy of this object.
	mutable HullHierarchyNode<pnpse::CBoundingBox>*		m_pHullHierarchy;


	// ----------------------------------------------------------------
	// internal methods
	// ----------------------------------------------------------------

	/// Builds the hull hierarchy for a given node.
	/// @param pNode The node, for which the hierarchy should be build.
	/// @param iLevel The level of the hierarchy.
	/// @param iMinTriangleNumber The minimum triangle number in a leaf node.
	/// @param iMaxLevelCount The maximum number of levels to build for this hierarchy.
	void		BuildHullHierarchy(HullHierarchyNode<pnpse::CBoundingBox>* pNode,int iLevel,int iMinTriangleNumber,int iMaxLevelCount) const;

	/// computes the bounding box of one node of the hull hierarchy.
	/// @param pNode The node, for which the bounding box should be computed.
	void		ComputeHullHierarchyBoundingBox(HullHierarchyNode<pnpse::CBoundingBox>* pNode) const;

};	// class C3DFaceSet

	
// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// object's mesh structure
class C3DMesh
{
	friend class C3DObjectData;

public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a new mesh.
	/// @param pObj The object to which this mesh will belong.
	/// @param bColor Whether or not vertex colors are used for the mesh.
	/// @param bNormal Whether or not the vertex has normals.
	/// @param iUVCoordinates Number of UV coordinates used for the vertices.
	C3DMesh(C3DObjectData* pObj,bool bColor,bool bNormal,int iUVCoordinates);

	/// Destruction of the mesh.
	~C3DMesh(void);

	/// Clones this mesh and creates a new mesh.
	/// @param pObj The object to which the cloned mesh will belong.
	/// @return Returns the newly created/cloned mesh if successful, otherwise <i>NULL</i>.
	C3DMesh*	Clone(C3DObjectData* pObj) const;



	// ----------------------------------------------------------------
	// initializations
	// ----------------------------------------------------------------

	/// Sets the name of the mesh.
	/// @param strName name to be set.
	void					SetName(const std::string& strName);

	/// Returns the name of the mesh.
	/// @return Returns the name of the mesh.
	const std::string		GetName() const {return this->m_strName;};


	// ----------------------------------------------------------------
	// retrieving elements from the mesh
	// ----------------------------------------------------------------

	/// Returns whether or not the mesh has vertex colors.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the mesh has vertex colors.
	bool					HasVertexColor() const;

	/// Returns whether or not the mesh has vertex normals.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the mesh has vertex normals.
	bool					HasVertexNormals() const;

	/// Returns the overall number of faces of all face sets.
	/// @return Overall number of faces.
	size_t					GetNumFaces() const;

	/// Returns the overall number of vertices.
	/// @return Overall number of vertices.
	size_t					GetNumVertices() const;

	/// Returns the number of mapping coords (textures used).
	/// @return number of mapping coords.
	size_t					GetNumMappingCoords() const;

	/// Returns one vertex.
	/// @param nVertex vertex index to be returned [0..GetNumVertices()-1].
	/// @return the data of the requested vertex.
	PNPTC_VERTEX&			GetVertex(unsigned int nVertex);

	/// Returns one vertex.
	/// @param nVertex vertex index to be returned [0..GetNumVertices()-1].
	/// @return the data of the requested vertex.
	const PNPTC_VERTEX&		GetVertex(unsigned int nVertex) const;

	/// Returns the number of face sets.
	/// @return Number of face sets.
	size_t					GetNumFaceSets() const;

	/// Returns one face set
	/// @param nFaceSet index of the face set to be returned.
	/// @return Returns a pointer to the requested face set
	/// or <i>NULL</i> in case of an error.
	C3DFaceSet*				GetFaceSet(unsigned int nFaceSet);

	/// Returns one face set
	/// @param nFaceSet index of the face set to be returned.
	/// @return Returns a pointer to the requested face set
	/// or <i>NULL</i> in case of an error.
	const C3DFaceSet*		GetFaceSet(unsigned int nFaceSet) const;


	// ----------------------------------------------------------------
	// creating the mesh structure
	// ----------------------------------------------------------------

	/// Reservers memory for vertices.
	/// @param dwNumVertices number of vertices for which memory
	/// should be reserved.
	bool					ReserveVertexMem(DWORD dwNumVertices);

	/// Sets the number of vertices.
	/// @param nVertices Number of vertices.
	bool					SetNumVertices(size_t nVertices);

	/// Adds a new vertex to the object.
	/// @param vert vertex to be added.
	/// @return returns the index of this vertex
	int						AddVertex(const PNPTC_VERTEX& vert);

	/// Adds a face set to the mesh.
	/// @param faceset face set to be added.
	/// @return returns the index of this face set
	int						AddFaceSet(C3DFaceSet* faceset);

	/// Sets the position value of one vertex.
	/// @param dwVertex Vertex index to be set.
	/// @param fPos Position value to be set.
	void					SetVertexPos(DWORD dwVertex,float fPos[3]);

	/// Sets the normal value of one vertex.
	/// @param dwVertex Vertex index to be set.
	/// @param fNormal Normal value to be set.
	void					SetVertexNormal(DWORD dwVertex,float fNormal[3]);

	/// Sets the color value of one vertex.
	/// @param dwVertex Vertex index to be set.
	/// @param rgbaColor Color value to be set.
	void					SetVertexColor(DWORD dwVertex,const PNPTC_RGBA& rgbaColor);

	/// Sets the texture mapping of one vertex.
	/// @param dwVertex Vertex index to be set.
	/// @param nUVCoordinate UV coordinate pair to be set.
	/// @param fUV UV coordinates to be set.
	void					SetVertexUV(DWORD dwVertex,unsigned int nUVCoordinate,float fUV[2]);

	/// Sets the number of mapping coords (textures used).
	/// @param nMappingCoords number of mapping coords.
	void					SetNumMappingCoords(size_t nMappingCoords);

	/// Sets, whether or not this mesh uses/has vertex normals.
	/// @param bVertexNormals Whether or not vertex normals are set.
	void					SetHasVertexNormals(bool bVertexNormals) {this->m_bNormal=bVertexNormals;};

	/// Sets, whether or not this mesh uses/has vertex colors.
	/// @param bVertexColors Whether or not vertex colors are set.
	void					SetHasVertexColors(bool bVertexColors) {this->m_bColor=bVertexColors;};


	// ----------------------------------------------------------------
	// bounding box and intersection test methods
	// ----------------------------------------------------------------

	/// Computes the mesh's bounding box.
	void					ComputeBoundingBox();

	/// Returns the mesh's bounding box.
	/// @return Bounding box of the mesh.
	const PNPTC_AABBOX&		GetBoundingBox() const {return this->m_BoundingBox;};

	/// Tests intersection with this mesh in the meshs's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the mesh.
	/// @return Returns the <i>true</i> if the mesh was intersected
	/// by the ray, otherwise <i>false</i>.
	bool					Intersect(const pnpse::CRay& ray) const;


	// ----------------------------------------------------------------
	// additional mesh methods
	// ----------------------------------------------------------------

	/// (Re-)compute normals of all meshes.
	/// @param bReCompute <i>true</i> or <i>false</i>, whether or not
	/// the normals should be recomputed if have been already available.
	/// @param pBBoxRegion An optional region to which the normal
	/// computation should be limited.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the normals could be computed. 
	bool					ComputeNormals(bool bReCompute=false,PNPTC_AABBOX* pBBoxRegion=NULL);

	/// Removes unused vertices.
	void					RemoveUnusedVertices();

	/// Combines face sets of the meshes which have the same material.
	void					CombineFaceSets();


	// ----------------------------------------------------------------
	// user data
	// ----------------------------------------------------------------

	/// Get the pointer to user data.
	/// @return Returns the pointer to the user data.
	inline void*			GetUserDataPtr() const {return this->m_pUserData;};

	/// Sets the pointer to the user data.
	/// @param pUserData User data pointer.
	inline void				SetUserDataPtr(void* pUserData) {this->m_pUserData=pUserData;};


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// parent object of this mesh.
	C3DObjectData*				m_pObject;

	/// name of the mesh
	std::string					m_strName;

	/// whether or not to use vertex's diffuse color component.
	bool						m_bColor;

	/// whether or not to use vertex's normal component.
	bool						m_bNormal;

	/// number of uv coordinate pairs used
	int							m_iUVCoordinates;

	/// list of all vertices
	std::vector<PNPTC_VERTEX>	m_vecVertices;

	/// list of all face sets
	std::vector<C3DFaceSet*>	m_vecFaceSets;

	/// Mesh's bounding box.
	PNPTC_AABBOX				m_BoundingBox;

	/// Pointer to associated user data.
	void*						m_pUserData;

};	// class C3DMesh





// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------




/// The instancing of a mesh within the 3D object.
class C3DMeshInstance
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Construction of a new 3D mesh instance.
	C3DMeshInstance(const std::string& strName,C3DMesh* pMesh,const D3DXMATRIX& matLocalTransform);

	/// Destruction of the 3D mesh instance.
	~C3DMeshInstance();


	// ----------------------------------------------------------------
	// mesh instance data
	// ----------------------------------------------------------------

	/// Returns the name of the mesh instance.
	/// @return The mesh instance's name.
	std::string				GetName() const {return this->m_strName;};

	/// Returns the local transformation of the mesh instance.
	/// @return The local mesh instance transformation matrix.
	const D3DXMATRIX&		GetLocalTransform() const {return this->m_matLocalTransform;};

	/// Sets the local transformation of the mesh instance.
	/// @param matLocalTransform The local mesh instance transformation matrix.
	void					SetLocalTransform(const D3DXMATRIX matLocalTransform) {this->m_matLocalTransform=matLocalTransform;};

	/// Returns a pointer to the mesh, which is instantiated by this object.
	/// @return The pointer ot the instantiated mesh.
	const C3DMesh*			GetMesh() const {return this->m_pMesh;};

	/// Sets the collision geometry of this mesh.
	/// @param pCollisionGeometry The collision geometry of this mesh.
	void					SetCollisionGeometry(C3DCollisionGeometry* pCollisionGeometry) {this->m_pCollisionGeometry=pCollisionGeometry;};

	/// Sets the collision geometry of this mesh.
	/// @param pCollisionGeometry The collision geometry of this mesh.
	C3DCollisionGeometry*	GetCollisionGeometry() const {return this->m_pCollisionGeometry;};


	// ----------------------------------------------------------------
	// intersection test methods
	// ----------------------------------------------------------------

	/// Tests intersection with this mesh instance in the object's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the mesh instance. The ray is given in the coordinate space
	/// of the object. The ray will be transformed automatically into
	/// the local coordinate system of the mesh instance for hit test.
	/// @return Returns the <i>true</i> if the mesh instance was intersected
	/// by the ray, otherwise <i>false</i>.
	bool					Intersect(const pnpse::CRay& ray) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The name of the mesh.
	std::string				m_strName;

	/// The instantiated mesh.
	C3DMesh*				m_pMesh;

	/// The local transformation matrix of the mesh.
	D3DXMATRIX				m_matLocalTransform;

	/// The collision geometry of this mesh.
	C3DCollisionGeometry*	m_pCollisionGeometry;

};	// class C3DMeshInstance



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// Collision shape base class
class C3DCollisionShape
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a collision shape.
	C3DCollisionShape();

	/// Destruction of the collision shape.
	virtual ~C3DCollisionShape(void) {};


	// ----------------------------------------------------------------
	// shape transformation
	// ----------------------------------------------------------------

	/// Returns the local transformation of this collision shape.
	/// @return The local transformation of this collision shape.
	D3DXMATRIX		GetLocalTransform() const {return this->m_matLocalTransform;};

	/// Sets the local transformation of this collision shape.
	/// @param matLocalTransform The local transformation of this collision shape.
	void			SetLocalTransform(const D3DXMATRIX& matLocalTransform) {this->m_matLocalTransform=matLocalTransform;};


	// ----------------------------------------------------------------
	// shape parameters
	// ----------------------------------------------------------------

	/// Returns, whether or not the collision shape is collidable.
	/// @return Whether or not the collision shape is collidable.
	bool			GetCollidable() const {return this->m_bCollidable;};

	/// Sets, whether or not the collision shape is collidable.
	/// @param bCollidable Whether or not the collision shape is collidable.
	void			SetCollidable(bool bCollidable) {this->m_bCollidable=bCollidable;};

	/// Returns the shape's collision skin width.
	/// @return The shape's collision skin width.
	float			GetSkinWidth() const {return this->m_fSkinWidth;};

	/// Sets the shape's collision skin width.
	/// @param fSkinWidth The shape's collision skin width.
	void			SetSkinWidth(float fSkinWidth) {this->m_fSkinWidth=fSkinWidth;};


	// ----------------------------------------------------------------
	// intersection tests
	// ----------------------------------------------------------------

	/// Computes the centroid of the intersection of two shapes.
	/// @param Shape2 The second collision shape, with which this one
	/// should be intersected.
	/// @param matTransform1 A transformation of the basis system of this shape.
	/// matTransform1 and matTransform2 should both be given with respect to
	/// the same world coordinate system.
	/// @param matTransform2 A transformation of the basis system of Shape2.
	/// matTransform1 and matTransform2 should both be given with respect to
	/// the same world coordinate system.
	/// @param v3Centroid (output) The centroid of the intersection of both shapes.
	/// This position is given with respect to the world coordinate system, which
	/// is the reference of matTransform1 and matTransform2.
	/// @param fIncreaseSize The amount by which the extend of both collision shapes
	/// should be increased in all direction.
	/// @return Returns <i>true</i> if the two shapes intersect, in which case the
	/// return parameter v3Centroid contains valid data, or <i>false</i> it the
	/// shapes do not intersect.
	bool			GetIntersectionCentroid(C3DCollisionShape& Shape2,const D3DXMATRIX& matTransform1,const D3DXMATRIX& matTransform2,D3DXVECTOR3& v3Centroid,float fIncreaseSize=0);


protected:

	// ----------------------------------------------------------------
	// overridable methods for the intersection test
	// ----------------------------------------------------------------

	/// Initializes all data structures, which are required
	/// for the intersection tests.
	/// @param matTransform The global transformation matrix of
	/// the collision geometry to which this shape belongs.
	/// @param fIncreaseSize The amount, by which the shape
	/// should be increased.
	virtual void			InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize=0)=0;

	/// Returns the globally axis aligned bounding box of the shape.
	/// @return The shape's global axis aligned bounding box.
	virtual PNPTC_AABBOX	GetBoundingBox() const {return this->m_BoundingBox;};

	/// Returns, whether or not a given coordinate is inside the shape.
	/// The default implementation returns just the result of a test
	/// against the shape's global axis aligned bounding box.
	/// @param v3Position The coordinate to be tested.
	/// @return Returns <i>true</i> if the coordinate is inside
	/// the shape, otherwise <i>false</i>.
	virtual bool			Inside(const D3DXVECTOR3& v3Position) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The local transformation matrix of this shape.
	D3DXMATRIX		m_matLocalTransform;

	/// Whether or not the collision shape is collidable.
	bool			m_bCollidable;

	/// The collision skin width.
	float			m_fSkinWidth;

	/// Whether or not the intersection test data was already initialized.
	bool			m_bIntersectionTestDataInitialized;


protected:

	/// The bounding box of this shape.
	PNPTC_AABBOX	m_BoundingBox;

};	// class C3DCollisionShape



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// Collision sphere class
class C3DCollisionSphere: public C3DCollisionShape
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a collision sphere.
	/// @param fRadius The radius of the sphere.
	C3DCollisionSphere(float fRadius)
		: m_fRadius(fRadius)
		, m_fIntersectionSquaredRadius(fRadius*fRadius)
		, m_v3IntersectionCenter(0,0,0)
	{};

	/// Destruction of the collision sphere.
	~C3DCollisionSphere(void) {};


	// ----------------------------------------------------------------
	// shape parameters
	// ----------------------------------------------------------------

	/// Returns the radius of this collision sphere.
	/// @return The radius of this collision sphere.
	float			GetRadius() const {return this->m_fRadius;};

	/// Sets the radius of this collision sphere.
	/// @param fRadius The radius of this collision sphere.
	void			SetRadius(float fRadius) {this->m_fRadius=fRadius;};


protected:

	// ----------------------------------------------------------------
	// overridden methods for the intersection test
	// ----------------------------------------------------------------

	/// Initializes all data structures, which are required
	/// for the intersection tests.
	/// @param matTransform The global transformation matrix of
	/// the collision geometry to which this shape belongs.
	/// @param fIncreaseSize The amount, by which the shape
	/// should be increased.
	virtual void	InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize=0);

	/// Returns, whether or not a given coordinate is inside the shape.
	/// @param v3Position The coordinate to be tested.
	/// @return Returns <i>true</i> if the coordinate is inside
	/// the shape, otherwise <i>false</i>.
	virtual bool	Inside(const D3DXVECTOR3& v3Position) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The radius of this shape.
	float			m_fRadius;


	// ----------------------------------------------------------------
	// member variables for intersection test
	// ----------------------------------------------------------------

	/// The squared radius used during the intersection test.
	float			m_fIntersectionSquaredRadius;

	/// The sphere's center used for the intersection test.
	D3DXVECTOR3		m_v3IntersectionCenter;

};	// class C3DCollisionSphere



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// Collision capsule class
class C3DCollisionCapsule: public C3DCollisionShape
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a collision capsule.
	/// @param fRadius The radius of the capsule.
	/// @param fHeight The height of the capsule.
	C3DCollisionCapsule(float fRadius,float fHeight)
		: m_fRadius(fRadius), m_fHeight(fHeight)
	{};

	/// Destruction of the collision capsule.
	~C3DCollisionCapsule(void) {};


	// ----------------------------------------------------------------
	// shape parameters
	// ----------------------------------------------------------------

	/// Returns the radius of this collision capsule.
	/// @return The radius of this collision capsule.
	float			GetRadius() const {return this->m_fRadius;};

	/// Sets the radius of this collision capsule.
	/// @param fRadius The radius of this collision capsule.
	void			SetRadius(float fRadius) {this->m_fRadius=fRadius;};

	/// Returns the height of this collision capsule.
	/// @return The height of this collision capsule.
	float			GetHeight() const {return this->m_fHeight;};

	/// Sets the height of this collision capsule.
	/// @param fHeight The height of this collision capsule.
	void			SetHeight(float fHeight) {this->m_fHeight=fHeight;};


protected:

	// ----------------------------------------------------------------
	// overridden methods for the intersection test
	// ----------------------------------------------------------------

	/// Initializes all data structures, which are required
	/// for the intersection tests.
	/// @param matTransform The global transformation matrix of
	/// the collision geometry to which this shape belongs.
	/// @param fIncreaseSize The amount, by which the shape
	/// should be increased.
	virtual void	InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize=0);


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The radius of this capsule.
	float			m_fRadius;

	/// The height of this capsule.
	float			m_fHeight;

};	// class C3DCollisionSphere



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// Collision box class
class C3DCollisionBox: public C3DCollisionShape
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a collision box.
	/// @param fSizeX The size of this box in x direction.
	/// @param fSizeY The size of this box in y direction.
	/// @param fSizeZ The size of this box in z direction.
	C3DCollisionBox(float fSizeX,float fSizeY,float fSizeZ);

	/// Destruction of the collision capsule.
	~C3DCollisionBox(void) {};


	// ----------------------------------------------------------------
	// shape parameters
	// ----------------------------------------------------------------

	/// Returns the size of this collision box in x direction.
	/// @return The size of this collision box in x direction.
	float			GetSizeX() const {return this->m_fSizeX;};

	/// Returns the size of this collision box in y direction.
	/// @return The size of this collision box in y direction.
	float			GetSizeY() const {return this->m_fSizeY;};

	/// Returns the size of this collision box in z direction.
	/// @return The size of this collision box in z direction.
	float			GetSizeZ() const {return this->m_fSizeZ;};

	/// Sets the size of this collision box.
	/// @param fSizeX The size of this box in x direction.
	/// @param fSizeY The size of this box in y direction.
	/// @param fSizeZ The size of this box in z direction.
	void			SetSize(float fSizeX,float fSizeY,float fSizeZ) {this->m_fSizeX=fSizeX;this->m_fSizeY=fSizeY;this->m_fSizeZ=fSizeZ;};


protected:

	// ----------------------------------------------------------------
	// overridden methods for the intersection test
	// ----------------------------------------------------------------

	/// Initializes all data structures, which are required
	/// for the intersection tests.
	/// @param matTransform The global transformation matrix of
	/// the collision geometry to which this shape belongs.
	/// @param fIncreaseSize The amount, by which the shape
	/// should be increased.
	virtual void	InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize=0);

	/// Returns, whether or not a given coordinate is inside the shape.
	/// @param v3Position The coordinate to be tested.
	/// @return Returns <i>true</i> if the coordinate is inside
	/// the shape, otherwise <i>false</i>.
	virtual bool	Inside(const D3DXVECTOR3& v3Position) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The size of this box in x direction.
	float			m_fSizeX;

	/// The size of this box in y direction.
	float			m_fSizeY;

	/// The size of this box in z direction.
	float			m_fSizeZ;


	// ----------------------------------------------------------------
	// member variables for intersection test
	// ----------------------------------------------------------------

	/// The inverse transformation matrix of the box.
	D3DXMATRIX		m_matIntersectionInversePose;

	/// The halfe size of the box used for intersection test.
	D3DXVECTOR3		m_v3IntersectionHalfSize;

};	// class C3DCollisionBox



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// Convex collision mesh class
class C3DCollisionConvexHull: public C3DCollisionShape
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a convex collision hull.
	C3DCollisionConvexHull();

	/// Destruction of the convex collision hull.
	~C3DCollisionConvexHull(void) {};


	// ----------------------------------------------------------------
	// shape parameters
	// ----------------------------------------------------------------

	/// Returns the number of vertices belonging to the convex hull.
	/// @return The number of vertices belonging to the convex hull.
	size_t				GetNumVertices() const {return this->m_vecVertices.size();};

	/// Returns one vertex of the convex hull.
	/// @param nVertex Vertex index to be returned in the
	/// range [0..GetNumVertices()-1].
	const PnPTC3DVec*	GetVertex(unsigned int nVertex) const;

	/// Adds one vertex to the convex hull
	/// @param v3Add Vertex to be added.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the vertex could be added.
	bool				AddVertex(const PnPTC3DVec& v3Add);


protected:

	// ----------------------------------------------------------------
	// overridden methods for the intersection test
	// ----------------------------------------------------------------

	/// Initializes all data structures, which are required
	/// for the intersection tests.
	/// @param matTransform The global transformation matrix of
	/// the collision geometry to which this shape belongs.
	/// @param fIncreaseSize The amount, by which the shape
	/// should be increased.
	virtual void		InitializeIntersectionTestData(const D3DXMATRIX& matTransform,float fIncreaseSize=0);


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// List of all vertices belonging to the convex hull
	std::vector<PnPTC3DVec>	m_vecVertices;


	// ----------------------------------------------------------------
	// member variables for intersection test
	// ----------------------------------------------------------------

	/// List of all transformed vertices for the intersection test.
	std::vector<D3DXVECTOR3>	m_vecIntersectionVertices;

};	// class C3DCollisionConvexHull



// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------


/// Collision geometry class
class C3DCollisionGeometry
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a collision geometry.
	/// @param pObj The object to which this collision geometry will belong.
	C3DCollisionGeometry(C3DObjectData* pObj);

	/// Destruction of the collision geometry.
	~C3DCollisionGeometry(void);


	// ----------------------------------------------------------------
	// collision geometry parameters
	// ----------------------------------------------------------------

	/// Sets the name of the mesh.
	/// @param strName name to be set.
	void				SetName(const std::string& strName) {this->m_strName=strName;};

	/// Returns the name of the mesh.
	/// @return Returns the name of the mesh.
	const std::string	GetName() const {return this->m_strName;};

	/// Set the mesh, which is associated with this collision geometry.
	/// @param pMesh The mesh, which is associated with this collision geometry.
//	void				SetMesh(C3DMesh* pMesh) {this->m_pMesh=pMesh;};

	/// Get the mesh, which is associated with this collision geometry.
	/// @return The mesh, which is associated with this collision geometry.
//	C3DMesh*			GetMesh() const {return this->m_pMesh;};

	/// Returns the local transformation of the mesh instance.
	/// @return The local mesh instance transformation matrix.
	const D3DXMATRIX&	GetLocalTransform() const {return this->m_matLocalTransform;};

	/// Sets the local transformation of the mesh instance.
	/// @param matLocalTransform The local mesh instance transformation matrix.
	void				SetLocalTransform(const D3DXMATRIX matLocalTransform) {this->m_matLocalTransform=matLocalTransform;};

	/// Returns, whether or not this is a dynamic geometry.
	/// @return <i>true</i> or <i>false</i>, whether or not this is a dynamic geometry.
	bool				GetDynamic() const {return this->m_bDynamic;};

	/// Sets, whether or not this is a dynamic geometry.
	/// @param bDynamic <i>true</i> or <i>false</i>, whether or not this is a dynamic geometry.
	void				SetDynamic(bool bDynamic) {this->m_bDynamic=bDynamic;};

	/// Returns, whether or not the collision geometry is collidable.
	/// @return Whether or not the collision geometry is collidable.
	bool				GetCollidable() const {return this->m_bCollidable;};

	/// Sets, whether or not the collision geometry is collidable.
	/// @param bCollidable Whether or not the collision geometry is collidable.
	void				SetCollidable(bool bCollidable) {this->m_bCollidable=bCollidable;};

	/// Returns the mass of this geometry.
	/// @return The mass of this geometry.
	float				GetMass() const {return this->m_fMass;};

	/// Sets the mass of this geometry.
	/// @param fMass The mass of this geometry.
	void				SetMass(float fMass) {this->m_fMass=fMass;};


	// ----------------------------------------------------------------
	// collision shapes
	// ----------------------------------------------------------------

	/// Adds a collision shape to the collision geometry.
	/// @param pShape the shape to be added.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the collision shape could be added.
	bool				AddShape(C3DCollisionShape* pShape);

	/// Returns the number of collisions shapes of this geometry.
	/// @return Number of collision shapes of this geometry.
	size_t				GetNumShapes() const {return this->m_vecShapes.size();}; 

	/// Returns on collision shape of this geometry.
	/// @param nShapeIndex The index of the collision shape to be
	/// returned in the range [0..GetNumShapes()-1].
	/// @return Returns a pointer to the collision shape if successful,
	/// otherwise <i>NULL</i> is returned.
	C3DCollisionShape*	GetShape(unsigned int nShapeIndex) const;


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// The object to which this collision geometry belongs.
	C3DObjectData*					m_pObject;

	/// The name of this collision geometry.
	std::string						m_strName;

	/// The mesh, which is associated with this collision geometry.
//	C3DMesh*						m_pMesh;

	/// List of all collision shapes, which build up the collision geometry.
	std::vector<C3DCollisionShape*>	m_vecShapes;

	/// The local transformation matrix of the mesh.
	D3DXMATRIX						m_matLocalTransform;

	/// Whether or not this is a dynamic geometry.
	bool							m_bDynamic;

	/// Whether or not the collision geometry is collidable.
	bool							m_bCollidable;

	/// The mass of the collision geometry.
	float							m_fMass;

};	// class C3DCollisionGeometry




// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// The class for physical joints
class C3DJoint
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Construction of a joint.
	C3DJoint();

	/// Destruction of the joint.
	virtual ~C3DJoint();


	// ----------------------------------------------------------------
	// joint parameters
	// ----------------------------------------------------------------

	/// Sets the name of the mesh.
	/// @param strName name to be set.
	void						SetName(const std::string& strName) {this->m_strName=strName;};

	/// Returns the name of the mesh.
	/// @return Returns the name of the mesh.
	const std::string			GetName() const {return this->m_strName;};

	/// Adds an attached collision geometry.
	/// @param pGeometry The collision geometry to be added.
	/// @param v3Position The local position of the collision
	/// geometry, at which the joint is attached.
	void						AddCollisionGeometry(const C3DCollisionGeometry* pGeometry,const D3DXVECTOR3& v3Position);

	/// Returns the number of collision geometries, which are
	/// attached to this joint.
	/// @return The number of attached collision geometries.
	size_t						GetNumCollisionGeometries() const;

	/// Returns one of the attached geometries.
	/// @param nGeometry The index of the collision geometry to be returned.
	/// @return Returns a pointer to the collision geometry
	const C3DCollisionGeometry*	GetCollisionGeometry(unsigned int nGeometry) const;

	/// Returns the local attachment position of one of the attached geometries.
	/// @param nGeometry The index of the collision geometry to be returned.
	/// @return Returns the position of one the collision geometry
	D3DXVECTOR3					GetAttachmentPosition(unsigned int nGeometry) const;

	/// Sets the local attachment position of one of the attached geometries.
	/// @param nGeometry The index of the collision geometry to be returned.
	/// @param v3AttachmentPosition The position of one the collision geometry
	void						SetAttachmentPosition(unsigned int nGeometry,const D3DXVECTOR3& v3AttachmentPosition);

	/// Sets the maximum force which can be applied to the joint before breaking it.
	/// @param fMaxForce The maximum force which can be applied to the joint.
	/// <i>-1</i> means unbreakable.
	void						SetMaxForce(float fMaxForce) {this->m_fMaxForce=fMaxForce;};

	/// Returns the maximum force which can be applied to the joint before breaking it.
	/// @return The maximum force which can be applied to the joint.
	/// <i>-1</i> means unbreakable.
	float						GetMaxForce() const {return this->m_fMaxForce;};

	/// Sets the maximum torque which can be applied to the joint before breaking it.
	/// @param fMaxTorque The maximum torque which can be applied to the joint.
	/// <i>-1</i> means unbreakable.
	void						SetMaxTorque(float fMaxTorque) {this->m_fMaxTorque=fMaxTorque;};

	/// Returns the maximum torque which can be applied to the joint before breaking it.
	/// @return The maximum torque which can be applied to the joint.
	/// <i>-1</i> means unbreakable.
	float						GetMaxTorque() const {return this->m_fMaxTorque;};


private:

	/// The name of this joint.
	std::string									m_strName;

	/// The geomteries which are connected by that joint.
	std::vector<const C3DCollisionGeometry*>	m_vecGeometries;

	/// The attachment positions.
	std::vector<D3DXVECTOR3>					m_vecAttachments;

	/// The maximum force, which can be applied to the joint before breaking it.
	/// <i>-1</i> means unbreakable.
	float										m_fMaxForce;

	/// The maximum torque, which can be applied to the joint before breaking it.
	/// <i>-1</i> means unbreakable.
	float										m_fMaxTorque;

};	// class C3DJoint


// -------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------



/// object class
class C3DObjectData
{
	friend class C3DMesh;

public:

	// ----------------------------------------------------------------
	// public data type
	// ----------------------------------------------------------------

	/// Definition of a reference frame, within the object
	typedef struct ReferenceFrame
	{
		std::string		strName;		//!< the name of the reference frame.
		D3DXMATRIX		matPos;			//!< the pose of the reference frame.
	} ReferenceFrame;


	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Creates a new object.
	C3DObjectData(void);

	/// Destruction of the object.
	~C3DObjectData(void);


	// ----------------------------------------------------------------
	// adding elements to the object
	// ----------------------------------------------------------------

	/// Adds a new material to the object.
	/// @param mat New material to be added.
	/// @return returns the index of this material
	int							AddMaterial(const PNPTC_MATERIAL& mat);

	/// Adds a mesh to the object.
	/// @param mesh mesh to be added.
	/// @param bAddMeshInstance Whether or not a mesh instance
	/// should be created automatically.
	/// @return returns the index of this mesh
	int							AddMesh(C3DMesh* mesh, bool bAddMeshInstance=true);

	/// Adds an instance to a mesh.
	/// @param pMesh The mesh for which to add an instance.
	/// @param strName The name of the mesh instance.
	/// @param matLocalTransform The local transformation of this mesh instance.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// mesh instance could be added.
	bool						AddMeshInstance(C3DMesh* pMesh,const std::string& strName,const D3DXMATRIX& matLocalTransform);

	/// Adds a collision geometry to the object.
	/// @param pCollisionGeometry The collision geometry to be added.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// collision geometry could be added.
	bool						AddCollisionGeometry(C3DCollisionGeometry* pCollisionGeometry);

	/// Adds one joint to the list.
	/// @param pJoint The joint to be added.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the joint could be added.
	bool						AddJoint(C3DJoint* pJoint);

	/// Adds one reference frame to the object.
	/// @param strName Name of the reference frame.
	/// @param matPose The pose of the reference frame.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the reference frame could be added.
	bool						AddReferenceFrame(const std::string& strName,const D3DXMATRIX& matPose);


	// ----------------------------------------------------------------
	// retrieving elements from the object
	// ----------------------------------------------------------------

	/// Returns the number of meshes stored in the object.
	/// @return Number of stored meshes.
	size_t						GetNumMeshes() const;

	/// Returns one mesh from the object.
	/// @param iMesh index of the mesh to be returned [0..GetNumMeshes()-1].
	/// @return Pointer to the requested mesh or <i>NULL</i> in case
	/// of an error.
	C3DMesh*					GetMesh(int iMesh);

	/// Returns one mesh from the object.
	/// @param iMesh index of the mesh to be returned [0..GetNumMeshes()-1].
	/// @return Pointer to the requested mesh or <i>NULL</i> in case
	/// of an error.
	const C3DMesh*				GetMesh(int iMesh) const;

	/// Returns the number of mesh instances stored in the object.
	/// @return The number of mesh instances stored in the object.
	size_t						GetNumMeshInstances() const;

	/// Returns one mesh instance from the object.
	/// @param iMeshInstance index of the mesh instance to be returned [0..GetNumMeshInstances()-1].
	/// @return Pointer to the requested mesh instance or <i>NULL</i> in case of an error.
	C3DMeshInstance*			GetMeshInstance(int iMeshInstance);

	/// Returns one mesh instance from the object.
	/// @param iMeshInstance index of the mesh instance to be returned [0..GetNumMeshInstances()-1].
	/// @return Pointer to the requested mesh instance or <i>NULL</i> in case of an error.
	const C3DMeshInstance*		GetMeshInstance(int iMeshInstance) const;

	/// Returns the number of collision geometries.
	/// @return The number of collision geometries in this object.
	size_t						GetNumCollisionGeometries() const;

	/// Returns one collision geometry from the object.
	/// @param iCollisionGeometry index of the collision geometry to be returned [0..GetNumCollisionGeometries()-1].
	/// @return Pointer to the requested collision geometry or <i>NULL</i> in case of an error.
	C3DCollisionGeometry*		GetCollisionGeometry(int iCollisionGeometry);

	/// Returns one collision geometry from the object.
	/// @param iCollisionGeometry index of the collision geometry to be returned [0..GetNumCollisionGeometries()-1].
	/// @return Pointer to the requested collision geometry or <i>NULL</i> in case of an error.
	const C3DCollisionGeometry*	GetCollisionGeometry(int iCollisionGeometry) const;

	/// Returns the number of materials used by the object.
	/// @return Number of stored materials.
	size_t						GetNumMaterials() const;

	/// Returns one material from the object.
	/// @param iMaterial index of the material to be returned [0..GetNumMaterials()-1].
	/// @return Pointer to the requested material or <i>NULL</i> in case
	/// of an error.
	PNPTC_MATERIAL*				GetMaterial(int iMaterial);

	/// Returns one material from the object.
	/// @param iMaterial index of the material to be returned [0..GetNumMaterials()-1].
	/// @return Pointer to the requested material or <i>NULL</i> in case
	/// of an error.
	const PNPTC_MATERIAL*		GetMaterial(int iMaterial) const;

	/// Returns the material index for a given material name.
	/// @param strMaterialName Name of the material to be returned.
	/// @return Returns the index of the material with the given
	/// name if successfull, otherwise <i>-1</i>.
	int							GetMaterialIndex(const std::string& strMaterialName) const;

	/// Returns the number of joints.
	/// @return Number of joints.
	size_t						GetNumJoints() const;

	/// Returns one joint.
	/// @param nJoint Index of the joint to be returned.
	/// @return Returns a pointer to the joint if successful, otherwise <i>NULL</i>.
	C3DJoint*					GetJoint(unsigned int nJoint) const;

	/// Gets the version of this object.
	/// This method only returns a valid version string, if
	/// the object serializer for reading the object supports it.
	/// @return The object's version string.
	std::string					GetObjectVersion() const {return this->m_strObjectVersion;};

	/// Sets the version of this object.
	/// @param strObjectVersion The object's version string.
	void						SetObjectVersion(const std::string& strObjectVersion) {this->m_strObjectVersion=strObjectVersion;};

	/// Returns the number of reference frames attached to the object.
	/// @return Number of reference frames.
	size_t						GetNumReferenceFrames() const;

	/// Returns one reference frame.
	/// @param nReferenceFrame Index of the reference frame to be returned.
	/// @return Pointer to the reference frame if successful, other wise <i>NULL</i>.
	const ReferenceFrame*		GetReferenceFrame(unsigned int nReferenceFrame) const;


	// ----------------------------------------------------------------
	// get data from meshes
	// ----------------------------------------------------------------

	/// Returns the overall number of faces of all meshes.
	/// @return Overall number of faces.
	size_t						GetNumFaces() const;

	/// Returns the overall number of vertices.
	/// @return Overall number of vertices.
	size_t						GetNumVertices() const;


	// ----------------------------------------------------------------
	// bounding box and intersection test methods
	// ----------------------------------------------------------------

	/// Computes the mesh's bounding box.
	void						ComputeBoundingBox();

	/// Returns the object's bounding box.
	/// @return Bounding box of the whole object.
	const PNPTC_AABBOX&			GetBoundingBox() const {return this->m_BoundingBox;};

	/// Tests intersection with this object in the object's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the object.
	/// @return Returns the <i>true</i> if the object was intersected
	/// by the ray, otherwise <i>false</i>.
	bool						Intersect(const pnpse::CRay& ray) const;


	// ----------------------------------------------------------------
	// additional mesh methods
	// ----------------------------------------------------------------

	/// Makes sure the meshes and face sets in this
	/// object don't exceed a given number of vertices/faces.
	/// @param nMaxNumVertices Maximum number of vertices in a mesh.
	/// @param nMaxNumFaces Maximum number of faces in a mesh.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be recreated in a way to support the 
	/// vertex and face limits. If the method fails, the state
	/// of the meshes is undefined. So it should be reloaded/
	/// recreated.
	bool						ReCreateMeshes(unsigned int nMaxNumVertices,unsigned int nMaxNumFaces);

	/// (Re-)compute normals of all meshes.
	/// @param bReCompute <i>true</i> or <i>false</i>, whether or not
	/// the normals should be recomputed if they have been available already.
	/// @param pBBoxRegion An optional region to which the normal
	/// computation should be limited.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the normals could be computed. 
	bool						ComputeNormals(bool bReCompute=false,PNPTC_AABBOX* pBBoxRegion=NULL);

	/// Removes unused vertices.
	void						RemoveUnusedVertices();

	/// Combines meshes which have the same parameters.
	void						CombineMeshes();

	/// Combines face sets of the meshes which have the same material.
	void						CombineFaceSets();


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// list of all materials.
	std::vector<PNPTC_MATERIAL>			m_vecMaterials;

	/// list of all meshes.
	std::vector<C3DMesh*>				m_vecMeshes;

	/// list of all mesh instances.
	std::vector<C3DMeshInstance*>		m_vecMeshInstances;

	/// List of all reference frames, attached to the object.
	std::vector<ReferenceFrame>			m_vecReferenceFrames;

	/// Object's bounding box.
	PNPTC_AABBOX						m_BoundingBox;

	/// The collision geometries of this object.
	std::vector<C3DCollisionGeometry*>	m_vecCollisionGeometries;

	/// The joints of this object.
	std::vector<C3DJoint*>				m_vecJoints;

	/// The version of the object data.
	std::string							m_strObjectVersion;

};	// class C3DObjectData


#endif // !defined(PNPTC_3DOBJECTDATA_H_INCLUDED)

