/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectInstance.h
// 3D object instance data.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  04. Oct. 2006 : Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECTINSTANCE_H_INCLUDED)
#define PNPTC_3DOBJECTINSTANCE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include <d3dx9.h>
#include "../PnPTCUtils/pnpse/pnpseBoundingBox.h"

// forward definition
class C3DObject;


/// Class to manage all object instance specific data.
class C3DObjectInstance
{
public:

	// ----------------------------------------------------------------
	// construction / destruction
	// ----------------------------------------------------------------

	/// Constructs a new object instance.
	/// @param pObj 3D object of this instance.
	C3DObjectInstance(C3DObject* pObj);

	/// Destruction of the object instance
	~C3DObjectInstance(void);


	// ----------------------------------------------------------------
	// Accessing instance data
	// ----------------------------------------------------------------

	/// Returns a pointer to the 3D object data object.
	/// @return Pointer to the 3D data object.
	inline C3DObject*			Get3DObject() const {return this->m_pObject;};


	// ----------------------------------------------------------------
	// Rendering ...
	// ----------------------------------------------------------------

	/// Sets the position of this 3d instance.
	/// @param matPosition New position of this 3d object instance
	void						SetPosition(const D3DXMATRIX& matPosition);

	/// Renders this object instance.
	/// @param bAlpha Whether or not this object has alpha.
	/// @param iMaterial The material (sub mesh) which is to be rendered.
	/// <i>-1</i> to render the whole mesh.
	/// @param bSetResources Whether or not the resources of this object
	/// should be set or the current resource settings should be reused.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object instance could be rendered successfully.
	bool						Render(bool bAlpha=false,int iMaterial=-1,bool bSetResources=true) const;

	/// Returns the bounding box of this object instance
	/// @return Returns the bounding box of the object instance
	/// at its current position in the 3D world.
	const pnpse::CBoundingBox&	GetBoundingBox() const {return this->m_BoundingBox;};

	/// Intersection test between the object instance and a given ray.
	/// @param vecStart Start point of the ray to be tested against
	/// intersection with this object, given in world coordinate space.
	/// @param vecDir Direction vector of the ray to be tested against
	/// intersection with this object, given in world coordinate space.
	/// @return Returns <i>true</i> if the ray hits the object, otherwise <i>false</i>.
	bool						Intersect(const D3DXVECTOR3& vecStart,const D3DXVECTOR3& vecDir);


private:

	// ----------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------

	/// World matrix of this object instance.
	D3DXMATRIX				m_matWorld;

	/// Inverse of the world matrix of this object instance.
	D3DXMATRIX				m_matInvWorld;

	/// Pointer to the 3D object data of this 3d instance.
	C3DObject*				m_pObject;

	/// Bounding box of this instance.
	pnpse::CBoundingBox		m_BoundingBox;

};	// class C3DObjectInstance

#endif // !defined(PNPTC_3DOBJECTINSTANCE_H_INCLUDED)