/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectRenderer.cpp
// Mesh renderer which is based on basic rendering data structure.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  11. Aug. 2008 : Added precise hit test.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include ".\3dobjectrenderer.h"
#include "./I3DObjectSerializer.h"
#include "../pnptcutils/dxutils.h"

C3DObjectRenderer::C3DObjectRenderer(IDirect3DDevice9* pD3DDevice)
	: C3DObjectRendererDX(pD3DDevice)
{
}

C3DObjectRenderer::~C3DObjectRenderer(void)
{
	// free everything
	this->Free();
}

bool C3DObjectRenderer::Load(I3DObjectSerializer* pSerializer,const std::string& strFileName)
{
	// free everything first
	this->Free();

	// check serializer
	if (!pSerializer)
	{
		return false;
	}

	// extract path name
	std::string strTexturePath=GetPath(strFileName);

	// Let the serializer load everything into the data object
	if (pSerializer->Load(this->m_ObjectData,strFileName))
	{
		// compute bounding box of object
		this->m_ObjectData.ComputeBoundingBox();
		const PNPTC_AABBOX& bbox=this->m_ObjectData.GetBoundingBox();
		this->m_BoundingBox.SetBorderPoints(bbox.vMin.x,bbox.vMin.y,bbox.vMin.z,bbox.vMax.x,bbox.vMax.y,bbox.vMax.z);

		// initialize materials
		this->m_vecMaterials.resize(this->m_ObjectData.GetNumMaterials(),NULL);

		// create rendering objects for all meshes
		for (unsigned int nMesh=0;nMesh<this->m_ObjectData.GetNumMeshes();nMesh++)
		{
			C3DMesh* pMesh=this->m_ObjectData.GetMesh(nMesh);
			if (pMesh)
			{
				// make sure, mesh has normals
				if (!pMesh->HasVertexNormals())
				{
					pMesh->ComputeNormals();
				}

				// create vertex buffer information of that mesh
				VERTEXBUFFER* pVertexBuffer=new VERTEXBUFFER;
				if (!pVertexBuffer) return false;

				// compute size of vertex and vertex format
				pVertexBuffer->dwVertexSize=sizeof(float)*3;
				pVertexBuffer->dwFVF=D3DFVF_XYZ;
				if (pMesh->HasVertexNormals())
				{
					pVertexBuffer->dwVertexSize+=sizeof(float)*3;
					pVertexBuffer->dwFVF+=D3DFVF_NORMAL;
				}
				if (pMesh->HasVertexColor())
				{
					pVertexBuffer->dwVertexSize+=sizeof(D3DCOLOR);
					pVertexBuffer->dwFVF+=D3DFVF_DIFFUSE;
				}
				pVertexBuffer->dwVertexSize+=static_cast<DWORD>(pMesh->GetNumMappingCoords())*sizeof(float)*2;
				pVertexBuffer->dwFVF+=D3DFVF_TEX1*static_cast<DWORD>(pMesh->GetNumMappingCoords());
				for (unsigned int nUVSet=0;nUVSet<pMesh->GetNumMappingCoords();nUVSet++)
				{
					pVertexBuffer->dwFVF+=D3DFVF_TEXCOORDSIZE2(nUVSet);
				}

				// create vertex buffer of that mesh
				if (SUCCEEDED(this->m_pD3DDevice->CreateVertexBuffer(static_cast<DWORD>(pMesh->GetNumVertices())*pVertexBuffer->dwVertexSize, 
																	 D3DUSAGE_WRITEONLY,
																	 pVertexBuffer->dwFVF, 
																	 D3DPOOL_MANAGED,
																	 &pVertexBuffer->pVertexBuffer,
																	 NULL))) 
				{
					// write vertex buffer
					void* pVerts;
					if (SUCCEEDED(pVertexBuffer->pVertexBuffer->Lock(0, 0, &pVerts, 0))) 
					{
						// write all vertices
						for (unsigned int nVertex=0,nOffset=0;nVertex<pMesh->GetNumVertices();nVertex++)
						{
							// get vertex
							const PNPTC_VERTEX& vert=pMesh->GetVertex(nVertex);

							// write position data
							float* pfPosData=reinterpret_cast<float*>(static_cast<BYTE*>(pVerts)+nOffset);
							pfPosData[0]=vert.vPos.x;
							pfPosData[1]=vert.vPos.y;
							pfPosData[2]=vert.vPos.z;
							nOffset+=sizeof(float)*3;

							// write normal data
							if (pMesh->HasVertexNormals())
							{
								float* pfNormalData=reinterpret_cast<float*>(static_cast<BYTE*>(pVerts)+nOffset);
								pfNormalData[0]=vert.vNormal.x;
								pfNormalData[1]=vert.vNormal.y;
								pfNormalData[2]=vert.vNormal.z;
								nOffset+=sizeof(float)*3;
							}

							// write color data
							if (pMesh->HasVertexColor())
							{
								D3DCOLOR* pColorData=reinterpret_cast<D3DCOLOR*>(static_cast<BYTE*>(pVerts)+nOffset);
								pColorData[0]=TCColor2DXColor(vert.colDiffuse);
								nOffset+=sizeof(D3DCOLOR);
							}

							// write UV coordinates
							for (unsigned int nUVSet=0;nUVSet<pMesh->GetNumMappingCoords();nUVSet++)
							{
								float* pfUVData=reinterpret_cast<float*>(static_cast<BYTE*>(pVerts)+nOffset);
								pfUVData[0]=vert.uv[nUVSet].u;
								pfUVData[1]=vert.uv[nUVSet].v;
								nOffset+=sizeof(float)*2;
							}
						}

						// unlock and add it
						pVertexBuffer->pVertexBuffer->Unlock();
						pMesh->SetUserDataPtr(pVertexBuffer);
					}
					else
					{
						// error locking vertex buffer
						SAFE_DELETE(pVertexBuffer);
						return false;
					}
				}
				else
				{
					// error creating vertex buffer
					SAFE_DELETE(pVertexBuffer);
					return false;
				}


				// go through all face sets
				for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
				{
					C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
					if (pFaceSet && pFaceSet->GetNumFaces()>0)
					{
						// create material
						int iMaterial=pFaceSet->GetMaterial();
						if (iMaterial>=0 && iMaterial<static_cast<int>(this->m_vecMaterials.size()))
						{
							const PNPTC_MATERIAL* pMatRead=this->m_ObjectData.GetMaterial(iMaterial);
							if (pMatRead)
							{
								CMaterialDef* pMaterial=new CMaterialDef(*pMatRead,this->m_pD3DDevice,strTexturePath);
								this->m_vecMaterials[iMaterial]=pMaterial;
							}
						}

						// get required format
						D3DFORMAT fmtIndexFormat=(pFaceSet->GetNumFaces()<65535)?D3DFMT_INDEX16:D3DFMT_INDEX32;
						DWORD dwIndexSize=(fmtIndexFormat==D3DFMT_INDEX16)?2:4;

						// create index buffer of that face set
						LPDIRECT3DINDEXBUFFER9 pIndexBuffer=NULL;
						if (SUCCEEDED(this->m_pD3DDevice->CreateIndexBuffer(static_cast<UINT>(pFaceSet->GetNumFaces())*3*dwIndexSize, 
																			D3DUSAGE_WRITEONLY, 
																			fmtIndexFormat, 
																			D3DPOOL_MANAGED, 
																			&pIndexBuffer,
																			NULL)))
						{
							// fill index buffer
							void* pIndices;
							if (SUCCEEDED(pIndexBuffer->Lock(0, 0, &pIndices, 0))) 
							{
								// copy all faces to index buffer
								for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
								{
									const PNPTC_FACE& face=pFaceSet->GetFace(nFace);
									if (fmtIndexFormat==D3DFMT_INDEX16)
									{
										(static_cast<WORD*>(pIndices)[nFace*3+0])=static_cast<WORD>(face.dwVert[0]);
										(static_cast<WORD*>(pIndices)[nFace*3+1])=static_cast<WORD>(face.dwVert[1]);
										(static_cast<WORD*>(pIndices)[nFace*3+2])=static_cast<WORD>(face.dwVert[2]);
									}
									else	// fmtIndexFormat==D3DFMT_INDEX32
									{
										(static_cast<DWORD*>(pIndices)[nFace*3+0])=face.dwVert[0];
										(static_cast<DWORD*>(pIndices)[nFace*3+1])=face.dwVert[1];
										(static_cast<DWORD*>(pIndices)[nFace*3+2])=face.dwVert[2];
									}
								}

								// unlock index buffer
								pIndexBuffer->Unlock();
							}
							else
							{
								// Error locking index buffer
								SAFE_RELEASE(pIndexBuffer);
								return false;
							}

							// set index buffer as user data of face set
							pFaceSet->SetUserDataPtr(pIndexBuffer);
						}
						else
						{
							// Error creating index buffer
							return false;
						}
					}
				}
			}
		}

		// store object information
		this->m_iNumFaces=static_cast<int>(this->m_ObjectData.GetNumFaces());
		this->m_iNumVertices=static_cast<int>(this->m_ObjectData.GetNumVertices());

		// ok
		return true;
	}
	else
	{
		// error loading
		return false;
	}
}

void C3DObjectRenderer::Free()
{
	// go through all meshes
	for (unsigned int nMesh=0;nMesh<this->m_ObjectData.GetNumMeshes();nMesh++)
	{
		C3DMesh* pMesh=this->m_ObjectData.GetMesh(nMesh);
		if (pMesh)
		{
			// delete vertex buffer of that mesh
			VERTEXBUFFER* pVertexBuffer=static_cast<VERTEXBUFFER*>(pMesh->GetUserDataPtr());
			if (pVertexBuffer)
			{
				SAFE_RELEASE(pVertexBuffer->pVertexBuffer);
			}
			SAFE_DELETE(pVertexBuffer);
			pMesh->SetUserDataPtr(NULL);

			// go through all face sets
			for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
			{
				C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet)
				{
					// delete index buffer of that face set
					LPDIRECT3DINDEXBUFFER9 pIndexBuffer=static_cast<LPDIRECT3DINDEXBUFFER9>(pFaceSet->GetUserDataPtr());
					SAFE_RELEASE(pIndexBuffer);
					pFaceSet->SetUserDataPtr(NULL);
				}
			}
		}
	}

	// free all materials
	for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
	{
		// delete material
		SAFE_DELETE(this->m_vecMaterials[nMaterial]);
	}
	this->m_vecMaterials.clear();
}

bool C3DObjectRenderer::Save(I3DObjectSerializer* pSerializer,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/)
{
	// check parameters
	if (!pSerializer)
	{
		MessageBox(NULL,(std::string("Error saving object to file'")+strFileName+"':\n\nInvalid serializer!").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// add file name to all materials
	std::vector<std::string> vecMaterialNames;
	std::string strFileBaseName;
	std::string strExtension;
	RemoveFileExtension(strFileName,strFileBaseName,strExtension);
	for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
	{
		// store name and add object name
		PNPTC_MATERIAL* pMaterial=this->m_ObjectData.GetMaterial(nMaterial);
		if (pMaterial)
		{
			vecMaterialNames.push_back(pMaterial->strName);
			pMaterial->strName=strFileBaseName+"_"+pMaterial->strName;
		}
		else
		{
			vecMaterialNames.push_back("");
		}
	}

	// save to file
	bool bReturn=pSerializer->Save(this->m_ObjectData,strFileName,fScale,iUpRotation,iFlipAxis,cBasePathName);

	// restore names
	for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
	{
		// restore name
		PNPTC_MATERIAL* pMaterial=this->m_ObjectData.GetMaterial(nMaterial);
		if (pMaterial)
		{
			pMaterial->strName=vecMaterialNames[nMaterial];
		}
	}

	// export textures
	if (bReturn)
	{
		// get textures directory
		std::string strTextureFolder=GetPath(strFileName);

		// save all textures
		for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
		{
			const PNPTC_MATERIAL* pMat=this->m_ObjectData.GetMaterial(nMaterial);
			if (pMat && this->m_vecMaterials[nMaterial] && this->m_vecMaterials[nMaterial]->GetTexture(0) && !pMat->strTexture[0].empty())
			{
				// save this texture
				if (FAILED(D3DXSaveTextureToFile((strTextureFolder+pMat->strTexture[0]).c_str(),GetImageFormat(pMat->strTexture[0]),this->m_vecMaterials[nMaterial]->GetTexture(0),NULL)))
				{
					MessageBox(NULL,(std::string("Error saving object to file'")+strFileName+"':\n\nError saving texture to file '"+strTextureFolder+pMat->strTexture[0]+"'!").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					bReturn=false;
				}
			}
		}
	}

	// return result
	return bReturn;
}

bool C3DObjectRenderer::Render(const D3DXMATRIX& matPosition,bool bAlpha/*=false*/,int iMaterial/*=-1*/,bool bSetResources/*=true*/) const
{
	// go through all meshes
	for (unsigned int nMesh=0;nMesh<this->m_ObjectData.GetNumMeshInstances();nMesh++)
	{
		const C3DMeshInstance* pMeshInstance=this->m_ObjectData.GetMeshInstance(nMesh);
		if (pMeshInstance)
		{
			const C3DMesh* pMesh=pMeshInstance->GetMesh();
			if (pMesh)
			{
				// compute and set world matrix
				D3DXMATRIX matRender;
				D3DXMatrixMultiply(&matRender,&pMeshInstance->GetLocalTransform(),&matPosition);
				this->m_pD3DDevice->SetTransform(D3DTS_WORLD, &matRender);

				// activate vertex buffer of that mesh
				VERTEXBUFFER* pVertexBuffer=static_cast<VERTEXBUFFER*>(pMesh->GetUserDataPtr());
				if (pVertexBuffer && pVertexBuffer->pVertexBuffer)
				{
					// set rendering parameters
					this->m_pD3DDevice->SetFVF(pVertexBuffer->dwFVF);
					this->m_pD3DDevice->SetStreamSource(0,pVertexBuffer->pVertexBuffer,0,pVertexBuffer->dwVertexSize);

					// go through all face sets
					for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
					{
						const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
						if (pFaceSet && 
							(iMaterial==-1 || iMaterial==pFaceSet->GetMaterial()))
						{
							// activate material
							if (bSetResources)
							{
								int iCurMaterial=pFaceSet->GetMaterial();
								if (iCurMaterial>=0 && iCurMaterial<static_cast<int>(this->m_vecMaterials.size()))
								{
									CMaterialDef* pMaterial=this->m_vecMaterials[iCurMaterial];
									if (pMaterial)
									{
										if (!pMaterial->Activate(this->m_pD3DDevice))
										{
											// error activating material
											return false;
										}
									}
									else
									{
										// activate default material
										this->m_DefaultMaterial.SetUseVertexColors(pMesh->HasVertexColor());
										this->m_DefaultMaterial.Activate(this->m_pD3DDevice);
									}
								}
								else
								{
									// activate default material
									this->m_DefaultMaterial.SetUseVertexColors(pMesh->HasVertexColor());
									this->m_DefaultMaterial.Activate(this->m_pD3DDevice);
								}
							}

							// render index buffer of that face set
							LPDIRECT3DINDEXBUFFER9 pIndexBuffer=static_cast<LPDIRECT3DINDEXBUFFER9>(pFaceSet->GetUserDataPtr());
							if (pIndexBuffer)
							{
								this->m_pD3DDevice->SetIndices(pIndexBuffer);
								if (FAILED(this->m_pD3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 
																					0,
																					0, 
																					static_cast<UINT>(pMesh->GetNumVertices()),
																					0, 
																					static_cast<UINT>(pFaceSet->GetNumFaces()))))
								{
									return false;
								}
							}
						}
					}
				}
				else
				{
					// no vertex buffer
					return false;
				}
			}
		}
	}

	// ok
	return true;
}

bool C3DObjectRenderer::Intersect(const pnpse::CRay& ray) const
{
	// use object data intersection methods
	return this->m_ObjectData.Intersect(ray);
}








// --------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------
// C3DObjectRenderer::CMaterialDef
// --------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------





C3DObjectRenderer::CMaterialDef::CMaterialDef()
	: m_bVertexColors(false)
	, m_bAlpha(false)
{
	this->m_DXMaterial.Ambient=D3DXCOLOR(1,1,1,1);
	this->m_DXMaterial.Diffuse=D3DXCOLOR(1,1,1,1);
	this->m_DXMaterial.Emissive=D3DXCOLOR(0,0,0,1);
	this->m_DXMaterial.Power=0.f;
	this->m_DXMaterial.Specular=D3DXCOLOR(0,0,0,1);
}

C3DObjectRenderer::CMaterialDef::CMaterialDef(const PNPTC_MATERIAL& mat,IDirect3DDevice9* pD3DDevice,const std::string& strTexturePath/*=""*/)
	: m_bVertexColors(mat.bUseVertexColors)
	, m_bAlpha(mat.bAlphaBlend)
{
	// fill DX material
	this->m_DXMaterial.Ambient=TCColor2DXColor(mat.colAmbient);
	this->m_DXMaterial.Diffuse=TCColor2DXColor(mat.colDiffuse);
	if (this->m_DXMaterial.Ambient.r==0 && this->m_DXMaterial.Ambient.g==0 && this->m_DXMaterial.Ambient.b==0)
	{
		this->m_DXMaterial.Ambient=this->m_DXMaterial.Diffuse;
	}
	this->m_DXMaterial.Emissive=D3DXCOLOR(0,0,0,1);
	this->m_DXMaterial.Power=0.f;
	this->m_DXMaterial.Specular=D3DXCOLOR(0,0,0,1);

	// create all textures
	this->m_vecTextures.resize(mat.iTextures,NULL);
	for (int iTexture=0;iTexture<mat.iTextures;iTexture++)
	{
		// load texture
		if ( ((this->m_vecTextures[iTexture]=C3DObjectRendererDX::s_TextureManager.CreateTexture(pD3DDevice,RelativeToAbsolutePath(strTexturePath,mat.strTexture[iTexture]))) == NULL) &&
 			 ((this->m_vecTextures[iTexture]=C3DObjectRendererDX::s_TextureManager.CreateTexture(pD3DDevice,mat.strTexture[iTexture])) == NULL) &&
   			 ((this->m_vecTextures[iTexture]=C3DObjectRendererDX::s_TextureManager.CreateTexture(pD3DDevice,RelativeToAbsolutePath(strTexturePath,GetFileName(mat.strTexture[iTexture])))) == NULL) &&
   			 ((this->m_vecTextures[iTexture]=C3DObjectRendererDX::s_TextureManager.CreateTexture(pD3DDevice,GetFileName(mat.strTexture[iTexture]))) == NULL) )
		{
			this->m_vecTextures[iTexture]=NULL;
		}
		if (this->m_vecTextures[iTexture])
		{
			// ensure material color is not black if a texture is applied
			if (this->m_DXMaterial.Ambient.r==0 && this->m_DXMaterial.Ambient.g==0 && this->m_DXMaterial.Ambient.b==0)
			{
				this->m_DXMaterial.Ambient.r=1.f;
				this->m_DXMaterial.Ambient.g=1.f;
				this->m_DXMaterial.Ambient.b=1.f;
			}
			if (this->m_DXMaterial.Diffuse.r==0 && this->m_DXMaterial.Diffuse.g==0 && this->m_DXMaterial.Diffuse.b==0)
			{
				this->m_DXMaterial.Diffuse.r=1.f;
				this->m_DXMaterial.Diffuse.g=1.f;
				this->m_DXMaterial.Diffuse.b=1.f;
			}
		}
	}
}

C3DObjectRenderer::CMaterialDef::~CMaterialDef()
{
	// free all textures
	for (unsigned int nTexture=0;nTexture<this->m_vecTextures.size();nTexture++)
	{
		if (this->m_vecTextures[nTexture])
		{
			C3DObjectRendererDX::s_TextureManager.ReleaseTexture(this->m_vecTextures[nTexture]);
			this->m_vecTextures[nTexture]=NULL;
		}
	}
	this->m_vecTextures.clear();
}

bool C3DObjectRenderer::CMaterialDef::Activate(IDirect3DDevice9* pD3DDevice) const
{
	// set material
	pD3DDevice->SetMaterial(&this->m_DXMaterial);

	// vertex colors
	pD3DDevice->SetRenderState(D3DRS_COLORVERTEX,this->m_bVertexColors);
	if (this->m_bVertexColors)
	{
		pD3DDevice->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE,D3DMCS_COLOR1);
		pD3DDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE,D3DMCS_COLOR1);
	}

	// set all textures, render, and sampler states
	if (this->m_vecTextures.size()==0)
	{
		// no textures
		pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
		pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
		if (this->m_bAlpha)
		{
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
			pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
		}
	}
	else
	{
		// set all textures
		for (unsigned int nTexture=0;nTexture<this->m_vecTextures.size();nTexture++)
		{
			// set texture stage parameters
			pD3DDevice->SetTextureStageState(nTexture, D3DTSS_COLORARG1, D3DTA_CURRENT);
			pD3DDevice->SetTextureStageState(nTexture, D3DTSS_COLORARG2, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(nTexture, D3DTSS_COLOROP, D3DTOP_MODULATE);
			if (this->m_bAlpha)
			{
				pD3DDevice->SetTextureStageState(nTexture, D3DTSS_ALPHAARG1, D3DTA_CURRENT);
				pD3DDevice->SetTextureStageState(nTexture, D3DTSS_ALPHAARG2, D3DTA_TEXTURE);
				pD3DDevice->SetTextureStageState(nTexture, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
			}
			else
			{
				pD3DDevice->SetTextureStageState(nTexture, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			}

			// set texture
			pD3DDevice->SetTexture(nTexture,this->m_vecTextures[nTexture]);
		}

		// clear last texture stage
		if (this->m_vecTextures.size()<8)
		{
			pD3DDevice->SetTexture(static_cast<DWORD>(this->m_vecTextures.size()),NULL);
			pD3DDevice->SetTextureStageState(static_cast<DWORD>(this->m_vecTextures.size()), D3DTSS_COLOROP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(static_cast<DWORD>(this->m_vecTextures.size()), D3DTSS_ALPHAOP, D3DTOP_DISABLE);
		}
	}

	// ok
	return true;
}

LPDIRECT3DTEXTURE9	C3DObjectRenderer::CMaterialDef::GetTexture(unsigned int nTextureIndex) const
{
	// check index 
	if (nTextureIndex<this->m_vecTextures.size())
	{
		// return texture
		return this->m_vecTextures[nTextureIndex];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

