/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectRenderer.h
// Mesh renderer which is based on basic rendering data structure.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////
#if !defined(PNPTC_3DOBJECTRENDERER_H_INCLUDED)
#define PNPTC_3DOBJECTRENDERER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include "3DObjectRendererDX.h"
#include "3DObjectData.h"
#include <d3dx9.h>


// forward definitions
class I3DObjectSerializer;


/// An object renderer class, which is based on 
/// basic rendering data structure.
class C3DObjectRenderer : public C3DObjectRendererDX
{
public:

	// --------------------------------------------------------------
	// construction / destruction
	// --------------------------------------------------------------

	/// Construction of a new renderer.
	/// @param pD3DDevice The DX rendering device.
	C3DObjectRenderer(IDirect3DDevice9* pD3DDevice);

	/// Destruction of the renderer
	~C3DObjectRenderer(void);


	// --------------------------------------------------------------
	// object loading/saving
	// --------------------------------------------------------------

	/// Loads a mesh file.
	/// @param pSerializer Serializer which should load that object.
	/// @param strFileName Name of the mesh file to be loaded,
	/// including the fully specified path.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the mesh could be loaded successfully.
	bool						Load(I3DObjectSerializer* pSerializer,const std::string& strFileName);

	/// Saves the mesh to a file.
	/// @param pSerializer Serializer which should save that object.
	/// @param strFileName Object file (including path) to be saved to.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be saved successfully.
	bool						Save(I3DObjectSerializer* pSerializer,const std::string& strFileName,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL);

	/// Frees everything
	void						Free();


	// ---------------------------------------------------------------------
	// 3D rendering methods
	// ---------------------------------------------------------------------

	/// Render object at a given position.
	/// @param matWorld World matrix of the object to be rendered.
	/// @param bAlpha Whether or not this object has alpha.
	/// @param iMaterial The material (sub mesh) which is to be rendered.
	/// <i>-1</i> to render the whole mesh.
	/// @param bSetResources Whether or not the resources of this object
	/// should be set or the current resource settings should be reused.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// mesh could be rendered successfully.
	bool						Render(const D3DXMATRIX& matPosition,bool bAlpha=false,int iMaterial=-1,bool bSetResources=true) const;

	/// Tests intersection with this object in the object's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the object.
	/// @return Returns the <i>true</i> if the object was intersected
	/// by the ray, otherwise <i>false</i>.
	bool						Intersect(const pnpse::CRay& ray) const;


	// ---------------------------------------------------------------------
	// Retrieve information about the mesh
	// ---------------------------------------------------------------------

	/// Returns the number of materials used in this object.
	/// @return Number of materials of this object.
	int							GetNumMaterials() const {return static_cast<int>(this->m_vecMaterials.size());};


private:

	// --------------------------------------------------------------
	// internal data types
	// --------------------------------------------------------------

	/// Data structure containing all relevant vertexbuffer
	/// information for rendering.
	typedef struct VERTEXBUFFER
	{
		// ---------------------------------------------
		// members
		DWORD					dwFVF;					//!< vertex format.
		DWORD					dwVertexSize;			//!< size of one vertex in bytes.
		LPDIRECT3DVERTEXBUFFER9	pVertexBuffer;			//!< the DirectX vertex buffer.

		// ---------------------------------------------
		// operations
		VERTEXBUFFER()									//!< construction/initialization
		{
			dwFVF=0;
			dwVertexSize=0;
			pVertexBuffer=NULL;
		};
	} VERTEXBUFFER;

	/// class definition containing all relevant material 
	/// information for rendering
	typedef class CMaterialDef
	{
	public:

		// ---------------------------------------------
		// construction / destruction
		// ---------------------------------------------

		/// Constructs a new default material definition.
		CMaterialDef();

		/// Constructs a new material definition
		/// based on the material data of the
		/// 3d data object.
		/// @param mat Material description as it was read
		/// by the data object.
		/// @param pD3DDevice The rendering device for which
		/// to initialize the material.
		/// @param strTexturePath An additional path in which
		/// to search for texture files.
		CMaterialDef(const PNPTC_MATERIAL& mat,IDirect3DDevice9* pD3DDevice,const std::string& strTexturePath="");

		/// Destruction of the material definition.
		~CMaterialDef();


		// ---------------------------------------------
		// set material parameters
		// ---------------------------------------------

		/// Sets, whether or not vertex colors should be used.
		/// @param bVertexColors Whether or not vertex colors should be used.
		void				SetUseVertexColors(bool bVertexColors) {this->m_bVertexColors=bVertexColors;};


		// ---------------------------------------------
		// rendering methods
		// ---------------------------------------------

		/// Activates this material for rendering
		/// @param pD3DDevice Rendering device for which
		/// to activate this material definition
		/// @return Returns <i>true</i> or <i>false</i>,
		/// whether or not the material could be activated.
		bool				Activate(IDirect3DDevice9* pD3DDevice) const;

		/// Returns a texture object of the material.
		/// @param nTextureIndex Index of the material texture to be returned.
		/// @return Returns a pointer to the texture if successful
		/// or <i>NULL</i> in case of an error.
		LPDIRECT3DTEXTURE9	GetTexture(unsigned int nTextureIndex) const;


	private:

		// ---------------------------------------------
		// member variables
		// ---------------------------------------------

		/// DirectX basic material definition.
		D3DMATERIAL9					m_DXMaterial;

		/// Textures used by this material.
		std::vector<LPDIRECT3DTEXTURE9>	m_vecTextures;

		/// Whether or not vertex colors are used for rendering.
		bool							m_bVertexColors;

		/// Whether or not to use alpha blending
		bool							m_bAlpha;
	};


	// --------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------

	/// Object data.
	C3DObjectData								m_ObjectData;

	/// List of all materials.
	std::vector<CMaterialDef*>					m_vecMaterials;

	/// Default material 
	mutable CMaterialDef						m_DefaultMaterial;

}; // class C3DObjectRenderer


#endif // !defined(PNPTC_3DOBJECTRENDERER_H_INCLUDED)