/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectRendererD3DXMESH.cpp
// Mesh renderer which is based on the DirectX D3DXMESH data structure.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "../pnptcutils/utils.h"
#include "../pnptcutils/DXUtils.h"
#include ".\3dobjectrendererd3dxmesh.h"
#include "../pnptcutils/pnpse/pnpseray.h"
#include "./3DObjectData.h"
#include "./I3DObjectSerializer.h"
#include "../PNPTCUtils/DXUtils.h"


C3DObjectRendererD3DXMESH::C3DObjectRendererD3DXMESH(IDirect3DDevice9* pD3DDevice)
	: C3DObjectRendererDX(pD3DDevice)
	, m_pMesh(NULL)
	, m_pbufAdjacency(NULL)
{
}

C3DObjectRendererD3DXMESH::~C3DObjectRendererD3DXMESH(void)
{
	// release everything
	this->Free();
}

bool C3DObjectRendererD3DXMESH::Load(const std::string& strFileName)
{
	// release everything first
	this->Free();

	// extract path name
	std::string strPath=GetPath(strFileName);

	// variable to hold materials
	LPD3DXBUFFER	pbufMaterials;

	// read mesh file
	DWORD dwNumMaterials=0;
    if (FAILED(D3DXLoadMeshFromX( strFileName.c_str(), 
								  D3DXMESH_MANAGED, 
								  this->m_pD3DDevice, 
								  &this->m_pbufAdjacency, 
								  &pbufMaterials, 
								  NULL,
								  &dwNumMaterials, 
								  &this->m_pMesh ) ) )
	{
		// error reading file
		return false;
	}

	// Load materials
	this->SetNumMaterials(dwNumMaterials);
	if ( pbufMaterials && dwNumMaterials>0 )
	{
		// load textures and materials
		D3DXMATERIAL* d3dxMtrls = (D3DXMATERIAL*)pbufMaterials->GetBufferPointer();
		for( UINT i=0; i<dwNumMaterials; i++ )
		{
			// copy material object
			if (d3dxMtrls[i].MatD3D.Ambient.r==0 && d3dxMtrls[i].MatD3D.Ambient.g==0 && d3dxMtrls[i].MatD3D.Ambient.b==0)
			{
				d3dxMtrls[i].MatD3D.Ambient=d3dxMtrls[i].MatD3D.Diffuse;
			}
			this->m_vecMaterials[i] = d3dxMtrls[i].MatD3D;
			this->m_vecTextures[i].strTexture=(d3dxMtrls[i].pTextureFilename)?d3dxMtrls[i].pTextureFilename:"";
			if (this->m_vecTextures[i].strTexture.length()>0)
			{
				// Load textur
				if ( ((this->m_vecTextures[i].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,RelativeToAbsolutePath(strPath,this->m_vecTextures[i].strTexture))) == NULL) &&
   					 ((this->m_vecTextures[i].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,this->m_vecTextures[i].strTexture)) == NULL) &&
   					 ((this->m_vecTextures[i].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,GetFileName(this->m_vecTextures[i].strTexture))) == NULL) &&
   					 ((this->m_vecTextures[i].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,RelativeToAbsolutePath(strPath,GetFileName(this->m_vecTextures[i].strTexture)))) == NULL) )
				{
					// error loading texture
					this->m_vecTextures[i].pTexture=NULL;
				}
			}
			else
			{
				// no texture
				this->m_vecTextures[i].pTexture=NULL;
			}

			// make sure material color is not black if a texture is applied
			if (this->m_vecTextures[i].pTexture)
			{
				if (this->m_vecMaterials[i].Ambient.r==0 && this->m_vecMaterials[i].Ambient.g==0 && this->m_vecMaterials[i].Ambient.b==0)
				{
					this->m_vecMaterials[i].Ambient.r=1.f;
					this->m_vecMaterials[i].Ambient.g=1.f;
					this->m_vecMaterials[i].Ambient.b=1.f;
				}
				if (this->m_vecMaterials[i].Diffuse.r==0 && this->m_vecMaterials[i].Diffuse.g==0 && this->m_vecMaterials[i].Diffuse.b==0)
				{
					this->m_vecMaterials[i].Diffuse.r=1.f;
					this->m_vecMaterials[i].Diffuse.g=1.f;
					this->m_vecMaterials[i].Diffuse.b=1.f;
				}
			}
		}
	}

	// release material buffer
	SAFE_RELEASE(pbufMaterials);

    // Optimize the mesh for performance
	if (this->m_pbufAdjacency)
	{
		if( FAILED( this->m_pMesh->OptimizeInplace(D3DXMESHOPT_COMPACT | D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE,
													(DWORD*)this->m_pbufAdjacency->GetBufferPointer(), 
													NULL, NULL, NULL ) ) )
		{
			this->Free();
			return false;
		}
	}
	else
	{
		DWORD *pBuffer=new DWORD[this->m_pMesh->GetNumFaces()*sizeof(DWORD)*3];
		if (pBuffer)
		{
			if (SUCCEEDED(this->m_pMesh->GenerateAdjacency(0.01f,pBuffer)))
			{
				if( FAILED( this->m_pMesh->OptimizeInplace(D3DXMESHOPT_COMPACT | D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE,
															pBuffer, 
															NULL, NULL, NULL ) ) )
				{
					this->Free();
					return false;
				}
			}
			SAFE_DELETEARRAY(pBuffer);
		}
	}

	// compute bounding box in object space
	BYTE* pVerts;
	if (SUCCEEDED(this->m_pMesh->LockVertexBuffer(D3DLOCK_READONLY,(LPVOID*)&pVerts)))
	{
		// let dx compute the bounding box
		D3DXVECTOR3 vecMin;
		D3DXVECTOR3 vecMax;
		D3DXComputeBoundingBox((D3DXVECTOR3*)&pVerts[0],this->m_pMesh->GetNumVertices(),this->m_pMesh->GetNumBytesPerVertex(),&vecMin,&vecMax);
		this->m_BoundingBox.SetBorderPoints(vecMin.x,vecMin.y,vecMin.z,vecMax.x,vecMax.y,vecMax.z);
		// unlock vertex buffer
		this->m_pMesh->UnlockVertexBuffer();
	}

    // Make sure there are normals
    if( !(this->m_pMesh->GetFVF() & D3DFVF_NORMAL) )
    {
		LPD3DXMESH pTempMesh;
        if (FAILED(this->m_pMesh->CloneMeshFVF( this->m_pMesh->GetOptions(), 
												this->m_pMesh->GetFVF() | D3DFVF_NORMAL, 
												this->m_pD3DDevice, &pTempMesh ) ) )
		{
			// error => free everything
			this->Free();
			return false;
		}

		// compute normals
        if (FAILED(D3DXComputeNormals( pTempMesh, NULL )))
		{
			return false;
		}

		// apply cloned mesh
        SAFE_RELEASE( this->m_pMesh );
        this->m_pMesh = pTempMesh;
    }

	// get object information
	if (this->m_pMesh)
	{
		this->m_iNumFaces=this->m_pMesh->GetNumFaces();
		this->m_iNumVertices=this->m_pMesh->GetNumVertices();
	}

	// ok
	return true;
}

bool C3DObjectRendererD3DXMESH::CreateD3DXMesh(const C3DObjectData& ObjectData,const std::string strTexturePath/*=""*/)
{
	// get mesh (we currently assume, that all meshes have the same parameters)
	const C3DMesh* pMesh=ObjectData.GetMesh(0);
	if (!pMesh)
	{
		return false;
	}

	// create FVF
	DWORD dwFVF=D3DFVF_XYZ;
	if (pMesh->HasVertexColor())
	{
		dwFVF|=D3DFVF_DIFFUSE;
	}
	if (pMesh->HasVertexNormals())
	{
		dwFVF|=D3DFVF_NORMAL;
	}
	if (pMesh->GetNumMappingCoords()>0)
	{
		dwFVF|=D3DFVF_TEX1|D3DFVF_TEXCOORDSIZE1(2);
	}
	else
	{
		dwFVF|=D3DFVF_TEX0;
	}

	// ----------------------------------------------------------------------------
	// create a mesh object
	if (FAILED(D3DXCreateMeshFVF(static_cast<DWORD>(ObjectData.GetNumFaces()),
								 static_cast<DWORD>(ObjectData.GetNumVertices()),
								 D3DXMESH_MANAGED|D3DXMESH_VB_MANAGED|D3DXMESH_IB_MANAGED|D3DXMESH_POINTS|D3DXMESH_32BIT,
								 dwFVF,
								 this->m_pD3DDevice,
								 &this->m_pMesh)))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------------------------
	// fill mesh vertices
	IDirect3DVertexBuffer9* pVB=NULL;
	if (FAILED(this->m_pMesh->GetVertexBuffer(&pVB)))
	{
		return false;
	}
	BYTE* pData;
	if (FAILED(pVB->Lock(0,0,(void**)&pData,0)))
	{
		SAFE_RELEASE(pVB);
		return false;
	}
	// fill vertices
	DWORD dwOffset=0;
	for (unsigned int nMesh=0;nMesh<ObjectData.GetNumMeshes();nMesh++)
	{
		pMesh=ObjectData.GetMesh(nMesh);
		if (pMesh)
		{
			for (unsigned int nVertex=0;nVertex<pMesh->GetNumVertices();nVertex++)
			{
				// get vertex
				const PNPTC_VERTEX& vert=pMesh->GetVertex(nVertex);
				// copy position data
				memcpy(&pData[dwOffset],&vert.vPos,sizeof(float)*3);
				dwOffset+=sizeof(float)*3;
				// copy normal data
				if (pMesh->HasVertexNormals())
				{
					memcpy(&pData[dwOffset],&vert.vNormal,sizeof(float)*3);
					dwOffset+=sizeof(float)*3;
				}
				// copy color data
				if (pMesh->HasVertexColor())
				{
					D3DCOLOR col(TCColor2DXColor(vert.colDiffuse));
					memcpy(&pData[dwOffset],&col,sizeof(DWORD));
					dwOffset+=sizeof(DWORD);
				}
				// copy uv data
				if (pMesh->GetNumMappingCoords()>0)
				{
					memcpy(&pData[dwOffset],&vert.uv[0],sizeof(float)*2);
					dwOffset+=sizeof(float)*2;
				}
			}
		}
	}
	pVB->Unlock();
	SAFE_RELEASE(pVB);

	// ----------------------------------------------------------------------------
	// fill faces
	IDirect3DIndexBuffer9* pIB=NULL;
	if (FAILED(this->m_pMesh->GetIndexBuffer(&pIB)))
	{
		return false;
	}
	if (FAILED(pIB->Lock(0,0,(void**)&pData,0)))
	{
		SAFE_RELEASE(pIB);
		return false;
	}
	// get format of index buffer
	D3DINDEXBUFFER_DESC desc;
	pIB->GetDesc(&desc);
	// fill faces
	dwOffset=0;
	unsigned int nVertexOffset=0;
	for (unsigned int nMesh=0;nMesh<ObjectData.GetNumMeshes();nMesh++)
	{
		pMesh=ObjectData.GetMesh(nMesh);
		if (pMesh)
		{
			for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet)
				{
					for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
					{
						PNPTC_FACE face=pFaceSet->GetFace(nFace);
						face.dwVert[0]+=nVertexOffset;
						face.dwVert[1]+=nVertexOffset;
						face.dwVert[2]+=nVertexOffset;
						if (desc.Format==D3DFMT_INDEX16)
						{
							static WORD wIndices[3];
							wIndices[0]=(WORD)face.dwVert[0];
							wIndices[1]=(WORD)face.dwVert[1];
							wIndices[2]=(WORD)face.dwVert[2];
							memcpy(&pData[dwOffset],wIndices,sizeof(WORD)*3);
							dwOffset+=sizeof(WORD)*3;
						}
						else
						{
							memcpy(&pData[dwOffset],&face,sizeof(DWORD)*3);
							dwOffset+=sizeof(DWORD)*3;
						}
					}
				}
			}
			nVertexOffset+=static_cast<unsigned int>(pMesh->GetNumVertices());
		}
	}
	pIB->Unlock();
	SAFE_RELEASE(pIB);

	// ----------------------------------------------------------------------------
	// fill attributes
	DWORD* pAttributeData;
	if (FAILED(this->m_pMesh->LockAttributeBuffer(0,&pAttributeData)))
	{
		return false;
	}
	unsigned int nAttribute=0;
	unsigned int nFaceCounter=0;
	for (unsigned int nMesh=0;nMesh<ObjectData.GetNumMeshes();nMesh++)
	{
		pMesh=ObjectData.GetMesh(nMesh);
		if (pMesh)
		{
			for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++,nAttribute++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet)
				{
					for (unsigned int nFace=0;nFace<pFaceSet->GetNumFaces();nFace++,nFaceCounter++)
					{
						pAttributeData[nFaceCounter]=nAttribute;
					}
				}
			}
		}
	}
	this->m_pMesh->UnlockAttributeBuffer();

	// ----------------------------------------------------------------------------
	// fill materials and load textures
	if (!this->SetNumMaterials(nAttribute))
	{
		return false;
	}
	unsigned int nMaterial=0;
	for (unsigned int nMesh=0;nMesh<ObjectData.GetNumMeshes();nMesh++)
	{
		pMesh=ObjectData.GetMesh(nMesh);
		if (pMesh)
		{
			for (unsigned int nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++,nMaterial++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet)
				{
					const PNPTC_MATERIAL* pMat=ObjectData.GetMaterial(pFaceSet->GetMaterial());
					this->m_vecTextures[nMaterial].pTexture=NULL;
					if (pMat)
					{
						this->m_vecMaterials[nMaterial].Ambient=TCColor2DXColor(pMat->colAmbient);
						this->m_vecMaterials[nMaterial].Diffuse=TCColor2DXColor(pMat->colDiffuse);;
						if (this->m_vecMaterials[nMaterial].Ambient.r==0 && this->m_vecMaterials[nMaterial].Ambient.g==0 && this->m_vecMaterials[nMaterial].Ambient.b==0)
						{
							this->m_vecMaterials[nMaterial].Ambient=this->m_vecMaterials[nMaterial].Diffuse;
						}
						this->m_vecMaterials[nMaterial].Emissive=D3DXCOLOR(0,0,0,1);
						this->m_vecMaterials[nMaterial].Power=0.f;
						this->m_vecMaterials[nMaterial].Specular=D3DXCOLOR(0,0,0,1);
						// load texture
						this->m_vecTextures[nMaterial].strTexture=pMat->strTexture[0];
						if (pMat->strTexture[0].length()>0)
						{
							// Load textur
							if ( ((this->m_vecTextures[nMaterial].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,strTexturePath+pMat->strTexture[0])) == NULL) &&
								((this->m_vecTextures[nMaterial].pTexture=C3DObjectRendererDX::s_TextureManager.CreateTexture(this->m_pD3DDevice,pMat->strTexture[0])) == NULL) )
							{
								// error loading texture
								this->m_vecTextures[nMaterial].pTexture=NULL;
							}
							else
							{
								// ensure material color is not black if a texture is applied
								if (this->m_vecMaterials[nMaterial].Ambient.r==0 && this->m_vecMaterials[nMaterial].Ambient.g==0 && this->m_vecMaterials[nMaterial].Ambient.b==0)
								{
									this->m_vecMaterials[nMaterial].Ambient.r=1.f;
									this->m_vecMaterials[nMaterial].Ambient.g=1.f;
									this->m_vecMaterials[nMaterial].Ambient.b=1.f;
								}
								if (this->m_vecMaterials[nMaterial].Diffuse.r==0 && this->m_vecMaterials[nMaterial].Diffuse.g==0 && this->m_vecMaterials[nMaterial].Diffuse.b==0)
								{
									this->m_vecMaterials[nMaterial].Diffuse.r=1.f;
									this->m_vecMaterials[nMaterial].Diffuse.g=1.f;
									this->m_vecMaterials[nMaterial].Diffuse.b=1.f;
								}
							}
						}
					}
					else
					{
						// use default material
						this->m_vecMaterials[nMaterial].Ambient.r=1.f;
						this->m_vecMaterials[nMaterial].Ambient.g=1.f;
						this->m_vecMaterials[nMaterial].Ambient.b=1.f;
						this->m_vecMaterials[nMaterial].Ambient.a=1.f;
						this->m_vecMaterials[nMaterial].Diffuse=this->m_vecMaterials[nMaterial].Ambient;
						this->m_vecMaterials[nMaterial].Emissive=D3DXCOLOR(0,0,0,1);
						this->m_vecMaterials[nMaterial].Power=0.f;
						this->m_vecMaterials[nMaterial].Specular=D3DXCOLOR(0,0,0,1);
					}
				}
			}
		}
	}

	// ok
	return true;
}

void C3DObjectRendererD3DXMESH::Free()
{
	// free all DX objects
	SAFE_RELEASE(this->m_pMesh);
	SAFE_RELEASE(this->m_pbufAdjacency);

	// release all textures
	for (unsigned int nTexture=0;nTexture<this->m_vecTextures.size();nTexture++)
	{
		if (this->m_vecTextures[nTexture].pTexture)
		{
			C3DObjectRendererDX::s_TextureManager.ReleaseTexture(this->m_vecTextures[nTexture].pTexture);
			this->m_vecTextures[nTexture].pTexture=NULL;
		}
	}
	this->m_vecTextures.clear();
	this->m_vecMaterials.clear();
}

bool C3DObjectRendererD3DXMESH::Save(I3DObjectSerializer* pSerializer,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/)
{
	// check parameters
	if (!pSerializer)
	{
		MessageBox(NULL,(std::string("Error saving object to file'")+strFileName+"':\n\nInvalid serializer!").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// get object name
	std::string strObjectName;
	std::string strExtension;
	RemoveFileExtension(strFileName,strObjectName,strExtension);

	// fill 3DObjectData
	C3DObjectData objectData;
	if (!this->FillObjectData(objectData,strObjectName))
	{
		return false;
	}

	// save to file
	bool bReturn=pSerializer->Save(objectData,strFileName,fScale,iUpRotation,iFlipAxis,cBasePathName);
	if (bReturn)
	{
		// get textures directory
		std::string strTextureFolder=GetPath(strFileName);

		// save all textures
		for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
		{
			if (nMaterial<this->m_vecTextures.size() && this->m_vecTextures[nMaterial].pTexture)
			{
				// save this texture
				if (FAILED(D3DXSaveTextureToFile((strTextureFolder+this->m_vecTextures[nMaterial].strTexture).c_str(),GetImageFormat(this->m_vecTextures[nMaterial].strTexture),this->m_vecTextures[nMaterial].pTexture,NULL)))
				{
					MessageBox(NULL,(std::string("Error saving object to file'")+strFileName+"':\n\nError saving texture to file '"+strTextureFolder+this->m_vecTextures[nMaterial].strTexture+"'!").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					bReturn=false;
				}
			}
		}
	}

	// return result
	return bReturn;
}

bool C3DObjectRendererD3DXMESH::FillObjectData(C3DObjectData& ObjectData,std::string strObjectName)
{
	// is the mesh available
	if (!this->m_pMesh)
	{
		return false;
	}

	// check format of the object name
	Replace(&strObjectName," ","_");
	Replace(&strObjectName,"/","_");
	Replace(&strObjectName,"\\","_");
	Replace(&strObjectName,".","_");

	// ------------------------------------------------------------------------------
	// copy materials

	// copy all materials
	bool bTextured=false;
	for (unsigned int nMaterial=0;nMaterial<this->m_vecMaterials.size();nMaterial++)
	{
		// create material
		PNPTC_MATERIAL mat;
		mat.bAlphaBlend=false;
		mat.bUseVertexColors=false;
		mat.colAmbient=DXColor2TCColor(this->m_vecMaterials[nMaterial].Ambient);
		mat.colDiffuse=DXColor2TCColor(this->m_vecMaterials[nMaterial].Diffuse);
		mat.strName=strObjectName+"_"+ToString(static_cast<int>(nMaterial));
		if (this->m_vecTextures[nMaterial].pTexture)
		{
			// set texture
			mat.iTextures=1;
			mat.strTexture[0]=this->m_vecTextures[nMaterial].strTexture;
			bTextured=true;

			// does the texture support alpha
			if (TextureSupportsAlpha(mat.strTexture[0]))
			{
				mat.bAlphaBlend=true;
			}
		}
		else
		{
			// no textures
			mat.iTextures=0;
		}

		// alpha color
		if (mat.colDiffuse.a<255 || mat.colAmbient.a<255)
		{
			mat.bAlphaBlend=true;
		}

		// add material
		ObjectData.AddMaterial(mat);
	}

	// ------------------------------------------------------------------------------
	// copy geometry

	// get mesh vertex format
	DWORD dwFVF=this->m_pMesh->GetFVF();
	DWORD dwVertexSize=this->m_pMesh->GetNumBytesPerVertex();
	bool bColor=(dwFVF & D3DFVF_DIFFUSE)!=0;
	bool bNormals=(dwFVF & D3DFVF_NORMAL)!=0;

	// now copy geometry into a new mesh
	C3DMesh* pMesh=new C3DMesh(&ObjectData,bColor,bNormals,bTextured);
	if (!pMesh)
	{
		return false;
	}
	ObjectData.AddMesh(pMesh);

	// copy all vertices
	if (!pMesh->ReserveVertexMem(this->m_pMesh->GetNumVertices()))
	{
		return false;
	}
	IDirect3DVertexBuffer9* pVB=NULL;
	if (SUCCEEDED(this->m_pMesh->GetVertexBuffer(&pVB)))
	{
		// lock vertex buffer
		BYTE* pData;
		if (FAILED(pVB->Lock(0,0,(void**)&pData,0)))
		{
			SAFE_RELEASE(pVB);
			return false;
		}

		// go through all vertices
		for (unsigned int nVertex=0;nVertex<this->m_pMesh->GetNumVertices();nVertex++)
		{
			// vertex data offset
			DWORD dwDataOffset=0;

			// set up vertex
			PNPTC_VERTEX vert;

			// get position
			vert.vPos.x=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
			dwDataOffset+=sizeof(float);
			vert.vPos.y=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
			dwDataOffset+=sizeof(float);
			vert.vPos.z=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
			dwDataOffset+=sizeof(float);

			// get normal value
			if (bNormals)
			{
				vert.vNormal.x=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
				dwDataOffset+=sizeof(float);
				vert.vNormal.y=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
				dwDataOffset+=sizeof(float);
				vert.vNormal.z=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
				dwDataOffset+=sizeof(float);
			}

			// get color value
			if (bColor)
			{
				vert.colDiffuse=DXColor2TCColor(*reinterpret_cast<D3DCOLOR*>(&pData[nVertex*dwVertexSize+dwDataOffset]));
				dwDataOffset+=sizeof(D3DCOLOR);
			}

			// get texture coordinate
			if (bTextured)
			{
				vert.uv[0].u=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
				dwDataOffset+=sizeof(float);
				vert.uv[0].v=*reinterpret_cast<float*>(&pData[nVertex*dwVertexSize+dwDataOffset]);
				dwDataOffset+=sizeof(float);
			}

			// add vertex
			pMesh->AddVertex(vert);

			// check data
			if (dwDataOffset>dwVertexSize)
			{
				// error
				pVB->Unlock();
				SAFE_RELEASE(pVB);
				return false;
			}
		}

		// unlock and release vertex buffer
		pVB->Unlock();
		SAFE_RELEASE(pVB);
	}
	else
	{
		// error returning vertex buffer
		return false;
	}


	// ------------------------------------------------------------------------------
	// copy face sets

	// get attributes
	DWORD dwNumAttributes=0;
	D3DXATTRIBUTERANGE* pAttributes=NULL;
	this->m_pMesh->GetAttributeTable(NULL,&dwNumAttributes);
	if (dwNumAttributes>0)
	{
		pAttributes=new D3DXATTRIBUTERANGE[dwNumAttributes];
		if (FAILED(this->m_pMesh->GetAttributeTable(pAttributes,&dwNumAttributes)))
		{
			return false;
		}
	}
	else
	{
		// error
		return false;
	}

	// get index buffer
	IDirect3DIndexBuffer9* pIB=NULL;
	if (SUCCEEDED(this->m_pMesh->GetIndexBuffer(&pIB)))
	{
		// lock index buffer
		BYTE* pData;
		if (FAILED(pIB->Lock(0,0,(void**)&pData,0)))
		{
			SAFE_RELEASE(pIB);
			SAFE_DELETEARRAY(pAttributes);
			return false;
		}

		// get format of index buffer
		D3DINDEXBUFFER_DESC desc;
		pIB->GetDesc(&desc);
		DWORD dwIndexSize=(desc.Format==D3DFMT_INDEX16)?sizeof(WORD):sizeof(DWORD);

		// create a face set for every material
		for (unsigned int nFaceSet=0;nFaceSet<this->m_vecMaterials.size();nFaceSet++)
		{
			// search for this attribute ID
			D3DXATTRIBUTERANGE* pAttribute=NULL;
			for (unsigned int nAtt=0;nAtt<dwNumAttributes;nAtt++)
			{
				if (pAttributes[nAtt].AttribId==nFaceSet)
				{
					pAttribute=&pAttributes[nAtt];
					break;
				}
			}

			// create face set
			if (pAttribute)
			{
				// add face set
				C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,nFaceSet);
				if (pFaceSet && pFaceSet->ReserveFaceMem(pAttribute->FaceCount))
				{
					// add face set
					pMesh->AddFaceSet(pFaceSet);

					// create all faces
					for (unsigned int nFace=0;nFace<pAttribute->FaceCount;nFace++)
					{
						// get face info
						PNPTC_FACE face;
						if (desc.Format==D3DFMT_INDEX16)
						{
							face.dwVert[0]=*reinterpret_cast<WORD*>(&pData[((nFace+pAttribute->FaceStart)*3+0)*dwIndexSize]);
							face.dwVert[1]=*reinterpret_cast<WORD*>(&pData[((nFace+pAttribute->FaceStart)*3+1)*dwIndexSize]);
							face.dwVert[2]=*reinterpret_cast<WORD*>(&pData[((nFace+pAttribute->FaceStart)*3+2)*dwIndexSize]);
						}
						else
						{
							face.dwVert[0]=*reinterpret_cast<DWORD*>(&pData[((nFace+pAttribute->FaceStart)*3+0)*dwIndexSize]);
							face.dwVert[1]=*reinterpret_cast<DWORD*>(&pData[((nFace+pAttribute->FaceStart)*3+1)*dwIndexSize]);
							face.dwVert[2]=*reinterpret_cast<DWORD*>(&pData[((nFace+pAttribute->FaceStart)*3+2)*dwIndexSize]);
						}

						// set face
						pFaceSet->AddFace(face);				
					}
				}
				else
				{
					// error
					SAFE_DELETE(pFaceSet);
					pIB->Unlock();
					SAFE_RELEASE(pIB);
					SAFE_DELETEARRAY(pAttributes);
					return false;
				}
			}
		}

		// unlock and release index buffer
		pIB->Unlock();
		SAFE_RELEASE(pIB);
	}

	// free everything
	SAFE_DELETEARRAY(pAttributes);

	// ok
	return true;
}

bool C3DObjectRendererD3DXMESH::Render(const D3DXMATRIX& matPosition,bool bAlpha/*=false*/,int iMaterial/*=-1*/,bool bSetResources/*=true*/) const
{
	// check initialization
	if (!this->m_pMesh)
	{
		return false;
	}

	// set render states
	if (bSetResources)
	{
		this->m_pD3DDevice->SetTexture(1,NULL);
		this->m_pD3DDevice->SetFVF(this->m_pMesh->GetFVF());
		this->m_pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
		this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
		this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TEXTURE);
	}

	// set world matrix
	this->m_pD3DDevice->SetTransform(D3DTS_WORLD, &matPosition);

	// compute start and end material index of rendering
	int iMatStart=(iMaterial<0)?0:iMaterial;
	int iNumRender=(iMaterial<0)?static_cast<int>(this->GetNumMaterials()):min(static_cast<int>(this->GetNumMaterials()),iMaterial+1);

	// render object
    for( int i = iMatStart; i < iNumRender; i++ )
    {
		// set material
		if (bSetResources)
		{
			this->m_pD3DDevice->SetMaterial( &this->m_vecMaterials[i] );
			LPDIRECT3DTEXTURE9 pTex=this->m_vecTextures[i].pTexture;
			this->m_pD3DDevice->SetTexture(0,pTex);
			if (pTex)
			{
				this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
				if (bAlpha)
				{
					this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
					this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
				}
				else
				{
					this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
				}
			}
			else
			{
				this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
				this->m_pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			}
		}

		// draw object's subset for this material
        this->m_pMesh->DrawSubset( i );
    }

	// ok
	return true;
}

bool C3DObjectRendererD3DXMESH::Intersect(const pnpse::CRay& ray) const
{
	// initialized?
	if (!this->m_pMesh)
	{
		return false;
	}

	// compute intersection
	float	fU=0;
	float	fV=0;
	BOOL	bHit=0;
	DWORD	dwFaceIndex=0;
	float	fDist=0;
	D3DXVECTOR3 vecStart=static_cast<const float*>(ray.GetStartPoint());
	D3DXVECTOR3 vecDir=static_cast<const float*>(ray.GetDirection());
	if (FAILED(D3DXIntersect(this->m_pMesh,&vecStart,&vecDir,&bHit,&dwFaceIndex,&fU,&fV,&fDist,NULL,NULL)))
	{
		// error
		return false;
	}	

	// return intersection result
	return (bHit!=0);
}

bool C3DObjectRendererD3DXMESH::SetNumMaterials(unsigned int nNumMaterials)
{
	// set size of lists
	this->m_vecMaterials.resize(nNumMaterials);
	this->m_vecTextures.resize(nNumMaterials);

	// ok
	return true;
}
