/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectRendererD3DXMESH.h
// Mesh renderer which is based on the DirectX D3DXMESH data structure.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////
#if !defined(PNPTC_3DOBJECTRENDERERD3DXMESH_H_INCLUDED)
#define PNPTC_3DOBJECTRENDERERD3DXMESH_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include "3DObjectRendererDX.h"


// forward definitions
class C3DObjectData;
class I3DObjectSerializer;


/// An object renderer class, which is based on 
/// the DirectX D3DXMESH data structure.
class C3DObjectRendererD3DXMESH : public C3DObjectRendererDX
{
public:

	// --------------------------------------------------------------
	// construction / destruction
	// --------------------------------------------------------------

	/// Construction of a new DX renderer.
	/// @param pD3DDevice The DX rendering device.
	C3DObjectRendererD3DXMESH(IDirect3DDevice9* pD3DDevice);

	/// Destruction of the renderer
	virtual ~C3DObjectRendererD3DXMESH(void);


	// --------------------------------------------------------------
	// object loading
	// --------------------------------------------------------------

	/// Loads a mesh file.
	/// @param strFileName Name of the mesh file to be loaded,
	/// including the fully specified path.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the mesh could be loaded successfully.
	bool						Load(const std::string& strFileName);

	/// Creates/initializes a D3DXMESH from a given 3D object data object.
	/// @param ObjectData Data object from which to initialize the mesh.
	/// @param strTexturePath The path in which to search for the texture files.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the mesh could be initialized successfully.
	bool						CreateD3DXMesh(const C3DObjectData& ObjectData,const std::string strTexturePath="");

	/// Saves the mesh to a file.
	/// @param pSerializer Serializer which should save that object.
	/// @param strFileName Object file (including path) to be saved to.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be saved successfully.
	bool						Save(I3DObjectSerializer* pSerializer,const std::string& strFileName,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL);

	/// Frees everything
	void						Free();


	// ---------------------------------------------------------------------
	// 3D rendering methods
	// ---------------------------------------------------------------------

	/// Render object at a given position.
	/// @param matWorld World matrix of the object to be rendered.
	/// @param bAlpha Whether or not this object has alpha.
	/// @param iMaterial The material (sub mesh) which is to be rendered.
	/// <i>-1</i> to render the whole mesh.
	/// @param bSetResources Whether or not the resources of this object
	/// should be set or the current resource settings should be reused.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// mesh could be rendered successfully.
	bool						Render(const D3DXMATRIX& matPosition,bool bAlpha=false,int iMaterial=-1,bool bSetResources=true) const;

	/// Tests intersection with this object in the object's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the object.
	/// @return Returns the <i>true</i> if the object was intersected
	/// by the ray, otherwise <i>false</i>.
	bool						Intersect(const pnpse::CRay& ray) const;


	// ---------------------------------------------------------------------
	// Retrieve information about the mesh
	// ---------------------------------------------------------------------

	/// Returns the number of materials used in this object.
	/// @return Number of materials of this object.
	int							GetNumMaterials() const {return static_cast<int>(this->m_vecMaterials.size());};


private:

	// ---------------------------------------------------------------------
	// internal methods
	// ---------------------------------------------------------------------

	/// Sets the number of materials/textures.
	/// @param nNumMaterials Number of materials to be set.
	/// @return Returns <i>true</i> or <i>false</i>, whether or
	/// not the number of materials could be set.
	bool							SetNumMaterials(unsigned int nNumMaterials);

	/// Fills a 3DObjecData object with the contents of the D3DXMESH.
	/// @param ObjectData The object, which should receive the data from the mesh.
	/// @param strObjectName The name of the object to be created.
	/// @return Returns <i>true</i> if the object data could be copied,
	/// otherwise <i>false</i>.
	bool							FillObjectData(C3DObjectData& ObjectData,std::string strObjectName);


	// --------------------------------------------------------------
	// internal data types
	// --------------------------------------------------------------

	/// Texture description.
	typedef struct TEXTURE_DESC
	{
		std::string			strTexture;		//!< the name of the texture file.
		LPDIRECT3DTEXTURE9	pTexture;		//!< the rendered texture.

		/// construction of a new texture descriptor
		TEXTURE_DESC()
			: pTexture(NULL) {};

	} TEXTURE_DESC;


	// --------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------

	/// mesh object
	LPD3DXMESH						m_pMesh;

	/// adjacency buffer loaded with mesh
	LPD3DXBUFFER					m_pbufAdjacency;

	/// List of materials.
	std::vector<D3DMATERIAL9>		m_vecMaterials;

	/// List of textures.
	std::vector<TEXTURE_DESC>		m_vecTextures;

};

#endif // !defined(PNPTC_3DOBJECTRENDERERD3DXMESH_H_INCLUDED)
