/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializer3DS.h
// 3d object serialization class for .3ds files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "3DObjectData.h"
#include ".\3dobjectserializer3ds.h"
#include <algorithm>
#include "../PnPTCUtils/Utils.h"



/// default material name
#define MATERIAL_DEFAULT		"PNP_DEFAULT"


/// 3DS chunk header.
typedef CChunkHeaderIDSize<WORD,DWORD> CChunkHeader3DS;


// --------------------------------------------------------------------------------------------
// 3ds chunks

 //------ Primary chunk

 #define MAIN3DS       0x4D4D

 //------ Main Chunks

 #define EDIT3DS       0x3D3D  // this is the start of the editor config
 #define KEYF3DS       0xB000  // this is the start of the keyframer config

 //------ sub defines of EDIT3DS

 #define EDIT_MATERIAL 0xAFFF
 #define EDIT_CONFIG1  0x0100
 #define EDIT_CONFIG2  0x3E3D
 #define EDIT_VIEW_P1  0x7012
 #define EDIT_VIEW_P2  0x7011
 #define EDIT_VIEW_P3  0x7020
 #define EDIT_VIEW1    0x7001
 #define EDIT_BACKGR   0x1200
 #define EDIT_AMBIENT  0x2100
 #define EDIT_OBJECT   0x4000

 #define EDIT_UNKNW01  0x1100
 #define EDIT_UNKNW02  0x1201
 #define EDIT_UNKNW03  0x1300
 #define EDIT_UNKNW04  0x1400
 #define EDIT_UNKNW05  0x1420
 #define EDIT_UNKNW06  0x1450
 #define EDIT_UNKNW07  0x1500
 #define EDIT_UNKNW08  0x2200
 #define EDIT_UNKNW09  0x2201
 #define EDIT_UNKNW10  0x2210
 #define EDIT_UNKNW11  0x2300
 #define EDIT_UNKNW12  0x2302 
 #define EDIT_UNKNW13  0x3000
 #define EDIT_UNKNW14  0xAFFF

 //------ sub defines of EDIT_MATERIAL
#define MAT_NAME			0xA000  // includes name (see mli doc for materials)
#define MAT_AMBIENT			0xA010	
#define MAT_DIFFUSE			0xA020	
#define MAT_SPECULAR		0xA030	
#define MAT_SHININESS		0xA040
#define MAT_TRANSPARENCY	0xA050
#define MAT_TRANSFALLOFF	0xA052
#define MAT_REFBLUR			0xA053

//------ texture chunks
#define TEX_TEXTURE1		0xA200
#define TEX_TEXTURE2		0xA33A
#define TEX_OPACITY			0xA210
#define TEX_BUMP			0xA230
#define TEX_SPECULAR		0xA204
#define TEX_SHININESS		0xA33C
#define TEX_SELFILLUM		0xA33D
#define TEX_REFLECTION		0xA220

#define TEX_FILENAME		0xA300
#define TEX_OPTIONS			0xA351
#define TEX_FILTERBLUR		0xA353

//------ sub defines of EDIT_OBJECT
 #define OBJ_TRIMESH   0x4100
 #define OBJ_LIGHT     0x4600
 #define OBJ_CAMERA    0x4700

 #define OBJ_UNKNWN01  0x4010
 #define OBJ_UNKNWN02  0x4012 //---- Could be shadow

 //------ sub defines of OBJ_CAMERA
 #define CAM_UNKNWN01  0x4710 
 #define CAM_UNKNWN02  0x4720 

 //------ sub defines of OBJ_LIGHT
 #define LIT_OFF       0x4620
 #define LIT_SPOT      0x4610
 #define LIT_UNKNWN01  0x465A

 //------ sub defines of OBJ_TRIMESH
 #define TRI_VERTEXL   0x4110
 #define TRI_FACEL2    0x4111 
 #define TRI_FACEL1    0x4120
 #define TRI_MATERIAL  0x4130
 #define TRI_MAPPING   0x4140
 #define TRI_SMOOTH    0x4150
 #define TRI_LOCAL     0x4160
 #define TRI_VISIBLE   0x4165

 //------ sub defs of KEYF3DS

 #define KEYF_UNKNWN01 0xB009
 #define KEYF_UNKNWN02 0xB00A
 #define KEYF_FRAMES   0xB008
 #define KEYF_OBJDES   0xB002

 //------  these define the different color chunk types
 #define COL_RGB  0x0010
 #define COL_TRU  0x0011
 #define COL_UNK  0x0013

 //------  these define the some more data format
 #define AMOUNT_OF 0x0030

 //------ defines for viewport chunks

 #define TOP           0x0001
 #define BOTTOM        0x0002
 #define LEFT          0x0003
 #define RIGHT         0x0004
 #define FRONT         0x0005
 #define BACK          0x0006
 #define USER          0x0007
 #define CAMERA        0x0008 // 0xFFFF is the actual code read from file
 #define LIGHT         0x0009
 #define DISABLED      0x0010
 #define BOGUS         0x0011


// --------------------------------------------------------------------------------------------
// reading/writing binary data

bool _3DSPrintWORD(CChunkedFile& file,WORD wValue)
{
	// print word
	return file.Write(&wValue,sizeof(WORD));
}

bool _3DSReadWORD(CChunkedFile& file,WORD& wValue)
{
	// read word
	return file.Read(&wValue,sizeof(WORD));
}

bool _3DSPrintDWORD(CChunkedFile& file,DWORD dwValue)
{
	// print dword
	return file.Write(&dwValue,sizeof(DWORD));
}

bool _3DSPrintInt(CChunkedFile& file,int iValue)
{
	// print int
	return file.Write(&iValue,sizeof(int));
}

bool _3DSPrintBYTE(CChunkedFile& file,BYTE byValue)
{
	// print byte
	return file.Write(&byValue,sizeof(BYTE));
}

bool _3DSReadBYTE(CChunkedFile& file,BYTE& byValue)
{
	// read byte
	return file.Read(&byValue,sizeof(BYTE));
}

bool _3DSPrintFloat(CChunkedFile& file,float fValue)
{
	// print word
	return file.Write(&fValue,sizeof(float));
}

bool _3DSReadFloat(CChunkedFile& file,float& fValue)
{
	// print word
	return file.Read(&fValue,sizeof(float));
}

bool _3DSPrintChars(CChunkedFile& file,const char* cString)
{
	// get string length
	DWORD dwLength=(DWORD)strlen(cString);

	// print string
	return file.Write(cString,dwLength+1);
}

bool _3DSReadChars(CChunkedFile& file,std::string& strRead)
{
	// initialize return string
	strRead="";

	// read until zero
	char cRead;	
	bool bReadOK=false;
	while ((bReadOK=file.Read(&cRead,1)) && cRead!=0)
	{
		strRead+=cRead;
	}

	// return result
	return bReadOK;
}

bool _3DSPrintChunkHeader(CChunkedFile& file,WORD wChunk,DWORD dwSize=0)
{
	// print chunk header
	if (!_3DSPrintWORD(file,wChunk))
	{
		return false;
	}

	// start chunk
	return file.StartChunkWriting();
}

bool _3DSFinishChunk(CChunkedFile& file)
{
	// finish chunk
	return file.FinishChunkWriting();
}

bool _3DSFinishChunk(CChunkedFile& file,DWORD dwSize)
{
	// finish chunk
	return file.FinishChunkWriting(dwSize);
}

bool _3DSPrintRGB(CChunkedFile& f,const PNPTC_RGBA& rgb)
{
	// ----------------------------------------------------------
	// print color
	if (!_3DSPrintChunkHeader(f,COL_TRU) || 
		!_3DSPrintBYTE(f,rgb.r) || 
		!_3DSPrintBYTE(f,rgb.g) || 
		!_3DSPrintBYTE(f,rgb.b) || 
		!_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ok
	return true;
}

bool _3DSReadRGB(CChunkedFile& f,PNPTC_RGBA& rgb)
{
	// read color
	return	_3DSReadBYTE(f,rgb.r) && 
			_3DSReadBYTE(f,rgb.g) &&
			_3DSReadBYTE(f,rgb.b) ;
}

bool _3DSPrintTexture(CChunkedFile& f,const std::string& strTextureFileName)
{
	// ----------------------------------------------------------
	// print texture chunk
	if (!_3DSPrintChunkHeader(f,TEX_TEXTURE1))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print amount of
	if (!_3DSPrintChunkHeader(f,AMOUNT_OF) || !_3DSPrintWORD(f,100) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print texture file name
	if (!_3DSPrintChunkHeader(f,TEX_FILENAME) || !_3DSPrintChars(f,strTextureFileName.c_str()) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print map options
	if (!_3DSPrintChunkHeader(f,TEX_OPTIONS) || !_3DSPrintWORD(f,0) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print blur factor
	if (!_3DSPrintChunkHeader(f,TEX_FILTERBLUR) || !_3DSPrintFloat(f,0.f) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// finish texture chunk
	if (!_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ok
	return true;
}

bool _3DSPrintMaterial(CChunkedFile& f,const PNPTC_MATERIAL& mat)
{
	// ----------------------------------------------------------
	// print material name
	if (!_3DSPrintChunkHeader(f,MAT_NAME) || !_3DSPrintChars(f,mat.strName.c_str()) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print ambient color
	if (!_3DSPrintChunkHeader(f,MAT_AMBIENT) || !_3DSPrintRGB(f,mat.colAmbient) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print diffuse color
	if (!_3DSPrintChunkHeader(f,MAT_DIFFUSE) || !_3DSPrintRGB(f,mat.colDiffuse) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print specular color
	PNPTC_RGBA rgb(0,0,0,0);
	if (!_3DSPrintChunkHeader(f,MAT_SPECULAR) || !_3DSPrintRGB(f,rgb) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print shininess
	if (!_3DSPrintChunkHeader(f,MAT_SHININESS) || !_3DSPrintChunkHeader(f,AMOUNT_OF) || !_3DSPrintWORD(f,0) || !_3DSFinishChunk(f) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print transparency
	if (!_3DSPrintChunkHeader(f,MAT_TRANSPARENCY) || !_3DSPrintChunkHeader(f,AMOUNT_OF) || !_3DSPrintWORD(f,0) || !_3DSFinishChunk(f) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print transparency falloff
	if (!_3DSPrintChunkHeader(f,MAT_TRANSFALLOFF) || !_3DSPrintChunkHeader(f,AMOUNT_OF) || !_3DSPrintWORD(f,0) || !_3DSFinishChunk(f) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print reflect blur
	if (!_3DSPrintChunkHeader(f,MAT_REFBLUR) || !_3DSPrintChunkHeader(f,AMOUNT_OF) || !_3DSPrintWORD(f,0) || !_3DSFinishChunk(f) || !_3DSFinishChunk(f))
	{
		// error
		return false;
	}

	// ----------------------------------------------------------
	// print textures
	if (mat.iTextures>0 && !mat.strTexture[0].empty())
	{
		if (!_3DSPrintTexture(f,mat.strTexture[0]))
		{
			// error 
			return false;
		}
	}

	// ok
	return true;
}




C3DObjectSerializer3DS::C3DObjectSerializer3DS(void)
	: CChunkedFile(sizeof(WORD))
	, m_pObject(NULL) 
	, m_bFileHeader(false)
	, m_bEditHeader(false)
	, m_bObjectHeader(false)
	, m_bMaterialHeader(false)
	, m_pMaterial(NULL)
	, m_pRGB(NULL)
	, m_pstrTextureName(NULL)
	, m_pMesh(NULL)
{
}

C3DObjectSerializer3DS::~C3DObjectSerializer3DS(void)
{
}




// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------



/// sort faces by their material
bool MaterialSort(const C3DObjectSerializer3DS::_3DS_FACE& face1, const C3DObjectSerializer3DS::_3DS_FACE& face2)
{
	// order by material ID
	return face1.iMaterial<face2.iMaterial;
}




bool C3DObjectSerializer3DS::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// reset everything
	this->m_pObject=&ObjectData;
	this->m_bFileHeader=false;
	this->m_bEditHeader=false;
	this->m_bObjectHeader=false;
	this->m_bMaterialHeader=false;
	this->m_pMaterial=NULL;
	this->m_pRGB=NULL;
	this->m_pstrTextureName=NULL;
	this->m_pMesh=NULL;

	//	check file name;
	if (strFileName.length()==0)
	{
		return false;
	}

	// initialize chunk file loader and load the 3ds file
	return this->OpenRead(strFileName) && this->ReadFile();
}

bool C3DObjectSerializer3DS::OnReadChunkData(const CChunkHeader& header)
{
	// check prerequisits
	if (!this->m_pObject)
	{
		return false;
	}

	// cast it to a 3DS header
	const CChunkHeader3DS& header3DS=static_cast<const CChunkHeader3DS&>(header);

	// check ID
	switch (header3DS.GetID())
	{
	case MAIN3DS:
		{
			// main chunk
			if (this->m_bFileHeader) 
			{
				// header already read => error
				return false;
			}
			else
			{
				// set header as beeing read and process sub chunks
				this->m_bFileHeader=true;
				bool bReturn=this->ProcessSubChunks(header);
				this->m_bFileHeader=false;
				return bReturn;
			}
		}
		break;
	case EDIT3DS:
		{
			// edit chunk
			if (this->m_bEditHeader) 
			{
				// header already read => error
				return false;
			}
			else if (!this->m_bFileHeader)
			{
				// file header not read
				return false;
			}
			else
			{
				// set header as beeing read and process sub chunks
				this->m_bEditHeader=true;
				bool bReturn=this->ProcessSubChunks(header);
				this->m_bEditHeader=false;
				return bReturn;
			}
		}
		break;
	case EDIT_MATERIAL:
		{
			// check if edit header was read
			if (!this->m_bEditHeader)
			{
				// edit header not read
				return false;
			}
			else if (this->m_bMaterialHeader)
			{
				// already processing materials
				return false;
			}

			// process material sub chunks
			this->m_bMaterialHeader=true;
			if (this->ProcessSubChunks(header))
			{
				// is there a material left to be added?
				if (this->m_pMaterial)
				{
					this->m_pObject->AddMaterial(*this->m_pMaterial);
				}
				SAFE_DELETE(this->m_pMaterial);

				// finished materials
				this->m_bMaterialHeader=false;
				return true;
			}
			else
			{
				// error reading materials
				SAFE_DELETE(this->m_pMaterial);
				this->m_bMaterialHeader=false;
				return false;
			}
		}
		break;
	case MAT_NAME:
		{
			// are we processing materials?
			if (!this->m_bMaterialHeader)
			{
				return false;
			}

			// finish previous material
			if (this->m_pMaterial)
			{
				this->m_pObject->AddMaterial(*this->m_pMaterial);
				SAFE_DELETE(this->m_pMaterial);
			}

			// start new material
			this->m_pMaterial=new PNPTC_MATERIAL;
			if (this->m_pMaterial)
			{
				// read material name
				return _3DSReadChars(*this,this->m_pMaterial->strName);
			}
			else
			{
				// not loading a material
				return false;
			}
		}
		break;
	case MAT_AMBIENT:
		{
			if (this->m_pMaterial)
			{
				// read material ambient color
				this->m_pRGB=&this->m_pMaterial->colAmbient;
				bool bReturn=this->ProcessSubChunks(header);
				this->m_pRGB=NULL;
				return bReturn;
			}
			else
			{
				// not loading a material
				return false;
			}
		}
		break;
	case MAT_DIFFUSE:
		{
			if (this->m_pMaterial)
			{
				// read material diffuse color
				this->m_pRGB=&this->m_pMaterial->colDiffuse;
				bool bReturn=this->ProcessSubChunks(header);
				this->m_pRGB=NULL;
				return bReturn;
			}
			else
			{
				// not loading a material
				return false;
			}
		}
		break;
	case COL_TRU:
		{
			if (this->m_pRGB)
			{
				// read color
				return _3DSReadRGB(*this,*this->m_pRGB);
			}
			else
			{
				// not loading a color
				return false;
			}
		}
		break;
	case TEX_TEXTURE1:
		{
			if (this->m_pMaterial)
			{
				// read material texture
				this->m_pMaterial->iTextures=1;
				this->m_pstrTextureName=&this->m_pMaterial->strTexture[0];
				bool bReturn=this->ProcessSubChunks(header);
				this->m_pstrTextureName=NULL;

				// does the texture support alpha or has the material alpha color
				if (TextureSupportsAlpha(this->m_pMaterial->strTexture[0]))
				{
					this->m_pMaterial->bAlphaBlend=true;
				}
				else if (this->m_pMaterial->colDiffuse.a<255 || this->m_pMaterial->colAmbient.a<255)
				{
					this->m_pMaterial->bAlphaBlend=true;
				}
				else
				{
					this->m_pMaterial->bAlphaBlend=false;
				}

				return bReturn;
			}
			else
			{
				// not loading a material
				return false;
			}
		}
		break;
	case TEX_TEXTURE2:
		{
			if (this->m_pMaterial)
			{
				// read material texture
				this->m_pMaterial->iTextures=2;
				this->m_pstrTextureName=&this->m_pMaterial->strTexture[1];
				bool bReturn=this->ProcessSubChunks(header);
				this->m_pstrTextureName=NULL;
				return bReturn;
			}
			else
			{
				// not loading a material
				return false;
			}
		}
		break;
	case TEX_FILENAME:
		{
			if (this->m_pstrTextureName)
			{
				// read texture name
				return _3DSReadChars(*this,*this->m_pstrTextureName);
			}
			else
			{
				// not loading a texture
				return false;
			}
		}
		break;
	case EDIT_OBJECT:
		{
			// object chunk
			if (this->m_bObjectHeader) 
			{
				// header already read => error
				return false;
			}
			else if (!this->m_bEditHeader)
			{
				// edit header not read
				return false;
			}
			else
			{
				// set header as beeing read and process sub chunks
				this->m_bObjectHeader=true;
				bool bReturn=_3DSReadChars(*this,this->m_strObjectName);
				bReturn=this->ProcessSubChunks(header) && bReturn;
				this->m_bObjectHeader=false;
				return bReturn;
			}
		}
		break;
	case OBJ_TRIMESH:
		{
			// check object chunk
			if (!this->m_bObjectHeader) 
			{
				// object header not read => error
				return false;
			}

			if (!this->m_pMesh)
			{
				// create new mesh
				this->m_pMesh=new C3DMesh(this->m_pObject,false,false,0);
				if (this->m_pMesh)
				{
					// process this mesh
					this->m_pMesh->SetName(this->m_strObjectName);
					if (this->ProcessSubChunks(header))
					{
						// add mesh
						this->m_pObject->AddMesh(this->m_pMesh);
						this->m_pMesh=NULL;
						return true;
					}
					else
					{
						// error reading mesh
						SAFE_DELETE(this->m_pMesh);
						return false;
					}
				}
				else
				{
					// error creating mesh
					return false;
				}
			}
			else
			{
				// already processing a mesh => error
				return false;
			}
		}
		break;
	case TRI_LOCAL:
		{
			// check if we have a mesh
/*			if (!this->m_pMesh)
			{
				return false;
			}

			// read transformation matrix
			D3DXMATRIX matLocalTransform;
			if (!_3DSReadFloat(*this,matLocalTransform._11) ||
				!_3DSReadFloat(*this,matLocalTransform._12) ||
				!_3DSReadFloat(*this,matLocalTransform._13) ||
				!_3DSReadFloat(*this,matLocalTransform._21) ||
				!_3DSReadFloat(*this,matLocalTransform._22) ||
				!_3DSReadFloat(*this,matLocalTransform._23) ||
				!_3DSReadFloat(*this,matLocalTransform._31) ||
				!_3DSReadFloat(*this,matLocalTransform._32) ||
				!_3DSReadFloat(*this,matLocalTransform._33) ||
				!_3DSReadFloat(*this,matLocalTransform._41) ||
				!_3DSReadFloat(*this,matLocalTransform._42) ||
				!_3DSReadFloat(*this,matLocalTransform._43))
/*			if (!_3DSReadFloat(*this,matLocalTransform._11) ||
				!_3DSReadFloat(*this,matLocalTransform._21) ||
				!_3DSReadFloat(*this,matLocalTransform._31) ||
				!_3DSReadFloat(*this,matLocalTransform._12) ||
				!_3DSReadFloat(*this,matLocalTransform._22) ||
				!_3DSReadFloat(*this,matLocalTransform._32) ||
				!_3DSReadFloat(*this,matLocalTransform._13) ||
				!_3DSReadFloat(*this,matLocalTransform._23) ||
				!_3DSReadFloat(*this,matLocalTransform._33) ||
				!_3DSReadFloat(*this,matLocalTransform._41) ||
				!_3DSReadFloat(*this,matLocalTransform._42) ||
				!_3DSReadFloat(*this,matLocalTransform._43))* /
			{
				return false;
			}



			matLocalTransform._14=0;
			matLocalTransform._24=0;
			matLocalTransform._34=0;
			matLocalTransform._44=1;
/*matLocalTransform._11=1;
matLocalTransform._12=0;
matLocalTransform._13=0;
matLocalTransform._21=0;
matLocalTransform._22=1;
matLocalTransform._23=0;
matLocalTransform._31=0;
matLocalTransform._32=0;
matLocalTransform._33=1;* /
matLocalTransform._41*=0.001;
matLocalTransform._42*=0.001;
matLocalTransform._43*=0.001;* /
D3DXMatrixIdentity(&matLocalTransform);
			D3DXMATRIX matLocalTransforminverse;
			D3DXMatrixInverse(&matLocalTransforminverse,NULL,&matLocalTransform);

			// set the transformation matrix
			this->m_pMesh->SetLocalTransform(matLocalTransform);*/

			// ok
			return true;
		}
		break;
	case TRI_VERTEXL:
		{
			// check if we have a mesh
			if (!this->m_pMesh)
			{
				return false;
			}

			// read number of vertices
			WORD wNumVertices=0;
			if (!_3DSReadWORD(*this,wNumVertices))
			{
				return false;
			}

			// store the number of vertices
			this->m_pMesh->SetNumVertices(wNumVertices);

			// read all vertices
			float fVert[3];
			for (DWORD dwVertex=0;dwVertex<static_cast<DWORD>(wNumVertices);dwVertex++)
			{
				// read one vertex					
				if (!_3DSReadFloat(*this,fVert[0]) || !_3DSReadFloat(*this,fVert[1]) || !_3DSReadFloat(*this,fVert[2]))
				{
					// error reading vertex
					return false;
				}

				// set vertex position
				this->m_pMesh->SetVertexPos(dwVertex,fVert);
			}

			// ok
			return true;
		}
		break;
	case TRI_MAPPING:
		{
			// check if we have a mesh
			if (!this->m_pMesh)
			{
				return false;
			}

			// read number of vertices
			WORD wNumVertices=0;
			if (!_3DSReadWORD(*this,wNumVertices))
			{
				return false;
			}

			// set number of uv coordinate pairs
			this->m_pMesh->SetNumMappingCoords(1);

			// read all vertices
			float fVert[2];
			for (DWORD dwVertex=0;dwVertex<static_cast<DWORD>(wNumVertices);dwVertex++)
			{
				// read one vertex					
				if (!_3DSReadFloat(*this,fVert[0]) || !_3DSReadFloat(*this,fVert[1]))
				{
					// error reading vertex
					return false;
				}

				// set vertex position
				this->m_pMesh->SetVertexUV(dwVertex,0,fVert);
			}

			// ok
			return true;
		}
		break;
	case TRI_FACEL1:
		{
			// check if we have a mesh
			if (!this->m_pMesh)
			{
				return false;
			}

			// read number of faces
			WORD wNumFaces=0;
			if (!_3DSReadWORD(*this,wNumFaces))
			{
				return false;
			}

			// read all faces into a temporary list
			this->m_FaceList.clear();
			this->m_FaceList.resize(wNumFaces);
			WORD wFace[4];
			for (DWORD dwFace=0;dwFace<static_cast<DWORD>(wNumFaces);dwFace++)
			{
				// read face
				if (!_3DSReadWORD(*this,wFace[0]) || !_3DSReadWORD(*this,wFace[1]) || !_3DSReadWORD(*this,wFace[2]) || !_3DSReadWORD(*this,wFace[3]))
				{
					this->m_FaceList.clear();
					return false;
				}

				// set up face
				this->m_FaceList[dwFace].Vertices.dwVert[0]=wFace[0];
				this->m_FaceList[dwFace].Vertices.dwVert[1]=wFace[1];
				this->m_FaceList[dwFace].Vertices.dwVert[2]=wFace[2];
				this->m_FaceList[dwFace].iMaterial=-1;
			}

			// process face materials
			if (this->ProcessSubChunks(header))
			{
				// create face sets => start with sorting the faces by their material
				std::sort(this->m_FaceList.begin(),this->m_FaceList.end(),MaterialSort);

				// create a face set for every material index
				while (this->m_FaceList.size()>0)
				{
					// read current material ID
					int iMaterial=this->m_FaceList[0].iMaterial;

					// read number of faces with this material ID
					DWORD dwFaces=0;
					for (dwFaces=0;dwFaces<static_cast<DWORD>(this->m_FaceList.size());dwFaces++)
					{
						if (this->m_FaceList[dwFaces].iMaterial!=iMaterial)
						{
							break;
						}
					}

					// create face set for this material
					C3DFaceSet* pFaceSet=new C3DFaceSet(this->m_pMesh,iMaterial);
					if (pFaceSet)
					{
						// create memory
						pFaceSet->SetNumFaces(dwFaces);

						// copy all faces
						for (DWORD dwFace=0;dwFace<dwFaces;dwFace++)
						{
							pFaceSet->SetFace(dwFace,this->m_FaceList[dwFace].Vertices);
						}

						// add face set to mesh
						this->m_pMesh->AddFaceSet(pFaceSet);

						// remove those faces from the temporary list
						this->m_FaceList.erase(this->m_FaceList.begin(),this->m_FaceList.begin()+dwFaces);
					}
					else
					{
						// error creating face set
						this->m_FaceList.clear();
						return false;
					}
				}

				// ok
				this->m_FaceList.clear();
				return true;
			}
			else
			{
				// error reading materials
				this->m_FaceList.clear();
				return false;
			}
		}
		break;
	case TRI_MATERIAL:
		{
			// check if we have a face list
			if (this->m_FaceList.size()==0)
			{
				return false;
			}

			// read material name
			std::string strMaterial;
			int iMaterial=-1;
			if (_3DSReadChars(*this,strMaterial))
			{
				// get material index
				iMaterial=this->m_pObject->GetMaterialIndex(strMaterial);

				// read number of faces for this material
				WORD wNumFaces=0;
				if (_3DSReadWORD(*this,wNumFaces))
				{
					// read all faces
					for (DWORD dwFace=0;dwFace<static_cast<DWORD>(wNumFaces);dwFace++)
					{
						WORD wFaceIndex;
						if (_3DSReadWORD(*this,wFaceIndex))
						{
							// set this face index
							if (wFaceIndex<this->m_FaceList.size())
							{
								this->m_FaceList[wFaceIndex].iMaterial=iMaterial;
							}
						}
						else
						{
							// error reading face index
							return false;
						}
					}

					// ok
					return true;
				}
				else
				{
					// error reading number of faces
					return false;
				}
			}
			else
			{
				// error reading material name
				return false;
			}
		}
		break;
	default:
		// unrecognized chunk => continue
		return true;
		break;
	}

	// some error (shouldn't get here)
	return false;
}

CChunkHeader* C3DObjectSerializer3DS::CreateChunk(unsigned long nFilePos) const
{
	return new CChunkHeader3DS(nFilePos);
}






// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------





bool C3DObjectSerializer3DS::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// -----------------------------------------------------------------------------
	// create file
	if (!this->OpenWrite(strFileName))
	{
		MessageBox(NULL,(std::string("The file ")+strFileName+std::string(" could not be opened for writing.")).c_str(),"Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}


	// -----------------------------------------------------------------------------
	// print file header
	if (!_3DSPrintChunkHeader(*this,MAIN3DS) || !_3DSPrintChunkHeader(*this,EDIT3DS))
	{
		// error
		MessageBox(NULL,"Error writing file header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}


	// -----------------------------------------------------------------------------
	// write all materials

	// print mesh header
	if (!_3DSPrintChunkHeader(*this,EDIT_MATERIAL))
	{
		// error
		MessageBox(NULL,"Error writing material list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write materials
	if (ObjectData.GetNumMaterials()>0)
	{
		for (unsigned long lMat=0;lMat<ObjectData.GetNumMaterials();lMat++)
		{
			const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMat);
			if (pMaterial)
			{
				if (!_3DSPrintMaterial(*this,*pMaterial))
				{
					// error
					MessageBox(NULL,"Error writing material to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}
		}
	}
	else
	{
		// write one default material
		PNPTC_MATERIAL mat;
		mat.strName=MATERIAL_DEFAULT;
		mat.colAmbient.r=255;
		mat.colAmbient.g=255;
		mat.colAmbient.b=255;
		mat.colAmbient.a=255;
		mat.colDiffuse.r=255;
		mat.colDiffuse.g=255;
		mat.colDiffuse.b=255;
		mat.colDiffuse.a=255;
		mat.iTextures=0;
		if (!_3DSPrintMaterial(*this,mat))
		{
			// error
			MessageBox(NULL,"Error writing material to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// finish materials
	if (!_3DSFinishChunk(*this))
	{
		// error
		MessageBox(NULL,"Error finishing material list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// write all meshes of object
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		DWORD dwVertices=(DWORD)pMesh->GetNumVertices();

		// print mesh header
		if (!_3DSPrintChunkHeader(*this,EDIT_OBJECT) || !_3DSPrintChars(*this,pMesh->GetName().c_str()) || !_3DSPrintChunkHeader(*this,OBJ_TRIMESH))
		{
			// error
			MessageBox(NULL,"Error writing mesh header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// -----------------------------------------------------------------------------
		// write vertices
		if (!_3DSPrintChunkHeader(*this,TRI_VERTEXL) || !_3DSPrintWORD(*this,(WORD)pMesh->GetNumVertices()))
		{
			// error
			MessageBox(NULL,"Error writing vertex header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// write all vertices
		for (unsigned long lVertex=0;lVertex<pMesh->GetNumVertices();lVertex++)
		{
			const PNPTC_VERTEX& vert=pMesh->GetVertex(lVertex);
			PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
			GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
			if (!_3DSPrintFloat(*this,vecPos.x) || 
				!_3DSPrintFloat(*this,vecPos.y) ||
				!_3DSPrintFloat(*this,vecPos.z) )
			{
				// error
				MessageBox(NULL,"Error writing vertex to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}

		// finish vertices
		if (!_3DSFinishChunk(*this))
		{
			// error
			MessageBox(NULL,"Error finishing vertex header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// -----------------------------------------------------------------------------
		// write UV coordinates
		if (pMesh->GetNumMappingCoords()>0)
		{
			// write header
			if (!_3DSPrintChunkHeader(*this,TRI_MAPPING) || !_3DSPrintWORD(*this,(WORD)pMesh->GetNumVertices()))
			{
				// error
				MessageBox(NULL,"Error writing mapping coordinates header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write all vertices
			for (unsigned long lVertex=0;lVertex<pMesh->GetNumVertices();lVertex++)
			{
				const PNPTC_VERTEX& vert=pMesh->GetVertex(lVertex);
				if (!_3DSPrintFloat(*this,vert.uv[0].u) || 
					!_3DSPrintFloat(*this,vert.uv[0].v) )
				{
					// error
					MessageBox(NULL,"Error writing mapping coordinates to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// finish vertices
			if (!_3DSFinishChunk(*this))
			{
				// error
				MessageBox(NULL,"Error finishing mapping coordinates header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}

		// -----------------------------------------------------------------------------
		// write faces

		// write header
		if (!_3DSPrintChunkHeader(*this,TRI_FACEL1) || !_3DSPrintWORD(*this,(WORD)pMesh->GetNumFaces()))
		{
			// error
			MessageBox(NULL,"Error writing face list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// write all faces
		for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
		{
			const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
			if (pFaceSet)
			{
				for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++)
				{
					const PNPTC_FACE& face=pFaceSet->GetFace(lFace);
					if (!_3DSPrintWORD(*this,(WORD)face.dwVert[0]) ||
						!_3DSPrintWORD(*this,(WORD)face.dwVert[1]) ||
						!_3DSPrintWORD(*this,(WORD)face.dwVert[2]) ||
						!_3DSPrintWORD(*this,7) )
					{
						// error
						MessageBox(NULL,"Error writing face to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
				}
			}
		}

		// -------------------------------------------------------------------------
		// write all face materials
		unsigned long lFaceIndex=0;
		for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
		{
			const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
			if (pFaceSet)
			{
				// get material name
				std::string strMaterial=MATERIAL_DEFAULT;
				if (pFaceSet->GetMaterial()>=0 && pFaceSet->GetMaterial()<static_cast<int>(ObjectData.GetNumMaterials()))
				{
					const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(pFaceSet->GetMaterial());
					if (pMaterial)
					{
						strMaterial=pMaterial->strName;
					}
				}

				// write header
				if (!_3DSPrintChunkHeader(*this,TRI_MATERIAL) || !_3DSPrintChars(*this,strMaterial.c_str()) || !_3DSPrintWORD(*this,(WORD)pFaceSet->GetNumFaces()))
				{
					// error
					MessageBox(NULL,"Error writing face material list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// write all face materials
				for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++,lFaceIndex++)
				{
					if (!_3DSPrintWORD(*this,(WORD)lFaceIndex) )
					{
						// error
						MessageBox(NULL,"Error writing face material to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
				}

				// finish face materials
				if (!_3DSFinishChunk(*this))
				{
					// error
					MessageBox(NULL,"Error finishing face material list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}
		}

		// -------------------------------------------------------------------------
		// finish faces
		if (!_3DSFinishChunk(*this))
		{
			// error
			MessageBox(NULL,"Error finishing faces list header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// -----------------------------------------------------------------------------
		// finish mesh
		if (!_3DSFinishChunk(*this) || !_3DSFinishChunk(*this))
		{
			// error
			MessageBox(NULL,"Error finishing mesh header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}


	// -----------------------------------------------------------------------------
	// finish file
	if (!_3DSFinishChunk(*this) || !_3DSFinishChunk(*this))
	{
		// error
		MessageBox(NULL,"Error finishing file header to 3ds-file.","Error exporting 3ds object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	this->Close();
	return true;
}



