/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializer3DS.h
// 3d object serialization class for .3ds files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECTSERIALIZER3DS_H_INCLUDED)
#define PNPTC_3DOBJECTSERIALIZER3DS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base class
#include "I3DObjectSerializer.h"
#include "../PnPTCUtils/ChunkedFile.h"


/// 3D Object serialization class for .3ds files.
/// Reads/writes objects/meshes to/from an .3ds file. 
class C3DObjectSerializer3DS : public I3DObjectSerializer, public CChunkedFile
{
public:

	// -----------------------------------------------------------------
	// construction / destruciton
	// -----------------------------------------------------------------

	/// Constructs a new serializer.
	C3DObjectSerializer3DS(void);

	/// Destruction of the serializer.
	~C3DObjectSerializer3DS(void);


	// -----------------------------------------------------------------
	// methods to load/save objects
	// -----------------------------------------------------------------

	/// Loads an object from a file.
	/// @param ObjectData 3D Object data into which to load the file.
	/// @param strFileName Object file (including path) to be loaded.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be loaded successfully.
	bool	Load(C3DObjectData& ObjectData,const std::string& strFileName);

	/// Writes an object to a file
	/// @param ObjectData 3D Object data to be saved to the file.
	/// @param strFileName Object file (including path) to be saved to.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @param bSeparateMeshFiles Whether or not to generate seperate files for the 
	/// meshes of the object.
	/// @param bExportTextures Whether or not the textures of the mesh should be exported.
	/// @param cSourceTexturePath The path, below which the source texture files
	/// of the mesh to be exported are located.
	/// @param pD3DDevice Pointer to the rendering device of the object and its textures
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be saved successfully.
	bool	Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL,bool bSeparateMeshFiles=false,bool bExportTextures=false,const char* cSourceTexturePath=NULL,IDirect3DDevice9* pD3DDevice=NULL);


protected:

	// -----------------------------------------------------
	// chunk handling
	// -----------------------------------------------------

	/// Reads the data from the current chunk.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the chunk data could be read.
	bool			OnReadChunkData(const CChunkHeader& header);

	/// Creates a new chunk header.
	/// @param nFilePos Absolute position of the new chunk to be
	/// created inside the current file.
	/// @return Returns a pointer to the newly created chunk
	/// or <i>NULL</i> in case of an error.
	CChunkHeader*	CreateChunk(unsigned long nFilePos) const;


public:

	// -----------------------------------------------------
	// internal data types required for file loading
	// -----------------------------------------------------

	/// A 3ds face definition.
	typedef struct _3DS_FACE
	{
		PNPTC_FACE	Vertices;		//!< The 3 vertex indices of this face.
		int			iMaterial;		//!< Material index used for this face (-1 means no material).
	} _3DS_FACE;


	/// 3ds face list.
	typedef std::vector<_3DS_FACE> _3DSFaceList;


private:

	// -----------------------------------------------------
	// private member variables for reading the file
	// -----------------------------------------------------

	/// Pointer to the object which will receive the data.
	C3DObjectData*	m_pObject;

	/// Whether or not the file header was found.
	bool			m_bFileHeader;

	/// Whether or not the edit header was found.
	bool			m_bEditHeader;

	/// Whether or not the object header was found.
	bool			m_bObjectHeader;

	/// Whether or not materials are currently processed.
	bool			m_bMaterialHeader;

	/// Currently processed material.
	PNPTC_MATERIAL*	m_pMaterial;

	/// Currently processed color value.
	PNPTC_RGBA*		m_pRGB;

	/// Name of the texture currently processed.
	std::string*	m_pstrTextureName;

	/// Currently processed mesh.
	C3DMesh*		m_pMesh;

	/// Name of the currently loaded object.
	std::string		m_strObjectName;

	/// Currently loaded face list
	_3DSFaceList	m_FaceList;

}; // class C3DObjectSerializer3DS

#endif // defined(PNPTC_3DOBJECTSERIALIZER3DS_H_INCLUDED)

