/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerB3D.cpp
// 3d object serialization class for .b3d files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
//  07. Oct. 2006	: Added b3d loader.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "3DObjectData.h"
#include ".\3dobjectserializerb3d.h"
#include "../pnptcutils/utils.h"
#include <map>


/// B3D chunk header.
typedef CChunkHeaderIDSize2<DWORD,DWORD> CChunkHeaderB3D;


// recognized B3D chunks
const DWORD CHUNK_BB3D = ('B'<<0) + ('B'<<8) + ('3'<<16) + ('D'<<24);
const DWORD CHUNK_TEXS = ('T'<<0) + ('E'<<8) + ('X'<<16) + ('S'<<24);
const DWORD CHUNK_BRUS = ('B'<<0) + ('R'<<8) + ('U'<<16) + ('S'<<24);
const DWORD CHUNK_NODE = ('N'<<0) + ('O'<<8) + ('D'<<16) + ('E'<<24);
const DWORD CHUNK_MESH = ('M'<<0) + ('E'<<8) + ('S'<<16) + ('H'<<24);
const DWORD CHUNK_VRTS = ('V'<<0) + ('R'<<8) + ('T'<<16) + ('S'<<24);
const DWORD CHUNK_TRIS = ('T'<<0) + ('R'<<8) + ('I'<<16) + ('S'<<24);



bool B3DPrintDWORD(CChunkedFile& file,DWORD dwValue)
{
	// print dword
	return file.Write(&dwValue,sizeof(DWORD));
}

bool B3DPrintInt(CChunkedFile& file,int iValue)
{
	// print int
	return file.Write(&iValue,sizeof(int));
}

bool B3DReadInt(CChunkedFile& file,int& iValue)
{
	// read int
	return file.Read(&iValue,sizeof(int));
}

bool B3DPrintFloat(CChunkedFile& file,float fValue)
{
	// print float
	return file.Write(&fValue,sizeof(float));
}

bool B3DReadFloat(CChunkedFile& file,float& fValue)
{
	// read float
	return file.Read(&fValue,sizeof(float));
}

bool B3DPrintChars(CChunkedFile& file,const char* cString)
{
	// get string length
	DWORD dwLength=(DWORD)strlen(cString);

	// print string
	return file.Write(cString,dwLength+1);
}

bool B3DReadChars(CChunkedFile& file,std::string& strRead)
{
	// initialize return string
	strRead="";

	// read until zero
	char cRead;	
	bool bReadOK=false;
	while ((bReadOK=file.Read(&cRead,1)) && cRead!=0)
	{
		strRead+=cRead;
	}

	// return result
	return bReadOK;
}

bool B3DPrintChunkHeader(CChunkedFile& file,const char* cName,DWORD dwSize)
{
	// print chunk header name
	if (!file.Write(cName,4))
	{
		return false;
	}

	// start chunk
	return file.StartChunkWriting();
}

bool B3DFinishChunk(CChunkedFile& file)
{
	// finish chunk
	return file.FinishChunkWriting();
}

bool B3DFinishChunk(CChunkedFile& file,DWORD dwSize)
{
	// finish chunk
	return file.FinishChunkWriting(dwSize);
}




C3DObjectSerializerB3D::C3DObjectSerializerB3D(void)
	: CChunkedFile(-(long)sizeof(DWORD))
	, m_fScale(1)
	, m_pObject(NULL)
	, m_bFileHeader(false)
	, m_pMesh(NULL)
	, m_iMeshMaterial(-1)
{
}

C3DObjectSerializerB3D::~C3DObjectSerializerB3D(void)
{
}




// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------




bool C3DObjectSerializerB3D::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// reset everything
	this->m_pObject=&ObjectData;
	this->m_bFileHeader=false;
	this->m_vecTextures.clear();
	this->m_strMeshName="";
	this->m_pMesh=NULL;
	this->m_iMeshMaterial=-1;

	//	check file name;
	if (strFileName.length()==0)
	{
		return false;
	}

	// initialize chunk file loader and load the 3ds file
	return this->OpenRead(strFileName) && this->ReadFile();
}

bool C3DObjectSerializerB3D::OnReadChunkData(const CChunkHeader& header)
{
	// check prerequisits
	if (!this->m_pObject)
	{
		return false;
	}

	// cast it to a 3DS header
	const CChunkHeaderB3D& headerB3D=static_cast<const CChunkHeaderB3D&>(header);

	// check ID
	switch (headerB3D.GetID())
	{
	case CHUNK_BB3D:
		{
			// main chunk
			if (this->m_bFileHeader) 
			{
				// header already read => error
				return false;
			}
			else
			{
				// set header as beeing read and process sub chunks
				this->m_bFileHeader=true;

				// read file version
				int iVersion=0;
				B3DReadInt(*this,iVersion);

				// read rest of file
				bool bReturn=this->ProcessSubChunks(header);
				this->m_bFileHeader=false;
				return bReturn;
			}
		}
		break;
	case CHUNK_TEXS:
		{
			while (!header.EnfOfChunk(this->GetFilePos()))
			{
				// read name of texture
				std::string strTexture;
				if (!B3DReadChars(*this,strTexture)) return false;

				// create that texture
				TEXTURE tex;
				tex.m_bAlphaMap=false;
				tex.m_strTextureName=strTexture;

				// read additional parameters which are not yet used
				int iFlags, iBlend;
				float fXPos, fYPos, fXScale, fYScale, fRotation;
				if (!B3DReadInt(*this,iFlags))return false;						// flags
				if (!B3DReadInt(*this,iBlend))return false;						// blend
				if (!B3DReadFloat(*this,fXPos))return false;					// x pos
				if (!B3DReadFloat(*this,fYPos))return false;					// y pos
				if (!B3DReadFloat(*this,fXScale))return false;					// x scale
				if (!B3DReadFloat(*this,fYScale))return false;					// y scale
				if (!B3DReadFloat(*this,fRotation))return false;				// rotation

				// test alpha map
				if (iFlags&2) tex.m_bAlphaMap=true;
				if (!tex.m_bAlphaMap)
				{
					tex.m_bAlphaMap=TextureSupportsAlpha(tex.m_strTextureName);
				}

				// add that texture
				this->m_vecTextures.push_back(tex);
			}
			return true;
		}
		break;
	case CHUNK_BRUS:
		{
			// read number of textures per brush
			int iTextures;
			if (!B3DReadInt(*this,iTextures)) return false;

			while (!header.EnfOfChunk(this->GetFilePos()))
			{
				// create new material
				PNPTC_MATERIAL mat;
				
				// read name of material
				if (!B3DReadChars(*this,mat.strName)) return false;

				// read material color
				float fRed,fGreen,fBlue,fAlpha;
				if (!B3DReadFloat(*this,fRed)) return false;
				if (!B3DReadFloat(*this,fGreen)) return false;
				if (!B3DReadFloat(*this,fBlue)) return false;
				if (!B3DReadFloat(*this,fAlpha)) return false;
				mat.colAmbient.r=static_cast<BYTE>(fRed*255.f);
				mat.colAmbient.g=static_cast<BYTE>(fGreen*255.f);
				mat.colAmbient.b=static_cast<BYTE>(fBlue*255.f);
				mat.colAmbient.a=static_cast<BYTE>(fAlpha*255.f);
				mat.colDiffuse.r=static_cast<BYTE>(fRed*255.f);
				mat.colDiffuse.g=static_cast<BYTE>(fGreen*255.f);
				mat.colDiffuse.b=static_cast<BYTE>(fBlue*255.f);
				mat.colDiffuse.a=static_cast<BYTE>(fAlpha*255.f);

				// read additional parameters which are not yet used
				int iDummy;
				float fDummy;
				if (!B3DReadFloat(*this,fDummy)) return false;			// shininess
				if (!B3DReadInt(*this,iDummy)) return false;			// brush blend

				// read material flags/fx
				int iFX;
				if (!B3DReadInt(*this,iFX)) return false;
				if (iFX&2) mat.bUseVertexColors=true;
				if (iFX&32) mat.bAlphaBlend=true;

				// set number of textures
				mat.iTextures=minmax(iTextures,0,3);

				// read all textures of this material
				for (unsigned int nTexture=0;nTexture<static_cast<unsigned int>(iTextures);nTexture++)
				{
					// read texture
					int iTextureID;
					if (!B3DReadInt(*this,iTextureID)) return false;
					if (iTextureID>=0 && nTexture<3 && iTextureID<static_cast<int>(this->m_vecTextures.size()))
					{
						// set texture name
						mat.strTexture[nTexture]=this->m_vecTextures[iTextureID].m_strTextureName;

						// test if texture is alpha mapped
						if (this->m_vecTextures[iTextureID].m_bAlphaMap)
						{
							mat.bAlphaBlend=true;
						}
					}
				}

				// add material
				this->m_pObject->AddMaterial(mat);
			}
			return true;
		}
		break;
	case CHUNK_NODE:
		{
			// read name of this node
			if (!B3DReadChars(*this,this->m_strMeshName)) return false;

			// read additional parameters which are not yet used
			float fDummy;
			if (!B3DReadFloat(*this,fDummy))return false;					// x position
			if (!B3DReadFloat(*this,fDummy))return false;					// y position
			if (!B3DReadFloat(*this,fDummy))return false;					// z position
			if (!B3DReadFloat(*this,fDummy))return false;					// x scale
			if (!B3DReadFloat(*this,fDummy))return false;					// y scale
			if (!B3DReadFloat(*this,fDummy))return false;					// z scale
			if (!B3DReadFloat(*this,fDummy))return false;					// quaternion rotation
			if (!B3DReadFloat(*this,fDummy))return false;					// quaternion rotation
			if (!B3DReadFloat(*this,fDummy))return false;					// quaternion rotation
			if (!B3DReadFloat(*this,fDummy))return false;					// quaternion rotation

			// process sub chunks of the node
			return this->ProcessSubChunks(header);
		}
		break;
	case CHUNK_MESH:
		{
			// create new mesh
			this->m_pMesh=new C3DMesh(this->m_pObject,false,false,0);
			if (!this->m_pMesh) return false;
			this->m_pObject->AddMesh(this->m_pMesh);

			// read default material
			if (!B3DReadInt(*this,this->m_iMeshMaterial)) return false;

			// process all sub chunks of the mesh
			bool bReturn=this->ProcessSubChunks(header);

			// finish this mesh
			this->m_pMesh=NULL;
			return bReturn;
		}
		break;
	case CHUNK_VRTS:
		{
			// is a mesh present
			if (!this->m_pMesh)
			{
				return false;
			}

			// read vertex set parameters
			int iFlags;
			int iUVCoords;
			int iUVSize;
			if (!B3DReadInt(*this,iFlags)) return false;		// flags (1 normals , 2 rgba
			if (!B3DReadInt(*this,iUVCoords)) return false;		// number of uv coordinate pairs
			if (!B3DReadInt(*this,iUVSize)) return false;		// size of one uv coordinate

			// set values of mesh
			this->m_pMesh->SetNumMappingCoords(iUVCoords);
			this->m_pMesh->SetHasVertexNormals((iFlags&1)!=0);
			this->m_pMesh->SetHasVertexColors((iFlags&2)!=0);

			// now read all vertices
			while (!header.EnfOfChunk(this->GetFilePos()))
			{
				// read vertex
				PNPTC_VERTEX vert;

				// read vertex position
				if (!B3DReadFloat(*this,vert.vPos.x)) return false;
				if (!B3DReadFloat(*this,vert.vPos.y)) return false;
				if (!B3DReadFloat(*this,vert.vPos.z)) return false;

				// read normal data
				if (this->m_pMesh->HasVertexNormals())
				{
					if (!B3DReadFloat(*this,vert.vNormal.x)) return false;
					if (!B3DReadFloat(*this,vert.vNormal.y)) return false;
					if (!B3DReadFloat(*this,vert.vNormal.z)) return false;
				}

				// read color data
				if (this->m_pMesh->HasVertexColor())
				{
					float fRed,fGreen,fBlue,fAlpha;
					if (!B3DReadFloat(*this,fRed)) return false;
					if (!B3DReadFloat(*this,fGreen)) return false;
					if (!B3DReadFloat(*this,fBlue)) return false;
					if (!B3DReadFloat(*this,fAlpha)) return false;
					vert.colDiffuse.r=static_cast<BYTE>(fRed*255.f);
					vert.colDiffuse.g=static_cast<BYTE>(fGreen*255.f);
					vert.colDiffuse.b=static_cast<BYTE>(fBlue*255.f);
					vert.colDiffuse.a=static_cast<BYTE>(fAlpha*255.f);
				}

				// read all texture coordinate pairs
				for (int iUV=0;iUV<static_cast<int>(this->m_pMesh->GetNumMappingCoords());iUV++)
				{
					for (int iUVElement=0;iUVElement<iUVSize;iUVElement++)
					{
						// read uv data
						float fUV;
						if (!B3DReadFloat(*this,fUV)) return false;
						if (iUV<3)
						{
							if (iUVElement==0) vert.uv[iUV].u=fUV;
							else if (iUVElement==1) vert.uv[iUV].v=fUV;
						}
					}
				}

				// add vertex
				this->m_pMesh->AddVertex(vert);
			}

			// ok
			return true;
		}
		break;
	case CHUNK_TRIS:
		{
			// is a mesh present
			if (!this->m_pMesh)
			{
				return false;
			}

			// read material
			int iMaterial;
			if (!B3DReadInt(*this,iMaterial)) return false;
			if (iMaterial<0) iMaterial=this->m_iMeshMaterial;
			if (iMaterial<0) iMaterial=-1;
			if (iMaterial>=static_cast<int>(this->m_pObject->GetNumMaterials())) iMaterial=-1;

			// create new face set
			C3DFaceSet* pFaceSet=new C3DFaceSet(this->m_pMesh,iMaterial);
			this->m_pMesh->AddFaceSet(pFaceSet);

			// read all faces
			while (!header.EnfOfChunk(this->GetFilePos()))
			{
				// read face
				PNPTC_FACE face;
				int iVert1,iVert2,iVert3;
				if (!B3DReadInt(*this,iVert1)) return false;
				if (!B3DReadInt(*this,iVert2)) return false;
				if (!B3DReadInt(*this,iVert3)) return false;
				face.dwVert[0]=iVert1;
				face.dwVert[1]=iVert2;
				face.dwVert[2]=iVert3;

				// add face
				pFaceSet->AddFace(face);
			}

			// ok
			return true;
		}
		break;
	default:
		// unrecognized chunk => continue
		return true;
		break;
	}

	// some error (shouldn't get here)
	return false;
}

CChunkHeader* C3DObjectSerializerB3D::CreateChunk(unsigned long nFilePos) const
{
	return new CChunkHeaderB3D(nFilePos);
}


// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------




typedef std::map<std::string,int>	TEXTURE_INDEX;
typedef std::pair<std::string,int>	TEXTURE_INDEX_PAIR;


bool C3DObjectSerializerB3D::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// -----------------------------------------------------------------------------
	// create file
	if (!this->OpenWrite(strFileName))
	{
		MessageBox(NULL,(std::string("The file ")+strFileName+std::string(" could not be opened for writing.")).c_str(),"Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// print file header
	if (!B3DPrintChunkHeader(*this,"BB3D",0))
	{
		// error
		MessageBox(NULL,"Error writing file header to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// print file version
	if (!B3DPrintInt(*this,1))
	{
		// error
		MessageBox(NULL,"Error writing file version to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// write all textures used in the object

	// write texture header
	if (!B3DPrintChunkHeader(*this,"TEXS",0))
	{
		// error
		MessageBox(NULL,"Error writing textures header to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write all textures
	TEXTURE_INDEX Textures;
	int iTextureIndex=0;
	for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)
	{
		// get material
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMaterial);
		if (pMaterial)
		{
			// write all textures used in this material
			for (int iTex=0;iTex<pMaterial->iTextures;iTex++)
			{		
				// try to find this texture file
				TEXTURE_INDEX::iterator itTex=Textures.find(pMaterial->strTexture[iTex]);

				if (itTex==Textures.end())
				{
					// write texture
					if (!B3DPrintChars(*this,pMaterial->strTexture[iTex].c_str()) ||	// file name of texture
						!B3DPrintInt(*this,1) ||										// flags (color)
//						!B3DPrintInt(*this,1+16+32) ||									// flags (color + clamp u + clamp v)
						!B3DPrintInt(*this,2) ||										// blend
						!B3DPrintFloat(*this,0) ||										// x pos
						!B3DPrintFloat(*this,0) ||										// y pos
						!B3DPrintFloat(*this,1) ||										// x scale
						!B3DPrintFloat(*this,1) ||										// y scale
						!B3DPrintFloat(*this,0) )										// rotation
					{
						MessageBox(NULL,"Error writing texture to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// add new texture to list
					Textures.insert(TEXTURE_INDEX_PAIR(pMaterial->strTexture[iTex],iTextureIndex));
					iTextureIndex++;
				}
			}
		}
	}

	// finish textures chunk
	if (!B3DFinishChunk(*this))
	{
		MessageBox(NULL,"Error finishing textures to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// write all textures as brushes used in the object

	// write brush header
	if (!B3DPrintChunkHeader(*this,"BRUS",0))
	{
		// error
		MessageBox(NULL,"Error writing brushes header to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// compute maximum number of textures used
	int iMaxTextures=0;
	for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)
	{
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMaterial);
		if (pMaterial)
		{
			iMaxTextures=max(iMaxTextures,pMaterial->iTextures);
		}
	}

	// write number of textures per brush
	if (!B3DPrintInt(*this,iMaxTextures) )			// texture per brush
	{
		MessageBox(NULL,"Error writing texture per brush to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write all brushes
	for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)
	{
		// get material
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMaterial);
		if (pMaterial)
		{
			// write brush
			if (!B3DPrintChars(*this,pMaterial->strName.c_str()) ||							// material name as brush name
				!B3DPrintFloat(*this,static_cast<float>(pMaterial->colDiffuse.r)/255.f) ||	// red color component
				!B3DPrintFloat(*this,static_cast<float>(pMaterial->colDiffuse.g)/255.f) ||	// green color component
				!B3DPrintFloat(*this,static_cast<float>(pMaterial->colDiffuse.b)/255.f) ||	// blue color component
				!B3DPrintFloat(*this,static_cast<float>(pMaterial->colDiffuse.a)/255.f) ||	// alpha color component
				!B3DPrintFloat(*this,0) ||													// shininess
				!B3DPrintInt(*this,0) ||													// brush blend
				!B3DPrintInt(*this,2+((pMaterial->iTextures>0)?32:0)) )						// brush fx (use vertex colors + 32?)
			{
				MessageBox(NULL,"Error writing brush to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write all textures of brush
			for (int iTex=0;iTex<iMaxTextures;iTex++)
			{		
				// get texture index
				int iTexture=-1;
				if (iTex<pMaterial->iTextures)
				{
					// try to find index of this texture
					TEXTURE_INDEX::iterator itTex=Textures.find(pMaterial->strTexture[iTex]);
					if (itTex!=Textures.end())
					{
						iTexture=itTex->second;
					}
				}

				// write texture
				if (!B3DPrintInt(*this,iTexture) )						// texture index
				{
					MessageBox(NULL,"Error writing brush texture index to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}
		}
	}

	// finish brush chunk
	if (!B3DFinishChunk(*this))
	{
		MessageBox(NULL,"Error finishing brushes to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// write all meshes of object
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);

		// write node
		if (!B3DPrintChunkHeader(*this,"NODE",0) ||						// node header
			!B3DPrintChars(*this,pMesh->GetName().c_str()) ||			// name of mesh
			!B3DPrintFloat(*this,0) ||									// x position
			!B3DPrintFloat(*this,0) ||									// y position
			!B3DPrintFloat(*this,0) ||									// z position
			!B3DPrintFloat(*this,this->m_fScale) ||						// x scale
			!B3DPrintFloat(*this,this->m_fScale) ||						// y scale
			!B3DPrintFloat(*this,this->m_fScale) ||						// z scale
			!B3DPrintFloat(*this,1) ||									// quaternion rotation
			!B3DPrintFloat(*this,0) ||									// quaternion rotation
			!B3DPrintFloat(*this,0) ||									// quaternion rotation
			!B3DPrintFloat(*this,0) )									// quaternion rotation
		{
			MessageBox(NULL,"Error writing mesh node to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// write mesh
		if (!B3DPrintChunkHeader(*this,"MESH",0) ||		// mesh header
			!B3DPrintInt(*this,-1) )					// material
		{
			MessageBox(NULL,"Error writing mesh to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// ----------------------------------------------------------------------
		// write vertices
		int iFlags=0;
		if (pMesh->HasVertexNormals()) iFlags=iFlags|1;
		if (pMesh->HasVertexColor()) iFlags=iFlags|2;
		if (!B3DPrintChunkHeader(*this,"VRTS",0) ||									// vertex header
			!B3DPrintInt(*this,iFlags) ||											// flags (1 normals , 2 rgba
			!B3DPrintInt(*this,static_cast<int>(pMesh->GetNumMappingCoords())) ||	// number of uv coordinate pairs
			!B3DPrintInt(*this,2) )													// size of one uv coordinate
		{
			MessageBox(NULL,"Error writing vertex header to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// write all vertices
		for (unsigned long lVert=0;lVert<pMesh->GetNumVertices();lVert++)
		{
			// write position data
			const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
			PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
			GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
			if (!B3DPrintFloat(*this,vecPos.x) ||			// x position of vertex
				!B3DPrintFloat(*this,vecPos.y) ||			// y position of vertex
				!B3DPrintFloat(*this,vecPos.z) )			// z position of vertex
			{
				MessageBox(NULL,"Error writing vertex position to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write normal data
			if (pMesh->HasVertexNormals())
			{
				PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
				GlobalVectorTransform(vecPos,1,iUpRotation,iFlipAxis);
				if (!B3DPrintFloat(*this,vecNormal.x) ||	// normal of vertex
					!B3DPrintFloat(*this,vecNormal.y) ||	// normal of vertex
					!B3DPrintFloat(*this,vecNormal.z) )		// normal of vertex
				{
					MessageBox(NULL,"Error writing vertex normal to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// write color data
			if (pMesh->HasVertexColor())
			{
				if (!B3DPrintFloat(*this,static_cast<float>(vert.colDiffuse.r)/255.f) ||	// red color component of vertex
					!B3DPrintFloat(*this,static_cast<float>(vert.colDiffuse.g)/255.f) ||	// green color component of vertex
					!B3DPrintFloat(*this,static_cast<float>(vert.colDiffuse.b)/255.f) ||	// blue color component of vertex
					!B3DPrintFloat(*this,static_cast<float>(vert.colDiffuse.a)/255.f) )	// alpha color component of vertex
				{
					MessageBox(NULL,"Error writing vertex color to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// write all texture coordinate pairs
			for (int iUV=0;iUV<static_cast<int>(pMesh->GetNumMappingCoords());iUV++)
			{
				if (!B3DPrintFloat(*this,vert.uv[iUV].u) ||	// u texture coordinate
					!B3DPrintFloat(*this,vert.uv[iUV].v) )	// v texture coordinate
				{
					MessageBox(NULL,"Error writing vertex uv to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}
		}

		// finish vertex chunk
		if (!B3DFinishChunk(*this))
		{
			MessageBox(NULL,"Error finishing vertex to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// ----------------------------------------------------------------------
		// write face sets
		for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
		{
			// get face set
			const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);

			// start face set
			if (!B3DPrintChunkHeader(*this,"TRIS",0) ||				// triangles header
				!B3DPrintInt(*this,pFaceSet->GetMaterial()) )		// material index of face set
			{
				MessageBox(NULL,"Error writing triangles header to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write all triangles
			for (unsigned long lTri=0;lTri<pFaceSet->GetNumFaces();lTri++)
			{
				// write triangle
				const PNPTC_FACE& face=pFaceSet->GetFace(lTri);

				if (!B3DPrintInt(*this,face.dwVert[0]) ||	// first vertex of triangle
					!B3DPrintInt(*this,face.dwVert[1]) ||	// second vertex of triangle
					!B3DPrintInt(*this,face.dwVert[2]) )	// third vertex of triangle
				{
					MessageBox(NULL,"Error writing triangle to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// finish tris chunk
			if (!B3DFinishChunk(*this))
			{
				MessageBox(NULL,"Error finishing triangles to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}

		// ----------------------------------------------------------------------
		// finish mesh chunk
		if (!B3DFinishChunk(*this))
		{
			MessageBox(NULL,"Error finishing mesh to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// finish node chunk
		if (!B3DFinishChunk(*this))
		{
			MessageBox(NULL,"Error finishing mesh node to b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}


	// -----------------------------------------------------------------------------
	// finish file
	if (!B3DFinishChunk(*this))
	{
		// error
		MessageBox(NULL,"Error finishing b3d-file.","Error exporting b3d object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	this->Close();
	return true;
}
