/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerDAE.cpp
// 3d object serialization class for .dae (collada) files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	01. Jul. 2007	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////


#define _CRT_SECURE_NO_WARNINGS

#include "3DObjectData.h"
#include ".\3dobjectserializerdae.h"
#include "../PnPTCUtils/Utils.h"
#include "../PnPTCUtils/XmlUtils.h"
#include "../3rd-party/tinyxml/tinyxml.h"
#include <map>



C3DObjectSerializerDAE::C3DObjectSerializerDAE(void)
{
}

C3DObjectSerializerDAE::~C3DObjectSerializerDAE(void)
{
}



// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------

/// Compares to dae ID strings with and without adding a # prefix.
/// @param str1 The first string to be compared.
/// @param str1 The first string to be compared.
/// @return Returns <i>true</i> if the two strings describe the same
/// dae ID, otherwise <i>false</i>.
bool DaeIdCompare(const std::string& str1,const std::string& str2)
{
	// directly compare
	if (str1==str2) return true;

	// compare with local reference symbol
	if ("#"+str1==str2) return true;
	if (str1=="#"+str2) return true;

	// not the same
	return false;
}

/// Remove the instance identifier from the name of a mesh
/// @param strMeshName The name of the mesh, from which the instance
/// string should be removed.
/// @return Returns the name of the mesh without the instance identifier.
std::string RemoveInstanceString(const std::string& strMeshName)
{
	// test if the instance strings should be removed
	static char cTemp[10];
	DWORD dwRead=::GetEnvironmentVariable("S2_ExportRemoveInstances",cTemp,9);
	if (dwRead==0 || dwRead>=sizeof(cTemp) || (atoi(cTemp)!=1))
	{
		// don't remove
		return strMeshName;
	}

	// instance identifier
	const std::string strInst="_inst";

	// check mesh name
	if (strMeshName.length()>strInst.length())
	{
		// try to find "_inst"
		std::string strWork=strMeshName;
		ToLower(strWork);
		size_t nPos=strWork.rfind(strInst);
		if (nPos==std::string::npos)
		{
			// not found
			return strMeshName;
		}
		else
		{
			if (strWork.length()>nPos+strInst.length())
			{
				// test if next character is a number
				char cNext=strWork.at(nPos+strInst.length());
				if ((cNext>='1' && cNext<='9') || cNext=='0')
				{
					// is a digit => is there another underscore?
					if (strWork.find("_",nPos+1)!=std::string::npos)
					{
						// yes
						return strMeshName;
					}
					else
					{
						// no further underscore => so this is an instance identifier => remove it
						return Left(strMeshName,static_cast<unsigned int>(nPos));
					}
				}
				else
				{
					// not a digit
					return strMeshName;
				}
			}
			else
			{
				// wrong format => return whole name
				return strMeshName;
			}
		}
	}
	else
	{
		// no suffix => return whole string
		return strMeshName;
	}
}



/// image definition of a texture stored in the collada file
typedef struct ImageDef
{
	std::string		strID;			//!< the ID of the image.
	std::string		strFileName;	//!< name of the texture file.
} ImageDef;

/// List of all image definitions.
typedef std::vector<ImageDef>	ImageList;

/// Texture sampler definition as stored in the collada file
typedef struct SamplerDef
{
	std::string		strID;			//!< the ID of the sampler.
	std::string		strImage;		//!< the image, which is used in this sampler.
} SamplerDef;

/// List of defined samplers
typedef std::vector<SamplerDef>	SamplerList;

/// Reads a color value from a COLLADA node
void ReadColor(TiXmlElement* pNode,PNPTC_RGBA& rgba)
{
	// read color value	
	if (pNode)
	{
		TiXmlElement* pColorNode=pNode->FirstChildElement("color");
		if (pColorNode)
		{
			float fColor[4]={0,0,0,0};
			std::string strText=pColorNode->GetText();
			sscanf(strText.c_str(),"%f %f %f %f",&fColor[0],&fColor[1],&fColor[2],&fColor[3]);
			rgba.r=static_cast<unsigned char>(fColor[0]*255.f);
			rgba.g=static_cast<unsigned char>(fColor[1]*255.f);
			rgba.b=static_cast<unsigned char>(fColor[2]*255.f);
			rgba.a=static_cast<unsigned char>(fColor[3]*255.f);
		}
	}
}

/// Reads the texture information from a COLLADA node
void ReadTexture(TiXmlElement* pNode,ImageList& images,SamplerList& samplers,PNPTC_MATERIAL& material)
{
	// read texture
	if (pNode)
	{
		TiXmlElement* pTextureNode=pNode->FirstChildElement("texture");
		if (pTextureNode)
		{
			// get sampler of that texture
			std::string strSampler=GetXmlAttributeString(pTextureNode,"texture");
			for (SamplerList::iterator itSampler=samplers.begin();itSampler!=samplers.end();++itSampler)
			{
				if (DaeIdCompare(itSampler->strID,strSampler))
				{
					// get the image of this sampler
					for (ImageList::iterator itImage=images.begin();itImage!=images.end();++itImage)
					{
						if (DaeIdCompare(itImage->strID,itSampler->strImage))
						{
							// this is the one, we are searching for
							std::string strPath;
							RemoveFilePath(itImage->strFileName,material.strTexture[material.iTextures],strPath);
							material.iTextures++;

							// stop here
							break;
						}
					}

					// stop here
					break;
				}
			}
		}
	}
}

/// A material (effect) definition stored in the collada file
typedef struct MaterialDef
{
	std::string		strID;			//!< the ID of the material.
	TiXmlElement*	pXMLNode;		//!< the XML node, where the material is stored.
	int				iMaterialIndex;	//!< the instantiated material index.

	/// construction / initialization
	MaterialDef()
		: pXMLNode(NULL)
		, iMaterialIndex(-1)
	{};

	/// Instantiates the material.
	/// @param ObjectData The object data object, in which the material
	/// should be instantiated.
	/// @return Returns the material index.
	int InstantiateMaterial(C3DObjectData& ObjectData,ImageList& images)
	{
		// already instantiated?
		if (this->iMaterialIndex>=0 || !this->pXMLNode)
		{
			return this->iMaterialIndex;
		}

		// initialize material
		PNPTC_MATERIAL mat;
		TiXmlElement* pProfileNode=this->pXMLNode->FirstChildElement("profile_COMMON");
		if (pProfileNode)
		{
			// read all samplers and surfaces
			SamplerList vecSamplers;
			TiXmlElement* pSamplerParamNode=pProfileNode->FirstChildElement("newparam");
			while (pSamplerParamNode)
			{
				// get texture sampler
				TiXmlElement* pSampler2DNode=pSamplerParamNode->FirstChildElement("sampler2D");
				if (pSampler2DNode)
				{
					// read sampler
					SamplerDef sampler;
					sampler.strID=GetXmlAttributeString(pSamplerParamNode,"sid");

					// get source
					std::string strSurface;
					TiXmlElement* pSourceElement=pSampler2DNode->FirstChildElement("source");
					if (pSourceElement)
					{
						// get surface name
						strSurface=pSourceElement->GetText();

						// search for this surface
						TiXmlElement* pSurfaceParamNode=pProfileNode->FirstChildElement("newparam");
						while (pSurfaceParamNode)
						{
							// is this the surface, we are searching for
							std::string strTest=GetXmlAttributeString(pSurfaceParamNode,"sid");
							if (DaeIdCompare(strTest,strSurface))
							{
								// yes, this is the surface
								TiXmlElement* pSurfaceNode=pSurfaceParamNode->FirstChildElement("surface");
								if (pSurfaceNode)
								{
									TiXmlElement* pInitFromNode=pSurfaceNode->FirstChildElement("init_from");
									if (pInitFromNode)
									{
										sampler.strImage=pInitFromNode->GetText();
									}
								}

								// add sampler
								vecSamplers.push_back(sampler);

								// stop here
								break;
							}

							// get next surface
							pSurfaceParamNode=pSurfaceParamNode->NextSiblingElement("newparam");
						}
					}
				}

				// get next sampler
				pSamplerParamNode=pSamplerParamNode->NextSiblingElement("newparam");
			}

			// get technique
			TiXmlElement* pTechniqueNode=pProfileNode->FirstChildElement("technique");
			if (pTechniqueNode)
			{
				// search for the illumination mode
				TiXmlElement* pIlluminationModeNode=pTechniqueNode->FirstChildElement("blinn");
				if (!pIlluminationModeNode)
				{
					pIlluminationModeNode=pTechniqueNode->FirstChildElement("phong");
				}
				if (!pIlluminationModeNode)
				{
					pIlluminationModeNode=pTechniqueNode->FirstChildElement("lambert");
				}
				if (pIlluminationModeNode)
				{
					// get material info
					TiXmlElement* pAmbientNode=pIlluminationModeNode->FirstChildElement("ambient");
					if (pAmbientNode)
					{
						ReadColor(pAmbientNode,mat.colAmbient);
					}
					TiXmlElement* pDiffuseNode=pIlluminationModeNode->FirstChildElement("diffuse");
					if (pDiffuseNode)
					{
						ReadColor(pDiffuseNode,mat.colDiffuse);
						ReadTexture(pDiffuseNode,images,vecSamplers,mat);
					}
					mat.bUseVertexColors=false;
					mat.strName=this->strID;

					// does the texture support alpha or has the material alpha color
					if (TextureSupportsAlpha(mat.strTexture[0]))
					{
						mat.bAlphaBlend=true;
					}
					else if (mat.colDiffuse.a<255 || mat.colAmbient.a<255)
					{
						mat.bAlphaBlend=true;
					}
					else
					{
						mat.bAlphaBlend=false;
					}

					// add material
					this->iMaterialIndex=ObjectData.AddMaterial(mat);
				}
			}
		}
		
		// return material index
		return this->iMaterialIndex;
	}

} MaterialDef;

/// List of all materials
typedef std::vector<MaterialDef>	MaterialList;

/// Reads a float array from a string
bool ReadStringArray(const std::string& strSource,unsigned int nElements,std::vector<float>& vecDest)
{
	// go through whole string
	unsigned int nPos=0;
	for (unsigned int nRead=0;nRead<nElements;nRead++)
	{
		// store last pos
		unsigned int nLastPos=nPos;

		// get string of the current value
		if (nRead<(nElements-1))
		{
			nPos=static_cast<unsigned int>(strSource.find_first_of(' ',nLastPos));
			if (nPos==std::string::npos)
			{
				return false;
			}
		}
		else
		{
			nPos=static_cast<unsigned int>(strSource.length());
		}
		std::string strValue=strSource.substr(nLastPos,nPos-nLastPos);
		nPos++;

		// convert to float and add
		float fValue=static_cast<float>(atof(strValue.c_str()));
		vecDest.push_back(fValue);
	}

	// ok
	return true;
}

/// Reads a float array from a string
bool ReadStringArray(const std::string& strSource,std::vector<unsigned int>& vecDest)
{
	// go through whole string
	unsigned int nPos=0;
	while (nPos!=std::string::npos)
	{
		// store last pos
		unsigned int nLastPos=nPos;

		// get next white space and string potion to be converted
		std::string strValue;
		nPos=static_cast<unsigned int>(strSource.find_first_of(' ',nLastPos));
		if (nPos==std::string::npos)
		{
			strValue=strSource.substr(nLastPos);
		}
		else
		{
			strValue=strSource.substr(nLastPos,nPos-nLastPos);
			nPos++;
		}

		// convert to unsigned int and add
		unsigned int nValue=static_cast<unsigned int>(atoi(strValue.c_str()));
		vecDest.push_back(nValue);
	}

	// ok
	return vecDest.size()>0;
}

/// Reads a float array from a string
bool ReadStringArray(const std::string& strSource,unsigned int nElements,std::vector<unsigned int>& vecDest)
{
	// go through whole string
	unsigned int nPos=0;
	for (unsigned int nRead=0;nRead<nElements;nRead++)
	{
		// store last pos
		unsigned int nLastPos=nPos;

		// get string of the current value
		if (nRead<(nElements-1))
		{
			nPos=static_cast<unsigned int>(strSource.find_first_of(' ',nLastPos));
			if (nPos==std::string::npos)
			{
				return false;
			}
		}
		else
		{
			nPos=static_cast<unsigned int>(strSource.length());
		}
		std::string strValue=strSource.substr(nLastPos,nPos-nLastPos);
		nPos++;

		// convert to unsigned int and add
		unsigned int nValue=static_cast<unsigned int>(atoi(strValue.c_str()));
		vecDest.push_back(nValue);
	}

	// ok
	return true;
}

/// an auxiliary class for reading vertex data streams.
class VertexDataStream
{
public:

	/// construction of a new vertex data stream.
	VertexDataStream() {};

	/// destruction of the vertex data stream.
	~VertexDataStream() {};

	/// Loads the vertex data stream from an XML node.
	bool Load(TiXmlElement* pNode)
	{
		// check node
		if (pNode)
		{
			// get name
			this->m_strName=GetXmlAttributeString(pNode,"id");

			// read values
			TiXmlElement* pFloatArrayNode=pNode->FirstChildElement("float_array");
			if (pFloatArrayNode)
			{
				// set size of array
				unsigned int nNumElements=GetXmlAttributeInt(pFloatArrayNode,"count");
				this->m_vecData.reserve(nNumElements);
				ReadStringArray(pFloatArrayNode->GetText(),nNumElements,this->m_vecData);
			}

			// read number of elements and stride
			TiXmlElement* pTechniqueNode=pNode->FirstChildElement("technique_common");
			if (pTechniqueNode)
			{
				TiXmlElement* pAccessorNode=pTechniqueNode->FirstChildElement("accessor");
				if (pAccessorNode)
				{
					this->m_nElements=GetXmlAttributeInt(pAccessorNode,"count");
					this->m_nStride=GetXmlAttributeInt(pAccessorNode,"stride");
				}
			}

			// ok
			return true;
		}

		// error
		return false;
	};

	/// Returns the name of the stream
	/// @return The streams name.
	std::string			GetName() const {return this->m_strName;};

	/// Returns the number of elements in the stream.
	/// @return The overall number of elements in the stream.
	unsigned int		GetNumElements() const {return this->m_nElements;};

	/// Returns the number of subelements per element.
	/// @return Returns the number subelements of an element.
	unsigned int		GetNumSubElements() const {return this->m_nStride;};

	/// Returns the value of one element.
	/// @param nElement The element index.
	/// @param nSubElement The sub element index.
	/// @return Returns the value of that element.
	float				GetElement(unsigned int nElement,unsigned int nSubElement) const
	{
		if (nElement<this->m_nElements && nSubElement<this->m_nStride)
		{
			// get element index
			unsigned int nIndex=nElement*this->m_nStride+nSubElement;
			if (nIndex<this->m_vecData.size())
			{
				// return that element
				return this->m_vecData[nIndex];
			}
		}

		// index out of range
		return 0;
	};

	/// Sets the usage of this stream.
	/// @param strUsage Stream usage string.
	void				SetUsage(const std::string& strUsage) {this->m_strUsage=strUsage;};

	/// Returns the usage of this stream.
	/// @return The stream usage string.
	std::string			GetUsage() const {return this->m_strUsage;};


private:

	/// The data array read from the xml node.
	std::vector<float>	m_vecData;

	/// The name of the stream
	std::string			m_strName;

	/// Number of elements.
	unsigned int		m_nElements;

	/// The stride between two subsequent elements.
	unsigned int		m_nStride;

	/// The usage of this stream
	std::string			m_strUsage;

};	// class VertexDataStream

/// List of available vertex data stream
typedef std::vector<VertexDataStream>	VertexStreamList;


/// Structure for material binding definition
typedef struct BindMaterial
{
	std::string	strSymbol;	//!< the material symbol used in the geometry definition.
	std::string	strTarget;	//!< the ID of the material to be used for it.
} BindMaterial;


/// List of material bindings
typedef std::vector<BindMaterial>	BindMaterialList;


/// Fills a list of material bindings for a given geometry instance
void FillMaterialBindings(TiXmlElement* pInstanceGeometryNode,BindMaterialList& vecBindMaterials)
{
	// check parameters
	if (pInstanceGeometryNode)
	{
		// search for bind material sub node
		TiXmlElement* pBindMaterialNode=pInstanceGeometryNode->FirstChildElement("bind_material");
		if (pBindMaterialNode)
		{
			// search for common technique sub node
			TiXmlElement* pTechniqueCommonNode=pBindMaterialNode->FirstChildElement("technique_common");
			if (pTechniqueCommonNode)
			{
				// go through all material instances
				TiXmlElement* pInstanceMaterialNode=pTechniqueCommonNode->FirstChildElement("instance_material");
				while (pInstanceMaterialNode)
				{
					// create new binding data
					BindMaterial bm;
					bm.strSymbol=GetXmlAttributeString(pInstanceMaterialNode,"symbol");
					bm.strTarget=GetXmlAttributeString(pInstanceMaterialNode,"target");
					if (!bm.strTarget.empty() && !bm.strSymbol.empty())
					{
						vecBindMaterials.push_back(bm);
					}

					// next material instance
					pInstanceMaterialNode=pInstanceMaterialNode->NextSiblingElement(pInstanceMaterialNode->Value());
				}
			}
		}
	}
}

/// Returns the material binding for a given symbol
std::string GetMaterialBinding(const std::string& strSymbol,const BindMaterialList& vecBindMaterials)
{
	// go through all bindings
	for (BindMaterialList::const_iterator itBinding=vecBindMaterials.begin();itBinding!=vecBindMaterials.end();++itBinding)
	{
		if (DaeIdCompare(strSymbol,itBinding->strSymbol))
		{
			return itBinding->strTarget;
		}
	}

	// not found
	return strSymbol;
}


/// A geometry definition stored in the collada file
typedef struct GeometryDef
{
	std::string		strID;			//!< the ID of the geometry.
	TiXmlElement*	pXMLNode;		//!< the XML node, where the geometry is stored.
	C3DMesh*		pMesh;			//!< the instantiated mesh.

	/// construction / initialization
	GeometryDef()
		: pXMLNode(NULL)
		, pMesh(NULL)
	{};

	/// Instantiates this geometry as a mesh for a given object data object.
	/// @param ObjectData The object data object, for which the geometry should be instantiated.
	/// @param vecBindMaterials The material binding for this geometry instance.
	/// @param materials The list of available materials.
	/// @param bAddToObject Whether or not the created mesh should be added to the object.
	/// @return Returns a pointer to the created mesh if successful.
	C3DMesh*	InstantiateMesh(C3DObjectData& ObjectData,const BindMaterialList& vecBindMaterials, MaterialList materials, ImageList& images,bool bAddToObject=true)
	{
		// allready instantiated?
		if (this->pMesh || !this->pXMLNode)
		{
			return this->pMesh;
		}

		// create new geometry/mesh
		TiXmlElement* pMeshNode=this->pXMLNode->FirstChildElement("mesh");
		if (!pMeshNode) pMeshNode=this->pXMLNode->FirstChildElement("convex_mesh");
		if (pMeshNode)
		{
			// read all streams
			VertexStreamList vecStreams;
			TiXmlElement* pStreamSourceNode=pMeshNode->FirstChildElement("source");
			while (pStreamSourceNode)
			{
				// create a new stream source
				vecStreams.push_back(VertexDataStream());
				if (vecStreams.size()>0)
				{
					vecStreams[vecStreams.size()-1].Load(pStreamSourceNode);
				}

				// get next stream source
				pStreamSourceNode=pStreamSourceNode->NextSiblingElement("source");
			}

			// get triangles
			TiXmlElement* pFaceDefinitionNode=pMeshNode->FirstChildElement();
			while (pFaceDefinitionNode)
			{
				std::string strNodeName=pFaceDefinitionNode->Value();
				if (strNodeName=="triangles" || strNodeName=="polygons")
				{
					// polygons?
					bool bPolygons=strNodeName=="polygons";

					// get the material
					std::string strMaterial=GetMaterialBinding(GetXmlAttributeString(pFaceDefinitionNode,"material"),vecBindMaterials);
					int iMaterialIndex=-1;
					for (MaterialList::iterator itMaterial=materials.begin();itMaterial!=materials.end();++itMaterial)
					{
						if (DaeIdCompare(itMaterial->strID,strMaterial))
						{
							iMaterialIndex=itMaterial->InstantiateMaterial(ObjectData,images);
							break;
						}
					}

					// all inputs
					TiXmlElement* pInputNode=pFaceDefinitionNode->FirstChildElement("input");
					std::vector<VertexDataStream*> vecUsedStreams;
					vecUsedStreams.resize(8,NULL);
					bool bNormals=false;
					int iTextureCoordinates=0;
					unsigned int nInputStreams=0;
					while (pInputNode)
					{
						// get stream index
						nInputStreams++;
						unsigned int nIndex=GetXmlAttributeInt(pInputNode,"offset");
						std::string strSemantic=GetXmlAttributeString(pInputNode,"semantic");
						std::string strStreamSource=GetXmlAttributeString(pInputNode,"source");

						// get the stream
						if (nIndex<8)
						{
							// is it a vertex stream?
							if (strSemantic=="VERTEX")
							{
								// search for a vertex list with the given stream id
								TiXmlElement* pVertexNode=pMeshNode->FirstChildElement("vertices");
								while (pVertexNode)
								{
									// is this the vertices element we were searching for?
									std::string strTest=GetXmlAttributeString(pVertexNode,"id");
									if (DaeIdCompare(strTest,strStreamSource))
									{
										// yes => search for position input
										TiXmlElement* pVertexInputNode=pVertexNode->FirstChildElement("input");
										while (pVertexInputNode)
										{
											// test type
											std::string strType=GetXmlAttributeString(pVertexInputNode,"semantic");
											if (strType=="POSITION")
											{
												// get name of referenced stream
												std::string strReferencesStream=GetXmlAttributeString(pVertexInputNode,"source");

												// search for that stream
												for (VertexStreamList::iterator itStream=vecStreams.begin();itStream!=vecStreams.end();++itStream)
												{
													if (DaeIdCompare(itStream->GetName(),strReferencesStream))
													{
														// this is the one
														vecUsedStreams[nIndex]=&(*itStream);
														break;
													}
												}

												// found it
												break;

											} // if POSITION stream type

											// next input node
											pVertexInputNode=pVertexInputNode->NextSiblingElement("input");

										} // while pVertexInputNode

										// stop the search here
										break;

									} // if right stream source

									// get next vertices element
									pVertexNode=pVertexNode->NextSiblingElement("vertices");

								} // while pVertexNode
							} // VERTEX stream semantic
							else
							{
								// directly search for the stream
								for (VertexStreamList::iterator itStream=vecStreams.begin();itStream!=vecStreams.end();++itStream)
								{
									if (DaeIdCompare(itStream->GetName(),strStreamSource))
									{
										// this is the one
										vecUsedStreams[nIndex]=&(*itStream);
										break;
									}
								}
							}

							// assign stream semantic
							if (vecUsedStreams[nIndex])
							{
								if (strSemantic=="POSITION" || strSemantic=="VERTEX")
								{
									vecUsedStreams[nIndex]->SetUsage("P");
								}
								else if (strSemantic=="NORMAL")
								{
									bNormals=true;
									vecUsedStreams[nIndex]->SetUsage("N");
								}
								else if (strSemantic=="TEXCOORD")
								{
									int iChannel=GetXmlAttributeInt(pInputNode,"set",1);
									vecUsedStreams[nIndex]->SetUsage("T"+ToString(iChannel));
									iTextureCoordinates=maximum(iTextureCoordinates,iChannel);
									if (iChannel>1)
									{
										// TODO : add support for more than one texture channel
										iTextureCoordinates=1;
										vecUsedStreams[nIndex]=NULL;
									}
								}
								else
								{
									// unsupported semantic
									vecUsedStreams[nIndex]=NULL;
								}
							} // stream available
						} // if stream index < 8
						
						// get next input node
						pInputNode=pInputNode->NextSiblingElement("input");

					} // while pInputNode

					// create the mesh
					if (!this->pMesh)
					{
						this->pMesh=new C3DMesh(&ObjectData,false,bNormals,iTextureCoordinates);
						if (this->pMesh)
						{
							// set name of mesh
							this->pMesh->SetName(GetXmlAttributeString(this->pXMLNode,"name"));
						}
					}

					// create triangles
					if (this->pMesh)
					{
						// create a new face set for this mesh
						C3DFaceSet* pFaceSet=new C3DFaceSet(this->pMesh,iMaterialIndex);
						this->pMesh->AddFaceSet(pFaceSet);

						// now read the face list
						TiXmlElement* pFaceListNode=pFaceDefinitionNode->FirstChildElement("p");
						while (pFaceListNode)
						{
							// read polygon or triangles?
							std::vector<unsigned int> vecVertexIndices;
							bool bReadSuccessfully=false;
							unsigned int nReadTriangles=0;
							if (bPolygons)
							{
								// just read the polygon data
								bReadSuccessfully=ReadStringArray(pFaceListNode->GetText(),vecVertexIndices);
								unsigned int nVertexIndices=static_cast<unsigned int>(vecVertexIndices.size())/nInputStreams;
								if (vecVertexIndices.size()%nInputStreams==0 && nVertexIndices>2)
								{
									nReadTriangles=nVertexIndices-2;	// triangle fan
								}
								else
								{
									bReadSuccessfully=false;
								}
							}
							else
							{
								// get number of entries in the triangles face list index array
								unsigned int nTriangles=GetXmlAttributeInt(pFaceDefinitionNode,"count");
								unsigned int nIndices=nTriangles*3*nInputStreams;
								vecVertexIndices.reserve(nIndices);
								bReadSuccessfully=ReadStringArray(pFaceListNode->GetText(),nIndices,vecVertexIndices);
								pFaceSet->ReserveFaceMem(nTriangles);
								nReadTriangles=nTriangles;
							}

							// was the triangle data read successfully?														
							if (bReadSuccessfully)
							{
								// create all vertices
								for (unsigned int nTriangle=0;nTriangle<nReadTriangles;nTriangle++)
								{
									// create a new face
									PNPTC_FACE face;

									// go through all vertices of the face
									for (unsigned int nVertex=0;nVertex<3;nVertex++)
									{
										// create a new vertex
										// TODO : try to reuse vertices
										PNPTC_VERTEX vert;
										vert.colDiffuse.r=255;
										vert.colDiffuse.g=255;
										vert.colDiffuse.b=255;
										vert.colDiffuse.a=255;

										// compute index for triangle, stream, and vertex
										unsigned int nIndex=0;
										if (bPolygons)
										{
											if (nVertex!=0)
											{
												nIndex=nTriangle+nVertex;
											}
										}
										else
										{
											nIndex=nTriangle*3+nVertex;
										}
										nIndex*=nInputStreams;

										// go through all streams
										for (unsigned int nStream=0;nStream<nInputStreams;nStream++,nIndex++)
										{
											if (nIndex<vecVertexIndices.size())
											{
												// get that index
												unsigned int nVertexIndex=vecVertexIndices[nIndex];
												if (vecUsedStreams[nStream])
												{
													if (vecUsedStreams[nStream]->GetUsage()=="P")
													{
														vert.vPos.x=vecUsedStreams[nStream]->GetElement(nVertexIndex,0);
														vert.vPos.y=vecUsedStreams[nStream]->GetElement(nVertexIndex,1);
														vert.vPos.z=vecUsedStreams[nStream]->GetElement(nVertexIndex,2);
													}
													else if (vecUsedStreams[nStream]->GetUsage()=="N")
													{
														vert.vNormal.x=vecUsedStreams[nStream]->GetElement(nVertexIndex,0);
														vert.vNormal.y=vecUsedStreams[nStream]->GetElement(nVertexIndex,1);
														vert.vNormal.z=vecUsedStreams[nStream]->GetElement(nVertexIndex,2);
													}
													else if (Left(vecUsedStreams[nStream]->GetUsage(),1)=="T")
													{
														int iUVIndex=atoi(Right(vecUsedStreams[nStream]->GetUsage(),1).c_str())-1;
														if (iUVIndex>=0 && iUVIndex<3)
														{
															vert.uv[iUVIndex].u=vecUsedStreams[nStream]->GetElement(nVertexIndex,0);
															vert.uv[iUVIndex].v=1.f-vecUsedStreams[nStream]->GetElement(nVertexIndex,1);
														}
													}
												} // valid stream
											} // valid vertex index
											else
											{
												// error 
												SAFE_DELETE(this->pMesh);
												return NULL;
											}
										} // for all streams

										// add that vertex
										face.dwVert[nVertex]=this->pMesh->AddVertex(vert);
									
									} // for all vertices of the face

									// add face
									pFaceSet->AddFace(face);

								} // for all triangles
							} // if bReadSuccessfully
							else
							{
								// error 
								SAFE_DELETE(this->pMesh);
							}

							// next face list
							pFaceListNode=pFaceListNode->NextSiblingElement(pFaceListNode->Value());

						} // while pFaceListNode

						// add the mesh
						if (bAddToObject && this->pMesh)
						{
							ObjectData.AddMesh(this->pMesh,false);
							bAddToObject=false; // only add mesh once
						}

					} // if pMesh
				} // if triangles or polygons node

				// get next triangle list
				pFaceDefinitionNode=pFaceDefinitionNode->NextSiblingElement();
			
			} // while (pFaceDefinitionNode)
		} // if pMeshNode

		// return the mesh
		return this->pMesh;

	} // GeometryDef::InstantiateMesh

} GeometryDef;



/// List of all geometries
typedef std::vector<GeometryDef>	GeometryList;


/// Computes the transformation of a given scene node.
D3DXMATRIX GetNodeTransform(TiXmlElement* pNode)
{
	// initialize return matrix
	D3DXMATRIX matReturn;
	D3DXMatrixIdentity(&matReturn);

	// go through all transform nodes
	TiXmlElement* pTransformNode=pNode->FirstChildElement();
	while (pTransformNode)
	{
		if (_stricmp(pTransformNode->Value(),"translate")==0)
		{
			// compute translational matrix
			D3DXMATRIX matTransform;
			float fTranslation[3]={0,0,0};
			std::string strText=pTransformNode->GetText();
			sscanf(strText.c_str(),"%f %f %f",&fTranslation[0],&fTranslation[1],&fTranslation[2]);
			D3DXMatrixTranslation(&matTransform,fTranslation[0],fTranslation[1],fTranslation[2]);
			
			// multiply with current transform matrix
			D3DXMatrixMultiply(&matReturn,&matTransform,&matReturn);
		}
		else if (_stricmp(pTransformNode->Value(),"rotate")==0)
		{
			// compute rotation matrix
			D3DXMATRIX matTransform;
			D3DXVECTOR3 vecRotAxis(0,0,0);
			float fAngle=0;
			std::string strText=pTransformNode->GetText();
			sscanf(strText.c_str(),"%f %f %f %f",&vecRotAxis.x,&vecRotAxis.y,&vecRotAxis.z,&fAngle);
			D3DXMatrixRotationAxis(&matTransform,&vecRotAxis,D3DXToRadian(fAngle));
			
			// multiply with current transform matrix
			D3DXMatrixMultiply(&matReturn,&matTransform,&matReturn);
		}
		else if (_stricmp(pTransformNode->Value(),"scale")==0)
		{
			// compute scaling matrix
			D3DXMATRIX matTransform;
			float fScale[3]={0,0,0};
			std::string strText=pTransformNode->GetText();
			sscanf(strText.c_str(),"%f %f %f",&fScale[0],&fScale[1],&fScale[2]);
			D3DXMatrixScaling(&matTransform,fScale[0],fScale[1],fScale[2]);
			
			// multiply with current transform matrix
			D3DXMatrixMultiply(&matReturn,&matTransform,&matReturn);
		}
		else if (_stricmp(pTransformNode->Value(),"matrix")==0)
		{
			// read matrix
			D3DXMATRIX matTransform;
			std::string strText=pTransformNode->GetText();
			sscanf( strText.c_str() ,"%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f"
									,&matTransform._11,&matTransform._21,&matTransform._31,&matTransform._41
									,&matTransform._12,&matTransform._22,&matTransform._32,&matTransform._42
									,&matTransform._13,&matTransform._23,&matTransform._33,&matTransform._43
									,&matTransform._14,&matTransform._24,&matTransform._34,&matTransform._44);
			
			// multiply with current transform matrix
			D3DXMatrixMultiply(&matReturn,&matTransform,&matReturn);
		}

		// next transform node
		pTransformNode=pTransformNode->NextSiblingElement();
	}

	// return final transformation
	return matReturn;
}

/// Adds one scene node as geometry to the object data
unsigned int AddNode(const std::string& strBaseName,const D3DXMATRIX& matBaseTransform,TiXmlElement* pNode,C3DObjectData& ObjectData,MaterialList& materials,GeometryList& geometries,ImageList& images)
{
	// Number of instantiated geometries
	unsigned int nInstantiated=0;

	// check input
	if (!pNode)
	{
		return nInstantiated;
	}

	// create name of this node
	std::string strName=strBaseName;
	bool bSubParts=false;
	if (!strName.empty())
	{
		strName+="/";
		bSubParts=true;
	}
	std::string strNodeName=GetXmlAttributeString(pNode,"name");
	strName+=strNodeName;

	// get transformation for this node
	D3DXMATRIX matTransform=GetNodeTransform(pNode);
	D3DXMatrixMultiply(&matTransform,&matTransform,&matBaseTransform);

	// is there a instanced geometry?
	TiXmlElement* pInstanceGeometryNode=pNode->FirstChildElement("instance_geometry");
	if (pInstanceGeometryNode)
	{
		// instantiate this geometry
		std::string strGeometry=GetXmlAttributeString(pInstanceGeometryNode,"url");

		// get bind materials
		BindMaterialList vecBindMaterials;
		FillMaterialBindings(pInstanceGeometryNode,vecBindMaterials);

		// get that geometry
		for (GeometryList::iterator itGeometry=geometries.begin();itGeometry!=geometries.end();++itGeometry)
		{
			if (DaeIdCompare(itGeometry->strID,strGeometry))
			{
				// instantiate this geometry
				C3DMesh* pMesh=itGeometry->InstantiateMesh(ObjectData,vecBindMaterials,materials,images);
				if (pMesh)
				{
					// TODO just use the first part for now
					std::string strInstanceName=strName;
					if (bSubParts)
					{
						Replace(&strInstanceName,"/","\\");
						strInstanceName=GetPathPart(strInstanceName,0);
					}

					// add instance
					if (ObjectData.AddMeshInstance(pMesh,strInstanceName,matTransform))
					{
						nInstantiated++;
					}
				}
				break;
			}
		}
	}
	else
	{
		// there is no mesh associated with this node => use it as reference position
		if (!strNodeName.empty())
		{
			// TODO : better use full name?
			ObjectData.AddReferenceFrame(strNodeName,matTransform);
		}
	}

	// go through all sub nodes
	TiXmlElement* pSubNode=pNode->FirstChildElement("node");
	while (pSubNode)
	{
		// instantiate sub node geometries
		nInstantiated+=AddNode(strName,matTransform,pSubNode,ObjectData,materials,geometries,images);

		// get next child
		pSubNode=pSubNode->NextSiblingElement("node");
	}

	// return number of instantiated geometries
	return nInstantiated;
}

/// Reads general settings of a physical shape.
/// @param pXmlElement The shape XML element.
/// @param pShape The shape for which the settings should be read.
void ReadGeneralShapeSettings(TiXmlElement* pXmlElement,C3DCollisionShape* pShape)
{
	// check parameters
	if (pXmlElement && pShape)
	{
		// read shape extra data
		TiXmlElement* pExtraXmlElement=pXmlElement->FirstChildElement("extra");
		if (pExtraXmlElement)
		{
			// get technique
			TiXmlElement* pTechniqueXmlElement=pExtraXmlElement->FirstChildElement("technique");
			while (pTechniqueXmlElement)
			{
				// is it a physx technique
				if (EqualNoCase(GetXmlAttributeString(pTechniqueXmlElement,"profile"),"PhysX"))
				{
					// read skin width
					TiXmlElement* pSkinWidthXmlElement=pTechniqueXmlElement->FirstChildElement("skinWidth");
					if (pSkinWidthXmlElement)
					{
						pShape->SetSkinWidth(static_cast<float>(atof(pSkinWidthXmlElement->GetText())));
					}

					// read disable collision
					TiXmlElement* pCollidableXmlElement=pTechniqueXmlElement->FirstChildElement("disable_collision");
					if (pCollidableXmlElement)
					{
						pShape->SetCollidable(!EqualNoCase(pCollidableXmlElement->GetText(),"true"));
					}

					// stop search for physical elements here
					break;
				}

				// read next technique
				pTechniqueXmlElement=pTechniqueXmlElement->NextSiblingElement(pTechniqueXmlElement->Value());
			}
		}
	}
}

/// Reads general settings of a physical geometry.
/// @param pXmlElement The geometry rigid body XML element.
/// @param pGeometry The Geometry for which the settings should be read.
void ReadGeneralGeometrySettings(TiXmlElement* pXmlElement,C3DCollisionGeometry* pGeometry)
{
	// check parameters
	if (pXmlElement && pGeometry)
	{
		// get technique
		TiXmlElement* pTechniqueXmlElement=pXmlElement->FirstChildElement("technique_common");
		if (pTechniqueXmlElement)
		{
			// read dynamic
			TiXmlElement* pDynamicXmlElement=pTechniqueXmlElement->FirstChildElement("dynamic");
			if (pDynamicXmlElement)
			{
				pGeometry->SetDynamic(!EqualNoCase(pDynamicXmlElement->GetText(),"false"));
			}

			// read mass
			TiXmlElement* pMassXmlElement=pTechniqueXmlElement->FirstChildElement("mass");
			if (pMassXmlElement)
			{
				pGeometry->SetMass(static_cast<float>(atof(pMassXmlElement->GetText())));
			}
		}

		// read shape extra data
		TiXmlElement* pExtraXmlElement=pXmlElement->FirstChildElement("extra");
		if (pExtraXmlElement)
		{
			// get technique
			TiXmlElement* pTechniqueXmlElement=pExtraXmlElement->FirstChildElement("technique");
			while (pTechniqueXmlElement)
			{
				// is it a physx technique
				if (EqualNoCase(GetXmlAttributeString(pTechniqueXmlElement,"profile"),"PhysX"))
				{
					// read disable collision
					TiXmlElement* pCollidableXmlElement=pTechniqueXmlElement->FirstChildElement("disable_collision");
					if (pCollidableXmlElement)
					{
						pGeometry->SetCollidable(!EqualNoCase(pCollidableXmlElement->GetText(),"true"));
					}

					// stop search for physical elements here
					break;
				}

				// read next technique
				pTechniqueXmlElement=pTechniqueXmlElement->NextSiblingElement(pTechniqueXmlElement->Value());
			}
		}
	}
}

/// Adds a collision geometry attachment to a joint.
/// @param pAttachmentXmlElement The attachment XML element
/// which should be added to the joint.
/// @param pJoint The joint to which the attachment
/// should be added.
/// @param ObjectData The object data object, which
/// contains the collision geometries, which should
/// be connected to the joint.
void AddJointAttachment(TiXmlElement* pAttachmentXmlElement,C3DJoint* pJoint,C3DObjectData& ObjectData)
{
	// check parameters
	if (pAttachmentXmlElement && pJoint)
	{
		// get name of collision geometry
		std::string strGeometry=GetXmlAttributeString(pAttachmentXmlElement,"name");

		// search for this collision geometry
		for (unsigned int nGeometry=0;nGeometry<ObjectData.GetNumCollisionGeometries();++nGeometry)
		{
			C3DCollisionGeometry* pGeometry=ObjectData.GetCollisionGeometry(nGeometry);
			if (pGeometry)
			{
				if (pGeometry->GetName()==strGeometry)
				{
					// get local transformation
					D3DXMATRIX matTransform=GetNodeTransform(pAttachmentXmlElement);

					// add attachment to joint
					pJoint->AddCollisionGeometry(pGeometry,D3DXVECTOR3(matTransform._41,matTransform._42,matTransform._43));

					// don't search any longer
					break;
				}
			}
		}
	}
}

/// Reads the joint's base parameters from an xml file.
/// @param pJoint The joint for which to read the parameters.
/// @param pXmlElement The xml node, below which the base parameters are stored.
/// @return Returns <i>true</i> of <i>false</i>, whether or not the
/// joint data could be read successfully.
bool ReadJointBaseParameters(C3DJoint* pJoint,TiXmlElement* pXmlElement)
{
	// check parameters
	if (!pJoint || !pXmlElement)
	{
		return false;
	}

	// test name
	std::string strID=GetXmlAttributeString(pXmlElement,"id");
	std::string strID2=strID;
	Replace(&strID2,"_","");
	std::string strName=pJoint->GetName();
	if (strID==strName || std::string(strID+"-RigidConstraint")==strName ||
		strID2==strName || std::string(strID2+"-RigidConstraint")==strName)
	{
		// read max force value
		TiXmlElement* pMaxForceXmlElement=pXmlElement->FirstChildElement("maxForce");
		if (pMaxForceXmlElement)
		{
			if (pMaxForceXmlElement->GetText()!="FLT_MAX")
			{
				pJoint->SetMaxForce(static_cast<float>(atof(pMaxForceXmlElement->GetText())));
			}
		}

		// read max torque value
		TiXmlElement* pMaxTorqueXmlElement=pXmlElement->FirstChildElement("maxTorque");
		if (pMaxTorqueXmlElement)
		{
			if (pMaxTorqueXmlElement->GetText()!="FLT_MAX")
			{
				pJoint->SetMaxTorque(static_cast<float>(atof(pMaxTorqueXmlElement->GetText())));
			}
		}

		// finished
		return true;
	}
	else
	{
		// wrong name
		return false;
	}
}

/// Reads additional joint parameters from an xml file.
/// This is deeded as long as not all relevant joint data
/// is present in the collada files.
/// @param pJoint The joint for which to read the parameters.
/// @param strFileName The file from which to read the joint data.
/// @return Returns <i>true</i> of <i>false</i>, whether or not the
/// joint data could be read successfully.
bool ReadJointParameters(C3DJoint* pJoint,const std::string& strFileName)
{
	// check parameters
	if (!pJoint)
	{
		return false;
	}

	// load that xml file for the joint definitions
	TiXmlDocument XmlDocument;
	if (XmlDocument.LoadFile(strFileName))
	{
		// get base node
		TiXmlElement* pStreamXmlElement=XmlDocument.FirstChildElement("NXUSTREAM2");
		if (pStreamXmlElement)
		{
			// get physics collection
			TiXmlElement* pPhysicsCollectionXmlElement=pStreamXmlElement->FirstChildElement("NxuPhysicsCollection");
			while (pPhysicsCollectionXmlElement)
			{
				// get scene
				TiXmlElement* pSceneXmlElement=pPhysicsCollectionXmlElement->FirstChildElement("NxSceneDesc");
				while (pSceneXmlElement)
				{
					// search for that joint in NxD6JointDescs
					TiXmlElement* pJointXmlElement=pSceneXmlElement->FirstChildElement("NxD6JointDesc");
					while (pJointXmlElement)
					{
						// get base desc
						TiXmlElement* pJointBaseDescXmlElement=pJointXmlElement->FirstChildElement("NxJointDesc");
						if (ReadJointBaseParameters(pJoint,pJointBaseDescXmlElement))
						{
							// finished here
							return true;
						}

						// read next joint
						pJointXmlElement=pJointXmlElement->NextSiblingElement(pJointXmlElement->Value());
					}

					// search for that joint in NxFixedJointDesc
					pJointXmlElement=pSceneXmlElement->FirstChildElement("NxFixedJointDesc");
					while (pJointXmlElement)
					{
						// get base desc
						TiXmlElement* pJointBaseDescXmlElement=pJointXmlElement->FirstChildElement("NxJointDesc");
						if (ReadJointBaseParameters(pJoint,pJointBaseDescXmlElement))
						{
							// finished here
							return true;
						}

						// read next joint
						pJointXmlElement=pJointXmlElement->NextSiblingElement(pJointXmlElement->Value());
					}

					// search for that joint in NxJointDesc
					pJointXmlElement=pSceneXmlElement->FirstChildElement("NxJointDesc");
					while (pJointXmlElement)
					{
						// read joint parameters
						if (ReadJointBaseParameters(pJoint,pJointXmlElement))
						{
							// finished here
							return true;
						}

						// read next joint
						pJointXmlElement=pJointXmlElement->NextSiblingElement(pJointXmlElement->Value());
					}

					// get next scene
					pSceneXmlElement=pSceneXmlElement->NextSiblingElement(pSceneXmlElement->Value());
				}

				// get next collection
				pPhysicsCollectionXmlElement=pPhysicsCollectionXmlElement->NextSiblingElement(pPhysicsCollectionXmlElement->Value());
			}
		}
	}

	// failed
	return false;
}

bool C3DObjectSerializerDAE::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	//	extract path and file name;
	if (strFileName.length()==0)
	{
		return false;
	}
	std::string strDummy;
	std::string strPath ;
	RemoveFilePath(strFileName,strDummy,strPath);

	// open xml document
	TiXmlDocument xml;
	if (!xml.LoadFile(strFileName))
	{
		return false;
	}

	// get collada base node
	TiXmlElement* pColladaNode=xml.FirstChildElement("COLLADA");
	if (!pColladaNode)
	{
		return false;
	}

	// read object version
	TiXmlElement* pAssetXmlElement=pColladaNode->FirstChildElement("asset");
	if (pAssetXmlElement)
	{
		// get "version" xml element
		TiXmlElement* pVersionXmlElement=pAssetXmlElement->FirstChildElement("modified");
		if (!pVersionXmlElement || EqualNoCase(pVersionXmlElement->GetText(),""))
		{
			pVersionXmlElement=pAssetXmlElement->FirstChildElement("created");
		}

		// read version string
		if (pVersionXmlElement)
		{
			ObjectData.SetObjectVersion(pVersionXmlElement->GetText());
		}
	}

	// ---------------------------------------------------------------------------------------
	// preprocessing (get node references of materials, textures and geometries)
	// ---------------------------------------------------------------------------------------

	// read all textures
	std::vector<ImageDef> vecImages;
	TiXmlElement* pLibImagesNode=pColladaNode->FirstChildElement("library_images");
	if (pLibImagesNode)
	{
		TiXmlElement* pImageNode=pLibImagesNode->FirstChildElement("image");
		while (pImageNode)
		{
			// create a new image from this node
			ImageDef img;
			img.strID=GetXmlAttributeString(pImageNode,"id");

			// get file name
			TiXmlElement* pInitFromNode=pImageNode->FirstChildElement("init_from");
			if (pInitFromNode)
			{
				const char* cTemp=pInitFromNode->GetText();
				if (cTemp)
				{
					img.strFileName=cTemp;
				}
			}

			// add image definition
			vecImages.push_back(img);

			// get next image
			pImageNode=pImageNode->NextSiblingElement("image");
		}
	}

	// create a list of all materials
	std::vector<MaterialDef> vecMaterials;
	TiXmlElement* pLibMaterialsNode=pColladaNode->FirstChildElement("library_materials");
	if (pLibMaterialsNode)
	{
		TiXmlElement* pMaterialNode=pLibMaterialsNode->FirstChildElement("material");
		while (pMaterialNode)
		{
			// read material
			MaterialDef material;
			material.strID=GetXmlAttributeString(pMaterialNode,"id");

			// find effect to this material
			TiXmlElement* pInstanceEffectNode=pMaterialNode->FirstChildElement("instance_effect");
			if (pInstanceEffectNode)
			{
				// get the name of the effect which we use
				std::string strEffectName=GetXmlAttributeString(pInstanceEffectNode,"url");

				// search for this effect
				TiXmlElement* pLibEffectsNode=pColladaNode->FirstChildElement("library_effects");
				if (pLibEffectsNode)
				{
					TiXmlElement* pEffectNode=pLibEffectsNode->FirstChildElement("effect");
					while (pEffectNode)
					{
						// is this the right effect?
						std::string strText=GetXmlAttributeString(pEffectNode,"id");
						if (DaeIdCompare(strText,strEffectName))
						{
							// yes => stop search here
							material.pXMLNode=pEffectNode;
							break;
						}

						// test next effect
						pEffectNode=pEffectNode->NextSiblingElement("effect");
					}
				}
			}

			// add material
			vecMaterials.push_back(material);

			// get next material
			pMaterialNode=pMaterialNode->NextSiblingElement("material");
		}
	}

	// create a list of all geometries
	std::vector<GeometryDef> vecGeometries;
	TiXmlElement* pLibGeometriesNode=pColladaNode->FirstChildElement("library_geometries");
	if (pLibGeometriesNode)
	{
		TiXmlElement* pGeometryNode=pLibGeometriesNode->FirstChildElement("geometry");
		while (pGeometryNode)
		{
			// read geometry
			GeometryDef geometry;
			geometry.pXMLNode=pGeometryNode;
			geometry.strID=GetXmlAttributeString(pGeometryNode,"id");

			// add geometry
			vecGeometries.push_back(geometry);

			// get next geometry
			pGeometryNode=pGeometryNode->NextSiblingElement("geometry");
		}
	}


	// ---------------------------------------------------------------------------------------
	// import visual scene
	// ---------------------------------------------------------------------------------------

	// a map of all visual scene nodes
	std::map<std::string,TiXmlElement*>	mapVisualSceneNodes;

	// get the name of the visual scene
	std::string strVisualScene;
	TiXmlElement* pSceneNode=pColladaNode->FirstChildElement("scene");
	if (pSceneNode)
	{
		TiXmlElement* pSceneInstanceNode=pSceneNode->FirstChildElement("instance_visual_scene");
		if (pSceneInstanceNode)
		{
			strVisualScene=GetXmlAttributeString(pSceneInstanceNode,"url");
		}
	}
	if (strVisualScene.empty())
	{
		return false;
	}

	// get the visual scene
	TiXmlElement* pVisualSceneNode=NULL;
	TiXmlElement* pLibVisualScenesNode=pColladaNode->FirstChildElement("library_visual_scenes");
	if (pLibVisualScenesNode)
	{
		TiXmlElement* pTestVisualSceneNode=pLibVisualScenesNode->FirstChildElement("visual_scene");
		while (pTestVisualSceneNode)
		{
			// get name of that scene
			std::string strID=GetXmlAttributeString(pTestVisualSceneNode,"id");
			if (DaeIdCompare(strID,strVisualScene))
			{
				// this is the scene, we searched for
				pVisualSceneNode=pTestVisualSceneNode;
			}

			// get a list of all nodes
			TiXmlElement* pNodeXmlElement=pTestVisualSceneNode->FirstChildElement("node");
			while (pNodeXmlElement)
			{
				// add node
				mapVisualSceneNodes[GetXmlAttributeString(pNodeXmlElement,"id")]=pNodeXmlElement;

				// get next node
				pNodeXmlElement=pNodeXmlElement->NextSiblingElement(pNodeXmlElement->Value());
			}

			// try next scene node
			pTestVisualSceneNode=pTestVisualSceneNode->NextSiblingElement("visual_scene");
		}
	}
	if (!pVisualSceneNode)
	{
		return false;
	}

	// ok, now instantiate all meshes
	TiXmlElement* pNode=pVisualSceneNode->FirstChildElement("node");
	while (pNode)
	{
		// get transformation of this node
		D3DXMATRIX matTransform;
		D3DXMatrixIdentity(&matTransform);
		AddNode("",matTransform,pNode,ObjectData,vecMaterials,vecGeometries,vecImages);

		// get next scene node
		pNode=pNode->NextSiblingElement("node");
	}


	// ---------------------------------------------------------------------------------------
	// import physical scene
	// ---------------------------------------------------------------------------------------

	// get the name of the physical scene
	std::string strPhysicalScene;
	pSceneNode=pColladaNode->FirstChildElement("scene");
	if (pSceneNode)
	{
		TiXmlElement* pSceneInstanceNode=pSceneNode->FirstChildElement("instance_physics_scene");
		if (pSceneInstanceNode)
		{
			strPhysicalScene=GetXmlAttributeString(pSceneInstanceNode,"url");
		}
	}
	if (!strPhysicalScene.empty())
	{
		// get the physical scene
		TiXmlElement* pPhysicalSceneNode=NULL;
		TiXmlElement* pLibPhysicalScenesNode=pColladaNode->FirstChildElement("library_physics_scenes");
		if (pLibPhysicalScenesNode)
		{
			TiXmlElement* pTestPhysicalSceneNode=pLibPhysicalScenesNode->FirstChildElement("physics_scene");
			while (pTestPhysicalSceneNode)
			{
				// get name of that scene
				std::string strID=GetXmlAttributeString(pTestPhysicalSceneNode,"id");
				if (DaeIdCompare(strID,strPhysicalScene))
				{
					// this is the scene, we searched for
					pPhysicalSceneNode=pTestPhysicalSceneNode;
					break;
				}

				// try next scene node
				pTestPhysicalSceneNode=pTestPhysicalSceneNode->NextSiblingElement("physics_scene");
			}
		}
		if (pPhysicalSceneNode)
		{
			// ok, now instantiate all physical model instances
			TiXmlElement* pPhysicsModelInstanceXMLNode=pPhysicalSceneNode->FirstChildElement("instance_physics_model");
			while (pPhysicsModelInstanceXMLNode)
			{
				// search for this model
				std::string strPhysicsModelID=GetXmlAttributeString(pPhysicsModelInstanceXMLNode,"url");
				TiXmlElement* pPhysicsModelsLibXMLNode=pColladaNode->FirstChildElement("library_physics_models");
				if (pPhysicsModelsLibXMLNode)
				{
					TiXmlElement* pPhysicsModelXMLNode=pPhysicsModelsLibXMLNode->FirstChildElement("physics_model");
					while(pPhysicsModelXMLNode)
					{
						std::string strID=GetXmlAttributeString(pPhysicsModelXMLNode,"id");
						if (DaeIdCompare(strID,strPhysicsModelID))
						{
							// again search for model instance (instance of an instance / funny structure)
							TiXmlElement* pPhysicsModelInstance2XMLNode=pPhysicsModelXMLNode->FirstChildElement("instance_physics_model");
							while (pPhysicsModelInstance2XMLNode)
							{
								std::string strPhysicsModel2ID=GetXmlAttributeString(pPhysicsModelInstance2XMLNode,"url");
								TiXmlElement* pPhysicsModel2XMLNode=pPhysicsModelsLibXMLNode->FirstChildElement("physics_model");
								while(pPhysicsModel2XMLNode)
								{
									std::string strID=GetXmlAttributeString(pPhysicsModel2XMLNode,"id");
									if (DaeIdCompare(strID,strPhysicsModel2ID))
									{
										// now go through all rigid bodie instances
										TiXmlElement* pInstanceRigidBodyXMLNode=pPhysicsModelInstance2XMLNode->FirstChildElement("instance_rigid_body");
										while (pInstanceRigidBodyXMLNode)
										{
											// get rigid body
											std::string strBodyID=GetXmlAttributeString(pInstanceRigidBodyXMLNode,"body");
											std::string strTargetID=GetXmlAttributeString(pInstanceRigidBodyXMLNode,"target");
											TiXmlElement* pRigidBodyXMLNode=pPhysicsModel2XMLNode->FirstChildElement("rigid_body");
											while (pRigidBodyXMLNode)
											{
												// TODO : support instancing
												std::string strTestID=GetXmlAttributeString(pRigidBodyXMLNode,"sid");
												if (DaeIdCompare(strTestID,strBodyID))
												{
													// get name of body
													std::string strBodyName=GetXmlAttributeString(pRigidBodyXMLNode,"name");

													// get technique
													TiXmlElement* pTechniqueXMLNode=pRigidBodyXMLNode->FirstChildElement("technique_common");
													if (pTechniqueXMLNode)
													{
														// create collision geometry
														C3DCollisionGeometry* pGeometry=new C3DCollisionGeometry(&ObjectData);
														if (pGeometry)
														{
															// is it dynamic
															TiXmlElement* pDynamicXmlElement=pTechniqueXMLNode->FirstChildElement("dynamic");
															if (pDynamicXmlElement)
															{
																pGeometry->SetDynamic(EqualNoCase(pDynamicXmlElement->GetText(),"true"));
															}

															// set base parameters of geometry
															pGeometry->SetName(strBodyName);

															// TODO : we read the object transformation from the visual scene, as the physics export seems to have some problems
															strTargetID=strBodyName+"-node";

															// set local transformation of this geometry
															// TODO : support instancing
															if (!strTargetID.empty())
															{
																std::map<std::string,TiXmlElement*>::iterator itActorNode=mapVisualSceneNodes.find(strTargetID);
																if (itActorNode!=mapVisualSceneNodes.end())
																{
																	pGeometry->SetLocalTransform(GetNodeTransform(itActorNode->second));
																}
																else
																{
																	itActorNode=mapVisualSceneNodes.find(Right(strTargetID,static_cast<int>(strTargetID.size())-1));
																	if (itActorNode!=mapVisualSceneNodes.end())
																	{
																		// TODO : get transformation from visual scene node of an actor in order to get the right scaling
																		pGeometry->SetLocalTransform(GetNodeTransform(itActorNode->second));
																	}
																}
															}

															// get shapes
															TiXmlElement* pShapeXMLNode=pTechniqueXMLNode->FirstChildElement("shape");
															while (pShapeXMLNode)
															{
																// read pose
																D3DXMATRIX matLocalTransform=GetNodeTransform(pShapeXMLNode);

																// process box shape
																TiXmlElement* pBoxXMLNode=pShapeXMLNode->FirstChildElement("box");
																if (pBoxXMLNode)
																{
																	// read size																	
																	TiXmlElement* pHalfExtendXMLNode=pBoxXMLNode->FirstChildElement("half_extents");
																	if (pHalfExtendXMLNode)
																	{
																		// read size
																		float fSize[3];
																		std::string strText=pHalfExtendXMLNode->GetText();
																		sscanf(strText.c_str(),"%f %f %f",&fSize[0],&fSize[1],&fSize[2]);

																		// create new box collision geometry
																		C3DCollisionBox* pBox=new C3DCollisionBox(fSize[0]*2,fSize[1]*2,fSize[2]*2);
																		pBox->SetLocalTransform(matLocalTransform);

																		// read general shape settings
																		ReadGeneralShapeSettings(pBoxXMLNode,pBox);

																		// add collision shape
																		pGeometry->AddShape(pBox);

																	} // if (pHalfExtendXMLNode)
																} // if (pBoxXMLNode)

																// process convex mesh shape
																TiXmlElement* pConvexMeshXMLNode=pShapeXMLNode->FirstChildElement("instance_geometry");
																if (pConvexMeshXMLNode)
																{
																	// get geometry name
																	std::string strGeometryID=GetXmlAttributeString(pConvexMeshXMLNode,"url");

																	// get bind materials
																	BindMaterialList vecBindMaterials;
																	FillMaterialBindings(pConvexMeshXMLNode,vecBindMaterials);

																	// get geometry for this ID
																	for (GeometryList::iterator itGeometry=vecGeometries.begin();itGeometry!=vecGeometries.end();++itGeometry)
																	{
																		if (DaeIdCompare(itGeometry->strID,strGeometryID))
																		{
																			// instantiate this geometry
																			C3DMesh *pMesh=itGeometry->InstantiateMesh(ObjectData,vecBindMaterials,vecMaterials,vecImages,false);
																			if (pMesh)
																			{
																				// fill convex mesh
																				size_t nVertices=pMesh->GetNumVertices();
																				if (nVertices>0)
																				{
																					// copy all vertices
																					C3DCollisionConvexHull* pConvexHull=new C3DCollisionConvexHull();
																					for (unsigned int nVertex=0;nVertex<nVertices;nVertex++)
																					{
																						const PNPTC_VERTEX& vert=pMesh->GetVertex(nVertex);
																						pConvexHull->AddVertex(vert.vPos);
																					}

																					// read general shape settings
																					ReadGeneralShapeSettings(pConvexMeshXMLNode,pConvexHull);

																					// add shape
																					pConvexHull->SetLocalTransform(matLocalTransform);
																					pGeometry->AddShape(pConvexHull);

																				} // if (nVertices>0)
																			} // if (pMesh)
																			break;

																		}  // if searched geometry ID
																	} // for all geometries
																} // if (pConvexMeshXMLNode)

																// get next shape
																pShapeXMLNode=pShapeXMLNode->NextSiblingElement("shape");

															}  // while (pShapeXMLNode)

															// read general geometry settings
															ReadGeneralGeometrySettings(pRigidBodyXMLNode,pGeometry);

															// add geometry to object
															ObjectData.AddCollisionGeometry(pGeometry);

														}  // if (pGeometry)
													} // if (pTechniqueXMLNode)

													// end search here
													pRigidBodyXMLNode=NULL;

												} // if searched rigid body ID
												else
												{
													// next xml node
													pRigidBodyXMLNode=pRigidBodyXMLNode->NextSiblingElement("rigid_body");
												}
											} // while (pRigidBodyXMLNode)

											// next rigid body instance
											pInstanceRigidBodyXMLNode=pInstanceRigidBodyXMLNode->NextSiblingElement("instance_rigid_body");

										} // while (pInstanceRigidBodyXMLNode)

										// now go through all rigid contraints instances
										TiXmlElement* pInstanceConstraintXmlElement=pPhysicsModelInstance2XMLNode->FirstChildElement("instance_rigid_constraint");
										while (pInstanceConstraintXmlElement)
										{
											// get rigid constraint
											std::string strConstraintID=GetXmlAttributeString(pInstanceConstraintXmlElement,"constraint");
											TiXmlElement* pRigidConstraintXmlElement=pPhysicsModel2XMLNode->FirstChildElement("rigid_constraint");
											while (pRigidConstraintXmlElement)
											{
												// check constraint ID
												std::string strTestID=GetXmlAttributeString(pRigidConstraintXmlElement,"sid");
												if (strTestID==strConstraintID || "#"+strTestID==strConstraintID)
												{
													// create a new joint
													C3DJoint* pJoint=new C3DJoint();
													if (pJoint)
													{
														// set name of joint
														pJoint->SetName(strConstraintID);

														// get reference attachment
														TiXmlElement* pRefAttachmentXmlElement=pRigidConstraintXmlElement->FirstChildElement("ref_attachment");
														AddJointAttachment(pRefAttachmentXmlElement,pJoint,ObjectData);

														// get attachment 
														TiXmlElement* pAttachmentXmlElement=pRigidConstraintXmlElement->FirstChildElement("attachment");
														AddJointAttachment(pAttachmentXmlElement,pJoint,ObjectData);

														// read additional data for the joint
														ReadJointParameters(pJoint,ReplaceFileExtension(strFileName,"xml"));

														// add joint
														ObjectData.AddJoint(pJoint);

													}	// if (pJoint)

													// end search here
													pRigidConstraintXmlElement=NULL;

												} // if searched constraint ID
												else
												{
													// next xml node
													pRigidConstraintXmlElement=pRigidConstraintXmlElement->NextSiblingElement(pRigidConstraintXmlElement->Value());
												}
											} // while (pRigidConstraintXmlElement)

											// get next rigid constraint instance
											pInstanceConstraintXmlElement=pInstanceConstraintXmlElement->NextSiblingElement(pInstanceConstraintXmlElement->Value());

										} // while (pInstanceConstraintXmlElement)

										// stop search for model here
										pPhysicsModel2XMLNode=NULL;
									}
									else
									{
										// get next node
										pPhysicsModel2XMLNode=pPhysicsModel2XMLNode->NextSiblingElement("physics_model");
									}

								} // while (pInstanceRigidBodyXMLNode)

								// get next instance node
								pPhysicsModelInstance2XMLNode=pPhysicsModelInstance2XMLNode->NextSiblingElement("instance_physics_model");

							} // while (pPhysicsModelInstance2XMLNode)

							// stop search for model here
							pPhysicsModelXMLNode=NULL;

						} // found searched physics model ID
						else
						{
							// get next node
							pPhysicsModelXMLNode=pPhysicsModelXMLNode->NextSiblingElement("physics_model");
						}

					} // while(pPhysicsModelXMLNode)
				} // if (pPhysicsModelsLibXMLNode)

				// get next instance node
				pPhysicsModelInstanceXMLNode=pPhysicsModelInstanceXMLNode->NextSiblingElement("instance_physics_model");

			}  // while (pPhysicsModelInstanceXMLNode)
		} // if (pPhysicalSceneNode)
	} // if (!strPhysicalScene.empty())

	// connect meshe instances with collision objects
	for (unsigned int nMeshInstance=0;nMeshInstance<ObjectData.GetNumMeshInstances();nMeshInstance++)
	{
		C3DMeshInstance* pMeshInstance=ObjectData.GetMeshInstance(nMeshInstance);
		if (pMeshInstance)
		{
			std::string strMeshInstanceName=pMeshInstance->GetName();
			if (!strMeshInstanceName.empty())
			{
				// search for a collision object with this name
				for (unsigned int nCollisionGeometry=0;nCollisionGeometry<ObjectData.GetNumCollisionGeometries();nCollisionGeometry++)
				{
					C3DCollisionGeometry* pCollisionGeometry=ObjectData.GetCollisionGeometry(nCollisionGeometry);
					if (pCollisionGeometry)
					{
						// check name
						if (EqualNoCase(pCollisionGeometry->GetName(),strMeshInstanceName))
						{
							// ok, connect these two geometries
							pMeshInstance->SetCollisionGeometry(pCollisionGeometry);
							break;
						}
					} // if (pCollisionGeometry)
				} // for all collision geometries
			} // if (!strMeshInstanceName.empty())
		} // if (pMeshInstance)
	} // for all meshes

	// ok 
	return true;
}




// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------




bool C3DObjectSerializerDAE::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// saving to collada is currently not implemented
	return false;
}
