/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerMD3.cpp
// 3d object serialization class for .md3 files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	17. Sep. 2008	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "3DObjectData.h"
#include ".\3dobjectserializermd3.h"
#include "../pnptcutils/utils.h"
#include <vector>
#include <map>

// byte alignment
#pragma pack(push)
#pragma pack(1)



// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// md3 types
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

// some defintions
#define MAX_QPATH			64
#define MD3_MAX_FRAMES		1024
#define MD3_MAX_TAGS		16
#define MD3_MAX_SURFACES	64



// vector 
typedef float vec3_t[3];

// MD3 header 
typedef struct md3_header_t
{
  int ident;					// magic number: "IDP3" 
  int version;					// version: must be 15

  char name[MAX_QPATH];			// the name of the mesh

  int flags;					// some flags

  int num_frames;				// the number of frames
  int num_tags;					// number of tags
  int num_surfaces;				// number of surfaces
  int num_skins;				// number of skins

  int offset_frames;			// offset frame data
  int offset_tags;				// offset tag data
  int offset_surface;			// offset surface data
  int offset_eof;				// offset to the end of the file

} md3_header_t;

/* Texture name */
typedef struct md2_skin_t
{
  char name[64];              /* texture file name */
} md2_skin_t;

/* Texture coords */
typedef struct md2_texcoord_t
{
  int s;
  int t;
} md2_texcoord_t;

/* Triangle info */
typedef struct md2_triangle_t
{
  unsigned short vertex[3];   /* vertex indices of the triangle */
  unsigned short st[3];       /* tex. coord. indices */
} md2_triangle_t;

/* Compressed vertex */
typedef struct md2_vertex_t
{
  unsigned char v[3];         /* position */
  unsigned char normalIndex;  /* normal vector index */
} md2_vertex_t;

/* Model frame */
typedef struct md2_frame_t
{
  vec3_t scale;               /* scale factor */
  vec3_t translate;           /* translation vector */
  char name[16];              /* frame name */
  std::vector<md2_vertex_t> verts;        /* list of frame's vertices */
} md2_frame_t;

/* GL command packet */
typedef struct md2_glcmd_t
{
  float s;
  float t;
  int index;
} md2_glcmd_t;

// MD3 model structure 
typedef struct md3_model_t
{
  md3_header_t header;

  std::vector<md2_skin_t> skins;
  std::vector<md2_texcoord_t> texcoords;
  std::vector<md2_triangle_t> triangles;
  std::vector<md2_frame_t> frames;  
  //std::vector<int> glcmds;

  //GLuint tex_id;
} md3_model_t;

/* combination of vertex and uv index */
typedef struct vertex_uv_t
{
	int vert;
	int uv;

	bool operator < (const vertex_uv_t& uv) const
	{
		return this->vert<uv.vert || (this->vert==uv.vert && this->uv<uv.uv);
	}
} vertex_uv_t;

/* mapping between a vertex and uv index combination to a final vertex index */
typedef std::map<vertex_uv_t,int> vertex_mapping_t;



/* Table of precalculated normals */
vec3_t md3_anorms_table[162] = {
#include "MDLNormals.h"
};





// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// C3DObjectSerializerMDL implementation
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------



C3DObjectSerializerMD3::C3DObjectSerializerMD3(CTempFileManager* pTempFileManager)
	: m_pTempFileManager(pTempFileManager)
{
}

C3DObjectSerializerMD3::~C3DObjectSerializerMD3()
{
}

bool C3DObjectSerializerMD3::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// open the file
	FILE* fp = fopen (strFileName.c_str(), "rb");
	if (!fp)
    {
		MessageBox(NULL,(std::string("Error opening md3 file for loading object data.\nFile name: '")+strFileName+"'").c_str(),"Error loading md3 object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// read header 
	md3_model_t mdl;
	fread (&mdl.header, 1, sizeof (md3_header_t), fp);

	// check identifier and version
	if ((mdl.header.ident != 860898377))/* ||
        (mdl.header.version != 15))*/
	{
		// error!
		MessageBox(NULL,(std::string("Error reading md3 file for loading object data.\nError: bad version of identifier.\nFile name: '")+strFileName+"'").c_str(),"Error loading md3 object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		fclose (fp);
		return false;
	}

	// check range of read values
	mdl.header.num_frames=maximum(mdl.header.num_frames,MD3_MAX_FRAMES);
	mdl.header.num_tags=maximum(mdl.header.num_tags,MD3_MAX_TAGS);
	mdl.header.num_surfaces=maximum(mdl.header.num_surfaces,MD3_MAX_SURFACES);

#if (0)
	/* Memory allocations */
	mdl.skins.resize(mdl.header.num_skins);
	mdl.texcoords.resize(mdl.header.num_st);
	mdl.triangles.resize(mdl.header.num_tris);
	mdl.frames.resize(mdl.header.num_frames);
	//mdl.glcmds.resize(mdl.header.num_glcmds);
	//mdl.tex_id = 0;

	/* Read skins */
	fseek (fp, mdl.header.offset_skins, SEEK_SET);
	fread (&mdl.skins[0], sizeof (md2_skin_t), mdl.header.num_skins, fp);

	/* Read texture coordinates */
	fseek (fp, mdl.header.offset_st, SEEK_SET);
	fread (&mdl.texcoords[0], sizeof (md2_texcoord_t), mdl.header.num_st, fp);

	/* Read triangle data */
	fseek (fp, mdl.header.offset_tris, SEEK_SET);
	fread (&mdl.triangles[0], sizeof (md2_triangle_t), mdl.header.num_tris, fp);

	/* Read GL commands */
	//fseek (fp, mdl.header.offset_glcmds, SEEK_SET);
	//fread (&mdl.glcmds[0], sizeof (int), mdl.header.num_glcmds, fp);

	/* Read frames */
	for (int i = 0; i < minimum(mdl.header.num_frames,1); ++i) // we only need the first frame => yet no animation support
//	for (int i = 0; i < mdl.header.num_frames; ++i)
	{
		/* Memory allocation for vertices of this frame */
		mdl.frames[i].verts.resize(mdl.header.num_vertices);

		/* Read frame data */
		fread (&mdl.frames[i].scale, sizeof (vec3_t), 1, fp);
		fread (&mdl.frames[i].translate, sizeof (vec3_t), 1, fp);
		fread (&mdl.frames[i].name, sizeof (char), 16, fp);
		fread (&mdl.frames[i].verts[0], sizeof (md2_vertex_t), mdl.header.num_vertices, fp);
	}

	// close the file
	fclose (fp);

	// test number of frames
	if (mdl.header.num_frames<=0)
	{
		MessageBox(NULL,(std::string("Error reading md2 file for loading object data.\nError: does not contain any frames.\nFile name: '")+strFileName+"'").c_str(),"Error loading md2 object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ----------------------------------------------------------------------
	// now convert the mdl model to the PnP object data structure

	// create mesh
	C3DMesh* pMesh=new C3DMesh(&ObjectData,false,true,true);
	if (pMesh)
	{
		// add the mesh to the object data
		ObjectData.AddMesh(pMesh);

		// initialize vertex mem
		PNPTC_VERTEX vert;
		memset(&vert,0,sizeof(PNPTC_VERTEX));
		vert.colDiffuse=PNPTC_RGB(255,255,255);
		pMesh->ReserveVertexMem(mdl.header.num_vertices);
		vertex_mapping_t vertexMapping;

		// create material
		if (mdl.header.num_skins>0 && mdl.skins.size()>0 &&
			mdl.header.skinwidth>0 && mdl.header.skinheight>0)
		{
			// create material
			PNPTC_MATERIAL mat;
			mat.bAlphaBlend=false;
			mat.bUseVertexColors=false;
			mat.colAmbient=PNPTC_RGB(255,255,255);
			mat.colDiffuse=PNPTC_RGB(255,255,255);
			mat.iTextures=1;
			mat.strName="skin";
			mat.strTexture[0]=mdl.skins[0].name;

			// add material
			ObjectData.AddMaterial(mat);
		}

		// create triangles/faces
		C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,0);
		if (pFaceSet)
		{
			// initialize face set
			pMesh->AddFaceSet(pFaceSet);
			pFaceSet->ReserveFaceMem(mdl.header.num_tris);

			// add all faces
			for (unsigned int nTri=0;static_cast<int>(nTri)<mdl.header.num_tris;++nTri)
			{
				// get all vertex indices of this triangle
				int iVertices[3];

				// go through all vertices
				for (unsigned int nVertex = 0; nVertex < 3; ++nVertex)
				{
					// get vertex and uv index
					vertex_uv_t vertuv;
					vertuv.vert=mdl.triangles[nTri].vertex[nVertex];
					vertuv.uv=mdl.triangles[nTri].st[nVertex];

					// does it already exist?
					vertex_mapping_t::iterator itVertexIndex=vertexMapping.find(vertuv);
					if (itVertexIndex==vertexMapping.end())
					{
						// does not yet exist => create
						vert.vPos.x=(mdl.frames[0].verts[vertuv.vert].v[0] * mdl.frames[0].scale[0]) + mdl.frames[0].translate[0];
						vert.vPos.y=(mdl.frames[0].verts[vertuv.vert].v[1] * mdl.frames[0].scale[1]) + mdl.frames[0].translate[1];
						vert.vPos.z=(mdl.frames[0].verts[vertuv.vert].v[2] * mdl.frames[0].scale[2]) + mdl.frames[0].translate[2];

						// create vertex uv data
						vert.uv[0].u=static_cast<float>(mdl.texcoords[vertuv.uv].s) / static_cast<float>(mdl.header.skinwidth);
						vert.uv[0].v=static_cast<float>(mdl.texcoords[vertuv.uv].t) / static_cast<float>(mdl.header.skinheight);

						// create vertex normal data
						vert.vNormal.x=md3_anorms_table[mdl.frames[0].verts[vertuv.vert].normalIndex][0];
						vert.vNormal.y=md3_anorms_table[mdl.frames[0].verts[vertuv.vert].normalIndex][1];
						vert.vNormal.z=md3_anorms_table[mdl.frames[0].verts[vertuv.vert].normalIndex][2];

						// add vertex
						iVertices[nVertex]=pMesh->AddVertex(vert);
						vertexMapping[vertuv]=iVertices[nVertex];	
					}
					else
					{
						// already exists => store index
						iVertices[nVertex]=itVertexIndex->second;
					}
				}

				// create and add face
				PNPTC_FACE face;
				face.dwVert[0]=iVertices[0];
				face.dwVert[2]=iVertices[1];
				face.dwVert[1]=iVertices[2];
				pFaceSet->AddFace(face);				
			}
		}
	}
#endif
	// ok
	return true;
}

bool C3DObjectSerializerMD3::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	return false;
}





// restore alignment
#pragma pack(pop)
