/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerMDL.cpp
// 3d object serialization class for .mdl, .mdl5, .mdl7 files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	26. Aug. 2008	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "3DObjectData.h"
#include ".\3dobjectserializermdl.h"
#include "../pnptcutils/utils.h"
#include "../pnptcutils/bitmapfile.h"
#include "../pnptcutils/TempFileManager.h"
#include <vector>

// byte alignment
#pragma pack(push)
#pragma pack(1)



// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// mdl types
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

/* Vector */
typedef float vec3_t[3];

/* MDL header */
typedef struct mdl_header_t
{
	int ident;            /* magic number: "IDPO" or 1330660425 */
	int version;          /* version: 6 */

	vec3_t scale;         /* scale factor */
	vec3_t translate;     /* translation vector */
	float boundingradius;
	vec3_t eyeposition;   /* eyes' position */

	int num_skins;        /* number of textures */
	int skinwidth;        /* texture width */
	int skinheight;       /* texture height */

	int num_verts;        /* number of vertices */
	int num_tris;         /* number of triangles */
	int num_frames;       /* number of frames */

	int synctype;         /* 0 = synchron, 1 = random */
	int flags;            /* state flag */
	float size;
} mdl_header_t;

/* Texture coords */
typedef struct mdl_texcoord_t
{
  int onseam;
  int s;
  int t;
} mdl_texcoord_t;

/* Triangle info */
typedef struct mdl_triangle_t
{
  int facesfront;  /* 0 = backface, 1 = frontface */
  int vertex[3];   /* vertex indices */
} mdl_triangle_t;

/* Compressed vertex */
typedef struct mdl_vertex_t
{
  unsigned char v[3];
  unsigned char normalIndex;
} mdl_vertex_t;

/* Simple frame */
typedef struct mdl_simpleframe_t
{
  mdl_vertex_t bboxmin;             /* bouding box min */
  mdl_vertex_t bboxmax;             /* bouding box max */
  char name[16];                    /* the name of the frame */
  std::vector<mdl_vertex_t> verts;  /* vertex list of the frame */
} mdl_simpleframe_t;

/* Model frame */
typedef struct mdl_frame_t
{
  int type;                 /* 0 = simple, !0 = group */
  mdl_simpleframe_t frame;  /* this program can't read models composed of group frames! */
} mdl_frame_t;

/* Group of simple frames */
/*typedef struct mdl_groupframe_t
{
  int type;                  // !0 = group
  mdl_vertex_t min;          // min pos in all simple frames
  mdl_vertex_t max;          // max pos in all simple frames
  float *time;               // time duration for each frame
  mdl_simpleframe_t *frames; // simple frame list
} mdl_groupframe_t; */

/* Skin */
typedef struct mdl_skin_t
{
  int group;               /* 0 = single, 1 = group */
  unsigned char* data;     /* texture data */
  CBitmapFile bitmap;      /* the rgb bitmap file */
} mdl_skin_t;

/* Group of pictures */
/*typedef struct mdl_groupskin_t
{
    int group;             // 1 = group 
    int nb;                // number of pics 
    float *time;           // time duration for each pic 
    unsigned char **data;  // texture data 
} mdl_groupskin_t;/*

/* MDL model structure */
typedef struct mdl_model_t
{
  mdl_header_t header;

  std::vector<mdl_skin_t>     skins;
  std::vector<mdl_texcoord_t> texcoords;
  std::vector<mdl_triangle_t> triangles;
  std::vector<mdl_frame_t>    frames;

  int iskin;
} mdl_model_t;

/* Table of precalculated normals */
vec3_t anorms_table[162] = {
#include "MDLNormals.h"
};

/* Palette */
unsigned char colormap[256][3] = {
#include "MDLColormap.h"
};

/**
 * Make a texture given a skin index 'n'.
 */
bool MakeTextureFromSkin (int n, mdl_model_t& mdl)
{
	// initialize bitmap
	if (!mdl.skins[n].bitmap.Initialize(bmfRGB,mdl.header.skinwidth, mdl.header.skinheight))
	{
		return false;
	}
	/* Convert indexed 8 bits texture to RGB 24 bits */
	for (int i = 0; i < mdl.header.skinwidth * mdl.header.skinheight; ++i)
    {
		// compute color
		PNPTC_RGB rgb;
		rgb.r=colormap[mdl.skins[n].data[i]][0];
		rgb.g=colormap[mdl.skins[n].data[i]][1];
		rgb.b=colormap[mdl.skins[n].data[i]][2];

		// set color
		mdl.skins[n].bitmap.SetPixel(i%mdl.header.skinwidth,i/mdl.header.skinwidth,rgb);
	}

	// ok
	return true;
}




// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// C3DObjectSerializerMDL implementation
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------



C3DObjectSerializerMDL::C3DObjectSerializerMDL(CTempFileManager* pTempFileManager)
	: m_pTempFileManager(pTempFileManager)
{
}

C3DObjectSerializerMDL::~C3DObjectSerializerMDL()
{
}

bool C3DObjectSerializerMDL::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// open the file
	FILE* fp = fopen(strFileName.c_str(), "rb");
	if (!fp)
    {
		MessageBox(NULL,(std::string("Error opening mdl file for loading object data.\nFile name: '")+strFileName+"'").c_str(),"Error loading mdl object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	/* Read header */
	mdl_model_t mdl;
	fread (&mdl.header, 1, sizeof (mdl_header_t), fp);

	// check identifier and version
	if ((mdl.header.ident != 1330660425) ||
        (mdl.header.version != 6))
	{
		/* Error! */
		MessageBox(NULL,(std::string("Error reading mdl file for loading object data.\nError: bad version of identifier.\nFile name: '")+strFileName+"'").c_str(),"Error loading mdl object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		fclose (fp);
		return false;
	}

	/* Memory allocations */
	mdl.skins.resize(mdl.header.num_skins);
	mdl.texcoords.resize(mdl.header.num_verts);
	mdl.triangles.resize(mdl.header.num_tris);
	mdl.frames.resize(mdl.header.num_frames);
	mdl.iskin = 0;

	/* Read texture data */
	for (int i = 0; i < mdl.header.num_skins; ++i)
	{
		// create temporary data for texture
		mdl.skins[i].data = (unsigned char*)malloc(sizeof(unsigned char)*mdl.header.skinwidth * mdl.header.skinheight);

		// read skin data
		fread (&mdl.skins[i].group, sizeof (int), 1, fp);
		fread (mdl.skins[i].data, sizeof(unsigned char), mdl.header.skinwidth * mdl.header.skinheight, fp);

		// create texture
		MakeTextureFromSkin(i,mdl);

		// free texture
		SAFE_FREE(mdl.skins[i].data);
	}

	// read texture coordinates and triangles
	fread(&mdl.texcoords[0], sizeof(mdl_texcoord_t),mdl.header.num_verts, fp);
	fread(&mdl.triangles[0], sizeof(mdl_triangle_t),mdl.header.num_tris, fp);

	/* Read frames */
	for (int i = 0; i < minimum(mdl.header.num_frames,1); ++i) // we only need the first frame => yet no animation support
//	for (int i = 0; i < mdl.header.num_frames; ++i)
	{
		/* Memory allocation for vertices of this frame */
		mdl.frames[i].frame.verts.resize(mdl.header.num_verts);

		/* Read frame data */
		fread (&mdl.frames[i].type, sizeof(long), 1, fp);
		fread (&mdl.frames[i].frame.bboxmin, sizeof(mdl_vertex_t), 1, fp);
		fread (&mdl.frames[i].frame.bboxmax, sizeof(mdl_vertex_t), 1, fp);
		fread (mdl.frames[i].frame.name, sizeof(char), 16, fp);
		fread (&mdl.frames[i].frame.verts[0], sizeof(mdl_vertex_t), mdl.header.num_verts, fp);
	}

	// close the file
	fclose (fp);

	// test number of frames
	if (mdl.header.num_frames<=0)
	{
		MessageBox(NULL,(std::string("Error reading mdl file for loading object data.\nError: does not contain any frames.\nFile name: '")+strFileName+"'").c_str(),"Error loading mdl object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ----------------------------------------------------------------------
	// now convert the mdl model to the PnP object data structure

	// create mesh
	C3DMesh* pMesh=new C3DMesh(&ObjectData,false,true,true);
	if (pMesh)
	{
		// add the mesh to the object data
		ObjectData.AddMesh(pMesh);

		// create vertices
		PNPTC_VERTEX vert;
		memset(&vert,0,sizeof(PNPTC_VERTEX));
		vert.colDiffuse=PNPTC_RGB(255,255,255);
		pMesh->ReserveVertexMem(mdl.header.num_verts*2);

		// create front vertices
		for (unsigned int n=0;static_cast<int>(n)<mdl.header.num_verts;n++)
		{
			// create vertex position data
			vert.vPos.x=static_cast<float>(mdl.frames[0].frame.verts[n].v[0])*mdl.header.scale[0]+mdl.header.translate[0];
			vert.vPos.y=static_cast<float>(mdl.frames[0].frame.verts[n].v[1])*mdl.header.scale[1]+mdl.header.translate[1];
			vert.vPos.z=static_cast<float>(mdl.frames[0].frame.verts[n].v[2])*mdl.header.scale[2]+mdl.header.translate[2];
			
			// create vertex uv data
			vert.uv[0].u = (static_cast<float>(mdl.texcoords[n].s) + 0.5f) / static_cast<float>(maximum(mdl.header.skinwidth,1));
			vert.uv[0].v = 1.f - (static_cast<float>(mdl.texcoords[n].t) + 0.5f) / static_cast<float>(maximum(mdl.header.skinheight,1));

			// create vertex normal data
			vert.vNormal.x=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][0];
			vert.vNormal.y=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][1];
			vert.vNormal.z=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][2];

			// add vertex to mesh
			pMesh->AddVertex(vert);
		}

		// create back vertices
		for (unsigned int n=0;static_cast<int>(n)<mdl.header.num_verts;n++)
		{
			// create vertex position data
			vert.vPos.x=static_cast<float>(mdl.frames[0].frame.verts[n].v[0])*mdl.header.scale[0]+mdl.header.translate[0];
			vert.vPos.y=static_cast<float>(mdl.frames[0].frame.verts[n].v[1])*mdl.header.scale[1]+mdl.header.translate[1];
			vert.vPos.z=static_cast<float>(mdl.frames[0].frame.verts[n].v[2])*mdl.header.scale[2]+mdl.header.translate[2];
			
			// create vertex uv data
			vert.uv[0].u = (static_cast<float>(mdl.texcoords[n].s) + 0.5f * static_cast<float>(mdl.header.skinwidth) + 0.5f) / static_cast<float>(maximum(mdl.header.skinwidth,1));
			vert.uv[0].v = 1.f - (static_cast<float>(mdl.texcoords[n].t) + 0.5f) / static_cast<float>(maximum(mdl.header.skinheight,1));

			// create vertex normal data
			vert.vNormal.x=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][0];
			vert.vNormal.y=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][1];
			vert.vNormal.z=anorms_table[mdl.frames[0].frame.verts[n].normalIndex][2];

			// add vertex to mesh
			pMesh->AddVertex(vert);
		}

		// create material
		if (mdl.header.num_skins>0 && mdl.skins.size()>0 &&
			mdl.header.skinwidth>0 && mdl.header.skinheight>0)
		{
			// create material
			PNPTC_MATERIAL mat;
			mat.bAlphaBlend=false;
			mat.bUseVertexColors=false;
			mat.colAmbient=PNPTC_RGB(255,255,255);
			mat.colDiffuse=PNPTC_RGB(255,255,255);
			mat.iTextures=1;
			mat.strName="skin";
			mat.strTexture[0]="";

			// store texture
			if (this->m_pTempFileManager)
			{
				mat.strTexture[0]=this->m_pTempFileManager->CreateTempFile("pnp");
				if (!mat.strTexture[0].empty())
				{
					mdl.skins[0].bitmap.Write(mat.strTexture[0]);
				}
			}

			// add material
			ObjectData.AddMaterial(mat);
		}

		// create triangles/faces
		C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,0);
		if (pFaceSet)
		{
			// initialize face set
			pMesh->AddFaceSet(pFaceSet);
			pFaceSet->ReserveFaceMem(mdl.header.num_tris);

			// add all faces
			for (unsigned int n=0;static_cast<int>(n)<mdl.header.num_tris;n++)
			{
				// create and add face
				PNPTC_FACE face;
				face.dwVert[0]=mdl.triangles[n].vertex[0]+((!mdl.triangles[n].facesfront && mdl.texcoords[mdl.triangles[n].vertex[0]].onseam)?mdl.header.num_verts:0);
				face.dwVert[2]=mdl.triangles[n].vertex[1]+((!mdl.triangles[n].facesfront && mdl.texcoords[mdl.triangles[n].vertex[1]].onseam)?mdl.header.num_verts:0);
				face.dwVert[1]=mdl.triangles[n].vertex[2]+((!mdl.triangles[n].facesfront && mdl.texcoords[mdl.triangles[n].vertex[2]].onseam)?mdl.header.num_verts:0);
				pFaceSet->AddFace(face);				
			}
		}
	}

	// ok
	return true;
}

bool C3DObjectSerializerMDL::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	return false;
}





// restore alignment
#pragma pack(pop)
