/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerOBJ.cpp
// 3d object serialization class for .obj files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////


#define _CRT_SECURE_NO_DEPRECATE 1

#include "3DObjectData.h"
#include ".\3dobjectserializerobj.h"
#include "../PnPTCUtils/Utils.h"
#include <map>



C3DObjectSerializerOBJ::C3DObjectSerializerOBJ(void)
{
}

C3DObjectSerializerOBJ::~C3DObjectSerializerOBJ(void)
{
}



// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------


/// one combination of vertex/texture/normal
struct ArrayIndex
{
	int vertexIndex;
	int vertexTextureIndex;
	int vertexNormalIndex;

	bool operator < (const ArrayIndex& cmp) const
	{
		return (this->vertexIndex< cmp.vertexIndex) ||
			   (this->vertexIndex==cmp.vertexIndex && this->vertexNormalIndex< cmp.vertexNormalIndex) ||
			   (this->vertexIndex==cmp.vertexIndex && this->vertexNormalIndex==cmp.vertexNormalIndex && this->vertexTextureIndex< cmp.vertexTextureIndex);
	};
};

/// list of vertices
typedef std::vector < ArrayIndex > ListArrayIndex;

/// face as list of vertices
struct Face
{
	ListArrayIndex arrayIndices;
};

/// list of faces
typedef std::vector<Face> ListFace;

/// one object grouping
struct Group
{
	std::string		name;
	PNPTC_MATERIAL*	material;
	ListFace		faces;

	Group()	: material(NULL),name(""){};
};

/// list of materials
typedef std::map<std::string, PNPTC_MATERIAL> MaterialList;

/// list of vertices
typedef std::vector<PnPTC3DVec> Vec3DList;

/// list of uv coordinates
typedef std::vector<PnPTCUV> UVList;

/// list of groups
typedef std::vector<Group> GroupList;


/// loading of material data
bool LoadMaterialFile(std::string strPath,std::string strMatFile,MaterialList& materials);

/// sets up one face of vertex, normal, uv
void SetupFace(std::string strLine, Face* newFace,int iLastVertexIndex,int iLastTextureIndex,int iLastNormalIndex);

/// convertes the data read from obj into the data structure of the object class
bool ConvertData(C3DObjectData& obj,Vec3DList& vertices,Vec3DList& normals,UVList& texcoords,GroupList& groups,MaterialList& materials);


bool C3DObjectSerializerOBJ::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	//	extract path and file name;
	if (strFileName.length()==0)
	{
		return false;
	}
	std::string strDummy;
	std::string strPath ;
	RemoveFilePath(strFileName,strDummy,strPath);

	// list of all vertices
	Vec3DList		vecVertices;
	Vec3DList		vecNormals;
	UVList			vecTexCoords;
	GroupList		vecGroups;
	MaterialList	mapMaterials;

	// only deals with matllib, usemtl, v, vn, vt, g, and f commands

	// open file
	FILE* inputFile = fopen(strFileName.c_str(), "r");
	if (!inputFile)
	{
		return false;
	}

	// current group
	Group* pGroup=NULL;

	// go through all lines
	char cline[256];
	std::string strLine;
	while(fgets(cline, 256, inputFile) != NULL)
	{
		// convert into string and trim it
		strLine=cline;
		Trim(strLine);

		// the first letter of every line tells us what to do with the rest of the line
		switch (strLine.at(0))
		{
			static char dummy[100];

		case 'm':	// material definition
			{
				// get material file name
				char cMatFile[MAX_PATH];				
				sscanf(strLine.substr(1,strLine.length()).c_str(),"%s %s",dummy,cMatFile);
				std::string strMatFile=cMatFile;
				// read material
				LoadMaterialFile(strPath,strMatFile,mapMaterials);
			}
			break;

		case 'v':	// new vertex of some type

			switch (strLine.at(1)) // switch on second char to determine which kind
			{
			case ' ': // plain vertex
				{
					PnPTC3DVec vertex;
					// initialize the vertex values (important if not all values are in file)
					vertex.x = vertex.y = vertex.z = 0.0f;
					// fill the values with the data in the line
					sscanf(strLine.substr(0,strLine.length()).c_str(), "%s %f %f %f", dummy, &(vertex.x), &(vertex.y), &(vertex.z));
					// add to list
					vecVertices.push_back(vertex);
				}
				break;
			case 't': // texture vertex
				{
					PnPTCUV vt;
					// initialize the texture vertex values
					vt.u = vt.v = 0.0f;
					// fill the values with the data in the line
					float fDummy;
					sscanf(strLine.substr(0,strLine.length()).c_str(), "%s %f %f %f", dummy, &(vt.u), &(vt.v), &(fDummy));
					// add to list
					vecTexCoords.push_back(vt);
				}
				break;
			case 'n': // vertex normal
				{
					PnPTC3DVec vn;
					// initialize the vertex normal values
					vn.x = vn.y = vn.z = 0.0f;
					// fill the values with the data in the line
					sscanf(strLine.substr(0,strLine.length()).c_str(), "%s %f %f %f", dummy, &(vn.x), &(vn.y), &(vn.z));
					// add to list
					vecNormals.push_back(vn);
				}
				break;
			}
			break;

		case 'g': // new group

			{
				// reset the current Group
				pGroup=NULL;

				// create new group
				char cGroupName[100];
				Group NewGroup;
				sscanf(strLine.substr(0,strLine.length()).c_str(), "%s %s",dummy,cGroupName);
				NewGroup.name=cGroupName;

				// add group to list
				vecGroups.push_back(NewGroup);

				// get pointer to new group
				if (vecGroups.size()>0)
				{
					pGroup=&vecGroups[vecGroups.size()-1];
				}
			}
			break;

		case 'u': // usemtl

			if (!pGroup || pGroup->faces.size()>0)
			{
				// create new default group
				Group NewGroup;
				vecGroups.push_back(NewGroup);
				if (vecGroups.size()>0)
				{
					pGroup=&vecGroups[vecGroups.size()-1];
				}
			}

			if (pGroup)
			{
				// get material name
				char cMatName[100];				
				sscanf(strLine.substr(0,strLine.length()).c_str(),"%s %s",dummy,cMatName);
				std::string strMatName=cMatName;
				// add material
				if (mapMaterials.find(strMatName)!=mapMaterials.end())
				{
					PNPTC_MATERIAL* pMat=&mapMaterials[strMatName];
					if (pMat)
					{
						pGroup->material=pMat;
					}
				}
			}
			break;

		case 'f': // a new face

			
			if (!pGroup)
			{
				// create new default group
				Group NewGroup;
				NewGroup.material=NULL;
				NewGroup.name="";
				vecGroups.push_back(NewGroup);
				if (vecGroups.size()>0)
				{
					pGroup=&vecGroups[vecGroups.size()-1];
				}
			}

			if (pGroup)
			{
				Face newFace;

				// there are two steps for faces: creating the face and adding it
				// to the groups

				//first, create the face
				SetupFace(strLine.substr(0,strLine.length()).c_str(), &newFace,static_cast<int>(vecVertices.size())-1,static_cast<int>(vecTexCoords.size())-1,static_cast<int>(vecNormals.size())-1);

				// second, add the face to the group
				pGroup->faces.push_back(newFace);
			}
			break;
		}
	}

	// close file
	fclose(inputFile);

	// convert data
	return ConvertData(ObjectData,vecVertices,vecNormals,vecTexCoords,vecGroups,mapMaterials);
}

bool LoadMaterialFile(std::string strPath,std::string strMatFile,MaterialList& materials)
{
	// try to open file directly
	char cline[256];
	std::string strLine;
	FILE* inputFile = fopen(strMatFile.c_str(), "r");
	if (!inputFile)
	{
		// try with append path
		inputFile = fopen((strPath+strMatFile).c_str(), "r");
		if (!inputFile)
		{
			return false;
		}
	}

	// current material
	PNPTC_MATERIAL* pMaterial=NULL;

	// go through whole file
	while(fgets(cline, 256, inputFile) != NULL)
	{
		// convert into string and trim it
		strLine=cline;
		Trim(strLine);
		while ((Right(strLine,1)=="\r" || Right(strLine,1)=="\n") &&
			   strLine.size()>0)
		{
			strLine=Left(strLine,static_cast<unsigned int>(strLine.size())-1);
		}

		// anything left?
		if (strLine.size()>0)
		{
			// the first letter of every line tells us what to do with the rest of the line
			switch (strLine.at(0))
			{
				static char dummy[100];

			case 'n':	// new material
				{
					// finish previous material
					pMaterial=NULL;

					// create new material
					PNPTC_MATERIAL NewMaterial;
					NewMaterial.iTextures=0;
					NewMaterial.colAmbient.a=NewMaterial.colAmbient.r=NewMaterial.colAmbient.g=NewMaterial.colAmbient.b=255;
					NewMaterial.colDiffuse.a=NewMaterial.colDiffuse.r=NewMaterial.colDiffuse.g=NewMaterial.colDiffuse.b=255;
					char cMatName[100];				
					sscanf(strLine.substr(0,strLine.length()).c_str(),"%s %s",dummy,cMatName);
					NewMaterial.strName=cMatName;
					// add material
					materials.insert(std::pair<std::string,PNPTC_MATERIAL>(NewMaterial.strName,NewMaterial));

					// get pointer to new material
					if (materials.size()>0)
					{
						pMaterial=&materials[NewMaterial.strName];
					}
				}
				break;

			case 'K':	// color value

				if (pMaterial)
				{
					float fColors[3];
					sscanf(strLine.substr(0,strLine.length()).c_str(), "%s %f %f %f", dummy, &(fColors[0]), &(fColors[1]), &(fColors[2]));
					switch (strLine.at(1))
					{
					case 'a':	// ambient
						pMaterial->colAmbient.r=(unsigned char)(fColors[0]*255.f);
						pMaterial->colAmbient.g=(unsigned char)(fColors[1]*255.f);
						pMaterial->colAmbient.b=(unsigned char)(fColors[2]*255.f);
						break;
					case 'd':	// diffuse
						pMaterial->colDiffuse.r=(unsigned char)(fColors[0]*255.f);
						pMaterial->colDiffuse.g=(unsigned char)(fColors[1]*255.f);
						pMaterial->colDiffuse.b=(unsigned char)(fColors[2]*255.f);
						break;
					case 's':	// specular
						break;
					}
				}
				break;

			case 'd':	// alpha

				if (pMaterial)
				{
	//				sscanf(line, "%s %f", dummy, &(pMaterial->alpha));
				}
				break;

			case 'i':	// illumination modell

				if (pMaterial)
				{
	//				sscanf(line, "%s %d", dummy, &(pMaterial->illumination));
				}
				break;

			case 'N':	// shininess

				if (pMaterial)
				{
	//				sscanf(line, "%s %f", dummy, &(pMaterial->shininess));
				}
				break;

			case 'm':	// texture map

				if (pMaterial)
				{
					sscanf(strLine.substr(0,strLine.length()).c_str(), "%s", dummy);
					pMaterial->iTextures=1;
					pMaterial->strTexture[0]=strLine.substr(strlen(dummy)+1);
					Trim(pMaterial->strTexture[0]);

					// does the texture support alpha or has the material alpha color
					if (TextureSupportsAlpha(pMaterial->strTexture[0]))
					{
						pMaterial->bAlphaBlend=true;
					}
					else if (pMaterial->colDiffuse.a<255 || pMaterial->colAmbient.a<255)
					{
						pMaterial->bAlphaBlend=true;
					}
					else
					{
						pMaterial->bAlphaBlend=false;
					}
				}
				break;
			}
		}
	}

	// ok
	fclose(inputFile);
	return true;
}

void SetupFace(std::string strLine, Face* newFace,int iLastVertexIndex,int iLastTextureIndex,int iLastNormalIndex)
{
	int lPos, rPos;

	strLine = strLine.substr(0, strLine.length() - 1); // chomp the newline

	// for each word that follows until the end of the line, those are
	// our new array indices for the face
	lPos = static_cast<int>(strLine.find(" "));
	while (lPos != std::string::npos)
	{
		std::string curWord;

		lPos++; // get the lPos past the space

		rPos = static_cast<int>(strLine.find(" ", lPos));
		if (rPos != std::string::npos)
			curWord = strLine.substr(lPos, rPos - (lPos));
		else
			curWord = strLine.substr(lPos, strLine.length());

		// ignore any extra spaces
		if (curWord != " ")
		{
			ArrayIndex arrayIndex;
			int vertexIndex, vtIndex, vnIndex;
			int posSlash1, posSlash2;
			posSlash1 = static_cast<int>(curWord.find("/"));
			posSlash2 = static_cast<int>(curWord.find("/", posSlash1 + 1));
			sscanf(curWord.substr(0, posSlash1).c_str(), "%d", &vertexIndex);
			if (posSlash2 > posSlash1 + 1 || posSlash2==std::string::npos)
				sscanf(curWord.substr(posSlash1 + 1, posSlash2 - (posSlash1 + 1)).c_str(), "%d", &vtIndex);
			else
				vtIndex = 0; // will get turned to -1 (see below)
			if (static_cast<int>(curWord.length()) > posSlash2 + 1 && posSlash2!=std::string::npos)
				sscanf(curWord.substr(posSlash2 + 1, curWord.length()).c_str(), "%d", &vnIndex);
			else
				vnIndex = 0; // will get turned to -1 (see below)

			// since .obj starts indexes at 1, not 0, take 1 away to compensate
			// negative indexes are relative to the last index occuring in the .obj
			// file so far.
			if (vertexIndex>=0)
			{
				arrayIndex.vertexIndex = vertexIndex - 1; 
			}
			else
			{
				arrayIndex.vertexIndex = minmax(iLastVertexIndex+(vertexIndex+1),0,iLastVertexIndex);
			}
			if (vnIndex>=0)
			{
				arrayIndex.vertexNormalIndex = vnIndex - 1; 
			}
			else
			{
				arrayIndex.vertexNormalIndex = minmax(iLastNormalIndex+(vnIndex+1),0,iLastNormalIndex);
			}
			if (vtIndex>=0)
			{
				arrayIndex.vertexTextureIndex = vtIndex - 1;
			}
			else
			{
				arrayIndex.vertexTextureIndex = minmax(iLastTextureIndex+(vtIndex+1),0,iLastTextureIndex);
			}

			// Okay, we have our array index.  Now add it to the face.
			newFace->arrayIndices.push_back(arrayIndex);
		}
		lPos = static_cast<int>(strLine.find(" ", lPos));
	}
}

/// mapping between vertex combination and unique vertex index
typedef std::map<ArrayIndex,int>	ArrayIndexMapping;

int GetArrayIndexMapping(ArrayIndexMapping& IndexMap,ArrayIndex& Index,C3DMesh* pMesh,Vec3DList& vertices,Vec3DList& normals,UVList& texcoords)
{
	// get combined index
	ArrayIndexMapping::iterator itIndex=IndexMap.find(Index);
	if (itIndex==IndexMap.end())
	{
		// create new vertex
		PNPTC_VERTEX vert;
		memset(&vert,0,sizeof(PNPTC_VERTEX));
		if (Index.vertexIndex>=0 && Index.vertexIndex<static_cast<int>(vertices.size()))
		{
			vert.vPos=vertices[Index.vertexIndex];
		}
		if (Index.vertexNormalIndex>=0 && Index.vertexNormalIndex<static_cast<int>(normals.size()))
		{
			vert.vNormal=normals[Index.vertexNormalIndex];
		}
		if (Index.vertexTextureIndex>=0 && Index.vertexTextureIndex<static_cast<int>(texcoords.size()))
		{
			vert.uv[0]=texcoords[Index.vertexTextureIndex];
			vert.uv[0].v=1.f-vert.uv[0].v;
		}
		// add vertex
		int iIndex=pMesh->AddVertex(vert);
		// add vertex to mapping
		if (iIndex>=0)
		{
			IndexMap.insert(ArrayIndexMapping::value_type(Index,iIndex));
		}
		// return new index
		return iIndex;
	}
	else
	{
		// return that index
		return itIndex->second;
	}
}

bool ConvertData(C3DObjectData& obj,Vec3DList& vertices,Vec3DList& normals,UVList& texcoords,GroupList& groups,MaterialList& materials)
{
	// create a mesh
	C3DMesh* pMesh=new C3DMesh(&obj,false,normals.size()!=0,texcoords.size()!=0);
	if (!pMesh)
	{
		return false;
	}
	obj.AddMesh(pMesh);

	// create vertices
	ArrayIndexMapping mapArrayIndex;

	// create materials
	for (MaterialList::iterator itMat=materials.begin();itMat!=materials.end();itMat++)
	{
		obj.AddMaterial(itMat->second);
	}

	// export face set
	C3DFaceSet* pFaceSet=NULL;
	PNPTC_MATERIAL* pMaterial=NULL;

	// go through all groups
	for (GroupList::iterator itGrp=groups.begin();itGrp!=groups.end();itGrp++)
	{
		// get group
		Group* pGrp=&(*itGrp);
		if (!pGrp || pGrp->faces.size()==0)
		{
			continue;
		}

		// check for new group
		if (!pFaceSet || pGrp->material!=pMaterial)
		{
			// initialization for next group
			pFaceSet=NULL;
			pMaterial=pGrp->material;

			// get material index of face set
			int iMaterial=-1;
			if (pMaterial)
			{
				int i=0;
				for (MaterialList::iterator itMat=materials.begin();itMat!=materials.end();itMat++,i++)
				{
					if (itMat->second.strName==pMaterial->strName)
					{
						iMaterial=i;
						break;
					}
				}
			}

			// create new group
			pFaceSet=new C3DFaceSet(pMesh,iMaterial);
			if (!pFaceSet)
			{
				return false;
			}

			// add to mesh
			pMesh->AddFaceSet(pFaceSet);
		}

		if (pFaceSet)
		{
			// go through all faces of group and setup the correct texture coordinates
			for (ListFace::iterator itFace=pGrp->faces.begin();itFace!=pGrp->faces.end();itFace++)
			{
				if (itFace->arrayIndices.size()>=3)
				{
					// add all triangles
					PNPTC_FACE face;
					face.dwVert[0]=GetArrayIndexMapping(mapArrayIndex,itFace->arrayIndices[0],pMesh,vertices,normals,texcoords);
					face.dwVert[1]=GetArrayIndexMapping(mapArrayIndex,itFace->arrayIndices[1],pMesh,vertices,normals,texcoords);
					for (int iTri=0;iTri<static_cast<int>(itFace->arrayIndices.size())-2;iTri++)
					{
						// add this triangle
						face.dwVert[2]=GetArrayIndexMapping(mapArrayIndex,itFace->arrayIndices[iTri+2],pMesh,vertices,normals,texcoords);
						if (face.dwVert[0]>=0 && face.dwVert[1]>=0 && face.dwVert[2]>=0)
						{
							pFaceSet->AddFace(face);
						}
						// make ready to add next triangle
						face.dwVert[1]=face.dwVert[2];
					}
				}
			}
		}
	}

	// ok
	return true;
}






// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------




bool C3DObjectSerializerOBJ::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// -----------------------------------------------------------------------------
	// create file
	FILE* f=fopen(strFileName.c_str(),"w");
	if (!f)
	{
		MessageBox(NULL,(std::string("The file ")+strFileName+std::string(" could not be opened for writing.")).c_str(),"Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// print file header
	if (fprintf(f,"# Wavefront OBJ file generated by PnP TerrainCreator\n\n")<0)
	{
		// error
		MessageBox(NULL,"Error writing file header to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		fclose(f);
		return false;
	}


	// -----------------------------------------------------------------------------
	// write all materials used in the object

	if (ObjectData.GetNumMaterials()>0)
	{
		// create material filename
		std::string strDummy;
		std::string strMaterialFilename;
		RemoveFileExtension(strFileName,strMaterialFilename,strDummy);
		std::string strMaterialLib=strMaterialFilename+".mtl";
		strMaterialFilename=GetPath(strFileName)+strMaterialLib;

		// create material file
		FILE* fileMaterial=fopen(strMaterialFilename.c_str(),"w");
		if (!fileMaterial)
		{
			MessageBox(NULL,(std::string("The file ")+strMaterialFilename+std::string(" could not be opened for writing.")).c_str(),"Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}

		// print file header
		if (fprintf(fileMaterial,"# Wavefront OBJ/MTL file generated by PnP TerrainCreator\n# num_materials: %d\n\n",ObjectData.GetNumMaterials())<0)
		{
			// error
			MessageBox(NULL,"Error writing file header to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			fclose(fileMaterial);
			return false;
		}

		// write all materials
		for (unsigned int iMat=0;iMat<ObjectData.GetNumMaterials();iMat++)
		{
			// get material
			const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(iMat);
			if (pMaterial)
			{
				// write material header
				if (fprintf(fileMaterial,"newmtl %s\n",pMaterial->strName.c_str())<0)
				{
					// error
					MessageBox(NULL,"Error writing material header to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}

				// write ambient color components
				if (fprintf(fileMaterial,"Ka %f %f %f\n",static_cast<float>(pMaterial->colAmbient.r)/255.f,static_cast<float>(pMaterial->colAmbient.g)/255.f,static_cast<float>(pMaterial->colAmbient.b)/255.f)<0)
				{
					// error
					MessageBox(NULL,"Error writing ambient color to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}

				// write diffuse color components
				if (fprintf(fileMaterial,"Kd %f %f %f\n",static_cast<float>(pMaterial->colDiffuse.r)/255.f,static_cast<float>(pMaterial->colDiffuse.g)/255.f,static_cast<float>(pMaterial->colDiffuse.b)/255.f)<0)
				{
					// error
					MessageBox(NULL,"Error writing diffuse color to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}

				// write specular color components
				if (fprintf(fileMaterial,"Ks 0.0 0.0 0.0\n")<0)
				{
					// error
					MessageBox(NULL,"Error writing specular color to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}

				// write alpha, shininess and illumination model
				if (fprintf(fileMaterial,"d 1\nNs 0\nillum 1\n")<0)
				{
					// error
					MessageBox(NULL,"Error writing additional color information to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}

				// write texture
				if (fprintf(fileMaterial,"map_Kd %s\n\n",pMaterial->strTexture[0].c_str())<0)
				{
					// error
					MessageBox(NULL,"Error writing texture to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					fclose(fileMaterial);
					return false;
				}
			}
		}

		// print file footer
		if (fprintf(fileMaterial,"# num_materials: %d\n",ObjectData.GetNumMaterials())<0)
		{
			// error
			MessageBox(NULL,"Error writing file footer to obj/mtl-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			fclose(fileMaterial);
			return false;
		}

		// close material file
		fclose(fileMaterial);

		// print used material lib
		if (fprintf(f,"mtllib %s\n\n",strMaterialLib.c_str())<0)
		{
			// error
			MessageBox(NULL,"Error writing material lib to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}
	}

	// -----------------------------------------------------------------------------
	// write all meshes of object
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		DWORD dwVertices=(DWORD)pMesh->GetNumVertices();

		// write mesh header
		if (fprintf(f,"g %s\n\n",pMesh->GetName().c_str())<0)
		{
			// error
			MessageBox(NULL,"Error writing mesh header to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}

		// write all vertices
		for (unsigned long lVert=0;lVert<dwVertices;lVert++)
		{
			const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
			PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
			GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
			if (fprintf(f,"v %f %f %f \n",vecPos.x,vecPos.y,vecPos.z)<0)
			{
				// error
				MessageBox(NULL,"Error writing vertex data to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}
		}
		if (fprintf(f,"# num_vertices: %d\n\n",dwVertices)<0)
		{
			// error
			MessageBox(NULL,"Error writing vertex footer to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}

		// optionally write all vertex normals
		if (pMesh->HasVertexNormals())
		{
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
				PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
				GlobalVectorTransform(vecNormal,1,iUpRotation,iFlipAxis);
				if (fprintf(f,"vn %f %f %f\n",vecNormal.x,vecNormal.y,vecNormal.z)<0)
				{
					// error
					MessageBox(NULL,"Error writing vertex normal data to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			if (fprintf(f,"# num_vertex_normals: %d\n\n",dwVertices)<0)
			{
				// error
				MessageBox(NULL,"Error writing vertex normal footer to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}
		}

		// optionally write all vertex texture coordinates
		if (pMesh->GetNumMappingCoords()>0)
		{
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
				if (fprintf(f,"vt %f %f\n",vert.uv[0].u,(1.f-vert.uv[0].v))<0)
				{
					// error
					MessageBox(NULL,"Error writing vertex uv coordinates to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			if (fprintf(f,"# num_uv_coordinates: %d\n\n",dwVertices)<0)
			{
				// error
				MessageBox(NULL,"Error writing vertex uv coordinates footer to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}
		}

		// get face format string
		std::string strFormat=" %d/";
		if (pMesh->GetNumMappingCoords()>0) strFormat+="%d";
		strFormat+="/";
		if (pMesh->HasVertexNormals()) strFormat+="%d";

		// write faces
		for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
		{
			const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
			if (pFaceSet)
			{
				// write material used for faceset
				int iMaterial=pFaceSet->GetMaterial();
				const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(iMaterial);
				if (pMaterial)
				{
					if (pMesh->GetNumMappingCoords()>0 && iMaterial>=0 && iMaterial<static_cast<int>(ObjectData.GetNumMaterials()))
					{
						if (fprintf(f,"usemtl %s\n",pMaterial->strName.c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing faceset material to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
				}

				// write faces
				unsigned long lFace=0;
				for (lFace=0;lFace<pFaceSet->GetNumFaces();lFace++)
				{
					const PNPTC_FACE& face=pFaceSet->GetFace(lFace);
					// write textual face data
					if (fprintf(f,(std::string("f")+strFormat).c_str(),face.dwVert[0]+1,face.dwVert[0]+1,face.dwVert[0]+1)<0 ||
						fprintf(f,strFormat.c_str(),face.dwVert[1]+1,face.dwVert[1]+1,face.dwVert[1]+1)<0 ||
						fprintf(f,(strFormat+std::string("\n")).c_str(),face.dwVert[2]+1,face.dwVert[2]+1,face.dwVert[2]+1)<0 )
					{
						// error
						MessageBox(NULL,"Error writing face data to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				if (fprintf(f,"# num_faces: %d\n\n",lFace)<0)
				{
					// error
					MessageBox(NULL,"Error writing face footer to obj-file.","Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
		}
	}

	// ok
	fclose(f);
	return true;
}
