/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerOGRE.cpp
// 3d object serialization class for ogre mesh files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
//  19. Aug. 2007   : Improved ogre entity naming conformity
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_DEPRECATE 1

#include "3DObjectData.h"
#include ".\3dobjectserializerogre.h"
#include "../PnPTCUtils/Utils.h"
#include "../3rd-party/tinyxml/tinyxml.h"
#include <map>



C3DObjectSerializerOGRE::C3DObjectSerializerOGRE(void)
	: m_bSaveMaterials(true)
	, m_bConvertToLowerCase(false)
{
}

C3DObjectSerializerOGRE::~C3DObjectSerializerOGRE(void)
{
}


/// Makes sure that the name of an entity is conform to ogre naming restrictions.
/// @param strName The name which has to be checked/modified.
void ValidateOgreEntityName(std::string& strName)
{
	// replaces all characters, which are not allowed
	Replace(&strName,"\\","/");
	ReplaceForbiddenCharacters(&strName,"/._1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ","_");
}

/// Converts a string into a valid ogre entity name.
/// @param strName The name which has to be validated.
/// @return Returns a valid ogre entity string.
std::string ConvertValidateOgreEntityName(const std::string& strName)
{
	// replaces all characters, which are not allowed
	std::string strReturn(strName);
	ValidateOgreEntityName(strReturn);
	return strReturn;
}

std::string GetOgreXmlAttributeString(TiXmlElement* pNode,const std::string& strAttribute,const std::string& strDefault="")
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return std::string(*pNode->Attribute(strAttribute));
	}
	else
	{
		// return default value
		return strDefault;
	}
}

int GetOgreXmlAttributeInt(TiXmlElement* pNode,const std::string& strAttribute,int iDefault=0)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return atoi(pNode->Attribute(strAttribute)->c_str());
	}
	else
	{
		// return default value
		return iDefault;
	}
}

float GetOgreXmlAttributeFloat(TiXmlElement* pNode,const std::string& strAttribute,float fDefault=0)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return static_cast<float>(atof(pNode->Attribute(strAttribute)->c_str()));
	}
	else
	{
		// return default value
		return fDefault;
	}
}


// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------





bool C3DObjectSerializerOGRE::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	//	extract path and file name;
	if (strFileName.length()==0)
	{
		return false;
	}

	std::string strDummy;
	std::string strPath ;
	std::string yeahRight;
	RemoveFilePath(strFileName,strDummy,strPath);
	RemoveFileExtension(strFileName,strDummy,yeahRight);

	// open xml document
	TiXmlDocument xml;
	if (!xml.LoadFile(strFileName))
	{
		return false;
	}

	// open xml document
	TiXmlDocument xml_materials;
	if (!xml_materials.LoadFile(strDummy + ".pnpogremat" ))
	{
		return false;
	}

	std::map<std::string,std::string> matMap;
	typedef std::pair <std::string, std::string> Material_Pair;
	// get materials node
	TiXmlElement* pMaterialsNode = xml_materials.FirstChildElement("materials");
	if (!pMaterialsNode)
	{
		return false;
	}
	// fill out the materials map
	for (TiXmlElement* material = pMaterialsNode->FirstChildElement("material");
		material != 0; material = material->NextSiblingElement())
	{
		std::string strMatName = GetOgreXmlAttributeString(material,"material");	
		std::string strTextureName = GetOgreXmlAttributeString(material,"texture");	
		matMap.insert(Material_Pair(strMatName,strTextureName));
	}


	// get ogre mesh base node
	TiXmlElement* pOgreMeshNode=xml.FirstChildElement("mesh");
	if (!pOgreMeshNode)
	{
		return false;
	}
	
	int iMaterialCount = 0;

	bool bUseSharedVertices = false;
	// Get a pointer to shared geometry node, if any.
	TiXmlElement* pSharedGeometryNode = pOgreMeshNode->FirstChildElement("sharedgeometry");

	// loop through submeshes
	TiXmlElement* pSubmeshesNode = pOgreMeshNode->FirstChildElement("submeshes");
    for (TiXmlElement* submesh = pSubmeshesNode->FirstChildElement();
        submesh != 0; submesh = submesh->NextSiblingElement())
    {
		// We don't support anything other than triangle_list based meshes.
		if (!EqualNoCase(submesh->Attribute("operationtype"), std::string("triangle_list")))
		{
			return false;
		}

		// Check if this mesh uses shared vertices
		bUseSharedVertices = EqualNoCase(submesh->Attribute("usesharedvertices"), std::string("true"));

		// create material
		PNPTC_MATERIAL mat;
		mat.bAlphaBlend=false;
		mat.bUseVertexColors=false;
		mat.colAmbient=PNPTC_RGB(255,255,255);
		mat.colDiffuse=PNPTC_RGB(255,255,255);
		mat.iTextures=1;
		mat.strName = GetOgreXmlAttributeString(submesh,"material");
		mat.strTexture[0]= matMap[mat.strName];

		// add material
		ObjectData.AddMaterial(mat);

		C3DMesh* pMesh = new C3DMesh(&ObjectData,false,true,true);
		if (!pMesh)
		{
			return false;
		}
		ObjectData.AddMesh(pMesh);

		// read the faces
		TiXmlElement* pFacesNode = submesh->FirstChildElement("faces");

		C3DFaceSet* pFaceSet = new C3DFaceSet(pMesh,iMaterialCount);
		pMesh->AddFaceSet(pFaceSet);
		int iFaceCount = GetOgreXmlAttributeInt(pFacesNode,"count");
		pFaceSet->ReserveFaceMem(iFaceCount);
		
		// add each face
		for (TiXmlElement* face = pFacesNode->FirstChildElement("face");
			face != 0; face = face->NextSiblingElement())
		{
			// create and add face
			PNPTC_FACE pnptc_face;
			pnptc_face.dwVert[0] = GetOgreXmlAttributeInt(face,"v1");
			pnptc_face.dwVert[2] = GetOgreXmlAttributeInt(face,"v2");
			pnptc_face.dwVert[1] = GetOgreXmlAttributeInt(face,"v3");
			pFaceSet->AddFace(pnptc_face);			
		}

		// read geometry
		TiXmlElement* pGeometryNode = 0;
		if(bUseSharedVertices)
		{
			pGeometryNode = pSharedGeometryNode;
		}
		else
		{
			pGeometryNode = submesh->FirstChildElement("geometry");
		}

		if (pGeometryNode)
        {
			int iVertexCount = GetOgreXmlAttributeInt(pGeometryNode,"vertexcount");
			pMesh->ReserveVertexMem(iVertexCount);

			// loop through vertexbuffters
			for (TiXmlElement* vertexbuffer = pGeometryNode->FirstChildElement();
				vertexbuffer != 0; vertexbuffer = vertexbuffer->NextSiblingElement())
			{
				
				// get all vertices
				for (TiXmlElement* vertex = vertexbuffer->FirstChildElement();
				vertex != 0; vertex = vertex->NextSiblingElement())
				{
					PNPTC_VERTEX vert;
					memset(&vert,0,sizeof(PNPTC_VERTEX));
					vert.colDiffuse=PNPTC_RGB(255,255,255);

					// get position
					TiXmlElement* position = vertex->FirstChildElement("position");
					if(position)
					{
						vert.vPos.x = GetOgreXmlAttributeFloat(position,"x");
						vert.vPos.y = GetOgreXmlAttributeFloat(position,"y");
						vert.vPos.z = GetOgreXmlAttributeFloat(position,"z");
						vert.vPos.z *= -1.0; // Flip handedness
					}
					// get normal
					TiXmlElement* normal = vertex->FirstChildElement("normal");
					if(normal)
					{
						vert.vNormal.x = GetOgreXmlAttributeFloat(normal,"x");
						vert.vNormal.y = GetOgreXmlAttributeFloat(normal,"y");
						vert.vNormal.z = GetOgreXmlAttributeFloat(normal,"z");
						vert.vNormal.z *= -1.0; // Flip handedness
					}
					// get texture coordinates
					TiXmlElement* texcoord = vertex->FirstChildElement("texcoord");
					if(texcoord)
					{
						vert.uv[0].u = GetOgreXmlAttributeFloat(texcoord,"u");
						vert.uv[0].v = GetOgreXmlAttributeFloat(texcoord,"v");
					}

					pMesh->AddVertex(vert);
				}			
			} 
		} // read geometry
		
		iMaterialCount++;
	} // read submeshes

	return true;
}



// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------

/// Converts an XML mesh file to binary format.
/// @param strXMLFileName The name of the XML file to be converted.
/// @return Returns <i>true</i> or <i>false</i>, whether or not the
/// file could be converted successfully.
bool ConvertToBinary(const std::string& strXMLFileName)
{
	// get file name of this plugin
	char cFileName[MAX_PATH+1];
	::GetModuleFileName(NULL,cFileName,MAX_PATH+1);
	std::string strPath=GetPath(cFileName)+"PlugIns\\utils\\";

	// get name of binary file
	std::string strBinaryFileName=strXMLFileName;
	if (EqualNoCase(Right(strBinaryFileName,4),".xml"))
	{
		strBinaryFileName=Left(strBinaryFileName,static_cast<int>(strBinaryFileName.length())-4);
	}
	if (!EqualNoCase(Right(strBinaryFileName,5),".mesh"))
	{
		strBinaryFileName+=".mesh";
	}

	// run the conversion tool
	std::string strTool="\""+strPath+"OgreXmlConverter.exe\" ";
	std::string strSource="\""+strXMLFileName+"\" ";
	std::string strDest="\""+strBinaryFileName+"\" ";
	std::string strCommandLine=strTool+strSource+strDest;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	ZeroMemory( &si, sizeof(si) );
	si.cb = sizeof(si);
	ZeroMemory( &pi, sizeof(pi) );
	if (!::CreateProcess(NULL,(LPSTR)strCommandLine.c_str(),NULL,NULL,TRUE,CREATE_NO_WINDOW,NULL,strPath.c_str(),&si,&pi))
	{
		// error converting mesh to binary
		MessageBox(NULL,(std::string("Error converting ogre mesh with command\n'")+strCommandLine+"'.").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// Wait until child process exits.
	WaitForSingleObject( pi.hProcess, INFINITE );

	// Close process and thread handles. 
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );

	// remove the xml file
	DeleteFile(strXMLFileName.c_str());

	// ok 
	return true;
}

bool C3DObjectSerializerOGRE::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/)
{
	// in either case (binary or xml), we first export into xml
	std::string strSaveTo=strFileName;
	bool bBinary=false;
	if (!EqualNoCase(Right(strSaveTo,3),"xml"))
	{
		bBinary=true;
		strSaveTo+=".xml";
	}
	if (this->m_bConvertToLowerCase)
	{
		ToLower(strSaveTo);
	}

	// -----------------------------------------------------------------------------
	// write materials
	if (this->m_bSaveMaterials && ObjectData.GetNumMaterials()>0)
	{
		// get material file name
		std::string strMaterialFile;
		std::string strExtension;
		std::string strPath;
		RemoveFileExtension(strFileName,strMaterialFile,strExtension);
		RemoveFilePath(strMaterialFile,strMaterialFile,strPath);
		if (this->m_bConvertToLowerCase)
		{
			ToLower(strMaterialFile);
		}
		ValidateOgreEntityName(strMaterialFile);
		strMaterialFile=strPath+strMaterialFile+".material";

		// save materials
		if (!this->SaveMaterials(ObjectData,ConvertToWindowsFormat(strMaterialFile),cBasePathName,true))
		{
			return false;
		}
	}

	// -----------------------------------------------------------------------------
	// write all meshes of object
	FILE* f=NULL;
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		if (pMesh)
		{
			// create (new) file
			std::string strSaveToMesh=strSaveTo;
			if (lMesh==0 || bSeparateMeshFiles || f==NULL)
			{
				// get name for this mesh
				if (bSeparateMeshFiles && ObjectData.GetNumMeshes()>1)
				{
					std::string strExtension;
					RemoveFileExtension(strSaveTo,strSaveToMesh,strExtension);
					strSaveToMesh+="_"+ConvertValidateOgreEntityName(pMesh->GetName())+strExtension;
				}
				if (this->m_bConvertToLowerCase)
				{
					ToLower(strSaveToMesh);
				}

				// open the mesh file for writing
				f=fopen(ConvertToWindowsFormat(strSaveToMesh).c_str(),"w");
				if (!f)
				{
					MessageBox(NULL,(std::string("The file ")+strSaveToMesh+std::string(" could not be opened for writing.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// -----------------------------------------------------------------------------
			// write mesh header
			if (fprintf(f,"<mesh>\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing mesh header to xml-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// -----------------------------------------------------------------------------
			// write vertices

			// write shared geometery
			DWORD dwVertices=(DWORD)pMesh->GetNumVertices();
			if (fprintf(f,"\t<sharedgeometry vertexcount=\"%d\">\n",dwVertices)<0)
			{
				// error
				MessageBox(NULL,"Error writing shared geometry header to xml-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// what elements are in the vertex buffer
			std::string strVertexBuffer("\t\t<vertexbuffer positions=\"true\"");
			if (pMesh->HasVertexColor())
			{
				strVertexBuffer+=" colours_diffuse=\"true\"";
			}
			if (pMesh->HasVertexNormals())
			{
				strVertexBuffer+=" normals=\"true\"";
			}
			char cTemp[100];
			for (int iTexCoord=0;iTexCoord<(int)pMesh->GetNumMappingCoords();iTexCoord++)
			{
				sprintf(cTemp," texture_coord_dimensions_%d=\"2\"",iTexCoord);
				strVertexBuffer+=std::string(cTemp);
			}
			sprintf(cTemp," texture_coords=\"%d\">\n",pMesh->GetNumMappingCoords());
			strVertexBuffer+=std::string(cTemp);	

			// write vertex buffer header
			if (fprintf(f,strVertexBuffer.c_str(),pMesh->GetNumVertices())<0)
			{
				// error
				MessageBox(NULL,"Error writing mesh vertex buffer header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// write vertices
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				// write vertex position
				PNPTC_VERTEX vert=pMesh->GetVertex(lVert);
				PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
				GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
				if (fprintf(f,"\t\t\t<vertex>\n\t\t\t\t<position x=\"%f\" y=\"%f\" z=\"%f\" />\n",vecPos.x,vecPos.y,vecPos.z)<0)
				{
					// error
					MessageBox(NULL,"Error writing mesh vertex position to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}

				// write color
				if (pMesh->HasVertexColor())
				{
					if (fprintf(f,"\t\t\t\t<colour_diffuse value=\"%f %f %f %f\" />\n",static_cast<float>(vert.colDiffuse.r)/255.f,static_cast<float>(vert.colDiffuse.g)/255.f,static_cast<float>(vert.colDiffuse.b)/255.f,static_cast<float>(vert.colDiffuse.a)/255.f)<0)
					{
						// error
						MessageBox(NULL,"Error writing mesh vertex color to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write normals
				if (pMesh->HasVertexNormals())
				{
					PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
					GlobalVectorTransform(vecNormal,1,iUpRotation,iFlipAxis);
					if (fprintf(f,"\t\t\t\t<normal x=\"%f\" y=\"%f\" z=\"%f\" />\n",vecNormal.x,vecNormal.y,vecNormal.z)<0)
					{
						// error
						MessageBox(NULL,"Error writing mesh vertex normal to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write texture coordinates
				for (int iTex=0;iTex<(int)pMesh->GetNumMappingCoords();iTex++)
				{
					if (fprintf(f,"\t\t\t\t<texcoord u=\"%f\" v=\"%f\" />\n",vert.uv[iTex].u,vert.uv[iTex].v)<0)
					{
						// error
						MessageBox(NULL,"Error writing mesh vertex texture coordinates to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// finish vertex
				if (fprintf(f,"\t\t\t</vertex>\n")<0)
				{
					// error
					MessageBox(NULL,"Error finishing mesh vertex to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}

			// write vertex buffer header
			if (fprintf(f,"\t\t</vertexbuffer>\n",pMesh->GetNumVertices())<0)
			{
				// error
				MessageBox(NULL,"Error writing mesh vertex buffer header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// finish shared geometry
			if (fprintf(f,"\t</sharedgeometry>\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing shared geometry header to xml-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}


			// ----------------------------------------------------------------------------
			// write all faces (sub meshes)

			// write sub meshes header
			if (fprintf(f,"\t<submeshes>\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing meshes header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// go through all face sets
			for (unsigned long nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet && pFaceSet->GetNumFaces()>0)
				{
					// get sub mesh material
					int iMaterial=pFaceSet->GetMaterial();
					const PNPTC_MATERIAL* pMaterial=NULL;
					if (iMaterial>=0 && iMaterial<static_cast<int>(ObjectData.GetNumMaterials()))
					{
						pMaterial=ObjectData.GetMaterial(iMaterial);
					}

					if (pMaterial)
					{
						// write sub mesh header
						if (fprintf(f,"\t\t<submesh material=\"%s\" usesharedvertices=\"true\" use32bitindexes=\"true\" operationtype=\"triangle_list\">\n",ConvertValidateOgreEntityName(pMaterial->strName).c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing sub mesh header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
					else
					{
						// write sub mesh header without material
						if (fprintf(f,"\t\t<submesh material=\"default\" usesharedvertices=\"true\" use32bitindexes=\"true\" operationtype=\"triangle_list\">\n")<0)
						{
							// error
							MessageBox(NULL,"Error writing sub mesh header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}

					// write face set header
					if (fprintf(f,"\t\t\t<faces count=\"%d\">\n",pFaceSet->GetNumFaces())<0)
					{
						// error
						MessageBox(NULL,"Error writing mesh faces header to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}

					// write all faces
					for (unsigned long nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
					{
						// write face 
						PNPTC_FACE face=pFaceSet->GetFace(nFace);
						if (fprintf(f,"\t\t\t\t<face v1=\"%d\" v2=\"%d\" v3=\"%d\" />\n",face.dwVert[0],face.dwVert[1],face.dwVert[2])<0)
						{
							// error
							MessageBox(NULL,"Error writing mesh face to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}

					// finish face set
					if (fprintf(f,"\t\t\t</faces>\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing mesh faces to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}

					// finish sub mesh
					if (fprintf(f,"\t\t</submesh>\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing sub mesh to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}

			// finish sub meshes 
			if (fprintf(f,"\t</submeshes>\n")<0)
			{
				// error
				MessageBox(NULL,"Error finishing sub meshes to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// finishing mesh
			if (fprintf(f,"</mesh>\n")<0)
			{
				// error
				MessageBox(NULL,"Error finishing mesh to xml-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}

			// finish file writing?
			if (bSeparateMeshFiles || lMesh==(ObjectData.GetNumMeshes()-1))
			{
				// close the file
				fclose(f);
				f=NULL;

				// convert to binary mesh
				if (bBinary)
				{
					if (!ConvertToBinary(ConvertToWindowsFormat(strSaveToMesh)))
					{
						return false;
					}
				}
			}
		}
	}

	// close the file
	if (f)
	{
		fclose(f);
	}

	// ok
	return true;
}

bool C3DObjectSerializerOGRE::SaveMaterial(const PNPTC_MATERIAL* pMaterial,FILE* fileMaterial,const std::string& strRelTexturePath)
{
	// check input
	if (!pMaterial)
	{
		MessageBox(NULL,"Error writing material to ogre material-file.\nInvalid material pointer.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}
	if (!fileMaterial)
	{
		MessageBox(NULL,"Error writing material to ogre material-file.\nInvalid file pointer.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write material header
	if (fprintf(fileMaterial,"material %s\n{\n\ttechnique\n\t{\n\t\tpass\n\t\t{\n",ConvertValidateOgreEntityName(pMaterial->strName).c_str())<0)
	{
		// error
		MessageBox(NULL,"Error writing material header to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write colors
	if (pMaterial->bUseVertexColors)
	{
		if (!pMaterial->bAlphaBlend)
		{
			if (fprintf(fileMaterial,"\t\t\tambient 0 0 0 0\n\t\t\tdiffuse vertexcolour\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing material colors to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}
		else
		{
			if (fprintf(fileMaterial,"\t\t\tambient vertexcolour\n\t\t\tdiffuse vertexcolour\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing material colors to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}
	}
	else
	{
		// write ambient color
		if (fprintf(fileMaterial,"\t\t\tambient %f %f %f\n",static_cast<float>(pMaterial->colAmbient.r)/255.f,static_cast<float>(pMaterial->colAmbient.g)/255.f,static_cast<float>(pMaterial->colAmbient.b)/255.f)<0)
		{
			// error
			MessageBox(NULL,"Error writing material ambient color to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}			

		// write diffuse color
		if (fprintf(fileMaterial,"\t\t\tdiffuse %f %f %f\n",static_cast<float>(pMaterial->colDiffuse.r)/255.f,static_cast<float>(pMaterial->colDiffuse.g)/255.f,static_cast<float>(pMaterial->colDiffuse.b)/255.f)<0)
		{
			// error
			MessageBox(NULL,"Error writing material ambient color to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}			
	}

	// write alpha blending
	if (pMaterial->bAlphaBlend)
	{
		if (fprintf(fileMaterial,"\t\t\tscene_blend alpha_blend\n")<0)
		{
			// error
			MessageBox(NULL,"Error writing material alpha blending to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// write all texture stages
	for (unsigned int iTex=0;iTex<static_cast<unsigned int>(pMaterial->iTextures);iTex++)
	{
		// write texture stage
		if (fprintf(fileMaterial,"\t\t\ttexture_unit\n\t\t\t{\n\t\t\t\ttexture %s\n\t\t\t\ttex_coord_set %d\n\t\t\t\tcolour_op_ex modulate %s src_texture\n\t\t\t}\n",(strRelTexturePath+ConvertValidateOgreEntityName(pMaterial->strTexture[iTex])).c_str(),iTex,((iTex>0)?"src_current":"src_diffuse"))<0)
		{
			// error
			MessageBox(NULL,"Error finishing material to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// finish material
	if (fprintf(fileMaterial,"\t\t}\n\t}\n}\n")<0)
	{
		// error
		MessageBox(NULL,"Error finishing material to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	return true;
}

bool C3DObjectSerializerOGRE::SaveMaterials(const C3DObjectData& ObjectData,const std::string& strMaterialFile,const char* cBasePathName/*=NULL*/,bool bSeperateFiles/*=false*/)
{
	// create material file
	FILE* f=NULL;
	if (!bSeperateFiles)
	{
		f=fopen(ConvertToWindowsFormat(strMaterialFile).c_str(),"w");
		if (!f)
		{
			MessageBox(NULL,(std::string("The file ")+strMaterialFile+std::string(" could not be opened for writing.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// get relative textures path
	std::string strRelTexturePath="";
	if (cBasePathName)
	{
		std::string strFullPath=GetPath(strMaterialFile);
		strRelTexturePath=Right(strFullPath,static_cast<unsigned int>(strFullPath.length()-strlen(cBasePathName)));
	}

	// export all materials
	for (unsigned int i=0;i<ObjectData.GetNumMaterials();i++)
	{
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(i);
		if (pMaterial)
		{
			// one file for every material?
			if (bSeperateFiles)
			{
				std::string strThisMaterialFile=GetPath(strMaterialFile)+ConvertValidateOgreEntityName(pMaterial->strName)+".material";
				f=fopen(ConvertToWindowsFormat(strThisMaterialFile).c_str(),"w");
				if (!f)
				{
					MessageBox(NULL,(std::string("The file ")+strThisMaterialFile+std::string(" could not be opened for writing ogre material.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// save this material
			if (!this->SaveMaterial(pMaterial,f,strRelTexturePath))
			{
				if (f) fclose(f);
				return false;
			}

			// close file if one file for every material
			if (bSeperateFiles && f)
			{
				fclose(f);
				f=NULL;
			}
		}
	}

	// close material file
	if (f) fclose(f);

	// ok
	return true;
}
