/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerOGRE.cpp
// 3d object serialization class for ogre mesh files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: (Ralf Westphal) Created.
//  19. Aug. 2007   : (Ralf Westphal) Improved ogre entity naming conformity.
//  03. Dec. 2008	: (Ralf Westphal) Can now correctly export multiple meshes into one file.
//  07. Jan. 2009	: (jacmoe) Added Ogre mesh xml loader.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_DEPRECATE 1

#include "3DObjectData.h"
#include ".\3dobjectserializerogre.h"
#include "../PnPTCUtils/Utils.h"
#include "./textureresourcemanager.h"
#include "../PnPTCUtils/dxutils.h"
#include "./OgreUtils.h"
#include "../PnPTCUtils/XmlUtils.h"
#include "../3rd-party/tinyxml/tinyxml.h"
#include <map>

C3DObjectSerializerOGRE::C3DObjectSerializerOGRE(void)
	: m_bSaveMaterials(true)
	, m_bConvertToLowerCase(false)
	, m_bSetBlackAmbientToDiffuse(false)
	, m_bRemoveMeshInstanceString(false)
{
}

C3DObjectSerializerOGRE::~C3DObjectSerializerOGRE(void)
{
}




// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Loading
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------



/// Settings of simplified ogre materials
typedef struct OgreMat
{
	std::string strTexture;
	bool		bAlpha;
} OgreMat;

/// A map to temporarily store the materials.
typedef std::map<std::string,OgreMat> OgreMatMap;

bool C3DObjectSerializerOGRE::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	//	extract path and file name;
	if (strFileName.length()==0)
	{
		return false;
	}

	std::string strDummy;
	std::string strPath ;
	std::string yeahRight;
	RemoveFilePath(strFileName,strDummy,strPath);
	RemoveFileExtension(strFileName,strDummy,yeahRight);

	// open xml document
	TiXmlDocument xml;
	if (!xml.LoadFile(strFileName))
	{
		return false;
	}

	// open material xml document
	OgreMatMap matMap;
	TiXmlDocument xml_materials;
	if (xml_materials.LoadFile(strDummy + ".pnpogremat" ))
	{
		// get materials node
		TiXmlElement* pMaterialsNode = xml_materials.FirstChildElement("materials");
		if (pMaterialsNode)
		{
			// fill out the materials map
			for (TiXmlElement* material = pMaterialsNode->FirstChildElement("material");
				material != 0; material = material->NextSiblingElement())
			{
				OgreMat mat;
				mat.strTexture=GetXmlAttributeString(material,"texture");
				mat.bAlpha=GetXmlAttributeBool(material,"alpha");
				matMap[GetXmlAttributeString(material,"material")]=mat;
			}
		}
	}


	// get ogre mesh base node
	TiXmlElement* pOgreMeshNode=xml.FirstChildElement("mesh");
	if (!pOgreMeshNode)
	{
		return false;
	}
	
	int iMaterialCount = 0;

	bool bUseSharedVertices = false;
	// Get a pointer to shared geometry node, if any.
	TiXmlElement* pSharedGeometryNode = pOgreMeshNode->FirstChildElement("sharedgeometry");

	// loop through submeshes
	TiXmlElement* pSubmeshesNode = pOgreMeshNode->FirstChildElement("submeshes");
    for (TiXmlElement* submesh = pSubmeshesNode->FirstChildElement();
        submesh != 0; submesh = submesh->NextSiblingElement())
    {
		// We don't support anything other than triangle_list based meshes.
		if (!EqualNoCase(submesh->Attribute("operationtype"), std::string("triangle_list")))
		{
			return false;
		}

		// Check if this mesh uses shared vertices
		bUseSharedVertices = EqualNoCase(submesh->Attribute("usesharedvertices"), std::string("true"));

		// create material
		PNPTC_MATERIAL mat;
		mat.bAlphaBlend=false;
		mat.bUseVertexColors=false;
		mat.colAmbient=PNPTC_RGB(255,255,255);
		mat.colDiffuse=PNPTC_RGB(255,255,255);
		mat.iTextures=1;
		mat.strName = GetXmlAttributeString(submesh,"material");
		mat.strTexture[0]="";

		// apply settings from material script
		OgreMatMap::iterator itMaterial=matMap.find(mat.strName);
		if (itMaterial!=matMap.end())
		{
			mat.strTexture[0]=itMaterial->second.strTexture;
			mat.bAlphaBlend=itMaterial->second.bAlpha;
		}

		// add material
		ObjectData.AddMaterial(mat);

		// create mesh
		C3DMesh* pMesh = new C3DMesh(&ObjectData,false,true,true);
		if (!pMesh)
		{
			return false;
		}
		ObjectData.AddMesh(pMesh);

		// read the faces
		TiXmlElement* pFacesNode = submesh->FirstChildElement("faces");
		C3DFaceSet* pFaceSet = new C3DFaceSet(pMesh,iMaterialCount);
		pMesh->AddFaceSet(pFaceSet);
		int iFaceCount = GetXmlAttributeInt(pFacesNode,"count");
		pFaceSet->ReserveFaceMem(iFaceCount);
		
		// add each face
		for (TiXmlElement* face = pFacesNode->FirstChildElement("face");
			face != 0; face = face->NextSiblingElement())
		{
			// create and add face
			PNPTC_FACE pnptc_face;
			pnptc_face.dwVert[0] = GetXmlAttributeInt(face,"v1");
			pnptc_face.dwVert[2] = GetXmlAttributeInt(face,"v2");
			pnptc_face.dwVert[1] = GetXmlAttributeInt(face,"v3");
			pFaceSet->AddFace(pnptc_face);			
		}

		// read geometry
		TiXmlElement* pGeometryNode = 0;
		if(bUseSharedVertices)
		{
			pGeometryNode = pSharedGeometryNode;
		}
		else
		{
			pGeometryNode = submesh->FirstChildElement("geometry");
		}

		if (pGeometryNode)
        {
			int iVertexCount = GetXmlAttributeInt(pGeometryNode,"vertexcount");
			pMesh->ReserveVertexMem(iVertexCount);

			// loop through vertexbuffters
			for (TiXmlElement* vertexbuffer = pGeometryNode->FirstChildElement();
				vertexbuffer != 0; vertexbuffer = vertexbuffer->NextSiblingElement())
			{
				
				// get all vertices
				for (TiXmlElement* vertex = vertexbuffer->FirstChildElement();
				vertex != 0; vertex = vertex->NextSiblingElement())
				{
					PNPTC_VERTEX vert;
					memset(&vert,0,sizeof(PNPTC_VERTEX));
					vert.colDiffuse=PNPTC_RGB(255,255,255);

					// get position
					TiXmlElement* position = vertex->FirstChildElement("position");
					if(position)
					{
						vert.vPos.x = GetXmlAttributeFloat(position,"x");
						vert.vPos.y = GetXmlAttributeFloat(position,"y");
						vert.vPos.z = GetXmlAttributeFloat(position,"z");
						vert.vPos.z *= -1.0; // Flip handedness
					}
					// get normal
					TiXmlElement* normal = vertex->FirstChildElement("normal");
					if(normal)
					{
						vert.vNormal.x = GetXmlAttributeFloat(normal,"x");
						vert.vNormal.y = GetXmlAttributeFloat(normal,"y");
						vert.vNormal.z = GetXmlAttributeFloat(normal,"z");
						vert.vNormal.z *= -1.0; // Flip handedness
					}
					// get texture coordinates
					TiXmlElement* texcoord = vertex->FirstChildElement("texcoord");
					if(texcoord)
					{
						vert.uv[0].u = GetXmlAttributeFloat(texcoord,"u");
						vert.uv[0].v = GetXmlAttributeFloat(texcoord,"v");
					}

					pMesh->AddVertex(vert);
				}			
			} 
		} // read geometry
		
		iMaterialCount++;
	} // read submeshes

	return true;
}




// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------
// Saving
// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------




/// Converts an XML mesh file to binary format.
/// @param strXMLFileName The name of the XML file to be converted.
/// @return Returns <i>true</i> or <i>false</i>, whether or not the
/// file could be converted successfully.
bool ConvertToBinary(const std::string& strXMLFileName)
{
	// get file name of this plugin
	char cFileName[MAX_PATH+1];
	::GetModuleFileName(NULL,cFileName,MAX_PATH+1);
	std::string strPath=GetPath(cFileName)+"PlugIns\\utils\\";

	// get name of binary file
	std::string strBinaryFileName=strXMLFileName;
	if (EqualNoCase(Right(strBinaryFileName,4),".xml"))
	{
		strBinaryFileName=Left(strBinaryFileName,static_cast<int>(strBinaryFileName.length())-4);
	}
	if (!EqualNoCase(Right(strBinaryFileName,5),".mesh"))
	{
		strBinaryFileName+=".mesh";
	}

	// run the conversion tool
	std::string strTool="\""+strPath+"OgreXmlConverter.exe\" ";
	std::string strSource="\""+strXMLFileName+"\" ";
	std::string strDest="\""+strBinaryFileName+"\" ";
	std::string strCommandLine=strTool+strSource+strDest;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	ZeroMemory( &si, sizeof(si) );
	si.cb = sizeof(si);
	ZeroMemory( &pi, sizeof(pi) );
	if (!::CreateProcess(NULL,(LPSTR)strCommandLine.c_str(),NULL,NULL,TRUE,CREATE_NO_WINDOW,NULL,strPath.c_str(),&si,&pi))
	{
		// error converting mesh to binary
		MessageBox(NULL,(std::string("Error converting ogre mesh with command\n'")+strCommandLine+"'.").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// Wait until child process exits.
	WaitForSingleObject( pi.hProcess, INFINITE );

	// Close process and thread handles. 
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );

	// remove the xml file
	DeleteFile(strXMLFileName.c_str());

	// ok 
	return true;
}

/// Returns the name from the material data base, which
/// should be used for a given material.
/// TODO : this is a hack. database needs to be implemented.
/// But maybe we do not really need a database.
/// @param Material The material, for which the name should be returned.
/// @param strMaterialDataBase The file name of the material data base.
/// @return Returns the name, which should be used for this material
std::string	GetMaterialNameFromDataBase(const PNPTC_MATERIAL& Material,const std::string& strMaterialDataBase)
{
	// check textures
	std::string strMaterialName="mat_";
	for (int iTexture=0;iTexture<Material.iTextures;++iTexture)
	{
		// add name of texture
		std::string strFilename;
		std::string strExtension;
		RemoveFileExtension(Material.strTexture[0],strFilename,strExtension);
		strMaterialName+=strFilename+"_";
	}

	// add color value
	char cTemp[20];
	sprintf(cTemp,"%02d%02d%02d",Material.colDiffuse.r,Material.colDiffuse.g,Material.colDiffuse.b);
	strMaterialName+=cTemp;

	// return that material name
	return OgreUtils::ValidateOgreEntityName(strMaterialName,true);
}

/// Returns the file name from the material data base, which
/// should be used for a given material.
/// TODO : this is a hack. database needs to be implemented.
/// But maybe we do not really need a database.
/// @param Material The material, for which the name should be returned.
/// @param strMaterialDataBase The file name of the material data base.
/// @param strBasePath The base path name for storing materials.
/// @param strMaterialName The name of the material.
/// @return Returns the file name, which should be used for this material
std::string GetMaterialFileNameFromDataBase(const PNPTC_MATERIAL& Material,const std::string& strBasePath,const std::string& strMaterialName)
{
	// get base path
	std::string strPath=strBasePath;
	if (Material.iTextures==0 && strPath.length()>1)
	{
		// no textures => so store them one folder higher in the hierarchie
		if (Right(strPath,1)=="\\" || Right(strPath,1)=="/")
		{
			strPath=Left(strPath,static_cast<int>(strPath.size())-1);
		}
		strPath=GetPath(strPath);
	}
	
	// return material name
	return strPath+strMaterialName+".material";
}


bool WriteVertexToXml(TiXmlElement& ParentXmlElement,const PNPTC_VERTEX& vert,const D3DXMATRIX& matTransform,bool bColors,bool bNormals,int iTexCoords,float fScale,int iUpRotation,int iFlipAxis)
{
	// create vertex
	TiXmlElement xmlVertex("vertex");
	TiXmlElement* pVertexXmlElement=dynamic_cast<TiXmlElement*>(ParentXmlElement.InsertEndChild(xmlVertex));
	if (!pVertexXmlElement)
	{
		MessageBox(NULL,"Error creating mesh vertex position xml node.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// add position
	PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
	GlobalVectorTransform(vecPos,matTransform,fScale,iUpRotation,iFlipAxis);
	TiXmlElement xmlPosition("position");
	xmlPosition.SetAttribute("x",ToString(vecPos.x));
	xmlPosition.SetAttribute("y",ToString(vecPos.y));
	xmlPosition.SetAttribute("z",ToString(vecPos.z));
	if (!pVertexXmlElement->InsertEndChild(xmlPosition))
	{
		MessageBox(NULL,"Error writing mesh vertex position to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// add color
	if (bColors)
	{
		TiXmlElement xmlColor("colour_diffuse");
		xmlColor.SetAttribute("value",ToString(static_cast<float>(vert.colDiffuse.r)/255.f)+" "+ToString(static_cast<float>(vert.colDiffuse.g)/255.f)+" "+ToString(static_cast<float>(vert.colDiffuse.b)/255.f)+" "+ToString(static_cast<float>(vert.colDiffuse.a)/255.f));
		if (!pVertexXmlElement->InsertEndChild(xmlColor))
		{
			MessageBox(NULL,"Error writing mesh vertex color to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}
	
	// add normals
	if (bNormals)
	{
		PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
		D3DXMATRIX matRot=matTransform;
		matRot._41=0;
		matRot._42=0;
		matRot._43=0;
		GlobalVectorTransform(vecNormal,matRot,1,iUpRotation,iFlipAxis);
		TiXmlElement xmlNormal("normal");
		xmlNormal.SetAttribute("x",ToString(vecNormal.x));
		xmlNormal.SetAttribute("y",ToString(vecNormal.y));
		xmlNormal.SetAttribute("z",ToString(vecNormal.z));
		if (!pVertexXmlElement->InsertEndChild(xmlNormal))
		{
			MessageBox(NULL,"Error writing mesh vertex normal to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// add texture coordinates
	for (int iTex=0;iTex<iTexCoords;++iTex)
	{
		TiXmlElement xmlUV("texcoord");
		xmlUV.SetAttribute("u",ToString(vert.uv[iTex].u));
		xmlUV.SetAttribute("v",ToString(vert.uv[iTex].v));
		if (!pVertexXmlElement->InsertEndChild(xmlUV))
		{
			MessageBox(NULL,"Error writing mesh vertex uv-coordinate to ogre-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// ok
	return true;
}

bool C3DObjectSerializerOGRE::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// in either case (binary or xml), we first export into xml
	std::string strSaveTo=strFileName;
	bool bBinary=false;
	if (!EqualNoCase(Right(strSaveTo,3),"xml"))
	{
		bBinary=true;
		strSaveTo+=".xml";
	}
	if (this->m_bConvertToLowerCase)
	{
		ToLower(strSaveTo);
	}

	// -----------------------------------------------------------------------------
	// write materials
	if (this->m_bSaveMaterials && ObjectData.GetNumMaterials()>0)
	{
		// get material file name
		std::string strMaterialFile;
		std::string strExtension;
		RemoveFileExtension(strFileName,strMaterialFile,strExtension);
		strMaterialFile=OgreUtils::ValidateOgreEntityName(::GetFileName(strMaterialFile),this->m_bConvertToLowerCase);
		strMaterialFile=GetPath(ConvertToWindowsFormat(strFileName))+strMaterialFile+".material";

		// save materials
		if (!this->SaveMaterials(ObjectData,ConvertToWindowsFormat(strMaterialFile),cBasePathName/*,true*/))
		{
			return false;
		}
	}

	// -----------------------------------------------------------------------------
	// write all meshes of object

	// can we use shared geometry data
	bool bSharedGeometry=bSeparateMeshFiles || (ObjectData.GetNumMeshes()==1);

	// test whether or not shared geometry contains color and normal data and get the number of texture coordinates
	bool bSharedGeometryColors=false;
	bool bSharedGeometryNormals=false;
	int iSharedGeometryTextureCoordinates=0;
	unsigned int nSharedGeometryVertexOffset=0;
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		if (pMesh)
		{
			// check whether or not to use colors and normals
			bSharedGeometryColors=bSharedGeometryColors || pMesh->HasVertexColor();
			bSharedGeometryNormals=bSharedGeometryNormals || pMesh->HasVertexColor();
			iSharedGeometryTextureCoordinates=maximum(iSharedGeometryTextureCoordinates,static_cast<int>(pMesh->GetNumMappingCoords()));
		}
	}

	// iterate through all export passes
	TiXmlDocument* pXmlDocument=NULL;
	TiXmlElement* pMeshXmlElement=NULL;
	TiXmlElement* pSharedGeometryXmlElement=NULL;
	TiXmlElement* pSubMeshesXmlElement=NULL;
	unsigned long lNumMeshes=static_cast<unsigned long>((bSeparateMeshFiles)?(ObjectData.GetNumMeshes()):(ObjectData.GetNumMeshInstances()));
	for (unsigned long lMesh=0;lMesh<lNumMeshes;lMesh++)
	{
		// get mesh and the local transformation of it
		D3DXMATRIX matLocalTransform;
		D3DXMatrixIdentity(&matLocalTransform);
		const C3DMesh* pMesh=NULL;
		if (bSeparateMeshFiles)
		{
			ObjectData.GetMesh(lMesh);			
		}
		else
		{
			const C3DMeshInstance* pMeshInstance=ObjectData.GetMeshInstance(lMesh);
			if (pMeshInstance)
			{
				pMesh=pMeshInstance->GetMesh();
				matLocalTransform=pMeshInstance->GetLocalTransform();
			}
		}
		if (pMesh)
		{
			// ----------------------------------------------------------------------------
			// create/initialize new file
			if (bSeparateMeshFiles || pXmlDocument==NULL)
			{
				// create new xml document
				SAFE_DELETE(pXmlDocument);
				pMeshXmlElement=NULL;
				pSharedGeometryXmlElement=NULL;
				pSubMeshesXmlElement=NULL;
				pXmlDocument=new TiXmlDocument();
				if (!pXmlDocument)
				{
					MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create xml document.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// create root element
				TiXmlElement xmlMesh("mesh");
				pMeshXmlElement=dynamic_cast<TiXmlElement*>(pXmlDocument->InsertEndChild(xmlMesh));
				if (!pMeshXmlElement)
				{
					MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create xml base node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// ----------------------------------------------------------------------------
			// add all vertices to shared geometry
			if (bSharedGeometry)
			{
				// check whether or not to use colors and normals for separate file export
				if (bSeparateMeshFiles)
				{
					bSharedGeometryColors=pMesh->HasVertexColor();
					bSharedGeometryNormals=pMesh->HasVertexColor();
					iSharedGeometryTextureCoordinates=static_cast<int>(pMesh->GetNumMappingCoords());
				}

				// add shared geometry node
				unsigned int nVertices=static_cast<unsigned int>(pMesh->GetNumVertices());
				if (!pSharedGeometryXmlElement)
				{
					TiXmlElement xmlGeometry("sharedgeometry");
					pSharedGeometryXmlElement=dynamic_cast<TiXmlElement*>(pMeshXmlElement->InsertEndChild(xmlGeometry));
					if (!pSharedGeometryXmlElement)
					{
						MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create shared geometry xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
				}
				pSharedGeometryXmlElement->SetAttribute("vertexcount",static_cast<int>(nSharedGeometryVertexOffset+nVertices));

				// add vertex buffer
				TiXmlElement xmlVB("vertexbuffer");
				xmlVB.SetAttribute("positions","true");
				if (bSharedGeometryColors)
				{
					xmlVB.SetAttribute("colours_diffuse","true");
				}
				if (bSharedGeometryNormals)
				{
					xmlVB.SetAttribute("normals","true");
				}
				for (int iTexCoord=0;iTexCoord<iSharedGeometryTextureCoordinates;iTexCoord++)
				{
					char cTemp[100];
					sprintf(cTemp,"texture_coord_dimensions_%d",iTexCoord);
					xmlVB.SetAttribute(cTemp,2);
				}
				xmlVB.SetAttribute("texture_coords",iSharedGeometryTextureCoordinates);
				TiXmlElement* pVBXmlElement=dynamic_cast<TiXmlElement*>(pSharedGeometryXmlElement->InsertEndChild(xmlVB));
				if (!pVBXmlElement)
				{
					MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create shared geometry VB xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// export all vertices
				for (unsigned int nVertex=0;nVertex<nVertices;++nVertex)
				{
					if (!WriteVertexToXml(*pVBXmlElement,pMesh->GetVertex(nVertex),matLocalTransform,bSharedGeometryColors,bSharedGeometryNormals,iSharedGeometryTextureCoordinates,fScale,iUpRotation,iFlipAxis))
					{
						MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create shared geometry vertex xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
				}
			}


			// ----------------------------------------------------------------------------
			// write all faces (sub meshes)

			// create sub meshes element
			if (!pSubMeshesXmlElement)
			{
				TiXmlElement xmlSubMeshes("submeshes");
				pSubMeshesXmlElement=dynamic_cast<TiXmlElement*>(pMeshXmlElement->InsertEndChild(xmlSubMeshes));
				if (!pSubMeshesXmlElement)
				{
					MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create sub meshes xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}				
			}

			// go through all face sets
			for (unsigned long nFaceSet=0;nFaceSet<pMesh->GetNumFaceSets();nFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(nFaceSet);
				if (pFaceSet && pFaceSet->GetNumFaces()>0)
				{
					// get sub mesh material
					int iMaterial=pFaceSet->GetMaterial();
					const PNPTC_MATERIAL* pMaterial=NULL;
					if (iMaterial>=0 && iMaterial<static_cast<int>(ObjectData.GetNumMaterials()))
					{
						pMaterial=ObjectData.GetMaterial(iMaterial);
					}

					// get material name
					std::string strMaterialName="default";
					if (pMaterial)
					{
						// read material name
						strMaterialName=OgreUtils::ValidateOgreEntityName(pMaterial->strName,false);
						if (!this->m_strMaterialDataBase.empty())
						{
							strMaterialName=GetMaterialNameFromDataBase(*pMaterial,this->m_strMaterialDataBase);
						}
					}

					// create sub mesh
					TiXmlElement xmlSubMesh("submesh");
					xmlSubMesh.SetAttribute("material",strMaterialName);
					xmlSubMesh.SetAttribute("usesharedvertices",(bSharedGeometry)?"true":"false");
					xmlSubMesh.SetAttribute("operationtype","triangle_list");
					TiXmlElement* pSubMeshXmlElement=dynamic_cast<TiXmlElement*>(pSubMeshesXmlElement->InsertEndChild(xmlSubMesh));
					if (!pSubMeshXmlElement)
					{
						MessageBox(NULL,(std::string("Error writing sub mesh header to ogre-file '")+strSaveTo+std::string("'.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// create geometry for this sub mesh if not using shared geometry
					TiXmlElement* pLocalGeometryXmlElement=NULL;
					TiXmlElement* pLocalVBXmlElement=NULL;
					if (!bSharedGeometry)
					{
						// add geometry node
						TiXmlElement xmlGeometry("geometry");
						pLocalGeometryXmlElement=dynamic_cast<TiXmlElement*>(pSubMeshXmlElement->InsertEndChild(xmlGeometry));
						if (!pLocalGeometryXmlElement)
						{
							MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create geometry xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}

						// add vertex buffer
						TiXmlElement xmlVB("vertexbuffer");
						xmlVB.SetAttribute("positions","true");
						if (pMesh->HasVertexColor())
						{
							xmlVB.SetAttribute("colours_diffuse","true");
						}
						if (pMesh->HasVertexNormals())
						{
							xmlVB.SetAttribute("normals","true");
						}
						for (int iTexCoord=0;iTexCoord<static_cast<int>(pMesh->GetNumMappingCoords());iTexCoord++)
						{
							char cTemp[100];
							sprintf(cTemp,"texture_coord_dimensions_%d",iTexCoord);
							xmlVB.SetAttribute(cTemp,2);
						}
						xmlVB.SetAttribute("texture_coords",static_cast<int>(pMesh->GetNumMappingCoords()));
						pLocalVBXmlElement=dynamic_cast<TiXmlElement*>(pLocalGeometryXmlElement->InsertEndChild(xmlVB));
						if (!pLocalVBXmlElement)
						{
							MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create geometry VB xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}
					}

					// create face set header
					TiXmlElement xmlFaces("faces");
					xmlFaces.SetAttribute("count",static_cast<int>(pFaceSet->GetNumFaces()));
					TiXmlElement* pFacesXmlElement=dynamic_cast<TiXmlElement*>(pSubMeshXmlElement->InsertEndChild(xmlFaces));
					if (!pFacesXmlElement)
					{
						MessageBox(NULL,(std::string("Error writing faces header to ogre-file '")+strSaveTo+std::string("'.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// write all faces
					unsigned int nMaxIndex=0;
					std::map<unsigned int,unsigned int> mapIndices;
					for (unsigned long nFace=0;nFace<pFaceSet->GetNumFaces();nFace++)
					{
						// get vertex indices
						PNPTC_FACE face=pFaceSet->GetFace(nFace);
						unsigned int nV[3];
						nV[0]=face.dwVert[0]+nSharedGeometryVertexOffset;
						nV[1]=face.dwVert[1]+nSharedGeometryVertexOffset;
						nV[2]=face.dwVert[2]+nSharedGeometryVertexOffset;

						// convert vertices to unshared geometry
						if (!bSharedGeometry && pLocalVBXmlElement)
						{
							for (unsigned int n=0;n<3;++n)
							{
								// new index?
								if (mapIndices.find(nV[n])==mapIndices.end())
								{
									// add this vertex
									if (!WriteVertexToXml(*pLocalVBXmlElement,pMesh->GetVertex(nV[n]-nSharedGeometryVertexOffset),matLocalTransform,pMesh->HasVertexColor(),pMesh->HasVertexNormals(),static_cast<int>(pMesh->GetNumMappingCoords()),fScale,iUpRotation,iFlipAxis))
									{
										MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveTo+std::string(".\nCould not create geometry vertex xml node.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
										return false;
									}

									// add this index value
									mapIndices[nV[n]]=static_cast<unsigned int>(mapIndices.size());
								}

								// set index
								nV[n]=mapIndices[nV[n]];
							}
						}

						// write face 
						TiXmlElement xmlFace("face");
						xmlFace.SetAttribute("v1",static_cast<int>(nV[0]));
						xmlFace.SetAttribute("v2",static_cast<int>(nV[1]));
						xmlFace.SetAttribute("v3",static_cast<int>(nV[2]));
						if (!pFacesXmlElement->InsertEndChild(xmlFace))
						{
							// error
							MessageBox(NULL,(std::string("Error writing face to ogre-file '")+strSaveTo+std::string("'.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}

						// compute maximum index
						nMaxIndex=maximum(nMaxIndex,nV[0]);
						nMaxIndex=maximum(nMaxIndex,nV[1]);
						nMaxIndex=maximum(nMaxIndex,nV[2]);
					}

					// use 32 bit indices?
					pSubMeshXmlElement->SetAttribute("use32bitindexes",(nMaxIndex>65535)?"true":"false");

					// set number of vertices of local geometry
					if (!bSharedGeometry && pLocalGeometryXmlElement)
					{
						pLocalGeometryXmlElement->SetAttribute("vertexcount",static_cast<int>(mapIndices.size()));
					}
				}
			}

			// write the file
			if (bSeparateMeshFiles || lMesh==(ObjectData.GetNumMeshes()-1))
			{
				// get save file name
				std::string strSaveToMesh=strSaveTo;
				if (bSeparateMeshFiles)
				{
					std::string strExtension;
					RemoveFileExtension(strSaveTo,strSaveToMesh,strExtension);
					if (bSeparateMeshFiles && this->m_bRemoveMeshInstanceString)
					{
						// TODO : remove instance string
						strSaveToMesh+="_"+OgreUtils::ValidateOgreEntityName(pMesh->GetName(),this->m_bConvertToLowerCase)+"."+strExtension;
					}
					else
					{
						strSaveToMesh+="_"+OgreUtils::ValidateOgreEntityName(pMesh->GetName(),this->m_bConvertToLowerCase)+"."+strExtension;
					}
				}
				if (this->m_bConvertToLowerCase)
				{
					ToLower(strSaveToMesh);
				}

				// save the file
				if (!pXmlDocument->SaveFile(strSaveToMesh))
				{
					MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveToMesh+std::string(".\nError writing file.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// convert to binary mesh
				if (bBinary)
				{
					/*if (!ConvertToBinary(ConvertToWindowsFormat(strSaveToMesh)))
					{
						MessageBox(NULL,(std::string("Error exporting mesh to ")+strSaveToMesh+std::string(".\nError converting xml to binary.")).c_str(),"Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}*/
				}
			}

			// update shared geometry info
			if (!bSeparateMeshFiles)
			{
				nSharedGeometryVertexOffset+=static_cast<unsigned int>(pMesh->GetNumVertices());
			}
		}
	}
	SAFE_DELETE(pXmlDocument);


	// ------------------------------------------------------------------------------------------------
	// save all textures
	bool bReturn=true;
	if (bExportTextures)
	{
		// get texture base path
		std::string strTexturePath=(cSourceTexturePath)?cSourceTexturePath:"";

		for (unsigned int nMaterial=0;nMaterial<ObjectData.GetNumMaterials();nMaterial++)
		{
			// get material
			const PNPTC_MATERIAL* pMat=ObjectData.GetMaterial(nMaterial);

			// go through all textures
			for (int iTexture=0;iTexture<pMat->iTextures;iTexture++)
			{
				// try to load texture
				LPDIRECT3DTEXTURE9 pTexture=CTextureResourceManager::LoadTexture(pD3DDevice,strTexturePath+pMat->strTexture[iTexture]);
				if (!pTexture)
				{
					pTexture=CTextureResourceManager::LoadTexture(pD3DDevice,pMat->strTexture[iTexture]);
				}

				// save the texture
				if (pTexture)
				{
					// save this texture
					std::string strTextureFolder=GetPath(strFileName);
					if (FAILED(D3DXSaveTextureToFile((strTextureFolder+OgreUtils::ValidateOgreEntityName(pMat->strTexture[iTexture])).c_str(),GetImageFormat(pMat->strTexture[iTexture]),pTexture,NULL)))
					{
						MessageBox(NULL,(std::string("Error saving object to file'")+strFileName+"':\n\nError saving texture to file '"+strTextureFolder+pMat->strTexture[iTexture]+"'!").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						bReturn=false;
					}

					// release texture
					SAFE_RELEASE(pTexture);
				}
			}
		}
	}

	// ok result
	return bReturn;
}

bool C3DObjectSerializerOGRE::WriteMaterialToFile(FILE* fileMaterial,const PNPTC_MATERIAL& Material,const std::string& strMaterialName,const std::string& strRelTexturePath)
{
	// check input
	if (!fileMaterial)
	{
		MessageBox(NULL,"Error writing material to ogre material-file.\nInvalid file pointer.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// write material header
	if (fprintf(fileMaterial,"material %s\n{\n\ttechnique\n\t{\n\t\tpass\n\t\t{\n",OgreUtils::ValidateOgreEntityName(strMaterialName,false).c_str())<0)
	{
		// error
		MessageBox(NULL,"Error writing material header to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// set culling
	//fprintf(fileMaterial,"\t\t\tcull_hardware clockwise\n");

	// write colors
	if (Material.bUseVertexColors)
	{
		if (!Material.bAlphaBlend)
		{
			if (fprintf(fileMaterial,"\t\t\tambient vertexcolour\n\t\t\tdiffuse 0 0 0 0\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing material colors to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}
		else
		{
			if (fprintf(fileMaterial,"\t\t\tambient vertexcolour\n\t\t\tdiffuse vertexcolour\n")<0)
			{
				// error
				MessageBox(NULL,"Error writing material colors to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}
	}
	else
	{
		// write ambient color
		PNPTC_RGBA rgbaAmbient=Material.colAmbient;
		if (this->m_bSetBlackAmbientToDiffuse)
		{
			if (rgbaAmbient.r<=0.001 && rgbaAmbient.g<=0.001 && rgbaAmbient.b<=0.001)
			{
				rgbaAmbient=Material.colDiffuse;
			}
		}
		if (fprintf(fileMaterial,"\t\t\tambient %f %f %f\n",static_cast<float>(rgbaAmbient.r)/255.f,static_cast<float>(rgbaAmbient.g)/255.f,static_cast<float>(rgbaAmbient.b)/255.f)<0)
		{
			// error
			MessageBox(NULL,"Error writing material ambient color to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}			

		// write diffuse color
		if (fprintf(fileMaterial,"\t\t\tdiffuse %f %f %f\n",static_cast<float>(Material.colDiffuse.r)/255.f,static_cast<float>(Material.colDiffuse.g)/255.f,static_cast<float>(Material.colDiffuse.b)/255.f)<0)
		{
			// error
			MessageBox(NULL,"Error writing material ambient color to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}			
	}

	// write alpha blending
	if (Material.bAlphaBlend)
	{
		if (fprintf(fileMaterial,"\t\t\tscene_blend alpha_blend\n")<0)
		{
			// error
			MessageBox(NULL,"Error writing material alpha blending to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// write all texture stages
	for (unsigned int iTex=0;iTex<static_cast<unsigned int>(Material.iTextures);iTex++)
	{
		// get texture
		std::string strTexture=Material.strTexture[iTex];
		if (!this->m_strTextureFormat.empty())
		{
			// extract file name
			std::string strTemp;
			std::string strExtension;
			RemoveFileExtension(strTexture,strTemp,strExtension);

			// create file name with new extension
			strTexture=ToLower(strTemp+"."+this->m_strTextureFormat);
		}

		// write texture stage
		if (fprintf(fileMaterial,"\t\t\ttexture_unit\n\t\t\t{\n\t\t\t\ttexture %s\n\t\t\t\ttex_coord_set %d\n\t\t\t\tcolour_op_ex modulate %s src_texture\n\t\t\t}\n",(strRelTexturePath+OgreUtils::ValidateOgreEntityName(strTexture,false)).c_str(),iTex,((iTex>0)?"src_current":"src_diffuse"))<0)
		{
			// error
			MessageBox(NULL,"Error finishing material to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// finish material
	if (fprintf(fileMaterial,"\t\t}\n\t}\n}\n")<0)
	{
		// error
		MessageBox(NULL,"Error finishing material to ogre material-file.","Error exporting ogre object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	return true;
}

bool C3DObjectSerializerOGRE::SaveMaterials(const C3DObjectData& ObjectData,const std::string& strMaterialFile,const char* cBasePathName/*=NULL*/)
{
	// get final material file name
	std::string strFinalMaterialFile=strMaterialFile;
	if (!this->m_strMaterialPath.empty())
	{
		std::string strFileName;
		std::string strPath;
		RemoveFilePath(strMaterialFile,strFileName,strPath);
		strFinalMaterialFile=this->m_strMaterialPath+strFileName;
	}

	// create material file, if everything should be stored into one file
	FILE* fileMaterial=NULL;
	if (this->m_strMaterialDataBase.empty())
	{
		fileMaterial=fopen(ConvertToWindowsFormat(strFinalMaterialFile).c_str(),"w");
		if (!fileMaterial)
		{
			MessageBox(NULL,(std::string("The file ")+strFinalMaterialFile+std::string(" could not be opened for writing.")).c_str(),"Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
	}

	// get relative textures path
	std::string strRelTexturePath="";
	if (!this->m_strRelDestTexturePath.empty())
	{
		strRelTexturePath=this->m_strRelDestTexturePath;
	}
	else if (cBasePathName)
	{
		std::string strFullPath=GetPath(strMaterialFile);
		strRelTexturePath=Right(strFullPath,static_cast<unsigned int>(strFullPath.length()-strlen(cBasePathName)));
	}

	// export all materials
	for (unsigned int i=0;i<ObjectData.GetNumMaterials();i++)
	{
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(i);
		if (pMaterial)
		{
			// open file, if writing everything into separate files
			std::string strMaterialName=pMaterial->strName;
			if (!this->m_strMaterialDataBase.empty())
			{
				// get this material name
				strMaterialName=GetMaterialNameFromDataBase(*pMaterial,this->m_strMaterialDataBase);

				// open the material file
				std::string strThisMaterialFileName=GetMaterialFileNameFromDataBase(*pMaterial,GetPath(strFinalMaterialFile),strMaterialName);
				fileMaterial=fopen(ConvertToWindowsFormat(strThisMaterialFileName).c_str(),"w");
				if (!fileMaterial)
				{
					MessageBox(NULL,(std::string("The file ")+strThisMaterialFileName+std::string(" could not be opened for writing.")).c_str(),"Error exporting obj object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

			}

			// write material
			if (!this->WriteMaterialToFile(fileMaterial,*pMaterial,strMaterialName,strRelTexturePath))
			{
				// error
				fclose(fileMaterial);
				return false;
			}

			// close separate material file
			if (!this->m_strMaterialDataBase.empty())
			{
				fclose(fileMaterial);
			}
		}
	}

	// close material file when writing everything into one file
	if (this->m_strMaterialDataBase.empty())
	{
		fclose(fileMaterial);
	}

	// ok
	return true;
}
