/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerOGRE.h
// 3d object serialization class for ogre mesh files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECTSERIALIZEROGRE_H_INCLUDED)
#define PNPTC_3DOBJECTSERIALIZEROGRE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base class
#include "I3DObjectSerializer.h"


/// 3D Object serialization class for ogre mesh files.
/// Reads/writes objects/meshes to/from an ogre mesh file. 
class C3DObjectSerializerOGRE : public I3DObjectSerializer
{
public:

	// -----------------------------------------------------------------
	// construction / destruciton
	// -----------------------------------------------------------------

	/// Constructs a new serializer.
	C3DObjectSerializerOGRE(void);

	/// Destruction of the serializer.
	~C3DObjectSerializerOGRE(void);

	// -----------------------------------------------------------------
	// methods to load/save objects
	// -----------------------------------------------------------------

	/// Loads an object from a file.
	/// @param ObjectData 3D Object data into which to load the file.
	/// @param strFileName Object file (including path) to be loaded.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be loaded successfully.
	bool	Load(C3DObjectData& ObjectData,const std::string& strFileName);

	/// Writes an object to a file
	/// @param ObjectData 3D Object data to be saved to the file.
	/// @param strFileName Object file (including path) to be saved to.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @param bSeparateMeshFiles Whether or not to generate seperate files for the 
	/// meshes of the object.
	/// @param bExportTextures Whether or not the textures of the mesh should be exported.
	/// @param cSourceTexturePath The path, below which the source texture files
	/// of the mesh to be exported are located.
	/// @param pD3DDevice Pointer to the rendering device of the object and its textures
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be saved successfully.
	bool	Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL,bool bSeparateMeshFiles=false,bool bExportTextures=false,const char* cSourceTexturePath=NULL,IDirect3DDevice9* pD3DDevice=NULL);

	/// Writes all materials of an object to a file.
	/// @param ObjectData 3D Object data to be saved to the file.
	/// @param strMaterialFile Material file (including path) to be saved to.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @param bSeperateFiles Whether or not every material should be written into 
	/// one seperate file.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the materials could be saved successfully.
	bool	SaveMaterials(const C3DObjectData& ObjectData,const std::string& strMaterialFile,const char* cBasePathName=NULL/*,bool bSeperateFiles=false*/);

	/// Writes one material into a file.
	/// @param fileMaterial The file into which to write the material.
	/// @param Material The material to be written.
	/// @param strMaterialName The name of the material (because it might
	/// be, that not the one from the material definition is used).
	/// @param strRelTexturePath The relativ path, below which the textures are stored.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the material could be saved successfully.
	bool	WriteMaterialToFile(FILE* fileMaterial,const PNPTC_MATERIAL& Material,const std::string& strMaterialName,const std::string& strRelTexturePath);


	// -----------------------------------------------------------------
	// serialization parameters
	// -----------------------------------------------------------------

	/// Sets whether or not materials should be saved.
	/// @param bSaveMaterials <i>true</i> or <i>false</i>, whether or not the
	/// materials of the object should be saved.
	void	SetSaveMaterials(bool bSaveMaterials) {this->m_bSaveMaterials=bSaveMaterials;};

	/// Sets whether or not names should be converted into lower cases.
	/// @param bConvertToLowerCase <i>true</i> or <i>false</i>, whether or
	/// not the names should be converted into lower cases.
	void	ConvertToLowerCase(bool bConvertToLowerCase) {this->m_bConvertToLowerCase=bConvertToLowerCase;};

	/// Sets whether or not instance identifier strings should be removed from mesh names.
	/// @param bRemoveMeshInstanceString <i>true</i> or <i>false</i>, whether or
	/// not instance identifier strings should be removed from mesh names.
	void	RemoveMeshInstanceString(bool bRemoveMeshInstanceString) {this->m_bRemoveMeshInstanceString=bRemoveMeshInstanceString;};

	/// Sets the texture format into which the material textures should be converted.
	/// @param strTextureFormat Target format for the exported textures.
	void	OverrideTextureExtension(const std::string& strTextureFormat) {this->m_strTextureFormat=strTextureFormat;};

	/// Sets the relative target directory for the texture/material export.
	/// @param strDestTexturePath The target directory to which the textures
	/// should be exported.
	void	OverrideRelativeTexturePath(const std::string& strRelDestTexturePath) {this->m_strRelDestTexturePath=strRelDestTexturePath;};

	/// Sets the target directory for the material files.
	/// @param strMaterialPath The target directory to which the material files
	/// should be exported.
	void	OverrideMaterialPath(const std::string& strMaterialPath) {this->m_strMaterialPath=strMaterialPath;};

	/// Sets whether or not black ambient colors should be set to the diffuse color value.
	/// @param bSetBlackAmbientToDiffuse Whether or not black ambient colors should be
	/// set to the diffuse color value
	void	SetBlackAmbientToDiffuse(bool bSetBlackAmbientToDiffuse) {this->m_bSetBlackAmbientToDiffuse=bSetBlackAmbientToDiffuse;};

	/// Sets, whether or not to use a material data base for reusing materials.
	/// @param strMaterialDataBase File name of a material data base used
	/// to manage the reusability of materials.
	void	SetMaterialDataBase(const std::string& strMaterialDataBase) {this->m_strMaterialDataBase=strMaterialDataBase;};


private:

	// -----------------------------------------------------------------
	// member variables
	// -----------------------------------------------------------------

	/// Whether or not the materials of the object should be saved.
	bool			m_bSaveMaterials;

	/// Whether or not all names should be converted into lower cases.
	bool			m_bConvertToLowerCase;

	/// Target format for the exported textures.
	std::string		m_strTextureFormat;

	/// The target directory to which the textures should be exported.
	std::string		m_strRelDestTexturePath;

	/// Whether or not black ambient colors should be set to the diffuse color value.
	bool			m_bSetBlackAmbientToDiffuse;

	/// The folder, where the material files should be located.
	std::string		m_strMaterialPath;

	/// whether or not to use a material data base for reusing materials.
	std::string		m_strMaterialDataBase;

	/// Whether or not to remove the mesh instance identifier string from the mesh name.
	bool			m_bRemoveMeshInstanceString;

};



#endif // !defined(PNPTC_3DOBJECTSERIALIZEROGRE_H_INCLUDED)
