/////////////////////////////////////////////////////////////////////////////////////
// I3DObjectRenderer.h
// Interface definition for 3d mesh rendering.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  11. Aug. 2008	: Added virtual destructor.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_I3DOBJECTRENDERER_H_INCLUDED)
#define PNPTC_I3DOBJECTRENDERER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include "../pnptcutils/pnpse/pnpseboundingbox.h"

// forward definition
class pnpse::CRay;

/// Interface definition for 3d object renderer.
class I3DObjectRenderer
{
public:

	// ---------------------------------------------------------------------
	// 3D rendering methods
	// ---------------------------------------------------------------------

	/// Render object at a given position.
	/// @param matPosition World matrix of the object to be rendered.
	/// @param bAlpha Whether or not this object has alpha.
	/// @param iMaterial The material (sub mesh) which is to be rendered.
	/// <i>-1</i> to render the whole mesh.
	/// @param bSetResources Whether or not the resources of this object
	/// should be set or the current resource settings should be reused.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// mesh could be rendered successfully.
	virtual bool						Render(const D3DXMATRIX& matPosition,bool bAlpha=false,int iMaterial=-1,bool bSetResources=true) const=0;

	/// Tests intersection with this object in the object's local space.
	/// @param ray The ray which should be tested against intersection
	/// with the object.
	/// @return Returns the <i>true</i> if the object was intersected
	/// by the ray, otherwise <i>false</i>.
	virtual bool						Intersect(const pnpse::CRay& ray) const=0;

	/// Returns the untransformed/original bounding box of the mesh.
	/// @return The mesh's bounding box.
	virtual const pnpse::CBoundingBox&	GetBoundingBox() const=0;


	// ---------------------------------------------------------------------
	// Retrieve information about the mesh
	// ---------------------------------------------------------------------

	/// Returns the number of faces.
	/// @return Number of faces of this object.
	virtual int							GetNumFaces() const=0;

	/// Returns the number of vertices.
	/// @return Number of vertices of this object.
	virtual int							GetNumVertices() const=0;

	/// Returns the number of materials used in this object.
	/// @return Number of materials of this object.
	virtual int							GetNumMaterials()const=0;


	// ---------------------------------------------------------------------
	// construction / destruction
	// ---------------------------------------------------------------------

	/// Virtual destructor in order to ensure the destruction of implementing classes' objects.
	virtual ~I3DObjectRenderer() {};

};	// class I3DObjectRenderer


#endif // !defined(PNPTC_I3DOBJECTRENDERER_H_INCLUDED)