/////////////////////////////////////////////////////////////////////////////////////
// TerrainMeshCreator.cpp
// creates a mesh from the terrain.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	27. Oct. 2005 : Added fully 3ds max support (Ralf Westphal).
//  20. Nov. 2005 : Changed the way, the extension is removed from a file name =>
//                  Correct ogre xml mesh file extension ".mesh.xml" (Ralf Westphal).
//  05. Dec. 2005 : Added overlay and lightmap export possibilities.
//  01. Jan. 2006 : Added y flipping of texture maps.
//  10. Mai  2008 :	Added quad tree for terrain mesh simplification.
//  23. Jun. 2008 :	Added possibility to exclude border quads from terrain 
//					mesh simplification.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "./TerrainMeshCreator.h"
#include <math.h>
#include <map>
#include <vector>
#include <algorithm>
#include "../PnPTCUtils/utils.h"
#include "../PnPTCUtils/BitmapFile.h"
#include "../PnPTCUtils/ProgressBar.h"


// random names are nothing more than numbers
int iRandomNames=0;




// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// TerrainMeshCreator
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------


CTerrainMeshCreator::CTerrainMeshCreator(void)
	: m_smMethod(smNoSimplification)
	, m_fMaxError(0)
	, m_nGridSpacing(1)
	, m_fSectorOffsetX(0)
	, m_fSectorOffsetY(0)
	, m_fPositionScale(1)
	, m_iUpAxis(0)
	, m_VertexGenerationMode(CTerrainMeshCreator::vgmUntexturedVertices)
	, m_iCurrentTextureID(-1)
	, m_iOverlayTextureIndex(-1)
	, m_iOverlayTextureSize(0)
	, m_iLightmapTextureIndex(-1)
	, m_iTextureMapSectorStartPosX(0)
	, m_iTextureMapSectorStartPosY(0)
	, m_nTextureMapBitmapSize(0)
	, m_nLightMapBitmapSize(0)
	, m_iSubPatchSize(0)
	, m_bUseRandomNames(false)
	, m_iDetailTextureSize(0)
	, m_fUVPixelOffset(1)
	, m_bSimplifyBorderQuads(true)
{
}

CTerrainMeshCreator::~CTerrainMeshCreator(void)
{
}

void CTerrainMeshCreator::SetSimplificationMethod(SIMPLIFICATION_METHOD smMethod)
{
	// set simplification method
	this->m_smMethod=smMethod;
}

void CTerrainMeshCreator::SetErrorValue(float fMaxError)
{
	// set error value
	this->m_fMaxError=static_cast<float>(fabs(fMaxError));
}

void CTerrainMeshCreator::SetRegularGridSpacing(unsigned int nGridSpacing)
{
	// check and set value
	switch (nGridSpacing)
	{
	case 1:
	case 2:
	case 4:
	case 8:
	case 16:
	case 32:
	case 64:
	case 128:
	case 256:
	case 512:
	case 1024:
		this->m_nGridSpacing=nGridSpacing;
		break;
	}
}




// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------
// global helper functions
// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------

/// A mapping between a vertex description and a vertex index.
std::map<std::string,int>		g_mapVertexCache;

/// A mapping between full length and random file names.
std::map<CString,CString>		g_mapExportFilenameCache;


/// Adds one vertex to a mesh.
/// @param pTriangulationCallback Callback interface for additional vertex data.
/// @param pMesh The mesh, to which the vertex should be added.
/// @param Sector The currently processed sector.
/// @param iSecPosX The currently x position on the sector.
/// @param iSecPosY The currently y position on the sector.
/// @param fPositionScale The position scaling factor for the vertex coordinates.
/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
/// <i>1</i> y is the up axis (left handed).
/// @param fOffsetX The x offset of the mesh.
/// @param fOffsetY The y offset of the mesh.
inline int AddVertex(ITriangulationCallback* pTriangulationCallback,C3DMesh* pMesh,CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,float fScale,int iUpAxis,float fOffsetX,float fOffsetY)
{
	// test if vertex is to be created
	if (!pTriangulationCallback->GetCreateVertex(Sector,iSecPosX,iSecPosY))
	{
		return -1;
	}

	// get hash code
	std::string strHash=pTriangulationCallback->GetVertexHash(Sector,iSecPosX,iSecPosY);
	std::map<std::string,int>::iterator itVertex=g_mapVertexCache.find(strHash);
	if (itVertex!=g_mapVertexCache.end())
	{
		return itVertex->second;
	}

	// create new vertex
	PNPTC_VERTEX vert;
	memset(&vert,0,sizeof(vert));

	// get x and y position of vertex
	int iCappedX=minmax(iSecPosX,Sector.GetMinXIndex(),Sector.GetMaxXIndex());
	int iCappedY=minmax(iSecPosY,Sector.GetMinYIndex(),Sector.GetMaxYIndex());

	// get height
	float fHeight=Sector.GetHeight(iCappedX,iCappedY);
	if (fHeight>=0)
	{
		// set position
		vert.vPos.x=static_cast<float>(iSecPosX)*fScale+fOffsetX;
		if (iUpAxis==0)
		{
			vert.vPos.y=fHeight;
			vert.vPos.z=static_cast<float>(iSecPosY)*fScale+fOffsetY;
		}
		else
		{
			vert.vPos.z=fHeight;
			vert.vPos.y=static_cast<float>(iSecPosY)*fScale+fOffsetY;
		}

		// get normal
		vert.vNormal=Sector.GetNormal(iCappedX,iCappedY);
		if (iUpAxis==1)
		{
			SWAP(vert.vNormal.y,vert.vNormal.z);
		}

		// set color and texture coordinates
		pTriangulationCallback->SetTextureCoordinates(Sector,iSecPosX,iSecPosY,vert);
		pTriangulationCallback->SetVertexColor(Sector,iSecPosX,iSecPosY,vert);

		// add that vertex
		int iVertex=pMesh->AddVertex(vert);
		g_mapVertexCache[strHash]=iVertex;
		return iVertex;
	}
	else
	{
		// could not add vertex
		return -1;
	}
}

/// Create one quad face of two triangles.
/// @param pFaceSet The face set to which the quad face should be added.
/// @param iBL The index of the buttom left vertex of the quad.
/// @param iBR The index of the buttom right vertex of the quad.
/// @param iTL The index of the top left vertex of the quad.
/// @param iTR The index of the top right vertex of the quad.
/// @param iOrientation The orientation of the deviding edge of the quad.
/// Can be PNPTC_QUADOR_BL2TR or PNPTC_QUADOR_TL2BR.
/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
/// <i>1</i> y is the up axis (left handed).
void CreateQuadFace(C3DFaceSet* pFaceSet,int iBL,int iBR,int iTL,int iTR,int iOrientation,int iUpAxis)
{
	static PNPTC_FACE face;

	if (iBL>=0 && iBR>=0 && iTL>=0 && iTR>=0)
	{
		// create face
		if (iOrientation==PNPTC_QUADOR_BL2TR)
		{
			face.dwVert[0]=iBL;
			if (iUpAxis==0)
			{
				face.dwVert[1]=iTL;
				face.dwVert[2]=iTR;
			}
			else
			{
				face.dwVert[1]=iTR;
				face.dwVert[2]=iTL;
			}
			pFaceSet->AddFace(face);
			if (iUpAxis==0)
			{
				face.dwVert[1]=iTR;
				face.dwVert[2]=iBR;
			}
			else
			{
				face.dwVert[1]=iBR;
				face.dwVert[2]=iTR;
			}
			pFaceSet->AddFace(face);
		}
		else
		{
			face.dwVert[0]=iBR;
			if (iUpAxis==0)
			{
				face.dwVert[1]=iBL;
				face.dwVert[2]=iTL;
			}
			else
			{
				face.dwVert[1]=iTL;
				face.dwVert[2]=iBL;
			}
			pFaceSet->AddFace(face);
			if (iUpAxis==0)
			{
				face.dwVert[1]=iTL;
				face.dwVert[2]=iTR;
			}
			else
			{
				face.dwVert[1]=iTR;
				face.dwVert[2]=iTL;
			}
			pFaceSet->AddFace(face);
		}
	}
}

/// Create one face.
/// @param pFaceSet The face set to which the face should be added.
/// @param iVert1 The first vertex of the face.
/// @param iVert2 The second vertex of the face.
/// @param iVert3 The third vertex of the face.
/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
/// <i>1</i> y is the up axis (left handed).
void CreateFace(C3DFaceSet* pFaceSet,int iVert1,int iVert2,int iVert3,int iUpAxis)
{
	// check indices
	if (iVert1>=0 && iVert2>=0 && iVert3>=0)
	{
		// fill face data
		static PNPTC_FACE face;
		face.dwVert[0]=iVert1;
		if (iUpAxis==0)
		{
			face.dwVert[1]=iVert3;
			face.dwVert[2]=iVert2;
		}
		else
		{
			face.dwVert[1]=iVert2;
			face.dwVert[2]=iVert3;
		}

		// add face
		pFaceSet->AddFace(face);
	}
}


// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------
// triangulation methods
// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------




bool CTerrainMeshCreator::GetCreateVertex(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const
{
	switch (this->m_VertexGenerationMode)
	{
	case CTerrainMeshCreator::vgmUntexturedVertices:
	case CTerrainMeshCreator::vgmTexturemapVertices:
		// untextured and texture mapped terrain vertices are always created
		return true;
		break;
	case CTerrainMeshCreator::vgmBlendBackgroundVertices:
		// blend background vertices are created if the surrounding vertices have a different texture
		{
			CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
			if (pTextures)
			{
				// get texture of this vertex
				int iVertexTextureID=pTextures->GetTexture(iSecPosX,iSecPosY);

				// check surrounding vertices
				if (iSecPosX<Sector.GetMaxXIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY)!=iVertexTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY)!=iVertexTextureID) return true;
				if (iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX,iSecPosY+1)!=iVertexTextureID) return true;
				if (iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX,iSecPosY-1)!=iVertexTextureID) return true;
				if (iSecPosX<Sector.GetMaxXIndex() && iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY+1)!=iVertexTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY+1)!=iVertexTextureID) return true;
				if (iSecPosX<Sector.GetMaxXIndex() && iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY-1)!=iVertexTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY-1)!=iVertexTextureID) return true;

				// all textures are the same
				return false;
			}
		}
		break;
	case CTerrainMeshCreator::vgmVertexBlendTextureVertices:
		// vertex alpha blended textured vertices have to be rendered, if at least one of the surrounding vertices uses the current texture
		{
			CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
			if (pTextures)
			{
				// check this vertex texture
				if (pTextures->GetTexture(iSecPosX,iSecPosY)==this->m_iCurrentTextureID) return true;

				// check surrounding vertices
				if (iSecPosX<Sector.GetMaxXIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY)==this->m_iCurrentTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY)==this->m_iCurrentTextureID) return true;
				if (iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX,iSecPosY+1)==this->m_iCurrentTextureID) return true;
				if (iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX,iSecPosY-1)==this->m_iCurrentTextureID) return true;
				if (iSecPosX<Sector.GetMaxXIndex() && iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY+1)==this->m_iCurrentTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && iSecPosY<Sector.GetMaxYIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY+1)==this->m_iCurrentTextureID) return true;
				if (iSecPosX<Sector.GetMaxXIndex() && iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX+1,iSecPosY-1)==this->m_iCurrentTextureID) return true;
				if (iSecPosX>Sector.GetMinXIndex() && iSecPosY>Sector.GetMinYIndex() && pTextures->GetTexture(iSecPosX-1,iSecPosY-1)==this->m_iCurrentTextureID) return true;

				// the current texture is not used in the surrounding
				return false;
			}
		}
		break;
	}

	// create vertex by default
	return true;
}

void CTerrainMeshCreator::SetTextureCoordinates(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const
{
	switch (this->m_VertexGenerationMode)
	{
	case CTerrainMeshCreator::vgmUntexturedVertices:
		// no texture coordinates
		break;
	case CTerrainMeshCreator::vgmTexturemapVertices:
		// get uv coordinates for texture maps
		{
			float fUVOffset=1.f/static_cast<float>(max(this->m_nTextureMapBitmapSize,1))*this->m_fUVPixelOffset;
			float fUVScale=1.f-2*fUVOffset;
			vert.uv[0].u=static_cast<float>(iSecPosX-this->m_iTextureMapSectorStartPosX)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset;
			vert.uv[0].v=1.f-(static_cast<float>(iSecPosY-this->m_iTextureMapSectorStartPosY)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset);
			if (this->m_iOverlayTextureIndex>0 && this->m_iOverlayTextureSize>0)
			{
				vert.uv[this->m_iOverlayTextureIndex].u=static_cast<float>(iSecPosX)/static_cast<float>(this->m_iOverlayTextureSize);
				vert.uv[this->m_iOverlayTextureIndex].v=static_cast<float>(Sector.GetBorderSize()-1-iSecPosY)/static_cast<float>(this->m_iOverlayTextureSize);
			}
			if (this->m_iLightmapTextureIndex>0)
			{
				float fUVOffset=1.f/static_cast<float>(max(this->m_nLightMapBitmapSize,1))*this->m_fUVPixelOffset;
				float fUVScale=1.f-2*fUVOffset;
				vert.uv[this->m_iLightmapTextureIndex].u=static_cast<float>(iSecPosX-this->m_iTextureMapSectorStartPosX)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset;
				vert.uv[this->m_iLightmapTextureIndex].v=1.f-(static_cast<float>(iSecPosY-this->m_iTextureMapSectorStartPosY)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset);
			}
		}
		break;
	case CTerrainMeshCreator::vgmBlendBackgroundVertices:
		// vertex for the color blended background
		vert.uv[0].u=0;
		vert.uv[0].v=0;
		if (this->m_iOverlayTextureIndex>0 && this->m_iOverlayTextureSize>0)
		{
			vert.uv[this->m_iOverlayTextureIndex].u=static_cast<float>(iSecPosX)/static_cast<float>(this->m_iOverlayTextureSize);
			vert.uv[this->m_iOverlayTextureIndex].v=static_cast<float>(Sector.GetBorderSize()-1-iSecPosY)/static_cast<float>(this->m_iOverlayTextureSize);
		}
		if (this->m_iLightmapTextureIndex>0)
		{
			float fUVOffset=1.f/static_cast<float>(max(this->m_nLightMapBitmapSize,1))*this->m_fUVPixelOffset;
			float fUVScale=1.f-2*fUVOffset;
			vert.uv[this->m_iLightmapTextureIndex].u=static_cast<float>(iSecPosX-this->m_iTextureMapSectorStartPosX)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset;
			vert.uv[this->m_iLightmapTextureIndex].v=1.f-(static_cast<float>(iSecPosY-this->m_iTextureMapSectorStartPosY)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset);
		}
		break;
	case CTerrainMeshCreator::vgmVertexBlendTextureVertices:
		// vertex for the vertex alpha blended textures
		vert.uv[0].u=static_cast<float>(iSecPosX)/static_cast<float>(this->m_iDetailTextureSize);
		vert.uv[0].v=static_cast<float>(iSecPosY)/static_cast<float>(this->m_iDetailTextureSize);
		if (this->m_iOverlayTextureIndex>0 && this->m_iOverlayTextureSize>0)
		{
			vert.uv[this->m_iOverlayTextureIndex].u=static_cast<float>(iSecPosX)/static_cast<float>(this->m_iOverlayTextureSize);
			vert.uv[this->m_iOverlayTextureIndex].v=static_cast<float>(Sector.GetBorderSize()-1-iSecPosY)/static_cast<float>(this->m_iOverlayTextureSize);
		}
		if (this->m_iLightmapTextureIndex>0)
		{
			float fUVOffset=1.f/static_cast<float>(max(this->m_nLightMapBitmapSize,1));
			float fUVScale=1.f-2*fUVOffset;
			vert.uv[this->m_iLightmapTextureIndex].u=static_cast<float>(iSecPosX-this->m_iTextureMapSectorStartPosX)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset;
			vert.uv[this->m_iLightmapTextureIndex].v=1.f-(static_cast<float>(iSecPosY-this->m_iTextureMapSectorStartPosY)/static_cast<float>(max(this->m_iSubPatchSize,1))*fUVScale+fUVOffset);
		}
		break;
	}
}

void CTerrainMeshCreator::SetVertexColor(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const
{
	switch (this->m_VertexGenerationMode)
	{
	case CTerrainMeshCreator::vgmUntexturedVertices:
	case CTerrainMeshCreator::vgmTexturemapVertices:
		// untextured and texture mapped terrain vertices are completly white
		{
			vert.colDiffuse.r=255;
			vert.colDiffuse.g=255;
			vert.colDiffuse.b=255;
			vert.colDiffuse.a=255;
		}
		break;
	case CTerrainMeshCreator::vgmBlendBackgroundVertices:
		// blend background vertices have the color of the texture
		{
			CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
			if (pTextures)
			{
				// set texture color
				vert.colDiffuse=pTextures->GetTexturedColor(minmax(iSecPosX,Sector.GetMinXIndex(),Sector.GetMaxXIndex()),minmax(iSecPosY,Sector.GetMinYIndex(),Sector.GetMaxYIndex()));
			}
		}
		break;
	case CTerrainMeshCreator::vgmVertexBlendTextureVertices:
		// vertex alpha blended textured vertices just change the alpha value
		{
			CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
			if (pTextures)
			{
				vert.colDiffuse.r=255;
				vert.colDiffuse.g=255;
				vert.colDiffuse.b=255;
				vert.colDiffuse.a=(pTextures->GetTexture(iSecPosX,iSecPosY)==this->m_iCurrentTextureID)?255:0;
			}
		}
		break;
	}
}

std::string CTerrainMeshCreator::GetVertexHash(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const
{
	// get hash code for this vertex
	return ToString(iSecPosX)+","+ToString(iSecPosY);
}

bool CTerrainMeshCreator::TriangulateQuad(CTerrainQuad* pQuad,CIPnPTCSector01& Sector,C3DMesh* pMesh,C3DFaceSet* pFaceSet)
{
	// if this is the base node => clear vertex cache
	if (pQuad->GetParentQuad()==NULL)
	{
		g_mapVertexCache.clear();
	}

	// check if quad is valid
	if (!pQuad)
	{
		return false;
	}

	// check if this is a lead quad
	if (pQuad->GetIsLeaf())
	{
		// build triangulation for this quad if it is visible
		if (pQuad->GetIsVisible())
		{
			// test if we have to create a simple or complex quad
			if (pQuad->GetSize()>1)
			{
				// read neighbours
				CTerrainMeshCreator::CTerrainQuad* pNeighbours[4];
				pNeighbours[CTerrainMeshCreator::CTerrainQuad::West]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::West);
				pNeighbours[CTerrainMeshCreator::CTerrainQuad::East]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::East);
				pNeighbours[CTerrainMeshCreator::CTerrainQuad::North]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::North);
				pNeighbours[CTerrainMeshCreator::CTerrainQuad::South]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::South);

				// get level of this quad and its neighbours
				unsigned int nLevel=pQuad->GetHierarchyLevel();
				unsigned int nNeighbourLevels[4];
				nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::West]=nLevel+((pNeighbours[CTerrainMeshCreator::CTerrainQuad::West])?(pNeighbours[CTerrainMeshCreator::CTerrainQuad::West]->GetNumLevelsBelow()):0);
				nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::East]=nLevel+((pNeighbours[CTerrainMeshCreator::CTerrainQuad::East])?(pNeighbours[CTerrainMeshCreator::CTerrainQuad::East]->GetNumLevelsBelow()):0);
				nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::North]=nLevel+((pNeighbours[CTerrainMeshCreator::CTerrainQuad::North])?(pNeighbours[CTerrainMeshCreator::CTerrainQuad::North]->GetNumLevelsBelow()):0);
				nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::South]=nLevel+((pNeighbours[CTerrainMeshCreator::CTerrainQuad::South])?(pNeighbours[CTerrainMeshCreator::CTerrainQuad::South]->GetNumLevelsBelow()):0);

				// test if this quad needs to be refined.
				if (nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::West] ||
					nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::East] ||
					nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::North] ||
					nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::South])
				{
					// create five base vertices
					PnPTCHMapPosition pos=pQuad->GetPosition();
					int iVert[5];
					iVert[0]=AddVertex(this,pMesh,Sector,pos.x,pos.y,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
					iVert[1]=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize(),pos.y,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
					iVert[2]=AddVertex(this,pMesh,Sector,pos.x,pos.y+pQuad->GetSize(),this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
					iVert[3]=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize(),pos.y+pQuad->GetSize(),this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
					iVert[4]=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize()/2,pos.y+pQuad->GetSize()/2,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);

					// connection to the east
					if (nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::East])
					{
						// needs to be refined in the east
						int iNewVert=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize(),pos.y+pQuad->GetSize()/2,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
						CreateFace(pFaceSet,iVert[1],iNewVert,iVert[4],this->m_iUpAxis);
						CreateFace(pFaceSet,iNewVert,iVert[3],iVert[4],this->m_iUpAxis);
					}
					else
					{
						// one face for the whole east area
						CreateFace(pFaceSet,iVert[1],iVert[3],iVert[4],this->m_iUpAxis);
					}

					// connection to the west
					if (nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::West])
					{
						// needs to be refined in the west
						int iNewVert=AddVertex(this,pMesh,Sector,pos.x,pos.y+pQuad->GetSize()/2,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
						CreateFace(pFaceSet,iVert[0],iVert[4],iNewVert,this->m_iUpAxis);
						CreateFace(pFaceSet,iVert[4],iVert[2],iNewVert,this->m_iUpAxis);
					}
					else
					{
						// one face for the whole west area
						CreateFace(pFaceSet,iVert[0],iVert[4],iVert[2],this->m_iUpAxis);
					}

					// connection to the south
					if (nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::South])
					{
						// needs to be refined in the south
						int iNewVert=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize()/2,pos.y,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
						CreateFace(pFaceSet,iVert[0],iNewVert,iVert[4],this->m_iUpAxis);
						CreateFace(pFaceSet,iNewVert,iVert[1],iVert[4],this->m_iUpAxis);
					}
					else
					{
						// one face for the whole south area
						CreateFace(pFaceSet,iVert[0],iVert[1],iVert[4],this->m_iUpAxis);
					}

					// connection to the north
					if (nLevel<nNeighbourLevels[CTerrainMeshCreator::CTerrainQuad::North])
					{
						// needs to be refined in the north
						int iNewVert=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize()/2,pos.y+pQuad->GetSize(),this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
						CreateFace(pFaceSet,iVert[2],iVert[4],iNewVert,this->m_iUpAxis);
						CreateFace(pFaceSet,iVert[4],iVert[3],iNewVert,this->m_iUpAxis);
					}
					else
					{
						// one face for the whole north area
						CreateFace(pFaceSet,iVert[2],iVert[4],iVert[3],this->m_iUpAxis);
					}

					// ok
					return true;
				}
			}

			// create four vertices
			PnPTCHMapPosition pos=pQuad->GetPosition();
			int iVert[4];
			iVert[0]=AddVertex(this,pMesh,Sector,pos.x,pos.y,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
			iVert[1]=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize(),pos.y,this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
			iVert[2]=AddVertex(this,pMesh,Sector,pos.x,pos.y+pQuad->GetSize(),this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);
			iVert[3]=AddVertex(this,pMesh,Sector,pos.x+pQuad->GetSize(),pos.y+pQuad->GetSize(),this->m_fPositionScale,this->m_iUpAxis,this->m_fSectorOffsetX,this->m_fSectorOffsetY);

			// create faces
			CreateQuadFace(pFaceSet,iVert[0],iVert[1],iVert[2],iVert[3],((pQuad->GetSize()==1)?(Sector.GetQuadOrientation(pos.x,pos.y)):(PNPTC_QUADOR_BL2TR)),this->m_iUpAxis);
			return true;
		}
		else
		{
			// invisible => no need to create it
			return true;
		}
	}
	else
	{
		// build triangulations for all childs
		return	this->TriangulateQuad(pQuad->GetChildQuad(0),Sector,pMesh,pFaceSet) &&
				this->TriangulateQuad(pQuad->GetChildQuad(1),Sector,pMesh,pFaceSet) &&
				this->TriangulateQuad(pQuad->GetChildQuad(2),Sector,pMesh,pFaceSet) &&
				this->TriangulateQuad(pQuad->GetChildQuad(3),Sector,pMesh,pFaceSet);
	}
}






// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------
// Untextured
// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------





bool CTerrainMeshCreator::WriteSectorUntextured(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iUpAxis,bool bGlobalPositioning,bool bUseRandomNames/*=false*/)
{
	// initialization for random values
	g_mapExportFilenameCache.clear();
	this->m_bUseRandomNames=bUseRandomNames;
	if (bUseRandomNames)
	{
		iRandomNames=min(::GetTickCount(),98765432);
	}

	// get export border size
	int iSectorSize=Sector.GetBorderSize();
	int iNumPatches=iSectorSize/iPatchSize;

	// compute scale
	this->m_fPositionScale=fScale*Sector.GetHorizontalScaleFactor();

	// store additional export parameters
	this->m_iUpAxis=iUpAxis;
	this->m_VertexGenerationMode=CTerrainMeshCreator::vgmUntexturedVertices;
	this->m_iCurrentTextureID=-1;
	this->m_iSubPatchSize=iPatchSize;
	this->m_iPatchStartX=0;
	this->m_iPatchStartY=0;
	this->m_iDetailTextureSize=0;

	// compute global position of this sector
	this->m_fSectorOffsetX=0.f;
	this->m_fSectorOffsetY=0.f;
	if (bGlobalPositioning)
	{
		this->m_fSectorOffsetX=(float)(Sector.GetBorderSize()*Sector.GetPosX())*this->m_fPositionScale;
		this->m_fSectorOffsetY=(float)(Sector.GetBorderSize()*Sector.GetPosY())*this->m_fPositionScale;
	}

	// ----------------------------------------------------------------------------------
	// extract file name, extension and path
	std::string strFilenameBase=strFileName;
	std::string strExtension="";
	std::string strPath="";
	std::string strTemp;
	RemoveFilePath(strFileName,strTemp,strPath);
	RemoveFileExtension(strTemp,strFilenameBase,strExtension);


	// ----------------------------------------------------------------------------------
	// initialize progress bar
	DWORD dwProgressBar=iNumPatches*iNumPatches*3;
	CString strProgressBar;
	strProgressBar.Format("Exporting Sector %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,strProgressBar,dwProgressBar);

	// ----------------------------------------------------------------------------------
	// go through all patches
	for (int iPatchY=0,iPatch=0;iPatchY<iNumPatches;iPatchY++)
	{
		for (int iPatchX=0;iPatchX<iNumPatches;iPatchX++,iPatch++)
		{
			// get start position of this patch
			this->m_iPatchStartX=iPatchX*iPatchSize;
			this->m_iPatchStartY=iPatchY*iPatchSize;
			int iPatchEndX=iPatchSize+this->m_iPatchStartX;
			int iPatchEndY=iPatchSize+this->m_iPatchStartY;

			// if the terrain is not to be simplified and this is the last patch, it does not have to have a power of two size
			if (this->m_smMethod!=smNoSimplification)
			{
				if (iPatchX==(iNumPatches-1) && Sector.GetPosX()==(Terrain.GetNumSectorsX()-1))
				{
					iPatchEndX--;
				}
				if (iPatchY==(iNumPatches-1) && Sector.GetPosY()==(Terrain.GetNumSectorsY()-1))
				{
					iPatchEndY--;
				}
			}

			// create patch file suffix
			CString strSuffix="";
			if (iNumPatches>1)
			{
				strSuffix.Format("00%d",iPatch);
				strSuffix=strSuffix.Right(3);
				strSuffix="_"+strSuffix;
			}

			// create object to store the data and the mesh
			C3DObjectData objStore;
			C3DMesh* pMesh=new C3DMesh(&objStore,false,true,0);
			if (!pMesh)
			{
				CString strError;
				strError.Format("Error creating mesh for sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}				
			CString strName;
			if (!bUseRandomNames)
			{
				strName.Format("Sector_%d_%d_Patch_%d",Sector.GetPosX(),Sector.GetPosY(),iPatch);
			}
			else
			{
				strName.Format("%d",iRandomNames++);
			}
			pMesh->SetName((char*)(LPCSTR)strName);
			objStore.AddMesh(pMesh);

			// build quad tree for this terrain mesh
			CTerrainQuadTree quadTree;
			quadTree.SetSimplifyBorderQuads(this->GetSimplifyBorderQuads());
			if (!quadTree.BuildTree(App,Sector,Terrain,this->m_iPatchStartX,this->m_iPatchStartY,iPatchSize,(this->m_smMethod==CTerrainMeshCreator::smRegular)?this->m_nGridSpacing:1,false,true))
			{
				CString strError;
				strError.Format("Error exporting sector %d/%d\n'%s'.\nCould not create quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// simplify quad tree			
			if (this->m_smMethod==CTerrainMeshCreator::smHeightDeviation)
			{
				if (!quadTree.SimplifyTree(App,Sector,Terrain,this->m_fMaxError))
				{
					CString strError;
					strError.Format("Error exporting sector %d/%d.\nCould not simplify quad tree terrain representation for vertex reduction.",Sector.GetPosX(),Sector.GetPosY());
					MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// increment progress bar
			ProgressBar.IncrementPos();

			// create a face set for the mesh triangulation
			C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,-1);
			if (!pFaceSet)
			{
				CString strError;
				strError.Format("Error creating face set for sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}			

			// create triangulation from quad tree
			if (!this->TriangulateQuad(quadTree.GetBaseQuad(),Sector,pMesh,pFaceSet))
			{
				CString strError;
				strError.Format("Error exporting sector %d/%d.\nCould not create mesh triangulation from quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
			pMesh->AddFaceSet(pFaceSet);

			// recompute normals if mesh was simplified
			if (this->m_smMethod!=smNoSimplification)
			{
				// recompute normals of inner vertices
				PNPTC_AABBOX bbox;
				bbox.vMin.x=static_cast<float>(this->m_iPatchStartX)*this->m_fPositionScale+this->m_fSectorOffsetX+Terrain.GetDetailHorizontalScaleFactor()/2.f;
				bbox.vMax.x=static_cast<float>(iPatchEndX)*this->m_fPositionScale+this->m_fSectorOffsetX-Terrain.GetDetailHorizontalScaleFactor()/2.f;
				bbox.vMin.y=static_cast<float>(this->m_iPatchStartY)*this->m_fPositionScale+this->m_fSectorOffsetY+Terrain.GetDetailHorizontalScaleFactor()/2.f;
				bbox.vMax.y=static_cast<float>(iPatchEndY)*this->m_fPositionScale+this->m_fSectorOffsetY-Terrain.GetDetailHorizontalScaleFactor()/2.f;
				bbox.vMin.z=Terrain.GetMinHeight()-1;
				bbox.vMax.z=Terrain.GetMaxHeight()+1;
				if (iUpAxis==0)
				{
					// y is up axis
					SWAP(bbox.vMin.z,bbox.vMin.y);
					SWAP(bbox.vMax.z,bbox.vMax.y);
				}
				pMesh->ComputeNormals(true,&bbox);
			}

			// increment progress bar
			ProgressBar.IncrementPos();

			// --------------------------------------------------------------------------
			// write object to file
			std::string strObjFileName=strPath+strFilenameBase+(LPCSTR)strSuffix+"."+strExtension;
			if (!this->SaveObject(&objStore,strObjFileName))
			{
				CString strError;
				strError.Format("Error writing sector %d/%d to file\n'%s'.",Sector.GetPosX(),Sector.GetPosY(),strObjFileName.c_str());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// increment progress bar
			ProgressBar.IncrementPos();
		}
	}

	// ok
	return true;
}






// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------
// Detail textures
// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------






/// Texture ID and the corresponding texture file name.
typedef struct TEXTURE_ID
{
	unsigned int	nID;				//!< ID of the texture as used inside TC.
	std::string		strFileName;		//!< File name under which the texture is to be saved.
	std::string		strMaterialName;	//!< Name of the material used for this texture.
} TEXTURE_ID;

/// List of textures used inside a sector.
typedef std::vector<TEXTURE_ID> TextureList;


/// Tests whether or not a texture index is already in the list.
/// @param vecTextures List of textures to be tested.
/// @param nID ID of the texture to be searched for.
/// @return Returns <i>true</i> if a texture with this ID is already
/// in the list, otherwise <i>false</i>.
bool TextureInList(TextureList& vecTextures,unsigned int nID)
{
	// try to find the texture in the list
	for (TextureList::iterator itTex=vecTextures.begin();itTex!=vecTextures.end();++itTex)
	{
		if (itTex->nID==nID)
		{
			// already in list
			return true;
		}
	}

	// not in list
	return false;
}

/// Adds a texture to the list of used textures.
/// @param vecTextures List of textures to which to add a new texture.
/// @param nID ID of the texture to be added.
/// @param strFileName File name under which the texture is saved.
/// @param strMaterialName Name of the material used for this texture.
/// @return Returns <i>true</i> if the texture was added to the list and
/// <i>false</i> if the texture was not added, because it was already in the list.
bool AddTexture(TextureList& vecTextures,unsigned int nID,const std::string& strFileName,const std::string& strMaterialName)
{
	// check if texture is already in the list
	if (TextureInList(vecTextures,nID))
	{
		return false;
	}

	// add new texture to list
	TEXTURE_ID tiNew;
	tiNew.nID=nID;
	tiNew.strFileName=strFileName;
	tiNew.strMaterialName=strMaterialName;
	vecTextures.push_back(tiNew);
	return true;
}

/// Defines the order of two textures. They are ordered in
/// with ascending ID.
/// @param t1 First texture to be compared for sorting.
/// @param t2 Second texture to be compared for sorting.
/// @param Returns whether or not the ID of the first texture
/// is smaller than the ID of the second one.
bool TextureSort(TEXTURE_ID& t1,TEXTURE_ID& t2)
{
	return t1.nID<t2.nID;
}

/// Creates an materail for a given texture.
/// @param texture Texture description for which to create a material.
/// @param iOverlayTextureIndex Texture index used for overlay texture.
/// @param strOverlayTextureFile File name of the overlay texture.
/// @param iLightMapTextureIndex Texture index used for light map texture.
/// @param strLightMapTextureFile File name of the light map texture.
/// @param iPatchIndex Patch index for which to create the material.
/// If light maps are exported, the patch index is appended to the material name.
PNPTC_MATERIAL CreateMaterial(TEXTURE_ID& texture,int iOverlayTextureIndex,const std::string& strOverlayTextureFile,int iLightMapTextureIndex,const std::string& strLightMapTextureFile,int iPatchIndex)
{
	// set up material
	PNPTC_MATERIAL mat;
	mat.colAmbient.a=255;
	mat.colAmbient.r=255;
	mat.colAmbient.g=255;
	mat.colAmbient.b=255;
	mat.colDiffuse.a=255;
	mat.colDiffuse.r=255;
	mat.colDiffuse.g=255;
	mat.colDiffuse.b=255;
	mat.iTextures=1;
	if (iOverlayTextureIndex>=0)
	{
		mat.iTextures++;
	}
	if (iLightMapTextureIndex>=0)
	{
		mat.iTextures++;
	}
	mat.bAlphaBlend=true;
	mat.bUseVertexColors=true;
	if (iLightMapTextureIndex>=0 && iPatchIndex>=0)
	{
		static char cTemp[255];
		sprintf(cTemp,"%s_%04d",texture.strMaterialName.c_str(),iPatchIndex);
		mat.strName=cTemp;
	}
	else
	{
		mat.strName=texture.strMaterialName;
	}
	mat.strTexture[0]=texture.strFileName;
	if (iOverlayTextureIndex>=0)
	{
		mat.strTexture[iOverlayTextureIndex]=strOverlayTextureFile;
	}
	if (iLightMapTextureIndex>=0)
	{
		mat.strTexture[iLightMapTextureIndex]=strLightMapTextureFile;
	}

	// return material
	return mat;
}


bool CTerrainMeshCreator::WriteSectorDetailTexture(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,bool bNoiseOverlay/*=false*/,const std::string& strLightMap/*=""*/,bool bUseRandomNames/*=false*/,int iExportTextureMapSize/*=-1*/)
{
	// initialization for random values
	g_mapExportFilenameCache.clear();
	this->m_bUseRandomNames=bUseRandomNames;
	if (bUseRandomNames)
	{
		iRandomNames=min(::GetTickCount(),98765432);
	}

	// list of material IDs used for this sector
	TextureList vecTextureIDs;

	// get export border size
	int iSectorSize=Sector.GetBorderSize();
	int iNumPatches=iSectorSize/iPatchSize;

	// texture indices and settings for overlay and lightmap
	this->m_iOverlayTextureIndex=-1;
	this->m_iLightmapTextureIndex=-1;
	this->m_iOverlayTextureSize=-1;
	bool bLightMap=false;

	// compute scale
	this->m_fPositionScale=fScale*Sector.GetHorizontalScaleFactor();

	// store additional export parameters
	this->m_iUpAxis=iUpAxis;
	this->m_VertexGenerationMode=CTerrainMeshCreator::vgmUntexturedVertices;
	this->m_iCurrentTextureID=-1;
	this->m_iPatchStartX=0;
	this->m_iPatchStartY=0;
	this->m_iDetailTextureSize=0;

	// compute global position of this sector
	this->m_fSectorOffsetX=0.f;
	this->m_fSectorOffsetY=0.f;
	if (bGlobalPositioning)
	{
		this->m_fSectorOffsetX=(float)(Sector.GetBorderSize()*Sector.GetPosX())*this->m_fPositionScale;
		this->m_fSectorOffsetY=(float)(Sector.GetBorderSize()*Sector.GetPosY())*this->m_fPositionScale;
	}


	// ----------------------------------------------------------------------------------
	// extract file name, extension and path
	std::string strFilenameBase=strFileName;
	std::string strExtension="";
	std::string strPath="";
	std::string strTemp;
	RemoveFilePath(strFileName,strTemp,strPath);
	RemoveFileExtension(strTemp,strFilenameBase,strExtension);

	// ----------------------------------------------------------------------------------
	// get texture extension
	std::string strTextureExtension="bmp";
	switch (iTextureFormat)
	{
	case PNPTC_TEXFMT_PNG:
		strTextureExtension="png";
		break;
	case PNPTC_TEXFMT_DDS:
		strTextureExtension="dds";
		break;
	}

	// ----------------------------------------------------------------------------------
	// get overlay texture
	std::string strOverlayTexture="";
	if (bNoiseOverlay)
	{
		CIPnPTCTexture01* pOverlay=Terrain.GetOverlayTexture();
		if (pOverlay)
		{
			// save overlay texture
			strOverlayTexture="overlay."+strTextureExtension;
			if (!pOverlay->SaveToFile((char*)(strPath+strOverlayTexture).c_str(),iTextureFormat))
			{
				MessageBox(NULL,"Error writing overlay noise texture to file","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
			this->m_iOverlayTextureSize=Terrain.GetOverlayTextureSize();
			if (this->m_iOverlayTextureSize<=0) 
			{
				this->m_iOverlayTextureSize=32;
			}
			this->m_iOverlayTextureIndex=1;
		}
		else
		{
			// no noise overlay available
			bNoiseOverlay=false;
			if (MessageBox(NULL,"The noise overlay texture is not available.\nDo you want to continue export without the overlay texture?","Continue without overlay texture?",MB_ICONQUESTION|MB_TASKMODAL|MB_YESNO)==IDNO)
			{
				return false;
			}
		}
	}

	// ----------------------------------------------------------------------------------
	// get sector textures 
	CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
	if (!pTextures)
	{
		// textures not available
		MessageBox(NULL,"Error reading terrain textures.","Error exporting sector heightmap",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// ----------------------------------------------------------------------------------
	// get sector lightmaps
	CIPnPTCLightMap01* pLightMap=NULL;
	if (!strLightMap.empty())
	{
		pLightMap=Sector.GetLightMap(strLightMap.c_str());
		if (pLightMap)
		{
			bLightMap=true;
		}
	}

	// ----------------------------------------------------------------------------------
	// get number of sub patches for texture light maps that are smaller than one path
	int iSubPatches=1;
	this->m_iSubPatchSize=iPatchSize;
	int iLightMapSize=min(iPatchSize,(iExportTextureMapSize==-1)?iPatchSize:iExportTextureMapSize);
	float fSubPatches=static_cast<float>(iPatchSize)/static_cast<float>(iLightMapSize);
	if (bLightMap && iLightMapSize<iPatchSize)
	{
		this->m_iSubPatchSize=iLightMapSize;
		iSubPatches=iPatchSize/this->m_iSubPatchSize;
	}
	int iNumLightMaps=iSectorSize/iLightMapSize;

	// ----------------------------------------------------------------------------------
	// get sector lightmaps
	if (bLightMap && pLightMap)
	{
		// get uv pair index of light map
		this->m_iLightmapTextureIndex=max(this->m_iOverlayTextureIndex,0)+1;

		// export light maps
		CLightmapExporterBMP exporterLightMap;
		exporterLightMap.SetParameter("LightMapName",strLightMap);
		exporterLightMap.SetParameter("FileFormat",strTextureExtension);
		exporterLightMap.SetParameter("PatchSize",iLightMapSize);
		exporterLightMap.SetParameter("BorderPixel",true);
		exporterLightMap.SetParameter("PowerOfTwo",true);
		std::string strLightMapFileName=strPath+"lightmap."+strTextureExtension;
		exporterLightMap.SetFileNameCallback(this);
		if (!exporterLightMap.WriteSector(App,Sector,Terrain,strLightMapFileName))
		{
			std::string strError="Error writing texturemaps of sector "+ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())
									+"\nto file '"+strLightMapFileName+"'.";
			MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
		this->m_nLightMapBitmapSize=exporterLightMap.GetBitmapBorderSize();
	}

	// ----------------------------------------------------------------------------------
	// initialize progress bar
	DWORD dwProgressBar=pTextures->GetNumUsedTextures()+iSectorSize+iNumPatches*iNumPatches*iSubPatches*iSubPatches*(this->m_iSubPatchSize+this->m_iSubPatchSize*pTextures->GetNumUsedTextures());
	CString strProgressBar;
	strProgressBar.Format("Exporting Sector %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,strProgressBar,dwProgressBar);

	// ----------------------------------------------------------------------------------
	// store all textures/texturemaps
	unsigned int iTextures=(pTextures)?pTextures->GetNumUsedTextures():0;
	for (unsigned int iTexture=0;iTexture<iTextures;iTexture++)
	{
		// get texture
		PnPTC_UsedTexture tex=pTextures->GetUsedTexture(iTexture);
		CIPnPTCTexture01* pTexture=pTextures->GetTexture(tex.ID);
		if (!pTexture)
		{
			std::string strError="Error retrieving detail texture "+std::string(tex.Name)
				                 +" of sector "+ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())+".";
			MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// get texture file name
		std::string strTextureFileName;
		std::string strTemp;
		RemoveFileExtension(pTexture->GetName(),strTextureFileName,strTemp);
		if (bUseRandomNames && strTextureFileName.length()>8)
		{
			strTextureFileName=ToString(iRandomNames++);
		}
		std::string strTextureFile=strTextureFileName+"."+strTextureExtension;

		// save texture
		if (!pTexture->SaveToFile((char*)(strPath+strTextureFile).c_str(),iTextureFormat))
		{
			std::string strError="Error writing detail texture "+std::string(tex.Name)
				                 +" of sector "+ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())
								 +" to file '"+strPath+strTextureFile+"'.";
			MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}

		// add texture ID
		AddTexture(vecTextureIDs,tex.ID,std::string(strTextureFile),std::string(strTextureFileName));

		// increment progress bar
		ProgressBar.IncrementPos();
	}
	// create entries for neighboring sector textures
	for (int iBorderPoint=0;iBorderPoint<=iSectorSize;iBorderPoint++)
	{
		for (int iBorder=0;iBorder<=1;iBorder++)
		{
			int iTex=pTextures->GetTexture((iBorder==0)?iSectorSize:iBorderPoint,(iBorder==1)?iSectorSize:iBorderPoint);
			if (iTex>=0 && !TextureInList(vecTextureIDs,static_cast<unsigned int>(iTex)))
			{
				// get texture
				CIPnPTCTexture01* pTexture=pTextures->GetTexture(iTex);
				if (!pTexture)
				{
					CString strError;
					strError.Format("Error retrieving one detail texture of neighbor sector of sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
					MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// get texture file name
				std::string strTextureFileName;
				std::string strTemp;
				RemoveFileExtension(pTexture->GetName(),strTextureFileName,strTemp);
				if (bUseRandomNames && strTextureFileName.length()>8)
				{
					strTextureFileName=ToString(iRandomNames++);
				}
				std::string strTextureFile=strTextureFileName+"."+strTextureExtension;

				// save texture
				if (!pTexture->SaveToFile((char*)(strPath+strTextureFile).c_str(),iTextureFormat))
				{
					std::string strError="Error writing one detail texture of a neighbor sector of the sector "
						                 +ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())
										 +"\nto file '"+strPath+strTextureFile+"'.";
					MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// add texture ID
				AddTexture(vecTextureIDs,static_cast<unsigned int>(iTex),std::string(strTextureFile),std::string(strTextureFileName));
			}
		}

		// increment progress bar
		ProgressBar.IncrementPos();
	}
	// sort list of textures
	std::sort(vecTextureIDs.begin(),vecTextureIDs.end(),TextureSort);

	// ----------------------------------------------------------------------------------
	// go through all patches
	for (int iPatchY=0,iSubPatch=0,iPatch=0;iPatchY<iNumPatches;iPatchY++)
	{
		for (int iPatchX=0;iPatchX<iNumPatches;iPatchX++,iPatch++)
		{
			// get start position of this patch
			this->m_iPatchStartX=iPatchX*iPatchSize;
			this->m_iPatchStartY=iPatchY*iPatchSize;
			int iPatchEndX=iPatchSize+this->m_iPatchStartX;
			int iPatchEndY=iPatchSize+this->m_iPatchStartY;

			// create patch file suffix
			CString strSuffix="";
			if (iNumPatches>1)
			{
				strSuffix.Format("00%d",iPatch);
				strSuffix=strSuffix.Right(3);
				strSuffix="_"+strSuffix;
			}

			// create object to store the data and the mesh
			C3DObjectData objStore;
			C3DMesh* pMesh=new C3DMesh(&objStore,true,true,max(this->m_iOverlayTextureIndex,max(this->m_iLightmapTextureIndex,0))+1);
			if (!pMesh)
			{
				CString strError;
				strError.Format("Error creating mesh for sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}				
			CString strName;
			if (!bUseRandomNames)
			{
				strName.Format("Sector_%d_%d_Patch_%d",Sector.GetPosX(),Sector.GetPosY(),iPatch);
			}
			else
			{
				strName.Format("%d",iRandomNames++);
			}
			pMesh->SetName((char*)(LPCSTR)strName);
			objStore.AddMesh(pMesh);

			// build quad tree for this terrain mesh
			CTerrainQuadTree quadTree;
			quadTree.SetSimplifyBorderQuads(this->GetSimplifyBorderQuads());
			if (!quadTree.BuildTree(App,Sector,Terrain,this->m_iPatchStartX,this->m_iPatchStartY,iPatchSize,(this->m_smMethod==CTerrainMeshCreator::smRegular)?this->m_nGridSpacing:1,true,true))
			{
				CString strError;
				strError.Format("Error exporting sector %d/%d\n'%s'.\nCould not create quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// simplify quad tree
			if (this->m_smMethod==CTerrainMeshCreator::smHeightDeviation)
			{
				if (!quadTree.SimplifyTree(App,Sector,Terrain,this->m_fMaxError))
				{
					CString strError;
					strError.Format("Error exporting sector %d/%d.\nCould not simplify quad tree terrain representation for vertex reduction.",Sector.GetPosX(),Sector.GetPosY());
					MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// go through all subpatches
			for (int iSubPatchY=0;iSubPatchY<iSubPatches;iSubPatchY++)
			{
				for (int iSubPatchX=0;iSubPatchX<iSubPatches;iSubPatchX++,iSubPatch++)
				{
					// get start position of this patch
					int iSubPatchStartX=iPatchX*iPatchSize+iSubPatchX*this->m_iSubPatchSize;
					int iSubPatchStartY=iPatchY*iPatchSize+iSubPatchY*this->m_iSubPatchSize;
					int iSubPatchEndX=this->m_iSubPatchSize+iSubPatchStartX;
					int iSubPatchEndY=this->m_iSubPatchSize+iSubPatchStartY;

					// if the terrain is not to be simplified and this is the last patch, it does not have to have a power of two size
					if (this->m_smMethod!=smNoSimplification)
					{
						if (iPatchX==(iNumPatches-1) && iSubPatchX==(iSubPatches-1) && Sector.GetPosX()==(Terrain.GetNumSectorsX()-1))
						{
							iSubPatchEndX--;
						}
						if (iPatchY==(iNumPatches-1) && iSubPatchY==(iSubPatches-1) && Sector.GetPosY()==(Terrain.GetNumSectorsY()-1))
						{
							iSubPatchEndY--;
						}
					}

					// get light map for this sub patch
					std::string strLightMapTextureFile("");
					if (this->m_iLightmapTextureIndex>=0 && pLightMap)
					{
						int iMapX=static_cast<int>(static_cast<float>(iPatchX)*fSubPatches);
						int iMapY=static_cast<int>(static_cast<float>(iPatchY)*fSubPatches);
						if (fSubPatches>=1)
						{
							iMapX+=iSubPatchX;
							iMapY+=iSubPatchY;
						}

						std::string strPath,strExtension;
						RemoveFilePath(this->GetExportFileName(&Sector,"lightmap."+strTextureExtension,iNumLightMaps,iMapX,iMapY),strLightMapTextureFile,strPath);
					}

					// -------------------------------------------------------------
					// triangulation of blend background

					// create blend background material
					PNPTC_MATERIAL matBackground;
					matBackground.colAmbient.a=255;
					matBackground.colAmbient.r=255;
					matBackground.colAmbient.g=255;
					matBackground.colAmbient.b=255;
					matBackground.colDiffuse.a=255;
					matBackground.colDiffuse.r=255;
					matBackground.colDiffuse.g=255;
					matBackground.colDiffuse.b=255;
					matBackground.iTextures=0;
					if (bNoiseOverlay && this->m_iOverlayTextureIndex>=0) matBackground.iTextures++;
					if (pLightMap && this->m_iLightmapTextureIndex>=0) matBackground.iTextures++;
					matBackground.bAlphaBlend=false;
					matBackground.bUseVertexColors=true;
					matBackground.strName="Background";
					if (bNoiseOverlay && this->m_iOverlayTextureIndex>=0)
					{
						matBackground.strTexture[this->m_iOverlayTextureIndex-1]=strOverlayTexture;
					}
					if (pLightMap && this->m_iLightmapTextureIndex>=0)
					{
						matBackground.strTexture[this->m_iLightmapTextureIndex-1]=strLightMapTextureFile;
						static char cTemp[255];
						sprintf(cTemp,"Background_%04d",iSubPatch);
						matBackground.strName=cTemp;
					}

					// create all transition background quads
					int iMaterialBackround=objStore.AddMaterial(matBackground);
					C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,iMaterialBackround);
					if (!pFaceSet)
					{
						MessageBox(NULL,"Could not create face set for mesh (out of memory).","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// create triangulation from quad tree
					g_mapVertexCache.clear();
					this->m_VertexGenerationMode=CTerrainMeshCreator::vgmBlendBackgroundVertices;
					this->m_iCurrentTextureID=-1;
					if (!quadTree.GetBaseQuad() ||
						!this->TriangulateQuad(quadTree.GetBaseQuad()->FindQuad(iSubPatchStartX,iSubPatchStartY,this->m_iSubPatchSize),Sector,pMesh,pFaceSet))
					{
						CString strError;
						strError.Format("Error exporting sector %d/%d.\nCould not create mesh triangulation from quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
						MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
					if (pFaceSet->GetNumFaces()>0)
					{
						pMesh->AddFaceSet(pFaceSet);
					}
					else
					{
						delete pFaceSet;
					}

					// --------------------------------------------------------------------------
					// create textured vertices

					// go through all textures
					int iCurTexIndex=0;
					for (TextureList::iterator itTex=vecTextureIDs.begin();itTex!=vecTextureIDs.end();++itTex,iCurTexIndex++)
					{
						// create face set
						C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,-1);
						if (!pFaceSet)
						{
							MessageBox(NULL,"Could not create face set for mesh (out of memory).","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}

						// create triangulation from quad tree
						g_mapVertexCache.clear();
						this->m_VertexGenerationMode=CTerrainMeshCreator::vgmVertexBlendTextureVertices;
						this->m_iCurrentTextureID=itTex->nID;
						this->m_iDetailTextureSize=Terrain.GetTextureSize();
						if (!quadTree.GetBaseQuad() ||
							!this->TriangulateQuad(quadTree.GetBaseQuad()->FindQuad(iSubPatchStartX,iSubPatchStartY,this->m_iSubPatchSize),Sector,pMesh,pFaceSet))
						{
							CString strError;
							strError.Format("Error exporting sector %d/%d.\nCould not create mesh triangulation from quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
							MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}

						// add face set to mesh
						if (pFaceSet->GetNumFaces()!=0)
						{										
							pFaceSet->SetMaterial(objStore.AddMaterial(CreateMaterial(*itTex,this->m_iOverlayTextureIndex,std::string(strOverlayTexture),this->m_iLightmapTextureIndex,strLightMapTextureFile,iSubPatch)));
							pMesh->AddFaceSet(pFaceSet);
						}
						else
						{
							delete pFaceSet;
						}
					} // for TextureID
				} // for SubPatchX
			} // for SubPatchY

			// --------------------------------------------------------------------------
			// write object to file
			std::string strObjFileName=strPath+strFilenameBase+(LPCSTR)strSuffix+"."+strExtension;
			if (!this->SaveObject(&objStore,strObjFileName))
			{
				CString strError;
				strError.Format("Error writing sector %d/%d to file\n'%s'.",Sector.GetPosX(),Sector.GetPosY(),strObjFileName);
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

		} // for PatchX
	} // for PatchY

	// ok
	return true;
}






// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------
// Texture map
// -----------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------





/// list of materials
typedef std::map<WORD,PNPTC_MATERIAL>	MATERIALMAP;
typedef std::pair<WORD,PNPTC_MATERIAL>	MATPAIR;

std::string GetExportFileNameHash(const std::string& strLongFileName)
{
	// remove path
	std::string strPath,strFilename,strExtension;
	RemoveFilePath(strLongFileName,strFilename,strPath);
	return strFilename;
}

std::string CTerrainMeshCreator::GetExportFileName(CIPnPTCSector01* pSector,const std::string& strBaseName,unsigned int nPatches,unsigned int nPatchX,unsigned int nPatchY,const std::string& strExtraInfo/*=""*/) const
{
	// build and return string
	CString strReturn;

	// build file name from sector and patch index
	std::string strPath,strFilename,strExtension;
	RemoveFilePath(strBaseName,strFilename,strPath);
	RemoveFileExtension(strFilename,strFilename,strExtension);
	if (pSector)
	{
		strReturn.Format("%s%s_%05d_%05d_%05d.%s",strPath.c_str(),strFilename.c_str(),pSector->GetPosX(),pSector->GetPosY(),nPatchX+nPatches*nPatchY,strExtension.c_str());
	}
	else
	{
		strReturn.Format("%s%s_%05d.%s",strPath.c_str(),strFilename.c_str(),nPatchX+nPatches*nPatchY,strExtension.c_str());
	}

	// random names?
	if (this->m_bUseRandomNames)
	{
		// use the long name as hash code
		CString strHash=GetExportFileNameHash((LPCSTR)strReturn).c_str();

		// test if we already have hashed this file name
		std::map<CString,CString>::iterator itReturnString=g_mapExportFilenameCache.find(strHash);
		if (itReturnString!=g_mapExportFilenameCache.end())
		{
			return (LPCSTR)itReturnString->second;
		}

		// 8.3 random file name, for good old DOS files
		strReturn.Format("%s%d.%s",strPath.c_str(),iRandomNames++,strExtension.c_str());
		g_mapExportFilenameCache[strHash]=strReturn;
	}

	// return the export file name
	return (LPCSTR)strReturn;
}

bool CTerrainMeshCreator::WriteSectorTexturemap(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,bool bNoiseOverlay/*=false*/,const std::string& strLightMap/*=""*/,bool bCombineTextureAndLightmap/*=false*/,bool bUseRandomNames/*=false*/,bool bFlipTextureY/*=false*/,int iExportTextureMapSize/*=-1*/)
{
	// initialization for random values
	g_mapExportFilenameCache.clear();
	this->m_bUseRandomNames=bUseRandomNames;
	if (bUseRandomNames)
	{
		iRandomNames=min(::GetTickCount(),98765432);
	}

	// get export border size
	int iSectorSize=Sector.GetBorderSize();
	int iNumPatches=iSectorSize/iPatchSize;

	// texture indices and settings for overlay and lightmap
	this->m_iOverlayTextureIndex=-1;
	this->m_iLightmapTextureIndex=-1;
	this->m_iOverlayTextureSize=-1;
	bool bLightMap=false;

	// compute scale
	this->m_fPositionScale=fScale*Sector.GetHorizontalScaleFactor();

	// store additional export parameters
	this->m_iUpAxis=iUpAxis;
	this->m_VertexGenerationMode=CTerrainMeshCreator::vgmTexturemapVertices;
	this->m_iCurrentTextureID=-1;
	this->m_iPatchStartX=0;
	this->m_iPatchStartY=0;
	this->m_iDetailTextureSize=0;

	// compute global position of this sector
	this->m_fSectorOffsetX=0.f;
	this->m_fSectorOffsetY=0.f;
	if (bGlobalPositioning)
	{
		this->m_fSectorOffsetX=(float)(Sector.GetBorderSize()*Sector.GetPosX())*this->m_fPositionScale;
		this->m_fSectorOffsetY=(float)(Sector.GetBorderSize()*Sector.GetPosY())*this->m_fPositionScale;
	}

	// ----------------------------------------------------------------------------------
	// extract file name, extension and path
	std::string strFilenameBase=strFileName;
	std::string strExtension="";
	std::string strPath="";
	std::string strTemp;
	RemoveFilePath(strFileName,strTemp,strPath);
	RemoveFileExtension(strTemp,strFilenameBase,strExtension);

	// ----------------------------------------------------------------------------------
	// get texture extension
	std::string strTextureExtension="bmp";
	switch (iTextureFormat)
	{
	case PNPTC_TEXFMT_PNG:
		strTextureExtension="png";
		break;
	case PNPTC_TEXFMT_DDS:
		strTextureExtension="dds";
		break;
	}

	// ----------------------------------------------------------------------------------
	// get overlay texture
	std::string strOverlayTexture="";
	if (bNoiseOverlay)
	{
		CIPnPTCTexture01* pOverlay=Terrain.GetOverlayTexture();
		if (pOverlay)
		{
			// save overlay texture
			strOverlayTexture="overlay."+strTextureExtension;
			if (!pOverlay->SaveToFile((char*)(strPath+strOverlayTexture).c_str(),iTextureFormat))
			{
				MessageBox(NULL,"Error writing overlay noise texture to file","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
			this->m_iOverlayTextureSize=Terrain.GetOverlayTextureSize();
			this->m_iOverlayTextureIndex=1;
		}
		else
		{
			// no noise overlay available
			bNoiseOverlay=false;
			if (MessageBox(NULL,"The noise overlay texture is not available.\nDo you want to continue export without the overlay texture?","Continue without overlay texture?",MB_ICONQUESTION|MB_TASKMODAL|MB_YESNO)==IDNO)
			{
				return false;
			}
		}
	}

	// ----------------------------------------------------------------------------------
	// get sector lightmaps
	CIPnPTCLightMap01* pLightMap=NULL;
	if (!strLightMap.empty())
	{
		pLightMap=Sector.GetLightMap(strLightMap.c_str());
		if (pLightMap)
		{
			bLightMap=true;
		}
		if (!bCombineTextureAndLightmap)
		{
			this->m_iLightmapTextureIndex=max(this->m_iOverlayTextureIndex,0)+1;
		}
	}

	// ----------------------------------------------------------------------------------
	// get texture map 
	CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
	CIPnPTCTextureMap01* pTexMap=NULL;
	if (pTextures)
	{
		pTexMap=pTextures->GetTextureMap();
		if (!pTexMap)
		{
			// texture map not available
			MessageBox(NULL,"Error reading terrain texture map.","Error exporting sector heightmap",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
			return false;
		}
	}
	else
	{
		// textures not available
		MessageBox(NULL,"Error reading terrain textures.","Error exporting sector heightmap",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// get number of sub patches for texture maps that are smaller than one path
	int iSubPatches=1;
	this->m_iSubPatchSize=iPatchSize;
	int iTextureMapSize=min(iPatchSize,(iExportTextureMapSize==-1)?iPatchSize:iExportTextureMapSize);
	float fSubPatches=static_cast<float>(iPatchSize)/static_cast<float>(iTextureMapSize);
	if (iTextureMapSize<iPatchSize)
	{
		this->m_iSubPatchSize=iTextureMapSize;
		iSubPatches=iPatchSize/this->m_iSubPatchSize;
	}
	int iNumTextureMaps=iSectorSize/iTextureMapSize;

	// ----------------------------------------------------------------------------------
	// initialize progress bar
	DWORD dwProgressBar=iNumPatches*iNumPatches*iSubPatches*iSubPatches*this->m_iSubPatchSize+iNumPatches;
	CString strProgressBar;
	strProgressBar.Format("Exporting Sector %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,strProgressBar,dwProgressBar);

	// ----------------------------------------------------------------------------------
	// export texture maps
	CTexturemapExporterBMP exporterTextureMap;
	exporterTextureMap.SetParameter("FileFormat",strTextureExtension);
	if (bCombineTextureAndLightmap)
	{
		exporterTextureMap.SetParameter("LightMapName",strLightMap);
	}
	exporterTextureMap.SetParameter("PatchSize",iTextureMapSize);
	exporterTextureMap.SetParameter("BorderPixel",true);
	exporterTextureMap.SetParameter("PowerOfTwo",true);
	exporterTextureMap.SetParameter("FlipY",bFlipTextureY);
	std::string strTextureMapFileName=strPath+"texturemap."+strTextureExtension;
	exporterTextureMap.SetFileNameCallback(this);
	if (!exporterTextureMap.WriteSector(App,Sector,Terrain,strTextureMapFileName))
	{
		std::string strError="Error writing texturemaps of sector "+ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())
								+"\nto file '"+strTextureMapFileName+"'.";
		MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}
	this->m_nTextureMapBitmapSize=exporterTextureMap.GetBitmapBorderSize();
	this->m_nLightMapBitmapSize=this->m_nTextureMapBitmapSize;

	// ----------------------------------------------------------------------------------
	// export light maps
	if (!strLightMap.empty() && !bCombineTextureAndLightmap)
	{
		CLightmapExporterBMP exporterLightMap;
		exporterLightMap.SetParameter("LightMapName",strLightMap);
		exporterLightMap.SetParameter("FileFormat",strTextureExtension);
		exporterLightMap.SetParameter("PatchSize",iTextureMapSize);
		exporterLightMap.SetParameter("BorderPixel",true);
		exporterLightMap.SetParameter("PowerOfTwo",true);
		exporterLightMap.SetParameter("FlipY",bFlipTextureY);
		std::string strLightMapFileName=strPath+"lightmap."+strTextureExtension;
		exporterLightMap.SetFileNameCallback(this);
		if (!exporterLightMap.WriteSector(App,Sector,Terrain,strLightMapFileName))
		{
			std::string strError="Error writing texturemaps of sector "+ToString(Sector.GetPosX())+"/"+ToString(Sector.GetPosY())
									+"\nto file '"+strLightMapFileName+"'.";
			MessageBox(NULL,strError.c_str(),"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
		this->m_nLightMapBitmapSize=exporterLightMap.GetBitmapBorderSize();
	}

	// ----------------------------------------------------------------------------------
	// create materials for all texture maps

	// go through all texture maps
	MATERIALMAP Materials;
	for (unsigned int nTexMapY=0,nTexMap=0;nTexMapY<static_cast<unsigned int>(iNumTextureMaps);++nTexMapY)
	{
		for (unsigned int nTexMapX=0;nTexMapX<static_cast<unsigned int>(iNumTextureMaps);++nTexMapX,++nTexMap)
		{
			// create material
			PNPTC_MATERIAL mat;
			mat.colAmbient.a=255;
			mat.colAmbient.r=255;
			mat.colAmbient.g=255;
			mat.colAmbient.b=255;
			mat.colDiffuse.a=255;
			mat.colDiffuse.r=255;
			mat.colDiffuse.g=255;
			mat.colDiffuse.b=255;
			mat.iTextures=1;
			if (bNoiseOverlay && this->m_iOverlayTextureIndex>=0)
			{
				mat.iTextures++;
			}
			if (bLightMap && this->m_iLightmapTextureIndex>=0 && !bCombineTextureAndLightmap)
			{
				mat.iTextures++;
			}
			mat.bAlphaBlend=false;
			mat.bUseVertexColors=false;
			std::string strPath,strFilename,strExtension;
			RemoveFilePath(this->GetExportFileName(&Sector,"texturemap."+strTextureExtension,iNumTextureMaps,nTexMapX,nTexMapY),strFilename,strPath);
			mat.strName=strFilename;
			mat.strTexture[0]=strFilename;
			if (bNoiseOverlay && this->m_iOverlayTextureIndex>=0)
			{
				mat.strTexture[this->m_iOverlayTextureIndex]=strOverlayTexture;
			}
			if (bLightMap && this->m_iLightmapTextureIndex>=0 && !bCombineTextureAndLightmap)
			{
				mat.strTexture[this->m_iLightmapTextureIndex]=this->GetExportFileName(&Sector,"lightmap."+strTextureExtension,iNumTextureMaps,nTexMapX,nTexMapY);
			}

			// add material to list
			Materials.insert(MATPAIR(nTexMap,mat));
		}
	}

	// ----------------------------------------------------------------------------------
	// go through all patches
	for (int iPatchY=0,iPatch=0;iPatchY<iNumPatches;iPatchY++)
	{
		for (int iPatchX=0;iPatchX<iNumPatches;iPatchX++,iPatch++)
		{
			// get start position of this patch
			this->m_iPatchStartX=iPatchX*iPatchSize;
			this->m_iPatchStartY=iPatchY*iPatchSize;
			int iPatchEndX=iPatchSize+this->m_iPatchStartX;
			int iPatchEndY=iPatchSize+this->m_iPatchStartY;

			// create patch file suffix
			CString strSuffix="";
			if (iNumPatches>1)
			{
				strSuffix.Format("00%d",iPatch);
				strSuffix=strSuffix.Right(3);
				strSuffix="_"+strSuffix;
			}

			// create object to store the data and the mesh
			C3DObjectData objStore;
			C3DMesh* pMesh=new C3DMesh(&objStore,true,true,max(this->m_iOverlayTextureIndex,max(this->m_iLightmapTextureIndex,0))+1);
			if (!pMesh)
			{
				CString strError;
				strError.Format("Error creating mesh for sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}				
			CString strName;
			if (!bUseRandomNames)
			{
				strName.Format("Sector_%d_%d_Patch_%d",Sector.GetPosX(),Sector.GetPosY(),iPatch);
			}
			else
			{
				strName.Format("%d",iRandomNames++);
			}
			pMesh->SetName((char*)(LPCSTR)strName);
			objStore.AddMesh(pMesh);

			// build quad tree for this terrain mesh
			CTerrainQuadTree quadTree;
			quadTree.SetSimplifyBorderQuads(this->GetSimplifyBorderQuads());
			if (!quadTree.BuildTree(App,Sector,Terrain,this->m_iPatchStartX,this->m_iPatchStartY,iPatchSize,(this->m_smMethod==CTerrainMeshCreator::smRegular)?this->m_nGridSpacing:1,false,true))
			{
				CString strError;
				strError.Format("Error exporting sector %d/%d\n'%s'.\nCould not create quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
				MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// simplify quad tree
			if (this->m_smMethod==CTerrainMeshCreator::smHeightDeviation)
			{
				if (!quadTree.SimplifyTree(App,Sector,Terrain,this->m_fMaxError))
				{
					CString strError;
					strError.Format("Error exporting sector %d/%d.\nCould not simplify quad tree terrain representation for vertex reduction.",Sector.GetPosX(),Sector.GetPosY());
					MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}
			}

			// go through all subpatches
			for (int iSubPatchY=0;iSubPatchY<iSubPatches;iSubPatchY++)
			{
				for (int iSubPatchX=0;iSubPatchX<iSubPatches;iSubPatchX++)
				{
					// -----------------------------------------------------------------
					// add texture map of this sub patch
					int iMaterial=-1;
					int iMapX=static_cast<int>(static_cast<float>(iPatchX)*fSubPatches);
					int iMapY=static_cast<int>(static_cast<float>(iPatchY)*fSubPatches);
					if (fSubPatches>=1)
					{
						iMapX+=iSubPatchX;
						iMapY+=iSubPatchY;
					}
					int iMap=iMapX+iMapY*iNumTextureMaps;
					MATERIALMAP::iterator itMat=Materials.find(iMap);
					if (itMat!=Materials.end())
					{
						// get material
						PNPTC_MATERIAL mat=itMat->second;
						// add to object materials
						iMaterial=objStore.AddMaterial(mat);
					}

					// get start and end position of this sub patch
					int iSubPatchStartX=iPatchX*iPatchSize+iSubPatchX*this->m_iSubPatchSize;
					int iSubPatchStartY=iPatchY*iPatchSize+iSubPatchY*this->m_iSubPatchSize;
					int iSubPatchEndX=this->m_iSubPatchSize+iSubPatchStartX;
					int iSubPatchEndY=this->m_iSubPatchSize+iSubPatchStartY;

					// store texture parameters for mesh triangulation
					this->m_iTextureMapSectorStartPosX=iMapX*iTextureMapSize;
					this->m_iTextureMapSectorStartPosY=iMapY*iTextureMapSize;

					// if the terrain is not to be simplified and this is the last patch, it does not have to have a power of two size
					if (this->m_smMethod!=smNoSimplification)
					{
						if (iPatchX==(iNumPatches-1) && iSubPatchX==(iSubPatches-1) && Sector.GetPosX()==(Terrain.GetNumSectorsX()-1))
						{
							iSubPatchEndX--;
						}
						if (iPatchY==(iNumPatches-1) && iSubPatchY==(iSubPatches-1) && Sector.GetPosY()==(Terrain.GetNumSectorsY()-1))
						{
							iSubPatchEndY--;
						}
					}

					// create a face set for the mesh triangulation
					C3DFaceSet* pFaceSet=new C3DFaceSet(pMesh,iMaterial);
					if (!pFaceSet)
					{
						CString strError;
						strError.Format("Error creating face set for sector %d/%d.",Sector.GetPosX(),Sector.GetPosY());
						MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}			

					// create triangulation from quad tree
					g_mapVertexCache.clear();
					if (!quadTree.GetBaseQuad() ||
						!this->TriangulateQuad(quadTree.GetBaseQuad()->FindQuad(iSubPatchStartX,iSubPatchStartY,this->m_iSubPatchSize),Sector,pMesh,pFaceSet))
					{
						CString strError;
						strError.Format("Error exporting sector %d/%d.\nCould not create mesh triangulation from quad tree terrain representation.",Sector.GetPosX(),Sector.GetPosY());
						MessageBox(NULL,strError,"Error exporting sector",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
					pMesh->AddFaceSet(pFaceSet);

				} // for SubPatchX
			} // for SubPatchY

			// recompute normals if mesh was simplified
			if (this->m_smMethod!=smNoSimplification)
			{
				// we need to refine it per sub patch in order to get normals right at their borders
				for (int iSubPatchY=0;iSubPatchY<iSubPatches;iSubPatchY++)
				{
					for (int iSubPatchX=0;iSubPatchX<iSubPatches;iSubPatchX++)
					{
						// get start and end position of this sub patch
						int iSubPatchStartX=iPatchX*iPatchSize+iSubPatchX*this->m_iSubPatchSize;
						int iSubPatchStartY=iPatchY*iPatchSize+iSubPatchY*this->m_iSubPatchSize;
						int iSubPatchEndX=this->m_iSubPatchSize+iSubPatchStartX;
						int iSubPatchEndY=this->m_iSubPatchSize+iSubPatchStartY;

						// recompute normals of inner vertices
						PNPTC_AABBOX bbox;
						bbox.vMin.x=static_cast<float>(iSubPatchStartX)*this->m_fPositionScale+this->m_fSectorOffsetX+Terrain.GetDetailHorizontalScaleFactor()/2.f;
						bbox.vMax.x=static_cast<float>(iSubPatchEndX)*this->m_fPositionScale+this->m_fSectorOffsetX-Terrain.GetDetailHorizontalScaleFactor()/2.f;
						bbox.vMin.y=static_cast<float>(iSubPatchStartY)*this->m_fPositionScale+this->m_fSectorOffsetY+Terrain.GetDetailHorizontalScaleFactor()/2.f;
						bbox.vMax.y=static_cast<float>(iSubPatchEndY)*this->m_fPositionScale+this->m_fSectorOffsetY-Terrain.GetDetailHorizontalScaleFactor()/2.f;
						bbox.vMin.z=Terrain.GetMinHeight()-1;
						bbox.vMax.z=Terrain.GetMaxHeight()+1;
						if (iUpAxis==0)
						{
							// y is up axis
							SWAP(bbox.vMin.z,bbox.vMin.y);
							SWAP(bbox.vMax.z,bbox.vMax.y);
						}
						pMesh->ComputeNormals(true,&bbox);
					}
				}
			}

			// --------------------------------------------------------------------------
			// write object to file
			std::string strObjFileName=strPath+strFilenameBase+(LPCSTR)strSuffix+"."+strExtension;
			if (!this->SaveObject(&objStore,strObjFileName))
			{
				CString strError;
				strError.Format("Error writing sector %d/%d to file\n'%s'.",Sector.GetPosX(),Sector.GetPosY(),strObjFileName.c_str());
				MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		} // for PatchX
	} // for PatchY

	// ok
	return true;
}









// -----------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------
// CTerrainMeshCreator::CTerrainQuadTree
// -----------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------





CTerrainMeshCreator::CTerrainQuadTree::CTerrainQuadTree()
	: m_pQuads(NULL)
	, m_bSimplifyBorderQuads(true)
{
}

CTerrainMeshCreator::CTerrainQuadTree::~CTerrainQuadTree()
{
	// free memory
	SAFE_DELETEARRAY(this->m_pQuads);
}

bool CTerrainMeshCreator::CTerrainQuadTree::BuildTree(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,int iOffsetX,int iOffsetY,int iBaseSize,int iLeafSize,bool bConsiderBlend,bool bConsiderHoles)
{
	// free memory first
	SAFE_DELETEARRAY(this->m_pQuads);

	// get textures if needed
	CIPnPTCSectorTextures01* pTextures=NULL;
	if (bConsiderBlend)
	{
		pTextures=Sector.GetSectorTextures();
	}

	// check sizes
	if (iBaseSize>static_cast<int>(Sector.GetBorderSize()))
	{
		iBaseSize=Sector.GetBorderSize();
	}
	if (iLeafSize>iBaseSize)
	{
		iLeafSize=iBaseSize;
	}
	if (iLeafSize<=0 || iBaseSize<=0)
	{
		CString strError;
		strError.Format("Error exporting sector %d/%d.\nThe sizes used for vertex reduction are not valid.",Sector.GetPosX(),Sector.GetPosY());
		MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}
	if (iLeafSize!=1 && iLeafSize!=2 && iLeafSize!=4 && iLeafSize!=8 && iLeafSize!=16 && iLeafSize!=32 && iLeafSize!=64 &&
		iLeafSize!=128 && iLeafSize!=256 && iLeafSize!=512 && iLeafSize!=1024)
	{
		CString strError;
		strError.Format("Error exporting sector %d/%d.\nThe sizes used for vertex reduction are not valid.",Sector.GetPosX(),Sector.GetPosY());
		MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}
	if (iBaseSize!=1 && iBaseSize!=2 && iBaseSize!=4 && iBaseSize!=8 && iBaseSize!=16 && iBaseSize!=32 && iBaseSize!=64 &&
		iBaseSize!=128 && iBaseSize!=256 && iBaseSize!=512 && iBaseSize!=1024)
	{
		CString strError;
		strError.Format("Error exporting sector %d/%d.\nThe sizes used for vertex reduction are not valid.",Sector.GetPosX(),Sector.GetPosY());
		MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// compute overall number of required quads
	int iNumLeafQuads=SQR(iBaseSize/iLeafSize);
	int iNumQuadsOnCurLevel=iNumLeafQuads;
	int iNumQuads=iNumQuadsOnCurLevel;
	while (iNumQuadsOnCurLevel!=1)
	{
		iNumQuadsOnCurLevel/=4;
		iNumQuads+=iNumQuadsOnCurLevel;
	}

	// create memory for quads
	this->m_pQuads=new CTerrainQuad[iNumQuads];
	if (!this->m_pQuads)
	{
		CString strError;
		strError.Format("Error exporting sector %d/%d.\nCould not created temporary memory for vertex reduction quad tree.",Sector.GetPosX(),Sector.GetPosY());
		MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// build the complete quad tree
	int iCurLevel=0;					// the current level in the quad tree hierarchy
	iNumQuadsOnCurLevel=1;					// the overall number of quads on this level
	int iQuad=0;						// the overall (over all levels) quad index
	int iCurLevelQuadSize=iBaseSize;	// the index of the first quad on this level
	int iCurLevelFirstQuad=0;			// the index of the first quad on this level
	int iPrevLevelFirstQuad=0;			// the index of the first quad on the previous level
	while (iCurLevelQuadSize>=iLeafSize)
	{
		// get the sqrt of the number of quads on this level
		int iSqrtNumQuadsOnCurLevel=static_cast<int>(sqrt(static_cast<double>(iNumQuadsOnCurLevel)));
		int iSqrtNumQuadsOnPrevLevel=iSqrtNumQuadsOnCurLevel/2;

		// leaf level?
		bool bLeaf=(iCurLevelQuadSize==iLeafSize);

		// go through all quads on this level
		for (int iQuadY=0;iQuadY<iSqrtNumQuadsOnCurLevel;++iQuadY)
		{
			for (int iQuadX=0;iQuadX<iSqrtNumQuadsOnCurLevel;++iQuadX,++iQuad)
			{
				// test quad index
				if (iQuad>=iNumQuads)
				{
					CString strError;
					strError.Format("Error exporting sector %d/%d.\nNot enough memory in quad tree used for vertex reduction.\n\nThis is an internal error!",Sector.GetPosX(),Sector.GetPosY());
					MessageBox(NULL,strError,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// create this new quad
				this->m_pQuads[iQuad].m_nSize=iCurLevelQuadSize;
				this->m_pQuads[iQuad].m_Position.x=iQuadX*iCurLevelQuadSize+iOffsetX;
				this->m_pQuads[iQuad].m_Position.y=iQuadY*iCurLevelQuadSize+iOffsetY;

				// leaf operations
				if (bLeaf)
				{
					// test texture blend
					if (pTextures)
					{
						// get all neighboring textures
						int iTex0=pTextures->GetTexture(this->m_pQuads[iQuad].m_Position.x,this->m_pQuads[iQuad].m_Position.y);
						int iTex1=pTextures->GetTexture(this->m_pQuads[iQuad].m_Position.x+1,this->m_pQuads[iQuad].m_Position.y);
						int iTex2=pTextures->GetTexture(this->m_pQuads[iQuad].m_Position.x,this->m_pQuads[iQuad].m_Position.y+1);
						int iTex3=pTextures->GetTexture(this->m_pQuads[iQuad].m_Position.x+1,this->m_pQuads[iQuad].m_Position.y+1);
// TODO: what happens at the sector border!!!!????
						if (iTex0>=0)
						{
							if (iTex1<0) iTex1=iTex0;
							if (iTex2<0) iTex2=iTex0;
							if (iTex3<0) iTex3=iTex0;
						}

						// test for blend
						if (iTex0!=iTex1 || iTex0!=iTex2 || iTex0!=iTex3)
						{
							this->m_pQuads[iQuad].m_bBlend=true;
						}
					}

					// test holes
					if (bConsiderHoles)
					{
						this->m_pQuads[iQuad].m_bVisible=Sector.GetQuadVisibility(this->m_pQuads[iQuad].m_Position.x,this->m_pQuads[iQuad].m_Position.y);
					}

					// set quad flip
					this->m_pQuads[iQuad].m_bFlipped=(Sector.GetQuadOrientation(this->m_pQuads[iQuad].m_Position.x,this->m_pQuads[iQuad].m_Position.y)==PNPTC_QUADOR_TL2BR);
				
				} // if (bLeaf)

				// connect with parent
				if (iCurLevel>0)
				{
					// compute parent quad index
					this->m_pQuads[iQuad].m_pParentQuad=&this->m_pQuads[iPrevLevelFirstQuad+(iQuadX/2)+(iQuadY/2*iSqrtNumQuadsOnPrevLevel)];

					// connect parent to this child
					if (this->m_pQuads[iQuad].m_pParentQuad)
					{
						// compute child index
						unsigned int nChildIndex=((iQuadX%2)==0)?0:1;
						nChildIndex+=((iQuadY%2)==0)?0:2;

						// set child pointer
						this->m_pQuads[iQuad].m_pParentQuad->m_pChildQuads[nChildIndex]=&this->m_pQuads[iQuad];

					} // if (this->m_pQuads[iQuad].m_pParentQuad)
				} // if (iCurLevel>0)
			} // for iQuadX
		} // for iQuadY

		// next level
		++iCurLevel;
		iCurLevelQuadSize/=2;
		iNumQuadsOnCurLevel*=4;
		iPrevLevelFirstQuad=iCurLevelFirstQuad;
		iCurLevelFirstQuad=iQuad;

	} // while (iCurLevelQuadSize>=iLeafSize)

	// successful
	return true;
}

bool CTerrainMeshCreator::CTerrainQuadTree::SimplifyTree(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,float fThreshold)
{
	// check root node
	if (!this->m_pQuads)
	{
		return false;
	}

	// simplify, until no further simplifications are possible
	while (this->SimplifyQuad(&this->m_pQuads[0],Sector,fThreshold)!=0) {}
	return true;
}

unsigned int CTerrainMeshCreator::CTerrainQuadTree::SimplifyQuad(CTerrainQuad* pQuad,CIPnPTCSector01& Sector,float fThreshold)
{
	// test input
	if (!pQuad)
	{
		return NULL;
	}

	// check the number of levels below this node
	unsigned int nLevelsBelow=pQuad->GetNumLevelsBelow();
	if (nLevelsBelow==0)
	{
		// nothing more to be simplified
		return 0;
	}
	else if (nLevelsBelow>1)
	{
		// more than one level => simplify all childs
		return	this->SimplifyQuad(pQuad->m_pChildQuads[0],Sector,fThreshold)+
				this->SimplifyQuad(pQuad->m_pChildQuads[1],Sector,fThreshold)+
				this->SimplifyQuad(pQuad->m_pChildQuads[2],Sector,fThreshold)+
				this->SimplifyQuad(pQuad->m_pChildQuads[3],Sector,fThreshold);
	}
	else
	{
		// exactly one level below => can it be simplified?
		if (this->TestQuadTreeSimplificationRestrictions(pQuad))
		{
			// test blending
			if (pQuad->m_pChildQuads[0]->m_bBlend || pQuad->m_pChildQuads[1]->m_bBlend ||
				pQuad->m_pChildQuads[2]->m_bBlend || pQuad->m_pChildQuads[3]->m_bBlend )
			{
				return 0;
			}

			// test holdes
			if (pQuad->m_pChildQuads[0]->m_bVisible!=pQuad->m_pChildQuads[1]->m_bVisible || 
				pQuad->m_pChildQuads[0]->m_bVisible!=pQuad->m_pChildQuads[2]->m_bVisible ||
				pQuad->m_pChildQuads[0]->m_bVisible!=pQuad->m_pChildQuads[3]->m_bVisible )
			{
				// some quads are visible, some not
				return 0;
			}
			else if (!pQuad->m_pChildQuads[0]->m_bVisible)
			{
				// all quads are either visible or invisble, and the first quad is invisble => all are invisible => simplify
				pQuad->m_bVisible=false;
				pQuad->m_pChildQuads[0]=NULL;
				pQuad->m_pChildQuads[1]=NULL;
				pQuad->m_pChildQuads[2]=NULL;
				pQuad->m_pChildQuads[3]=NULL;
				return 1;
			}

			// now test height deviation
			bool bQuadEast=(pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::East)!=NULL);
			bool bQuadNorth=(pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::North)!=NULL);
			bool bQuadSouth=(pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::South)!=NULL);
			bool bQuadWest=(pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::West)!=NULL);
			int iStepSize=pQuad->m_nSize/2;

			// test north west vertex
			float fRealHeight=Sector.GetHeight(pQuad->m_Position.x,pQuad->m_Position.y+iStepSize);
			if (bQuadNorth)
			{
				float fMeanHeight=(Sector.GetHeight(pQuad->m_Position.x,pQuad->m_Position.y)+Sector.GetHeight(pQuad->m_Position.x,pQuad->m_Position.y+static_cast<int>(pQuad->m_nSize)))/2.f;
				if (fabs(fMeanHeight-fRealHeight)>fThreshold) return 0;
			}
			if (bQuadWest)
			{
				float fMeanHeight=(Sector.GetHeight(pQuad->m_Position.x-iStepSize,pQuad->m_Position.y+iStepSize)+Sector.GetHeight(pQuad->m_Position.x+iStepSize,pQuad->m_Position.y+iStepSize))/2.f;
				if (fabs(fMeanHeight-fRealHeight)>fThreshold) return 0;
			}

			// test south east vertex
			fRealHeight=Sector.GetHeight(pQuad->m_Position.x+iStepSize,pQuad->m_Position.y);
			if (bQuadSouth)
			{
				float fMeanHeight=(Sector.GetHeight(pQuad->m_Position.x+iStepSize,pQuad->m_Position.y-iStepSize)+Sector.GetHeight(pQuad->m_Position.x+iStepSize,pQuad->m_Position.y+iStepSize))/2.f;
				if (fabs(fMeanHeight-fRealHeight)>fThreshold) return 0;
			}
			if (bQuadEast)
			{
				float fMeanHeight=(Sector.GetHeight(pQuad->m_Position.x,pQuad->m_Position.y)+Sector.GetHeight(pQuad->m_Position.x+static_cast<int>(pQuad->m_nSize),pQuad->m_Position.y))/2.f;
				if (fabs(fMeanHeight-fRealHeight)>fThreshold) return 0;
			}

			// can be simplified
			pQuad->m_pChildQuads[0]=NULL;
			pQuad->m_pChildQuads[1]=NULL;
			pQuad->m_pChildQuads[2]=NULL;
			pQuad->m_pChildQuads[3]=NULL;
			return 1;
		}
		else
		{
			// this quad can not be simplified 
			return 0;
		}
	}
}

bool CTerrainMeshCreator::CTerrainQuadTree::TestQuadTreeSimplificationRestrictions(CTerrainMeshCreator::CTerrainQuad* pQuad) const
{
	// check quad
	if (pQuad)
	{
		// don't simplify border quads?
		if (!this->m_bSimplifyBorderQuads)
		{
			// check if border quad
			if( pQuad->GetNeighbourQuad( CTerrainMeshCreator::CTerrainQuad::North ) == NULL || 
				pQuad->GetNeighbourQuad( CTerrainMeshCreator::CTerrainQuad::South ) == NULL || 
				pQuad->GetNeighbourQuad( CTerrainMeshCreator::CTerrainQuad::East )  == NULL || 
				pQuad->GetNeighbourQuad( CTerrainMeshCreator::CTerrainQuad::West )  == NULL ) 
			{
				return false;
			}
		}

		// get level of this quad
		unsigned int nLevel=pQuad->GetHierarchyLevel();

		// get neighboring quads
		CTerrainMeshCreator::CTerrainQuad* pQuads[4];
		pQuads[0]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::West);
		pQuads[1]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::East);
		pQuads[2]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::North);
		pQuads[3]=pQuad->GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::South);

		// check neighboring quads
		for (unsigned int n=0;n<4;++n)
		{
			if (pQuads[n])
			{
				if (pQuads[n]->GetHierarchyLevel()==nLevel)
				{
					// at the same level => test if there are more levels below
					if (pQuads[n]->GetNumLevelsBelow()>1)
					{
						return false;
					}
				}
			}
		}

		// can be simplified
		return true;
	}
	else
	{
		// invalid quad pointer
		return false;
	}
}





// -----------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------
// CTerrainMeshCreator::CTerrainQuad
// -----------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------





CTerrainMeshCreator::CTerrainQuad::CTerrainQuad()
	: m_nSize(1)
	, m_bVisible(true)
	, m_bBlend(false)
	, m_bFlipped(false)
	, m_pParentQuad(NULL)
{
	// further initializations
	this->m_Position.x=0;
	this->m_Position.y=0;
	this->m_pChildQuads[0]=NULL;
	this->m_pChildQuads[1]=NULL;
	this->m_pChildQuads[2]=NULL;
	this->m_pChildQuads[3]=NULL;
}

CTerrainMeshCreator::CTerrainQuad::~CTerrainQuad()
{
}

CTerrainMeshCreator::CTerrainQuad* CTerrainMeshCreator::CTerrainQuad::GetChildQuad(unsigned int nChild) const
{
	// check index 
	if (nChild<=3)
	{
		return this->m_pChildQuads[nChild];
	}
	else
	{
		// invalid index
		return NULL;
	}
}

unsigned int CTerrainMeshCreator::CTerrainQuad::GetHierarchyLevel() const
{
	// has it a parent?
	if (this->m_pParentQuad)
	{
		// one level below the parent
		return this->m_pParentQuad->GetHierarchyLevel()+1;
	}
	else
	{
		// root node
		return 0;
	}
}

unsigned int CTerrainMeshCreator::CTerrainQuad::GetNumLevelsBelow() const
{
	// is there a child node
	if (this->m_pChildQuads[0])
	{
		// at least on level below this node
		return maximum(this->m_pChildQuads[0]->GetNumLevelsBelow(),
					   maximum(this->m_pChildQuads[1]->GetNumLevelsBelow(),
							   maximum(this->m_pChildQuads[2]->GetNumLevelsBelow(),
							           this->m_pChildQuads[3]->GetNumLevelsBelow())))+1;
	}
	else
	{
		// no, this is the leaf
		return 0;
	}
}

CTerrainMeshCreator::CTerrainQuad* CTerrainMeshCreator::CTerrainQuad::GetNeighbourQuad(CTerrainMeshCreator::CTerrainQuad::DIRECTION Direction) const
{
	// go to the parent
	CTerrainQuad* pParent=this->GetParentQuad();
	if (pParent)
	{
		// get direction in which we want to go
		int iXDir=(Direction==CTerrainMeshCreator::CTerrainQuad::West)?-1:((Direction==CTerrainMeshCreator::CTerrainQuad::East)?1:0);
		int iYDir=(Direction==CTerrainMeshCreator::CTerrainQuad::South)?-1:((Direction==CTerrainMeshCreator::CTerrainQuad::North)?1:0);

		// find this node in the parent
		int iIndex=-1;
		if (pParent->m_pChildQuads[CTerrainMeshCreator::CTerrainQuad::SouthEastChild]==this) iIndex=CTerrainMeshCreator::CTerrainQuad::SouthEastChild;
		else if (pParent->m_pChildQuads[CTerrainMeshCreator::CTerrainQuad::SouthWestChild]==this) iIndex=CTerrainMeshCreator::CTerrainQuad::SouthWestChild;
		else if (pParent->m_pChildQuads[CTerrainMeshCreator::CTerrainQuad::NorthEastChild]==this) iIndex=CTerrainMeshCreator::CTerrainQuad::NorthEastChild;
		else if (pParent->m_pChildQuads[CTerrainMeshCreator::CTerrainQuad::NorthWestChild]==this) iIndex=CTerrainMeshCreator::CTerrainQuad::NorthWestChild;
		else
		{
			// not found in parent => error
			MessageBox(NULL,"Error walking through terrain export quad tree.\nThe interconnection between the tree levels is corrupted.\n\nThis is an internal error!","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return NULL;
		}
		int iX=iIndex%2;
		int iY=iIndex/2;
		int iNeighbourX=iX+iXDir;
		int iNeighbourY=iY+iYDir;		

		// test neighbour index
		if (iNeighbourX>=0 && iNeighbourX<2 && iNeighbourY>=0 && iNeighbourY<2)
		{
			// return that node
			int iNeighbourIndex=iNeighbourX+2*iNeighbourY;
			return pParent->m_pChildQuads[iNeighbourIndex];
		}
		else // neighbour id not available via the parent node
		{
			// we have to go one level heigher in the hierarchy

			// get information from nodes
			unsigned int nLevel=this->GetHierarchyLevel();
			unsigned int nParentLevel=pParent->GetHierarchyLevel();

			// get neighbour of parent
			CTerrainQuad* pParentNeighbour=pParent->GetNeighbourQuad(Direction);
			if (pParentNeighbour)
			{
				// move down the hierarchy of the parent's neighbour
				if (pParentNeighbour->GetHierarchyLevel()==nParentLevel && pParentNeighbour->GetNumLevelsBelow()>0)
				{
					// go down one level
					int iChildIndex=(abs(iX+iXDir)%2)+2*(abs(iY+iYDir)%2);
					return pParentNeighbour->m_pChildQuads[iChildIndex];
				}
				else
				{
					// return the parent neighbour on the final level
					return pParentNeighbour;
				}
			}
			else // !pParentNeighbour
			{
				// parent has no neighbour in this direction => there is no neighbour, border of the tree
				return NULL;
			}
		}
	}
	else // !pParent
	{
		// root node => no neighbour
		return NULL;
	}
}

CTerrainMeshCreator::CTerrainQuad* CTerrainMeshCreator::CTerrainQuad::FindQuad(int iQuadPosX,int iQuadPosY,unsigned int nQuadSize)
{
	// check if this is the quad to be returned
	if (this->m_Position.x==iQuadPosX && this->m_Position.y==iQuadPosY &&
		this->m_nSize==nQuadSize)
	{
		return this;
	}
	else if (this->m_pChildQuads[0])
	{
		// search below it
		CTerrainMeshCreator::CTerrainQuad* pQuadReturnBelow=NULL;
		for (unsigned int n=0;n<4;++n)
		{
			if (this->m_pChildQuads[n])
			{
				CTerrainMeshCreator::CTerrainQuad* pQuadReturnBelow=this->m_pChildQuads[n]->FindQuad(iQuadPosX,iQuadPosY,nQuadSize);
				if (pQuadReturnBelow)
				{
					return pQuadReturnBelow;
				}
			}		
		}

		// not found
		return NULL;
	}
	else
	{
		// not found
		return NULL;
	}
}
