/////////////////////////////////////////////////////////////////////////////////////
// TerrainMeshCreator.h
// creates a mesh from the terrain.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	27. Oct. 2005	: Added fully 3ds max support.
//  05. Dec. 2005	: Added overlay and lightmap export possibilities.
//  01. Jan. 2006	: Added y flipping of texture maps.
//  10. Oct. 2006	: Achieved VS2005 compatibility, made compilation easier.
//  10. Mai  2008	: Added quad tree for terrain mesh simplification.
//					  Changed all CString to std::string
//  23. Jun. 2008	: Added possibility to exclude border quads from terrain 
//					  mesh simplification.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_TERRAINMESHCREATOR_H_INCLUDED)
#define PNPTC_TERRAINMESHCREATOR_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef WINVER
#define WINVER 0x0500
#endif

#define _CRT_SECURE_NO_DEPRECATE 1

#include <afxwin.h>
#include "../../Includes/IPnPTCExportPlugIn.h"
#include "../../Includes/IPnPTCPlugIn.h"
#include "3DObjectData.h"
#include "../PnPTCUtils/Exporter.h"


// forward declarations
class C3DMesh;
class C3DFaceSet;


/// A callback interface used during mesh triangulation.
class ITriangulationCallback
{
public:

	/// Sets the texture coordinates of a vertex.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	/// @param vert The currently processed vertex.
	virtual void		SetTextureCoordinates(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const=0;

	/// Sets the vertex color.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	/// @param vert The currently processed vertex.
	virtual void		SetVertexColor(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const=0;

	/// Returns a hash code for a given vertex.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	virtual std::string	GetVertexHash(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const=0;

	/// Returns, whether or not this vertex should be created.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	virtual bool		GetCreateVertex(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const=0;

};	// class ITriangulationCallback


/// Mesh creator class.
class CTerrainMeshCreator : public ITriangulationCallback, public IExportFileNameCallback
{
public:

	// ---------------------------------------------------------------------------
	// construction/destruction
	// ---------------------------------------------------------------------------

	/// Default construction and initialization of
	/// mesh creator.
	CTerrainMeshCreator();

	/// Destruction of the mesh creator.
	~CTerrainMeshCreator(void);


	// ---------------------------------------------------------------------------
	// public data types for mesh creation
	// ---------------------------------------------------------------------------

	typedef enum SIMPLIFICATION_METHOD
	{
		smNoSimplification	= 0,	//!< the mesh is build without any vertex reduction
		smRegular			= 1,	//!< the mesh is build with a regular vertex reduction
		smHeightDeviation	= 2		//!< the mesh is build using an error metric depending on height deviations
//		smSlopeDeviation	= 3		//!< the mesh is build using an error metric depending on slope deviations
	} SIMPLIFICATION_METHOD;


	// ---------------------------------------------------------------------------
	// setting parameters for mesh simplification
	// ---------------------------------------------------------------------------

	/// Sets the simplification method to be used for mesh creation.
	/// @param smMethod Simplification method to be used.
	void			SetSimplificationMethod(SIMPLIFICATION_METHOD smMethod);

	/// Sets the error value which should be used for height and
	/// slope based error metrics.
	/// @param fMaxError The maximum error before a triangle is split.
	void			SetErrorValue(float fMaxError);

	/// Sets the desired regular grid spacing, which is used for
	/// regular mesh simplification.
	/// @param iGridSpacing Grid spacing for regular simplification.
	/// This parameter must have a value of 1, 2, 4, 8, 16, ...
	void			SetRegularGridSpacing(unsigned int nGridSpacing);

	/// Sets whether or not border quads can be simplified.
	/// @param bSimplifyBorderQuads Whether or not border quads can be simplified.
	void			SetSimplifyBorderQuads(bool bSimplifyBorderQuads) {this->m_bSimplifyBorderQuads=bSimplifyBorderQuads;};

	/// Returns whether or not border quads can be simplified.
	/// @retrun Whether or not border quads can be simplified.
	bool			GetSimplifyBorderQuads() const {return this->m_bSimplifyBorderQuads;};


	// ---------------------------------------------------------------------------
	// IExportFileNameCallback implementation
	// ---------------------------------------------------------------------------

	/// Returns a file name for a given patch.
	/// @param pSector The sector, which should be exported.
	/// @param strBaseName The base name of the export file name.
	/// @param nPatches The number of patches per sector border.
	/// @param nPatchX The index (starting with 0) of the exported
	/// patch of this sector in x direction.
	/// @param nPatchY The index (starting with 0) of the exported
	/// patch of this sector in y direction.
	/// @param strExtraInfo Extra info string to be contained in the file name.
	std::string		GetExportFileName(CIPnPTCSector01* pSector,const std::string& strBaseName,unsigned int nPatches,unsigned int nPatchX,unsigned int nPatchY,const std::string& strExtraInfo="") const;


	// ---------------------------------------------------------------------------
	// ITriangulationCallback implementation
	// ---------------------------------------------------------------------------

	/// Sets the texture coordinates of a vertex.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	/// @param vert The currently processed vertex.
	void			SetTextureCoordinates(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const;

	/// Sets the vertex color.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	/// @param vert The currently processed vertex.
	void			SetVertexColor(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY,PNPTC_VERTEX& vert) const;

	/// Returns a hash code for a given vertex.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	std::string		GetVertexHash(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const;

	/// Returns, whether or not this vertex should be created.
	/// @param Sector The currently processed sector.
	/// @param iSecPosX The currently x position on the sector
	/// @param iSecPosY The currently y position on the sector
	bool			GetCreateVertex(CIPnPTCSector01& Sector,int iSecPosX,int iSecPosY) const;


protected:

	// ---------------------------------------------------------------------------
	// creating the heightmap mesh
	// ---------------------------------------------------------------------------

	/// Creating an untextured terrain mesh.
	/// @param App PnP TerrainCreator application object.
	/// @param Sector The sector which should be exported.
	/// @param Terrain The terrain project from which the sector should be exported.
	/// @param strFileName The file to which the terrain should be written.
	/// @param fScale The scaling factor from metrical units to 3D units used in the mesh.
	/// @param iPathSize The border size of the patches to be created from the sector.
	/// Possible values are 64, 128, 256, 512, 1024.
	/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
	/// <i>1</i> y is the up axis (left handed).
	/// @param bGlobalPositioning Whether or not the coordinates of the terrain sector vertices 
	/// should be placed at their global position. In this way, the meshes of terrains with more
	/// than one sector will be placed correctly next to each other.
	/// @param bUseRandomNames Wheter or not to use random names for dependend files (textures)
	/// and entity names within the mesh. Those random names are compatible with the DOS file
	/// 8.3 file nameing convention.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the terrain mesh could be
	/// created successfully.
	bool			WriteSectorUntextured(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iUpAxis,bool bGlobalPositioning,bool bUseRandomNames=false);

	/// Creating a mesh with blended detail textures.
	/// @param App PnP TerrainCreator application object.
	/// @param Sector The sector which should be exported.
	/// @param Terrain The terrain project from which the sector should be exported.
	/// @param strFileName The file to which the terrain should be written.
	/// @param fScale The scaling factor from metrical units to 3D units used in the mesh.
	/// @param iPathSize The border size of the patches to be created from the sector.
	/// Possible values are 64, 128, 256, 512, 1024.
	/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
	/// <i>1</i> y is the up axis (left handed).
	/// @param bGlobalPositioning Whether or not the coordinates of the terrain sector vertices 
	/// should be placed at their global position. In this way, the meshes of terrains with more
	/// than one sector will be placed correctly next to each other.
	/// @param bNoiseOverlay Whether or not to use the overlay texture within the mesh.
	/// @param strLightMap Optional name of the lightmap to be used within the mesh.
	/// @param bUseRandomNames Wheter or not to use random names for dependend files (textures)
	/// and entity names within the mesh. Those random names are compatible with the DOS file
	/// 8.3 file nameing convention.
	/// @param iExportTextureMapSize The size in heightmap elements of the texture maps to be created.
	/// <i>-1</i> means the patch size is used.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the terrain mesh could be
	/// created successfully.
	bool			WriteSectorDetailTexture(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,bool bNoiseOverlay=false,const std::string& strLightMap="",bool bUseRandomNames=false,int iExportTextureMapSize=-1);

	/// Creating a texture mapped mesh.
	/// @param App PnP TerrainCreator application object.
	/// @param Sector The sector which should be exported.
	/// @param Terrain The terrain project from which the sector should be exported.
	/// @param strFileName The file to which the terrain should be written.
	/// @param fScale The scaling factor from metrical units to 3D units used in the mesh.
	/// @param iPathSize The border size of the patches to be created from the sector.
	/// Possible values are 64, 128, 256, 512, 1024.
	/// @param iUpAxis The up axis used within the mesh. <i>0</i> z is the up axis (right handed), 
	/// <i>1</i> y is the up axis (left handed).
	/// @param bGlobalPositioning Whether or not the coordinates of the terrain sector vertices 
	/// should be placed at their global position. In this way, the meshes of terrains with more
	/// than one sector will be placed correctly next to each other.
	/// @param bNoiseOverlay Whether or not to use the overlay texture within the mesh.
	/// @param strLightMap Optional name of the lightmap to be used within the mesh.
	/// @param bCombineTextureAndLightmap Whether or not to combine the texturemap and
	/// the lightmap into one single texture file.
	/// @param bUseRandomNames Wheter or not to use random names for dependend files (textures)
	/// and entity names within the mesh. Those random names are compatible with the DOS file
	/// 8.3 file nameing convention.
	/// @param bFlipTextureY Whether or not to flip the y coordinates of texture map. Depends on
	/// whether texture coordinates are intepreted as going from the images top to bottom or 
	/// vice versa.
	/// @param iExportTextureMapSize The size in heightmap elements of the texture maps to be created.
	/// <i>-1</i> means the patch size is used.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the terrain mesh could be
	/// created successfully.
	bool			WriteSectorTexturemap(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& strFileName,float fScale,int iPatchSize,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,bool bNoiseOverlay=false,const std::string& strLightMap="",bool bCombineTextureAndLightmap=false,bool bUseRandomNames=false,bool bFlipTextureY=false,int iExportTextureMapSize=-1);



	/// Writes the created object to a file.
	/// This method is called by the WriteSectorXXX methods
	/// and has to be implemented in derived classes in order
	/// to save to the appropriate file format.
	/// @param pObj object to be saved.
	/// @param strFileName destination file name.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the object could successfully be saved.
	virtual bool	SaveObject(C3DObjectData* pObj,const std::string& strFileName)=0;

private:

	// ---------------------------------------------------------------------------
	// internal data types
	// ---------------------------------------------------------------------------

	/// Enumeration of vertex generation modes.
	typedef enum VERTEXGENMODE
	{
		vgmUntexturedVertices			= 0,		//!< Vertices for untextured terrain mesh creation.
		vgmTexturemapVertices			= 1,		//!< Vertices for texturemap terrain mesh creation.
		vgmBlendBackgroundVertices		= 2,		//!< Vertices for the colored blend back ground used for vertex alpha blended textured terrain mesh creation.
		vgmVertexBlendTextureVertices	= 3			//!< Vertices for vertex alpha blended textured terrain mesh creation.
	} VERTEXGENMODE;


	// forward declarations
	class CTerrainQuad;


	/// A terrain surface quad tree used for vertex reduction.
	class CTerrainQuadTree
	{

	public:

		// -----------------------------------------------------------------------
		// construction / destruction
		// -----------------------------------------------------------------------

		/// Construction of a new quad tree.
		CTerrainQuadTree();

		/// Destruction of the quad tree.
		~CTerrainQuadTree();


		// -----------------------------------------------------------------------
		// quad tree generation parameters
		// -----------------------------------------------------------------------

		/// Sets whether or not border quads can be simplified.
		/// @param bSimplifyBorderQuads Whether or not border quads can be simplified.
		void			SetSimplifyBorderQuads(bool bSimplifyBorderQuads) {this->m_bSimplifyBorderQuads=bSimplifyBorderQuads;};

		/// Returns whether or not border quads can be simplified.
		/// @retrun Whether or not border quads can be simplified.
		bool			GetSimplifyBorderQuads() const {return this->m_bSimplifyBorderQuads;};


		// -----------------------------------------------------------------------
		// quad tree generation
		// -----------------------------------------------------------------------

		/// Builds the quad tree.
		/// @param App PnP TerrainCreator application object.
		/// @param Sector The sector which should be exported.
		/// @param Terrain The terrain project from which the sector should be exported.
		/// @param iOffsetX X offset on the sector where to start with the quad tree generation.
		/// @param iOffsetY Y offset on the sector where to start with the quad tree generation.
		/// @param iBaseSize Size of the base node of the quad tree.
		/// @param iLeafSize Size of the leaf nodes of the quad tree.
		/// @param bConsiderBlend Whether or not texture blending should be considered
		/// during quad tree simplification.
		/// @param bConsiderHoles Whether or not holes blending should be considered
		/// during quad tree simplification.
		/// @return Returns <i>true</i> or <i>false</i>, whether or not the quad tree
		/// could be build successfully.
		bool			BuildTree(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,int iOffsetX,int iOffsetY,int iBaseSize,int iLeafSize,bool bConsiderBlend,bool bConsiderHoles);

		/// Simplifies the quad tree.
		/// @param App PnP TerrainCreator application object.
		/// @param Sector The sector which should be exported.
		/// @param Terrain The terrain project from which the sector should be exported.
		/// @param fThreshold The height deviation threshold used for the quad tree simplification.
		/// @return Returns <i>true</i> or <i>false</i>, whether or not the quad tree
		/// could be simplified successfully.
		bool			SimplifyTree(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,float fThreshold);


		// -----------------------------------------------------------------------
		// access to quad tree
		// -----------------------------------------------------------------------

		/// Get base quad of tree.
		/// @return Returns the base quad of tree.
		CTerrainQuad*	GetBaseQuad() const {return this->m_pQuads;};


	private:

		// -----------------------------------------------------------------------
		// member variables
		// -----------------------------------------------------------------------

		/// All the quads of this tree.
		CTerrainQuad*	m_pQuads;

		/// Whether or not border quads can be simplified.
		bool			m_bSimplifyBorderQuads;

		/// Simplifies one quad and its children of the tree.
		/// @param pQuad The quad to be simplified.
		/// @param Sector The sector which should be exported.
		/// @param fThreshold The height deviation threshold used for the quad tree simplification.
		/// @return Returns the number of simplifications performed on that quad.
		unsigned int	SimplifyQuad(CTerrainQuad* pQuad,CIPnPTCSector01& Sector,float fThreshold);


		// -----------------------------------------------------------------------
		// internal methods
		// -----------------------------------------------------------------------

		/// Tests, whether or not a quad can be simplified, given the restriced nature
		/// of a quad tree for terrain surface/mesh simplification.
		/// No neighbouring quad on the same level is allowed to have
		/// have any further child quads below it. Neighbouring quads
		/// may only differ by 1 in the level in the hierarchy.
		/// @param pQuad The quad to be simplified.
		/// @return Returns <i>true</i> if the quad can be simplified or 
		/// <i>false</i> if not.
		bool			TestQuadTreeSimplificationRestrictions(CTerrainQuad* pQuad) const;


	}; // class CTerrainQuadTree


	/// Class representing one quad of the terrain in a quad-tree.
	class CTerrainQuad
	{

		// access to members
		friend class CTerrainQuadTree;

	public:

		// -----------------------------------------------------------------------
		// public data types
		// -----------------------------------------------------------------------

		/// Direction enumeration used to access neighboring nodes.
		typedef enum DIRECTION
		{
			South	= 0,
			West	= 1,
			North	= 2,
			East	= 3
		} DIRECTION;


		// -----------------------------------------------------------------------
		// get data from quad
		// -----------------------------------------------------------------------

		/// Returns the size of this quad.
		/// @return The size of this quad.
		unsigned int		GetSize() const {return this->m_nSize;};

		/// Returns the position of the south/west corner of this quad.
		/// @return The position of the south/west corner of this quad.
		PnPTCHMapPosition	GetPosition() const {return this->m_Position;};

		/// Returns whether or not this quad is visible.
		/// @return Whether or not this quad is visible.
		bool				GetIsVisible() const {return this->m_bVisible;};

		/// Returns whether or not this quad blends between two or more textures.
		/// @return Whether or not this quad blends between two or more textures.
		bool				GetIsBlendQuad() const {return this->m_bBlend;};

		/// Returns whether or not this quad is flipped.
		/// @return Whether or not this quad is flipped.
		bool				GetIsFlipped() const {return this->m_bFlipped;};

		/// Returns whether or not this quad is a leaf.
		/// @return Whether or not this quad is a leaf.
		bool				GetIsLeaf() const {return this->m_pChildQuads[0]==NULL;};

		/// Returns the level in the quad tree hierarchy of this quad.
		/// @return the level in the quad tree hierarchy of this quad.
		unsigned int		GetHierarchyLevel() const;

		/// Returns the number of levels below this quad in the quad tree hierarchy.
		/// @return the number of levels below this quad in the quad tree hierarchy.
		unsigned int		GetNumLevelsBelow() const;


		// -----------------------------------------------------------------------
		// access to neighboring, parent, and child nodes
		// -----------------------------------------------------------------------

		/// Returns one child quad.
		/// @param nChild Index of the child quad to be returned in the range [0..3].
		/// @return Returns a pointer to the child quad if successful or <i>NULL</i>
		/// in case of an error or if this child does not exist.
		CTerrainQuad*		GetChildQuad(unsigned int nChild) const;

		/// Returns the parent quad.
		/// @return Returns a pointer to the parent quad if successful or <i>NULL</i>
		/// in case of an error, i.e. if this is the root quad.
		CTerrainQuad*		GetParentQuad() const {return this->m_pParentQuad;};

		/// Returns the neighboring node. The returned node is on the same
		/// level as this node in the quad tree hierarchy or above it.
		/// @param Direction The direction in which to search for
		/// the neighboring node.
		/// @return Returns a pointer to the neighboring node if
		/// successful, or <i>NULL</i> if no such node exists
		CTerrainQuad*		GetNeighbourQuad(DIRECTION Direction) const;

		/// Returns one quad below this quad with a given size and at a given position.
		/// @param iQuadPosX The x position of the quad to be returned.
		/// @param iQuadPosY The y position of the quad to be returned.
		/// @param nQuadSize The size of the quad to be returned.
		CTerrainQuad*		FindQuad(int iQuadPosX,int iQuadPosY,unsigned int nQuadSize);


	private:

		// -----------------------------------------------------------------------
		// construction / destruction is restricted
		// -----------------------------------------------------------------------

		/// Construction of a new quad.
		CTerrainQuad();

		/// Destruction of the quad.
		~CTerrainQuad();


		// -----------------------------------------------------------------------
		// internal data types
		// -----------------------------------------------------------------------

		/// Enumeration for child node indices.
		typedef enum CHILDNODEINDEX
		{
			SouthWestChild	= 0,
			SouthEastChild	= 1,
			NorthWestChild	= 2,
			NorthEastChild	= 3
		} CHILDNODEINDEX;


		// -----------------------------------------------------------------------
		// member variables
		// -----------------------------------------------------------------------

		/// Size of this quad.
		unsigned int		m_nSize;

		/// The position of the south/west corner of this quad.
		PnPTCHMapPosition	m_Position;

		/// Whether or not this quad is visible.
		bool				m_bVisible;

		/// Whether or not this quad blends between two or more textures.
		bool				m_bBlend;

		/// Whether or not this quad is flipped.
		bool				m_bFlipped;

		/// The parent quad of this terrain quad.
		// TODO: as they are all stored in a sequential memory region, these values can be recoverd from an index.
		CTerrainQuad*		m_pParentQuad;

		/// Child terrain quads of this terrain quad
		// TODO: as they are all stored in a sequential memory region, these values can be recoverd from an index and a boolean IsLeaf flag.
		CTerrainQuad*		m_pChildQuads[4];

		/// List of all vertices created by this quad
		std::vector<int>	m_vecVertices;

	}; // class CTerrainQuad


	// ---------------------------------------------------------------------------
	// member variables
	// ---------------------------------------------------------------------------

	/// Mesh simplification method used during mesh creation.
	SIMPLIFICATION_METHOD	m_smMethod;

	/// The maximum error before a triangle is split in height/slope based mesh simplification.
	float					m_fMaxError;

	/// Grid spacing for regular mesh simplification.
	unsigned int			m_nGridSpacing;

	/// Whether or not border quads can be simplified.
	bool					m_bSimplifyBorderQuads;

	/// X position of the exported sector, used for global positioning.
	float					m_fSectorOffsetX;

	/// Y position of the exported sector, used for global positioning.
	float					m_fSectorOffsetY;

	/// The scaling factor used for all positions.
	float					m_fPositionScale;

	/// The up axis used within the mesh. <i>0</i> z is the up axis (right handed), <i>1</i> y is the up axis (left handed).
	int						m_iUpAxis;

	/// The current vertex generation mode.
	VERTEXGENMODE			m_VertexGenerationMode;

	/// The ID of the texture for which the mesh triangulation is currently created.
	int						m_iCurrentTextureID;

	/// The UV coordinate pair index for the overlay texture.
	int						m_iOverlayTextureIndex;

	/// The size of the overlay texture.
	int						m_iOverlayTextureSize;

	/// The UV coordinate pair index for the lightmap texture.
	int						m_iLightmapTextureIndex;

	/// Start position of the currently exported texture map in x direction on the sector.
	int						m_iTextureMapSectorStartPosX;

	/// Start position of the currently exported texture map in y direction on the sector.
	int						m_iTextureMapSectorStartPosY;

	/// The border size of the texturemap bitmap in pixels
	unsigned int			m_nTextureMapBitmapSize;

	/// The border size of the lighmap bitmap in pixels
	unsigned int			m_nLightMapBitmapSize;

	/// The size of the currently exported sub patch.
	int						m_iSubPatchSize;

	/// Whether to use random names for the textures (for 8.3 file name compatibility).
	bool					m_bUseRandomNames;

	/// The start position of the currently exported patch in x direction.
	int						m_iPatchStartX;

	/// The start position of the currently exported patch in x direction.
	int						m_iPatchStartY;

	/// The size of the currently exported detail texture.
	int						m_iDetailTextureSize;

	/// The number of pixel by which the uv coordinates of texture/lightmaps should be offset.
	float					m_fUVPixelOffset;


	// ---------------------------------------------------------------------------
	// internal methods
	// ---------------------------------------------------------------------------

	/// Triangulates one terrain quad.
	/// @param pQuad The quad to be triangulated.
	/// @param Sector The sector from which the triangulation should be created.
	/// @param pMesh The mesh into which the triangulation should be stored.
	/// @param pFaceSet The face set, which will be used for the triangulation.
	bool					TriangulateQuad(CTerrainQuad* pQuad,CIPnPTCSector01& Sector,C3DMesh* pMesh,C3DFaceSet* pFaceSet);

}; // class CTerrainMeshCreator

	
#endif // !defined(PNPTC_TERRAINMESHCREATOR_H_INCLUDED)
