/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerX.cpp
// 3d object serialization class for .x files.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//	03. Oct. 2006	: Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_DEPRECATE 1

#include "3DObjectData.h"
#include ".\3dobjectserializerx.h"
#include "../PnPTCUtils/Utils.h"


// --------------------------------------------------------------------------------------------
// binary tokens

// Record-bearing
#define TOKEN_NAME 1
#define TOKEN_STRING 2
#define TOKEN_INTEGER 3
#define TOKEN_INTEGER_LIST 6
#define TOKEN_REALNUM_LIST 7

// Stand-alone tokens
#define TOKEN_OBRACE 10
#define TOKEN_CBRACE 11
#define TOKEN_SEMICOLON 20


// --------------------------------------------------------------------------------------------
// writing binary data

bool XPrintWORD(FILE* f,WORD wValue)
{
	// print word
	return (fwrite(&wValue,sizeof(WORD),1,f)==1);
}

bool XPrintDWORD(FILE* f,DWORD dwValue)
{
	// print dword
	return (fwrite(&dwValue,sizeof(DWORD),1,f)==1);
}

bool XPrintFloat(FILE* f,float fValue)
{
	// print float
	return (fwrite(&fValue,sizeof(float),1,f)==1);
}

bool XPrintChars(FILE* f,const char* cString)
{
	// print string length
	DWORD dwLength=(DWORD)strlen(cString);
	if (!XPrintDWORD(f,dwLength))
	{
		return false;
	}

	// print string
	return (fwrite(cString,dwLength,1,f)==1);
}

// --------------------------------------------------------------------------------------------
// writing token

bool XPrintToken(FILE* f,WORD wToken)
{
	// print one token
	return XPrintWORD(f,wToken);
}

// --------------------------------------------------------------------------------------------
// writing record-bearing tokens

bool XPrintInteger(FILE* f,DWORD dwInteger)
{
	// print integer token
	if (!XPrintToken(f,TOKEN_INTEGER))
	{
		return false;
	}

	// print value
	return XPrintDWORD(f,dwInteger);
}

bool XPrintName(FILE* f,const char* cName)
{
	// print name token
	if (!XPrintToken(f,TOKEN_NAME))
	{
		return false;
	}

	// print string
	return XPrintChars(f,cName);
}

bool XPrintIntList(FILE* f,DWORD dwNum,DWORD* dwList)
{
	// print integer list token
	if (!XPrintToken(f,TOKEN_INTEGER_LIST))
	{
		return false;
	}

	// print number of integers
	if (!XPrintDWORD(f,dwNum))
	{
		return false;
	}

	// print integers
	return (fwrite(dwList,dwNum*sizeof(DWORD),1,f)==1);
}

bool XPrintFloatList(FILE* f,DWORD dwNum,float* fList)
{
	// print real number list token
	if (!XPrintToken(f,TOKEN_REALNUM_LIST))
	{
		return false;
	}

	// print number of floats
	if (!XPrintDWORD(f,dwNum))
	{
		return false;
	}

	// print floats
	return (fwrite(fList,dwNum*sizeof(float),1,f)==1);
}

bool XPrintFloatList(FILE* f,DWORD dwNum,double* dList)
{
	// print real number list token
	if (!XPrintToken(f,TOKEN_REALNUM_LIST))
	{
		return false;
	}

	// print number of doubles
	if (!XPrintDWORD(f,dwNum))
	{
		return false;
	}

	// print doubles
	return (fwrite(dList,dwNum*sizeof(double),1,f)==1);
}

std::string MakeCompatibleString(const std::string& strInput)
{
	// initialize return string
	std::string strReturn(strInput);

	// Replace blanks
	Replace(&strReturn," ","_");
	// Replace +
	Replace(&strReturn,"+","_");
	// Replace :
	Replace(&strReturn,":","_");
	// Replace ;
	Replace(&strReturn,";","_");
	// Replace /
	Replace(&strReturn,"/","_");
	// Replace \ 
	Replace(&strReturn,"\\","_");

	// ensure not to start with a digit
	if (strReturn.length()>0)
	{
		if (strReturn.at(0)>='0' && strReturn.at(0)<='9')
		{
			strReturn=std::string("_")+strReturn;
		}
	}

	// return string
	return strReturn;
}




C3DObjectSerializerX::C3DObjectSerializerX(void)
	: m_iFormat(0)
{
}

C3DObjectSerializerX::~C3DObjectSerializerX(void)
{
}

bool C3DObjectSerializerX::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// loading not yet supported => loading is implemented using the D3DXLoadMesh routines
	return false;
}

bool C3DObjectSerializerX::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// -----------------------------------------------------------------------------
	// create file
	std::string strFormat="w";
	if (this->m_iFormat!=0)
	{
		strFormat+="b";
	}
	FILE* f=fopen(strFileName.c_str(),strFormat.c_str());
	if (!f)
	{
		MessageBox(NULL,(std::string("The file ")+strFileName+std::string(" could not be opened for writing.")).c_str(),"Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// -----------------------------------------------------------------------------
	// print file header
	if (this->m_iFormat==0)
	{
		// text format
		if (fprintf(f,"xof 0303txt 0032\n\n")<0)
		{
			// error
			MessageBox(NULL,"Error writing file header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}
	}
	else
	{
		// binary format
		if (fprintf(f,"xof 0303bin 0032")<0)
		{
			// error
			MessageBox(NULL,"Error writing file header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			fclose(f);
			return false;
		}
	}



	// -----------------------------------------------------------------------------
	// write all materials used in the object

	for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)
	{
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMaterial);
		if (pMaterial)
		{
			// write material data
			if (this->m_iFormat==0)
			{
				// textual
				if (fprintf(f,"Material %s {\n %f;%f;%f;%f;;\n 0.0;\n 0.0;0.0;0.0;;\n 0.0;0.0;0.0;;\n",
							MakeCompatibleString(pMaterial->strName).c_str(),
							static_cast<float>(pMaterial->colAmbient.r)/255.f,
							static_cast<float>(pMaterial->colAmbient.g)/255.f,
							static_cast<float>(pMaterial->colAmbient.b)/255.f,
							static_cast<float>(pMaterial->colAmbient.a)/255.f)<0)
				{
					// error
					MessageBox(NULL,"Error writing material to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			else
			{
				// binary
				if (!XPrintName(f,"Material") ||
					!XPrintName(f,MakeCompatibleString(pMaterial->strName).c_str()) ||
					!XPrintToken(f,TOKEN_OBRACE) ||
					!XPrintToken(f,TOKEN_REALNUM_LIST) ||
					!XPrintDWORD(f,11) ||
					!XPrintFloat(f,static_cast<float>(pMaterial->colAmbient.r)/255.f) ||
					!XPrintFloat(f,static_cast<float>(pMaterial->colAmbient.g)/255.f) ||
					!XPrintFloat(f,static_cast<float>(pMaterial->colAmbient.b)/255.f) ||
					!XPrintFloat(f,static_cast<float>(pMaterial->colAmbient.a)/255.f) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) ||
					!XPrintFloat(f,0) )
				{
					// error
					MessageBox(NULL,"Error writing material to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}

			// write textures used in this material
			if (pMaterial->iTextures>0)
			{		
				// write texture data
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," TextureFilename {\"%s\";}\n",pMaterial->strTexture[0].c_str())<0)
					{
						// error
						MessageBox(NULL,"Error writing material texture to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintName(f,"TextureFilename") ||
						!XPrintToken(f,TOKEN_OBRACE) ||
						!XPrintToken(f,TOKEN_STRING) ||
						!XPrintChars(f,pMaterial->strTexture[0].c_str()) ||
						!XPrintToken(f,TOKEN_SEMICOLON) ||
						!XPrintToken(f,TOKEN_CBRACE) )
					{
						// error
						MessageBox(NULL,"Error writing material texture to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}

			// write material footer
			if (this->m_iFormat==0)
			{
				// textual
				if (fprintf(f,"}\n")<0)
				{
					// error
					MessageBox(NULL,"Error finishing material to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			else
			{
				// binary
				if (!XPrintToken(f,TOKEN_CBRACE))
				{
					// error
					MessageBox(NULL,"Error writing material footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
		}
	}


	// -----------------------------------------------------------------------------
	// write all meshes of object
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		// get mesh
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		if (pMesh)
		{
			DWORD dwVertices=(DWORD)pMesh->GetNumVertices();

			// print mesh header
			if (this->m_iFormat==0)
			{
				// text
				if (fprintf(f,"Mesh %s {\n %d;\n",pMesh->GetName().c_str(),dwVertices)<0)
				{
					// error
					MessageBox(NULL,"Error writing mesh header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			else
			{
				// binary			
				DWORD dwVertexCoords=dwVertices*3;
				if (!XPrintName(f,"Mesh") || !XPrintName(f,pMesh->GetName().c_str()) ||		// mesh header
					!XPrintToken(f,TOKEN_OBRACE) ||											// start of mesh section
					!XPrintIntList(f,1,&dwVertices) ||										// number of vertices of mesh
					!XPrintToken(f,TOKEN_REALNUM_LIST) ||									// start list of vertex coordinates
					!XPrintDWORD(f,dwVertexCoords) )										// number of vertex coordinates
				{
					// error
					MessageBox(NULL,"Error writing mesh header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}

			// write all vertices
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				// write position data
				const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
				PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
				GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," %f;%f;%f;",vecPos.x,vecPos.y,vecPos.z)<0)
					{
						// error
						MessageBox(NULL,"Error writing vertex data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}

					// write seperator
					std::string strSeperator=(lVert<(dwVertices-1))?",":";";
					if (fprintf(f,"%s\n",strSeperator.c_str())<0)
					{
						// error
						MessageBox(NULL,"Error writing coordinate delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintFloat(f,vecPos.x) ||		// x position of vertex
						!XPrintFloat(f,vecPos.y) ||		// y position of vertex
						!XPrintFloat(f,vecPos.z) )		// z position of vertex
					{
						MessageBox(NULL,"Error writing vertex position to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}

			// --------------------------------------------------------------------
			// write all faces
			DWORD dwFaces=static_cast<DWORD>(pMesh->GetNumFaces());

			// write face list header
			if (this->m_iFormat==0)
			{
				// textual
				if (fprintf(f,"%d;\n",dwFaces)<0)
				{
					// error
					MessageBox(NULL,"Error writing face header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			else
			{
				// binary
				DWORD dwFaceIndices=dwFaces*4;
				if (!XPrintIntList(f,1,&dwFaces) ||
					!XPrintToken(f,TOKEN_INTEGER_LIST) ||
					!XPrintDWORD(f,dwFaceIndices))
				{
					// error
					MessageBox(NULL,"Error writing face header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}

			// write faces
			for (unsigned long lFaceSet=0,lFaceCount=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
				if (pFaceSet)
				{
					for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++,lFaceCount++)
					{
						const PNPTC_FACE& face=pFaceSet->GetFace(lFace);
						if (this->m_iFormat==0)
						{
							// textual
							if (fprintf(f," 3;%d;%d;%d;",face.dwVert[0],face.dwVert[1],face.dwVert[2])<0)
							{
								// error
								MessageBox(NULL,"Error writing triangle data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
								fclose(f);
								return false;
							}

							// write seperator
							std::string strSeperator=(lFaceCount<(dwFaces-1))?",":";";
							if (fprintf(f,"%s\n",strSeperator.c_str())<0)
							{
								// error
								MessageBox(NULL,"Error writing triangle delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
								fclose(f);
								return false;
							}
						}
						else
						{
							// binary
							if (!XPrintDWORD(f,3) ||					// number of vertices per face
								!XPrintDWORD(f,face.dwVert[0]) ||		// first vertex index
								!XPrintDWORD(f,face.dwVert[1]) ||		// second vertex index
								!XPrintDWORD(f,face.dwVert[2]) )		// third vertex index
							{
								// error
								MessageBox(NULL,"Error writing triangle data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
								fclose(f);
								return false;
							}
						}
					}
				}
			}

			// -----------------------------------------------------------------------
			// Write Materials

			// get number of materials used by the mesh
			std::vector<std::string>	vecMaterials;
			for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
				if (pFaceSet)
				{
					int iMaterial=pFaceSet->GetMaterial();
					if (iMaterial>=0 && iMaterial<static_cast<int>(ObjectData.GetNumMaterials()))
					{
						const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(iMaterial);
						if (pMaterial)
						{
							// try to find material in material list
							bool bFound=false;
							for (unsigned long lMat=0;lMat<vecMaterials.size();lMat++)
							{
								if (vecMaterials[lMat]==MakeCompatibleString(pMaterial->strName))
								{
									// already in list
									bFound=true;
									break;
								}
							}
							if (!bFound)
							{
								// add material to list
								vecMaterials.push_back(MakeCompatibleString(pMaterial->strName));
							}
						}
					}
				}
			}

			// write all materials used by this mesh
			if (vecMaterials.size()>0)
			{
				DWORD dwNumMaterials=(DWORD)vecMaterials.size();

				// write material list header
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," MeshMaterialList {\n %d;\n %d;\n",dwNumMaterials,dwFaces)<0)
					{
						// error
						MessageBox(NULL,"Error writing material list header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintName(f,"MeshMaterialList") ||
						!XPrintToken(f,TOKEN_OBRACE) ||
						!XPrintIntList(f,1,&dwNumMaterials) ||
						!XPrintIntList(f,1,&dwFaces) ||
						!XPrintToken(f,TOKEN_INTEGER_LIST) ||
						!XPrintDWORD(f,dwFaces))
					{
						// error
						MessageBox(NULL,"Error writing material list header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write materials for all faces
				for (unsigned long lFaceSet=0,lFaceCount=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
				{
					// get material index of face set
					const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
					if (pFaceSet)
					{
						int iMaterialIndex=pFaceSet->GetMaterial();
						if (iMaterialIndex>=0 && iMaterialIndex<static_cast<int>(ObjectData.GetNumMaterials()))
						{
							const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(iMaterialIndex);
							if (pMaterial)
							{
								for (unsigned long lMat=0;lMat<vecMaterials.size();lMat++)
								{
									if (vecMaterials[lMat]==MakeCompatibleString(pMaterial->strName))
									{
										iMaterialIndex=(int)lMat;
										break;
									}
								}
							}
						}
						else
						{
							// no valid material index
							iMaterialIndex=-1;
						}

						// write material for all faces
						for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++,lFaceCount++)
						{
							if (this->m_iFormat==0)
							{
								// textual
								if (fprintf(f," %d",iMaterialIndex)<0)
								{
									// error
									MessageBox(NULL,"Error writing face material to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}

								// write seperator
								std::string strSeperator=(lFaceCount<(dwFaces-1))?",":";;";
								if (fprintf(f,"%s\n",strSeperator.c_str())<0)
								{
									// error
									MessageBox(NULL,"Error writing material list delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}
							}
							else
							{
								// binary
								if (!XPrintDWORD(f,iMaterialIndex) )		// material index
								{
									// error
									MessageBox(NULL,"Error writing face material to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}
							}
						}
					}
				}

				// write names of materials used by this mesh
				for (unsigned long lMat=0;lMat<vecMaterials.size();lMat++)
				{
					if (this->m_iFormat==0)
					{
						// textual
						if (fprintf(f," {%s}\n",vecMaterials[lMat].c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing material list name to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
					else
					{
						// binary
						if (!XPrintToken(f,TOKEN_OBRACE) || 
							!XPrintName(f,(char*)vecMaterials[lMat].c_str()) || 
							!XPrintToken(f,TOKEN_CBRACE) )
						{
							// error
							MessageBox(NULL,"Error writing material name to x-file.","Error x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
				}

				// write material list footer
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f,"}\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing material list to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintToken(f,TOKEN_CBRACE))
					{
						// error
						MessageBox(NULL,"Error writing material list footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}


			// -----------------------------------------------------------------------
			// write optional normal data
			if (pMesh->HasVertexNormals())
			{
				// write mesh normal header
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," MeshNormals {\n %d;\n",dwVertices)<0)
					{
						// error
						MessageBox(NULL,"Error writing mesh normal header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintName(f,"MeshNormals") ||
						!XPrintToken(f,TOKEN_OBRACE) ||
						!XPrintIntList(f,1,&dwVertices) ||
						!XPrintToken(f,TOKEN_REALNUM_LIST) ||
						!XPrintDWORD(f,dwVertices*3))
					{
						// error
						MessageBox(NULL,"Error writing mesh normal header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write normals
				for (unsigned long lVert=0;lVert<dwVertices;lVert++)
				{
					// write normal data
					const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
					PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
					GlobalVectorTransform(vecNormal,1,iUpRotation,iFlipAxis);
					if (this->m_iFormat==0)
					{
						// textual
						if (fprintf(f," %f;%f;%f;",vecNormal.x,vecNormal.y,vecNormal.z)<0)
						{
							// error
							MessageBox(NULL,"Error writing vertex data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}

						// write seperator
						std::string strSeperator=(lVert<(dwVertices-1))?",":";";
						if (fprintf(f,"%s\n",strSeperator.c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing coordinate delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
					else
					{
						// binary
						if (!XPrintFloat(f,vecNormal.x) ||	// normal of vertex
							!XPrintFloat(f,vecNormal.y) ||	// normal of vertex
							!XPrintFloat(f,vecNormal.z) )	// normal of vertex
						{
							MessageBox(NULL,"Error writing vertex normal to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
				}

				// --------------------------------------------------------------------
				// write all faces again for the normals

				// write face list header
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f,"%d;\n",dwFaces)<0)
					{
						// error
						MessageBox(NULL,"Error writing face header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					DWORD dwFaceIndices=dwFaces*4;
					if (!XPrintIntList(f,1,&dwFaces) ||
						!XPrintToken(f,TOKEN_INTEGER_LIST) ||
						!XPrintDWORD(f,dwFaceIndices))
					{
						// error
						MessageBox(NULL,"Error writing face header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write faces
				for (unsigned long lFaceSet=0,lFaceCount=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
				{
					const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
					if (pFaceSet)
					{
						for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++,lFaceCount++)
						{
							const PNPTC_FACE& face=pFaceSet->GetFace(lFace);
							if (this->m_iFormat==0)
							{
								// textual
								if (fprintf(f," 3;%d;%d;%d;",face.dwVert[0],face.dwVert[1],face.dwVert[2])<0)
								{
									// error
									MessageBox(NULL,"Error writing triangle data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}

								// write seperator
								std::string strSeperator=(lFaceCount<(dwFaces-1))?",":";";
								if (fprintf(f,"%s\n",strSeperator.c_str())<0)
								{
									// error
									MessageBox(NULL,"Error writing triangle delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}
							}
							else
							{
								// binary
								if (!XPrintDWORD(f,3) ||				// number of vertices per face
									!XPrintDWORD(f,face.dwVert[0]) ||	// first vertex index
									!XPrintDWORD(f,face.dwVert[1]) ||	// second vertex index
									!XPrintDWORD(f,face.dwVert[2]) )	// third vertex index
								{
									// error
									MessageBox(NULL,"Error writing triangle data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									fclose(f);
									return false;
								}
							}
						}
					}
				}

				// write mesh normal footer
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f,"}\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing mesh normal to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintToken(f,TOKEN_CBRACE))
					{
						// error
						MessageBox(NULL,"Error writing mesh normal footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}

			// -----------------------------------------------------------------------
			// write optional color data
			if (pMesh->HasVertexColor())
			{
				// write mesh normal header
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," MeshVertexColors {\n %d;\n",dwVertices)<0)
					{
						// error
						MessageBox(NULL,"Error writing vertex color header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintName(f,"MeshVertexColors") ||
						!XPrintToken(f,TOKEN_OBRACE) ||
						!XPrintIntList(f,1,&dwVertices) )
					{
						// error
						MessageBox(NULL,"Error writing vertex color header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write vertex colors
				for (unsigned long lVert=0;lVert<dwVertices;lVert++)
				{
					const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);

					// write color data
					if (this->m_iFormat==0)
					{
						// textual
						if (fprintf(f," %d;%f;%f;%f;%f;",
							lVert,
							static_cast<float>(vert.colDiffuse.r)/255.f,
							static_cast<float>(vert.colDiffuse.g)/255.f,
							static_cast<float>(vert.colDiffuse.b)/255.f,
							static_cast<float>(vert.colDiffuse.a)/255.f)<0)
						{
							// error
							MessageBox(NULL,"Error writing vertex data to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}

						// write seperator
						std::string strSeperator=(lVert<(dwVertices-1))?",":";";
						if (fprintf(f,"%s\n",strSeperator.c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing coordinate delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
					else
					{
						// binary
						static float fColor[4];
						fColor[0]=static_cast<float>(vert.colDiffuse.r)/255.f;
						fColor[1]=static_cast<float>(vert.colDiffuse.g)/255.f;
						fColor[2]=static_cast<float>(vert.colDiffuse.b)/255.f;
						fColor[3]=static_cast<float>(vert.colDiffuse.a)/255.f;
						if (!XPrintIntList(f,1,&lVert) ||
							!XPrintFloatList(f,4,fColor) )
						{
							MessageBox(NULL,"Error writing vertex color to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
				}

				// write vertex colors footer
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f,"}\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing vertex color writing to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintToken(f,TOKEN_CBRACE))
					{
						// error
						MessageBox(NULL,"Error writing vertex color footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}


			// -----------------------------------------------------------------------
			// write UV coordinates
			if (pMesh->GetNumMappingCoords()>0)
			{
				// write UV coordinates header
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f," MeshTextureCoords {\n %d;\n",dwVertices)<0)
					{
						// error
						MessageBox(NULL,"Error writing UV coordinates header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintName(f,"MeshTextureCoords") ||
						!XPrintToken(f,TOKEN_OBRACE) ||
						!XPrintIntList(f,1,&dwVertices) ||
						!XPrintToken(f,TOKEN_REALNUM_LIST) ||
						!XPrintDWORD(f,dwVertices*2))
					{
						// error
						MessageBox(NULL,"Error writing UV coordinates header to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// write UV coordinates
				for (unsigned long lVert=0;lVert<dwVertices;lVert++)
				{
					const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);

					// write color data
					if (this->m_iFormat==0)
					{
						// textual
						if (fprintf(f," %f;%f;",
							vert.uv[0].u,
							vert.uv[0].v)<0)
						{
							// error
							MessageBox(NULL,"Error writing UV coordinate to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}

						// write seperator
						std::string strSeperator=(lVert<(dwVertices-1))?",":";";
						if (fprintf(f,"%s\n",strSeperator.c_str())<0)
						{
							// error
							MessageBox(NULL,"Error writing UV coordinate delimiter to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
					else
					{
						// binary
						if (!XPrintFloat(f,vert.uv[0].u) ||
							!XPrintFloat(f,vert.uv[0].v) )
						{
							MessageBox(NULL,"Error writing UV coordinate to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}
				}

				// write UV coordinate footer
				if (this->m_iFormat==0)
				{
					// textual
					if (fprintf(f,"}\n")<0)
					{
						// error
						MessageBox(NULL,"Error finishing UV coordinates writing to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
				else
				{
					// binary
					if (!XPrintToken(f,TOKEN_CBRACE))
					{
						// error
						MessageBox(NULL,"Error writing UV coordinates footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}
			}


			// -----------------------------------------------------------------------
			// finish mesh
			if (this->m_iFormat==0)
			{
				// textual
				if (fprintf(f,"}\n")<0)
				{
					// error
					MessageBox(NULL,"Error finishing mesh writing to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
			else
			{
				// binary
				if (!XPrintToken(f,TOKEN_CBRACE))
				{
					// error
					MessageBox(NULL,"Error writing mesh footer to x-file.","Error exporting x object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					fclose(f);
					return false;
				}
			}
		}
	}

	// ok
	fclose(f);
	return true;
}
