#ifndef IDXANIMATION_H
#define IDXANIMATION_H

namespace DX
{
	/**
		IDXAnimation is a collection of IDXGroupAnims, which form 
		a complete animation (such as run, walk, die). 
	*/
	struct  IDXAnimation : public IDXUserData, public IDXModelOwned
	{
		virtual				DXMCALL ~IDXAnimation() = 0;
		/**
			The ID of this Animation.	
		*/
		virtual const char* DXMCALL GetID() const = 0;
		/**
			The time period over which this animation will be played.
		*/
		virtual float		DXMCALL GetDuration() const = 0;
		/**
			Controls the interval at which keyframes can be added in the
			editor. This is only used in the editor.
		*/
		virtual float		DXMCALL GetSnap() const = 0;
		/**
			Specifies if this animation will play automatically when an 
			object using this mesh is created.
		*/
		virtual bool		DXMCALL GetAutoplay() const = 0;
		/**
			Specifies whether the animation will automatically loop when 
			it reaches the end.
		*/
		virtual bool		DXMCALL GetLoop() const = 0;
		/**
			The number of IDXGroupAnim that are contained in this IDXAnimation
		*/
		virtual uint		DXMCALL GetNumGroupAnims() const = 0;
		/**
			Get the number of keyframes that are in this IDXAnimation.
		*/
		virtual uint		DXMCALL GetNumKeyframes( ) const = 0;
		/**
			Get the IDXGroupAnim with the specified groupID.
			@param groupID The groupID to search for
			@param out A pointer to an IDXGroupAnimPtr where a copy will be saved.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if groupID was null.\n
					Error_NotFound if a IDXGroupAnimPtr with the specified groupID was not found.\n
					Error_Success if the groupID was found and a pointer was saved in out.\n
		*/
		virtual DXError		DXMCALL GetGroupAnim(const char* groupID, IDXGroupAnimPtr* out) const = 0;
		/**
			Get the IDXGroupAnim at the specified index.
			@param groupID The index of the IDXGroupAnim to retrieve.
			@param out A pointer to an IDXGroupAnimPtr where a copy will be saved.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was greater than the number of GroupAnims in this Animation.\n
					Error_Success if an IDXGroupAnimPtr was saved in out.\n
		*/
		virtual DXError		DXMCALL GetGroupAnim(uint index, IDXGroupAnimPtr* out) const = 0;
		/**
			Get all IDXGroupAnim in this IDXAnimation.
			@param out A user allocated array of IDXGroupAnimPtr which is at least GetNumGroupAnims
					   big.
		    @return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetGroupAnims( IDXGroupAnimArray out ) const = 0;
		/**
			Get a range of IDXGroupAnim.
			@param start The index (inclusive) of the first elements to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXGroupAnimPtr at least GetNumGroupAnims big.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetGroupAnims( uint start, uint count, IDXGroupAnimArray out ) const = 0;
		/**
			Get the index of the IDXGroupAnim with the specified groupID.
			@param groupID The id of the IDXGroupAnim to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if groupID was null.\n
					Error_NotFound if there was no IDXGroupAnim with the specified id.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetGroupAnimIndex( const char* groupID, uint* out ) const = 0;
		/**
			Get the index of the IDXGroupAnim with the specified groupID.
			@param groupAnim The IDXGroupAnim to search for
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if groupAnim was null.\n
					Error_NotFound if there was no matching IDXGroupAnim on the IDXAnimation.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetGroupAnimIndex( IDXGroupAnimPtr groupAnim, uint* out ) const = 0;
		/**
			Get the IDXAnimationKeyframe at the specified index.
			@param index The index of the keyframe to retrieve.
			@param out A pointer to an IDXAnimationKeyframePtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetKeyframe( uint index, IDXAnimationKeyframePtr* out ) const = 0;
		/**
			Get all IDXAnimationKeyframe.
			@param out A user allocated array of IDXAnimationKeyFramePtr of length at least GetNumKeyframes
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetKeyframes( IDXAnimationKeyframeArray out ) const = 0;
		/**
			Get all IDXAnimationKeyframe.
			@param start The inclusive index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXAnimationKeyFramePtr of length at least count
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetKeyframes( uint start, uint count, IDXAnimationKeyframeArray out ) const = 0;
		/**
			Get the index of a keyframe.
			@param frame The IDXAnimationKeyframe to get the index of.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if frame was null.\n
					Error_NotFound if the frame was not found on the animation.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetKeyframeIndex( IDXAnimationKeyframePtr frame, uint* out ) const = 0;
		/**
			Set the ID of this Animation.
			@param id The new ID to use.
			@return Error_NullParam if id was null.\n
					Error_InvalidParam if the id contained invalid characters.\n
					Error_Duplicate if another animation already exists in the model with the same name.\n
					Error_Success if the id was successfully updated.
		*/
		virtual DXError		DXMCALL SetID(const char* id) = 0;
		/**
			Set the period of time the animation will play over.
			@param duration The new length
		*/
		virtual void		DXMCALL SetDuration(float duration)= 0;
		/**
			Set the snap value, which is used in the editor with keyframing.
			@param snap The new snap interval
		*/
		virtual void		DXMCALL SetSnap(float snap) = 0;
		/**
			Set whether this should play automatically when this model is used.
		*/
		virtual void		DXMCALL SetAutoplay(bool value) = 0;
		/**
			Set whether this animation should loop when it reaches the end of its
			duration.
		*/
		virtual void		DXMCALL SetLoop(bool value) = 0;
		/**
			Create a new IDXGroupAnim. This will animate the specified groupID.
			@param groupID The ID of the group in the model to animate
			@param out A pointer to an IDXGroupAnimPtr where a copy will be saved. This may be null.
			@return Error_NullParam if groupID was null.\n
					Error_NotFound if the groupID specified could not be found in the model.\n
					Error_Duplicate if an IDXGroupAnim already exists in the animation for the specified groupID.\n
					Error_Success if the IDXGroupAnim was created and a copy saved in out.
		*/
		virtual DXError		DXMCALL AddGroupAnim(const char* groupID, IDXGroupAnimPtr* out = 0) = 0;
		/**
			Create a new IDXAnimationKeyframe at the specified time.
			@param time The time index to create the keyframe at.
			@param out A pointer to an IDXAnimationKeyframePtr where the result will be stored. Optional.
			@DX_SUCCESS If the keyframe was added.
		*/
		virtual DXError		DXMCALL AddKeyframe( float time, IDXAnimationKeyframePtr* out = 0) = 0;
		/**
			Remove a Group animation from this animation.
			@param groupID The group ID for the Group animation to remove.
			@return Error_NullParam if groupID was null.\n
					Error_NotFound if a groupAnim for the group specified by groupID was not found.\n
					Error_Success if the GroupAnim was removed.
		*/
		virtual DXError		DXMCALL RemoveGroupAnim(const char* groupID) = 0;
		/**
			Remove a Group animation from this animation.
			@param anim The IDXGroupAnim to remove from the animation.
			@return Error_NullParam if anim was null.\n
					Error_NotFound if the GroupAnim specified is not part of this animation.\n
					Error_Success if the GroupAnim was removed.
		*/
		virtual DXError		DXMCALL RemoveGroupAnim(IDXGroupAnimPtr anim) = 0;
		/**
			Remove a Group Animation from this animation.
			@param index The index of the group animation to remove from the animation.
			@return Error_OutOfRange if the index provided was not less than the number of GroupAnims in this Animation.\n
					Error_Success if the GroupAnim was removed.
		*/
		virtual DXError		DXMCALL RemoveGroupAnim(uint index) = 0;
		/**
			Remove a range of IDXGroupAnim from this animation.
			@param start The index of the first Group Animation to remove
			@param count The number of elements to remove. 
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveGroupAnim( uint start, uint count ) = 0;
		/**
			Remove a keyframe from the animation.
			@param frame The IDXAnimationKeyframe to remove.
			@return Error_NullParam if frame was null.\n
					Error_NotFound if the frame was not found in the animation.\n
					Error_Success if the frame was removed and destroyed.					
		*/
		virtual DXError		DXMCALL RemoveKeyframe( IDXAnimationKeyframePtr frame )= 0;
		/**
			Remove the keyframe at the specified index.
			@param index The index of the keyframe to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the frame was removed.
		*/
		virtual DXError		DXMCALL RemoveKeyframe( uint index ) = 0;
		/**
			Remove one or more keyframes from this animation. 
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the keyframes were removed.
		*/
		virtual DXError		DXMCALL RemoveKeyframe( uint start, uint count ) = 0;
		/**
			Removes all Group Animations from this Animation.
		*/
		virtual void		DXMCALL ClearGroupAnims() = 0;
		/**
			Removes all keyframes from this animation.
		*/
		virtual void		DXMCALL ClearKeyframes( ) = 0;
	};
}

#endif