#ifndef IDXFILE_H
#define IDXFILE_H

namespace DX
{
	enum DXFileType
	{
		File_Unknown = 0,	//A file of unknown type
		File_Int32,			//A file of ints stored in sequence
		File_Float32,		//A file of floats stored in sequence
		File_Resource,		//A generic byte file
		File_Link,			//A link to an external file - no data here. not used
		File_Reserved		//Reserved - do not use.
	};

	typedef byte* BytePtr;
	typedef BytePtr ByteArray;

	/**
		Base class for all files.
	*/
	struct IDXFile : public IDXUserData, public IDXFileSystemOwned
	{
		virtual DXMCALL ~IDXFile( ) = 0;
		/**
			Get the filename of this file.
			@return A wchar_t* which is the filename.
		*/
		virtual const wchar_t* DXMCALL GetFilename( ) const = 0;
		/**
			Get the type of this file.
			@return A member of the FileType enumeration.
		*/
		virtual DXFileType DXMCALL GetType( ) const = 0;
		/**
			Get the number of bytes in the file.
			@return The number of bytes.
		*/
		virtual uint DXMCALL GetLength( ) const = 0;
		/**
			Get all the data in this file.
			@param out A pointer to a user allocated array of bytes, equal
					   to GetLength.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( ByteArray out ) const = 0;
		/**
			Get the byte at the specified index.
			@param index The index of the byte to retrieve.
			@param out A pointer to a byte to receive the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint index, BytePtr out ) const = 0;
		/**
			Get a chunk of bytes.
			@param start The index of the first byte to retrieve.
			@param count The number of bytes to retrieve.
			@param out A user allocated array of bytes of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint start, uint count, ByteArray out ) const = 0;
		/**
			Set the filename that this file uses.
			@param filename A wchar_t* to use as the filename.
		*/
		virtual DXError DXMCALL SetFilename( const wchar_t* filename ) = 0;
		/**
			Remove any data in this file.
		*/
		virtual void DXMCALL Clear( ) = 0;
	};
}

#endif