#ifndef IDXFILEBINDING_H
#define IDXFILEBINDING_H

namespace DX
{
	/**
		Controls binding file outputs to interfaces.
	*/
	struct IDXFileBinding
	{
		/**
			FileID is used to identify the file binding to use. For additional 
			texture coordinates, use FileID_TextureCoordinates + index.
		*/
		enum FileID
		{
			FileID_TranslationKeys = 0,
			FileID_RotationKeys,
			FileID_ScaleKeys,
			FileID_SkinVerts,
			FileID_SkinWeights,
			FileID_SurfaceDetailXYZ,
			FileID_FaceMaterials,
			FileID_Faces,
			FileID_Vertices,
			FileID_Normals,
			FileID_TextureCoordinates
		};

		virtual DXMCALL ~IDXFileBinding( ) = 0;
		/**
			Get the number of files bound.
			@return The number of files.
		*/
		virtual uint GetNumFiles( ) const = 0;
		/**
			Get the file binding at the specified id.
			@param index The index of the file to retrieve.
			@param id A reference to a FileID where the file id type will be stored.
			@param out A Pointer to an IDXFilePtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetFile( uint index, FileID& id, IDXFilePtr* out ) const = 0;
		/**
			Get the binding with the specified id.
			@param id A member of the FileID enumeration.
			@param out A pointer to an IDXFilePtr where the result will be stored. 
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no binding entry with ID.\n
					Error_Success if the result was stored in out. 
		*/
		virtual DXError DXMCALL GetFile( FileID id, IDXFilePtr* out ) const = 0;
		/**
			Get the binding with the specified id.
			@param id A member of the FileID enumeration.
			@param out A pointer to an IDXFileFloat32Ptr where the result will be stored. 
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no binding entry with ID.\n
					Error_InvalidParam if the file was not of type IDXFileFloat32.\n
					Error_Success if the result was stored in out. 
		*/
		virtual DXError DXMCALL GetFile( FileID id, IDXFileFloat32Ptr* out ) const = 0;
		/**
			Get the binding with the specified id.
			@param id A member of the FileID enumeration.
			@param out A pointer to an IDXFileInt32Ptr where the result will be stored. 
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no binding entry with ID.\n
					Error_InvalidParam if the file was not of type IDXFileInt32.\n
					Error_Success if the result was stored in out. 
		*/
		virtual DXError DXMCALL GetFile( FileID id, IDXFileInt32Ptr* out ) const = 0;
		/**
			Get the binding with the specified id.
			@param id A member of the FileID enumeration.
			@param out A pointer to an IDXFileResourcePtr where the result will be stored. 
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no binding entry with ID.\n
					Error_InvalidParam if the file was not of type IDXFileResource.\n
					Error_Success if the result was stored in out. 
		*/
		virtual DXError DXMCALL GetFile( FileID id, IDXFileResourcePtr* out ) const = 0;
		/**
			Set the binding of the specfied id.
			@param id A member of the FileID enumeration.
			@param file The file to bind to.
			@return Error_NullParam if file was null.\n
					Error_Success if the binding was updated.
		*/
		virtual DXError DXMCALL SetFile( FileID id, IDXFilePtr file ) = 0;
		/**
			Remove the binding of the specified id.
			@param id A member of the FileID enumeration.
			@return Error_NotFound if there was no binding with this id.\n
					Error_Success if the binding was removed.
		*/
		virtual DXError DXMCALL RemoveBinding( FileID id ) = 0;
		/**
			Clear all bindings.
		*/
		virtual void DXMCALL ClearBindings( ) = 0;
	};
}

#endif