#ifndef IDXFILEFLOAT32_H
#define IDXFILEFLOAT32_H

namespace DX
{
	typedef float* FloatPtr;
	typedef FloatPtr FloatArray;
	/**
		File for storing floats.
	*/
	struct IDXFileFloat32 : public IDXFile
	{
		virtual DXMCALL ~IDXFileFloat32( ) = 0;
		/**
			Get the number of floats in this file.
		*/
		virtual uint DXMCALL GetCount( ) const = 0;
		/**
			Get the float at the specified index.
			@param index The index of the float to retrieve.
			@param out A pointer to a float where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint index, FloatPtr out ) const = 0;
		/**
			Get all the floats.
			@param out A user allocated array of floats. Must be
					   of length greater than or equal to GetCount.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( FloatArray out ) const = 0;
		/**
			Get a chunk of floats.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of floats of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint start, uint count, FloatArray out ) const = 0;
		/**
			Set the float at the specified index.
			@param index The index of the float to set.
			@param value The value to set the float to.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the float was updated.
		*/
		virtual DXError DXMCALL Set( uint index, float value ) = 0;
		/**
			Replace all the values in the array with the values in the float array
			passed in.
			@param values An array of floats.
			@param length The number of floats to copy from the array.
			@return Error_NullParam if values was null.\n
					Error_Success if the values were updated.
		*/
		virtual DXError DXMCALL Set( FloatArray values, uint length ) = 0;
		/**
			Add the value to the end of the data.
			@param value The float to add.
			@DX_SUCCESS always.
		*/
		virtual DXError DXMCALL Add( float value ) = 0;
		/**
			Add the data in the float array specified to the end of the data.
			@param values A float array to copy data from.
			@param length A uint which is the number of floats to copy from the
						  array passed in.
			@return Error_NullParam if values was null.\n
					Error_Success if the values were appended,
		*/
		virtual DXError DXMCALL Add( FloatArray values, uint length ) = 0;
		/**
			Add the data in the float array specified to any point in the data.
			@param value A float array to copy data from.
			@param length The amount of data to copy.
			@param start The point to start adding data.
			@return Error_NullParam if value was null.\n
					Error_OutOfRange if start was greater than GetCount.\n
					Error_Success if the data was added.					
		*/
		virtual DXError DXMCALL Add( FloatArray value, uint length, uint start ) = 0;
		/**
			Remove the element at the specified index.
			@param index The index of the element to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint index ) = 0;
		/**
			Remove a range of elements from the file.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint start, uint count ) = 0;
	};
}

#endif