#ifndef IDXFILESYSTEM_H
#define IDXFILESYSTEM_H

/*
	Experimental file interfaces. Intended to provide access to the files
	inside the zip.

	IDXFileSystem : Controls access to all files included in the model.
*/

namespace DX
{
	/**
		Controls access to all files included in the model.
	*/
	struct IDXFileSystem : public IDXUserData, public IDXModelOwned
	{
		virtual DXMCALL ~IDXFileSystem( ) = 0;
		/**
			Get the total number of files stored in the filesystem.
			@return The number of files.
		*/
		virtual uint DXMCALL GetNumFiles( ) const = 0;
		/**
			Get the file at the specified index.
			@param idOrIndex The index to use
			@param out A pointer to an IDXFilePtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if the index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetFile( uint index, IDXFilePtr* out ) const = 0;
		/**
			Get the file with the specified name.
			@param name The filename of the file to retrieve.
			@param out A pointer to an IDXFilePtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if name was null.\n
					Error_NotFound if there was no file with that name.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFile( const wchar_t* name, IDXFilePtr* out ) const = 0;
		/**
			Get all files in the filesystem.
			@param out A user allocated array of IDXFilePtr of length at least equal to GetNumFiles.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFiles( IDXFileArray out ) const = 0;
		/**
			Get a chunk of files in the filesystem.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXFilePtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFiles( uint start, uint count, IDXFileArray out ) const = 0;
		/**
			Get the index of the specified file.
			@param file The file to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if file was null.\n
					Error_NotFound if the file was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFileIndex( IDXFilePtr file, uint* out ) const = 0;
		/**
			Get the index of the file with the specified name.
			@param name The name of the file to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if name was null.\n
					Error_NotFound if the file was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFileIndex( const wchar_t* name, uint* out ) const = 0;
		/**
			Add a new int32 file to the file system using data passed in.
			@param data An int array which is the data to load.
			@param length The number of ints to load.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileInt32Ptr where a handle will be saved.
			@return Error_NullParam if data was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddInt32File( int* data, uint length, const wchar_t* destinationName, IDXFileInt32Ptr* out = 0 ) = 0;
		/**
			Add a new empty int32 file to the file system.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileInt32Ptr where a handle will be saved.
			@return Error_NullParam if destinationName was null.\n
					Error_Duplicate if a file with the destinationName already exists in the zip.\n
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddInt32File( const wchar_t* destinationName, IDXFileInt32Ptr* out = 0 ) = 0;
		/**
			Add a new Int32 file from the file passed in.
			@param source The source file (on disk) that the resouce will be taken from.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileInt32Ptr where a handle will be saved.
			@return Error_NullParam if source was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_NotFound if the source file was not found on disk.\n
					Error_InvalidParam if the source was not of the correct size.
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddInt32File( const wchar_t* source, const wchar_t* destinationName, IDXFileInt32Ptr* out = 0 ) = 0; 
		/**
			Add a new Float32 file to the file system using data passed in.
			@param data A float array which is the data to load.
			@param length The number of ints to load.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileFloat32Ptr where a handle will be saved.
			@return Error_NullParam if data was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddFloat32File( float* data, uint length, const wchar_t* destinationName, IDXFileFloat32Ptr* out = 0 ) = 0;
		/**
			Add a new empty Float32 file to the file system.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileFloat32Ptr where a handle will be saved.
			@return Error_NullParam if destinationName was null.\n
					Error_Duplicate if a file with the destinationName already exists in the zip.\n
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddFloat32File( const wchar_t* destinationName, IDXFileFloat32Ptr* out = 0 ) = 0;
		/**
			Add a new Float32 file from the file passed in.
			@param source The source file (on disk) that the resouce will be taken from.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileFloat32Ptr where a handle will be saved.
			@return Error_NullParam if source was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_NotFound if the source file was not found on disk.\n
					Error_Success if the file was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddFloat32File( const wchar_t* source, const wchar_t* destinationName, IDXFileFloat32Ptr* out = 0 ) = 0; 
		/**
			Add a new resource file to the file system.
			@param source The source file (on disk) that the resource will be taken from.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileResource where a handle will be saved.
			@return Error_NullParam if source was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_NotFound if the source file was not found on disk.\n
					Error_Success if the resource was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddResourceFile( const wchar_t* source, const wchar_t* destinationName, IDXFileResourcePtr* out = 0 ) = 0;
		/**
			Add a new empty resource file to the file system.
			@param destinationName The name for the file in the zip.
			@param out An optional pointer to an IDXFileResourcePtr where a handle will be saved.
			@return Error_NullParam if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_Success if the resource was created and the result stored in out.
		*/
		virtual DXError DXMCALL AddResourceFile( const wchar_t* destinationName, IDXFileResourcePtr* out = 0 ) = 0;
		/**
			Add a new resource file filled with the specified data.
			@param data A byte array with the data to use.
			@param length The number of bytes in data.
			@param destinationName The name to use for the file in the zip.
			@param out An optional pointer to an IDXFileResource where a handle will be saved.
			@return Error_NullParam if data was null.\n
					Error_NullParam2 if destinationName was null.\n
					Error_Duplicate if a file with the destination name already exists.\n
					Error_Success if the resource was created and the result was stored in out.
		*/
		virtual DXError DXMCALL AddResourceFile( ByteArray data, uint length, const wchar_t* destinationName, IDXFileResourcePtr* out = 0 ) = 0;
		/**
			Remove the speicified file from the file system.
			@param file A file to remove.
			@return Error_NullParam if file was null.\n
					Error_NotFound if the file was not found in the system.\n
					Error_Success if the file was removed
		*/
		virtual DXError DXMCALL RemoveFile( IDXFilePtr file ) = 0;
		/**
			Remove the file at the specified index from the filesystem.
			@param index The index of the file to remove.
			@return Error_OutOfRange if the index was out of range.\n
					Error_Success if the file was removed.
		*/
		virtual DXError DXMCALL RemoveFile( uint index ) = 0;
		/**
			Remove a range of files from the filesystem.
			@param start The index of the first file to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError DXMCALL RemoveFile( uint start, uint count ) = 0;
		/**
			Remove the file with the specified name from the filesystem.
			@param filename The name of the file to remove.
			@return Error_NullParam if filename was null.\n
					Error_NotFound if there was no matching file.\n
					Error_Success if the file was removed.
		*/
		virtual DXError DXMCALL RemoveFile( const wchar_t* filename ) = 0;
		/**
			Remove all files from the file system.
		*/
		virtual void DXMCALL ClearFiles( ) = 0;
	};
}

#endif