#ifndef IDXGROUP_H
#define IDXGROUP_H

namespace DX
{
	/**
		An IDXGroup represents a node in the model heirarchy. It can have a mesh, and
		a number of children.
	*/
	struct  IDXGroup : public IDXUserData, public IDXModelOwned
	{
		virtual					DXMCALL ~IDXGroup() = 0;
		/**
			Get the IDXGroup that is one level above in the tree.
			@return If the group has a parent, an IDXGroupPtr. If this is a
					top level group, returns 0.
		*/
		virtual IDXGroupPtr		DXMCALL GetParent() const = 0;
		/**
			The current ID that this group will be identified by in 
			script and in the API.
			@return The current ID for this group.
		*/
		virtual const char*		DXMCALL GetID() const = 0;
		/**
			Get the current IDXMesh for this Group.
			@param out A pointer to an IDXMeshPtr where the result will be stored.
			@return Error_NullParm if out was null.\n
					Error_NotFound if there was no mesh present.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetMesh( IDXMeshPtr* out ) const = 0;
		/**
			The current relative transform for this group.
			@return A DXMatrix for the transform.
		*/
		virtual const DXMatrix	DXMCALL GetTransform() const = 0;
		/**
			@return The number of child groups.
		*/
		virtual uint			DXMCALL GetNumGroups() const = 0;
		/**
			Get the child group at the specified index.
			@param index The index of the child group to retrieve.
			@param out A pointer to an IDXGroupPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetGroup( uint index, IDXGroupPtr* out ) const = 0;
		/**
			Get the first group with the specified id. 
			@param id The id of the group to find.
			@param out A pointer to an IDXGroupPtr where the result will be stored.
			@param recurse If true, this will scan all child groups as well.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no group found with the specified id.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetGroup( const char* id, IDXGroupPtr* out, bool recurse = false ) const = 0;
		/**
			Get any groups with the specified id.
			@param id The id to find.
			@param out A pointer to an IDXGetGroupResultPtr which will store the results.
			@param recurse If true, thiswill scan all child groups.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no group found with the specfied id.\n
					Error_Success if a number of groups were stored in out.
		*/
		virtual DXError			DXMCALL GetGroup( const char* id, IDXGetGroupResultPtr* out, bool recurse = false ) = 0;
		/**
			Get the groups which are direct children of this group.
			@param out A user allocated array of IDXGroupPtr of length at least GetNumGroups.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetGroups( IDXGroupArray out ) const = 0;
		/**
			Get a block of groups which are direct children of this group.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXGroupPtr of at least count where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetGroups( uint start, uint count, IDXGroupArray out ) const = 0;
		/**
			Get the index of the group with the specified id.
			@param id The id of the group to find.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no group with the specified id.\n
					Error_Success if the group was found and the result stored in out.
		*/
		virtual DXError			DXMCALL GetGroupIndex( const char* id, uint* out ) const = 0;
		/**
			Get the index of the specified group.
			@param group The group to find.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the group was not found.\n
					Error_Success if the group was found and the result stored in out.
		*/
		virtual DXError			DXMCALL GetGroupIndex( IDXGroupPtr group, uint* out ) const = 0;
		/**
			Get if this group is visible.
			@return True if visible.
		*/
		virtual bool			DXMCALL GetVisible( ) const = 0;
		/**
			Set the ID for this IDXGroup. This will update all references if successful.
			@param id The new ID.
			@return Error_NullParam if id was null.
					Error_InvalidParam if the id contained invalid characters
					Error_Success if the id was changed.
		*/
		virtual DXError			DXMCALL SetID(const char* id) = 0;
		/**
			The relative transform for this IDXGroup relative to the parent.
			@param matrix A DXMatrix for this transform.
			@see CreateMatrix
		*/
		virtual void			DXMCALL SetTransform(const DXMatrix& matrix) = 0;
		/**
			Set if this group should be visible. If false, will not be renderered. This will also
			hide subgroups.
		*/
		virtual void			DXMCALL SetVisible( bool value ) = 0;
		/**
			Create a new subgroup on this group.
			@param id The id that will be used to refer to this group in the API and in 
					  javascript inside DXStudio.
			@param out An optional pointer to an IDXGroupPtr where a pointer to the new IDXGroupPtr will 
					   be saved.
		    @return Error_NullParam if id was null.\n
					Error_InvalidID if id contained invalid characters.\n
					Error_Success if the group was added successfully.
			@see IDXGroup
		*/
		virtual DXError			DXMCALL CreateGroup(const char* id, IDXGroupPtr* out = 0) = 0;
		/**
			Remove the group with the specified id from the group.
			@param id The id of the group to remove.
			@param recurse Set to true if you want to search child groups for the group.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no group with the specified id.\n
					Error_Success if the group was removed.
		*/
		virtual DXError			DXMCALL RemoveGroup( const char* id, bool recurse = false ) = 0;
		/**
			Removes the specfied group from the group.
			@param group The group to remove.
			@param recurse Set to true if you want to search child groups for this group.
			@return Error_NullParam if group was null.\n
					Error_NotFound if the group was not found.\n
					Error_Success if the group was removed.
		*/
		virtual DXError			DXMCALL RemoveGroup( IDXGroupPtr group, bool recurse = false ) = 0;
		/**
			Remove the group at the specified index from the group.
			@param index The index of the group to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the group was removed.
		*/
		virtual DXError			DXMCALL RemoveGroup( uint index ) = 0;
		/**
			Remove a range of groups from the group.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError			DXMCALL RemoveGroup( uint start, uint count ) = 0;
		/**
			Create a mesh on this IDXGroup. Any existing mesh will be deleted, invalidating all
			data created by that mesh.
			@param out A pointer to an IDXMeshPtr where a copy will be saved.	This may be null.
			@return Error_Duplicate if an IDXMeshPtr already exists on this group.\n
					Error_Success if the IDXMeshPtr was created.
		*/
		virtual DXError			DXMCALL CreateMesh(IDXMeshPtr* out = 0) = 0;
		/**
			Destroy the mesh that is present on this group.
			@return Error_NotFound if there was no mesh to destroy.\n
					Error_Success if the mesh was destroyed.
		*/
		virtual DXError			DXMCALL DestroyMesh( ) = 0;
		/**
			Optimise for a faster render.
			@param recurse Should this optimize all subgroups too. Defaults to true.
		*/
		virtual void			DXMCALL Optimize( bool recurse = true ) = 0;
	};
}

#endif