#ifndef IDXGROUPANIM_H
#define IDXGROUPANIM_H

namespace DX
{
	/**
		A group anim controls animations (translation, rotation, scale) for a specific
		group in the model.
	*/
	struct  IDXGroupAnim : public IDXUserData, public IDXModelOwned, public IDXAnimationOwned, public IDXHasFileBinding
	{
		virtual				DXMCALL ~IDXGroupAnim() = 0;
		/**
			Get the group that this is animation.
			@return The ID of the group that this is animating.
		*/
		virtual const char* DXMCALL GetGroupID() const = 0;
		/**
			The number of translation keys there are in the IDXGroupAnim. If any of the remove
			or add methods are called, this number is changed.
			@return The number of translation keys.
		*/
		virtual uint		DXMCALL GetNumTranslationKeys() const = 0;
		/**
			The number of rotation keys there are in the IDXGroupAnim. If any of the remove
			or add methods are called, this number is changed.
			@return The number of rotation keys.
		*/
		virtual uint		DXMCALL GetNumRotationKeys() const = 0;
		/**
			The number of scale keys there are in the IDXGroupAnim. If any of the remove
			or add methods are called, this number is changed.
			@return The number of scale keys.
		*/
		virtual uint		DXMCALL GetNumScaleKeys() const = 0;
		/**
			Get the translation key at the specified index.
			@param index The index of the key to retrieve.
			@param out A pointer to an IDXTranslationKeyPtr which will receive the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetTranslationKey(uint index, IDXTranslationKeyPtr* out) const = 0;
		/**
			Get a block of IDXTranslationKeyPtr.
			@param start The index of the first element to retrieve.
			@param count The number of elemnents to retreive.
			@param out A user allocated array of IDXTranslationKeyPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out,
		*/
		virtual DXError		DXMCALL GetTranslationKeys( uint start, uint count, IDXTranslationKeyArray out ) const = 0;
		/**
			Fill a user allocated array of IDXTranslationKeyPtr with data. Array must be of length
			equal to GetNumTranslationKeys.
			@param out A pointer to an array of IDXTranslationKeyPtr.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetTranslationKeys( IDXTranslationKeyArray out ) const = 0;
		/**
			Get the rotation key at the specified index.
			@param index The index of the key to retrieve.
			@param out A pointer to an IDXRotationKeyPtr which will receive the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetRotationKey(uint index, IDXRotationKeyPtr* out) const  = 0;
		/**
			Get a block of IDXRotationKeyPtr.
			@param start The index of the first element to retrieve.
			@param count The number of elemnents to retreive.
			@param out A user allocated array of IDXRotationKeyPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out,
		*/
		virtual DXError		DXMCALL GetRotationKeys( uint start, uint count, IDXRotationKeyArray out ) const = 0;
		/**
			Fill a user allocated array of IDXRotationKeyPtr with data. Array must be of length
			equal to GetNumRotationKeys.
			@param out A pointer to an array of IDXRotationKeyPtr.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetRotationKeys( IDXRotationKeyArray out ) const = 0;
		/**
			Get the scale key at the specified index.
			@param index The index of the key to retrieve.
			@param out A pointer to an IDXScaleKeyPtr which will receive the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetScaleKey(uint index, IDXScaleKeyPtr* out) const = 0;
		/**
			Get a block of IDXScaleKeyPtr.
			@param start The index of the first element to retrieve.
			@param count The number of elemnents to retreive.
			@param out A user allocated array of IDXScaleKeyPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out,
		*/
		virtual DXError		DXMCALL GetScaleKeys( uint start, uint count, IDXScaleKeyArray out ) const = 0;
		/**
			Fill a user allocated array of IDXScaleKeyPtr with data. Array must be of length
			equal to GetNumScaleKeys.
			@param out A pointer to an array of IDXScaleKeyPtr.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetScaleKeys( IDXScaleKeyArray out ) const = 0;
		/**
			Get the index of the specified IDXTranslationKey.
			@param key An IDXTranslationKeyPtr to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if key was null.\n
					Error_NotFound if key was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetTranslationKeyIndex( IDXTranslationKeyPtr key, uint* out ) const = 0;
		/**
			Get the index of the specified IDXRotationKey.
			@param key An IDXRotationKeyPtr to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if key was null.\n
					Error_NotFound if key was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetRotationKeyIndex( IDXRotationKeyPtr key, uint* out ) const = 0;
		/**
			Get the index of the specified IDXScaleKey.
			@param key An IDXScaleKeyPtr to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if key was null.\n
					Error_NotFound if key was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetScaleKeyIndex( IDXScaleKeyPtr key, uint* out ) const = 0;
		/**
			Set the group that this groupanim is animating.
			@param id The id of the group to use.
			@return Error_NullParam if id was null.\n
					Error_Duplicate if there was already a groupanim for the specified group id.\n
					Error_NotFound if there was no group found with the specified id.\n
					Error_Success if the group id was changed.
		*/
		virtual DXError		DXMCALL SetGroupID( const char* id ) = 0;
		/**
			Add a translation key to the group animation.
			@param key The key to add. Can be created with CreateTranslationKey.
			@return Error_Success if the key was added to the timeline.
		*/
		virtual DXError		DXMCALL AddTranslationKey( float time, const DXVector& translation ) = 0;
		/**
			Add a rotation key to the group animation.
			@param key The key to add. Can be created with CreateRotationKey.
			@return Error_Sucess if the key was added to the timeline.
		*/
		virtual DXError		DXMCALL AddRotationKey( float time, const DXRotation& rotation ) = 0; 
		/**
			Add a scale key to the group animation.
			@param key The key to add. Can be created with CreateScaleKey.
			@return Error_Sucess if the key was added to the timeline.
		*/
		virtual DXError		DXMCALL AddScaleKey( float time, const DXVector& scale ) = 0;
		/**
			Remove a translation key from the translation keys.
			@param index The index of the key to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the key was removed.
		*/
		virtual DXError		DXMCALL RemoveTranslationKey(uint index) = 0;
		/**
			Remove the specified IDXTranslationKey from the IDXGroupAnim.
			@param key The IDXTranslationKeyPtr to remove.
			@return Error_NullParam if key was null.\n
					Error_NotFound if key was not found on the IDXGroupAnim.\n
					Error_Success if the key was removed.
		*/
		virtual DXError		DXMCALL RemoveTranslationKey( IDXTranslationKeyPtr key ) = 0;
		/**
			Remove a range of translation keys from translation keys
			@param start The start index of the range to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveTranslationKey( uint start, uint count ) = 0;
		/**
			Remove a rotation key from the rotation keys.
			@param index The index of the key to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the key was removed.
		*/
		virtual DXError		DXMCALL RemoveRotationKey(uint index) = 0;
		/**
			Remove the specified IDXRotationKey from the IDXGroupAnim.
			@param key The IDXRotationKeyPtr to remove.
			@return Error_NullParam if key was null.\n
					Error_NotFound if key was not found on the IDXGroupAnim.\n
					Error_Success if the key was removed.
		*/
		virtual DXError		DXMCALL RemoveRotationKey( IDXRotationKeyPtr key ) = 0;
		/**
			Remove a range of rotation keys from rotation keys
			@param start The start index of the range to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveRotationKey(uint start, uint count) = 0;
		/**
			Remove a scale key from the scale keys.
			@param index The location in the scale keys array to remove.
			@return Error_OutOfRange if index was not less the number of scale keys.\n
					Error_Success if the scale key was removed successfully.
		*/
		virtual DXError		DXMCALL RemoveScaleKey(uint index) = 0;
		/**
			Remove the specified IDXScaleKey from the IDXGroupAnim.
			@param key The IDXScaleKeyPtr to remove.
			@return Error_NullParam if key was null.\n
					Error_NotFound if key was not found on the IDXGroupAnim.\n
					Error_Success if the key was removed.
		*/
		virtual DXError		DXMCALL RemoveScaleKey( IDXScaleKeyPtr key ) = 0;
		/**
			Remove a range of scale keys from scale keys
			@param start The start index of the range to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveScaleKey(uint start, uint count) = 0;
		/**
			Force a sort of the translation keys.
		*/
		virtual void		DXMCALL SortTranslationKeys( ) = 0;
		/**
			Force a sort of the rotation keys.
		*/
		virtual void		DXMCALL SortRotationKeys( ) = 0;
		/**
			Force a sort of the scale keys.
		*/
		virtual void		DXMCALL SortScaleKeys( ) = 0;
	};
}

#endif