#ifndef IDXLIGHTMAP_H
#define IDXLIGHTMAP_H

namespace DX
{
	/**
		Controls properties specific to lightmaps.
	*/
	struct  IDXLightMap : public IDXMaterialMap
	{
		virtual				DXMCALL ~IDXLightMap() = 0;
		/**
			Get the size of the texture that will be used in light baking.
		*/
		virtual uint		DXMCALL GetGenerateSize( ) const = 0;
		/**
			@return The number of lights that this lightmap will be used for,
		*/
		virtual uint		DXMCALL GetNumLights( ) const = 0;
		/**
			Get the name of the light at the specified index in the list of lights that
			this lightmap will be used for.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetLight( uint index, const char** out ) const = 0;
		/**
			Get the index of the specified light id.
			@param id The id to search for
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the id was not found on this map.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetLightIndex( const char* id, uint* out ) const = 0;
		/**
			Set the size of the texture that will be used in light
			baking. Must be power of 2 (or 0 )
			@param value The length of the side of the texture to be created.
			@return Error_InvalidParam if value was not a power of 2, or was not 0.\n
					Error_Success if the generate size was changed.
		*/
		virtual DXError		DXMCALL SetGenerateSize( uint value ) = 0;
		/**
			Add a light to the list of lights that this lightmap is used
			for.
			@param id The id of the light.
			@return Error_NullParam if id was null.\n
					Error_Success if the id was added to the list.
		*/
		virtual DXError		DXMCALL AddLight( const char* id ) = 0;
		/**
			Remove a light from the list of lights that this lightmap
			will be used for.
			@param index The index of the light to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the light at the index was removed.
		*/
		virtual DXError		DXMCALL RemoveLight( uint index ) = 0;
		/**
			Remove a light from the list of lights that this lightmap is 
			used for which has the specified id.
			@param id The id of the light to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there are no lights that match the specified id.\n
					Error_Success if the light was removed.
		*/
		virtual DXError		DXMCALL RemoveLight( const char* id ) = 0;
		/**
			Remove a range of lights from the list of lights that this lightmap 
			will be used for.
			@param start The index of the first element to remove
			@param count The number of elements to remove
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveLight( uint start, uint count ) = 0;
		/**
			Remove all the lights from the list of lights that this lightmap will
			be used for.
		*/
		virtual void		DXMCALL ClearLights( ) = 0;
	};
}

#endif