#ifndef IDXMATERIAL_H
#define IDXMATERIAL_H

namespace DX
{
	/**
		This controls the reflection properties of this material.\n
		None : This material will not reflect anything and will be rendered normally.\n
		Environment : This material will reflect the current environment map of the scene.\n
		Cubic : This materal will reflect the other objects in the scene in realtime.\n
		Reserved : This is used internally, and should not be specified.
	*/
	enum DXReflectionType
	{
		Reflection_None = 0,
		Reflection_Environment,
		Reflection_Cubic,
		Reflection_Reserved
	};

	enum DXStandardMapType
	{
		StandardMap_Diffuse = 0,
		StandardMap_Emissive,
		StandardMap_Opacity,
		StandardMap_Bump,
		StandardMap_Normal,
		StandardMap_Specular,
		StandardMap_Reserved
	};

	/**
		An IDXMaterial controls how DXStudio will render a face when it has the material applied to it.
	*/
	struct  IDXMaterial : public IDXUserData, public IDXModelOwned
	{
		virtual						DXMCALL ~IDXMaterial() = 0;
		/**
			Retrieve the ID of the IDXMaterial.
		*/
		virtual const char*			DXMCALL GetID() const = 0;
		/**
			Retrieve the color that will be used with the diffuse component of lighting.
		*/
		virtual const DXColor		DXMCALL GetDiffuse() const = 0;
		/**
			Retrieve the color that will be used with the ambient component of lighting.
		*/
		virtual const DXColor		DXMCALL GetAmbient() const = 0;
		/**
			The specular color that will be used on this material. This is seperate from the actual
			sharpness value.
		*/
		virtual const DXColor		DXMCALL GetSpecular() const = 0;
		/**
			The emissive color that will be used when rendering this material.
		*/
		virtual const DXColor		DXMCALL GetEmissive() const = 0;
		/**
			How opaque this material is, with 0 being fully transparent, and 1 being completely opaque
		*/
		virtual float				DXMCALL GetOpacity() const = 0;
		/**
			How sharp the specular highlights will be on this model. A higher value focuses
			the specular more.
		*/
		virtual float				DXMCALL GetSpecularSharpness() const = 0;
		/**
			The type of reflection that will be used on this material when it is rendered.
			@see DXReflectionType
		*/
		virtual DXReflectionType	DXMCALL GetReflectionType() const = 0;
		/**
			Get the opacity value that is used for reflections.
		*/
		virtual float				DXMCALL GetEnvironmentOpacity( ) const = 0;
		/**
			Get the gradient value that is used for reflections.
		*/
		virtual float				DXMCALL GetEnvironmentGradient( ) const = 0;
		/**
			If set to true, the material will not allow UV values below 0 or above 1, and they
			will be clamped appropriately.
		*/
		virtual bool				DXMCALL GetClampUV() const = 0;
		/**
			Set the ID that this material will be referred to by in script.
			@param id The new ID to use for this material
			@return Error_NullParam if id was null.\n
					Error_InvalidID if id contained invalid characters.\n
					Error_Duplicate if a material already exists with the new ID .\n
					Error_Success if the material ID was changed.\n
		*/
		virtual DXError				DXMCALL SetID(const char* id) = 0;
		/**
			Set the diffuse color of this material.
			@param color A DXColor that represents the new color.
		*/
		virtual void				DXMCALL SetDiffuse(const DXColor& color) = 0;
		/**
			Set the ambient color of this material.
			@param color A DXColor that represents the new color.
		*/
		virtual void				DXMCALL SetAmbient(const DXColor& color) = 0;
		/**
			Set the specular color of this material.
			@param color A DXColor that represents the new color.
		*/
		virtual void				DXMCALL SetSpecular(const DXColor& color) = 0;
		/**
			Set the emissive color of this material.
			@param color A DXColor that represents the new color.
		*/
		virtual void				DXMCALL SetEmissive(const DXColor& color) = 0;
		/**
			Set the opacity of the material.
			@param opactiy A floating point value between 0 and 1 that represents the new opacity.
		*/
		virtual void				DXMCALL SetOpacity(float opacity) = 0;
		/**
			Set the specular sharpness of the materil. This affects how the specular
			is applied to the material.
			@param sharpness The new sharpness of the material.
		*/
		virtual void				DXMCALL SetSpecularSharpness(float sharpness) = 0;
		/**
			Set the kind of reflection this material will use.
			@param type A member of the DXReflectionType enumeration describing the new reflection type.
			@return Error_InvalidParam if an invalid value is used (< 0, >= Reserved).\n
					Error_Success if the reflection type was updated.
			@see DXReflectionType
		*/
		virtual DXError				DXMCALL SetReflectionType(DXReflectionType type) = 0;
		/**
			Set the opacity value for reflections.
			@param value A value between 0 and 100.
		*/
		virtual void				DXMCALL SetEnvironmentOpacity( float value ) = 0;
		/**
			Set the gradient value for reflections.
			@param value A value between 0 and 100.
		*/
		virtual void				DXMCALL SetEnvironmentGradient( float value ) = 0;
		/**
			Sets whether this material will clamp uv values outside the range 0 to 1.
			True to clamp, false to not clamp.
			@param value The new value to use.
		*/
		virtual void				DXMCALL SetClampUV(bool value) = 0;
		/**
			Create surface detail for this material. This will overwrite any exisiting surface detail on this
			material.
			@param billboardSplit The number of splits on the billboard texture. This must be a power of 2.
			@param billboardSway The sway factor that will be applied.
			@param billboardSize The size of the billboards (base size)
			@param out An optional parameter where a pointer to the new IDXSurfaceDetail will be stored.
			@return Error_InvalidParam if billboardSplit was not a power of 2.\n
					Error_Success if the surface detail was created.
		*/
		virtual DXError				DXMCALL CreateSurfaceDetail( uint billboardSplit, float billboardSway, float billboardSize, IDXSurfaceDetailPtr* out = 0) = 0;
		/**
			Get the IDXSurfaceDetailPtr for this material.
			@param out A pointer to an IDXSurfaceDetailPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no IDXSurfaceDetailPtr on this material.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetSurfaceDetail( IDXSurfaceDetailPtr* out ) const = 0;
		/**
			Remove any surface detail that has been added to this material.
			@return Error_NotFound if there was not an IDXSurfaceDetailPtr present.\n
					Error_Success if the IDXSurfaceDetailPtr was destroyed.
		*/
		virtual DXError				DXMCALL DestroySurfaceDetail( ) = 0;
		/**
			Get one of the standard maps for this material.
			@param type A member of the DXStandardMapType enumeration which identifies the map type
						to remove.
			@param out A pointer to an IDXMaterialMapPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if type was invalid.\n
					Error_NotFound if the specified standard map type was not present.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetStandardMap( DXStandardMapType type, IDXMaterialMapPtr* out ) = 0;
		/**
			Add a standard map to this material.
			@param type A member of the DXStandardMapType enumeration which indicates which type to add.
			@param source The filename of the texture to be used for the map.
			@param out An optional pointer to an IDXMaterialMapPtr where a link to the newly added map will be
					   stored.
			@return Error_NullParam if source was null.\n
					Error_OutOfRange if type was invalid.\n
					Error_Duplicate if there is already a map with this type.\n
					Error_Success if the type was added.
		*/
		virtual DXError				DXMCALL AddStandardMap( DXStandardMapType type, const wchar_t* source, IDXMaterialMapPtr* out = 0 ) = 0;
		/**
			Remove a standard map from this material.
			@param type A member of the DXStandardMapType enumeration which indicates the type to remove.
			@return Error_OutOfRange if type was invalid.\n
					Error_NotFound if the specified map was not present on the material.\n
					Error_Success if the map was removed.
		*/
		virtual DXError				DXMCALL RemoveStandardMap( DXStandardMapType type ) = 0;
		/**
			Create a light map on this material.
			@param source The filename that this map will use as a texture.
			@param generateSize The size of the side of the texture that will be used during autobaking of light.
			@param channel The UV channel that the lightmap will be applied to.
			@param out A pointer to an IDXLightMapPtr where a link will be stored. Optional.
			@return Error_NullParam if source was null.\n
					Error_InvalidParam if generateSize was not a power of 2.\n
					Error_Duplicate if a lightmap already exists.\n
					Error_Success if the light map was created.
		*/
		virtual DXError				DXMCALL CreateLightMap( const wchar_t* source, uint generateSize, uint channel, IDXLightMapPtr* out = 0 ) = 0;
		/**
			Get the light map that is being used on this IDXMaterial.
			@param out A pointer to an IDXLightMapPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there is no light map being used.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetLightMap( IDXLightMapPtr* out ) const = 0;
		/**
			Remove the light map that is being used on this IDXMaterial.
			@return Error_NotFound if there was no light map on this material to remove.\n
					Error_Success if the lightmap was removed.
		*/
		virtual DXError				DXMCALL RemoveLightMap( ) = 0;
		/**
			Get the number of additional maps that this material
			has.
			@return The count of additional maps.
		*/
		virtual uint				DXMCALL GetNumMultiMaps( ) const = 0;
		/**
			Get the additional map at the specified index.
			@param index The index of the additional map to retrieve.
			@param out A pointer to an IDXMultiMapPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetMultiMap( uint index, IDXMultiMapPtr* out ) = 0;
		/**
			Add an additional map to this material.
			@param source The filename of the texture that will be used for this map.
			@param coordChannel The UV channel that will be used for the color of this map.
			@param alphaChannel The UV channle that will be used for the alpha of this map.
			@param opacity A floating point value between 0 and 1 which represents how opaque this map is.
			@param combineType A member of the DXMapCombine enumeration which describes how this map will be combined with the 
							   standard map.
			@param out An optional pointer to an IDXMultiMapPtr where a link to the new map will be stored.
			@return Error_NullParam if source was null.\n
					Error_OutOfRange if combineType was invalid.\n
					Error_Success if the map was created.
		*/
		virtual DXError				DXMCALL AddMultiMap( const wchar_t* source, uint coordChannel, uint alphaChannel, float opacity,
				DXMapCombine combineType, IDXMultiMapPtr* out = 0 ) = 0;
		/**
			Remove the additional map at the specified index.
			@param index The index of the additional map to retrieve.
			@return Error_OutOfRange if the index was out of range.\n
					Error_Success if the additional map was removed.
		*/
		virtual DXError				DXMCALL RemoveMultiMap( uint index ) = 0;
		/**
			Remove the specified additional map.
			@param map The map to remove.
			@return Error_NullParam if map was null.\n
					Error_NotFound if the map was not present in the material.\n
					Error_Success if the map was removed.
		*/
		virtual DXError				DXMCALL RemoveMultiMap( IDXMultiMapPtr map ) = 0;
		/**
			Remove a range of additional maps from the material.
			@param start The index of the first map to remove.
			@param count The number of maps to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError				DXMCALL RemoveMultiMap( uint start, uint count ) = 0;
		/**
			Remove all the additional maps from this material.
		*/
		virtual void				DXMCALL ClearMultiMaps( ) = 0;
	};

	DXMESHEXPORT const wchar_t* TranslateStandardMap( DXStandardMapType type  ) ;
	DXMESHEXPORT const char* TranslateCombine(DXMapCombine combine);
}

#endif