#ifndef IDXMESH_H
#define IDXMESH_H

namespace DX
{
	struct IDXMaterial;
	/**
		An IDXMesh can be added to a group and contains all the vertices and 
		faces for a mesh. It can also have skin.
	*/
	struct IDXMesh : public IDXUserData, public IDXModelOwned, public IDXGroupOwned, public IDXHasFileBinding
	{
		virtual				DXMCALL ~IDXMesh() = 0;
		/**
			Get the number of vertices that are in the IDXMesh.
			@return The number of vertices in the mesh.
		*/
		virtual uint		DXMCALL GetNumVertices() const = 0;
		/**
			Get the number of faces in the IDXMesh
			@return The number of faces in the mesh.
		*/
		virtual uint		DXMCALL GetNumFaces() const = 0;
		/**
			Get the number of material maps that are on this IDXMesh
		*/
		virtual uint		DXMCALL GetNumMaterialMaps() const = 0;
		/**
			Get the number of uv channels that this mesh has.
			@return The number of channels
		*/
		virtual uint		DXMCALL GetNumUVChannels( ) const = 0;
		/**
			Get the vertex at the specified index.
			@param index The index of the vertex to retrieve.
			@param out A pointer to an IDXVertexPtr which will receive the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetVertex( uint index, IDXVertexPtr* out ) const = 0;
		/**
			Fill a user allocated array with vertex data. The user must create and delete this
			array. Takes a range from the vertices data.
			@param start The inclusive index to start retrieving data from.
			@param count The number of vertices to retrieve.
			@param out An array of IDXVertexPtr of at least equal to count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetVertices( uint start, uint count, IDXVertexArray out ) const = 0;
		/**
			Fill a user allocated array with vertex data. The user must create and delete this
			array.
			@param out A pointer to an array of IDXVertex of length at least equal to GetNumVertices.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetVertices( IDXVertexArray out ) const = 0;
		/**
			Get the face at the specified index.
			@param index The index of the face to retrieve.
			@param out A pointer to an IDXFacePtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetFace( uint index, IDXFacePtr* out ) const = 0;
		/**
			Fill a user allocated array with faces data. The user must create and delete this
			array. Takes a range from the faces data.
			@param start The inclusive index to start retrieving data from.
			@param count The number of vertices to retrieve.
			@param out An array of IDXFacePtr of at least equal to count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetFaces( uint start, uint count, IDXFaceArray out ) const = 0;
		/**
			Fill a user allocated array with face data. The user must create and delete this
			array.
			@param out A pointer to an array of IDXFacePtr of length equal to GetNumFaces
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetFaces( IDXFaceArray out ) const = 0;
		/**
			Get the index of the material map at the specified index.
			@param index The index of the material map to retrieve
			@param out A uint pointer where the index will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was not in range.\n
					Error_Success if the index was output.
		*/
		virtual DXError		DXMCALL GetMaterialMap(uint index, uint* out) const = 0;
		/**
			Get the skin on this mesh.
			@param A pointer to an IDXSkinPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no skin on this mesh.\n
					Error_Success if the skin was stored in out.
		*/
		virtual DXError		DXMCALL GetSkin( IDXSkinPtr* out ) const = 0;
		/**
			Set the number of uv channels that this mesh has.
			@param numChannels The number of channels to set this to.
		*/
		virtual void		DXMCALL SetNumUVChannels( uint numChannels ) = 0;
		/**
			Set the material map at the specified index.
			@param index The index of the material map to set.
			@param materialIndex The index of the material in the model to use.
			@return Error_OutOfRange if index was out of range.\n
					Error_OutOfRange2 if materialIndex was out of range and was not DXMESH_UNASSIGNED_MATERIAL.\n
					Error_Success if the material was updated.
		*/
		virtual DXError		DXMCALL SetMaterialMap( uint index, uint materialIndex ) = 0;
		/**
			Set the material map at the specified index.
			@param index The index of the material map to set.
			@param material An IDXMaterialPtr which is the the material to use.
			@return Error_NullParam if material was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_NotFound if the material was not found on the model.\n
					Error_Success if the material map was updated.
		*/
		virtual DXError		DXMCALL SetMaterialMap( uint index, IDXMaterialPtr material ) = 0;
		/**
			Set the material map at the specified index.
			@param index The index of the material map to set.
			@param materialID The id of the material to use.
			@return Error_NullParam if materialID was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_NotFound if the material was not found on the model.\n
					Error_Success if the material map was updated.
		*/
		virtual DXError		DXMCALL SetMaterialMap( uint index, const char* materialID ) = 0;
		/**
			Add a new vertex to this IDXMesh.
			@param vector The position value of the vertex.
			@param normal The normal to this vertex.
			@param uvCount The number of UV coordinates that this vertex has. Defaults to 0
			@param uvs If uvCount is non-zero, then this should be a point to an array of DXUV, of length equal to uvCount.
			@return A pointer to the new vertex.
		*/
		virtual IDXVertexPtr	DXMCALL AddVertex(const DXVector& vector, const DXVector& normal, uint uvCount = 0, const DXUV* uvs = 0) = 0;
		/**
			Add a new face to this IDXMesh.
			@param v0 The index of the zeroth vertex of the new face.
			@param v1 The index of the first vertex of the new face.
			@param v2 The index of the second vertex of the new face.
			@param material The index of the material to use on the new face.
			@param out An optional pointer to an IDXFacePtr where a pointer to the new face will be stored.
			@return Error_InvalidVertex if any of the vertex indices were out of range.\n
					Error_InvalidMaterial if the material was out of range.\n
					Error_Success if the face was added.
		*/
		virtual DXError		DXMCALL AddFace(uint v0, uint v1, uint v2, uint material, IDXFacePtr* out = 0) = 0;
		/**
			Add a material map to this IDXMesh.
			@param materialName This is the name of a material that has been created on the model.
			@param out This is an optional paremeter. a pointer to an uint variable which
						will be set to the index of the map that has been added. If the function returns
						Error_Success, this will be set to the index for the material that has been added.
						If the function returns Error_Duplicate, this will be set to the index of the
						map that already exists in the IDXMesh.
			@return Error_NullParam if materialName was null.\n
					Error_OutOfRange if the material was not found.\n
					Error_Duplicate if the material is already a map.\n
					Error_Success if the material map was added.
		*/
		virtual DXError		DXMCALL AddMaterialMap(const char* materialName, bool allowDuplicate, uint* out = 0) = 0;
		/**
			Add a material map to this IDXMesh.
			@param index This is the index of a material that has been created on the model.
			@param out This is an optional paremeter, a pointer to an uint variable which
						will be set to the index of the map that has been added. If the function returns
						Error_Success, this will be set to the index for the material that has been added.
						If the function returns Error_Duplicate, this will be set to the index of the
						map that already exists in the IDXMesh.
			@return Error_OutOfRange if the material was not found.\n
					Error_Duplicate if the material is already a map.\n
					Error_Success if the material map was added.
		*/
		virtual DXError		DXMCALL AddMaterialMap(uint index, bool allowDuplicate, uint* out = 0) = 0;
		/**
			Remove the specified vertex from the mesh. Any faces that reference this vertex
			will have the appropriate index set to DXMESH_UNASSIGNED_VERTEX.
			@param vertex The vertex to remove.
			@return Error_NullParam if vertex was null.\n
					Error_NotFound if the vertex was not found on the mesh.\n
					Error_Success if the vertex was removed.
		*/
		virtual DXError		DXMCALL RemoveVertex( IDXVertexPtr vertex ) = 0;
		/**
			Remove the vertex at the specified index from the mesh. Any faces that reference this vertex
			will have the appropriate index set to DXMESH_UNASSIGNED_VERTEX.
			@param index The index of the vertex to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the vertex was removed.
		*/
		virtual DXError		DXMCALL RemoveVertex( uint index ) = 0;
		/**
			Remove a range of vertices from the mesh. Any faces that reference these vertices
			will have the appropriate index set to DXMESH_UNASSIGNED_VERTEX.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveVertex( uint start, uint count ) = 0;
		/**
			Remove the given face from the mesh.
			@param face An IDXFacePtr to remove from the mesh.
			@return Error_NullParam if face was null.\n
					Error_NotFound if the face was not found on this mesh.\n
					Error_Success if the face was removed.
		*/
		virtual DXError		DXMCALL RemoveFace( IDXFacePtr face ) = 0;
		/**
			Remove the face at the specified index,
			@param index The index of the face to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the face was removed.
		*/
		virtual DXError		DXMCALL RemoveFace( uint index ) = 0;
		/**
			Remove a range of faces.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveFace( uint start, uint count ) = 0;
		/**
			Remove any material maps with the specified name from this mesh.
			@param materialName The name of the material to remove.
			@return Error_NullParam if materialName was null.\n
					Error_NotFound if there were no material maps with this name.\n
					Error_Success if materials matching this name were removed.
		*/
		virtual DXError		DXMCALL RemoveMaterialMap( const char* materialName ) = 0;
		/**
			Remove the specified material from this mesh's material maps.
			@param map The material to remove.
			@return Error_NullParam if map was null.\n
					Error_NotFound if there were no instances of map on this mesh.\n
					Error_Success if meterials were removed.
		*/
		virtual DXError		DXMCALL RemoveMaterialMap( IDXMaterialPtr map ) = 0;
		/**
			Remove the material map at the specified index.
			@param index The index of the material map to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the material was removed.
		*/
		virtual DXError		DXMCALL RemoveMaterialMap( uint index ) = 0;
		/**
			Remove a range of materials from this mesh's material maps.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveMaterialMap( uint start, uint count ) = 0;
		/**
			Remove all vertices from the mesh.
			Any faces that reference these vertices will have the appropriate 
			index set to DXMESH_UNASSIGNED_VERTEX.
		*/
		virtual void		DXMCALL ClearVertices( ) = 0;
		/**
			Remove all faces from this mesh.
		*/
		virtual void		DXMCALL ClearFaces( ) = 0;
		/**
			Remove all material maps.
		*/
		virtual void		DXMCALL ClearMaterialMaps( ) = 0;
		/**
			Create a skin on this mesh.
			@param out An optional pointer to an IDXSkinPtr where a pointer to the skin will
					   be stored.
			@return Error_Duplicate if a skin already exists on this mesh.\n
					Error_Success if the skin was created.
		*/
		virtual DXError		DXMCALL CreateSkin( IDXSkinPtr* out = 0 ) = 0;
		/**
			Destroy any skin that exists on this mesh.
			@return Error_NotFound if there was no skin to destroy on this mesh.\n
					Error_Success if the skin was destroyed.
		*/
		virtual DXError		DXMCALL DestroySkin( ) = 0;
		/**
			Optimize the mesh for a faster render.
		*/
		virtual void		DXMCALL Optimize( ) = 0;
	};
}

#endif