#ifndef IDXMODEL_H
#define IDXMODEL_H

namespace DX
{
	//Load options.
	//Retain bindings - keeps old files.
	enum DXLoadOptions
	{
		Load_Standard		 = 0x00000000,	//Default behaviour. Data files are bound and no resource files are removed.
		Load_RemoveBindings  = 0x00000001,	//If specified, data files loaded will be removed.
		Load_RemoveResources = 0x00000002,	//If specified, resources (any non int/float/xyz file) will be removed.
	};

	/**
		IDXModel is the main interface in the DXMeshAPI. It represents
		an entire model. It directly contains groups, animations and materials,
		and supports storage of relevant metadata.
	*/
	struct  IDXModel : public IDXUserData
	{
		virtual						DXMCALL ~IDXModel() = 0;
		/**
			@return The number of materials that are in this model.
		*/
		virtual uint				DXMCALL GetNumMaterials() const = 0;
		/**
			@return The number of groups that are in this model.
		*/
		virtual uint				DXMCALL GetNumGroups() const = 0;
		/**
			@return The number of animations that are in this model.
		*/
		virtual uint				DXMCALL GetNumAnimations() const = 0;
		/**
			@return The number of user tags in this model.
		*/
		virtual uint				DXMCALL GetNumUserTags() const = 0;
		/**
			Get the material at the specified index.
			@param index The index of the material to retrieve.
			@param out A pointer to an IDXMaterialPtr to store the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetMaterial( uint index, IDXMaterialPtr* out ) const = 0;
		/**
			Get the material with the specified id.
			@param id The id to search for.
			@param out A pointer to an IDXMaterialPtr to store the result.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the material was not found.\n
					Error_Success if the material was found and the result was stored in out.
		*/
		virtual DXError				DXMCALL GetMaterial( const char* id, IDXMaterialPtr* out ) const = 0;
		/**
			Get all IDXMaterialPtr on this model.
			@param out A user allocated array of IDXMaterialPtr of length at least GetNumMaterials.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetMaterials( IDXMaterialArray out ) const = 0;
		/**
			Get a group of materials
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXMaterialPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetMaterials( uint start, uint count, IDXMaterialArray out ) const = 0;
		/**
			Get the index of the material with the specified id.
			@param id The id to search for.
			@param out A pointer to a uint where the index will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if a material with the specified id was not found.\n
					Error_Success if the material was found and the index stored in out.
		*/
		virtual DXError				DXMCALL GetMaterialIndex( const char* id, uint* out ) const = 0;
		/**
			Get the index of the material with the specified id.
			@param material An IDXMaterialPtr to search for.
			@param out A pointer to a uint where the index will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if material was null.\n
					Error_NotFound if the material was not found.\n
					Error_Success if the material was found and the index stored in out.
		*/
		virtual DXError				DXMCALL GetMaterialIndex( IDXMaterialPtr material, uint* out ) const = 0;
		/**
			Get the group at the specified index.
			@param index The index of the group to retrieve.
			@param out A pointer to an IDXGroupPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the group was stored in out.
		*/
		virtual DXError				DXMCALL GetGroup( uint index, IDXGroupPtr* out ) const = 0;
		/**
			Get the first group with the specified id.
			@param id The id of the group to search for.
			@param out A pointer to an IDXGroupPtr where the result will be stored.
			@param recurse Set this to true to search through all subgroups.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the group was not found.\n
					Error_Success if the result was stored in out
		*/
		virtual DXError				DXMCALL GetGroup( const char* id, IDXGroupPtr* out, bool recurse = false ) const = 0;
		/**
			Get any groups with the specified id.
			@param id The id to find.
			@param out A pointer to an IDXGetGroupResultPtr which will store the results.
			@param recurse If true, thiswill scan all child groups.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no group found with the specfied id.\n
					Error_Success if a number of groups were stored in out.
		*/
		virtual DXError				DXMCALL GetGroup( const char* id, IDXGetGroupResultPtr* out, bool recurse = false ) = 0;
		/**
			Get the groups which are direct children of this model.
			@param out A user allocated array of IDXGroupPtr of length at least GetNumGroups.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetGroups( IDXGroupArray out ) const = 0;
		/**
			Get a block of groups which are direct children of this model.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXGroupPtr of at least count where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetGroups( uint start, uint count, IDXGroupArray out ) const = 0;
		/**
			Get the index of the group with the specified id.
			@param id The id of the group to find.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no group with the specified id.\n
					Error_Success if the group was found and the result stored in out.
		*/
		virtual DXError				DXMCALL GetGroupIndex( const char* id, uint* out ) const = 0;
		/**
			Get the index of the specified group.
			@param group The group to find.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the group was not found.\n
					Error_Success if the group was found and the result stored in out.
		*/
		virtual DXError				DXMCALL GetGroupIndex( IDXGroupPtr group, uint* out ) const = 0;
		/**
			Get the animation at the specified index.
			@param index The index of the animation to retrieve.
			@param out A pointer to an IDXAnimationPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetAnimation( uint index, IDXAnimationPtr* out ) const = 0;
		/**
			Get the animation with the specified id.
			@param id The id to search for.
			@param out A pointer to an IDXAnimationPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no animation with the specified id.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetAnimation( const char* id, IDXAnimationPtr* out ) const = 0;
		/**
			Get all animations on this model.
			@param out A user allocated array of IDXAnimationPtr of length at least GetNumAnimations
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetAnimations( IDXAnimationArray out ) const = 0;
		/**
			Get a group of animations from this model.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXAnimationPtr of length at least count.
		*/
		virtual DXError				DXMCALL GetAnimations( uint start, uint count, IDXAnimationArray out ) const = 0;
		/**
			Get the index of the animation with the specified id.
			@param id The id of the animation to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no animation with the specified id.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetAnimationIndex( const char* id, uint* out ) const = 0;
		/**
			Get the index of the specified animation.
			@param animation The animation to search for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if animation was null.\n
					Error_NotFound if the specified animation was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetAnimationIndex( IDXAnimationPtr animation, uint* out ) const = 0;
		/**
			Get the user tag at the specified index.
			@param index The index of the IDXUserTag to retrieve.
			@param out A pointer to an IDXUserTagPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTag( uint index, IDXUserTagPtr* out ) const = 0;
		/**
			Get the first user tag with the specified id.
			@param id The id of the user tag to retrieve.
			@param out A pointer to an IDXUserTagPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if the tag was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTag( const wchar_t* id, IDXUserTagPtr* out ) const = 0;
		/**
			Get all user tags on this model.
			@param out A user allocated array of IDXUserTagPtr of length at least GetNumUserTags
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTags( IDXUserTagArray out ) const = 0;
		/**
			Get a group of user tags from this model.
			@param start The index of the first element to retrieve
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXUserTagPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTags( uint start, uint count, IDXUserTagArray out ) const = 0;
		/**
			Get the index of the specified tag.
			@param tag The IDXUserTag to get the index of.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if tag was null.\n
					Error_NotFound if tag was not found in the model.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTagIndex( IDXUserTagPtr tag, uint* out ) const = 0;
		/**
			Get the index of the first tag with the specified id.
			@param id The id to serach for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
				    Error_NullParam2 if id was null.\n
					Error_NotFound if there was no tag with a matching ID.
					Error_Success if the result was stored in out.
		*/
		virtual DXError				DXMCALL GetUserTagIndex( const wchar_t* id, uint* out ) const = 0;
		/**
			Get the IDXModelInfoPtr for this IDXModel. Allows you to
			set properties such as title, summary and add tags, amongst
			other things.
			@return An IDXModelInfoPtr.
		*/
		virtual IDXModelInfoPtr		DXMCALL GetInfo() const = 0;
		/**
			Get the IDXModelPropertiesPtr for this IDXModel. Allows you to
			add default object properties, default script and so on.
			@return An IDXModelPropertiesPtr.
		*/
		virtual IDXModelPropertiesPtr DXMCALL GetProperties() const = 0;
		/**
			Get this model's IDXFileSystem. This allows you to manage the
			files that will be added to the zip, or to examine the files
			that have been loaded from a zip.
		*/
		virtual IDXFileSystemPtr DXMCALL GetFileSystem( ) const = 0;
		/**
			Create a new top level group on this model.
			@param id The id that will be used to refer to this group in the API and in 
					  javascript inside DXStudio.
			@param out An optional pointer to an IDXGroupPtr where a pointer to the new IDXGroupPtr will 
					   be saved.
			@return Error_NullParam if id was null.\n
					Error_InvalidParam if id contained invalid characters.\n
					Error_Success if the group was added successfully.
			@see IDXGroup
		*/
		virtual DXError				DXMCALL CreateGroup(const char* id, IDXGroupPtr* out = 0) = 0;
		/**
			Create a new animation on this model.
			@param id The name that will be used to refer to this animation in the API and
					  also in javascript inside DXStudio.
		    @param duration The period of time that this animation will play for
			@param snap This value is used by the editor to determine the interval at which keyframes
						can be added.
			@param autoplay If this is true, then this animation will automatically play when the
							model is used by an object.
			@param loop If this is true the animation will loop around when it reaches the end.
			@param out Optional. A pointer to an IDXAnimationPtr which will be set to point to the newly added
					   animation.	
		    @return Error_NullParam if id was null.\n
					Error_InvalidParam if id contained invalid characters.\n
					Error_Duplicate if an animation already exists with the same ID.\n
					Error_Success if the animation was added successfully.
			@see IDXAnimation
		*/
		virtual DXError				DXMCALL CreateAnimation(const char* id, float duration, float snap, bool autoplay, bool loop, IDXAnimationPtr* out = 0) = 0;
		/**
			Create a new material and add it to the model.
			@param id The name that will be used to refer to the new material in the API and in
					  script.
			@param out Optional. A pointer to an IDXMaterialPtr variable where a pointer to the new
					   material will be stored.
			@return Error_NullParam if id was null.\n
					Error_InvalidParam if id contained invalid characters.\n
					Error_Duplicate if a material with the same id already exists in this model.\n
					Error_Success if the material was added successfully.
			@see IDXMaterial
		*/
		virtual DXError				DXMCALL CreateMaterial(const char* id, IDXMaterialPtr* out = 0) = 0;
		/**
			Create a new usertag and add it to the model.
			@param id The id that will be used for the tag.
			@param value The value that will be used for the tag.
			@param out An optional pointer to an IDXUserTagPtr variable where a pointer to the new material
						will be stored.
			@return Error_NullParam if id was null.\n
					Error_NullParam2 if value was null.\n
					Error_Success if the tag was added.
		*/
		virtual DXError				DXMCALL CreateUserTag( const wchar_t* id, const wchar_t* value, IDXUserTagPtr* out = 0 ) = 0;
		/**
			Save the model to the specified filename. 
			@param filename The location to save the model to.
			@return Error_NullParam if filename was null.\n
					Error_Success if the model was saved successfully.
		*/
		virtual DXError				DXMCALL SaveToFile(const wchar_t* const filename) = 0;
		/**
			Load a model from a .dxmesh file.
			@param filename The location to load the model from.
		*/
		virtual DXError				DXMCALL LoadFromFile( const wchar_t* const filename, DXLoadOptions options = Load_Standard ) = 0;
		/**
			Remove the group with the specified id from the model.
			@param id The id of the group to remove.
			@param recurse Set to true if you want to search child groups for the group.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no group with the specified id.\n
					Error_Success if the group was removed.
		*/
		virtual DXError				DXMCALL RemoveGroup( const char* id, bool recurse = false ) = 0;
		/**
			Removes the specfied group from the model.
			@param group The group to remove.
			@param recurse Set to true if you want to search child groups for this group.
			@return Error_NullParam if group was null.\n
					Error_NotFound if the group was not found.\n
					Error_Success if the group was removed.
		*/
		virtual DXError				DXMCALL RemoveGroup( IDXGroupPtr group, bool recurse = false ) = 0;
		/**
			Remove the group at the specified index from the model.
			@param index The index of the group to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the group was removed.
		*/
		virtual DXError				DXMCALL RemoveGroup( uint index ) = 0;
		/**
			Remove a range of groups from the model.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError				DXMCALL RemoveGroup( uint start, uint count ) = 0;
		/**
			Remove all the groups in this model.
		*/
		virtual void				DXMCALL ClearGroups() = 0;
		/**
			Remove the animation with the specified id from the model.
			@param id The id of the animation to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no animation with the specified id.\n
					Error_Success if the animation was removed.
		*/
		virtual DXError				DXMCALL RemoveAnimation( const char* id ) = 0;
		/**
			Remove the specified animation from the model.
			@param animation The animation to remove.
			@return Error_NullParam if animation was null.\n
					Error_NotFound if animation was not found in this model.\n
					Error_Success if the animation was removed from the model.
		*/
		virtual DXError				DXMCALL RemoveAnimation( IDXAnimationPtr animation ) = 0;
		/**
			Remove the animation at the specified index from the model.
			@param index The index of the animation to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the animation was removed successfully.
		*/
		virtual DXError				DXMCALL RemoveAnimation( uint index ) = 0;
		/**
			Remove a range of animations from the model.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError				DXMCALL RemoveAnimation( uint start, uint count ) = 0;
		/**
			Remove all the animations from this model.
		*/
		virtual void				DXMCALL ClearAnimations() = 0;
		/**
			Remove the material with the specified id from the model.
			@param id The id of the material to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no material with the specified id on the model.\n
					Error_Success if the material was removed.
		*/
		virtual DXError				DXMCALL RemoveMaterial(const char* id) = 0;
		/**
			Remove the specified material from the Model.
			@param material The material to remove from the model.
			@return Error_NullParam if material was null.\n
					Error_NotFound if the material was not found on the model.\n
					Error_Success if the material was removed.
		*/
		virtual DXError				DXMCALL RemoveMaterial(IDXMaterialPtr material) = 0;
		/**
			Remove the material at the specified index from the model.
			@param index The index of the material to remove.
			@param Error_OutOfRange if index was out of range.\n
					Error_Success if the material was removed.				
		*/
		virtual DXError				DXMCALL RemoveMaterial(uint index) = 0;
		/**
			Remove a range of materials from the model.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError				DXMCALL RemoveMaterial(uint start, uint count ) = 0;		
		/**
			Remove all materials from this model.
		*/
		virtual void				DXMCALL ClearMaterials() = 0;
		/**
			Remove the IDXUserTag at the specified index.
			@param index The index of the IDXUserTag to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the IDXUserTag was removed.
		*/
		virtual DXError				DXMCALL RemoveUserTag( uint index ) = 0;
		/**
			Remove the first IDXUserTag with the specified id.
			@param id The id to search for.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no tag with matching id.\n
					Error_Success if the tag was removed.
		*/
		virtual DXError				DXMCALL RemoveUserTag( const wchar_t* id ) = 0;
		/**
			Remove a range of IDXUserTag.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError				DXMCALL RemoveUserTag( uint start, uint count ) = 0;
		/**
			Remove the specified IDXUserTag.
			@param tag The IDXUserTag to remove.
			@return Error_NullParam if tag was null.\n
					Error_NotFound if tag was not found in the model.\n
					Error_Success if the tag was removed.
		*/
		virtual DXError				DXMCALL RemoveUserTag( IDXUserTagPtr tag ) = 0;
		/**
			Remove all user tags from this model.
		*/
		virtual void				DXMCALL ClearUserTags() = 0;
		/**
			Sorts the faces in meshes to be in material order. This greatly improves
			rendering speed.
		*/
		virtual void		   DXMCALL Optimize( ) = 0;
	};

	DXMESHEXPORT IDXModelPtr CreateModel();
	DXMESHEXPORT IDXModelPtr CreateModelFromFile(const wchar_t* const filename, DXLoadOptions options = Load_Standard );
	DXMESHEXPORT void DestroyModel(IDXModelPtr model);
}

#endif