#ifndef IDXMODELINFO_H
#define IDXMODELINFO_H

namespace DX
{
	/**
		IDXModelInfo contains metadata for a model, including summary, tags 
		and title.
	*/
	struct  IDXModelInfo : public IDXModelOwned, public IDXUserData
	{
		virtual					DXMCALL ~IDXModelInfo() = 0;
		/**
			The current title for this model.
			@return The title that is being used for this model.
		*/
		virtual const wchar_t*	DXMCALL GetTitle() const = 0;
		/**
			The current summary for this model.
			@return The summary that is being used.
		*/
		virtual const wchar_t*	DXMCALL GetSummary() const = 0;
		/**
			Get the number of tags currently on this IDXModel
			@return The number of tags.
		*/
		virtual uint			DXMCALL GetNumTags() const = 0;
		/**
			Get the tag at an index.
			@param index The index of the tag to retrieve.
			@param out A pointer to an IDXTagPtr to receive the tag.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was not less than the number of tags on the model.\n
					Error_Success if the tag was returned.
		*/
		virtual DXError			DXMCALL GetTag(uint index, IDXTagPtr* out) const = 0;
		/**
			Get all tags that are on this model.
			@param out A user allocated array of IDXTagPtr of length at least GetNumTags.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetTags( IDXTagArray out ) const = 0;
		/**
			Get a block of IDXTags.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXTagPtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetTags( uint start, uint count, IDXTagArray out ) const = 0;
		/**
			Get the thumbnail filename.
			@return The name of the thumbnail file to use.
		*/
		virtual const wchar_t*	DXMCALL GetThumbnail() const = 0;
		/**
			Set the title of the model that will appear in the editor
			@param title The new title to use for the model
			@return Error_NullParam if title was null.\n
					Error_Success if the title was changed.
		*/
		virtual DXError			DXMCALL SetTitle(const wchar_t* title) = 0;
		/**
			Set the summary that will be displayed in the editor.
			@param summary The new summary to use.
			@return Error_NullParam if summary was null.\n
					Error_Success if the summary was changed.
		*/
		virtual DXError			DXMCALL SetSummary(const wchar_t* summary) = 0;
		/**
			Set the thumbnail filename for this model.
			@param filename The filename to use.
			@return Error_NullParam if filename was null.\n
					Error_Success if the thumbnail was set.
		*/
		virtual DXError			DXMCALL SetThumbnail(const wchar_t* filename) = 0;
		/**
			Add a new tag to this IDXModel
			@param tag The new tag to add
			@return Error_NullParam if tag was null.\n
					Error_Duplicate if the tag is already present.\n
					Error_Success if the tag was added.
		*/
		virtual DXError			DXMCALL AddTag(const wchar_t* tag, IDXTagPtr* out = 0 ) = 0;
		/**
			Remove a tag from the taglist
			@param index The index of the tag to remove.
			@return Error_OutOfRange if index was not less than the number of tags on the model.\n
					Error_Success if the tag was removed from the model.
		*/
		virtual DXError			DXMCALL RemoveTag(uint index) = 0;
		/**
			Remove a range of tags.
			@param start The index of the first tag to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the range was removed.
		*/	
		virtual DXError			DXMCALL RemoveTag( uint start, uint count ) = 0;
		/**
			Remove a tag from the taglist
			@param tag The tag to remove. (Compared based on content, not on address)
			@return Error_NullParam if tag was null.\n
					Error_NotFound if the tag was not found.\n		
					Error_Success if the tag was removed.
		*/
		virtual DXError			DXMCALL RemoveTag(const wchar_t* tag) = 0;
		/**
			Remove all tags from the IDXModelInfo
		*/
		virtual void			DXMCALL ClearTags() = 0;
	};
}

#endif