#ifndef IDXMODELPROPERTIES_H
#define IDXMODELPROPERTIES_H

namespace DX
{
	/**
		IDXModelProperties contains script, metres per unit, and
		any number of default object properties.
	*/	
	struct  IDXModelProperties : public IDXModelOwned, public IDXUserData
	{
		virtual				DXMCALL ~IDXModelProperties() = 0;
		/**
			The current MPU for this model. This is a way of
			scaling the model.
			@return The current meters per unit.
		*/
		virtual float		DXMCALL GetMetresPerUnit() const = 0;
		/**
			Get the default script that is attached to this model
			@param out A pointer to a const char* which will receive the script.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no script on this model.\n
					Error_Success if the result was stored in out
		*/
		virtual DXError		DXMCALL GetScript( const char** out ) const = 0;
		/**
			Get the number of properties that are on this model.
			@return The number of properties.
		*/
		virtual uint		DXMCALL GetNumProperties() const = 0;
		/**
			Get the property at the specified index.
			@param index The index of the property to retrieve.
			@param out A pointer to an IDXObjectPropPtr to store the result in.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was invalid.\n
					Error_Success if the property was stored in out.
		*/
		virtual DXError		DXMCALL GetProperty( uint index, IDXObjectPropPtr* out ) const = 0;
		/**
			Get the property at the specified index.
			@param index The index of the property to retrieve.
			@param out A pointer to an IDXObjectPropPtr to store the result in.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no prop with the id.\n
					Error_Success if the property was stored in out.
		*/
		virtual DXError		DXMCALL GetProperty( const char* id, IDXObjectPropPtr* out ) const = 0;
		/**
			Set the metres per unit value for the model. This must be positive.
			@param value The new value.
			@return Error_InvalidParam if value was negative.\n
					Error_Success if the meters per unit was updated.
		*/
		virtual DXError		DXMCALL SetMetresPerUnit(float value) = 0;
		/**
			Set the default script for this model. This will overwrite the current script.
			@param script Character string containing the new script.
			@return Error_NullParam if script was null.\n
					Error_Success if the script was set.
		*/
		virtual DXError		DXMCALL SetScript(const char* script) = 0;
		/**
			Add a property to this model.
			@param id The ID of the new property. eg = "dynamicShadowCast"
			@param type The type of the new propery. eg = "bool"
			@param out An optional pointer to an IDXObjectPropPtr where a pointer will be stored.
			@return Error_NullParam if id was null.\n
					Error_NullParam if type was null.\n
					Error_Success if the property was added.
		*/
		virtual DXError		DXMCALL AddProperty( const char* id, const char* type, IDXObjectPropPtr* out = 0 ) = 0;
		/**
			Remove the property at the specified index.
			@param index The index of the property to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the property was removed.
		*/
		virtual DXError		DXMCALL RemoveProperty( uint index ) = 0;
		/**
			Remove the specified property.
			@param property The IDXObjectPropPtr to remove.
			@return Error_NullParam if property was null.\n
					Error_NotFound if the property was not found.\n
					Error_Success if the property was removed.
		*/
		virtual DXError		DXMCALL RemoveProperty( IDXObjectPropPtr property ) = 0;
		/**
			Remove a range of properties.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveProperty( uint start, uint count ) = 0;
		/**
			Remove the property with the specified id.
			@param id The id of the property to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no property with that id .\n
					Error_Success if the property was removed.
		*/
		virtual DXError		DXMCALL RemoveProperty( const char* id ) = 0;
		/**
			Remove all properties from this IDXModelProperties.
		*/
		virtual void DXMCALL ClearProperties( ) = 0;
		/**
			Remove the script attached to this mesh.
			@return Error_NotFound if there was no script on this mesh to remove.\n
					Error_Success if the script was removed.
		*/			
		virtual DXError		DXMCALL RemoveScript( ) = 0;
	};
}

#endif