#ifndef IDXOBJECTPROPERTY_H
#define IDXOBJECTPROPERTY_H

namespace DX
{
	/**
		IDXObjectProp is used for default object properties on
		the model.
	*/
	struct  IDXObjectProp : public IDXModelPropertiesOwned, public IDXModelOwned, public IDXUserData
	{
		virtual				DXMCALL ~IDXObjectProp() = 0;
		/**
			Get the id of this property.
			@return The current ID.
		*/
		virtual const char* DXMCALL GetID() const = 0;
		/**
			Get the current type of the property. eg : "vector"
			@return The current type.
		*/
		virtual const char* DXMCALL GetType() const = 0;
		/**
			Get how many attributes this IDXObjectProp has.
			@return The number of attributes.
		*/
		virtual uint		DXMCALL GetNumAttributes() const = 0;
		/**
			Get the attribute at the specified index.
			@param index The index of the attribute to retrieve.
			@param out A pointer to an IDXObjectPropAttributePtr to receive the attribute.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was invalid.\n
					Error_Success if the attribute was stored in out.					
		*/
		virtual DXError		DXMCALL GetAttribute( uint index, IDXObjectPropAttributePtr* out ) const = 0;
		/**
			Get the first attribute with the specified id.
			@param id The id of the attribute to look for.
			@param out A pointer to an IDXObjectPropAttributePtr to receive the attribute.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if id was null.\n
					Error_NotFound if there was no attribute with the specified id.
					Error_Success if the attribute was stored in out.					
		*/
		virtual DXError		DXMCALL GetAttribute( const char* id, IDXObjectPropAttributePtr* out ) const = 0;
		/**
			@param out A user allocated array of IDXObjectPropAttributePtr of length at least GetNumAttributes
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetAttributes( IDXObjectPropAttributeArray out ) const = 0;
		/**
			@param out A user allocated array of IDXObjectPropAttributePtr of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetAttributes( uint start, uint count, IDXObjectPropAttributeArray out ) const = 0;
		/**
			Set the id field of this property.
			@param id The new ID for this property
			@return Error_NullParam if id was null.\n
					Error_Success if the id was set.
		*/
		virtual DXError		DXMCALL SetID( const char* id ) = 0;
		/**
			Set the type of the property.
			@param type The new type for this property.
			@return Error_NullParam if type was null.\n
					Error_Success if the type was changed.
		*/
		virtual DXError		DXMCALL SetType( const char* type ) = 0;
		/**
			Add an attribute to this property.
			@param The id of the property. eg "x"
			@param The value of this property.eg "1.0451"
			@return Error_NullParam if id was null.\n
					Error_NullParam2 if value was null.\n
					Error_InvalidParam if id contained invalid characters.\n
					Error_Success if the attribute was added.
		*/
		virtual DXError		DXMCALL AddAttribute( const char* id, const char* value ) = 0;
		/**
			Remove all attributes on this prop
		*/
		virtual void		DXMCALL ClearAttributes() = 0;
		/**
			Remove the attribute at the specified index from this prop.
			@param index The index of the attribute to remove.
			@return Error_OutOfRange if the index was invalid.\n
					Error_Success if the attribute was removed.
		*/
		virtual DXError		DXMCALL RemoveAttribute( uint index ) = 0;
		/**
			Remove a range of attributes from this prop.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError		DXMCALL RemoveAttribute( uint start, uint count ) = 0;
		/**
			Remove the attribute with the specified id.
			@param id The id of the attribute to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no attribute with this id.
					Error_Success if the attribute was removed.
		*/
		virtual DXError		DXMCALL RemoveAttribute( const char* id ) = 0;
		/**
			Remove the specified attribute.
			@param attribute The attribute to remove.
			@return Error_NullParam if attribute was not null.\n
					Error_NotFound if attribute was not found.\n
					Error_Success if attribute was removed.
		*/
		virtual DXError		DXMCALL RemoveAttribute( IDXObjectPropAttributePtr attribute ) = 0;
	};
}

#endif