#ifndef IDXSKIN_H
#define IDXSKIN_H

namespace DX
{
	/**
		Controls assignment of vertices for skinning.
	*/
	struct  IDXSkin : public IDXUserData, public IDXModelOwned, public IDXMeshOwned
	{
		virtual			DXMCALL ~IDXSkin() = 0;
		/**
			Create a new IDXSkinEntry. This will skin the specified groupID.
			@param groupID The ID of the group in the model to skin
			@param out A pointer to an IDXSkinEntryPtr where a copy will be saved. This may be null.
			@return	Error_NullParam if groupID was null.\n
					Error_NotFound if the groupID specified could not be found in the model.\n
					Error_Duplicate if an IDXSkinEntry already exists in the animation for the specified groupID.\n
					Error_Success if the IDXSkinEntry was created and a copy saved in out.
		*/
		virtual DXError DXMCALL AddSkinEntry(const char* groupid, const DXMatrix& offsetMatrix, IDXSkinEntryPtr* out) = 0;
		/**
			The number of IDXSkinEntry there are in this Skin.
		*/
		virtual uint	DXMCALL GetNumSkinEntries() const = 0;
		/**
			Get the IDXSkinEntryPtr with the specified groupID.
			@param groupID The ID to search for.
			@param out A pointer to an IDXSkinEntryPtr to output the results.
			@return Error_NullParam if out was null.\n
					Error_NullParam if groupID was null.\n
					Error_NotFound if there is no IDXSkinEntryPtr in the IDXSkinPtr with that group ID.\n
					Error_Success if an IDXSkinEntryPtr was saved in out.
		*/
		virtual DXError DXMCALL GetSkinEntry(const char* groupID, IDXSkinEntryPtr* out) const = 0;
		/**
			Get the IDXSkinEntryPtr at the speicified index.
			@param index The index of the IDXSkinEntryPtr to retrieve.
			@param out A pointer to an IDXSkinEntryPtr to output the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was not less than the number of IDXSkinEntryPtr in the IDXSkin.\n
					Error_Success if an IDXSkinEntryPtr was saved in out.
		*/
		virtual DXError DXMCALL GetSkinEntry(uint index, IDXSkinEntryPtr* out) const = 0;
		/**
			Get all the skin entries on this skin.
			@param out A user allocated array of IDXSkinEntryPtr of length at least GetNumSkinEntries
					   where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetSkinEntries( IDXSkinEntryArray out ) const = 0;
		/**
			Get a block of skin entries for this skin.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXSkinEntryPtr of length at least count where the result
					   will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetSkinEntries( uint start, uint count, IDXSkinEntryArray out ) const = 0;
		/**
			Remove the IDXSkinEntry from the Skin with the specified groupID.
			@param groupID The groupID to search for.
			@return Error_NullParam if the groupID was null.\n
					Error_NotFound if there was no Skin Entry for the specified groupID.\n
					Error_Success if the Skin Entry was removed
		*/
		virtual DXError DXMCALL RemoveSkinEntry(const char* groupID) = 0;
		/**
			Remove the IDXSkinEntryPtr from the Skin with the specified groupID.
			@param skinEntry The IDXSkinEntryPtr to remove.
			@return Error_NullParam if IDXSkinEntryPtr was null.\n
					Error_NotFound if the IDXSkinEntryPtr is not owned by this Skin.\n
					Error_Success if the IDXSkinEntryPtr was removed
		*/
		virtual DXError DXMCALL RemoveSkinEntry(IDXSkinEntryPtr skinEntry) = 0;
		/**
			Remove the IDXSkinEntryPtr from the skin at the specified index.
			@param index The index of the IDXSkinEntryPtr to remove from the Skin.
			@return Error_OutOfRange if index was not less than the number of skin entries.\n
					Error_Success if the IDXSkinEntryPtr was removed.
		*/
		virtual DXError DXMCALL RemoveSkinEntry(uint index) = 0;
		/**
			Remove a range of IDXSkinEntry from this IDXSkin.
    		@param start The index of the first element to remove.
    		@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the IDXSkinEntryPtr was removed. 
		*/
		virtual DXError DXMCALL RemoveSkinEntry(uint start, uint count) = 0;
		/**
			Remove all IDXSkinEntryPtr from this IDXSkin.
		*/
		virtual void DXMCALL ClearSkinEntries( ) = 0;
	};
}

#endif