#ifndef IDXSKINENTRY_H
#define IDXSKINENTRY_H

namespace DX
{
	/**
		Skin data for a specific group.
	*/
	struct  IDXSkinEntry : public IDXModelOwned, public IDXSkinOwned, public IDXUserData, public IDXHasFileBinding
	{
		virtual					DXMCALL ~IDXSkinEntry() = 0;
		/**
			Get the matrix that will be applied by this skin entry.
			@return A DXMatrix representing the skin transform
		*/
		virtual const DXMatrix	DXMCALL GetOffsetMatrix() const = 0;
		/**
			Get the ID of the group that this skin entry is for.
			@return a const char* representing the group this skin entry skins.
		*/
		virtual const char*		DXMCALL GetGroupID() const = 0;
		/**
			The number of verts that are attached to this skin entry.
			@return Number of attached verts.
		*/
		virtual uint			DXMCALL GetNumSkinVerts() const = 0;
		/**
			Get the skin vert at the specified index.
			@param index The index of the skin vert to retrieve.
			@param out A pointer to a DXSkinVert* where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/		
		virtual DXError			DXMCALL GetSkinVert(uint index, IDXSkinVertPtr* out) const = 0;
		/**
			Get all the skin verts on the skin entry.
			@param out A user allocated array of IDXSkinVertPtr of length at least GetNumSkinVerts.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in otu.
		*/
		virtual DXError			DXMCALL GetSkinVerts( IDXSkinVertArray out ) const = 0;
		/**
			Get a block of IDXSkinVertPtr from the entry.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXSkinVertPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetSkinVerts( uint start, uint count, IDXSkinVertArray out ) const = 0;
		/**
			Get the index of the specified skin vert.
			@param vert The IDXSkinVertPtr to look for.
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if vert was null.\n
					Error_NotFound if the vert was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetSkinVertIndex( IDXSkinVertPtr vert, uint* out ) const = 0;
		/**
			Set the matrix that will be applied with this skin entry.
			@param matrix A DXMatrix.
		*/
		virtual void			DXMCALL SetOffsetMatrix(const DXMatrix& matrix) = 0;
		/**
			Set the group ID that this skin entry is for.
			@param id The id of the group.
			@return Error_NullParam if id was null.\n
					Error_Duplicate if there was already an entry for this id.\n
					Error_NotFound if there was no group with this id.\n
					Error_Success if the id was updated.
		*/
		virtual DXError			DXMCALL SetGroupID( const char* id ) = 0;
		/**
			Add a skin vert to this SkinEntry.
			@param vert A DXSkinVert to add.
		*/
		virtual void			DXMCALL AddSkinVert( uint index, float weight ) = 0;
		/**
			Remove the skin vert at the specified index.
			@param index The index of the skin vert to remove.
			@return Error_OutOfRange if the index was out of range.\n
					Error_Success if the SkinVert was removed.
		*/
		virtual DXError			DXMCALL RemoveSkinVert( uint index ) = 0;
		/**
			Remove the specified vert from the skin entry.
			@param vert The IDXSkinVert to remove.
			@return Error_NullParam if vert was null.\n
					Error_NotFound if the vert was not found.\n
					Error_Success if the vert was removed.
		*/
		virtual DXError			DXMCALL RemoveSkinVert( IDXSkinVertPtr vert ) = 0;
		/**
			Remove a range of skin verts from the skin entry.
			@param start The index of the first element to remove.
			@param end The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError			DXMCALL RemoveSkinVert( uint start, uint count ) = 0;
		/**
			Remove all skin verts from this skin entry.
		*/
		virtual void			DXMCALL ClearSkinVerts( ) = 0;
	};
}


#endif