#ifndef IDXSURFACEDETAIL_H
#define IDXSURFACEDETAIL_H

namespace DX
{
	struct IDXSurfaceDetailXYZ;
	/**
		This controls surface detail options for this material map.
		It allows for control of billboard options.
	*/
	struct  IDXSurfaceDetail : public IDXMaterialOwned, public IDXModelOwned, public IDXUserData
	{
		virtual					DXMCALL ~IDXSurfaceDetail() = 0;
		/**
			Get the number of squares that the billboard texture is being 
			divided into.
		*/
		virtual uint			DXMCALL GetBillboardSplit( ) const = 0;
		/**
			Get the sway value for the billboards. This controls how fast the
			billboards will sway.
		*/
		virtual float			DXMCALL GetBillboardSway( ) const = 0;
		/**
			Get the base world size that the billboards will appear at. There will
			be some additional radnomisation on top.
		*/
		virtual float			DXMCALL GetBillboardSize( ) const = 0;
		/**
			Get the name of the texture that is being used as the billboard.
			@return The name of the texture. 0 if not set.		
		*/
		virtual const wchar_t*	DXMCALL GetBillboardTexture( ) const = 0;
		/**
			Get the name of the texture that is being used as the lookup.
			@return The name of the texture.		
		*/
		virtual const wchar_t*	DXMCALL GetLookupTexture( ) const = 0;
		/**
			Get the number of meshes that there are on the surface 
			detail.
		*/
		virtual uint			DXMCALL GetNumMeshes( ) const = 0;
		/**
			Get the mesh at the specified index. 
			@param index The index of the mesh id to retrieve
			@return A wchar_t* which is the id of the mesh, or 0 if the
					index was out of range. (May change soon)
		*/
		virtual const wchar_t*	DXMCALL GetMesh( uint index ) const = 0;
		/**
			Get the IDXSurfaceDetailXYZ for this mesh.
			@param A point to an IDXSurfaceDetailXYZPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was not surface detail XYZ.\n					
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetSurfaceDetailXYZ( IDXSurfaceDetailXYZPtr* out ) const = 0;
		/**
			Set the number of squares that the texture being
			used for the billboard texture is divided into. This must be 
			a power of 2.
			@param value The new value for the split.
			@return Error_InvalidParam if value was not a power of 2
		*/
		virtual DXError			DXMCALL SetBillboardSplit( uint value ) = 0;
		/**
			Set the sway for the billboards. The higher the value, the more they
			will sway.
		*/
		virtual void			DXMCALL SetBillboardSway( float value ) = 0;
		/**
			Set the base world size that the billboards will be rendered at.
			This is further randomised at runtime.
		*/
		virtual void			DXMCALL SetBillboardSize( float value ) = 0;
		/**
			Set the map that will be used for the billboards on this surface detail.
			@param source The name of the map that will be used.
			@return Error_NullParam if source was null.\n
					Error_Success if the source was set.
		*/
		virtual DXError			DXMCALL SetBillboardTexture ( const wchar_t* source ) = 0;
		/**
			Add a new mesh id to this surface detail.
			This should be in the form "index=id:options" (at time of writing (~3.1 beta ) )
			For example : "1=gumtree:6"
			@param id The id of the mesh with an index and options
			@return Error_NullParam if id was null.\n
					Error_Success if the mesh was added.
		*/
		virtual DXError			DXMCALL AddMesh( const wchar_t* id ) = 0;
		/**
			Create an IDXSurfaceDetailXYZ.
			@param out An optional pointer to an IDXSurfaceDetailXYZPtr where the result will be stored.
			@return Error_Duplicate if there is already a surface detail.\n
					Error_Success if the IDXSurfaceDetailXYZ was created and the result was stored in out.
		*/
		virtual DXError			DXMCALL CreateSurfaceDetailXYZ( IDXSurfaceDetailXYZPtr* out = 0 ) = 0;
		/**
			Remove the billboard texture from this surface detail.
			@return Error_NotFound if there was no texture to remove.\n
					Error_Success if the texture was removed.
		*/
		virtual DXError			DXMCALL RemoveBillboardTexture ( ) = 0;
		/**
			Remove the lookup texture from this surface detail.
			@return Error_NotFound if there was no texture to remove.\n
					Error_Success if the texture was removed.
		*/
		virtual DXError			DXMCALL RemoveLookupTexture ( ) = 0;
		/**
			Set the map that will be used for the lookup on this surface detail.
			@param source The name of the map that will be used.
			@return Error_NullParam if source was null.\n
					Error_Success if the source was set.
		*/
		virtual DXError			DXMCALL SetLookupTexture( const wchar_t* source ) = 0;
		/**
			Remove the mesh id at the specified index.
			@param Index of mesh ID to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the mesh ID was removed.
		*/
		virtual DXError			DXMCALL RemoveMesh( uint index ) = 0;
		/**
			Remove the mesh with the specified id.
			@param id The id of the mesh to remove.
			@return Error_NullParam if id was null.\n
					Error_NotFound if there was no mesh id with the specified id.\n
					Error_Success if the mesh was remvoed.
		*/
		virtual DXError			DXMCALL RemoveMesh( const wchar_t* id ) = 0;
		/**
			Remove a range of mesh ids from the surface detail.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError			DXMCALL RemoveMesh( uint start, uint count ) = 0;
		/**
			Destroy the IDXSurfaceDetailXYZ.
			@return Error_NotFound if there was no IDXSurfaceDetailXYZ found.\n
					Error_Success if the IDXSurfaceDetailXYZ was destroyed.
		*/
		virtual DXError			DXMCALL DestroySurfaceDetailXYZ( ) = 0;
		/**
			Remove all meshes from this Surface Detail. Always succeeds.
		*/
		virtual void			DXMCALL ClearMeshes( ) = 0;
	};
}

#endif