/////////////////////////////////////////////////////////////////////////////////////
// BitmapFile.cpp
// bitmap file management and writing.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  01. Jan. 2006	: Added y flipping of bitmap.
//  01. Jan. 2006	: Initialization from TC texture.
//  10. Oct. 2006	: Achieved VS2005 compatibility, made compilation easier.
//  29. Dec. 2006	: Added RGBA format.
//  31. Jan. 2007   : Added GetPixel and Read methods.
//  19. Feb. 2007   : Added FlipX method.
//  01. Sep. 2008	: Now using DevIL instead of DirectX methods for loading,
//                    saving, and converting images.
//  12. Sep. 2008	: Added the methods BlurGaussian(), FillFromBitmap(), Clear(),
//					  and RemoveBorder().
//  03. Oct. 2008	: Implemented resampling, because of some scaling problems
//					  in the DevIL lib version 1.7.2
//  23. Nov. 2008	: Faster blurring and resampling
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#define _USE_MATH_DEFINES
#include ".\bitmapfile.h"
#include <windows.h>	// The Windows PlatformSDK has to be installed in order to compile PlugIns
#include "utils.h"
#include <d3dx9.h>
#include "../../Includes/CIPnPTCTexture.h"
#include "IL/ilu.h"
#include <vector>
#include <math.h>


// used for faster pixel access
#define USE_GET_DATA_FOR_GETPIXEL


CBitmapFile::CBitmapFile(void)
{
	// initialize IL and ILU
	ilInit();
	iluInit();

	// create devil image name for this bitmap
	ilGenImages(1,&this->m_ImageName);

	// initializations
	this->m_nSizeX=0;
	this->m_nSizeY=0;
}

CBitmapFile::~CBitmapFile(void)
{
	// deinitialize the bitmap object
	this->DeInitialize();

	// destroy the image
	ilDeleteImages(1,&this->m_ImageName);
}

void CBitmapFile::DeInitialize()
{
	// set all parameters to zero
	this->m_nSizeX=0;
	this->m_nSizeY=0;

	// preserve image mem
	ilBindImage(this->m_ImageName);
	if (ilTexImage(0,0,1,1,IL_COLOUR_INDEX,IL_UNSIGNED_BYTE,NULL)==0)
	{
		int i=0;
	}
}

bool CBitmapFile::Initialize(BITMAPFORMAT iFormat,unsigned int nSizeX,unsigned int nSizeY)
{
	// free old mem first
	this->m_nSizeX=0;
	this->m_nSizeY=0;

	// initialize format parameters
	unsigned int nBytesPerPixel=3;
	this->m_Format=IL_RGB;
	switch (iFormat)
	{
	case bmfPalette:
		nBytesPerPixel=1;
		this->m_Format=IL_COLOUR_INDEX;
		break;
	case bmfRGB:
		nBytesPerPixel=3;
		this->m_Format=IL_RGB;
		break;
	case bmfRGBA:
		nBytesPerPixel=4;
		this->m_Format=IL_RGBA;
		break;
	default:
		// error
		MessageBox(NULL,"Unsupported bitmap format.","Error initializing bitmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// set image attributes
	ilBindImage(this->m_ImageName);
	if (ilTexImage(nSizeX,nSizeY,1,nBytesPerPixel,this->m_Format,IL_UNSIGNED_BYTE,NULL)!=0)
	{
		// set palletized format
		if (iFormat==bmfPalette)
		{
			ilSetInteger(IL_PALETTE_BPP,3);
			ilSetInteger(IL_PALETTE_NUM_COLS,256);
			ilSetInteger(IL_PALETTE_TYPE,IL_PAL_RGB24);
			this->CreateGrayScalePalette();
		}

		// store size
		this->m_nSizeX=nSizeX;
		this->m_nSizeY=nSizeY;
		return true;
	}
	else
	{
		return false;
	}
}

bool CBitmapFile::Initialize(CIPnPTCTexture01* pTexture,BITMAPFORMAT iFormat/*=bmfRGB*/)
{
	// initialize bitmap
	if (!pTexture || !this->Initialize(iFormat,pTexture->GetWidth(),pTexture->GetHeight()))
	{
		return false;
	}

	// go through whole texture
	for (int iPixY=0;iPixY<pTexture->GetHeight();iPixY++)
	{
		for (int iPixX=0;iPixX<pTexture->GetWidth();iPixX++)
		{
			// get pixel color
			PNPTC_RGB rgb=pTexture->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);

			// write pixel
			this->SetPixel(iPixX,iPixY,rgb);
		}
	}

	// ok
	return true;
}

bool CBitmapFile::IsInitialized() const
{
	// check if initialized
	return this->m_nSizeX>0 && this->m_nSizeY>0;
}

BITMAPFORMAT CBitmapFile::GetBitmapFormat() const
{
	// get and return bitmap format
	switch (this->m_Format)
	{
	case IL_COLOUR_INDEX:
		return bmfPalette;
		break;
	case IL_RGB:
	case IL_BGR:
		return bmfRGB;
		break;
	case IL_RGBA:
	case IL_BGRA:
		return bmfRGBA;
		break;
	default:
		return bmfRGB;
		break;
	}
}

void CBitmapFile::CreateGrayScalePalette()
{
	// bin image for which a palette should be created
	ilBindImage(this->m_ImageName);

	// create a palette
	PNPTC_RGB rgb[256];
	for (int i=0;i<=255;i++)
	{
		rgb[i].r=i;
		rgb[i].g=i;
		rgb[i].b=i;
	}

	// register this palette
	ilRegisterPal(&rgb,sizeof(PNPTC_RGB)*256,IL_PAL_RGB24);
}

void* CBitmapFile::GetPixelDataPtr(unsigned int nX,unsigned int nY)
{
	// get data of image
	ilBindImage(this->m_ImageName);
	void* pData = ilGetData();

	// compute pixel size
	unsigned int nPixelSize=1;
	switch (this->m_Format)
	{
	case IL_RGB:
	case IL_BGR:
		nPixelSize=3;
		break;
	case IL_RGBA:
	case IL_BGRA:
		nPixelSize=4;
		break;
	}

	// compute offset
	unsigned int nOffset=(nX+nY*this->GetSizeX())*nPixelSize;

	// return data pointer
	return &(((char*)pData)[nOffset]);
}

const void* CBitmapFile::GetPixelDataPtr(unsigned int nX,unsigned int nY) const
{
	// get data of image
	ilBindImage(this->m_ImageName);
	void* pData = ilGetData();

	// compute pixel size
	unsigned int nPixelSize=1;
	switch (this->m_Format)
	{
	case IL_RGB:
	case IL_BGR:
		nPixelSize=3;
		break;
	case IL_RGBA:
	case IL_BGRA:
		nPixelSize=4;
		break;
	}

	// compute offset
	unsigned int nOffset=(nX+nY*this->GetSizeX())*nPixelSize;

	// return data pointer
	return &(((char*)pData)[nOffset]);
}

void CBitmapFile::SetPixel(unsigned int nX,unsigned int nY,BYTE byValue)
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format
		if (this->m_Format==IL_COLOUR_INDEX)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			*(BYTE*)this->GetPixelDataPtr(nX,nY)=byValue;
#else
			// set pixel
			ilBindImage(this->m_ImageName);
			ilSetPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,&byValue);
#endif
		}
	}
}

void CBitmapFile::SetPixel(unsigned int nX,unsigned int nY,const PNPTC_RGB& rgb)
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format		
		if (this->m_Format==IL_RGB)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(this->GetPixelDataPtr(nX,nY),&rgb,3);
#else
			// set pixel
			ilBindImage(this->m_ImageName);
			ilSetPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&rgb);
#endif
		}
		else if (this->m_Format==IL_BGR)
		{
			// convert pixel format
			PNPTC_BGR bgrTemp(rgb);
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(this->GetPixelDataPtr(nX,nY),&bgrTemp,3);
#else
			// set pixel
			ilBindImage(this->m_ImageName);
			ilSetPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&bgrTemp);
#endif
		}
		else if (this->m_Format==IL_RGBA || this->m_Format==IL_BGRA)
		{
			// use rgba method instead?
			this->SetPixel(nX,nY,PNPTC_RGBA(rgb));
		}
	}
}

void CBitmapFile::SetPixel(unsigned int nX,unsigned int nY,const PNPTC_RGBA& rgba)
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format		
		if (this->m_Format==IL_RGBA)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(this->GetPixelDataPtr(nX,nY),&rgba,4);
#else
			// set pixel
			ilBindImage(this->m_ImageName);
			ilSetPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&rgba);
#endif
		}
		else if (this->m_Format==IL_BGRA)
		{
			// convert pixel format
			PNPTC_BGRA bgraTemp(rgba);
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(this->GetPixelDataPtr(nX,nY),&bgraTemp,4);
#else
			// set pixel
			ilBindImage(this->m_ImageName);
			ilSetPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&bgraTemp);
#endif
		}
		else if (this->m_Format==IL_RGB || this->m_Format==IL_BGR)
		{
			// use rgba method instead?
			this->SetPixel(nX,nY,PNPTC_RGB(rgba.r,rgba.g,rgba.b));
		}
	}
}

void CBitmapFile::GetPixel(unsigned int nX,unsigned int nY,BYTE& byValue) const
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format
		if (this->m_Format==IL_COLOUR_INDEX)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			byValue=*(BYTE*)this->GetPixelDataPtr(nX,nY);
#else
			// get pixel
			ilBindImage(this->m_ImageName);
			ilCopyPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,&byValue);
#endif
		}
	}
}

void CBitmapFile::GetPixel(unsigned int nX,unsigned int nY,PNPTC_RGB& rgb) const
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format		
		if (this->m_Format==IL_RGB)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(&rgb,this->GetPixelDataPtr(nX,nY),3);
#else
			// get pixel
			ilBindImage(this->m_ImageName);
			ilCopyPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&rgb);
#endif
		}
		else if (this->m_Format==IL_BGR)
		{
			// convert format
			PNPTC_BGR bgrTemp;
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(&bgrTemp,this->GetPixelDataPtr(nX,nY),3);
#else
			// get pixel
			ilBindImage(this->m_ImageName);
			ilCopyPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&bgrTemp);
#endif
			// return pixel
			rgb=PNPTC_RGB(bgrTemp);
		}
		else if (this->m_Format==IL_BGRA || this->m_Format==IL_RGBA)
		{
			// use rgba method instead?
			PNPTC_RGBA col;
			this->GetPixel(nX,nY,col);
			rgb=col;
		}
	}
}

void CBitmapFile::GetPixel(unsigned int nX,unsigned int nY,PNPTC_RGBA& rgba) const
{
	// check range
	if (nX<this->m_nSizeX && nY<this->m_nSizeY)
	{
		// check format		
		if (this->m_Format==IL_RGBA)
		{
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(&rgba,this->GetPixelDataPtr(nX,nY),4);
#else
			// get pixel
			ilBindImage(this->m_ImageName);
			ilCopyPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&rgba);
#endif
		}
		else if (this->m_Format==IL_BGRA)
		{
			// convert format
			PNPTC_BGRA bgraTemp;
#if defined(USE_GET_DATA_FOR_GETPIXEL)
			// copy pixel data
			memcpy(&bgraTemp,this->GetPixelDataPtr(nX,nY),4);
#else
			// get pixel
			ilBindImage(this->m_ImageName);
			ilCopyPixels(nX,nY,0,1,1,1,this->m_Format,IL_UNSIGNED_BYTE,(ILvoid*)&bgraTemp);
#endif
			// return pixel
			rgba=PNPTC_RGBA(bgraTemp);
		}
		else if (this->m_Format==IL_RGB || this->m_Format==IL_BGR)
		{
			// use rgb method instead?
			PNPTC_RGB col;
			this->GetPixel(nX,nY,col);
			rgba=col;
		}
	}
}

bool CBitmapFile::Read(const std::string& strSourceFile)
{
	// try to load bitmap
	ilBindImage(this->m_ImageName);
	if (ilLoadImage(strSourceFile.c_str())!=0)
	{
		// get size and format
		this->m_nSizeX=ilGetInteger(IL_IMAGE_WIDTH);
		this->m_nSizeY=ilGetInteger(IL_IMAGE_HEIGHT);
		this->m_Format=ilGetInteger(IL_IMAGE_FORMAT);

		// ok
		return true;
	}
	else
	{
		// error 
		return false;
	}
}

/// Structure for one contributing pixel
typedef struct Contributor
{
	int							iPixel;			//!< the pixel coordinate.
	double						dWeight;		//!< the contributing weight.
} Contributor;

/// Structure for storing all pixels contributing to one resampled pixel
typedef struct ContributorEntry 
{
    int							iPixels;		//!< the number of pixels contributing to this entry
	std::vector<Contributor>	vecPixels;		//!< the list of all pixels contributing to this entry
    double						dWeightedSum;	//!< the weighted sum over all contributing pixels
} ContributorEntry;


bool CBitmapFile::Resample(unsigned int nSizeX,unsigned int nSizeY,ResampleFilter Filter)
{
	// check input
	if (this->GetSizeX()<1 || this->GetSizeY()<1 || nSizeX<1 || nSizeY<1)
	{
		return false;
	}

	// create the resampling filter
	CResamplingFilter* pFilter = CResamplingFilter::CreateResamplingFilter(Filter);
	if (!pFilter)
	{
		return false;
	}

	// create working bitmaps
	CBitmapFile bmpWork;
	CBitmapFile bmpOutput;
	if (!bmpWork.Initialize(this->GetBitmapFormat(),nSizeX,this->GetSizeY()) ||
		!bmpOutput.Initialize(this->GetBitmapFormat(),nSizeX,nSizeY))
	{
		return false;
	}

	// compute scales
	double dScaleX = static_cast<double>(nSizeX) / static_cast<double>(this->GetSizeX());
	double dScaleY = static_cast<double>(nSizeY) / static_cast<double>(this->GetSizeY());

	// -----------------------------------------------------------------------------
	// list of all contributing values
	std::vector<ContributorEntry> vecContributors;
	vecContributors.resize(nSizeX);
	
	// horizontal downsampling
	if (dScaleX < 1.0) 
	{
		// scales from bigger to smaller width
		double dFilterWidth = (pFilter->GetDefaultFilterRadius() / dScaleX);
		for (int i = 0; i < static_cast<int>(nSizeX); ++i)
		{
			vecContributors[i].iPixels = 0;
			vecContributors[i].vecPixels.resize(static_cast<int>(floor(2. * dFilterWidth + 1.)));
			vecContributors[i].dWeightedSum = 0;
			double dCenter = ((static_cast<double>(i) + 0.5) / dScaleX);
			int iLeft = static_cast<int>(dCenter - dFilterWidth);
			int iRight = static_cast<int>(dCenter + dFilterWidth);
			for (int j = iLeft; j <= iRight; ++j)
			{
				double dWeight = pFilter->GetValue((dCenter - static_cast<double>(j) - 0.5) * dScaleX);
				if ((dWeight == 0) || (j < 0) || (j >= static_cast<int>(this->GetSizeX())))
				{
					continue;
				}
				vecContributors[i].vecPixels[vecContributors[i].iPixels].iPixel = j;
				vecContributors[i].vecPixels[vecContributors[i].iPixels].dWeight = dWeight;
				vecContributors[i].dWeightedSum += dWeight;
				vecContributors[i].iPixels++;
		    }
		}
	}
	else 
	{
		// horizontal upsampling
		// scales from smaller to bigger width
		for (int i = 0; i < static_cast<int>(nSizeX); ++i)
		{
			vecContributors[i].iPixels = 0;
			vecContributors[i].vecPixels.resize(static_cast<int>(floor(2. * pFilter->GetDefaultFilterRadius() + 1.)));
			vecContributors[i].dWeightedSum = 0;
			double dCenter = ((static_cast<double>(i) + 0.5) / dScaleX);
			int iLeft = static_cast<int>(floor(dCenter - pFilter->GetDefaultFilterRadius()));
			int iRight = static_cast<int>(ceil(dCenter + pFilter->GetDefaultFilterRadius()));
			for (int j = iLeft; j <= iRight; ++j)
			{
				double dWeight = pFilter->GetValue(dCenter - static_cast<double>(j) - 0.5);
				if ((dWeight == 0) || (j < 0) || (j >= static_cast<int>(this->GetSizeX())))
				{
					continue;
				}
				vecContributors[i].vecPixels[vecContributors[i].iPixels].iPixel = j;
				vecContributors[i].vecPixels[vecContributors[i].iPixels].dWeight = dWeight;
				vecContributors[i].dWeightedSum += dWeight;
				vecContributors[i].iPixels++;          
			}
		}
	}

	// -----------------------------------------------------------------------------
	// filter horizontally from input to work
	for (int iY = 0; iY < static_cast<int>(this->GetSizeY()); ++iY) 
	{
		for (int iX = 0; iX < static_cast<int>(nSizeX); ++iX) 
		{
			double dIntensity[4] = {0,0,0,0};
			for (int j = 0; j < vecContributors[iX].iPixels; ++j) 
			{
				// compute weight
				double dWeight = vecContributors[iX].vecPixels[j].dWeight;
				if (dWeight == 0)
				{
					continue;
				}

				// process pixel data
				if (this->GetBitmapFormat()==bmfPalette)
				{
					BYTE byValue;
					this->GetPixel(vecContributors[iX].vecPixels[j].iPixel,iY,byValue);
					dIntensity[0]+=static_cast<double>(byValue)*dWeight;
				}
				else
				{
					PNPTC_RGBA rgbaValue;
					this->GetPixel(vecContributors[iX].vecPixels[j].iPixel,iY,rgbaValue);
					dIntensity[0]+=static_cast<double>(rgbaValue.r)*dWeight;
					dIntensity[1]+=static_cast<double>(rgbaValue.g)*dWeight;
					dIntensity[2]+=static_cast<double>(rgbaValue.b)*dWeight;
					dIntensity[3]+=static_cast<double>(rgbaValue.a)*dWeight;
				}
			}

			// set pixel data
			if (this->GetBitmapFormat()==bmfPalette)
			{
				BYTE byValue=minmax(static_cast<int>(dIntensity[0]/vecContributors[iX].dWeightedSum),0,255);
				bmpWork.SetPixel(iX,iY,byValue);
			}
			else
			{
				PNPTC_RGBA rgbaValue;
				rgbaValue.r=minmax(static_cast<int>(dIntensity[0]/vecContributors[iX].dWeightedSum),0,255);
				rgbaValue.g=minmax(static_cast<int>(dIntensity[1]/vecContributors[iX].dWeightedSum),0,255);
				rgbaValue.b=minmax(static_cast<int>(dIntensity[2]/vecContributors[iX].dWeightedSum),0,255);
				rgbaValue.a=minmax(static_cast<int>(dIntensity[3]/vecContributors[iX].dWeightedSum),0,255);
				bmpWork.SetPixel(iX,iY,rgbaValue);
			}
		}
	}

	// -----------------------------------------------------------------------------
	// pre-calculate filter contributions for a column
	vecContributors.resize(nSizeY);

	// vertical downsampling
	if (dScaleY < 1.0) 
	{
		// scales from bigger to smaller height
		double dFilterWidth = (pFilter->GetDefaultFilterRadius() / dScaleY);
		for (int i = 0; i < static_cast<int>(nSizeY); ++i) 
		{
			vecContributors[i].iPixels = 0;
			vecContributors[i].vecPixels.resize(static_cast<int>(floor(2. * dFilterWidth + 1.)));
			vecContributors[i].dWeightedSum = 0;
			double dCenter = ((static_cast<double>(i) + 0.5) / dScaleY);
			int iLeft = static_cast<int>(dCenter - dFilterWidth);
			int iRight = static_cast<int>(dCenter + dFilterWidth);
			for (int j = iLeft; j <= iRight; ++j)
			{
				double dWeight = pFilter->GetValue((dCenter - static_cast<double>(j) - 0.5) * dScaleY);
				if ((dWeight == 0) || (j < 0) || (j >= static_cast<int>(this->GetSizeY())))
				{
					continue;
				}
				vecContributors[i].vecPixels[vecContributors[i].iPixels].iPixel = j;
				vecContributors[i].vecPixels[vecContributors[i].iPixels].dWeight = dWeight;
				vecContributors[i].dWeightedSum += dWeight;
				vecContributors[i].iPixels++;
		    }
		}
	} 
	else 
	{
		// vertical upsampling
		// scales from smaller to bigger height
		for (int i = 0; i < static_cast<int>(nSizeY); ++i)
		{
			vecContributors[i].iPixels = 0;
			vecContributors[i].vecPixels.resize(static_cast<int>(floor(2. * pFilter->GetDefaultFilterRadius() + 1.)));
			vecContributors[i].dWeightedSum = 0;
			double dCenter = ((static_cast<double>(i) + 0.5) / dScaleY);
			int iLeft = static_cast<int>(dCenter - pFilter->GetDefaultFilterRadius());
			int iRight = static_cast<int>(dCenter + pFilter->GetDefaultFilterRadius());
			for (int j = iLeft; j <= iRight; ++j)
			{
				double dWeight = pFilter->GetValue((dCenter - static_cast<double>(j) - 0.5));
				if ((dWeight == 0) || (j < 0) || (j >= static_cast<int>(this->GetSizeY())))
				{
					continue;
				}
				vecContributors[i].vecPixels[vecContributors[i].iPixels].iPixel = j;
				vecContributors[i].vecPixels[vecContributors[i].iPixels].dWeight = dWeight;
				vecContributors[i].dWeightedSum += dWeight;
				vecContributors[i].iPixels++;
			}
		}
	}

	// -----------------------------------------------------------------------------
	// filter vertically from work to output
	for (int iX = 0; iX < static_cast<int>(nSizeX); ++iX) 
	{
		for (int iY = 0; iY < static_cast<int>(nSizeY); ++iY) 
		{
			double dIntensity[4] = {0,0,0,0};
			for (int j = 0; j < vecContributors[iY].iPixels; ++j) 
			{
				// compute weight
				double dWeight = vecContributors[iY].vecPixels[j].dWeight;
				if (dWeight == 0)
				{
					continue;
				}

				// process pixel data
				if (this->GetBitmapFormat()==bmfPalette)
				{
					BYTE byValue;
					bmpWork.GetPixel(iX,vecContributors[iY].vecPixels[j].iPixel,byValue);
					dIntensity[0]+=static_cast<double>(byValue)*dWeight;
				}
				else
				{
					PNPTC_RGBA rgbaValue;
					bmpWork.GetPixel(iX,vecContributors[iY].vecPixels[j].iPixel,rgbaValue);
					dIntensity[0]+=static_cast<double>(rgbaValue.r)*dWeight;
					dIntensity[1]+=static_cast<double>(rgbaValue.g)*dWeight;
					dIntensity[2]+=static_cast<double>(rgbaValue.b)*dWeight;
					dIntensity[3]+=static_cast<double>(rgbaValue.a)*dWeight;
				}
			}

			// set pixel data
			if (this->GetBitmapFormat()==bmfPalette)
			{
				BYTE byValue=minmax(static_cast<int>(dIntensity[0]/vecContributors[iY].dWeightedSum),0,255);
				bmpOutput.SetPixel(iX,iY,byValue);
			}
			else
			{
				PNPTC_RGBA rgbaValue;
				rgbaValue.r=minmax(static_cast<int>(dIntensity[0]/vecContributors[iY].dWeightedSum),0,255);
				rgbaValue.g=minmax(static_cast<int>(dIntensity[1]/vecContributors[iY].dWeightedSum),0,255);
				rgbaValue.b=minmax(static_cast<int>(dIntensity[2]/vecContributors[iY].dWeightedSum),0,255);
				rgbaValue.a=minmax(static_cast<int>(dIntensity[3]/vecContributors[iY].dWeightedSum),0,255);
				bmpOutput.SetPixel(iX,iY,rgbaValue);
			}
		}
	}

	// delete filter
	SAFE_DELETE(pFilter);

	// copy final bitmap
	if (this->Initialize(this->GetBitmapFormat(),nSizeX,nSizeY))
	{
		this->FillFromBitmap(bmpOutput);
		return true;
	}
	else
	{
		return false;
	}
}

bool CBitmapFile::Resize(unsigned int nSizeX,unsigned int nSizeY,ResampleFilter Filter/*=ResampleFilter::Mitchell*/)
{
	// test if size is changed
	if (nSizeX==this->m_nSizeX && nSizeY==this->m_nSizeY)
	{
		return true;
	}

	// resize the image
	ilBindImage(this->m_ImageName);
	if (Filter!=None)
	{
		// DevIL (version 1.7.2) resize has some problems, that's why the scaling is re-implemented here
		return this->Resample(nSizeX,nSizeY,Filter);
//		iluImageParameter(ILU_FILTER,ILU_SCALE_LANCZOS3);
//		iluImageParameter(ILU_FILTER,ILU_BILINEAR);
	}
	else
	{
		iluImageParameter(ILU_FILTER,ILU_NEAREST);
	}
	if (iluScale(nSizeX,nSizeY,1)!=0)
	{
		this->m_nSizeX=nSizeX;
		this->m_nSizeY=nSizeY;
		return true;
	}
	else
	{
		this->m_nSizeX=0;
		this->m_nSizeY=0;
		return false;
	}
}

bool CBitmapFile::Write(const std::string& strDestFile,const std::string& strFileFormat/*="BMP"*/) const
{
	// get image format
	ILenum nType=IL_BMP;
	if (EqualNoCase(strFileFormat,"jpg"))
	{
		nType=IL_JPG;
	}
	else if (EqualNoCase(strFileFormat,"png"))
	{
		nType=IL_PNG;
	}
	else if (EqualNoCase(strFileFormat,"dds"))
	{
		nType=IL_DDS;
	}
	else if (EqualNoCase(strFileFormat,"pnm"))
	{
		nType=IL_PNM;
	}
	else if (EqualNoCase(strFileFormat,"raw"))
	{
		nType=IL_RAW;
	}
	else if (EqualNoCase(strFileFormat,"sgi"))
	{
		nType=IL_SGI;
	}
	else if (EqualNoCase(strFileFormat,"tga"))
	{
		nType=IL_TGA;
	}
	else if (EqualNoCase(strFileFormat,"tif"))
	{
		nType=IL_TIF;
	}
	else if (EqualNoCase(strFileFormat,"bmp"))
	{
		nType=IL_BMP;
	}
	else if (EqualNoCase(strFileFormat,"pcx"))
	{
		nType=IL_PCX;
	}
	else if (EqualNoCase(strFileFormat,"hdr"))
	{
		nType=IL_HDR;
	}
	else
	{
		// unsupported format
		return false;
	}

	// save image
	ilBindImage(this->m_ImageName);
	ilEnable(IL_FILE_OVERWRITE);
	return (ilSave(nType,(char*)strDestFile.c_str())!=0);
}

template <class T>
void SwapPixels(CBitmapFile& bmp,unsigned int nX1,unsigned int nY1,unsigned int nX2,unsigned int nY2)
{
	// swap both pixel values
	T t1,t2;
	bmp.GetPixel(nX1,nY1,t1);
	bmp.GetPixel(nX2,nY2,t2);
	bmp.SetPixel(nX1,nY1,t2);
	bmp.SetPixel(nX2,nY2,t1);
}

void CBitmapFile::SwapPixels(unsigned int nX1,unsigned int nY1,unsigned int nX2,unsigned int nY2)
{
	// swap the pixel values 
	switch (this->m_Format)
	{
	case IL_RGB:
	case IL_BGR:
		::SwapPixels<PNPTC_RGB>(*this,nX1,nY1,nX2,nY2);
		break;
	case IL_RGBA:
	case IL_BGRA:
		::SwapPixels<PNPTC_RGBA>(*this,nX1,nY1,nX2,nY2);
		break;
	case IL_COLOUR_INDEX:
		::SwapPixels<BYTE>(*this,nX1,nY1,nX2,nY2);
		break;
	}
}

void CBitmapFile::FlipY()
{
	// go through all image rows
	for (int iRow1=0,iRow2=static_cast<int>(this->m_nSizeY)-1;iRow2>=0 && iRow1<static_cast<int>(this->m_nSizeY) && iRow1<iRow2;iRow1++,iRow2--)
	{
		// go through all pixels of that row
		for (unsigned int nCol=0;nCol<this->m_nSizeX;nCol++)
		{
			this->SwapPixels(nCol,iRow1,nCol,iRow2);
		}
	}
}

void CBitmapFile::FlipX()
{
	// go through all image columns
	for (int iCol1=0,iCol2=static_cast<int>(this->m_nSizeX)-1;iCol2>=0 && iCol1<static_cast<int>(this->m_nSizeX) && iCol1<iCol2;iCol1++,iCol2--)
	{
		// go through all pixels of that row
		for (unsigned int nRow=0;nRow<this->m_nSizeY;nRow++)
		{
			this->SwapPixels(iCol1,nRow,iCol2,nRow);
		}
	}
}

void CBitmapFile::FlipDiagonal(bool bTopLeft2BottomRight)
{
	// compute aspect ration
	float fAR=static_cast<float>(this->GetSizeX())/static_cast<float>(this->GetSizeY());

	// go through the whole image
	for (unsigned int nY=0;nY<this->GetSizeY();nY++)
	{
		for (unsigned int nX=0;nX<this->GetSizeX();nX++)
		{
			if ((!bTopLeft2BottomRight && nX>static_cast<unsigned int>(static_cast<float>(nY)*fAR)) ||
				(bTopLeft2BottomRight && nX>static_cast<unsigned int>(static_cast<float>(this->GetSizeY()-nY-1)*fAR)))
			{
				// get the pixel coordinate with which to swap the values
				unsigned int nX2=minmax(nY,static_cast<unsigned int>(0),this->GetSizeX()-1);
				unsigned int nY2=minmax(nX,static_cast<unsigned int>(0),this->GetSizeY()-1);
				if (bTopLeft2BottomRight)
				{
					nX2=minmax(this->GetSizeX()-nY-1,static_cast<unsigned int>(0),this->GetSizeX()-1);
					nY2=minmax(this->GetSizeY()-nX-1,static_cast<unsigned int>(0),this->GetSizeY()-1);
				}

				// copy the pixels
				this->SwapPixels(nX,nY,nX2,nY2);
			}
		}
	}
}

void CBitmapFile::Clear()
{
	// go through whole bitmap
	for (unsigned int nY=0;nY<this->GetSizeY();++nY)
	{
		for (unsigned int nX=0;nX<this->GetSizeX();++nX)
		{
			// set pixel
			if (this->m_Format==IL_COLOUR_INDEX)
			{
				this->SetPixel(nX,nY,0);
			}
			else
			{
				this->SetPixel(nX,nY,RGB(0,0,0));
			}
		}
	}
}

bool CBitmapFile::BlurGaussian(int iSmoothFactor,bool bUseMax/*=false*/)
{
	// check smooth factor
	if (iSmoothFactor<=0) return true;

	// create kernel
	int iKernelSize=iSmoothFactor*2+1;
	float* fFilterKernel=new float[iKernelSize];
	float fKernelDivisor=0;
	if (fFilterKernel)
	{
		// fill kernel
		float fStdDeviation=static_cast<float>(iSmoothFactor)*2.f/3.f;
		float fFactor=1.f/(sqrt(2.f*static_cast<float>(M_PI))*fStdDeviation);
		float fStdDevSqr=fStdDeviation*fStdDeviation;
		float fInvStdDevSqr=1.f/fStdDevSqr;
		for (int i=-iSmoothFactor;i<=iSmoothFactor;++i)
		{
			float fValue=fFactor*exp(-0.5f*((static_cast<float>(i*i))*fInvStdDevSqr));
			fKernelDivisor+=fValue;
			fFilterKernel[i+iSmoothFactor]=fValue;
		}
	}

	// check if maximum is used
	if (fFilterKernel && iSmoothFactor!=0 && bUseMax)
	{
		fKernelDivisor=fFilterKernel[iSmoothFactor];
	}

	// check filter kernel
	if (fFilterKernel==NULL || fKernelDivisor==0)
	{
		SAFE_DELETEARRAY(fFilterKernel);
		return false;
	}
	float fInvKernelDivisor=1.f/fKernelDivisor;

	// create temporary bitmap
	CBitmapFile newBitmap;
	newBitmap.Initialize(this->GetBitmapFormat(),this->GetSizeX(),this->GetSizeY());

	// the maximum range values
	int iMaxSourceX=static_cast<int>(this->GetSizeX())-1;
	int iMaxSourceY=static_cast<int>(this->GetSizeY())-1;

	// temporary variables
	BYTE byValue;
	PNPTC_RGBA rgba;
	float fSum[4];

	// smooth it
	if (iSmoothFactor>0 && iKernelSize>0 && fFilterKernel)
	{
		// blur it in x direction
		for (int iY=0;iY<static_cast<int>(this->GetSizeY());++iY)
		{
			for (int iX=0;iX<static_cast<int>(this->GetSizeX());++iX)
			{
				// compute filter sum				
				fSum[0]=0;fSum[1]=0;fSum[2]=0;fSum[3]=0;
				for (int iKernel=-iSmoothFactor;iKernel<=iSmoothFactor;++iKernel)
				{
					int iSourceX=minmax(iX+iKernel,0,iMaxSourceX);
					float fKernel=fFilterKernel[iKernel+iSmoothFactor];
					if (this->m_Format==IL_COLOUR_INDEX)
					{							
						this->GetPixel(iSourceX,iY,byValue);
						if (bUseMax)
						{
							fSum[0]=ceil(maximum(fSum[0],fKernel*static_cast<float>(byValue)));
						}
						else
						{
							fSum[0]+=fKernel*static_cast<float>(byValue);
						}
					}
					else
					{							
						this->GetPixel(iSourceX,iY,rgba);
						if (bUseMax)
						{
							fSum[0]=ceil(maximum(fSum[0],fKernel*static_cast<float>(rgba.r)));
							fSum[1]=ceil(maximum(fSum[1],fKernel*static_cast<float>(rgba.g)));
							fSum[2]=ceil(maximum(fSum[2],fKernel*static_cast<float>(rgba.b)));
							fSum[3]=ceil(maximum(fSum[3],fKernel*static_cast<float>(rgba.a)));
						}
						else
						{
							fSum[0]+=fKernel*static_cast<float>(rgba.r);
							fSum[1]+=fKernel*static_cast<float>(rgba.g);
							fSum[2]+=fKernel*static_cast<float>(rgba.b);
							fSum[3]+=fKernel*static_cast<float>(rgba.a);
						}
					}
				} 

				// set pixel value   
				if (this->m_Format==IL_COLOUR_INDEX)
				{
					newBitmap.SetPixel(iX,iY,static_cast<BYTE>(minmax(fSum[0]*fInvKernelDivisor,0.f,255.f)));
				}
				else
				{
					rgba.r = static_cast<BYTE>(minmax(fSum[0]*fInvKernelDivisor,0.f,255.f));
					rgba.g = static_cast<BYTE>(minmax(fSum[1]*fInvKernelDivisor,0.f,255.f));
					rgba.b = static_cast<BYTE>(minmax(fSum[2]*fInvKernelDivisor,0.f,255.f));
					rgba.a = static_cast<BYTE>(minmax(fSum[3]*fInvKernelDivisor,0.f,255.f));
					newBitmap.SetPixel(iX,iY,rgba);
				}
			}
		} 

		// copy bitmap
		this->FillFromBitmap(newBitmap);

		// blur it in y direction
		for (int iX=0;iX<static_cast<int>(this->GetSizeX());++iX)
		{
			for (int iY=0;iY<static_cast<int>(this->GetSizeY());++iY)
			{
				// compute filter sum				
				fSum[0]=0;fSum[1]=0;fSum[2]=0;fSum[3]=0;
				for (int iKernel=-iSmoothFactor;iKernel<=iSmoothFactor;++iKernel)
				{
					int iSourceY=minmax(iY+iKernel,0,iMaxSourceY);
					float fKernel=fFilterKernel[iKernel+iSmoothFactor];
					if (this->m_Format==IL_COLOUR_INDEX)
					{							
						newBitmap.GetPixel(iX,iSourceY,byValue);
						if (bUseMax)
						{
							fSum[0]=ceil(maximum(fSum[0],fKernel*static_cast<float>(byValue)));
						}
						else
						{
							fSum[0]+=fKernel*static_cast<float>(byValue);
						}
					}
					else
					{							
						newBitmap.GetPixel(iX,iSourceY,rgba);
						if (bUseMax)
						{
							fSum[0]=ceil(maximum(fSum[0],fKernel*static_cast<float>(rgba.r)));
							fSum[1]=ceil(maximum(fSum[1],fKernel*static_cast<float>(rgba.g)));
							fSum[2]=ceil(maximum(fSum[2],fKernel*static_cast<float>(rgba.b)));
							fSum[3]=ceil(maximum(fSum[3],fKernel*static_cast<float>(rgba.a)));
						}
						else
						{
							fSum[0]+=fKernel*static_cast<float>(rgba.r);
							fSum[1]+=fKernel*static_cast<float>(rgba.g);
							fSum[2]+=fKernel*static_cast<float>(rgba.b);
							fSum[3]+=fKernel*static_cast<float>(rgba.a);
						}
					}
				} 

				// set pixel value   
				if (this->m_Format==IL_COLOUR_INDEX)
				{
					this->SetPixel(iX,iY,static_cast<BYTE>(minmax(fSum[0]*fInvKernelDivisor,0.f,255.f)));
				}
				else
				{
					rgba.r = static_cast<BYTE>(minmax(fSum[0]*fInvKernelDivisor,0.f,255.f));
					rgba.g = static_cast<BYTE>(minmax(fSum[1]*fInvKernelDivisor,0.f,255.f));
					rgba.b = static_cast<BYTE>(minmax(fSum[2]*fInvKernelDivisor,0.f,255.f));
					rgba.a = static_cast<BYTE>(minmax(fSum[3]*fInvKernelDivisor,0.f,255.f));
					this->SetPixel(iX,iY,rgba);
				}
			}
		} 
	}

	// clean up
	SAFE_DELETEARRAY(fFilterKernel);

	// ok
	return true;
}

bool CBitmapFile::FillFromBitmap(const CBitmapFile& bmpSource,int iRemoveBorder/*=0*/)
{
	// check size
	if ((bmpSource.GetSizeX()-iRemoveBorder*2)!=this->GetSizeX())
	{
		return false;
	}
	if ((bmpSource.GetSizeY()-iRemoveBorder*2)!=this->GetSizeY())
	{
		return false;
	}

	// copy bitmap
	for (unsigned int nY=0;nY<this->m_nSizeY;++nY)
	{
		for (unsigned int nX=0;nX<this->m_nSizeX;++nX)
		{
			// copy pixel
			if (this->m_Format==IL_COLOUR_INDEX)
			{
				BYTE byValue;
				bmpSource.GetPixel(nX+iRemoveBorder,nY+iRemoveBorder,byValue);
				this->SetPixel(nX,nY,byValue);
			}
			else
			{
				PNPTC_RGBA rgba;
				bmpSource.GetPixel(nX+iRemoveBorder,nY+iRemoveBorder,rgba);
				this->SetPixel(nX,nY,rgba);
			}
		}
	}

	// ok
	return true;
}

bool CBitmapFile::RemoveBorder(int iBorderLeft,int iBorderTop,int iBorderRight/*=-1*/,int iBorderBottom/*=-1*/)
{
	// check parameters
	if (iBorderLeft<0 || iBorderTop<0)		
	{
		return false;
	}

	// get right and bottom border values
	if (iBorderRight<0)
	{
		iBorderRight=iBorderLeft;
	}
	if (iBorderBottom<0)
	{
		iBorderBottom=iBorderTop;
	}

	// anything to be removed
	if (iBorderLeft == 0 && iBorderTop == 0 &&
		iBorderRight == 0 && iBorderBottom == 0)
	{
		return true;
	}

	// compute new size of this image
	int iNewWidth=static_cast<int>(this->GetSizeX())-iBorderLeft-iBorderRight;
	int iNewHeight=static_cast<int>(this->GetSizeY())-iBorderTop-iBorderBottom;
	if (iNewWidth<=0 || iNewHeight<=0)
	{
		return false;
	}

	// create a copy of that bitmap
	CBitmapFile bmpTemp;
	if (!bmpTemp.Initialize(this->GetBitmapFormat(),this->GetSizeX(),this->GetSizeY()))
	{
		return false;
	}
	if (!bmpTemp.FillFromBitmap(*this))
	{
		return false;
	}

	// set new size of this image
	if (!this->Initialize(this->GetBitmapFormat(),iNewWidth,iNewHeight))
	{
		return false;
	}

	// temp values
	BYTE byValue;
	PNPTC_RGBA rgbaValue;

	// copy bitmap data
	bool bPalettized=(this->GetBitmapFormat()==bmfPalette);
	for (int iY=0;iY<iNewHeight;++iY)
	{
		for (int iX=0;iX<iNewWidth;++iX)
		{
			if (bPalettized)
			{				
				bmpTemp.GetPixel(iX+iBorderLeft,iY+iBorderTop,byValue);
				this->SetPixel(iX,iY,byValue);
			}
			else
			{				
				bmpTemp.GetPixel(iX+iBorderLeft,iY+iBorderTop,rgbaValue);
				this->SetPixel(iX,iY,rgbaValue);
			}
		}
	}

	// ok
	return true;
}





// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// resampling filters
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------




CBitmapFile::CResamplingFilter::CResamplingFilter(double dDefaultFilterRadius)
	: m_dDefaultFilterRadius(dDefaultFilterRadius)
{
}

CBitmapFile::CResamplingFilter* CBitmapFile::CResamplingFilter::CreateResamplingFilter(ResampleFilter Filter)
{
	switch (Filter) 
	{
	case CBitmapFile::Box:
		return new CBitmapFile::CBoxFilter();
	case CBitmapFile::Triangle:
		return new CBitmapFile::CTriangleFilter();
	case CBitmapFile::Hermite:
		return new CBitmapFile::CHermiteFilter();
	case CBitmapFile::Bell:
		return new CBitmapFile::CBellFilter();
	case CBitmapFile::CubicBSpline:
		return new CBitmapFile::CCubicBSplineFilter();
	case CBitmapFile::Lanczos3:
		return new CBitmapFile::CLanczos3Filter();
	case CBitmapFile::Mitchell:
		return new CBitmapFile::CMitchellFilter();
	case CBitmapFile::Cosine:
		return new CBitmapFile::CCosineFilter();
	case CBitmapFile::CatmullRom:
		return new CBitmapFile::CCatmullRomFilter();
	case CBitmapFile::Quadratic:
		return new CBitmapFile::CQuadraticFilter();
	case CBitmapFile::QuadraticBSpline:
		return new CBitmapFile::CQuadraticBSplineFilter();
	case CBitmapFile::CubicConvolution:
		return new CBitmapFile::CCubicConvolutionFilter();
	case CBitmapFile::Lanczos8:
		return new CBitmapFile::CLanczos8Filter();
	default:
		return NULL;
	}
}

CBitmapFile::CHermiteFilter::CHermiteFilter()
	: CBitmapFile::CResamplingFilter(1)
{
}

double CBitmapFile::CHermiteFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX < this->m_dDefaultFilterRadius) return ((2*dX - 3)*dX*dX + this->m_dDefaultFilterRadius);
	return 0;
}

CBitmapFile::CBoxFilter::CBoxFilter()
	: CBitmapFile::CResamplingFilter(0.5)
{
}

double CBitmapFile::CBoxFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX <= this->m_dDefaultFilterRadius) return 1;
	return 0;
}

CBitmapFile::CTriangleFilter::CTriangleFilter()
	: CBitmapFile::CResamplingFilter(1)
{
}

double CBitmapFile::CTriangleFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX < this->m_dDefaultFilterRadius) return (this->m_dDefaultFilterRadius - dX);
	return 0;
}

CBitmapFile::CBellFilter::CBellFilter()
	: CBitmapFile::CResamplingFilter(1.5)
{
}

double CBitmapFile::CBellFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX < 0.5) return (0.75 - dX*dX);
	if (dX < this->m_dDefaultFilterRadius) return (0.5*SQR(dX - this->m_dDefaultFilterRadius));
	return 0;
}

CBitmapFile::CCubicBSplineFilter::CCubicBSplineFilter()
	: CBitmapFile::CResamplingFilter(2)
{
}

double CBitmapFile::CCubicBSplineFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX < 1 ) 
	{
		double dTemp = dX*dX;
		return (0.5*dTemp*dX - dTemp + 2./3.);
	}
	if (dX < this->m_dDefaultFilterRadius) 
	{
		dX = this->m_dDefaultFilterRadius - dX;
		return ((dX*dX*dX)/6.);
	}
	return 0;
}

CBitmapFile::CLanczosFilter::CLanczosFilter(double dFilterRadius)
	: CBitmapFile::CResamplingFilter(dFilterRadius)
{
}

double CBitmapFile::CLanczosFilter::SinC(double dX)
{
    if (dX != 0)
	{
		dX *= M_PI;
		return (sin(dX)/dX);
    }
    return 1;
}

double CBitmapFile::CLanczosFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX < this->m_dDefaultFilterRadius) return (SinC(dX)*SinC(dX/this->m_dDefaultFilterRadius));
	return 0;
}

CBitmapFile::CLanczos3Filter::CLanczos3Filter()
	: CBitmapFile::CLanczosFilter(3)
{
}

CBitmapFile::CLanczos8Filter::CLanczos8Filter()
	: CBitmapFile::CLanczosFilter(8)
{
}

CBitmapFile::CMitchellFilter::CMitchellFilter()
	: CBitmapFile::CResamplingFilter(2)
{
}

double CBitmapFile::CMitchellFilter::GetValue(double dX) 
{
	const double dC=1./3.;
	if (dX < 0) dX = - dX;
	double dTemp = dX*dX;
	if (dX < 1) 
	{
		dX = (((12. - 9.*dC - 6.*dC)*(dX*dTemp)) + ((- 18. + 12.*dC + 6.*dC)*dTemp) + (6. - 2.*dC));
		return (dX/6.);
	}
	if (dX < 2) 
	{
		dX = (((- dC - 6.*dC)*(dX*dTemp)) + ((6.*dC + 30.*dC)*dTemp) + ((- 12.*dC - 48.*dC)*dX) + (8.*dC + 24.*dC));
		return (dX/6.);
	}
	return 0;
}

CBitmapFile::CCosineFilter::CCosineFilter()
	: CBitmapFile::CResamplingFilter(1)
{
}

double CBitmapFile::CCosineFilter::GetValue(double dX) 
{
	if ((dX >= -1) && (dX <= 1)) return ((cos(dX*M_PI) + 1.)/2.);
	return 0;
}

CBitmapFile::CCatmullRomFilter::CCatmullRomFilter()
	: CBitmapFile::CResamplingFilter(2)
{
}

double CBitmapFile::CCatmullRomFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	double dTemp = dX*dX;
	if (dX <= 1) return (1.5*dTemp*dX - 2.5*dTemp + 1.);
	if (dX <= 2) return (- 0.5*dTemp*dX + 2.5*dTemp - 4.*dX + 2.);
	return 0;
}

CBitmapFile::CQuadraticFilter::CQuadraticFilter()
	: CBitmapFile::CResamplingFilter(1.5)
{
}

double CBitmapFile::CQuadraticFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX <= 0.5) return (- 2.*dX*dX + 1.);
	if (dX <= 1.5) return (dX*dX - 2.5*dX + 1.5);
	return 0;
}

CBitmapFile::CQuadraticBSplineFilter::CQuadraticBSplineFilter()
	: CBitmapFile::CResamplingFilter(1.5)
{
}

double CBitmapFile::CQuadraticBSplineFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	if (dX <= 0.5) return (- dX*dX + 0.75);
	if (dX <= 1.5) return (0.5*dX*dX - 1.5*dX + 1.125);
	return 0;
}

CBitmapFile::CCubicConvolutionFilter::CCubicConvolutionFilter()
	: CBitmapFile::CResamplingFilter(3)
{
}

double CBitmapFile::CCubicConvolutionFilter::GetValue(double dX) 
{
	if (dX < 0) dX = - dX;
	double dTemp = dX*dX;
	if (dX <= 1) return ((4./3.)*dTemp*dX - (7./3.)*dTemp + 1.);
	if (dX <= 2) return (- (7./12.)*dTemp*dX + 3.*dTemp - (59./12.)*dX + 2.5);
	if (dX <= 3) return ((1./12.)*dTemp*dX - (2./3.)*dTemp + 1.75*dX - 1.5);
	return 0;
}


