/////////////////////////////////////////////////////////////////////////////////////
// BitmapFile.h
// bitmap file management and writing.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  01. Jan. 2006	: Added y flipping of bitmap.
//  01. Jan. 2006	: Initialization from TC texture.
//  10. Oct. 2006	: Achieved VS2005 compatibility, made compilation easier.
//  29. Dec. 2006	: Added RGBA format.
//  31. Jan. 2007   : Added GetSizeX, GetSizeY, GetPixel, and Read methods.
//  19. Feb. 2007   : Added FlipX method.
//  17. Apr. 2007	: Added bitmap scaling.
//  01. Sep. 2008	: Now using DevIL instead of DirectX methods for loading,
//                    saving, and converting images.
//  12. Sep. 2008	: Added the methods BlurGaussian(), FillFromBitmap(), Clear(),
//					  and RemoveBorder().
//  03. Oct. 2008	: Implemented resampling, because of some scaling problems
//					  in the DevIL lib version 1.7.2
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_BITMAPFILE_H_INCLUDED)
#define PNPTC_BITMAPFILE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define _CRT_SECURE_NO_DEPRECATE 1

#include <string>
#include "../../includes/IPnPTCPlugIn.h"
#include "IL/il.h"


#pragma pack(push)
#pragma pack(1)

// forward definitions
struct IDirect3DDevice9;
class CIPnPTCTexture01;

/// supported bitmap formats
typedef enum BITMAPFORMAT
{
	bmfPalette	= 0,		//!< palettized bitmap
	bmfRGB		= 1,		//!< 24 bit RGB format
	bmfRGBA		= 2			//!< 32 bit RGBA format
} BITMAPFORMAT;



/// Bitmap file class for all bitmap loading, saving, and manipulation
/// operations. This is mostly a wrapper class for the DevIL library.
class CBitmapFile
{
public:

	// ---------------------------------------------------------------------
	// constructin / destruction
	// ---------------------------------------------------------------------

	/// construction of a new bitmap object
	CBitmapFile(void);

	/// destruction of a bitmap object
	~CBitmapFile(void);


	// ---------------------------------------------------------------------
	// bitmap initialization
	// ---------------------------------------------------------------------

	/// initializes the bitmap.
	/// @param iFormat bitmap format (bmfPalette, bmfRGB, or bmfRGBA).
	/// @param nSizeX x dimension of bitmap.
	/// @param nSizeY y dimension of bitmap.
	/// @return returns <i>true</i> or <i>false</i>, whether or not
	/// the bitmap could be initialized.
	bool	Initialize(BITMAPFORMAT iFormat,unsigned int nSizeX,unsigned int nSizeY);

	/// initializes the bitmap from a TC bitmap.
	/// @param pTexture texture from which to initialize the bitmap.
	/// @param iFormat bitmap format (bmfRGB or bmfRGBA).
	/// @return returns <i>true</i> or <i>false</i>, whether or not
	/// the bitmap could be initialized.
	bool	Initialize(CIPnPTCTexture01* pTexture,BITMAPFORMAT iFormat=bmfRGB);

	/// Returns whether or not this bitmap was initialized.
	/// @return Returns <i>true</i> if this bitmap object has already
	/// been initialized, otherwise <i>false</i>.
	bool	IsInitialized() const;

	/// Deinitializes the bitmap object. The memory of the bitmap
	/// is freed and all parameters are set to zero.
	void	DeInitialize();

	/// creates a gray scale palette
	void	CreateGrayScalePalette();


	// ---------------------------------------------------------------------
	// get information from bitmap
	// ---------------------------------------------------------------------

	/// Returns the size of the bitmap in x direction.
	/// @return Bitmap's size in x direction.
	unsigned int GetSizeX() const {return this->m_nSizeX;};

	/// Returns the size of the bitmap in y direction.
	/// @return Bitmap's size in y direction.
	unsigned int GetSizeY() const {return this->m_nSizeY;};

	/// Returns the bitmap format of this bitmap.
	/// @return The bitmap format.
	BITMAPFORMAT GetBitmapFormat() const;


	// ---------------------------------------------------------------------
	// access to pixel data
	// ---------------------------------------------------------------------

	/// sets one pixel for a palettized bitmap.
	/// @param nX x coordinate of the pixel to be set.
	/// @param nY y coordinate of the pixel to be set.
	/// @param byValue palette index to be set for that pixel.
	void	SetPixel(unsigned int nX,unsigned int nY,BYTE byValue);

	/// sets on pixel for a rgb bitmap.
	/// @param nX x coordinate of the pixel to be set.
	/// @param nY y coordinate of the pixel to be set.
	/// @param rgb rgb value to be set for that pixel.
	void	SetPixel(unsigned int nX,unsigned int nY,const PNPTC_RGB& rgb);

	/// sets on pixel for a rgb bitmap.
	/// @param nX x coordinate of the pixel to be set.
	/// @param nY y coordinate of the pixel to be set.
	/// @param rgba rgba value to be set for that pixel.
	void	SetPixel(unsigned int nX,unsigned int nY,const PNPTC_RGBA& rgba);

	/// returns the value of one pixel.
	/// @param nX x coordinate of the pixel to be returned.
	/// @param nY y coordinate of the pixel to be returned.
	/// @param byValue (output) The value of the pixel.
	void	GetPixel(unsigned int nX,unsigned int nY,BYTE& byValue) const;

	/// returns the value of one pixel.
	/// @param nX x coordinate of the pixel to be returned.
	/// @param nY y coordinate of the pixel to be returned.
	/// @param rgb (output) The value of the pixel.
	void	GetPixel(unsigned int nX,unsigned int nY,PNPTC_RGB& rgb) const;

	/// returns the value of one pixel.
	/// @param nX x coordinate of the pixel to be returned.
	/// @param nY y coordinate of the pixel to be returned.
	/// @param rgba (output) The value of the pixel.
	void	GetPixel(unsigned int nX,unsigned int nY,PNPTC_RGBA& rgba) const;


	// ---------------------------------------------------------------------
	// storing and loading
	// ---------------------------------------------------------------------

	/// Reads a bitmap from a file.
	/// @param strSourceFile Name of the bitmap file to be read.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// file could be loaded.
	bool	Read(const std::string& strSourceFile);

	/// write bitmap to a file.
	/// @param strDestFile destination file to write to.
	/// @param strFileFormat format of the file to be written.
	/// (BMP,DDS,JPG,PNG,DIB,HDR)
	/// @return returns <i>true</i> or <i>false</i>, whether or not
	/// the bitmap could be written to the file.
	bool	Write(const std::string& strDestFile,const std::string& strFileFormat="BMP") const;


	// ---------------------------------------------------------------------
	// bitmap manipulations
	// ---------------------------------------------------------------------

	/// Available resampling filters to be used by Resize operation.
	typedef enum ResampleFilter
	{
		None				= -1,
		Box					= 0,
		Triangle			= 1,
		Hermite				= 2,
		Bell				= 3,
		CubicBSpline		= 4,
		Lanczos3			= 5,
		Mitchell			= 6,
		Cosine				= 7,
		CatmullRom			= 8,
		Quadratic			= 9,
		QuadraticBSpline	= 10,
		CubicConvolution	= 11,
		Lanczos8			= 12
	} ResampleFilter;

	/// Resizes the bitmap.
	/// @param nSizeX The new width of the bitmap.
	/// @param nSizeY The new height of the bitmap.
	/// @param Filter The filter to be used for resampling.
	/// @return returns <i>true</i> or <i>false</i>, whether or not
	/// the bitmap could be resized.
	bool	Resize(unsigned int nSizeX,unsigned int nSizeY,ResampleFilter Filter=Mitchell);

	/// Flips the y coordinates of the image. The pixels are mirrored
	/// at a horizontal plane through the vertical image center.
	void	FlipY();

	/// Flips the x coordinates of the image. The pixels are mirrored
	/// at a vertical plane through the vertical image center.
	void	FlipX();

	/// Flips the image at a diagonal plane.
	/// @param bTopLeft2BottomRight <i>true</i> if the mirror plane
	/// goes from the top left to the bottom right of the image or
	/// <i>false</i> if it goes from the bottom left to the top right.
	void	FlipDiagonal(bool bTopLeft2BottomRight);

	/// Smooth the image.
	/// @param nSmoothFactor the smooth factor, wich is the half
	/// of the width/height of the gaussian filter kernel.
	/// @param bUseMax If <i>true</i>, the maximum value of the
	/// pixels in the filter kernel is used. If <i>false</i>, the
	/// gaussian filter kernel is used in the usual way.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the bitmap could be smoothed successfully.
	bool	BlurGaussian(int iSmoothFactor,bool bUseMax=false);

	/// Fills the bitmap from specified bitmap.
	/// @param bmpSource Source bitmap file.
	/// @param iRemoveBorder The size of the border, which should be 
	/// removed from the source bitmap when copying it into the destination.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the bitmap could be filled/copied successfully.
	bool	FillFromBitmap(const CBitmapFile& bmpSource,int iRemoveBorder=0);

	/// Clears the whole bitmap.
	void	Clear();

	/// Removes a border area from this image.
	/// @param iBorderLeft The border size in pixel to be removed from the left of the image.
	/// @param iBorderTop The border size in pixel to be removed from the top of the image.
	/// @param iBorderRight The border size in pixel to be removed from the right of the image.
	/// If <i>-1</i> the same value is used as for the left border.
	/// @param iBorderBottom The border size in pixel to be removed from the bottom of the image.
	/// If <i>-1</i> the same value is used as for the top border.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the bitmap border could be removed successfully.
	bool	RemoveBorder(int iBorderLeft,int iBorderTop,int iBorderRight=-1,int iBorderBottom=-1);


protected:

	// ---------------------------------------------------------------------
	// member variables
	// ---------------------------------------------------------------------

	/// width of bitmap.
	unsigned int		m_nSizeX;

	/// height of bitmap.
	unsigned int		m_nSizeY;

	/// The image name.
	ILuint				m_ImageName;

	/// The image format.
	ILenum				m_Format;


	// ---------------------------------------------------------------------
	// internal methods
	// ---------------------------------------------------------------------

	/// Returns a direct pointer to the data of one pixel.
	/// @param nX X coordinate of the pixel.
	/// @param nY Y coordinate of the pixel.
	/// @return Returns a pointer to the first byte of the pixel.
	void*				GetPixelDataPtr(unsigned int nX,unsigned int nY);

	/// Returns a direct pointer to the data of one pixel.
	/// @param nX X coordinate of the pixel.
	/// @param nY Y coordinate of the pixel.
	/// @return Returns a pointer to the first byte of the pixel.
	const void*			GetPixelDataPtr(unsigned int nX,unsigned int nY) const;

	/// Swaps two pixels.
	void				SwapPixels(unsigned int nX1,unsigned int nY1,unsigned int nX2,unsigned int nY2);

	/// Internal method used to resample/resizes the bitmap.
	/// @param nSizeX The new width of the bitmap.
	/// @param nSizeY The new height of the bitmap.
	/// @param Filter The filter to be used for resampling.
	/// @return returns <i>true</i> or <i>false</i>, whether or not
	/// the bitmap could be resized.
	bool				Resample(unsigned int nSizeX,unsigned int nSizeY,ResampleFilter Filter);


	// ---------------------------------------------------------------------
	// internal classes
	// ---------------------------------------------------------------------

	/// The resampling filter base class.
	class CResamplingFilter
	{
	public:

		/// Construction of a new resampling filter object.
		/// @param dDefaultFilterRadius The default filter radius.
		CResamplingFilter(double dDefaultFilterRadius);

		/// Factory method to create a new resampling filter.
		/// @param Filter The filter method to be used.
		/// @return Returns a pointer to the new filter object
		/// if successful, or <i>NULL</i> in case of an error.
		static CResamplingFilter*	CreateResamplingFilter(ResampleFilter Filter);

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		virtual	double				GetValue(double dX) = 0;

		/// Returns the default filter radius.
		/// @return The default filter radius.
		double						GetDefaultFilterRadius() const {return this->m_dDefaultFilterRadius;};


	protected:

		/// The default filter radius.
		double						m_dDefaultFilterRadius;

	}; // class ResamplingFilter

	/// The hermit resampling filter class
	class CHermiteFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new hermit filter.
		CHermiteFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);
	
	}; // class CHermiteFilter

	/// The box resampling filter class
	class CBoxFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new box filter.
		CBoxFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CBoxFilter

	/// The triangle resampling filter class
	class CTriangleFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new triangle filter.
		CTriangleFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CTriangleFilter

	/// The bell resampling filter class
	class CBellFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new bell filter.
		CBellFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CResamplingFilter

	/// The cubic b-spline resampling filter class
	class CCubicBSplineFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new cubic b-spline filter.
		CCubicBSplineFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CCubicBSplineFilter

	/// The laczos resampling filter class
	class CLanczosFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new laczos filter.
		/// @param dFilterRadius The filter radius.
		CLanczosFilter(double dFilterRadius);

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double			GetValue(double dX);

	private:

		/// Computes the normalized sinc value.
		/// @param dX The value for which the sinc value should be computed.
		/// @return Returns SinC(dX).
		static double	SinC(double dX);

	}; // class CLanczosFilter

	/// The laczos3 resampling filter class
	class CLanczos3Filter : public CLanczosFilter
	{
	public:

		/// Construction of a new laczos3 filter.
		CLanczos3Filter();

	}; // class CLanczos3Filter

	/// The laczos8 resampling filter class
	class CLanczos8Filter : public CLanczosFilter
	{
	public:

		/// Construction of a new laczos8 filter.
		CLanczos8Filter();

	}; // class CLanczos8Filter

	/// The mitchell resampling filter class
	class CMitchellFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new mitchell filter.
		CMitchellFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CMitchellFilter

	/// The cosine resampling filter class
	class CCosineFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new cosine filter.
		CCosineFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CCosineFilter

	/// The catmull rom resampling filter class
	class CCatmullRomFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new catmull rom filter.
		CCatmullRomFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CCatmullRomFilter

	/// The quadratic resampling filter class
	class CQuadraticFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new quadratic filter.
		CQuadraticFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CQuadraticFilter

	/// The quadratic b-spline resampling filter class
	class CQuadraticBSplineFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new quadratic b-spline filter.
		CQuadraticBSplineFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CQuadraticBSplineFilter

	/// The cubic convolution resampling filter class
	class CCubicConvolutionFilter : public CResamplingFilter
	{
	public:

		/// Construction of a new cubic convolution filter.
		CCubicConvolutionFilter();

		/// Returns one value from the filter kernel.
		/// @param dX The offset from the filter center.
		/// @return Returns the value at the given offset.
		double		GetValue(double dX);

	}; // class CCubicConvolutionFilter

};


#pragma pack(pop)

#endif // !defined(PNPTC_BITMAPFILE_H_INCLUDED)
