/////////////////////////////////////////////////////////////////////////////////////
// ChunkedFile.cpp
// Chunked file reading and writing.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  24. Sep. 2006 : Added chunk file loading
//  07. Oct. 2006 : Added CChunkedFile::GetFilePos, CChunkHeader::EnfOfChunk, and
//                  CChunkHeaderIDSize2.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "chunkedfile.h"

CChunkedFile::CChunkedFile(long lAddToSize/*=0*/)
	: m_File(NULL)
	, m_lAddToSize(lAddToSize)
	, m_nFileSize(0)
{
}

CChunkedFile::~CChunkedFile()
{
	// close file
	this->ClearChunkStack();
	this->Close();
}

bool CChunkedFile::OpenWrite(std::string strFilename)
{
	// open file for binary writing
	this->ClearChunkStack();
	this->m_File=fopen(strFilename.c_str(),"wb");
	this->m_nFileSize=0;
	return (this->m_File!=NULL);
}

bool CChunkedFile::OpenRead(std::string strFilename)
{
	// open file for binary writing
	this->ClearChunkStack();
	this->m_File=fopen(strFilename.c_str(),"rb");
	if (this->m_File)
	{
		fseek(this->m_File,0,SEEK_END);
		this->m_nFileSize=ftell(this->m_File)+1;
		fseek(this->m_File,0,SEEK_SET);
	}
	else
	{
		this->m_nFileSize=0;
	}
	return (this->m_File!=NULL);
}

void CChunkedFile::Close()
{
	// close opened file
	if (this->m_File)
	{
		fclose(this->m_File);
		this->m_File=NULL;
		this->m_nFileSize=0;
		this->ClearChunkStack();
	}
}

bool CChunkedFile::Write(const void* pData,DWORD dwSize)
{
	// write to file
	if  ((this->m_File!=NULL) && (fwrite(pData,dwSize,1,this->m_File)==1))
	{
		// ok check file size
		this->m_nFileSize=max(this->m_nFileSize,static_cast<unsigned long>(ftell(this->m_File)));
		return true;
	}
	else
	{
		// error writing to file
		return false;
	}
}

bool CChunkedFile::StartChunkWriting(DWORD dwSize/*=0*/)
{
	// check file
	if (!this->m_File)
	{
		return false;
	}

	// push position on stack
	this->m_ChunkPositions.push(ftell(this->m_File));

	// print size
	return this->Write(&dwSize,sizeof(DWORD));
}

bool CChunkedFile::FinishChunkWriting()
{
	// check file
	if (!this->m_File || this->m_ChunkPositions.size()==0)
	{
		return false;
	}

	// compute chunk size
	long lChunkPos=this->m_ChunkPositions.top();
	this->m_ChunkPositions.pop();
	long lCurPos=ftell(this->m_File);
	long lSize=lCurPos-lChunkPos+this->m_lAddToSize;

	// write chunk size
	if (fseek(this->m_File,lChunkPos,SEEK_SET)!=0)
	{
		return false;
	}
	if (!this->Write(&lSize,sizeof(DWORD)))
	{
		return false;
	}

	// restore position
	return (fseek(this->m_File,lCurPos,SEEK_SET)==0);
}

bool CChunkedFile::FinishChunkWriting(DWORD dwSize)
{
	// check file
	if (!this->m_File || this->m_ChunkPositions.size()==0)
	{
		return false;
	}

	// get chunk position
	long lChunkPos=this->m_ChunkPositions.top();
	this->m_ChunkPositions.pop();
	long lCurPos=ftell(this->m_File);

	// write chunk size
	if (fseek(this->m_File,lChunkPos,SEEK_SET)!=0)
	{
		return false;
	}
	if (!this->Write(&dwSize,sizeof(DWORD)))
	{
		return false;
	}

	// restore position
	return (fseek(this->m_File,lCurPos,SEEK_SET)==0);
}

bool CChunkedFile::ReadFile()
{
	// file opened?
	if (!this->m_File)
	{
		return false;
	}

	// read until end of file
	this->ClearChunkStack();
	while (static_cast<unsigned long>(ftell(this->m_File))<(this->m_nFileSize-1))
	{
		// create new chunk
		CChunkHeader* pChunk=this->CreateChunk(static_cast<unsigned long>(ftell(this->m_File)));
		if (pChunk)
		{
			// read chunk header
			if (pChunk->ReadHeader(*this))
			{
				// read data
				if (this->OnReadChunkData(*pChunk))
				{
					// chunk read successfully => set pointer to the beginning of the next chunk
					fseek(this->m_File,pChunk->GetFilePos()+pChunk->GetDataSize()+pChunk->GetHeaderSize(),SEEK_SET);
					delete pChunk;
				}
				else
				{
					// error reading chunk.
					delete pChunk;
					return false;
				}
			}
			else
			{
				// error reading chunk header
				delete pChunk;
				return false;
			}
		}
		else
		{
			// error creating chunk
			return false;
		}
	}

	// ok, seems to have been successfull
	return true;
}

bool CChunkedFile::Read(void* pData,DWORD dwSize)
{
	// read from file
	return ((this->m_File!=NULL) && (fread(pData,dwSize,1,this->m_File)==1));
}

bool CChunkedFile::ProcessSubChunks(const CChunkHeader& headerParent)
{
	// check file
	if (!this->m_File)
	{
		return false;
	}

	// check if really inside chunk
	if (static_cast<unsigned long>(ftell(this->m_File))<headerParent.GetFilePos()+headerParent.GetHeaderSize())
	{
		return false;
	}

	// compute end of chunk
	unsigned long nEndOfChunk=headerParent.GetFilePos()+headerParent.GetHeaderSize()+headerParent.GetDataSize();

	// read all sub chunks
	while (static_cast<unsigned long>(ftell(this->m_File))<(nEndOfChunk-1))
	{
		// create new chunk
		CChunkHeader* pChunk=this->CreateChunk(static_cast<unsigned long>(ftell(this->m_File)));
		if (pChunk)
		{
			// read chunk header
			if (pChunk->ReadHeader(*this))
			{
				// read data
				if (this->OnReadChunkData(*pChunk))
				{
					// chunk read successfully => set pointer to the beginning of the next chunk
					fseek(this->m_File,pChunk->GetFilePos()+pChunk->GetDataSize()+pChunk->GetHeaderSize(),SEEK_SET);
					delete pChunk;
				}
				else
				{
					// error reading chunk.
					delete pChunk;
					return false;
				}
			}
			else
			{
				// error reading chunk header
				delete pChunk;
				return false;
			}
		}
		else
		{
			// error creating chunk
			return false;
		}
	}

	// ok, seems to have been successfull
	return true;
}

bool CChunkedFile::OnReadChunkData(const CChunkHeader& header)
{
	// no default implementation
	return false;
}

void CChunkedFile::ClearChunkStack()
{
	// remove all entries from the stack
	while (!this->m_ChunkPositions.empty())
	{
		this->m_ChunkPositions.pop();
	}
}

unsigned long CChunkedFile::GetFilePos() const
{
	// file opened?
	if (!this->m_File)
	{
		return 0;
	}

	// get current file position
	return ftell(this->m_File);
}

