/////////////////////////////////////////////////////////////////////////////////////
// ChunkedFile.h
// Chunked file reading and writing.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  24. Sep. 2006 : Added chunk file loading
//  07. Oct. 2006 : Added CChunkedFile::GetFilePos, CChunkHeader::EnfOfChunk, and
//                  CChunkHeaderIDSize2.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CHUNKEDFILE_H_INCLUDED)
#define PNPTC_CHUNKEDFILE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>		// The PlatformSDK has to be installed in order to compile this PlugIn
#include <string>
#include <stack>


#pragma pack(push)
#pragma pack(1)


// forward definitions
class CChunkedFile;


// ----------------------------------------------------------------------------------------

/// Chunk header class
class CChunkHeader
{
public:

	// -----------------------------------------------------
	// construction/destruction
	// -----------------------------------------------------

	/// Constructs a new empty chunk header data structure.
	/// @param nFilePos Absolute position of this chunk inside the file.
	CChunkHeader(unsigned long nFilePos)
		: m_nFilePos(nFilePos)
	{};

	/// destruction of the chunk header data structure.
	virtual ~CChunkHeader() {};


	// -----------------------------------------------------
	// access to chunk data
	// -----------------------------------------------------

	/// Returns the absolute position of this chunk inside the file.
	/// @return Absolute position of this chunk inside the file.
	unsigned long			GetFilePos() const {return this->m_nFilePos;};

	/// Returns whether or not the end of the chunk
	/// was reached during reading the chunk's data.
	/// @param nFilePos Current position in the file.
	/// @return Returns <i>true</i> if the end of the chunk was 
	/// reached, otherwise <i>false</i>.
	bool					EnfOfChunk(unsigned long nFilePos) const {return this->m_nFilePos+this->GetHeaderSize()+this->GetDataSize()<=nFilePos;};

	/// Returns the size of the chunk header in bytes.
	/// @return Chunk header size in bytes.
	virtual unsigned long	GetHeaderSize() const = 0;

	/// Returns the size of the chunk's data in bytes.
	/// This value should include the size of all subchunks as well.
	/// @return Chunk data size in bytes.
	virtual unsigned long	GetDataSize() const = 0;

	/// Reads the header from the chunk file.
	/// @param file File from which to read the header.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the header could be read.
	virtual bool			ReadHeader(CChunkedFile& file) = 0;


private:

	// -----------------------------------------------------
	// member variables
	// -----------------------------------------------------

	/// Position of this chunk inside the file.
	unsigned long			m_nFilePos;

};


// ----------------------------------------------------------------------------------------

/// Chunked file class
class CChunkedFile
{
public:

	// -----------------------------------------------------
	// construction/destruction
	// -----------------------------------------------------

	/// construction of a new chunk file.
	/// @param lAddToSize Additional number of bytes
	/// to be added to every chunk while writing.
	CChunkedFile(long lAddToSize=0);

	/// destruction of the chunked file.
	virtual ~CChunkedFile();


	// -----------------------------------------------------
	// open/close files
	// -----------------------------------------------------

	/// Opens a file for writing.
	/// @param strFilename destination file name.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the file could be opened.
	bool					OpenWrite(std::string strFilename);

	/// Opens a file for reading.
	/// @param strFilename source file name.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the file could be opened.
	bool					OpenRead(std::string strFilename);

	/// Closes a currently opened file.
	void					Close();


	// -----------------------------------------------------
	// writing data
	// -----------------------------------------------------

	/// writes data to the chunk file.
	/// @param pData pointer to the data to be written.
	/// @param dwSize size of the data to be written.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the data could be written to the file.
	bool					Write(const void* pData,DWORD dwSize);

	/// starts a chunk by writing a (blank) chunk size and storing
	/// the file position of the blank.
	/// @param dwSize optional size of the chunk
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the chunk could be started.
	bool					StartChunkWriting(DWORD dwSize=0);

	/// finishes the last opened chunk by setting its
	/// size with respect to the current file position.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the chunk could be finished.
	bool					FinishChunkWriting();

	/// finishes the last opened chunk by setting its
	/// size to a defined value.
	/// @param dwSize Size of the chunk to be set
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the chunk could be finished.
	bool					FinishChunkWriting(DWORD dwSize);


	// -----------------------------------------------------
	// reading data
	// -----------------------------------------------------

	/// Reads the file.
	/// @return Returns <i>true</i> or <i>false</i>, whether or 
	/// not the file could be read successfully and completely.
	bool					ReadFile();

	/// Reads data from the chunk file.
	/// @param pData pointer to a data area to which the data
	/// which is read from the file will be written.
	/// @param dwSize size of the data to be read.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the data could be read from the file.
	bool					Read(void* pData,DWORD dwSize);


	// -----------------------------------------------------
	// reading data
	// -----------------------------------------------------

	/// Returns the current file position.
	/// @return Current position within the chunk file.
	unsigned long			GetFilePos() const;


protected:

	// -----------------------------------------------------
	// chunk handling
	// -----------------------------------------------------

	/// Processes sub chunks of the current chunk.
	/// @param headerParent Chunk header of the parent chunk from
	/// which to process the sub chunks.
	/// @return Returns <i>true</i> or <i>false</i>, whether or 
	/// not the sub chunks could be read successfully and completely.
	bool					ProcessSubChunks(const CChunkHeader& headerParent);

	/// Reads the data from the current chunk.
	/// This method should be overridden by a derived class,
	/// as the default implementation will not read anything.
	/// This method is called from the ReadFile or ProcessSubChunks
	/// methods, whenever a new chunk is to be read.
	/// @param header Header of the chunk to be read.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the chunk data could be read.
	virtual bool			OnReadChunkData(const CChunkHeader& header);

	/// Creates a new chunk header.
	/// This method should be overridden by a derived class,
	/// as the default implementation will not create new chunks.
	/// @param nFilePos Absolute position of the new chunk to be
	/// created inside the current file.
	/// @return Returns a pointer to the newly created chunk
	/// or <i>NULL</i> in case of an error.
	virtual CChunkHeader*	CreateChunk(unsigned long nFilePos) const {return NULL;};


private:

	// -----------------------------------------------------
	// member variables
	// -----------------------------------------------------

	/// file pointer
	FILE*					m_File;

	/// stack of chunk positions for setting chunk sizes during writing
	std::stack<long>		m_ChunkPositions;

	/// additional value to be added to the computed size of the chunk to be written.
	long					m_lAddToSize;

	/// Size of the file.
	unsigned long			m_nFileSize;


	// -----------------------------------------------------
	// internal methods
	// -----------------------------------------------------

	/// Clears the complete stack of chunks.
	void					ClearChunkStack();

};


// ----------------------------------------------------------------------------------------

/// Most common form of chunk headers
/// which contains a chunk identifier in the first data element
/// and the size of the chunk in the second data element.
/// The data types used for these data elements can be set
/// as template parameters. The size value stored in the file
/// is the size of the chunk's data area plus the chunks 
/// header in bytes.
template <class tID,class tSize>
class CChunkHeaderIDSize : public CChunkHeader
{
public:

	// -----------------------------------------------------
	// construction/destruction
	// -----------------------------------------------------

	/// Constructs a new empty chunk header data structure.
	/// @param nFilePos Absolute position of this chunk inside the file.
	CChunkHeaderIDSize(unsigned long nFilePos)
		: CChunkHeader(nFilePos)
		, m_ChunkID(0)
		, m_ChunkSize(0)
	{};

	/// destruction of the chunk header data structure.
	virtual ~CChunkHeaderIDSize() {};


	// -----------------------------------------------------
	// access to chunk data
	// -----------------------------------------------------

	/// Returns the size of the chunk header in bytes.
	/// @return Chunk header size in bytes.
	unsigned long	GetHeaderSize() const {return sizeof(tID)+sizeof(tSize);};

	/// Returns the size of the chunk's data in bytes.
	/// This value should include the size of all subchunks as well.
	/// @return Chunk data size in bytes.
	unsigned long	GetDataSize() const {return this->m_ChunkSize-this->GetHeaderSize();};

	/// Reads the header from the chunk file.
	/// @param file File from which to read the header.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the header could be read.
	virtual bool	ReadHeader(CChunkedFile& file)
	{
		// read ID and size
		return file.Read(&this->m_ChunkID,sizeof(tID)) && file.Read(&this->m_ChunkSize,sizeof(tSize));
	};

	/// Returns the ID of the chunk.
	/// @return Chunk ID.
	tID				GetID() const {return this->m_ChunkID;};


protected:

	// -----------------------------------------------------
	// member variables
	// -----------------------------------------------------

	/// ID of that chunk.
	tID				m_ChunkID;

	/// size of that chunk (data + header).
	tSize			m_ChunkSize;
};


// ----------------------------------------------------------------------------------------

/// The only difference from this class to the class
/// CChunkHeaderIDSize, is that the size value stored in the file
/// is _only_ the size of the chunk's data area.
template <class tID,class tSize>
class CChunkHeaderIDSize2 : public CChunkHeaderIDSize<tID,tSize>
{
public:

	// -----------------------------------------------------
	// construction/destruction
	// -----------------------------------------------------

	/// Constructs a new empty chunk header data structure.
	/// @param nFilePos Absolute position of this chunk inside the file.
	CChunkHeaderIDSize2(unsigned long nFilePos)
		: CChunkHeaderIDSize<tID,tSize>(nFilePos)
	{};


	// -----------------------------------------------------
	// access to chunk data
	// -----------------------------------------------------

	/// Reads the header from the chunk file.
	/// @param file File from which to read the header.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the header could be read.
	virtual bool	ReadHeader(CChunkedFile& file)
	{
		// read ID and size
		bool bReturn=CChunkHeaderIDSize<tID,tSize>::ReadHeader(file);
		this->m_ChunkSize+=this->GetHeaderSize();
		return bReturn;
	};
};

#pragma pack(pop)


#endif // !defined(PNPTC_CHUNKEDFILE_H_INCLUDED)