/////////////////////////////////////////////////////////////////////////////////////
// DXUtils.cpp
// various utility methods to support common plugin functionality working
// with DirectX data types and methods.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  03. Dec. 2008 :	Added GlobalVectorTransform
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "./DXUtils.h"

/// Affects a 3D vector by a given global transformation.
/// @param vecTransform The vector, which should be transformed.
/// @param matTransform An additional transformation matrix to be applied.
/// @param fScale A scaling factor, which will be applied to all vertex
/// positions during export.
/// @param iUpRotation A rotation, which will be applied to all vertices
/// in order to correclty align the object's up axis. Possible values are
/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
/// @param iFlipAxis The axis along which all vertex positions are negated
/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
void GlobalVectorTransform(PnPTC3DVec& vecTransform,const D3DXMATRIX& matTransform,float fScale,int iUpRotation,int iFlipAxis)
{
	// apply transformation matrix
	D3DXVECTOR3 v3Temp(vecTransform.x,vecTransform.y,vecTransform.z);
	D3DXVECTOR4 v4Return;
	D3DXVec3Transform(&v4Return,&v3Temp,&matTransform);
	vecTransform=PnPTC3DVec(v4Return.x,v4Return.y,v4Return.z);

	// apply transformations
	GlobalVectorTransform(vecTransform,fScale,iUpRotation,iFlipAxis);
}

/// compresses a texture into DX5 format.
/// @param strTexture The name of the texture file to be compressed.
/// @param strFormat The format into which the texture should be compressed
/// @return Returns <i>true</i> or <i>false</i>, whether or not the
/// file could be compressed successfully.
bool DXCompressTexture(const std::string& strTexture,const std::string& strFormat/*="DXT5"*/)
{
	// get file name of this plugin
	char cFileName[MAX_PATH+1];
	::GetModuleFileName(NULL,cFileName,MAX_PATH+1);
	std::string strPath=GetPath(cFileName)+"PlugIns\\utils\\";

	// run the texture conversion tool
	std::string strOutputPath=GetPath(strTexture);
	RemoveTrailingBackslash(strOutputPath);
	std::string strTool="\""+strPath+"texconv.exe\" -f "+strFormat+" -ft DDS ";
	std::string strOutput="-o \""+strOutputPath+"\" ";
	std::string strFile="\""+strTexture+"\" ";
	std::string strCommandLine=strTool+strFile+strOutput;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	ZeroMemory( &si, sizeof(si) );
	si.cb = sizeof(si);
	ZeroMemory( &pi, sizeof(pi) );
	if (!::CreateProcess(NULL,(LPSTR)strCommandLine.c_str(),NULL,NULL,TRUE,CREATE_NO_WINDOW,NULL,strPath.c_str(),&si,&pi))
	{
		// error converting mesh to binary
		MessageBox(NULL,(std::string("Error compressing texture with command\n'")+strCommandLine+"'.").c_str(),"Error saving object",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// Wait until child process exits.
	WaitForSingleObject( pi.hProcess, INFINITE );

	// Close process and thread handles. 
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );

	// ok 
	return true;
}

