/////////////////////////////////////////////////////////////////////////////////////
// Exporter.cpp
// Implementation of all exporter classes which can be used from within
// the template scene exporter.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  01. Dec. 2005 :	Added support for lightmap overlay to texture map to bitmap export.
//  31. Jan. 2006 : Added alphamap exporter.
//  01. Feb. 2006 : Added support to not write black files in alphmap exporter.
//  07. Feb. 2006 : Added y flipping of exported bitmaps.
//  10. Mai. 2006 : Fixed bug in exporting RAW heightmaps for a whole terrain
//                  into one file with other data formats than float.
//  12. Mai. 2006 : Added full scale feature to RAW export.
//  16. Mai. 2006 : Added size+1 feature for bitmap exporter.
//  19. Mai. 2006 : Added flipY option to raw export.
//  24. Aug. 2006 : Added possibility to invert lightmap texture.
//  29. Dec. 2006 : Added grass map exporter.
//  15. Mar. 2007 : Added export file name callback interface.
//  16. Mar. 2007 : Added possibility to scale texturemap bitmap to a power of 2 border size.
//  29. Mai  2007 : Added constant offset value to raw exporter.
//  30. Mai  2007 : Updated progress bars
//  11. Mai  2008 : Added possibility to scale lightmap bitmap to a power of 2 border size.
//  17. Mai  2008 : Added/activated power of two feature for alpha map exporter.
//  01. Sep. 2008 : Now using DevIL instead of DirectX methods for loading,
//                  saving, and converting images. For a small change in the
//                  bitmap interface, the exporter had to be changed.
//  08. Sep. 2008 : Added possibility to export border pixels only on two sides of the exported texture maps.
//  13. Sep. 2008 : Heavy update to alphamap exporter to work with OGRE PLSM2 coverage maps.
//					Added possibility to export multichannel (RGBA) alphamaps.
//					Added border pixel functionality to normal map and alpha map exporter.
//  23. Nov. 2008 : Faster alphamap export
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "exporter.h"
#include "utils.h"
#include "BitmapFile.h"
#include <math.h>
#include <map>
#include <string>
#include "progressbar.h"
#include "../pnptc3dobject/ogreutils.h"
#include "TempFileManager.h"


// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// helper methods
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

/// Computes a texture border size value with a power of two
/// closest to a given initial texture border size.
/// @param nInitialBorderSize The initial non power of two border size.
/// @return Returns the border size at a power of two.
unsigned int GetPowerOfTwoBorderSize(const unsigned int nInitialBorderSize)
{
	// initialize return value
	unsigned int nReturnBorderSize=1;

	// find a border size at the power of two which is no longer smaller than the initial size
	while (nReturnBorderSize<nInitialBorderSize)
	{
		// if we are not too small and very close to the initial size, we are 
		// going to use this size even if it is not larger than the inital one
		if (nReturnBorderSize>=16 && nReturnBorderSize+4>=nInitialBorderSize)
		{
			return nReturnBorderSize;
		}

		// next power of two border size
		nReturnBorderSize*=2;
	}

	// return the result
	return nReturnBorderSize;
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// exporter base class
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------



CExporter::CExporter()
	: m_pFileNameCallback(NULL)
{
}
CExporter::~CExporter()
{
}

void CExporter::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
}
void CExporter::SetParameter(const std::string& strParameterName,int iValue)
{
}
void CExporter::SetParameter(const std::string& strParameterName,float fValue)
{
}
void CExporter::SetParameter(const std::string& strParameterName,bool bValue)
{
}

void CExporter::SetFileNameCallback(IExportFileNameCallback* pFileNameCallback)
{
	// set file name callback
	this->m_pFileNameCallback=pFileNameCallback;
}

CExporter::CExportedPatch* CExporter::GetExportedPatch(unsigned int nPatchX,unsigned int nPatchY)
{
	CExporter::PatchIndex ind;
	ind.first=nPatchX;
	ind.second=nPatchY;
	return &this->m_mapPatches[ind];
}




// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// heightmap to bitmap export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------


CHeightmapExporterBMP::CHeightmapExporterBMP()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_iColorMode=0;
	this->m_bSectorHeightRange=false;
	this->m_strFileFormat="BMP";
	this->m_bFlipY=false;
	this->m_bSizePlus1=false;
}

void CHeightmapExporterBMP::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
}
void CHeightmapExporterBMP::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"ColorMode"))
	{
		this->m_iColorMode=iValue;
	}
	else if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
}
void CHeightmapExporterBMP::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"UseSectorHeightRange"))
	{
		this->m_bSectorHeightRange=bValue;
	}
	else if (EqualNoCase(strParameterName,"SizePlus1"))
	{
		this->m_bSizePlus1=bValue;
	}
	else if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
}

std::string CHeightmapExporterBMP::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"heightmap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("heightmap.")+this->m_strFileFormat;
	}
}

bool CHeightmapExporterBMP::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// compute number of required patches
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	iPatches*=iPatches;

	// add extra pixel
	int iPatchExportSize=this->m_iPatchSize;
	if (this->m_bSizePlus1)
	{
		iPatchExportSize++;
	}

	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector bmp heightmap %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,iPatches*iPatchExportSize);

	// initialize bitmap file
	CBitmapFile bmp;
	if (!bmp.Initialize((this->m_iColorMode==0)?bmfPalette:bmfRGB,iPatchExportSize,iPatchExportSize))
	{
		MessageBox(NULL,"Error initializing bitmap object.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all patches
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// write color data
			int iMaxXIndex=Sector.GetMaxXIndex();
			int iMaxYIndex=Sector.GetMaxYIndex();
			for (int iY=0;iY<iPatchExportSize;iY++)
			{
				for (int iX=0;iX<iPatchExportSize;iX++)
				{
					// get height
					float fHeight=Sector.GetHeight(min(iX+iPatchStartX,iMaxXIndex-1),min(iY+iPatchStartY,iMaxYIndex-1));
					if (fHeight<0)
					{
						// invalid height
						MessageBox(NULL,"Invalid height found within sector's heightmap.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// write pixel
					if (this->m_iColorMode==0)
					{
						// grayscale image
						if (this->m_bSectorHeightRange)
						{
							float fRange=Sector.GetMaxHeight()-Sector.GetMinHeight();
							fHeight-=Sector.GetMinHeight();
							fHeight/=fRange;
							fHeight*=255.;
						}
						else
						{
							float fRange=Terrain.GetMaxHeight()-Terrain.GetMinHeight();
							fHeight-=Terrain.GetMinHeight();
							fHeight/=fRange;
							fHeight*=255.;
						}
						unsigned char ucColor=(unsigned char)max(min((int)fHeight,255),0);
						// set color
						bmp.SetPixel(iX,iY,ucColor);
					}
					else	// colored image
					{
						// get color
						PNPTC_RGB rgbColor;
						switch (this->m_iColorMode)
						{
						case 1:
							// display color
							if (this->m_bSectorHeightRange)
							{
								rgbColor=Sector.GetHeightColor(fHeight);							
							}
							else
							{
								rgbColor=Terrain.GetHeightColor(fHeight);
							}
							break;
						case 2:
							// 24 bit r-g-b
							{ 
								int iHeight=(int)(fHeight/20000.f*16777215.f);
								int iMask=255;
								rgbColor.b=(iHeight & iMask);
								iMask = iMask << 8;
								rgbColor.g=(iHeight & iMask) >> 8;
								iMask = iMask << 8;
								rgbColor.r=(iHeight & iMask) >> 16;
							}
							break;
						case 3:
							// 24 bit b-g-r
							{ 
								int iHeight=(int)(fHeight/20000.f*16777215.f);
								int iMask=255;
								rgbColor.r=(iHeight & iMask);
								iMask = iMask << 8;
								rgbColor.g=(iHeight & iMask) >> 8;
								iMask = iMask << 8;
								rgbColor.b=(iHeight & iMask) >> 16;
							}
							break;
						}

						// set color
						bmp.SetPixel(iX,iY,rgbColor);
					}
				}

				// set progress bar
				ProgressBar.IncrementPos();
			}

			// flip bitmap?
			if (this->m_bFlipY)
			{
				bmp.FlipY();
			}

			// write file
			if (!bmp.Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
			{
				MessageBox(NULL,"Error writing bitmap file.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}
		}
	}

	// ok
	return true;
}

bool CHeightmapExporterBMP::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// set progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain bmp heightmap",Terrain.GetNumSectorsY()*Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize());

	// add extra pixel?
	int iSizePlus1=(this->m_bSizePlus1)?1:0;

	// initialize bitmap file
	CBitmapFile bmp;
	if (!bmp.Initialize((this->m_iColorMode==0)?bmfPalette:bmfRGB,Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize()+iSizePlus1,Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()+iSizePlus1))
	{
		MessageBox(NULL,"Error initializing bitmap object.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all sectors
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector data.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write color data
			unsigned int iMaxXIndex=pSector->GetMaxXIndex();
			unsigned int iMaxYIndex=pSector->GetMaxYIndex();
			for (unsigned int iY=0;iY<Terrain.GetSectorDetailBorderSize()+iSizePlus1;iY++)
			{
				for (unsigned int iX=0;iX<Terrain.GetSectorDetailBorderSize()+iSizePlus1;iX++)
				{
					// get height
					float fHeight=pSector->GetHeight(min(iX,iMaxXIndex-1),min(iY,iMaxYIndex-1));
					if (fHeight<0)
					{
						// invalid height
						MessageBox(NULL,"Invalid height found within sector's heightmap.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// write pixel
					if (this->m_iColorMode==0)
					{
						// grayscale image
						float fRange=Terrain.GetMaxHeight()-Terrain.GetMinHeight();
						fHeight-=Terrain.GetMinHeight();
						fHeight/=fRange;
						fHeight*=255.;
						unsigned char ucColor=(unsigned char)max(min((int)fHeight,255),0);
						// set color
						bmp.SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
							         nSecY*Terrain.GetSectorDetailBorderSize()+iY,
									 ucColor);
					}
					else	// colored image
					{
						// get color
						PNPTC_RGB rgbColor;
						switch (this->m_iColorMode)
						{
						case 1:
							// display color
							rgbColor=Terrain.GetHeightColor(fHeight);
							break;
						case 2:
							// 24 bit r-g-b
							{ 
								int iHeight=(int)(fHeight/20000.f*16777215.f);
								int iMask=255;
								rgbColor.b=(iHeight & iMask);
								iMask = iMask << 8;
								rgbColor.g=(iHeight & iMask) >> 8;
								iMask = iMask << 8;
								rgbColor.r=(iHeight & iMask) >> 16;
							}
							break;
						case 3:
							// 24 bit b-g-r
							{ 
								int iHeight=(int)(fHeight/20000.f*16777215.f);
								int iMask=255;
								rgbColor.r=(iHeight & iMask);
								iMask = iMask << 8;
								rgbColor.g=(iHeight & iMask) >> 8;
								iMask = iMask << 8;
								rgbColor.b=(iHeight & iMask) >> 16;
							}
							break;
						}

						// set color
						bmp.SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
							         nSecY*Terrain.GetSectorDetailBorderSize()+iY,
									 rgbColor);
					}
				}

				// set progress bar
				ProgressBar.IncrementPos();
			}
		}
	}

	// flip bitmap?
	if (this->m_bFlipY)
	{
		bmp.FlipY();
	}

	// write file
	if (!bmp.Write(ConvertToWindowsFormat(Filename),this->m_strFileFormat))
	{
		MessageBox(NULL,"Error writing bitmap file.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	return true;
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// normalmap to bitmap export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------


CNormalmapExporterBMP::CNormalmapExporterBMP()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_strFileFormat="BMP";
	this->m_strRGB="XYZ";
	this->m_iRGB[0]=0;
	this->m_iRGB[1]=1;
	this->m_iRGB[2]=2;
	this->m_bFlipY=false;
	this->m_bPowerOfTwo=false;
	this->m_iBorderPixel=0;
}

void CNormalmapExporterBMP::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
	else if (EqualNoCase(strParameterName,"RGB"))
	{
		this->m_strRGB=strValue;
		// set up indices
		for (int i=0;i<3;i++)
		{
			std::string strRGBIndex(this->m_strRGB.substr(i,1));
			if (EqualNoCase(strRGBIndex,"X"))
			{
				this->m_iRGB[i]=0;
			}
			else if (EqualNoCase(strRGBIndex,"Y"))
			{
				this->m_iRGB[i]=1;
			}
			else if (EqualNoCase(strRGBIndex,"Z"))
			{
				this->m_iRGB[i]=2;
			}
		}
	}
}
void CNormalmapExporterBMP::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=minmax(iValue,0,2);
	}
}

void CNormalmapExporterBMP::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
	else if (EqualNoCase(strParameterName,"PowerOfTwo"))
	{
		this->m_bPowerOfTwo=bValue;
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=(bValue)?2:0;
	}
}

std::string CNormalmapExporterBMP::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"normalmap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("normalmap.")+this->m_strFileFormat;
	}
}

unsigned char NormalizedFloat2Char(float& fValue)
{
	// convert to char
	return static_cast<unsigned char>(minmax(static_cast<int>(fValue*128.f+128.f),0,255));
}

bool CNormalmapExporterBMP::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// compute number of required patches
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	iPatches*=iPatches;

	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector bmp normalmap %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,iPatches*this->m_iPatchSize);

	// go through all patches
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// initialize bitmap file
			CBitmapFile bmp;
			if (!bmp.Initialize(bmfRGB,this->m_iPatchSize+this->m_iBorderPixel,this->m_iPatchSize+this->m_iBorderPixel))
			{
				MessageBox(NULL,"Error initializing bitmap object.","Error exporting sector normalmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get start position offset
			int iStartOffset=(this->m_iBorderPixel==2)?-1:0;

			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// write color data
			for (int iY=0;iY<static_cast<int>(bmp.GetSizeY());iY++)
			{
				for (int iX=0;iX<static_cast<int>(bmp.GetSizeX());iX++)
				{
					// get normal
					PnPTC3DVec vecNormal=Sector.GetNormal(iX+iPatchStartX+iStartOffset,iY+iPatchStartY+iStartOffset);

					// create color value
					PNPTC_RGB rgbColor(128,128,128);
					rgbColor.r=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[0]]);
					rgbColor.g=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[1]]);
					rgbColor.b=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[2]]);

					// set color
					bmp.SetPixel(iX,iY,rgbColor);
				}

				// set progress bar
				ProgressBar.IncrementPos();
			}

			// flip bitmap?
			if (this->m_bFlipY)
			{
				bmp.FlipY();
			}

			// test best bitmap size
			if (this->m_bPowerOfTwo)
			{
				unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
				unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
				bmp.Resize(nSaveSizeX,nSaveSizeY);
			}

			// write file
			if (!bmp.Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
			{
				MessageBox(NULL,"Error writing bitmap file.","Error exporting sector normalmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

		} // for iPatchStartX
	} // for iPatchStartX

	// ok
	return true;
}

bool CNormalmapExporterBMP::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// set progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain bmp normalmap",Terrain.GetNumSectorsY()*Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize());

	// initialize bitmap file
	CBitmapFile bmp;
	if (!bmp.Initialize(bmfRGB,Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize(),Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()))
	{
		MessageBox(NULL,"Error initializing bitmap object.","Error exporting terrain normalmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all sectors
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector data.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write color data
			for (unsigned int iY=0;iY<Terrain.GetSectorDetailBorderSize();iY++)
			{
				for (unsigned int iX=0;iX<Terrain.GetSectorDetailBorderSize();iX++)
				{
					// get normal
					PnPTC3DVec vecNormal=pSector->GetNormal(iX,iY);

					// create color value
					PNPTC_RGB rgbColor(128,128,128);
					rgbColor.r=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[0]]);
					rgbColor.g=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[1]]);
					rgbColor.b=NormalizedFloat2Char(((float*)&vecNormal.x)[this->m_iRGB[2]]);

					// set color
					bmp.SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
						         nSecY*Terrain.GetSectorDetailBorderSize()+iY,
								 rgbColor);
				}

				// set progress bar
				ProgressBar.IncrementPos();
			}
		}
	}

	// flip bitmap?
	if (this->m_bFlipY)
	{
		bmp.FlipY();
	}

	// write file
	if (!bmp.Write(ConvertToWindowsFormat(Filename),this->m_strFileFormat))
	{
		MessageBox(NULL,"Error writing bitmap file.","Error exporting terrain normalmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// ok
	return true;
}




// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// heightmap to raw export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CHeightmapExporterRAW::CHeightmapExporterRAW()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_bOffsetToZero=false;
	this->m_bSizePlus1=false;
	this->m_iFormat=0;
	this->m_fFactor=1.f;
	this->m_bFullScale=false;
	this->m_bFlipY=false;
	this->m_fOffset=0;
}

void CHeightmapExporterRAW::SetParameter(const std::string& strParameterName,float fValue)
{
	if (EqualNoCase(strParameterName,"Factor"))
	{
		this->m_fFactor=fValue;
	}
	else if (EqualNoCase(strParameterName,"Offset"))
	{
		this->m_fOffset=fValue;
	}
}
void CHeightmapExporterRAW::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"Format"))
	{
		this->m_iFormat=iValue;
	}
	else if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
}
void CHeightmapExporterRAW::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"OffsetToZero"))
	{
		this->m_bOffsetToZero=bValue;
	}
	else if (EqualNoCase(strParameterName,"SizePlus1"))
	{
		this->m_bSizePlus1=bValue;
	}
	else if (EqualNoCase(strParameterName,"FullScale"))
	{
		this->m_bFullScale=bValue;
	}
	else if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
}

std::string CHeightmapExporterRAW::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"heightmap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),"RAW");
		return cSector;
	}
	else
	{
		return std::string("heightmap.raw");
	}
}

inline float GetRAWMaxValue(int iFormat)
{
	// return maximum value for the RAW format
	switch (iFormat)
	{
	case 0:
		return 1.f;
		break;
	case 1:
		return 4294967295.f;
		break;
	case 2:
		return 65535.f;
		break;
	case 3:
		return 255.f;
		break;
	default:
		return 0.f;
		break;
	}
}

inline long GetRAWDataSize(int iFormat)
{
	// return data size of used RAW format
	switch (iFormat)
	{
	case 0:
		return sizeof(float);
		break;
	case 1:
		return sizeof(DWORD);
		break;
	case 2:
		return sizeof(WORD);
		break;
	case 3:
		return sizeof(BYTE);
		break;
	default:
		return 0;
		break;
	}
}

inline bool WriteRAWHeight(FILE* file,int iFormat,float fHeight)
{
	// write format dependend height
	switch (iFormat)
	{
	case 0:
		{
			if (!fwrite(&fHeight,sizeof(float),1,file))
			{
				return false;
			}
		}
		break;
	case 1:
		{
			DWORD dwHeight=(DWORD)fHeight;
			if (!fwrite(&dwHeight,sizeof(DWORD),1,file))
			{
				return false;
			}
		}
		break;
	case 2:
		{
			WORD wHeight=(WORD)fHeight;
			if (!fwrite(&wHeight,sizeof(WORD),1,file))
			{
				return false;
			}
		}
		break;
	case 3:
		{
			BYTE byHeight=(BYTE)fHeight;
			if (!fwrite(&byHeight,sizeof(BYTE),1,file))
			{
				return false;
			}
		}
		break;
	default:
		return false;
		break;
	}

	// ok
	return true;
}

bool CHeightmapExporterRAW::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// patch size to be written
	int iPatchSizeWrite=this->m_iPatchSize;
	if (this->m_bSizePlus1) 
	{
		iPatchSizeWrite++;
	}

	// compute number of required patches
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	iPatches*=iPatches;

	// offset to zero, if full scale is used
	if (this->m_bFullScale)
	{
		this->m_bOffsetToZero=true;
	}

	// get offset value
	if (this->m_bOffsetToZero)
	{
		this->m_fOffset=-Terrain.GetMinHeight();
	}

	// compute full scale from terrain height range
	if (this->m_bFullScale)
	{
		this->m_fFactor=(1.f/(Terrain.GetMaxHeight()-Terrain.GetMinHeight()))*GetRAWMaxValue(this->m_iFormat);
	}

	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector RAW heightmap %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,iPatches*iPatchSizeWrite);

	// go through all patches
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// create file
			FILE* f=fopen(ConvertToWindowsFormat(strFileName).c_str(),"wb");
			if (!f)
			{
				static char cMessage[1000];
				sprintf(cMessage,"The file '%s'\ncould not be opened for writing.",strFileName.c_str());
				MessageBox(NULL,cMessage,"Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// write height data
			for (int iY=0;iY<iPatchSizeWrite;iY++)
			{
				int iYPos=(this->m_bFlipY)?(min(iPatchSizeWrite-iY-1+iPatchStartY,Sector.GetMaxYIndex())):(min(iY+iPatchStartY,Sector.GetMaxYIndex()));
				for (int iX=0;iX<iPatchSizeWrite;iX++)
				{
					int iXPos=min(iX+iPatchStartX,Sector.GetMaxXIndex());
					// get height
					float fHeight=(Sector.GetHeight(iXPos,iYPos)+this->m_fOffset)*this->m_fFactor;
					if (fHeight<0)
					{
						// invalid height
						MessageBox(NULL,"Invalid height found within sector's heightmap.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}

					// write height
					if (!WriteRAWHeight(f,this->m_iFormat,fHeight))
					{
						MessageBox(NULL,"Error writing height data to raw file.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						fclose(f);
						return false;
					}
				}

				// set progress bar
				ProgressBar.IncrementPos();
			}

			// close file
			fclose(f);
		}
	}

	// ok
	return true;
}

bool CHeightmapExporterRAW::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{

	// create increment variable for size+1
	int iSizePlus1=0;
	if (this->m_bSizePlus1)
	{
		iSizePlus1=1;
	}

	// get data element size
	long lDataElementSize=GetRAWDataSize(this->m_iFormat);

	// get terrain sizes
	int iSectorBorderSize=Terrain.GetSectorDetailBorderSize();
	int iTerrainBorderSizeX=Terrain.GetNumSectorsX()*iSectorBorderSize+iSizePlus1;

	// offset to zero, if full scale is used
	if (this->m_bFullScale)
	{
		this->m_bOffsetToZero=true;
	}

	// get offset value
	if (this->m_bOffsetToZero)
	{
		this->m_fOffset=-Terrain.GetMinHeight();
	}

	// compute full scale from terrain height range
	if (this->m_bFullScale)
	{
		this->m_fFactor=(1.f/(Terrain.GetMaxHeight()-Terrain.GetMinHeight()))*GetRAWMaxValue(this->m_iFormat);
	}

	// set progress bar
	int iNumIncrements=Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY()*iSectorBorderSize;
	if (this->m_bSizePlus1)
	{
		iNumIncrements+=Terrain.GetNumSectorsX();
	}
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain RAW heightmap",iNumIncrements);

	// create file
	FILE* f=fopen(ConvertToWindowsFormat(Filename).c_str(),"w+b");
	if (!f)
	{
		static char cMessage[1000];
		sprintf(cMessage,"The file '%s'\ncould not be opened for writing.",Filename.c_str());
		MessageBox(NULL,cMessage,"Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// export all sectors
	unsigned int nSector=0;
	for (unsigned int nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned int nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++,nSector++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (pSector)
			{
				// export sector
				for (int iY=0;iY<iSectorBorderSize+iSizePlus1;iY++)
				{
					// set file position
					long lPos=nSecX*iSectorBorderSize;
					lPos+=nSecY*iSectorBorderSize*iTerrainBorderSizeX;
					lPos+=iY*iTerrainBorderSizeX;
					lPos*=lDataElementSize;
					fseek(f,lPos,SEEK_SET);

					int iYPos=(this->m_bFlipY)?(min(iSectorBorderSize+iSizePlus1-iY-1,pSector->GetMaxYIndex())):(min(iY,pSector->GetMaxYIndex()));

					for (int iX=0;iX<iSectorBorderSize+iSizePlus1;iX++)
					{
						int iXPos=min(iX,pSector->GetMaxXIndex());

						// get height
						float fHeight=(pSector->GetHeight(iXPos,iYPos)+this->m_fOffset)*this->m_fFactor;
						if (fHeight<0)
						{
							// invalid height
							MessageBox(NULL,"Invalid height found within sector's heightmap.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}

						// write height
						if (!WriteRAWHeight(f,this->m_iFormat,fHeight))
						{
							MessageBox(NULL,"Error writing height data to raw file.","Error exporting sector heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							fclose(f);
							return false;
						}
					}

					// set progress bar
					ProgressBar.IncrementPos();
				}
			}
			else
			{
				MessageBox(NULL,"Error reading sector data.","Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				fclose(f);
				return false;
			}
		}
	}

	// close file
	fclose(f);

	// ok
	return true;
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// texturemap to bmp export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CTexturemapExporterBMP::CTexturemapExporterBMP()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_iBorderPixel=0;
	this->m_strFileFormat="BMP";
	this->m_strLightMapName="";
	this->m_bFlipY=false;
	this->m_bPowerOfTwo=false;
	this->m_nBorderSize=0;
}

void CTexturemapExporterBMP::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
	else if (EqualNoCase(strParameterName,"LightMapName"))
	{
		this->m_strLightMapName=strValue;
	}
}
void CTexturemapExporterBMP::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=minmax(iValue,0,2);
	}
}
void CTexturemapExporterBMP::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=(bValue)?2:0;
	}
	else if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
	else if (EqualNoCase(strParameterName,"PowerOfTwo"))
	{
		this->m_bPowerOfTwo=bValue;
	}
}

std::string CTexturemapExporterBMP::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"texturemap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("texturemap.")+this->m_strFileFormat;
	}
}

bool CTexturemapExporterBMP::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// get texture object
	CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
	CIPnPTCTextureMap01* pTexMap=(pTextures)?pTextures->GetTextureMap():NULL;
	CIPnPTCTexture01* pTexture=(pTexMap)?pTexMap->GetTexture(0,0):NULL;
	if (!pTextures || !pTexMap || !pTexture)
	{
		MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// get lightmap object
	CIPnPTCLightMap01* pLightMap=Sector.GetLightMap(this->m_strLightMapName.c_str());	

	// get number of patches per border
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;

	// compute column and row size (The number of bytes per row must be a multiple of 4)
	// every single texture within the texture maps overlap with the surrounding texture
	// by one pixel; that's why we have the -2 to avoid double pixels when combining them
	// into one single texture. Around the outer border of the large destination texture 
	// we want to preserve this extra pixel; that's why we finally have the +2.
	unsigned int nHeight=(pTexture->GetHeight()-2)*pTexMap->GetNumTexturesPerSectorBorder()+2;
	unsigned int nWidth =(pTexture->GetWidth()-2)*pTexMap->GetNumTexturesPerSectorBorder()+2;

	// compute size of final textures
	unsigned int nTexBorderSize=(nHeight-2)/iPatches;
	int iPatchFactor=static_cast<int>(nTexBorderSize);
	int iBorderOffset=1;
	if (this->m_iBorderPixel==1)
	{
		iBorderOffset=1;
		nTexBorderSize+=1;
	}
	else if (this->m_iBorderPixel==2)
	{
		iBorderOffset=0;
		nTexBorderSize+=2;
	}

	// initialize progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector texture map %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,pTexMap->GetNumTexturesPerSectorBorder()*pTexMap->GetNumTexturesPerSectorBorder()+iPatches*iPatches*nTexBorderSize);

	// create sector bitmap
	CBitmapFile bmpSector;
	if (!bmpSector.Initialize(bmfRGB,nWidth,nHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all textures
	for (int iTexY=0;iTexY<pTexMap->GetNumTexturesPerSectorBorder();++iTexY)
	{
		for (int iTexX=0;iTexX<pTexMap->GetNumTexturesPerSectorBorder();++iTexX)
		{
			// get texture
			pTexture=pTexMap->GetTexture(iTexX,iTexY);
			if (!pTexture)
			{
				MessageBox(NULL,"Error reading texture data.","Error exporting sector texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get lightmap
			CIPnPTCTexture01* pTexLightMap=NULL;
			if (pLightMap)
			{
				pTexLightMap=pLightMap->GetTexture(iTexX,iTexY);
				if (pTexLightMap)
				{
					// check dimensions
					if (pTexture->GetWidth()!=pTexLightMap->GetWidth() ||
						pTexture->GetHeight()!=pTexLightMap->GetHeight())
					{
						MessageBox(NULL,"Export not possible: The texture map and the light map have different dimensions.\nPlease create both maps with the same number of pixels per heightmap element.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}
				}
			}

			// go through whole texture
			for (int iPixY=0;iPixY<pTexture->GetHeight();++iPixY)
			{
				for (int iPixX=0;iPixX<pTexture->GetWidth();++iPixX)
				{
					// get pixel color
					PNPTC_RGB rgb=pTexture->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);
					if (pTexLightMap)
					{
						PNPTC_RGB rgbLightMap=pTexLightMap->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);
						rgb.r=static_cast<unsigned char>(((static_cast<float>(rgb.r)/255.f)*(static_cast<float>(rgbLightMap.r)/255.f))*255.f);
						rgb.g=static_cast<unsigned char>(((static_cast<float>(rgb.g)/255.f)*(static_cast<float>(rgbLightMap.g)/255.f))*255.f);
						rgb.b=static_cast<unsigned char>(((static_cast<float>(rgb.b)/255.f)*(static_cast<float>(rgbLightMap.b)/255.f))*255.f);
					}

					// write pixel
					bmpSector.SetPixel(iPixX+iTexX*(pTexture->GetWidth()-2),iPixY+iTexY*(pTexture->GetHeight()-2),rgb);
				}
			}

			// increment progress bar
			ProgressBar.IncrementPos();
		}
	}

	// fill and write bitmaps
	for (int iPatchY=0,iPatch=0;iPatchY<iPatches;iPatchY++)
	{
		for (int iPatchX=0;iPatchX<iPatches;iPatchX++,iPatch++)
		{
			// create bitmap
			CBitmapFile bmp;
			if (!bmp.Initialize(bmfRGB,nTexBorderSize,nTexBorderSize))
			{
				MessageBox(NULL,"Error creating bitmap file.","Error exporting sector texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// set all pixel
			for (unsigned int nY=0;nY<nTexBorderSize;++nY)
			{
				for (unsigned int nX=0;nX<nTexBorderSize;++nX)
				{
					// get color value
					PNPTC_RGB rgb;
					bmpSector.GetPixel(iPatchX*iPatchFactor+nX+iBorderOffset,iPatchY*iPatchFactor+nY+iBorderOffset,rgb);

					// write pixel
					bmp.SetPixel(nX,nY,rgb);
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}

			// flip bitmap?
			if (this->m_bFlipY)
			{
				bmp.FlipY();
			}

			// test bitmap size
			if (this->m_bPowerOfTwo)
			{
				unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
				unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
				bmp.Resize(nSaveSizeX,nSaveSizeY);
			}
			this->m_nBorderSize=bmp.GetSizeX();

			// write texture to file
			if (!bmp.Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
			{
				MessageBox(NULL,"Error writing bitmap file.","Error exporting sector texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

		} // for iPatchX
	} // for iPatchY

	// ok
	return true;
}

bool CTexturemapExporterBMP::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// get first sector
	CIPnPTCSector01* pFirstSector=Terrain.GetSectorDetail(0,0);
	if (!pFirstSector)
	{
		MessageBox(NULL,"Error reading detail data of first sector.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// get texture object
	CIPnPTCSectorTextures01* pTextures=pFirstSector->GetSectorTextures();
	CIPnPTCTextureMap01* pTexMap=(pTextures)?pTextures->GetTextureMap():NULL;
	CIPnPTCTexture01* pTexture=(pTexMap)?pTexMap->GetTexture(0,0):NULL;
	if (!pTextures || !pTexMap || !pTexture)
	{
		MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// get height and with of every single texture
	DWORD dwSingleTextureWidth=pTexture->GetWidth();
	DWORD dwSingleTextureHeight=pTexture->GetHeight();

	// compute column and row size (The number of bytes per row must be a multiple of 4)
	// every single texture within the texture maps overlap with the surrounding texture
	// by one pixel; that's why we have the -2 to avoid double pixels when combining them
	// into one single texture. 
	DWORD dwHeight=(dwSingleTextureHeight-2)*pTexMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsY();
	DWORD dwWidth =(dwSingleTextureWidth-2)*pTexMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsX();

	// initialize progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain texture map",pTexMap->GetNumTexturesPerSectorBorder()*pTexMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY());

	// create mem
	CBitmapFile bmp;
	if (!bmp.Initialize(bmfRGB,dwWidth,dwHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all sectors
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector detail data.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get texture object for this sector
			CIPnPTCSectorTextures01* pTextures=pSector->GetSectorTextures();
			CIPnPTCTextureMap01* pTexMap=(pTextures)?pTextures->GetTextureMap():NULL;
			CIPnPTCTexture01* pTexture=(pTexMap)?pTexMap->GetTexture(0,0):NULL;
			if (!pTextures || !pTexMap || !pTexture)
			{
				MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
				return false;
			}

			// get lightmap object
			CIPnPTCLightMap01* pLightMap=pSector->GetLightMap(this->m_strLightMapName.c_str());	

			// go through all textures
			DWORD dwOffset=0;
			DWORD dwLastWidth=0;
			DWORD dwLastHeight=0;
			for (int iTexY=0;iTexY<pTexMap->GetNumTexturesPerSectorBorder();iTexY++)
			{
				// get pixel range within texture
				int iPixStartY=(iTexY==0)?0:1;
				int iPixEndY=(iTexY==pTexMap->GetNumTexturesPerSectorBorder()-1)?0:-1;

				for (int iTexX=0;iTexX<pTexMap->GetNumTexturesPerSectorBorder();iTexX++)
				{
					// get pixel range within texture
					int iPixStartX=(iTexX==0)?0:1;
					int iPixEndX=(iTexX==pTexMap->GetNumTexturesPerSectorBorder()-1)?0:-1;

					// get texture
					pTexture=pTexMap->GetTexture(iTexX,iTexY);
					if (!pTexture)
					{
						MessageBox(NULL,"Error reading texture data.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// get lightmap
					CIPnPTCTexture01* pTexLightMap=NULL;
					if (pLightMap)
					{
						pTexLightMap=pLightMap->GetTexture(iTexX,iTexY);
						if (pTexLightMap)
						{
							// check dimensions
							if (pTexture->GetWidth()!=pTexLightMap->GetWidth() ||
								pTexture->GetHeight()!=pTexLightMap->GetHeight())
							{
								MessageBox(NULL,"Export not possible: The texture map and the light map have different dimensions.\nPlease create both maps with the same number of pixels per heightmap element.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
								return false;
							}
						}
					}

					// check texture size
					if (dwSingleTextureWidth!=pTexture->GetWidth() ||
						dwSingleTextureHeight!=pTexture->GetHeight() )
					{
						MessageBox(NULL,"The size of one texture dosen't match.\nThis might be an internal error, please contact the support.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// go through whole texture
					for (int iPixY=1;iPixY<(pTexture->GetHeight()-1);iPixY++)
					{
						for (int iPixX=1;iPixX<(pTexture->GetWidth()-1);iPixX++)
						{
							// get pixel color
							PNPTC_RGB rgb=pTexture->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);
							if (pTexLightMap)
							{
								PNPTC_RGB rgbLightMap=pTexLightMap->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);
								rgb.r=static_cast<unsigned char>(((static_cast<float>(rgb.r)/255.f)*(static_cast<float>(rgbLightMap.r)/255.f))*255.f);
								rgb.g=static_cast<unsigned char>(((static_cast<float>(rgb.g)/255.f)*(static_cast<float>(rgbLightMap.g)/255.f))*255.f);
								rgb.b=static_cast<unsigned char>(((static_cast<float>(rgb.b)/255.f)*(static_cast<float>(rgbLightMap.b)/255.f))*255.f);
							}

							// write pixel
							bmp.SetPixel(nSecX*pTexMap->GetNumTexturesPerSectorBorder()*(dwSingleTextureWidth-2)+iTexX*(dwSingleTextureWidth-2)+iPixX-1,
								         nSecY*pTexMap->GetNumTexturesPerSectorBorder()*(dwSingleTextureHeight-2)+iTexY*(dwSingleTextureHeight-2)+iPixY-1,
										 rgb);
						}
					}

					// increment progress bar
					ProgressBar.IncrementPos();
				}
			}
		}
	}

	// flip bitmap?
	if (this->m_bFlipY)
	{
		bmp.FlipY();
	}

	// test best bitmap size
	if (this->m_bPowerOfTwo)
	{
		unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
		unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
		bmp.Resize(nSaveSizeX,nSaveSizeY);
	}
	this->m_nBorderSize=bmp.GetSizeX();

	// write texture to file
	if (!bmp.Write(ConvertToWindowsFormat(Filename),this->m_strFileFormat))
	{
		MessageBox(NULL,"Error writing bitmap file.","Error exporting terrain texture map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}	

	// ok
	return true;
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// texturemap to indexmap export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CTexturemapExporterIndexmap::CTexturemapExporterIndexmap()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_strFileFormat="BMP";
	this->m_bFlipY=false;
}

void CTexturemapExporterIndexmap::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
}
void CTexturemapExporterIndexmap::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
}
void CTexturemapExporterIndexmap::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
}

std::string CTexturemapExporterIndexmap::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"indexmap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("indexmap.")+this->m_strFileFormat;
	}
}

bool CTexturemapExporterIndexmap::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// compute number of required patches
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	iPatches*=iPatches;

	// get texture object
	CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
	if (!pTextures)
	{
		MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector texture indexmap %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,iPatches*this->m_iPatchSize);

	// compute col and row size
	DWORD dwHeight=Sector.GetBorderSize();
	DWORD dwWidth =Sector.GetBorderSize();

	// create bitmap file
	CBitmapFile bmp;
	if (!bmp.Initialize(bmfPalette,this->m_iPatchSize,this->m_iPatchSize))
	{
		MessageBox(NULL,"Error initializing bitmap file.\nPossibly not enough memory available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// write bitmap(s)
	bool bIndexError=false;
	int iMaxTextureID=-1;
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// get textures
			for (int iY=0;iY<this->m_iPatchSize;iY++)
			{
				for (int iX=0;iX<this->m_iPatchSize;iX++)
				{
					int iTexture=pTextures->GetTexture(iX+iPatchStartX,iY+iPatchStartY);
					iMaxTextureID=max(iMaxTextureID,iTexture);
					if (iTexture<0 || iTexture>255)
					{
						// invalid index
						bIndexError=true;
					}
					else
					{
						// write pixel
						bmp.SetPixel(iX,iY,(BYTE)iTexture);
					}
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}

			// flip bitmap?
			if (this->m_bFlipY)
			{
				bmp.FlipY();
			}

			// write bitmap to file
			if (!bmp.Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
			{
				return false;
			}
		}
	}

	// index error message
	if (bIndexError)
	{
		MessageBox(NULL,"There have been more than 256 texture indices within your terrain.\nThe index map only supports 256 different texture indices.\nSo your output file might differ strongly from the rendering within TC.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
	}

	// ok
	return true;
}

bool CTexturemapExporterIndexmap::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// set progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain texture indexmap",Terrain.GetNumSectorsY()*Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize());

	// create bitmap file
	CBitmapFile bmp;
	if (!bmp.Initialize(bmfPalette,Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize(),Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()))
	{
		MessageBox(NULL,"Error initializing bitmap file.\nPossibly not enough memory available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// write bitmap(s)
	bool bIndexError=false;
	int iMaxTextureID=-1;

	// go through all sectors
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector data.","Error exporting terrain texturemap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get sector textures
			CIPnPTCSectorTextures01* pTextures=pSector->GetSectorTextures();
			if (!pTextures)
			{
				MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
				return false;
			}

			// get textures
			for (unsigned int iY=0;iY<Terrain.GetSectorDetailBorderSize();iY++)
			{
				for (unsigned int iX=0;iX<Terrain.GetSectorDetailBorderSize();iX++)
				{
					int iTexture=pTextures->GetTexture(iX,iY);
					iMaxTextureID=max(iMaxTextureID,iTexture);
					if (iTexture<0 || iTexture>255)
					{
						// invalid index
						bIndexError=true;
					}
					else
					{
						// write pixel
						bmp.SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
							         nSecY*Terrain.GetSectorDetailBorderSize()+iY,
									 (BYTE)iTexture);
					}
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}
		}
	}

	// flip bitmap?
	if (this->m_bFlipY)
	{
		bmp.FlipY();
	}

	// write file
	if (!bmp.Write(ConvertToWindowsFormat(Filename),this->m_strFileFormat))
	{
		MessageBox(NULL,"Error writing bitmap file.","Error exporting terrain texturemap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// index error message
	if (bIndexError)
	{
		MessageBox(NULL,"There have been more than 256 texture indices within your terrain.\nThe index map only supports 256 different texture indices.\nSo your output file might differ strongly from the rendering within TC.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
	}

	// ok
	return true;
}





// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// texturemap to alphamap export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CTexturemapExporterAlphamap::CTexturemapExporterAlphamap()
	: m_iPatchSize(1024)
	, m_strFileFormat("BMP")
	, m_strAlphamapFormat("")
	, m_bFlipY(false)
	, m_iMaxNumChannels(1)
	, m_fScaleFactor(1)
	, m_iSmoothFactor(0)
	, m_bNoBlackFiles(false)
	, m_bExportMaterialFiles(true)
	, m_iBorderPixel(0)
	, m_GaussSmoothMax(false)
	, m_nBorderSize(0)
{
}

void CTexturemapExporterAlphamap::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
	else if (EqualNoCase(strParameterName,"AlphamapFormat"))
	{
		this->m_strAlphamapFormat=strValue;
	}

	else if (EqualNoCase(strParameterName,"PLSM2RelativeMapDir"))
	{
		this->m_strPLSM2RelativeMapDir=strValue;
	}
	else if (EqualNoCase(strParameterName,"PLSM2TerrainName"))
	{
		this->m_strPLSM2TerrainName=strValue;
	}
	else if (EqualNoCase(strParameterName,"PLSM2DetailTextureFormat"))
	{
		this->m_strPLSM2DetailTextureFormat=strValue;
	}
	else if (EqualNoCase(strParameterName,"PLSM2DetailTextureBasePath"))
	{
		this->m_strPLSM2DetailTextureBasePath=strValue;
	}
	else if (EqualNoCase(strParameterName,"PLSM2NormalMap"))
	{
		this->m_strPLSM2NormalMap=strValue;
	}
	else if (EqualNoCase(strParameterName,"PLSM2TextureMap"))
	{
		this->m_strPLSM2TextureMap=strValue;
	}
}
void CTexturemapExporterAlphamap::SetParameter(const std::string& strParameterName,float fValue)
{
	if (EqualNoCase(strParameterName,"ScaleFactor"))
	{
		this->m_fScaleFactor=minmax(fValue,0.01f,100.f);
	}
}
void CTexturemapExporterAlphamap::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
	else if (EqualNoCase(strParameterName,"SmoothFactor"))
	{
		this->m_iSmoothFactor=iValue;
	}	
	else if (EqualNoCase(strParameterName,"MaxNumChannels"))
	{
		this->m_iMaxNumChannels=minmax(iValue,0,4);
	}
	else if (EqualNoCase(strParameterName,"ScaleFactor"))
	{
		this->m_fScaleFactor=minmax(static_cast<float>(iValue),0.01f,100.f);
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=minmax(iValue,0,2);
	}
}

void CTexturemapExporterAlphamap::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"NoBlackFiles"))
	{
		this->m_bNoBlackFiles=bValue;
	}
	else if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
	else if (EqualNoCase(strParameterName,"PowerOfTwo"))
	{
		this->m_bPowerOfTwo=bValue;
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=(bValue)?2:0;
	}
	else if (EqualNoCase(strParameterName,"ExportMaterialFiles"))
	{
		this->m_bExportMaterialFiles=bValue;
	}
	else if (EqualNoCase(strParameterName,"GaussSmoothMax"))
	{
		this->m_GaussSmoothMax=bValue;
	}
}

std::string CTexturemapExporterAlphamap::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"alphamap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("alphamap.")+this->m_strFileFormat;
	}
}

typedef std::map<unsigned int,std::string> DetailTextures;

bool GetDetailTexture(CTempFileManager& tempFileManager,DetailTextures& textures,unsigned int nTextureID,CBitmapFile& bmpDetail,CIPnPTCTerrain01& Terrain,unsigned int nSizeX,unsigned int nSizeY)
{
	// try to find the texture
	DetailTextures::iterator itTempFile=textures.find(nTextureID);
	if (itTempFile!=textures.end())
	{
		// try to load the bitmap
		if (bmpDetail.Read(itTempFile->second))
		{
			return true;
		}
	}

	// get texture
	CIPnPTCTexture01* pTexture=Terrain.GetTexture(nTextureID);
	if (pTexture)
	{
		// create detail map
		if (bmpDetail.Initialize(pTexture,bmfRGBA) && bmpDetail.Resize(nSizeX,nSizeY,CBitmapFile::Lanczos3))
		{
			// successful => try to store it
			std::string strTempFile=tempFileManager.CreateTempFile("pnp");
			if (bmpDetail.Write(strTempFile,"PNG"))
			{
				textures[nTextureID]=strTempFile;
			}

			// ok
			return true;
		}
		else
		{
			// error
			return false;
		}
	}
	else
	{
		// error
		return false;
	}
}

bool CTexturemapExporterAlphamap::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// set target dimension. 
	int iTargetBmpWidth = static_cast<int>(static_cast<float>(this->m_iPatchSize)*this->m_fScaleFactor);
	int iTargetBmpHeight = static_cast<int>(static_cast<float>(this->m_iPatchSize)*this->m_fScaleFactor);
	int iBorder=this->m_iSmoothFactor;
	int iWorkingBmpWidth=iTargetBmpWidth+iBorder*2+this->m_iBorderPixel;
	int iWorkingBmpHeight=iTargetBmpHeight+iBorder*2+this->m_iBorderPixel;
	int iExtraPixelLeft=(this->m_iBorderPixel>1)?1:0;
	int iExtraPixelRight=(this->m_iBorderPixel>0)?1:0;
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	bool bIntegrateDetailTexture=false;

	// create a temp file manager for the detail textures
	CTempFileManager tempFiles;
	DetailTextures mapDetailTempFiles;

	// get texture object
	CIPnPTCSectorTextures01* pTextures=Sector.GetSectorTextures();
	if (!pTextures)
	{
		MessageBox(NULL,"Could not read sector textures.\nNo textures available.","Error exporting textures",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// set progress bar
	char cProgressBar[100];
	sprintf(cProgressBar,"Exporting sector texture alpha map %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cProgressBar,iPatches*iPatches*iWorkingBmpHeight*pTextures->GetNumUsedTextures());

	// get bitmap format
	BITMAPFORMAT bmf=bmfPalette;
	switch (this->m_iMaxNumChannels)
	{
	case 3:
		bmf=bmfRGB;
		break;
	case 4:
		bmf=bmfRGBA;
		break;
	default:
		bmf=bmfPalette;
		break;
	}
	if (this->m_strAlphamapFormat=="3DGS")
	{
		bmf=bmfPalette;
		bIntegrateDetailTexture=true;
	}

	// create bitmap
	CBitmapFile bmp;
	if (!bmp.Initialize(bmf,iWorkingBmpWidth,iWorkingBmpHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// compute scaling between patch size and bitmap size
	float fXScale = static_cast<float>(this->m_iPatchSize) / static_cast<float>(iTargetBmpWidth);
	float fYScale = static_cast<float>(this->m_iPatchSize) / static_cast<float>(iTargetBmpHeight);

	// go through all patches of this sector
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// list of all create coverage map and detail textures
			std::vector<std::string> vecCoverageMaps;
			std::vector<std::string> vecDetailTextures;

			// default file name
			std::string strFileName=Filename;

			// the current texture channel
			int iTextureChannel=0;

			// whether or not the bitmap contains data
			bool bBlackFile=true;

			// -----------------------------------------------------------------------
			// write coverage maps

			// go through all textures of the sector
			for (unsigned int nUsedTexture=0;nUsedTexture<pTextures->GetNumUsedTextures();nUsedTexture++)
			{
				// get ID and name of this texture
				PnPTC_UsedTexture UsedTexture=pTextures->GetUsedTexture(nUsedTexture);
				WORD wTextureID=UsedTexture.ID;
				std::string strTextureID(UsedTexture.Name);
				if (this->m_strAlphamapFormat=="PLSM2")
				{
					strTextureID=OgreUtils::ValidateOgreEntityName(UsedTexture.Name,true);
				}
				std::string strTexture;
				std::string strTextureExtension;
				RemoveFileExtension(strTextureID,strTexture,strTextureExtension);

				// get file name
				strFileName=Filename;
				if (this->m_pFileNameCallback)
				{
					// get filename from callback object
					std::string strExtraData;
					if (this->m_iMaxNumChannels==1)
					{
						strExtraData=strTexture;
					}
					else
					{
						strExtraData=ToString(static_cast<int>(vecCoverageMaps.size()));
					}
					strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY,strExtraData);
				}
				else
				{	
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					if (strExtension.empty())
					{
						strExtension=this->m_strFileFormat;
					}
					if (this->m_iPatchSize!=Sector.GetBorderSize() || GetFileName(strFileName).empty())
					{
						char cPatch[100];
						if (strFileName.empty())
						{
							sprintf(cPatch,"%05d_%05d_%03d",Sector.GetPosX(),Sector.GetPosY(),iPatch);
						}
						else
						{
							sprintf(cPatch,"_%03d",iPatch);
						}
						strFileName+=cPatch;
					}
					if (this->m_iMaxNumChannels==1)
					{
						strFileName=strTexture+"_"+strFileName;
					}
					else
					{
						strFileName="alphamap_"+strFileName+"_"+ToString(static_cast<int>(vecCoverageMaps.size()));
					}
					strFileName=strFolder+strFileName+"."+strExtension;
				}

				// copy bytes to scaled bitmap 
				bool bBlackChannel=true;
				for (int iY=0;iY<iWorkingBmpHeight;iY++)
				{ 
					// get texture coordinate
					int iTexY = static_cast<int>(floor(static_cast<float>(iY-iBorder-iExtraPixelLeft)*fYScale))+iPatchStartY;

					// check range
					if (iTexY<0 && Sector.GetPosY()==0)
					{
						iTexY=0;
					}
					if (iTexY>=static_cast<int>(Sector.GetBorderSize()) && Sector.GetPosY()==(Terrain.GetNumSectorsY()-1))
					{
						iTexY=Sector.GetBorderSize()-1;
					}

					// x direction
					for (int iX=0;iX<iWorkingBmpWidth;iX++)
					{
						// get texture coordinate
						int iTexX = static_cast<int>(floor(static_cast<float>(iX-iBorder-iExtraPixelLeft)*fXScale))+iPatchStartX;

						// check range
						if (iTexX<0 && Sector.GetPosX()==0)
						{
							iTexX=0;
						}
						if (iTexX>=static_cast<int>(Sector.GetBorderSize()) && Sector.GetPosX()==(Terrain.GetNumSectorsX()-1))
						{
							iTexX=Sector.GetBorderSize()-1;
						}

						// get texture
						int iTexture=pTextures->GetTexture(iTexX,iTexY);
						if (iTexture==static_cast<int>(wTextureID))
						{
							// channel and bitmap contain data
							bBlackFile=false;
							bBlackChannel=false;

							// set that pixel value
							this->SetPixelValue(bmp,iX,(this->m_bFlipY)?(iWorkingBmpHeight-iY-1):iY,(this->m_iMaxNumChannels>1)?iTextureChannel:-1,255);
						}
						else
						{
							this->SetPixelValue(bmp,iX,(this->m_bFlipY)?(iWorkingBmpHeight-iY-1):iY,(this->m_iMaxNumChannels>1)?iTextureChannel:-1,0);
						}
					} // for iX

					// increment progress bar
					ProgressBar.IncrementPos();

				} // for iY

				// was the channel filled with data or was it the last texture of the patch?
				if (!bBlackChannel || (nUsedTexture==pTextures->GetNumUsedTextures()-1))
				{
					// add texture to list of textures, if it was not black
					if (!bBlackChannel)
					{
						std::string strTextureFileName=strTexture;
						if (this->m_strPLSM2DetailTextureFormat.empty())
						{
							strTextureFileName+="."+strTextureExtension;
						}
						else
						{
							strTextureFileName+="."+this->m_strPLSM2DetailTextureFormat;
						}
						vecDetailTextures.push_back(ToLower(strTextureFileName));
					}

					// yes = next channel
					iTextureChannel++;
					
					// all channels filled and finish bitmap?
					if (iTextureChannel>=this->m_iMaxNumChannels || (nUsedTexture==pTextures->GetNumUsedTextures()-1))
					{
						// does the bitmap contain data
						if (!bBlackFile)
						{
							// blurred?
							if (this->m_iSmoothFactor>0)
							{
								// perform blurring
								bmp.BlurGaussian(this->m_iSmoothFactor,this->m_GaussSmoothMax);

								// remove border
								if (!bmp.RemoveBorder(iBorder,iBorder))
								{
									// error
									MessageBox(NULL,"Error exporting texture map.\nCould not shrink texture to appropriate size.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									return false;
								}
							}

							// test best bitmap size
							if (this->m_bPowerOfTwo)
							{
								unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
								unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
								bmp.Resize(nSaveSizeX,nSaveSizeY,CBitmapFile::Mitchell);
							}

							// add detail map to the texture
							if (bIntegrateDetailTexture)
							{
								// get detail texture
								CBitmapFile bmpDetail;
								if (GetDetailTexture(tempFiles,mapDetailTempFiles,wTextureID,bmpDetail,Terrain,bmp.GetSizeX(),bmp.GetSizeY()))
								{
									// fill detail texture data
									for (unsigned int nY=0;nY<bmp.GetSizeY();++nY)
									{
										for (unsigned int nX=0;nX<bmp.GetSizeX();++nX)
										{
											// get detail color
											PNPTC_RGBA rgbaDetail;
											bmpDetail.GetPixel(nX,nY,rgbaDetail);

											// get alphamap value
											BYTE byValue;
											bmp.GetPixel(nX,nY,byValue);

											// store detail map combined with alpha map
											rgbaDetail.a=byValue;
											bmpDetail.SetPixel(nX,nY,rgbaDetail);
										}
									}

									// copy to working bitmap
									bmp.Initialize(bmfRGBA,bmp.GetSizeX(),bmp.GetSizeY());
									bmp.FillFromBitmap(bmpDetail);
								}
								else
								{
									// error
									MessageBox(NULL,"Error exporting texture map.\nCould not get detail texture.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
									return false;
								}
							}

							// save bitmap							
							std::string strTextureFileName = strFileName;
							if (this->m_strAlphamapFormat=="PLSM2")
							{
								strTextureFileName=this->PLSM2GetTextureFileName(&Sector,strFileName,iPatchX,iPatchY,static_cast<int>(vecCoverageMaps.size()));
							}
							Replace(&strTextureFileName,"/","\\");
							this->m_nBorderSize=bmp.GetSizeX();
							if (!bmp.Write(ConvertToWindowsFormat(strTextureFileName),this->m_strFileFormat))
							{
								return false;
							}
							std::string strCoverageMap=this->m_strPLSM2RelativeMapDir+GetFileName(strTextureFileName);
							if (this->m_strAlphamapFormat=="PLSM2")
							{
								Replace(&strCoverageMap,"\\","/");
							}
							vecCoverageMaps.push_back(strCoverageMap);

							// add it to the exported patches
							CExportedPatch* pExportedPatch=this->GetExportedPatch(iPatchX,iPatchY);
							if (pExportedPatch)
							{
								pExportedPatch->AddExportFile(ConvertToWindowsFormat(strTextureFileName));
							}
						}

						// start with first channel in a new bitmap
						bBlackFile=true;
						iTextureChannel=0;
						if (!bmp.Initialize(bmf,iWorkingBmpWidth,iWorkingBmpHeight))
						{
							MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}

					} // bitmap finished
				} // bitmap contains data

				// increment progress bar for every texture and every patch
				ProgressBar.IncrementPos();

			} // for all texture of the sector


			// -----------------------------------------------------------------------
			// write material

			if (this->m_bExportMaterialFiles)
			{
				if (this->m_strAlphamapFormat=="PLSM2")
				{
					this->PLSM2WriteMaterialFile(&Sector,strFileName,vecCoverageMaps,vecDetailTextures,iPatchX,iPatchY);
				}
				else
				{
					this->XMLWriteMaterialFile(&Sector,strFileName,vecCoverageMaps,vecDetailTextures,iPatchX,iPatchY,!bIntegrateDetailTexture);
				}
			}

		} // for PatchX
	} // for PatchY

	// ok
	return true;
}

bool CTexturemapExporterAlphamap::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& strPath)
{
	// set progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain texture alpha map",Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()*Terrain.GetNumTextures());

	// compute col and row size
	unsigned int nHeight=static_cast<int>(static_cast<float>(Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize())*this->m_fScaleFactor);
	unsigned int nWidth =static_cast<int>(static_cast<float>(Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize())*this->m_fScaleFactor);

	// initialize export parameters
	bool bIntegrateDetailTexture=false;

	// get bitmap format
	BITMAPFORMAT bmf=bmfPalette;
	switch (this->m_iMaxNumChannels)
	{
	case 3:
		bmf=bmfRGB;
		break;
	case 4:
		bmf=bmfRGBA;
		break;
	default:
		bmf=bmfPalette;
		break;
	}
	if (this->m_strAlphamapFormat=="3DGS")
	{
		bmf=bmfPalette;
		bIntegrateDetailTexture=true;
	}

	// create bitmap
	CBitmapFile bmp;
	if (!bmp.Initialize(bmf,nWidth,nHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}
	std::string strFileName;

	// list of all create coverage map and detail textures
	std::vector<std::string> vecCoverageMaps;
	std::vector<std::string> vecDetailTextures;

	// initialize texture used?
	bool bBlackFile=true;

	// the current texture channel
	int iTextureChannel=0;

	// go through all textures	
	for (unsigned long nTexture=0;nTexture<Terrain.GetNumTextures();nTexture++)
	{
		// get ID and name of this texture
		CIPnPTCTexture01* pTexture=Terrain.GetTexture(nTexture);
		if (!pTexture)
		{
			MessageBox(NULL,"Error reading texture information.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
			return false;
		}
		std::string strTextureID(pTexture->GetName());
		if (this->m_strAlphamapFormat=="PLSM2")
		{
			strTextureID=OgreUtils::ValidateOgreEntityName(pTexture->GetName(),true);
		}
		std::string strTexture;
		std::string strTextureExtension;
		RemoveFileExtension(strTextureID,strTexture,strTextureExtension);

		// get file name
		strFileName=strPath;
		if (this->m_pFileNameCallback)
		{
			// get filename from callback object
			std::string strExtraData;
			if (this->m_iMaxNumChannels==1)
			{
				strExtraData=strTexture;
			}
			else
			{
				strExtraData=ToString(static_cast<int>(vecCoverageMaps.size()));
			}
			strFileName=this->m_pFileNameCallback->GetExportFileName(NULL,"",0,0,0,strExtraData);
		}
		else
		{	
			if (GetFileName(strFileName).empty())
			{
				std::string strExtension;
				std::string strFolder(GetPath(strFileName));
				RemoveFileExtension(strFileName,strFileName,strExtension);
				if (strExtension.empty())
				{
					strExtension=this->m_strFileFormat;
				}
				if (this->m_iMaxNumChannels==1)
				{
					strFileName="alphamap_"+strTexture+((!strFileName.empty())?("_"+strFileName):"");
				}
				else
				{
					strFileName="alphamap"+((!strFileName.empty())?("_"+strFileName):"")+"_"+ToString(static_cast<int>(vecCoverageMaps.size()));
				}
				strFileName=strFolder+strFileName+"."+strExtension;
			}
		}

		// initialize texture used?
		bool bBlackChannel=true;

		// go through all sectors
		for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
		{
			for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
			{
				// compute sector offset pixel coordinates
				int iSecOffsetX=static_cast<int>(static_cast<float>(nSecX*Terrain.GetSectorDetailBorderSize())*this->m_fScaleFactor);
				int iSecOffsetY=static_cast<int>(static_cast<float>(nSecY*Terrain.GetSectorDetailBorderSize())*this->m_fScaleFactor);

				// get sector
				CIPnPTCSector01* pSec=Terrain.GetSectorDetail(nSecX,nSecY);
				if (pSec)
				{
					// get textures
					CIPnPTCSectorTextures01* pTextures=pSec->GetSectorTextures();
					if (pTextures)
					{
						// go through whole sector
						for (int iY=0;iY<static_cast<int>(ceil(static_cast<float>(pSec->GetBorderSize())*this->m_fScaleFactor));iY++)
						{
							for (int iX=0;iX<static_cast<int>(ceil(static_cast<float>(pSec->GetBorderSize())*this->m_fScaleFactor));iX++)
							{
								// get sector coordinate
								int iScaledX=static_cast<int>(static_cast<float>(iX)/this->m_fScaleFactor);
								int iScaledY=static_cast<int>(static_cast<float>(iY)/this->m_fScaleFactor);
								int iTexture=pTextures->GetTexture(iScaledX,iScaledY);
								if (iTexture==nTexture)
								{
									// texture and channel are not black
									bBlackFile=false;
									bBlackChannel=false;

									// set that pixel value
									this->SetPixelValue(bmp,iSecOffsetX+iX,nHeight-1-(iSecOffsetY+iY),(this->m_iMaxNumChannels>1)?iTextureChannel:-1,255);
								}
								else
								{
									this->SetPixelValue(bmp,iSecOffsetX+iX,nHeight-1-(iSecOffsetY+iY),(this->m_iMaxNumChannels>1)?iTextureChannel:-1,0);
								}
							
							} // for iX

							// increment progress bar
							ProgressBar.IncrementPos();

						} // for iY
					} // if pTextures
				} // if pSec
			} // for SecX
		} // for SecY

		// create bitmap
		if (!bBlackChannel || (nTexture==Terrain.GetNumTextures()-1))
		{
			// add texture to list of textures, if it was not black
			if (!bBlackChannel)
			{
				std::string strTextureFileName=strTexture;
				if (this->m_strPLSM2DetailTextureFormat.empty())
				{
					strTextureFileName+="."+strTextureExtension;
				}
				else
				{
					strTextureFileName+="."+this->m_strPLSM2DetailTextureFormat;
				}
				vecDetailTextures.push_back(ToLower(strTextureFileName));
			}

			// yes = next channel
			iTextureChannel++;
			
			// all channels filled and finish bitmap?
			if (iTextureChannel>=this->m_iMaxNumChannels || (nTexture==Terrain.GetNumTextures()-1))
			{
				// does the bitmap contain data
				if (!bBlackFile)
				{
					// blurred?
					if (this->m_iSmoothFactor>0)
					{
						// perform blurring
						bmp.BlurGaussian(this->m_iSmoothFactor,this->m_GaussSmoothMax);
					}

					// test best bitmap size
					if (this->m_bPowerOfTwo)
					{
						unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
						unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
						bmp.Resize(nSaveSizeX,nSaveSizeY,CBitmapFile::Mitchell);
					}

					// add detail map to the texture
					if (bIntegrateDetailTexture)
					{
						// get detail texture
						CIPnPTCTexture01* pTexture=Terrain.GetTexture(nTexture);
						if (pTexture)
						{
							// create detail map
							CBitmapFile bmpDetail;
							if (bmpDetail.Initialize(pTexture,bmfRGBA) && bmpDetail.Resize(bmp.GetSizeX(),bmp.GetSizeY(),CBitmapFile::Lanczos3))
							{
								// fill detail texture data
								for (unsigned int nY=0;nY<bmp.GetSizeY();++nY)
								{
									for (unsigned int nX=0;nX<bmp.GetSizeX();++nX)
									{
										// get detail color
										PNPTC_RGBA rgbaDetail;
										bmpDetail.GetPixel(nX,nY,rgbaDetail);

										// get alphamap value
										BYTE byValue;
										bmp.GetPixel(nX,nY,byValue);

										// store detail map combined with alpha map
										rgbaDetail.a=byValue;
										bmpDetail.SetPixel(nX,nY,rgbaDetail);
									}
								}

								// copy to working bitmap
								bmp.Initialize(bmfRGBA,bmp.GetSizeX(),bmp.GetSizeY());
								bmp.FillFromBitmap(bmpDetail);
							}
							else
							{
								// error
								MessageBox(NULL,"Error exporting texture map.\nCould not initialize and resize detail texture.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
								return false;
							}
						}
						else
						{
							// error
							MessageBox(NULL,"Error exporting texture map.\nCould not get detail texture.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
							return false;
						}
					}

					// save bitmap							
					std::string strTextureFileName = strFileName;
					if (this->m_strAlphamapFormat=="PLSM2")
					{
						strTextureFileName=this->PLSM2GetTextureFileName(NULL,strFileName,-1,-1,static_cast<int>(vecCoverageMaps.size()));
					}
					Replace(&strTextureFileName,"/","\\");
					this->m_nBorderSize=bmp.GetSizeX();
					if (!bmp.Write(ConvertToWindowsFormat(strTextureFileName),this->m_strFileFormat))
					{
						return false;
					}
					std::string strCoverageMap=this->m_strPLSM2RelativeMapDir+GetFileName(strTextureFileName);
					if (this->m_strAlphamapFormat=="PLSM2")
					{
						Replace(&strCoverageMap,"\\","/");
					}
					vecCoverageMaps.push_back(strCoverageMap);

					// add it to the exported patches
					CExportedPatch* pExportedPatch=this->GetExportedPatch(0,0);
					if (pExportedPatch)
					{
						pExportedPatch->AddExportFile(ConvertToWindowsFormat(strTextureFileName));
					}
				}

				// start with first channel in a new bitmap
				bBlackFile=true;
				iTextureChannel=0;
				if (!bmp.Initialize(bmf,nWidth,nHeight))
				{
					MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector textures",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

			} // bitmap finished
		} // bitmap contains data
	} // for all textures

	// -----------------------------------------------------------------------
	// write material

	if (this->m_bExportMaterialFiles)
	{
		if (this->m_strAlphamapFormat=="PLSM2")
		{
			this->PLSM2WriteMaterialFile(NULL,strFileName,vecCoverageMaps,vecDetailTextures,0,0);
		}
		else
		{
			this->XMLWriteMaterialFile(NULL,strFileName,vecCoverageMaps,vecDetailTextures,0,0,!bIntegrateDetailTexture);
		}
	}

	// ok
	return true;
}

void CTexturemapExporterAlphamap::SetPixelValue(CBitmapFile& bmp,int iX,int iY,int iChannelIndex,BYTE byValue) 
{
	// palettized bitmap?
	if (iChannelIndex==-1)
	{
		bmp.SetPixel(iX,iY,byValue);
	}
	else
	{
		// update color value of that pixel
		PNPTC_RGBA rgbaPixelValue;
		bmp.GetPixel(iX,iY,rgbaPixelValue);
		switch (iChannelIndex) 
		{
			case 0 : {
				rgbaPixelValue.r = byValue;
				break;
			}
			case 1 : {
				rgbaPixelValue.g = byValue;
				break;
			}
			case 2 : {
				rgbaPixelValue.b = byValue;
				break;
			}
			case 3 : {
				rgbaPixelValue.a = byValue;
				break;
			}
		}
		bmp.SetPixel(iX,iY,rgbaPixelValue);
	}
}

FILE* CTexturemapExporterAlphamap::PLSM2CreateMaterialFile(CIPnPTCSector01* pSector,const std::string& strPath,const std::string& strMapName,int iPatchX,int iPatchY)
{
	// create the material file
	std::string strFile=ConvertToWindowsFormat(this->PLSM2GetPatchMaterialFileName(pSector,strPath,strMapName,iPatchX,iPatchY));
	FILE* file=fopen(strFile.c_str(),"w");
	if (!file)
	{
		MessageBox(NULL,(std::string("Error opening sector material file '")+strFile+"'.").c_str(),"Error exporting terrain",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return NULL;
	}
	return file;
}

std::string CTexturemapExporterAlphamap::PLSM2GetPatchMaterialFileName(CIPnPTCSector01* pSector,const std::string& strPath,const std::string& strMapName,int iPatchX,int iPatchY)
{
	if (this->m_pFileNameCallback)
	{
		// use file name callback for the material file name
		return this->m_pFileNameCallback->GetExportFileName(pSector,strPath+"terrain.material",(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY);
	}
	else
	{
		// compute a default file name
		char cPatch[24];
		sprintf(cPatch,".%d.%d",iPatchY,iPatchX);
		std::string materialFileName= strPath + strMapName + cPatch+".material";
		return materialFileName;
	}
}

std::string CTexturemapExporterAlphamap::PLSM2GetTextureFileName(CIPnPTCSector01* pSector,const std::string& strBaseFileName,int iPatchX,int iPatchY,int iNumber)
{
	// get material file name
	if (this->m_pFileNameCallback)
	{
		// use file name callback for the material file name
		return this->m_pFileNameCallback->GetExportFileName(pSector,strBaseFileName,(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY,ToString(iNumber));
	}
	else
	{
		// compute a default file name
		char cPatch[24];
		sprintf(cPatch,".%d.%d.%d",iPatchY,iPatchX,iNumber);
		std::string strFilename,strPath,strExtension;
		RemoveFilePath(strBaseFileName,strFilename,strPath);
		std::string materialFileName = strPath + strFilename + cPatch + "." + strExtension;
		return materialFileName;
	}
}

bool CTexturemapExporterAlphamap::PLSM2WriteMaterialFile(CIPnPTCSector01* pSector,const std::string& strFileNameAlphaMap,const std::vector<std::string>& vecCoverageMaps,const std::vector<std::string>& vecDetailTextures,int iPatchX,int iPatchY)
{
	// create material file
	FILE* fileMaterial = this->PLSM2CreateMaterialFile(pSector,GetPath(strFileNameAlphaMap),ToLower(this->m_strPLSM2TerrainName),iPatchX,iPatchY);
	if (fileMaterial)
	{
		// start material
		if (this->m_pFileNameCallback)
		{
			std::string strMaterialName=this->m_pFileNameCallback->GetExportFileName(pSector,this->m_strPLSM2TerrainName,(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY);
			fprintf(fileMaterial,"material %s : terrain%dCov%d\n",strMaterialName.c_str(),this->m_iMaxNumChannels,vecCoverageMaps.size());
		}
		else
		{
			fprintf(fileMaterial,"material %s.%d.%d : terrain%dCov%d\n",this->m_strPLSM2TerrainName.c_str(),iPatchY,iPatchX,this->m_iMaxNumChannels,vecCoverageMaps.size());
		}
		fprintf(fileMaterial,"{\n");

		// go through all coverage passes
		for (int iPass=0;iPass<static_cast<int>(vecCoverageMaps.size());++iPass)
		{
			// write coverage map
			fprintf(fileMaterial,"\tset_texture_alias Cov%d %s\n",iPass,vecCoverageMaps[iPass].c_str());

			// write all textures
			for (int iTexture=0;iTexture<this->m_iMaxNumChannels;++iTexture)
			{
				int iTextureIndex=iTexture+iPass*this->m_iMaxNumChannels;
				if (iTextureIndex>=0 && iTextureIndex<static_cast<int>(vecDetailTextures.size()))
				{
					fprintf(fileMaterial,"\tset_texture_alias splattingCov%d_%d %s\n",iPass,iTexture,(this->m_strPLSM2DetailTextureBasePath+vecDetailTextures[iTextureIndex]).c_str());
				}
			}

			// finish pass
			fprintf(fileMaterial,"\n");
		}

		// write normal map for lighting
		if (!this->m_strPLSM2NormalMap.empty() && this->m_pFileNameCallback)
		{
			std::string strNormalName=this->m_pFileNameCallback->GetExportFileName(pSector,this->m_strPLSM2NormalMap,(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY);
			fprintf(fileMaterial,"\tset_texture_alias NormalMap %s\n",strNormalName.c_str());
		}

		// write texture map for distance LOD
		if (!this->m_strPLSM2TextureMap.empty() && this->m_pFileNameCallback)
		{
			std::string strTextureName=this->m_pFileNameCallback->GetExportFileName(pSector,this->m_strPLSM2TextureMap,(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY);
			fprintf(fileMaterial,"\tset_texture_alias ImageMap %s\n",strTextureName.c_str());
		}

		// finish materail
		fprintf(fileMaterial,"}\n");

		// close file
		fclose(fileMaterial);

		// ok
		return true;
	}
	else
	{
		// error
		return false;
	}
}

std::string CTexturemapExporterAlphamap::XMLGetPatchMaterialFileName(CIPnPTCSector01* pSector,const std::string& strPath,int iPatchX,int iPatchY)
{
	if (this->m_pFileNameCallback)
	{
		// use file name callback for the material file name
		return this->m_pFileNameCallback->GetExportFileName(pSector,strPath+"terrain.xml",(pSector)?(pSector->GetBorderSize()/this->m_iPatchSize):0,iPatchX,iPatchY);
	}
	else
	{
		// compute a default file name
		char cPatch[24];
		if (pSector)
		{
			sprintf(cPatch,"coveragemap_%05d_%05d_%03d",pSector->GetPosX(),pSector->GetPosY(),iPatchY*(pSector->GetBorderSize()/this->m_iPatchSize)+iPatchX);
		}
		else
		{
			sprintf(cPatch,"coveragemap");
		}
		std::string materialFileName= strPath + cPatch+".xml";
		return materialFileName;
	}
}

bool CTexturemapExporterAlphamap::XMLWriteMaterialFile(CIPnPTCSector01* pSector,const std::string& strFileNameAlphaMap,const std::vector<std::string>& vecCoverageMaps,const std::vector<std::string>& vecDetailTextures,int iPatchX,int iPatchY,bool bWriteDetailReferences/*=true*/)
{
	// create the material file
	std::string strFile=ConvertToWindowsFormat(this->XMLGetPatchMaterialFileName(pSector,GetPath(strFileNameAlphaMap),iPatchX,iPatchY));
	FILE* fileMaterial=fopen(strFile.c_str(),"w");
	if (fileMaterial)
	{
		// write texture data
		std::string channels[4]={"R","G","B","A"};
		int iTexture=0;
		if (pSector)
		{
			fprintf(fileMaterial,"<coveragemaps sector_x=\"%d\" sector_y=\"%d\" patch_x=\"%d\" patch_y=\"%d\">\n",pSector->GetPosX(),pSector->GetPosY(),iPatchX,iPatchY);
		}
		else
		{
			fprintf(fileMaterial,"<coveragemaps>\n");
		}
		for (unsigned int n=0;n<vecCoverageMaps.size();++n)
		{
			if (bWriteDetailReferences)
			{
				fprintf(fileMaterial,"\t<coveragemap alphamap_texture=\"%s\">\n",vecCoverageMaps[n].c_str());
				for (int iMap=0;iMap<this->m_iMaxNumChannels && (iMap+iTexture)<static_cast<int>(vecDetailTextures.size());++iMap)
				{
					if (this->m_iMaxNumChannels>1)
					{
						fprintf(fileMaterial,"\t\t<alphamap channel=\"%s\" detail_texture=\"%s\" />\n",channels[iMap].c_str(),vecDetailTextures[iMap+iTexture].c_str());
					}
					else
					{
						fprintf(fileMaterial,"\t\t<alphamap detail_texture=\"%s\" />\n",vecDetailTextures[iMap+iTexture].c_str());
					}
				}
				iTexture+=this->m_iMaxNumChannels;
				fprintf(fileMaterial,"\t</coveragemap>\n");
			}
			else
			{
				fprintf(fileMaterial,"\t<coveragemap alphamap_texture=\"%s\" />\n",vecCoverageMaps[n].c_str());
			}
		}
		fprintf(fileMaterial,"</coveragemaps>\n");

		// close the file and ok
		fclose(fileMaterial);
		return true;
	}
	else
	{
		// error creating material file
		return false;
	}
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// lightmap to bmp export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CLightmapExporterBMP::CLightmapExporterBMP()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_strFileFormat="BMP";
	this->m_strLightMapName="";
	this->m_bFlipY=false;
	this->m_bInvert=false;
	this->m_iBorderPixel=0;
	this->m_nBorderSize=0;
	this->m_bPowerOfTwo=false;
}

void CLightmapExporterBMP::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
	else if (EqualNoCase(strParameterName,"LightMapName"))
	{
		this->m_strLightMapName=strValue;
	}
}
void CLightmapExporterBMP::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
	else if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=minmax(iValue,0,2);
	}
}

void CLightmapExporterBMP::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"BorderPixel"))
	{
		this->m_iBorderPixel=(bValue)?2:0;
	}
	else if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
	else if (EqualNoCase(strParameterName,"Invert"))
	{
		this->m_bInvert=bValue;
	}
	else if (EqualNoCase(strParameterName,"PowerOfTwo"))
	{
		this->m_bPowerOfTwo=bValue;
	}
}

std::string CLightmapExporterBMP::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"lightmap_%s_%05d_%05d.%s",this->m_strLightMapName.c_str(),pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		static char cSector[100];
		sprintf(cSector,"lightmap_%s.%s",this->m_strLightMapName.c_str(),this->m_strFileFormat.c_str());
		return cSector;
	}
}

bool CLightmapExporterBMP::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// get light map of sector
	CIPnPTCLightMap01* pLightMap=Sector.GetLightMap(this->m_strLightMapName.c_str());
	CIPnPTCTexture01* pTexture=(pLightMap)?pLightMap->GetTexture(0,0):NULL;
	if (!pLightMap || !pTexture)
	{
		MessageBox(NULL,"Could not read sector light maps.\nLight map not available.","Error exporting light map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// get number of patches per border
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;

	// compute column and row size (The number of bytes per row must be a multiple of 4)
	// every single texture within the texture maps overlap with the surrounding texture
	// by one pixel; that's why we have the -2 to avoid double pixels when combining them
	// into one single texture. Around the outer border of the large destination texture 
	// we want to preserve this extra pixel; that's why we finally have the +2.
	unsigned int nHeight=(pTexture->GetHeight()-2)*pLightMap->GetNumTexturesPerSectorBorder()+2;
	unsigned int nWidth =(pTexture->GetWidth()-2)*pLightMap->GetNumTexturesPerSectorBorder()+2;

	// compute size of final textures
	unsigned int nTexBorderSize=(nHeight-2)/iPatches;
	int iPatchFactor=static_cast<int>(nTexBorderSize);
	int iBorderOffset=1;
	if (this->m_iBorderPixel==1)
	{
		iBorderOffset=1;
		nTexBorderSize+=1;
	}
	else if (this->m_iBorderPixel==2)
	{
		iBorderOffset=0;
		nTexBorderSize+=2;
	}

	// initialize progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector light map %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,pLightMap->GetNumTexturesPerSectorBorder()*pLightMap->GetNumTexturesPerSectorBorder()+iPatches*iPatches*nTexBorderSize);

	// create mem
	CBitmapFile bmpSector;
	if (!bmpSector.Initialize(bmfRGB,nWidth,nHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting sector light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all textures
	for (int iTexY=0;iTexY<pLightMap->GetNumTexturesPerSectorBorder();++iTexY)
	{
		for (int iTexX=0;iTexX<pLightMap->GetNumTexturesPerSectorBorder();++iTexX)
		{
			// get texture
			pTexture=pLightMap->GetTexture(iTexX,iTexY);
			if (!pTexture)
			{
				MessageBox(NULL,"Error reading texture data.","Error exporting sector light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// go through whole texture
			for (int iPixY=0;iPixY<pTexture->GetHeight();++iPixY)
			{
				for (int iPixX=0;iPixX<pTexture->GetWidth();++iPixX)
				{
					// get pixel color
					PNPTC_RGB rgb=pTexture->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);

					// write pixel
					bmpSector.SetPixel(iPixX+iTexX*(pTexture->GetWidth()-2),iPixY+iTexY*(pTexture->GetHeight()-2),rgb);
				}
			}

			// increment progress bar
			ProgressBar.IncrementPos();
		}
	}

	// fill and write bitmaps
	for (int iPatchY=0,iPatch=0;iPatchY<iPatches;iPatchY++)
	{
		for (int iPatchX=0;iPatchX<iPatches;iPatchX++,iPatch++)
		{
			// create bitmap
			CBitmapFile bmp;
			if (!bmp.Initialize(bmfRGB,nTexBorderSize,nTexBorderSize))
			{
				MessageBox(NULL,"Error creating bitmap file.","Error exporting sector light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get file name
			std::string strFileName=Filename;
			if (this->m_pFileNameCallback)
			{
				// get filename from callback object
				strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
			}
			else
			{	
				if (this->m_iPatchSize!=Sector.GetBorderSize())
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					char cPatch[10];
					sprintf(cPatch,"_%.3d",iPatch);
					strFileName=strFolder+strFileName+cPatch+"."+strExtension;
				}
			}

			// set all pixel
			for (unsigned int nY=0;nY<nTexBorderSize;nY++)
			{
				for (unsigned int nX=0;nX<nTexBorderSize;nX++)
				{
					// get color value
					PNPTC_RGB rgb;
					bmpSector.GetPixel(iPatchX*iPatchFactor+nX+iBorderOffset,iPatchY*iPatchFactor+nY+iBorderOffset,rgb);

					// invert that pixel
					if (this->m_bInvert)
					{
						rgb.r=255-rgb.r;
						rgb.g=255-rgb.g;
						rgb.b=255-rgb.b;
					}

					// write pixel
					bmp.SetPixel(nX,nY,rgb);
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}

			// flip bitmap?
			if (this->m_bFlipY)
			{
				bmp.FlipY();
			}

			// test best bitmap size
			if (this->m_bPowerOfTwo)
			{
				unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
				unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
				bmp.Resize(nSaveSizeX,nSaveSizeY);
			}
			this->m_nBorderSize=bmp.GetSizeX();

			// write texture to file
			if (!bmp.Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
			{
				MessageBox(NULL,"Error writing bitmap file.","Error exporting sector light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}			

		} // for iPatchX
	} // for iPatchY

	// ok
	return true;
}

bool CLightmapExporterBMP::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// get first sector
	CIPnPTCSector01* pFirstSector=Terrain.GetSectorDetail(0,0);
	if (!pFirstSector)
	{
		MessageBox(NULL,"Error reading detail data of first sector.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// get light map of sector
	CIPnPTCLightMap01* pLightMap=pFirstSector->GetLightMap(this->m_strLightMapName.c_str());
	CIPnPTCTexture01* pTexture=(pLightMap)?pLightMap->GetTexture(0,0):NULL;
	if (!pLightMap || !pTexture)
	{
		MessageBox(NULL,"Could not read sector light maps.\nLight map not available.","Error exporting light map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// get height and with of every single texture
	DWORD dwSingleTextureWidth=pTexture->GetWidth();
	DWORD dwSingleTextureHeight=pTexture->GetHeight();

	// compute column and row size (The number of bytes per row must be a multiple of 4)
	// every single texture within the texture maps overlap with the surrounding texture
	// by one pixel; that's why we have the -2 to avoid double pixels when combining them
	// into one single texture. 
	DWORD dwHeight=(dwSingleTextureHeight-2)*pLightMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsY();
	DWORD dwWidth =(dwSingleTextureWidth-2)*pLightMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsX();

	// initialize progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain light map",pLightMap->GetNumTexturesPerSectorBorder()*pLightMap->GetNumTexturesPerSectorBorder()*Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY());

	// create mem
	CBitmapFile bmp;
	if (!bmp.Initialize(bmfRGB,dwWidth,dwHeight))
	{
		MessageBox(NULL,"Error allocationg mem for bitmap data.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}

	// go through all sectors
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector detail data.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get texture object for this sector
			CIPnPTCLightMap01* pLightMap=pSector->GetLightMap(this->m_strLightMapName.c_str());
			CIPnPTCTexture01* pTexture=(pLightMap)?pLightMap->GetTexture(0,0):NULL;
			if (!pLightMap || !pTexture)
			{
				MessageBox(NULL,"Could not read sector light maps.\nLight map not available.","Error exporting light map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
				return false;
			}

			// go through all textures
			DWORD dwOffset=0;
			DWORD dwLastWidth=0;
			DWORD dwLastHeight=0;
			for (int iTexY=0;iTexY<pLightMap->GetNumTexturesPerSectorBorder();iTexY++)
			{
				// get pixel range within texture
				int iPixStartY=(iTexY==0)?0:1;
				int iPixEndY=(iTexY==pLightMap->GetNumTexturesPerSectorBorder()-1)?0:-1;

				for (int iTexX=0;iTexX<pLightMap->GetNumTexturesPerSectorBorder();iTexX++)
				{
					// get pixel range within texture
					int iPixStartX=(iTexX==0)?0:1;
					int iPixEndX=(iTexX==pLightMap->GetNumTexturesPerSectorBorder()-1)?0:-1;

					// get texture
					pTexture=pLightMap->GetTexture(iTexX,iTexY);
					if (!pTexture)
					{
						MessageBox(NULL,"Error reading texture data.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// check texture size
					if (dwSingleTextureWidth!=pTexture->GetWidth() ||
						dwSingleTextureHeight!=pTexture->GetHeight() )
					{
						MessageBox(NULL,"The size of one texture dosen't match.\nThis might be an internal error, please contact the support.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
						return false;
					}

					// go through whole texture
					for (int iPixY=1;iPixY<(pTexture->GetHeight()-1);iPixY++)
					{
						for (int iPixX=1;iPixX<(pTexture->GetWidth()-1);iPixX++)
						{
							// get pixel color
							PNPTC_RGB rgb=pTexture->GetPixel(iPixX,pTexture->GetHeight()-iPixY-1);

							// invert that pixel
							if (this->m_bInvert)
							{
								rgb.r=255-rgb.r;
								rgb.g=255-rgb.g;
								rgb.b=255-rgb.b;
							}

							// write pixel
							bmp.SetPixel(nSecX*pLightMap->GetNumTexturesPerSectorBorder()*(dwSingleTextureWidth-2)+iTexX*(dwSingleTextureWidth-2)+iPixX-1,
								         nSecY*pLightMap->GetNumTexturesPerSectorBorder()*(dwSingleTextureHeight-2)+iTexY*(dwSingleTextureHeight-2)+iPixY-1,
										 rgb);
						}
					}

					// increment progress bar
					ProgressBar.IncrementPos();
				}
			}
		}
	}

	// flip bitmap?
	if (this->m_bFlipY)
	{
		bmp.FlipY();
	}

	// test best bitmap size
	if (this->m_bPowerOfTwo)
	{
		unsigned int nSaveSizeX=GetPowerOfTwoBorderSize(bmp.GetSizeX());
		unsigned int nSaveSizeY=GetPowerOfTwoBorderSize(bmp.GetSizeY());
		bmp.Resize(nSaveSizeX,nSaveSizeY);
	}
	this->m_nBorderSize=bmp.GetSizeX();

	// write texture to file
	if (!bmp.Write(ConvertToWindowsFormat(Filename),this->m_strFileFormat))
	{
		MessageBox(NULL,"Error writing bitmap file.","Error exporting terrain light map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
		return false;
	}	

	// ok
	return true;
}






// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// grass coverage to bitmap export
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

CGrassExporterBMP::CGrassExporterBMP()
{
	// initializations
	this->m_iPatchSize=1024;
	this->m_strFileFormat="BMP";
	this->m_bFlipY=false;
	this->m_bIndexMap=true;
}

void CGrassExporterBMP::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
	if (EqualNoCase(strParameterName,"FileFormat"))
	{
		this->m_strFileFormat=strValue;
	}
}
void CGrassExporterBMP::SetParameter(const std::string& strParameterName,int iValue)
{
	if (EqualNoCase(strParameterName,"PatchSize"))
	{
		this->m_iPatchSize=iValue;
	}
}
void CGrassExporterBMP::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"FlipY"))
	{
		this->m_bFlipY=bValue;
	}
	else if (EqualNoCase(strParameterName,"IndexMap"))
	{
		this->m_bIndexMap=bValue;
	}
}

std::string CGrassExporterBMP::GetDefaultFileName(CIPnPTCSector01* pSector)
{
	if (pSector)
	{
		// create sector file name
		static char cSector[100];
		sprintf(cSector,"grassmap_%05d_%05d.%s",pSector->GetPosX(),pSector->GetPosY(),this->m_strFileFormat.c_str());
		return cSector;
	}
	else
	{
		return std::string("grassmap.")+this->m_strFileFormat;
	}
}

bool CGrassExporterBMP::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// initialize result
	bool bResult=true;

	// compute number of required patches
	this->m_iPatchSize=min(this->m_iPatchSize,static_cast<int>(Sector.GetBorderSize()));
	int iPatches=Sector.GetBorderSize()/this->m_iPatchSize;
	iPatches*=iPatches;

	// get vegetation object
	CIPnPTCSectorVegetation01* pVegetation=Sector.GetSectorVegetation();
	if (!pVegetation)
	{
		MessageBox(NULL,"Could not read sector vegetation.\nNo vegetation available.","Error exporting grass coverage",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// test if grasses are available
	if (Terrain.GetNumGrasses()==0)
	{
		MessageBox(NULL,"Could not export sector vegetation.\nNo grass definitions available.","Error exporting grass coverage",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// create a map to convert a grass name to an ID and vice versa
	std::map<std::string,unsigned int> mapGrassNames;
	for (unsigned int nGrass=0;nGrass<Terrain.GetNumGrasses();nGrass++)
	{
		CIPnPTCGrass01* pGrass=Terrain.GetGrass(nGrass);
		if (pGrass)
		{
			mapGrassNames[pGrass->GetGrassName()]=nGrass;
		}
	}

	// get maximum number of bitmaps to be created
	unsigned int nGrassTypes=(this->m_bIndexMap)?1:Terrain.GetNumGrasses();

	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Exporting sector grass map %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(App,cSectorName,iPatches*this->m_iPatchSize);

	// create bitmap array
	CBitmapFile* pBitmaps=new CBitmapFile[nGrassTypes];
	if (!pBitmaps)
	{
		MessageBox(NULL,"Error initializing bitmap files.\nPossibly not enough memory available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// initialize index map
	if (this->m_bIndexMap)
	{
		if (!pBitmaps[0].Initialize(bmfRGBA,this->m_iPatchSize,this->m_iPatchSize))
		{
			MessageBox(NULL,"Error initializing bitmap file.\nPossibly not enough memory available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
			bResult=false;
			goto end;
		}
	}

	// write bitmap(s)
	bool bIndexError=false;
	for (int iPatchStartY=0,iPatch=0,iPatchY=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize,iPatchY++)
	{
		for (int iPatchStartX=0,iPatchX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++,iPatchX++)
		{
			// get grasses
			for (int iY=0;iY<this->m_iPatchSize;iY++)
			{
				for (int iX=0;iX<this->m_iPatchSize;iX++)
				{
					// initialize color value for indexed grass map export
					PNPTC_RGBA rgba(0,0,0,0);

					// go through all grass defs for this position
					for (unsigned int nIndex=0;nIndex<4;nIndex++)
					{
						// read grass placement information
						GRASS_PLACEMENT gp=pVegetation->GetGrassPlacement(iX+iPatchStartX,iY+iPatchStartY,nIndex);

						// is this grass placement defined=
						if (gp.fDensity>0 && gp.cGrassType)
						{
							// get and check grass index for this grass type name
							unsigned int nGrass=mapGrassNames[gp.cGrassType];
							if (nGrass>=255)
							{
								// invalid index
								bIndexError=true;
							}

							// now set color/pixel
							if (this->m_bIndexMap)
							{
								// set the index in the appropriate color chanel
								switch (nIndex)
								{
								case 0:
									rgba.r=static_cast<unsigned char>(nGrass+1);
									break;
								case 1:
									rgba.g=static_cast<unsigned char>(nGrass+1);
									break;
								case 2:
									rgba.b=static_cast<unsigned char>(nGrass+1);
									break;
								case 3:
									rgba.a=static_cast<unsigned char>(nGrass+1);
									break;
								}
							}
							else // if (this->m_bIndexMap)
							{
								// set gray value in bitmap for this grass type
								// compute gray value
								unsigned char cGrayValue=static_cast<unsigned char>(gp.fDensity*255.f);

								// check index
								if (nGrass<nGrassTypes)
								{
									// make sure that the bitmap is initialized
									if (!pBitmaps[nGrass].IsInitialized())
									{
										if (!pBitmaps[nGrass].Initialize(bmfPalette,this->m_iPatchSize,this->m_iPatchSize))
										{
											MessageBox(NULL,"Error initializing bitmap file for grass type.\nPossibly not enough memory available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
											bResult=false;
											goto end;
										}
									}

									// set pixel
									if (pBitmaps[nGrass].IsInitialized())
									{
										pBitmaps[nGrass].SetPixel(iX,iY,cGrayValue);
									}
								}
								else // if (nGrass<nGrassTypes)
								{
									MessageBox(NULL,"Error exporting grass bitmap file.\nThe grass index is out of range.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
									bResult=false;
									goto end;
								} // else if (nGrass<nGrassTypes)

							} // else (if (this->m_bIndexMap))
						} // if (gp.fDensity>0 && gp.cGrassType)
					} // for (unsigned int nIndex=0;nIndex<4;nIndex++)

					// set pixel in index map
					if (this->m_bIndexMap)
					{
						pBitmaps[0].SetPixel(iX,iY,rgba);
					}

				} // for (int iX=0;iX<this->m_iPatchSize;iX++)

				// increment progress bar
				ProgressBar.IncrementPos();
			
			} // for (int iY=0;iY<this->m_iPatchSize;iY++)

			// flip bitmap(s)?
			if (this->m_bFlipY)
			{
				for (unsigned int nBitmap=0;nBitmap<nGrassTypes;nBitmap++)
				{
					if (pBitmaps[nBitmap].IsInitialized())
					{
						pBitmaps[nBitmap].FlipY();
					}
				}
			}

			// write bitmap(s) to file
			if (this->m_bIndexMap)
			{
				// create file name
				std::string strFileName=Filename;
				if (this->m_pFileNameCallback)
				{
					// get filename from callback object
					strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
				}
				else
				{	
					// add patch index
					if (this->m_iPatchSize!=Sector.GetBorderSize())
					{
						std::string strExtension;
						std::string strFolder(GetPath(strFileName));
						RemoveFileExtension(strFileName,strFileName,strExtension);
						char cPatch[10];
						sprintf(cPatch,"_%.3d",iPatch);
						strFileName=strFolder+strFileName+cPatch+"."+strExtension;
					}
				}

				// save file
				if (!pBitmaps[0].Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
				{
					MessageBox(NULL,"Error writing grass bitmap file.\nMaybe the destination for writing the file is write locked or full.\nTry exporting using the .bmp format.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
					bResult=false;
					goto end;
				}
			}
			else // if (this->m_bIndexMap)
			{
				// write bitmap files for all grass types
				for (unsigned int nBitmap=0;nBitmap<nGrassTypes;nBitmap++)
				{
					if (pBitmaps[nBitmap].IsInitialized())
					{
						// create file name
						std::string strFileName=Filename;
						// add grass type
						CIPnPTCGrass01* pGrass=Terrain.GetGrass(nBitmap);
						if (pGrass)
						{
							std::string strExtension;
							std::string strFolder(GetPath(strFileName));
							RemoveFileExtension(strFileName,strFileName,strExtension);
							std::string strGrassName=pGrass->GetGrassName();
							std::string strTemp;
							RemoveFileExtension(strGrassName,strGrassName,strTemp);
							strFileName=strFolder+strFileName+"_"+strGrassName+"."+strExtension;
						}
						if (this->m_pFileNameCallback)
						{
							// get filename from callback object
							strFileName=this->m_pFileNameCallback->GetExportFileName(&Sector,Filename,Sector.GetBorderSize()/this->m_iPatchSize,iPatchX,iPatchY);
						}
						else
						{	
							// add patch index
							if (this->m_iPatchSize!=Sector.GetBorderSize())
							{
								std::string strExtension;
								std::string strFolder(GetPath(strFileName));
								RemoveFileExtension(strFileName,strFileName,strExtension);
								char cPatch[10];
								sprintf(cPatch,"_%.3d",iPatch);
								strFileName=strFolder+strFileName+cPatch+"."+strExtension;
							}
						}

						// save file
						if (!pBitmaps[nBitmap].Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
						{
							MessageBox(NULL,"Error writing grass bitmap file.\nMaybe the destination for writing the file is write locked or full.\nTry exporting using the .bmp format.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
							bResult=false;
							goto end;
						}
					}
				}
			} // else if (this->m_bIndexMap)

			// deinitialize all bitmaps if we are not using the index map
			if (!this->m_bIndexMap)
			{
				for (unsigned int nBitmap=0;nBitmap<nGrassTypes;nBitmap++)
				{
					pBitmaps[nBitmap].DeInitialize();
				}
			}

		} // for (int iPatchStartX=0;iPatchStartX<static_cast<int>(Sector.GetBorderSize());iPatchStartX+=this->m_iPatchSize,iPatch++)
	} // for (int iPatchStartY=0,iPatch=0;iPatchStartY<static_cast<int>(Sector.GetBorderSize());iPatchStartY+=this->m_iPatchSize)

	// index error message
	if (bIndexError)
	{
		MessageBox(NULL,"There have been more than 256 grass indices within your terrain.\nThe grass map only supports 256 different grass indices.\nSo your output file might differ strongly from the rendering within TC.","Error exporting grass coverage",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
	}

end:

	// clean-up
	SAFE_DELETEARRAY(pBitmaps);

	// ok
	return bResult;
}

bool CGrassExporterBMP::WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)
{
	// initialize result
	bool bResult=true;

	// test if grasses are available
	if (Terrain.GetNumGrasses()==0)
	{
		MessageBox(NULL,"Could not export sector vegetation.\nNo grass definitions available.","Error exporting grass coverage",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// create a map to convert a grass name to an ID and vice versa
	std::map<std::string,unsigned int> mapGrassNames;
	for (unsigned int nGrass=0;nGrass<Terrain.GetNumGrasses();nGrass++)
	{
		CIPnPTCGrass01* pGrass=Terrain.GetGrass(nGrass);
		if (pGrass)
		{
			mapGrassNames[pGrass->GetGrassName()]=nGrass;
		}
	}

	// get maximum number of bitmaps to be created
	unsigned int nGrassTypes=(this->m_bIndexMap)?1:Terrain.GetNumGrasses();

	// set progress bar
	pnptc::CProgressBar ProgressBar(App,"Exporting terrain grass map",Terrain.GetNumSectorsY()*Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize());

	// create bitmap array
	CBitmapFile* pBitmaps=new CBitmapFile[nGrassTypes];
	if (!pBitmaps)
	{
		MessageBox(NULL,"Error initializing bitmap files.\nPossibly not enough memory available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}

	// initialize index map
	if (this->m_bIndexMap)
	{
		if (!pBitmaps[0].Initialize(bmfRGBA,Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize(),Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()))
		{
			MessageBox(NULL,"Error initializing bitmap file.\nPossibly not enough memory available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
			bResult=false;
			goto end;
		}
	}

	// go through all sectors
	bool bIndexError=false;
	for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
	{
		for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
		{
			// get sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
			if (!pSector)
			{
				MessageBox(NULL,"Error reading sector data.","Error exporting terrain grass map",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
				return false;
			}

			// get sector vegetation
			CIPnPTCSectorVegetation01* pVegetation=pSector->GetSectorVegetation();
			if (!pVegetation)
			{
				MessageBox(NULL,"Could not read sector vegetation.\nNo vegetation available.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
				return false;
			}

			// get vegetation
			for (unsigned int iY=0;iY<Terrain.GetSectorDetailBorderSize();iY++)
			{
				for (unsigned int iX=0;iX<Terrain.GetSectorDetailBorderSize();iX++)
				{
					// initialize color value for indexed grass map export
					PNPTC_RGBA rgba(0,0,0,0);

					// go through all grass defs for this position
					for (unsigned int nIndex=0;nIndex<4;nIndex++)
					{
						// read grass placement information
						GRASS_PLACEMENT gp=pVegetation->GetGrassPlacement(iX,iY,nIndex);

						// is this grass placement defined=
						if (gp.fDensity>0 && gp.cGrassType)
						{
							// get and check grass index for this grass type name
							unsigned int nGrass=mapGrassNames[gp.cGrassType];
							if (nGrass>=255)
							{
								// invalid index
								bIndexError=true;
							}

							// now set color/pixel
							if (this->m_bIndexMap)
							{
								// set the index in the appropriate color chanel
								switch (nIndex)
								{
								case 0:
									rgba.r=static_cast<unsigned char>(nGrass+1);
									break;
								case 1:
									rgba.g=static_cast<unsigned char>(nGrass+1);
									break;
								case 2:
									rgba.b=static_cast<unsigned char>(nGrass+1);
									break;
								case 3:
									rgba.a=static_cast<unsigned char>(nGrass+1);
									break;
								}
							}
							else // if (this->m_bIndexMap)
							{
								// set gray value in bitmap for this grass type
								// compute gray value
								unsigned char cGrayValue=static_cast<unsigned char>(gp.fDensity*255.f);

								// check index
								if (nGrass<nGrassTypes)
								{
									// make sure that the bitmap is initialized
									if (!pBitmaps[nGrass].IsInitialized())
									{
										if (!pBitmaps[nGrass].Initialize(bmfPalette,Terrain.GetNumSectorsX()*Terrain.GetSectorDetailBorderSize(),Terrain.GetNumSectorsY()*Terrain.GetSectorDetailBorderSize()))
										{
											MessageBox(NULL,"Error initializing bitmap file for grass type.\nPossibly not enough memory available.","Error exporting terrain grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
											bResult=false;
											goto end;
										}
									}

									// set pixel
									if (pBitmaps[nGrass].IsInitialized())
									{
										pBitmaps[nGrass].SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
																  nSecY*Terrain.GetSectorDetailBorderSize()+iY,
																  cGrayValue);
									}
								}
								else // if (nGrass<nGrassTypes)
								{
									MessageBox(NULL,"Error exporting grass bitmap file.\nThe grass index is out of range.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
									bResult=false;
									goto end;
								} // else if (nGrass<nGrassTypes)

							} // else (if (this->m_bIndexMap))
						} // if (gp.fDensity>0 && gp.cGrassType)
					} // for (unsigned int nIndex=0;nIndex<4;nIndex++)

					// set pixel in index map
					if (this->m_bIndexMap)
					{
						pBitmaps[0].SetPixel(nSecX*Terrain.GetSectorDetailBorderSize()+iX,
											 nSecY*Terrain.GetSectorDetailBorderSize()+iY,
											 rgba);
					}

				} // for (unsigned int iX=0;iX<Terrain.GetSectorDetailBorderSize();iX++)

				// increment progress bar
				ProgressBar.IncrementPos();

			} // for (unsigned int iY=0;iY<Terrain.GetSectorDetailBorderSize();iY++)

		} // for (unsigned long nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++)
	} // for (unsigned long nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)

	// write bitmap(s) to file
	if (this->m_bIndexMap)
	{
		// create file name
		std::string strFileName=Filename;

		// save file
		if (!pBitmaps[0].Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
		{
			MessageBox(NULL,"Error writing grass bitmap file.\nMaybe the destination for writing the file is write locked or full.\nTry exporting using the .bmp format.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
			bResult=false;
			goto end;
		}
	}
	else // if (this->m_bIndexMap)
	{
		// write bitmap files for all grass types
		for (unsigned int nBitmap=0;nBitmap<nGrassTypes;nBitmap++)
		{
			if (pBitmaps[nBitmap].IsInitialized())
			{
				// create file name
				std::string strFileName=Filename;
				// add grass type
				CIPnPTCGrass01* pGrass=Terrain.GetGrass(nBitmap);
				if (pGrass)
				{
					std::string strExtension;
					std::string strFolder(GetPath(strFileName));
					RemoveFileExtension(strFileName,strFileName,strExtension);
					std::string strGrassName=pGrass->GetGrassName();
					std::string strTemp;
					RemoveFileExtension(strGrassName,strGrassName,strTemp);
					strFileName=strFolder+strFileName+"_"+strGrassName+"."+strExtension;
				}

				// save file
				if (!pBitmaps[nBitmap].Write(ConvertToWindowsFormat(strFileName),this->m_strFileFormat))
				{
					MessageBox(NULL,"Error writing grass bitmap file.\nMaybe the destination for writing the file is write locked or full.\nTry exporting using the .bmp format.","Error exporting grass map",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
					bResult=false;
					goto end;
				}
			}
		}
	} // else if (this->m_bIndexMap)

	// index error message
	if (bIndexError)
	{
		MessageBox(NULL,"There have been more than 256 grass indices within your terrain.\nThe grass map only supports 256 different grass indices.\nSo your output file might differ strongly from the rendering within TC.","Error exporting grass coverage",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
	}

end:

	// clean-up
	SAFE_DELETEARRAY(pBitmaps);

	// ok
	return bResult;
}



