/////////////////////////////////////////////////////////////////////////////////////
// Exporter.h
// Definition of all exporter classes which can be used from within
// the template scene exporter.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  01. Dec. 2005 :	Added support for lightmap overlay to texture map to bitmap export.
//  31. Jan. 2006 : Added alphamap exporter.
//  01. Feb. 2006 : Added support to not write black files in alphmap exporter.
//  07. Feb. 2006 : Added y flipping of exported bitmaps.
//  12. Mai. 2006 : Added full scale feature to RAW export.
//  16. Mai. 2006 : Added size+1 feature for bitmap exporter.
//  19. Mai. 2006 : Added flipY option to raw export.
//  24. Aug. 2006 : Added possibility to invert lightmap texture.
//  10. Oct. 2006 : Achieved VS2005 compatibility, made compilation easier.
//  29. Dec. 2006 : Added grass map exporter.
//  15. Mar. 2007 : Added export file name callback interface.
//  16. Mar. 2007 : Added possibility to scale texturemap bitmap to a power of 2 border size.
//  29. Mai  2007 : Added constant offset value to raw exporter.
//  11. Mai  2008 : Added possibility to scale lightmap bitmap to a power of 2 border size.
//  08. Sep. 2008 : Added possibility to export border pixels only on two sides of the exported texture maps.
//  13. Sep. 2008 : Heavy update to alphamap exporter to work with OGRE PLSM2 coverage maps.
//					Added possibility to export multichannel (RGBA) alphamaps.
//					Added border pixel functionality to normal map and alpha map exporter.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_EXPORTER_H_INCLUDED)
#define PNPTC_EXPORTER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>	// The Windows PlatformSDK has to be installed in order to compile PlugIns
#include <string>
#include <vector>
#include <map>

// pnptc includes
#include "../../Includes/IPnPTCExportPlugIn.h"


// forward declarations
class CBitmapFile;



//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

/// This interface can be used to override the default
/// export file names used by the exporter plugins.
class IExportFileNameCallback
{
public:

	/// Returns a file name for a given patch.
	/// @param pSector The sector, which should be exported.
	/// Can be <i>NULL</i> if no sector is used (whole terrain export).
	/// @param strBaseName The base name of the export file name.
	/// @param nPatches The number of patches per sector border.
	/// @param nPatchX The index (starting with 0) of the exported
	/// patch of this sector in x direction.
	/// @param nPatchY The index (starting with 0) of the exported
	/// patch of this sector in y direction.
	/// @param strExtraInfo Extra info string to be contained in the file name.
	virtual std::string		GetExportFileName(CIPnPTCSector01* pSector,const std::string& strBaseName,unsigned int nPatches,unsigned int nPatchX,unsigned int nPatchY,const std::string& strExtraInfo="") const=0;

};	// class IExportFileNameCallback


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// exporter base class.
class CExporter
{
public:

	// -----------------------------------------------------
	// public data types
	// -----------------------------------------------------

	/// A class to store information of one exported patch.
	class CExportedPatch
	{
	public:

		// -----------------------------------------------------
		// construction/destruction
		// -----------------------------------------------------

		/// Construction of a new exported patch.
		CExportedPatch() {};

		/// Destruction of the exported patch.
		virtual ~CExportedPatch() {};


		// -----------------------------------------------------
		// access to the files exported for this patch
		// -----------------------------------------------------

		/// Adds one file to the exported file list of this patch.
		/// @param strFile The name of file which was exported and
		/// should be added to this patch.
		void			AddExportFile(const std::string& strFile)
		{
			this->m_vecExportedFiles.push_back(strFile);
		};

		/// Returns the number of files exported for this patch.
		/// @return The number of files exported for this patch.
		unsigned int	GetNumExportedFiles() const
		{
			return static_cast<unsigned int>(this->m_vecExportedFiles.size());
		};

		/// Returns the name of one of the exported files
		std::string		GetExportedFile(unsigned int nFile) const
		{
			if (nFile<this->GetNumExportedFiles())
			{
				return this->m_vecExportedFiles[nFile];
			}
			else
			{
				return "";
			}
		};


	private:

		// -----------------------------------------------------
		// member variables
		// -----------------------------------------------------

		/// The list of exported files.
		std::vector<std::string>	m_vecExportedFiles;

	}; // class CExportedPatch


	// -----------------------------------------------------
	// construction/destruction
	// -----------------------------------------------------
	
	/// Construction of a new exporter.
	CExporter();

	/// Destruction of the exporter
	virtual ~CExporter();


	// -----------------------------------------------------
	// export data
	// -----------------------------------------------------

	/// sets an export string parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param strValue parameter value to be set.
	virtual void				SetParameter(const std::string& strParameterName,const std::string& strValue);

	/// sets an export string parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param strValue parameter value to be set.
	virtual void				SetParameter(const std::string& strParameterName,const char* cValue) {this->SetParameter(strParameterName,std::string(cValue));};

	/// sets an export integer parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param iValue parameter value to be set.
	virtual void				SetParameter(const std::string& strParameterName,int iValue);

	/// sets an export float parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param fValue parameter value to be set.
	virtual void				SetParameter(const std::string& strParameterName,float fValue);

	/// sets an export bool parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param bValue parameter value to be set.
	virtual void				SetParameter(const std::string& strParameterName,bool bValue);

	/// exports one sector to a file.
	/// @param App reference to the application object.
	/// @param Sector reference to the sector to be exported.
	/// @param Terrain reference to the terrain to be exported.
	/// @param Filename destination file name for the export.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the export was successfull.
	virtual bool				WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename)=0;

	/// exports the complete terrain to a file.
	/// @param App reference to the application object.
	/// @param Terrain reference to the terrain to be exported.
	/// @param Filename destination file name for the export.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the export was successfull.
	virtual bool				WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename)=0;

	/// returns the default file name for given settings and a given sector.
	/// @param pSector sector for which the default file name should be returned.
	/// Pass <i>NULL</i> if you want the default filename for exporting the
	/// whole terrain into one file.
	/// @return Returns the default file name for the sector
	virtual std::string			GetDefaultFileName(CIPnPTCSector01* pSector)=0;

	/// Sets the object, which will be used as export file name callback.
	/// @param pFileNameCallback Pointer to the file name callback object.
	void						SetFileNameCallback(IExportFileNameCallback* pFileNameCallback);


	// -----------------------------------------------------
	// access to the patches exported by this exporter
	// -----------------------------------------------------

	/// Returns one exported patch.
	/// @param nPatchX The x index of the exported patch to be returned.
	/// @param nPatchY The y index of the exported patch to be returned.
	/// @return Returns a pointer to the exported patch if successful,
	/// or <i>NULL</i> in case of an error.
	CExportedPatch*				GetExportedPatch(unsigned int nPatchX,unsigned int nPatchY);


protected:

	// -----------------------------------------------------
	// export data
	// -----------------------------------------------------

	/// A call back object for retreiving the export file name.
	IExportFileNameCallback*	m_pFileNameCallback;


private:

	// -----------------------------------------------------
	// internal data types
	// -----------------------------------------------------

	/// patch index data type.
	typedef std::pair<int,int>				PatchIndex;


	// -----------------------------------------------------
	// member variables
	// -----------------------------------------------------

	/// The list/map of all exported patches.
	std::map<PatchIndex,CExportedPatch>		m_mapPatches;

}; // class CExporter


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// height map to bmp exporter class
class CHeightmapExporterBMP : public CExporter
{
public:
	/// construction
	CHeightmapExporterBMP();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,bool bValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to add one element to the size of the sector
	bool		m_bSizePlus1;

	/// color mode used for export
	int			m_iColorMode;
	
	/// whether or not to use the height range of a sector for the export bitmap
	bool		m_bSectorHeightRange;
	
	/// patch size used for export
	int			m_iPatchSize;
	
	/// file format used for export
	std::string	m_strFileFormat;

	/// Whether or not to flip exported bitmap vertically
	bool		m_bFlipY;

}; // class CHeightmapExporterBMP


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// normal map to bmp exporter class
class CNormalmapExporterBMP : public CExporter
{
public:
	/// construction
	CNormalmapExporterBMP();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,bool bValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to preserve the border pixel for seamless sectors (0: no border pixels; 1: border pixels on right and top border; 2: border pixels on all borders)
	int			m_iBorderPixel;

	/// patch size used for export
	int			m_iPatchSize;
	
	/// file format used for export
	std::string	m_strFileFormat;

	/// XYZ -> RGB conversion
	std::string	m_strRGB;

	/// indizes for XYZ -> RGB conversion
	int			m_iRGB[3];

	/// Whether or not to flip exported bitmap vertically
	bool		m_bFlipY;

	/// Whether or not the texture should be scaled to a power of two during export.
	bool		m_bPowerOfTwo;

}; // class CNormalmapExporterBMP


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// height map to raw exporter class
class CHeightmapExporterRAW : public CExporter
{
public:
	/// construction
	CHeightmapExporterRAW();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,bool bValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,float fValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to add one element to the size of the sector
	bool		m_bSizePlus1;
	
	/// whether or not to offset all heights to zero with respect to the terrain
	bool		m_bOffsetToZero;

	/// patch size used for export
	int			m_iPatchSize;

	/// factor to be multiplied with the heights
	float		m_fFactor;

	/// file format used for export
	int			m_iFormat;

	/// Whether or not to scale the terrain heights to the full value range of the export format
	bool		m_bFullScale;

	/// Whether or not to flip exported bitmap vertically
	bool		m_bFlipY;

	/// Constant offset value for the terrain heights
	float		m_fOffset;

}; // class CHeightmapExporterRAW


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// texture map to bitmap exporter class
class CTexturemapExporterBMP : public CExporter
{
public:
	/// construction
	CTexturemapExporterBMP();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,bool bValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

	// --------------------------------------------------------------------------------
	// get some information from the export
	// --------------------------------------------------------------------------------

	/// Returns the border size of the generated bitmaps.
	/// This method will only return correct values after the export took place.
	/// @return The border size of the exported bitmaps.
	unsigned int	GetBitmapBorderSize() const {return this->m_nBorderSize;};


private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to preserve the border pixel for seamless sectors (0: no border pixels; 1: border pixels on right and top border; 2: border pixels on all borders)
	int				m_iBorderPixel;
	
	/// patch size used for export
	int				m_iPatchSize;

	/// file format used for export
	std::string		m_strFileFormat;

	/// name of the light map to be overlayed
	std::string		m_strLightMapName;

	/// Whether or not to flip exported bitmap vertically
	bool			m_bFlipY;

	/// Whether or not the texture should be scaled to a power of two during export.
	bool			m_bPowerOfTwo;

	/// The border size of the exported bitmaps.
	unsigned int	m_nBorderSize;

}; // class CTexturemapExporterBMP


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// texture map to index map exporter class
class CTexturemapExporterIndexmap : public CExporter
{
public:
	/// construction
	CTexturemapExporterIndexmap();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,bool bValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// patch size used for export
	int			m_iPatchSize;

	/// file format used for export
	std::string	m_strFileFormat;

	/// Whether or not to flip exported bitmap vertically
	bool		m_bFlipY;

}; // class CTexturemapExporterIndexmap


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// texture map to alpha map exporter class
class CTexturemapExporterAlphamap : public CExporter
{
public:
	/// construction
	CTexturemapExporterAlphamap();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,int iValue);
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,float fValue);
	void		SetParameter(const std::string& strParameterName,bool bValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

	// --------------------------------------------------------------------------------
	// get some information from the export
	// --------------------------------------------------------------------------------

	/// Returns the border size of the generated bitmaps.
	/// This method will only return correct values after the export took place.
	/// @return The border size of the exported bitmaps.
	unsigned int	GetBitmapBorderSize() const {return this->m_nBorderSize;};


private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to preserve the border pixel for seamless sectors (0: no border pixels; 1: border pixels on right and top border; 2: border pixels on all borders).
	int				m_iBorderPixel;

	/// patch size used for export.
	int				m_iPatchSize;

	/// smoothing factor / filter kernel size.
	int				m_iSmoothFactor;

	/// whether or not to write completely black files.
	bool			m_bNoBlackFiles;

	/// file format used for export.
	std::string		m_strFileFormat;

	/// Whether or not to flip exported bitmap vertically.
	bool			m_bFlipY;

	/// Whether or not the texture should be scaled to a power of two during export.
	bool			m_bPowerOfTwo;

	/// The maximum number of channels to be stored in one alphamap.
	int				m_iMaxNumChannels;

	/// The scale factor for the exported alpha map.
	float			m_fScaleFactor;

	/// The format of the alphamap to be used. Only used, if alphamaps should be exported in an engine specific format.
	std::string		m_strAlphamapFormat;

	/// Whether or not to export material files.
	bool			m_bExportMaterialFiles;

	/// Whether or not the maximum value functionality should be used during gaussian smoothing.
	bool			m_GaussSmoothMax;

	/// The border size of the exported bitmaps.
	unsigned int	m_nBorderSize;


	// --------------------------------------------------------------------------------
	// member variables for PLSM2 export
	// --------------------------------------------------------------------------------

	/// The directory in which the PLSM2 map is located.
	std::string	m_strPLSM2RelativeMapDir;

	/// The name of the PLSM2 terrain.
	std::string	m_strPLSM2TerrainName;

	/// The format used for the detail textures.
	std::string	m_strPLSM2DetailTextureFormat;

	/// The directory in which the detail textures are located.
	std::string	m_strPLSM2DetailTextureBasePath;

	/// The name used for the normal maps.
	std::string	m_strPLSM2NormalMap;

	/// The name used for the texture maps.
	std::string	m_strPLSM2TextureMap;


	// --------------------------------------------------------------------------------
	// private helper methods
	// --------------------------------------------------------------------------------

	/// Sets a new pixel value dependend on channel index and coordinate.
	/// @param bmp The bitmap for which a pixel should be set.
	/// @param iX The x coordinate of the pixel to be set.
	/// @param iY The y coordinate of the pixel to be set.
	/// @param iChannelIndex The channel index to be set.
	/// <i>-1</i> for palettized bitmap.
	/// @param byValue the pixel value to be set.
	void		SetPixelValue(CBitmapFile& bmp,int iX,int iY,int iChannelIndex,BYTE byValue);

	/// Returns a material filename for specified patch and path.
	/// @param pSector The sector, for which the material file name should be returned.
	/// @param strPath The directory, in which the material file is located for which the material file name should be returned.
	/// @param iPatchX The x index of the patch within the sector, for which the material file name should be returned.
	/// @param iPatchY The y index of the patch within the sector, for which the material file name should be returned.
	/// @return Returns the name of the material file if successful,
	/// or an empty string in case of an error.
	std::string	XMLGetPatchMaterialFileName(CIPnPTCSector01* pSector,const std::string& strPath,int iPatchX,int iPatchY);

	/// Writes the material files for the alphamap export.
	/// @param pSector The sector, for which the material file should be written.
	/// @param strFileNameAlphaMap The name of the alphamap file, for which the material file should be written.
	/// @param vecCoverageMaps The list of coverage maps used in this material file.
	/// @param vecDetailTextures The list of detail textures associated with this material file coverage maps.
	/// @param iPatchX The x index of the patch within the sector, for which the material file name be written.
	/// @param iPatchY The y index of the patch within the sector, for which the material file name be written.
	/// @param bWriteDetailReferences Whether or not references to the detail textures should be written.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the material file could be written.
	bool		XMLWriteMaterialFile(CIPnPTCSector01* pSector,const std::string& strFileNameAlphaMap,const std::vector<std::string>& vecCoverageMaps,const std::vector<std::string>& vecDetailTextures,int iPatchX,int iPatchY,bool bWriteDetailReferences=true);


	// --------------------------------------------------------------------------------
	// private methods for PLSM2 export
	// --------------------------------------------------------------------------------

	/// Creates a new material file.
	/// @param pSector The sector, for which the material file should be created.
	/// @param strPath The directory, in which the material file should be created.
	/// @param strMapName The map, for which the material file should be created.
	/// @param iPatchX The x index of the patch within the sector, for which the material file should be created.
	/// @param iPatchY The y index of the patch within the sector, for which the material file should be created.
	/// @return Returns a handle to the created material file if successful,
	/// or <i>NULL</i> in case of an error.
	FILE*		PLSM2CreateMaterialFile(CIPnPTCSector01* pSector,const std::string& strPath,const std::string& strMapName,int iPatchX,int iPatchY);

	/// Returns a material filename for specified patch and path.
	/// @param pSector The sector, for which the material file name should be returned.
	/// @param strPath The directory, in which the material file is located for which the material file name should be returned.
	/// @param strMapName The name of the map, for which the material file name should be returned.
	/// @param iPatchX The x index of the patch within the sector, for which the material file name should be returned.
	/// @param iPatchY The y index of the patch within the sector, for which the material file name should be returned.
	/// @return Returns the name of the material file if successful,
	/// or an empty string in case of an error.
	std::string	PLSM2GetPatchMaterialFileName(CIPnPTCSector01* pSector,const std::string& strPath,const std::string& strMapName,int iPatchX,int iPatchY);

	/// Returns a texture filename for specified patch and path.
	/// @param pSector The sector, for which the texture file name should be returned.
	/// @param strBaseFileName The base texture file name, from which a patch and render pass specific texture file name should be created.
	/// @param iPatchX The x index of the patch within the sector, for which the texture file name should be returned.
	/// @param iPatchY The y index of the patch within the sector, for which the texture file name should be returned.
	/// @param iNumber The render pass index within this patch, for which the texture file name should be returned.
	/// @return Returns the name of the texture file for the given parameters.
	std::string	PLSM2GetTextureFileName(CIPnPTCSector01* pSector,const std::string& strBaseFileName,int iPatchX,int iPatchY,int iNumber);

	/// Writes the material files for the PLSM2 export.
	/// @param pSector The sector, for which the material file should be written.
	/// @param strFileNameAlphaMap The name of the alphamap file, for which the material file should be written.
	/// @param vecCoverageMaps The list of coverage maps used in this material file.
	/// @param vecDetailTextures The list of detail textures associated with this material file coverage maps.
	/// @param iPatchX The x index of the patch within the sector, for which the material file name be written.
	/// @param iPatchY The y index of the patch within the sector, for which the material file name be written.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the material file could be written.
	bool		PLSM2WriteMaterialFile(CIPnPTCSector01* pSector,const std::string& strFileNameAlphaMap,const std::vector<std::string>& vecCoverageMaps,const std::vector<std::string>& vecDetailTextures,int iPatchX,int iPatchY);

}; // class CTexturemapExporterAlphamap


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// light map to bmp exporter class
class CLightmapExporterBMP : public CExporter
{
public:
	/// construction
	CLightmapExporterBMP();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,bool bValue);
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

	// --------------------------------------------------------------------------------
	// get some information from the export
	// --------------------------------------------------------------------------------

	/// Returns the border size of the generated bitmaps.
	/// This method will only return correct values after the export took place.
	/// @return The border size of the exported bitmaps.
	unsigned int	GetBitmapBorderSize() const {return this->m_nBorderSize;};


private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to preserve the border pixel for seamless sectors (0: no border pixels; 1: border pixels on right and top border; 2: border pixels on all borders)
	int				m_iBorderPixel;

	/// patch size used for export
	int				m_iPatchSize;
	
	/// file format used for export
	std::string		m_strFileFormat;

	/// name of the light map to be exported
	std::string		m_strLightMapName;

	/// whether or not to flip exported bitmap vertically
	bool			m_bFlipY;

	/// whether or not to invert the exported bitmap
	bool			m_bInvert;

	/// The border size of the exported bitmaps.
	unsigned int	m_nBorderSize;

	/// Whether or not the texture should be scaled to a power of two during export.
	bool			m_bPowerOfTwo;

}; // class CLightmapExporterBMP


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// grass coverage to bmp exporter class
class CGrassExporterBMP : public CExporter
{
public:
	/// construction
	CGrassExporterBMP();

	/// see CExporter
	void		SetParameter(const std::string& strParameterName,bool bValue);
	void		SetParameter(const std::string& strParameterName,const std::string& strValue);
	void		SetParameter(const std::string& strParameterName,int iValue);
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	bool		WriteTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain,const std::string& Filename);
	std::string	GetDefaultFileName(CIPnPTCSector01* pSector);

private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// whether or not to create an index map
	bool		m_bIndexMap;

	/// patch size used for export
	int			m_iPatchSize;
	
	/// file format used for export
	std::string	m_strFileFormat;

	/// name of the grass map to be exported
	std::string	m_strGrassMapName;

	/// whether or not to flip exported bitmap vertically
	bool		m_bFlipY;

}; // class CGrassExporterBMP




#endif // !defined(PNPTC_EXPORTER_H_INCLUDED)