/////////////////////////////////////////////////////////////////////////////////////
// Importer.cpp
// Implementation of all heightmap importer classes.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:
//  30. Jan. 2007
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "importer.h"
#include "QuadInterpolator.h"
#include <math.h>
#include <string>
#include "utils.h"
#include "ProgressBar.h"

CImporter::CImporter(CIPnPTCApplication01& App)
	: m_Application(App)
	, m_fScale(1)
	, m_fOffset(0)
	, m_bImportHeightmap(true)
	, m_bImportTextures(false)
	, m_nSizeX(0)
	, m_nSizeY(0)
	, m_nSize(0)
	, m_fSourceDestFactorX(0)
	, m_fSourceDestFactorY(0)
	, m_bSizeDefinedByTerrainSource(false)
{
	// further initializations
	this->m_ptSourceArea[0].x=0;
	this->m_ptSourceArea[0].y=0;
	this->m_ptSourceArea[1].x=0;
	this->m_ptSourceArea[1].y=0;
	this->m_ptDestArea[0].x=0;
	this->m_ptDestArea[0].y=0;
	this->m_ptDestArea[1].x=0;
	this->m_ptDestArea[1].y=0;
}
CImporter::~CImporter()
{
}

bool CImporter::SetImportFileName(const std::string& strImportFileName)
{
	// has the file name changed?
	if (EqualNoCase(this->m_strImportFileName,strImportFileName) && !EqualNoCase(this->m_strImportFileName,""))
	{
		// nothing changed
		return true;
	}

	// try to open that file
	FILE* f=fopen(strImportFileName.c_str(),"r");
	if (f)
	{
		// ok
		fclose(f);

		// let derived class process this new file name
		if (this->OnNewFileName(strImportFileName))
		{
			// everything is ok
			this->m_strImportFileName=strImportFileName;
			return true;
		}
		else
		{
			// derived class says "no"
			return false;
		}
	}
	else
	{
		// does not exist or no right to access this file
		this->m_strImportFileName="";
		return false;
	}
}

void CImporter::SetParameter(const std::string& strParameterName,const std::string& strValue)
{
}
void CImporter::SetParameter(const std::string& strParameterName,int iValue)
{
}
void CImporter::SetParameter(const std::string& strParameterName,float fValue)
{
	if (EqualNoCase(strParameterName,"Offset"))
	{
		this->m_fOffset=fValue;
	}
	else if (EqualNoCase(strParameterName,"Scale"))
	{
		this->m_fScale=fValue;
	}
}
void CImporter::SetParameter(const std::string& strParameterName,bool bValue)
{
	if (EqualNoCase(strParameterName,"ImportTextures"))
	{
		this->m_bImportTextures=bValue;
	}
	else if (EqualNoCase(strParameterName,"ImportHeightmap"))
	{
		this->m_bImportHeightmap=bValue;
	}
}

unsigned int CImporter::GetSourceElement(unsigned int nX,unsigned int nY)
{
	// return terrain element
	return this->GetSourceSizeX()*minmax(nY,static_cast<unsigned int>(0),this->GetSourceSizeY()-1)+minmax(nX,static_cast<unsigned int>(0),this->GetSourceSizeX()-1);
}

bool CImporter::ImportSector(CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain)
{
	// initialize area for the sector import
	PnPTCHMapPosition ptMin;
	PnPTCHMapPosition ptMax;
	ptMin.x=0;
	ptMin.y=0;
	ptMax.x=Sector.GetBorderSize()-1;
	ptMax.y=Sector.GetBorderSize()-1;
	this->SetSourceImportArea(ptMin,ptMax);
	this->SetDestImportArea(ptMin,ptMax);

	// check source size
	if ((this->GetSourceSizeX()==0 && this->GetSourceSize()!=Sector.GetBorderSize()*Sector.GetBorderSize()) ||
		(this->GetSourceSizeX()!=Sector.GetBorderSize() || this->GetSourceSizeY()!=Sector.GetBorderSize()) )
	{
		// this doesn't work for the sector import
		MessageBox(NULL,"The size of the terrain source is invalid.\nFor sector import, the size of the source terrain should exactly match the size of a sector.\nUse the whole terrain import function instead, if you want to import areas which are larger or smaller than one sector.","Cannot import sector!",MB_OK|MB_ICONSTOP|MB_TASKMODAL);
		return false;
	}
	else if (!this->IsSizeDefinedByTerrainSource())
	{
		// set manual size
		this->SetSourceSizeX(Sector.GetBorderSize());
	}

	// import that sector
	return this->OnStartImport(Terrain) && this->ImportSector(Sector,this->m_ptDestArea[0],this->m_ptDestArea[1]);
}

bool CImporter::ImportSector(CIPnPTCSector01& Sector,const PnPTCHMapPosition& ptStart,const PnPTCHMapPosition& ptEnd,float fSourceOffsetX/*=0*/,float fSourceOffsetY/*=0*/)
{
	// set progress bar
	char cSectorName[100];
	sprintf(cSectorName,"Importing sector %d/%d",Sector.GetPosX(),Sector.GetPosY());
	pnptc::CProgressBar ProgressBar(this->m_Application,cSectorName,ptEnd.y-ptStart.y);

	// get textures
	CIPnPTCSectorTextures01* pTextures=(this->m_bImportTextures)?Sector.GetSectorTextures():NULL;

	// now import that sector
	PnPTC2DVec vecSource;
	for (int iDestY=ptStart.y;iDestY<=ptEnd.y;iDestY++)
	{
		// get source coordinate for this destination coordinate
		vecSource.y=static_cast<float>(iDestY-ptStart.y)*this->m_fSourceDestFactorY+fSourceOffsetY;

		for (int iDestX=ptStart.x;iDestX<=ptEnd.x;iDestX++)
		{
			// get source coordinate for this destination coordinate
			vecSource.x=static_cast<float>(iDestX-ptStart.x)*this->m_fSourceDestFactorX+fSourceOffsetX;

			// import heightmap
			if (this->m_bImportHeightmap)
			{
				// get height
				float fHeight=-1;
				if (this->m_fSourceDestFactorX!=1 || this->m_fSourceDestFactorY!=1)
				{
					// interpolate between four height values
					float fHeights[4];
					PnPTC2DVec vec;
					vec.x=floor(vecSource.x);
					vec.y=floor(vecSource.y);
					fHeights[0]=this->GetSourceHeight(vec);
					vec.x=ceil(vecSource.x);
					fHeights[1]=this->GetSourceHeight(vec);
					vec.x=floor(vecSource.x);
					vec.y=ceil(vecSource.y);
					fHeights[2]=this->GetSourceHeight(vec);
					vec.x=ceil(vecSource.x);
					fHeights[3]=this->GetSourceHeight(vec);
					CQuadInterpolator<float>::Interpolate(fHeights,minmax(vecSource.x-floor(vecSource.x),0.f,1.f),minmax(vecSource.y-floor(vecSource.y),0.f,1.f),fHeight);
				}
				else
				{
					// get one single height value
					fHeight=this->GetSourceHeight(vecSource);
				}
				if (fHeight>=0)
				{
					// adapt height by scale and offset
					fHeight=fHeight*this->m_fScale+this->m_fOffset;

					// set height
					Sector.SetHeight(iDestX,iDestY,fHeight);
				}
			}

			// import texture
			if (pTextures)
			{
				// get and set texture
				const char* cTexture=this->GetSourceTexture(vecSource);
				if (cTexture)
				{
					pTextures->SetTexture(iDestX,iDestY,cTexture);
				}
			}
		}

		// increment progress bar
		ProgressBar.IncrementPos();
	}

	// ok
	return true;
}

bool CImporter::ImportTerrain(CIPnPTCTerrain01& Terrain)
{
	// get the sector range, which is affected by the import operation
	int iSecStartX=this->m_ptDestArea[0].x/Terrain.GetSectorDetailBorderSize();
	int iSecStartY=this->m_ptDestArea[0].y/Terrain.GetSectorDetailBorderSize();
	int iSecEndX=this->m_ptDestArea[1].x/Terrain.GetSectorDetailBorderSize();
	int iSecEndY=this->m_ptDestArea[1].y/Terrain.GetSectorDetailBorderSize();

	// import range within the current sector
	PnPTCHMapPosition ptStart;
	PnPTCHMapPosition ptEnd;
	float fSourceOffsetX=0;
	float fSourceOffsetY=0;

	// start import
	if (!this->OnStartImport(Terrain))
	{
		return false;
	}

	// go through all sectors
	for (int iSecY=iSecStartY;iSecY<=iSecEndY;iSecY++)
	{
		// get start and end coordinates
		ptStart.y=0;
		ptEnd.y=Terrain.GetSectorDetailBorderSize()-1;
		if (iSecY==iSecStartY)
		{
			ptStart.y=this->m_ptDestArea[0].y%Terrain.GetSectorDetailBorderSize();
			fSourceOffsetY=static_cast<float>(this->m_ptSourceArea[0].y);
		}
		else if (iSecY==iSecEndY)
		{
			ptEnd.y=this->m_ptDestArea[1].y%Terrain.GetSectorDetailBorderSize();
		}

		for (int iSecX=iSecStartX;iSecX<=iSecEndX;iSecX++)
		{
			// get start and end coordinates
			ptStart.x=0;
			ptEnd.x=Terrain.GetSectorDetailBorderSize()-1;
			if (iSecX==iSecStartX)
			{
				ptStart.x=this->m_ptDestArea[0].x%Terrain.GetSectorDetailBorderSize();
				fSourceOffsetX=static_cast<float>(this->m_ptSourceArea[0].x);
			}
			else if (iSecX==iSecEndX)
			{
				ptEnd.x=this->m_ptDestArea[1].x%Terrain.GetSectorDetailBorderSize();
			}

			// get that sector
			CIPnPTCSector01* pSector=Terrain.GetSectorDetail(iSecX,iSecY);
			if (pSector)
			{
				// import that sector
				if (!this->ImportSector(*pSector,ptStart,ptEnd,fSourceOffsetX,fSourceOffsetY))
				{
					// error 
					return false;
				}
				pSector->Save();
			}
			else
			{
				// error
				return false;
			}

			// increment source offset
			if (iSecX==iSecStartX)
			{
				fSourceOffsetX+=static_cast<float>(Terrain.GetSectorDetailBorderSize()-(this->m_ptDestArea[0].x%Terrain.GetSectorDetailBorderSize()))*this->m_fSourceDestFactorX;
			}
			else
			{
				fSourceOffsetX+=static_cast<float>(Terrain.GetSectorDetailBorderSize())*this->m_fSourceDestFactorX;
			}
		}

		// increment source offset
		if (iSecY==iSecStartY)
		{
			fSourceOffsetY+=static_cast<float>(Terrain.GetSectorDetailBorderSize()-(this->m_ptDestArea[0].y%Terrain.GetSectorDetailBorderSize()))*this->m_fSourceDestFactorY;
		}
		else
		{
			fSourceOffsetY+=static_cast<float>(Terrain.GetSectorDetailBorderSize())*this->m_fSourceDestFactorY;
		}
	}

	// ok
	Terrain.Save();
	return true;
}
void CImporter::SetSourceImportArea(const PnPTCHMapPosition& ptMin,const PnPTCHMapPosition& ptMax)
{
	// set source area
	this->m_ptSourceArea[0]=ptMin;
	this->m_ptSourceArea[1]=ptMax;

	// compute scaling factors between source and dest area
	if (this->m_ptDestArea[1].x-this->m_ptDestArea[0].x>0)
	{
		this->m_fSourceDestFactorX=static_cast<float>(this->m_ptSourceArea[1].x-this->m_ptSourceArea[0].x)/static_cast<float>(this->m_ptDestArea[1].x-this->m_ptDestArea[0].x);
	}
	else
	{
		this->m_fSourceDestFactorX=1;
	}
	if (this->m_ptDestArea[1].y-this->m_ptDestArea[0].y>0)
	{
		this->m_fSourceDestFactorY=static_cast<float>(this->m_ptSourceArea[1].y-this->m_ptSourceArea[0].y)/static_cast<float>(this->m_ptDestArea[1].y-this->m_ptDestArea[0].y);
	}
	else
	{
		this->m_fSourceDestFactorY=1;
	}
}

void CImporter::SetDestImportArea(const PnPTCHMapPosition& ptMin,const PnPTCHMapPosition& ptMax)
{
	// set dest area
	this->m_ptDestArea[0]=ptMin;
	this->m_ptDestArea[1]=ptMax;

	// compute scaling factors between source and dest area
	if (this->m_ptDestArea[1].x-this->m_ptDestArea[0].x>0)
	{
		this->m_fSourceDestFactorX=static_cast<float>(this->m_ptSourceArea[1].x-this->m_ptSourceArea[0].x)/static_cast<float>(this->m_ptDestArea[1].x-this->m_ptDestArea[0].x);
	}
	else
	{
		this->m_fSourceDestFactorX=1;
	}
	if (this->m_ptDestArea[1].y-this->m_ptDestArea[0].y>0)
	{
		this->m_fSourceDestFactorY=static_cast<float>(this->m_ptSourceArea[1].y-this->m_ptSourceArea[0].y)/static_cast<float>(this->m_ptDestArea[1].y-this->m_ptDestArea[0].y);
	}
	else
	{
		this->m_fSourceDestFactorY=1;
	}
}

void CImporter::SetSourceSize(unsigned int nSizeX,unsigned int nSizeY,unsigned int nSize,bool bSizeDefinedByTerrainSource/*=true*/) 
{
	// set sizes
	this->m_nSizeX=nSizeX;
	this->m_nSizeY=nSizeY;
	this->m_nSize=nSize;
	this->m_bSizeDefinedByTerrainSource=bSizeDefinedByTerrainSource;
}

void CImporter::SetSourceSizeX(unsigned int nSizeX)
{
	// set size in x direction
	this->m_nSizeX=nSizeX;
	if (this->m_nSizeX!=0)
	{
		this->m_nSizeY=this->m_nSize/this->m_nSizeX;
	}
	else
	{
		this->m_nSizeY=0;
	}
}

void CImporter::SetSourceSizeY(unsigned int nSizeY)
{
	// set size in y direction
	this->m_nSizeY=nSizeY;
	if (this->m_nSizeY!=0)
	{
		this->m_nSizeX=this->m_nSize/this->m_nSizeY;
	}
	else
	{
		this->m_nSizeX=0;
	}
}

bool CImporter::OnStartImport(CIPnPTCTerrain01& Terrain)
{
	// ok, by default
	return true;
}

