/////////////////////////////////////////////////////////////////////////////////////
// Importer.h
// base class definition of heightmap importer classes.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:
//  30. Jan. 2007
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IMPORTER_H_INCLUDED)
#define PNPTC_IMPORTER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>	// The Windows PlatformSDK has to be installed in order to compile PlugIns
#include <string>
#include "bitmapfile.h"

// pnptc includes
#include "../../Includes/IPnPTCImportPlugIn.h"


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


/// importer base class.
class CImporter
{
public:

	// -----------------------------------------------------
	/// construction/destruction
	// -----------------------------------------------------

	/// Construction of a new importer instance.
	/// @param App A reference to the main application object.
	CImporter(CIPnPTCApplication01& App);

	/// Destruction and clean-up of the importer
	virtual ~CImporter();


	// -----------------------------------------------------
	// import source file(s)
	// -----------------------------------------------------

	/// sets the name of the file from which to import the terrain.
	/// @param strImportFileName Name of the file from which to import the terrain.
	/// @return Returns <i>true</i> or <i>false</i>, whether a file with that name exists.
	bool					SetImportFileName(const std::string& strImportFileName);
	
	/// returns the name of the file from which to import the terrain.
	/// @return Name of the file from which to import the terrain. 
	std::string				GetImportFileName() const {return this->m_strImportFileName;};
	

	// -----------------------------------------------------
	// import parameters
	// -----------------------------------------------------

	/// sets the area of the source terrain, which should be imported
	/// @param ptMin The coordinate of the lower/left (south/west) corner
	/// of a rectangular area in the source terrain which should be imported.
	/// @param ptMax The coordinate of the upper/right (north/east) corner
	/// of a rectangular area in the source terrain which should be imported.
	void					SetSourceImportArea(const PnPTCHMapPosition& ptMin,const PnPTCHMapPosition& ptMax);

	/// sets the area of the destination terrain, which should be imported
	/// @param ptMin The coordinate of the lower/left (south/west) corner
	/// of a rectangular area in the destination terrain which should be imported.
	/// @param ptMax The coordinate of the upper/right (north/east) corner
	/// of a rectangular area in the destination terrain which should be imported.
	void					SetDestImportArea(const PnPTCHMapPosition& ptMin,const PnPTCHMapPosition& ptMax);

	/// sets a vertical scaling factor for the imported height values.
	/// @param fScale vertical scaling factor.
	void					SetScalingFactor(float fScale) {this->m_fScale=fScale;};

	/// sets a vertical offset for the imported height values.
	/// @param fOffset vertical offset of the imported height values.
	void					SetOffset(float fOffset) {this->m_fOffset=fOffset;};

	/// sets, whether or not the heightmap should be imported.
	/// @param bImportHeightmap <i>true</i> or <i>false</i>, whether or not
	/// heightmap should be imported.
	void					SetImportHeightmap(bool bImportHeightmap) {this->m_bImportHeightmap=bImportHeightmap;};
	
	/// sets, whether or not textures should be imported.
	/// @param bImportTextures <i>true</i> or <i>false</i>, whether or not
	/// textures should be imported.
	void					SetImportTextures(bool bImportTextures) {this->m_bImportTextures=bImportTextures;};


	// -----------------------------------------------------
	// import data methods, which can/should be overridden
	// -----------------------------------------------------

	/// sets an import string parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param strValue parameter value to be set.
	virtual void			SetParameter(const std::string& strParameterName,const std::string& strValue);

	/// sets an import integer parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param iValue parameter value to be set.
	virtual void			SetParameter(const std::string& strParameterName,int iValue);

	/// sets an import float parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param fValue parameter value to be set.
	virtual void			SetParameter(const std::string& strParameterName,float fValue);

	/// sets an import bool parameter.
	/// @param strParameterName name of the parameter to be set.
	/// @param bValue parameter value to be set.
	virtual void			SetParameter(const std::string& strParameterName,bool bValue);

	/// Returns the height of the terrain source at a given position.
	/// @param vecPos Terrain position at which the source terrain height 
	/// should be returned.
	/// @return Returns the height of the source terrain at the given
	/// position. In case of an error <i>-1</i> is returned.
	virtual float			GetSourceHeight(const PnPTC2DVec& vecPos)=0;

	/// Returns the texture of the terrain source at a given position.
	/// @param vecPos Terrain position at which the source terrain texture 
	/// should be returned.
	/// @return Returns the name of the texture of the source terrain at the given
	/// position. In case of an error <i>NULL</i> is returned.
	virtual const char*		GetSourceTexture(const PnPTC2DVec& vecPos)=0;


	// -----------------------------------------------------
	// retrieve and set parameters from the source terrain
	// -----------------------------------------------------

	/// Returns the size of the terrain source file in x direction.
	/// @return Returns the size of the terrain source file in x direction.
	/// A return value of zero indicates either an error or that the
	/// size in this direction is not defined.
	unsigned int			GetSourceSizeX() const {return this->m_nSizeX;};

	/// Returns the size of the terrain source file in y direction.
	/// @return Returns the size of the terrain source file in y direction.
	/// A return value of zero indicates either an error or that the
	/// size in this direction is not defined.
	unsigned int			GetSourceSizeY() const {return this->m_nSizeY;};

	/// Returns the overall size of the terrain source file.
	/// @return Returns the overall size of the terrain source file.
	/// A return value of zero indicates an error.
	unsigned int			GetSourceSize() const {return this->m_nSize;};

	/// Manually sets the size of the terrain in x direction.
	/// This is required if the terrain size can not be
	/// determined by the file format and the user has to set the 
	/// source terrain size manually.
	/// @param nSizeX Manually set terrain size in x direction
	void					SetSourceSizeX(unsigned int nSizeX);

	/// Manually sets the size of the terrain in y direction.
	/// This is required if the terrain size can not be
	/// determined by the file format and the user has to set the 
	/// source terrain size manually.
	/// @param nSizeY Manually set terrain size in y direction
	void					SetSourceSizeY(unsigned int nSizeY);

	/// Returns, whether or not the source terrain size is completely
	/// defined within the source file.
	bool					IsSizeDefinedByTerrainSource() const {return this->m_bSizeDefinedByTerrainSource;};


	// -----------------------------------------------------
	// terrain import
	// -----------------------------------------------------

	/// imports one single sector from a file.
	/// @param Sector reference to the sector to be imported.
	/// @param Terrain reference to the terrain to be imported.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the import was successfull.
	bool					ImportSector(CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain);

	/// imports the complete terrain to a file.
	/// @param Terrain reference to the terrain to be imported.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the import was successfull.
	bool					ImportTerrain(CIPnPTCTerrain01& Terrain);


protected:

	// -----------------------------------------------------
	// internal overridable methods
	// -----------------------------------------------------

	/// this method is called, when the import file name is changed.
	/// override this method to read all required parameters from that file.
	/// @param strImportFileName Name of the file from which to import the terrain.
	/// @return Returns <i>true</i> if this file can be imported, otherwise <i>false</i>.
	virtual bool			OnNewFileName(const std::string& strImportFileName)=0;

	/// Called directly before the import of the terrain starts.
	/// @param Terrain reference to the terrain to be imported.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the import can be continued.
	virtual bool			OnStartImport(CIPnPTCTerrain01& Terrain);


	// -----------------------------------------------------
	// internal methods
	// -----------------------------------------------------

	/// Sets the size of the source terrain. This method should be called
	/// from within the OnNewFileName method.
	/// @param nSizeX size of the source terrain in x direction.
	/// @param nSizeY size of the source terrain in y direction.
	/// @param nSize overall size of the source terrain.
	/// @param bSizeDefinedByTerrainSource Whether or not the source terrain size
	/// is completely defined by the source file.
	void					SetSourceSize(unsigned int nSizeX,unsigned int nSizeY,unsigned int nSize,bool bSizeDefinedByTerrainSource=true);

	/// Returns the index of an element given its x and y coordinates.
	/// @param nX X coordinate for which to return the heightmap element index.
	/// @param nY Y coordinate for which to return the heightmap element index.
	/// @return Returns the heightmap element index of the given heightmap coordinate.
	unsigned int			GetSourceElement(unsigned int nX,unsigned int nY);

	/// imports one single sector from a file.
	/// @param Sector reference to the sector to be imported.
	/// @param ptStart Position on the sector, where the import should start.
	/// @param ptEnd Position on the sector, where the import should end.
	/// @param fSourceOffsetX Offset between the destination and source 
	/// coordinates in x direction.
	/// @param fSourceOffsetY Offset between the destination and source 
	/// coordinates in y direction.
	/// @return returns <i>true</i> or <i>false</i>, whether or
	/// not the import was successfull.
	bool					ImportSector(CIPnPTCSector01& Sector,const PnPTCHMapPosition& ptStart,const PnPTCHMapPosition& ptEnd,float fSourceOffsetX=0,float fSourceOffsetY=0);


private:

	// --------------------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------------------

	/// Name of the file from which to import the terrain.
	std::string				m_strImportFileName;

	/// vertical scaling factor.
	float					m_fScale;

	/// vertical offset value.
	float					m_fOffset;

	/// size of the source terrain in x direction.
	unsigned int			m_nSizeX;

	/// size of the source terrain in y direction.
	unsigned int			m_nSizeY;

	/// overall size of the source terrain. If the single size components
	/// in x and y direction are not defined, the overall size still might be
	/// defined. That's why this is stored in a separate variable.
	unsigned int			m_nSize;

	/// Whether or not the terrain size is completely defined within the source file.
	bool					m_bSizeDefinedByTerrainSource;

	/// Factor between the size of the source and the dest 
	/// terrain import area in x direction
	float					m_fSourceDestFactorX;

	/// Factor between the size of the source and the dest 
	/// terrain import area in y direction
	float					m_fSourceDestFactorY;

	/// whether or not the heightmap should be imported.
	bool					m_bImportHeightmap;
	
	/// whether or not textures should be imported.
	bool					m_bImportTextures;


protected:

	/// rectangular area of the source terrain which should be imported.
	PnPTCHMapPosition		m_ptSourceArea[2];

	/// rectangular area of the destination terrain which should be imported.
	PnPTCHMapPosition		m_ptDestArea[2];


	// --------------------------------------------------------------------------------
	// member variables which are accessable from derived classes
	// --------------------------------------------------------------------------------

	/// reference to the main application object.
	CIPnPTCApplication01&	m_Application;
};



#endif // !defined(PNPTC_IMPORTER_H_INCLUDED)