/////////////////////////////////////////////////////////////////////////////////////
// NumericEdit.cpp
// MFC edit/text control supporting numeric input format.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  10. Oct. 2006 :	Achieved VS2005 compatibility, made compilation easier.
//  04. Feb. 2007 : Added NotifyParent
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#ifndef WINVER
#if _MSC_VER >= 1300
#define WINVER 0x0500
#else
#define WINVER 0x0400
#endif
#endif


#include ".\numericedit.h"


// CNumericEdit

IMPLEMENT_DYNAMIC(CNumericEdit, CEdit)

CNumericEdit::CNumericEdit() 
{
	this->m_strDelim=".";
	this->m_bChanged=false;
	this->m_dwLastKey=GetTickCount();
	this->m_bAllowDecimals=true;
	this->m_bCheckInputValueRange=false;
	this->m_fMin=0;
	this->m_fMax=0;
	this->m_bAllowNegatives=false;
	this->m_nTimerID=0;
}

CNumericEdit::~CNumericEdit()
{
	// stop timer
	this->StopTimer();
}


BEGIN_MESSAGE_MAP(CNumericEdit, CEdit)	
	ON_WM_CHAR()	
	ON_CONTROL_REFLECT(EN_KILLFOCUS, OnEnKillfocus)
	ON_CONTROL_REFLECT(EN_SETFOCUS, OnEnSetfocus)
	ON_WM_DESTROY()
	ON_WM_TIMER()
END_MESSAGE_MAP()


void CNumericEdit::SetAllowDecimals(bool bAllowDecimals)
{
	// store allow decimals
	this->m_bAllowDecimals=bAllowDecimals;
}

void CNumericEdit::SetAllowNegatives(bool bAllowNegatives)
{
	// store allow negatives
	this->m_bAllowNegatives=bAllowNegatives;
}

void CNumericEdit::SetRange(float fMin,float fMax)
{
	// check input value range
	this->m_bCheckInputValueRange=true;
	this->m_fMin=min(fMin,fMax);
	this->m_fMax=max(fMax,fMin);
}

void CNumericEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	// create timer
	if (this->m_nTimerID==0)
	{
		this->m_nTimerID=static_cast<UINT>(this->SetTimer(100,200,NULL));
	}

	// set time stamp
	this->m_dwLastKey=GetTickCount();

	// delimiter entered
	if( nChar == m_strDelim )
	{
		// decimals allowed?
		if (!this->m_bAllowDecimals)
		{
			return;
		}

		CString strText;
		GetWindowText( strText );

		if( strText.IsEmpty() )
		{
			SetWindowText( (_T("0") + m_strDelim) );
			SetSel( 2, 2, FALSE );
		}
		else
		{
			// if the decimal point already entered, not allow enter more points
			if( strText.Find( m_strDelim ) >= 0 )
				return;
		}
	}

	// negative value
	if (nChar == '-')
	{
		// negative values allowed
		if (!this->m_bAllowNegatives)
		{
			return;
		}

		// has to be at first position
		int iStart,iEnd;
		this->GetSel(iStart,iEnd);
		if (iStart!=0)
		{
			return;
		}
	}


	// 8 - back space
	// 46 - .
	// 48 - 1
	// 57 - 0
	if( (nChar == m_strDelim) || (nChar >= '0' && nChar <= '9') || (nChar == 8) || (nChar == '-') )
	{
		CEdit::OnChar(nChar, nRepCnt, nFlags);
	}
	// return
	else if (nChar == 13)
	{
		this->CheckInputValueRange();
		this->GetParent()->SendMessage(WM_COMMAND, MAKEWPARAM( GetDlgCtrlID(), EN_NEWDATA),(LPARAM) GetSafeHwnd());
	}

	// data changed
	this->m_bChanged=true;
}

float CNumericEdit::GetValue()
{
	// return text
	CString strText;
	this->GetWindowText(strText);
	float fVal;
	sscanf((char*)(LPCSTR)strText,"%f",&fVal);
	return fVal;
}

void CNumericEdit::SetValue(float fValue)
{
	// display text
	CString strText;
	if (this->m_bAllowDecimals)
	{
		strText.Format("%.3f",fValue);
	}
	else
	{
		strText.Format("%d",(int)fValue);
	}
	this->SetWindowText(strText);
}

void CNumericEdit::OnEnKillfocus()
{
	// send new data
	if (this->m_bChanged)
	{
		this->CheckInputValueRange();
		this->GetParent()->SendMessage(WM_COMMAND, MAKEWPARAM( GetDlgCtrlID(), EN_NEWDATA),(LPARAM) GetSafeHwnd());
		this->m_bChanged=false;
	}

	// stop timer
	this->StopTimer();
}

void CNumericEdit::OnEnSetfocus()
{
	// nothing changed yet
	this->m_bChanged=false;
}

void CNumericEdit::CheckInputValueRange()
{
	if (this->m_bCheckInputValueRange)
	{
		float fValue=this->GetValue();
		if (fValue<this->m_fMin)
		{
			this->SetValue(this->m_fMin);
		}
		else if (fValue>this->m_fMax)
		{
			this->SetValue(this->m_fMax);
		}
	}
}

void CNumericEdit::OnDestroy()
{
	// stop timer
	this->StopTimer();

	// destroy window
	CEdit::OnDestroy();
}

void CNumericEdit::OnTimer(UINT nIDEvent)
{
	if (nIDEvent==this->m_nTimerID)
	{
		// check for change
		if (!this->m_bChanged)
		{
			// nothing changed => continue
			return;
		}

		// get parent window handle
		HWND hwndParent=NULL;
		if (!this->GetSafeHwnd() || !this->GetParent() || 
			!(hwndParent=this->GetParent()->GetSafeHwnd()))
		{
			// error, no window handle
			return;
		}

		// check time out
		if ((GetTickCount()-this->m_dwLastKey)>=1000)
		{
			this->CheckInputValueRange();
			::PostMessage(hwndParent,WM_COMMAND, MAKEWPARAM( this->GetDlgCtrlID(), EN_NEWDATA),(LPARAM)this->GetSafeHwnd());
			this->m_bChanged=false;
		}
	}
	else
	{
		// default timer
		CEdit::OnTimer(nIDEvent);
	}
}

void CNumericEdit::StopTimer()
{
	// stop timer
	if (this->m_nTimerID!=0)
	{
		this->KillTimer(this->m_nTimerID);
		this->m_nTimerID=0;
	}
}

void CNumericEdit::NotifyParent()
{
	// get parent window handle
	HWND hwndParent=NULL;
	if (!this->GetSafeHwnd() || !this->GetParent() || 
		!(hwndParent=this->GetParent()->GetSafeHwnd()))
	{
		// error, no window handle
		return;
	}

	// notifies the parent of a changed value
	::PostMessage(hwndParent,WM_COMMAND, MAKEWPARAM( this->GetDlgCtrlID(), EN_NEWDATA),(LPARAM)this->GetSafeHwnd());
}

