/////////////////////////////////////////////////////////////////////////////////////
// pnpseBox.h
// Definition of a 3 dimensional box class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_BOX_H_INCLUDED)
#define PNPSE_BOX_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// base includes
#include "../utils.h"
#include "pnpseVector3.h"


// pnp sensory game engine
namespace pnpse	
{

// forward definitions
class CRay;

/// Box area class
class CBox
{
public:

	// -----------------------------------------------------------------
	// construction of a box
	// -----------------------------------------------------------------

	/// Constructs a box without initialization.
	/// all elements are initialized with 0 by default.
	inline CBox() {};

	/// Construction of a box from two points.
	/// @param vecMin One border point of the box to be created.
	/// @param vecMin Second border point of the box to be created.
	inline CBox(const CVector3& vecMin,const CVector3& vecMax)
		: m_vecMin(minimum(vecMin.x,vecMax.x),minimum(vecMin.y,vecMax.y),minimum(vecMin.z,vecMax.z)),
		  m_vecMax(maximum(vecMin.x,vecMax.x),maximum(vecMin.y,vecMax.y),maximum(vecMin.z,vecMax.z)) {};

	/// Construction of a box from two coordinate pairs.
	/// @param fX1 x coordinate of one border point of the box to be created.
	/// @param fY1 y coordinate of one border point of the box to be created.
	/// @param fZ1 z coordinate of one border point of the box to be created.
	/// @param fX2 x coordinate of the second border point of the box to be created.
	/// @param fY2 y coordinate of the second border point of the box to be created.
	/// @param fZ2 z coordinate of the second border point of the box to be created.
	inline CBox(const float fX1,const float fY1,const float fZ1,const float fX2,const float fY2,const float fZ2)
		: m_vecMin(minimum(fX1,fX2),minimum(fY1,fY2),minimum(fZ1,fZ2)),
		  m_vecMax(maximum(fX1,fX2),maximum(fY1,fY2),maximum(fZ1,fZ2)) {};


	// -----------------------------------------------------------------
	// box data
	// -----------------------------------------------------------------

	/// Get minimum border point.
	/// @return Returns the minimum border point.
	inline const CVector3&	GetMin() const {return this->m_vecMin;};

	/// Get maximum border point.
	/// @return Returns the maximum border point.
	inline const CVector3&	GetMax() const {return this->m_vecMax;};

	/// Get center of box
	/// @return Returns the box's center.
	inline CVector3		GetCenter() const {return (this->m_vecMax+this->m_vecMin)/2;};

	/// Get height of box (y coordinate).
	/// @return Returns the height of the box.
	inline float		GetHeight() const {return this->m_vecMax.y-this->m_vecMin.y;};

	/// Get width of box (x coordinate).
	/// @return Returns the width of the box.
	inline float		GetWidth() const {return this->m_vecMax.x-this->m_vecMin.x;};

	/// Get depth of box (z coordinate).
	/// @return Returns the depth of the box.
	inline float		GetDepth() const {return this->m_vecMax.z-this->m_vecMin.z;};


	// -----------------------------------------------------------------
	// box manipulation
	// -----------------------------------------------------------------

	/// Sets the edges of the box from two points.
	/// @param vecMin One border point of the box to be created.
	/// @param vecMin Second border point of the box to be created.
	inline void	SetBorderPoints(const CVector3& vecMin,const CVector3& vecMax)
	{
		this->m_vecMin.x=minimum(vecMin.x,vecMax.x);
		this->m_vecMin.y=minimum(vecMin.y,vecMax.y);
		this->m_vecMin.z=minimum(vecMin.z,vecMax.z);
		this->m_vecMax.x=maximum(vecMin.x,vecMax.x);
		this->m_vecMax.y=maximum(vecMin.y,vecMax.y);
		this->m_vecMax.z=maximum(vecMin.y,vecMax.z);
	};

	/// Sets the edges of the box from two coordinate pairs.
	/// @param fX1 x coordinate of one border point of the box to be created.
	/// @param fY1 y coordinate of one border point of the box to be created.
	/// @param fZ1 z coordinate of one border point of the box to be created.
	/// @param fX2 x coordinate of the second border point of the box to be created.
	/// @param fY2 y coordinate of the second border point of the box to be created.
	/// @param fZ2 z coordinate of the second border point of the box to be created.
	inline void	SetBorderPoints(const float fX1,const float fY1,const float fZ1,const float fX2,const float fY2,const float fZ2)
	{
		this->m_vecMin.x=minimum(fX1,fX2);
		this->m_vecMin.y=minimum(fY1,fY2);
		this->m_vecMin.z=minimum(fZ1,fZ2);
		this->m_vecMax.x=maximum(fX1,fX2);
		this->m_vecMax.y=maximum(fY1,fY2);
		this->m_vecMax.z=maximum(fZ1,fZ2);
	};

	/// Moves the box. The minimum border point of the box
	/// will be moved to the given position. The width, height and depth
	/// will be unchanged.
	/// @param vecMoveTo Position to which the box should be moved.
	inline void MoveTo(const CVector3& vecMoveTo) 
	{
		this->m_vecMax.x=vecMoveTo.x+this->GetWidth();
		this->m_vecMax.y=vecMoveTo.y+this->GetHeight();
		this->m_vecMax.z=vecMoveTo.z+this->GetDepth();
		this->m_vecMin=vecMoveTo;
	};

	/// Moves the box. The minimum border point of the box
	/// will be moved to the given position. The width, height and depth
	/// will be unchanged.
	/// @param fMoveToX x coordinate where the box should be moved to.
	/// @param fMoveToY y coordinate where the box should be moved to.
	/// @param fMoveToZ z coordinate where the box should be moved to.
	inline void MoveTo(const float fMoveToX,const float fMoveToY,const float fMoveToZ) 
	{
		this->m_vecMax.x=fMoveToX+this->GetWidth();
		this->m_vecMax.y=fMoveToX+this->GetHeight();
		this->m_vecMax.z=fMoveToZ+this->GetDepth();
		this->m_vecMin.x=fMoveToX;
		this->m_vecMin.y=fMoveToY;
		this->m_vecMin.z=fMoveToZ;
	};

	/// Moves the box. The box is moved relatively to
	/// its current position without changing height and width.
	/// @param vecMoveRel Relative movement of box.
	inline void MoveRel(const CVector3& vecMoveRel) 
	{
		this->m_vecMin+=vecMoveRel;
		this->m_vecMax+=vecMoveRel;
	};

	/// Moves the box. The box is moved relatively to
	/// its current position without changing height, width and depth.
	/// @param fMoveRelX Movement in x direction.
	/// @param fMoveRelY Movement in y direction.
	/// @param fMoveRelZ Movement in z direction.
	inline void MoveRel(const float fMoveRelX,const float fMoveRelY,const float fMoveRelZ) 
	{
		this->m_vecMin.x+=fMoveRelX;
		this->m_vecMin.y+=fMoveRelY;
		this->m_vecMin.z+=fMoveRelZ;
		this->m_vecMax.x+=fMoveRelX;
		this->m_vecMax.y+=fMoveRelY;
		this->m_vecMax.z+=fMoveRelZ;
	};

	/// Inflates the box by moving its borders outwards.
	/// Negative inflation corresponds to a deflation.
	/// @param fInflateX Amount of inflation in x direction.
	/// @param fInflateY Amount of inflation in y direction.
	/// @param fInflateZ Amount of inflation in z direction.
	inline void Inflate(float fInflateX,float fInflateY,float fInflateZ)
	{
		if (fInflateX<0)
		{
			fInflateX=minimum(fInflateX,this->GetWidth()/2);
		}
		this->m_vecMax.x+=fInflateX;
		this->m_vecMin.x-=fInflateX;
		if (fInflateY<0)
		{
			fInflateY=minimum(fInflateY,this->GetHeight()/2);
		}
		this->m_vecMax.y+=fInflateY;
		this->m_vecMin.y-=fInflateY;
		if (fInflateZ<0)
		{
			fInflateZ=minimum(fInflateZ,this->GetDepth()/2);
		}
		this->m_vecMax.z+=fInflateZ;
		this->m_vecMin.z-=fInflateZ;
	};

	/// Deflates the box by moving its borders inwards.
	/// Negative deflation corresponds to a inflation.
	/// @param fDeflateX Amount of deflation in x direction.
	/// @param fDeflateY Amount of deflation in y direction.
	/// @param fDeflateZ Amount of deflation in z direction.
	inline void Deflate(float fDeflateX,float fDeflateY,float fDeflateZ) {this->Inflate(-fDeflateX,-fDeflateY,-fDeflateZ);};


	// -----------------------------------------------------------------
	// box test
	// -----------------------------------------------------------------

	/// Tests whether or not a point is inside the box.
	/// @param vecTest Point/vector to be tested.
	/// @return Returns <i>true</i> if the point lies inside the
	/// box otherwise <i>false</i>.
	inline bool		Inside(const CVector3& vecTest) const {return vecTest>=this->m_vecMin && vecTest<=this->m_vecMax;};

	/// Tests whether or not a box is completely inside this box.
	/// @param boxTest box to be tested.
	/// @return Returns <i>true</i> if the tested box lies completely inside this
	/// box otherwise <i>false</i>.
	inline bool		Inside(const CBox& boxTest) const {return this->Inside(boxTest.m_vecMin) && this->Inside(boxTest.m_vecMax);};

	/// Tests whether or not two boxes overlap.
	/// @param boxTest Box to be tested.
	/// @return Returns <i>true</i> if the two boxes overlap,
	/// otherwise <i>false</i>.
	inline bool		Overlap(const CBox& boxTest) const {return this->Inside(boxTest.m_vecMin) || this->Inside(boxTest.m_vecMax) || boxTest.Inside(this->m_vecMin) || boxTest.Inside(this->m_vecMax);};

	/// Converts a point into local space of the box.
	/// This means the position of the point with respect to the minimum
	/// coordinate of the box
	/// @param vecConvert Point to be converted.
	inline void		ConvertToLocalSpace(CVector3& vecConvert) const {vecConvert-=this->m_vecMin;};

	/// Tests whether or not a ray hits the box.
	/// @param rayTest Ray to be tested for hitting the box.
	/// @return Returns <i>true</i> if the ray hits the box,
	/// otherwise <i>false</i>.
	bool			Hit(const CRay& rayTest) const;


protected:

	// -----------------------------------------------------------------
	// private members of the box
	// -----------------------------------------------------------------

	/// Minimum coordinate of box
	CVector3	m_vecMin;

	/// Maximum coordinate of box
	CVector3	m_vecMax;

};	// class CBox

};	// namespace pnpse

#endif // !defined(PNPSE_BOX_H_INCLUDED)
