/////////////////////////////////////////////////////////////////////////////////////
// pnpseRect.h
// Definition of a rectangle class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_RECT_H_INCLUDED)
#define PNPSE_RECT_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// base includes
#include "../utils.h"
#include "pnpseVector2.h"


// pnp sensory game engine
namespace pnpse	
{

/// Rectangle area class
class CRect
{
public:

	// -----------------------------------------------------------------
	// construction of a rectangle
	// -----------------------------------------------------------------

	/// Constructs a rectangle without initialization.
	/// all elements are initialized with 0 by default.
	inline CRect() {};

	/// Construction of a rectangle from two points.
	/// @param vecMin One border point of the rectangle to be created.
	/// @param vecMin Second border point of the rectangle to be created.
	inline CRect(const CVector2& vecMin,const CVector2& vecMax)
		: m_vecMin(minimum(vecMin.x,vecMax.x),minimum(vecMin.y,vecMax.y)),
		  m_vecMax(maximum(vecMin.x,vecMax.x),maximum(vecMin.y,vecMax.y)) {};

	/// Construction of a rectangle from two coordinate pairs.
	/// @param fX1 x coordinate of one border point of the rectangle to be created.
	/// @param fY1 y coordinate of one border point of the rectangle to be created.
	/// @param fX2 x coordinate of the second border point of the rectangle to be created.
	/// @param fY2 y coordinate of the second border point of the rectangle to be created.
	inline CRect(const float fX1,const float fY1,const float fX2,const float fY2)
		: m_vecMin(minimum(fX1,fX2),minimum(fY1,fY2)),
		  m_vecMax(maximum(fX1,fX2),maximum(fY1,fY2)) {};


	// -----------------------------------------------------------------
	// rectangle data
	// -----------------------------------------------------------------

	/// Get minimum border point.
	/// @return Returns the minimum border point.
	inline const CVector2&	GetMin() const {return this->m_vecMin;};

	/// Get maximum border point.
	/// @return Returns the maximum border point.
	inline const CVector2&	GetMax() const {return this->m_vecMax;};

	/// Get center of rectangle
	/// @return Returns the rectangle's center.
	inline CVector2		GetCenter() const {return (this->m_vecMax+this->m_vecMin)/2;};

	/// Get height of rectangle.
	/// @return Returns the height of the rectangle.
	inline float		GetHeight() const {return this->m_vecMax.y-this->m_vecMin.y;};

	/// Sets the height of the rectangle by changing
	/// the maximum extend border of it. The minimum
	/// border point will stay unchanged.
	/// @param fHeight The height of the rectangle to be set.
	inline void			SetHeight(float fHeight) {this->m_vecMax.y=this->m_vecMin.y+fHeight;};

	/// Get width of rectangle.
	/// @return Returns the width of the rectangle.
	inline float		GetWidth() const {return this->m_vecMax.x-this->m_vecMin.x;};

	/// Sets the width of the rectangle by changing
	/// the maximum extend border of it. The minimum
	/// border point will stay unchanged.
	/// @param fWidth The width of the rectangle to be set.
	inline void			SetWidth(float fWidth) {this->m_vecMax.x=this->m_vecMin.x+fWidth;};


	// -----------------------------------------------------------------
	// rectangle manipulation
	// -----------------------------------------------------------------

	/// Sets the edges of the rectangle from two points.
	/// @param vecMin One border point of the rectangle to be created.
	/// @param vecMin Second border point of the rectangle to be created.
	inline void	SetBorderPoints(const CVector2& vecMin,const CVector2& vecMax)
	{
		this->m_vecMin.x=minimum(vecMin.x,vecMax.x);
		this->m_vecMin.y=minimum(vecMin.y,vecMax.y);
		this->m_vecMax.x=maximum(vecMin.x,vecMax.x);
		this->m_vecMax.y=maximum(vecMin.y,vecMax.y);
	};

	/// Sets the edges of the rectangle from two coordinate pairs.
	/// @param fX1 x coordinate of one border point of the rectangle to be created.
	/// @param fY1 y coordinate of one border point of the rectangle to be created.
	/// @param fX2 x coordinate of the second border point of the rectangle to be created.
	/// @param fY2 y coordinate of the second border point of the rectangle to be created.
	inline void	SetBorderPoints(const float fX1,const float fY1,const float fX2,const float fY2)
	{
		this->m_vecMin.x=minimum(fX1,fX2);
		this->m_vecMin.y=minimum(fY1,fY2);
		this->m_vecMax.x=maximum(fX1,fX2);
		this->m_vecMax.y=maximum(fY1,fY2);
	};

	/// Moves the rectangle. The minimum border point of the rectangle
	/// will be moved to the given position. The width and height will
	/// be unchanged.
	/// @param vecMoveTo Position to which the rectangle should be moved.
	inline void MoveTo(const CVector2& vecMoveTo) 
	{
		this->m_vecMax.x=vecMoveTo.x+this->GetWidth();
		this->m_vecMax.y=vecMoveTo.y+this->GetHeight();
		this->m_vecMin=vecMoveTo;
	};

	/// Moves the rectangle. The minimum border point of the rectangle
	/// will be moved to the given position. The width and height will
	/// be unchanged.
	/// @param fMoveToX x coordinate where the rectangle should be moved to.
	/// @param fMoveToY y coordinate where the rectangle should be moved to.
	inline void MoveTo(const float fMoveToX,const float fMoveToY) 
	{
		this->m_vecMax.x=fMoveToX+this->GetWidth();
		this->m_vecMax.y=fMoveToX+this->GetHeight();
		this->m_vecMin.x=fMoveToX;
		this->m_vecMin.y=fMoveToY;
	};

	/// Moves the rectangle. The rectangle is moved relatively to
	/// its current position without changing height and width.
	/// @param vecMoveRel Relative movement of rectangle.
	inline void MoveRel(const CVector2& vecMoveRel) 
	{
		this->m_vecMin+=vecMoveRel;
		this->m_vecMax+=vecMoveRel;
	};

	/// Moves the rectangle. The rectangle is moved relatively to
	/// its current position without changing height and width.
	/// @param fMoveRelX Movement in x direction.
	/// @param fMoveRelY Movement in y direction.
	inline void MoveRel(const float fMoveRelX,const float fMoveRelY) 
	{
		this->m_vecMin.x+=fMoveRelX;
		this->m_vecMin.y+=fMoveRelY;
		this->m_vecMax.x+=fMoveRelX;
		this->m_vecMax.y+=fMoveRelY;
	};

	/// Inflates the rectangle by moving its borders outwards.
	/// Negative inflation corresponds to a deflation.
	/// @param fInflateX Amount of inflation in x direction.
	/// @param fInflateY Amount of inflation in y direction.
	inline void Inflate(float fInflateX,float fInflateY)
	{
		if (fInflateX<0)
		{
			fInflateX=minimum(fInflateX,this->GetWidth()/2);
		}
		this->m_vecMax.x+=fInflateX;
		this->m_vecMin.x-=fInflateX;
		if (fInflateY<0)
		{
			fInflateY=minimum(fInflateY,this->GetHeight()/2);
		}
		this->m_vecMax.y+=fInflateY;
		this->m_vecMin.y-=fInflateY;
	};

	/// Deflates the rectangle by moving its borders inwards.
	/// Negative deflation corresponds to a inflation.
	/// @param fDeflateX Amount of deflation in x direction.
	/// @param fDeflateY Amount of deflation in y direction.
	inline void Deflate(float fDeflateX,float fDeflateY) {this->Inflate(-fDeflateX,-fDeflateY);};

	/// Scales the rectangle.
	/// @param fScale Scaling factor.
	inline void Scale(float fScale) {this->m_vecMin*=fScale;this->m_vecMax*=fScale;};

	/// Scales the rectangle.
	/// @param fScaleX Scaling factor in x direction.
	/// @param fScaleY Scaling factor in y direction.
	inline void Scale(float fScaleX,float fScaleY) {this->m_vecMin.x*=fScaleX;this->m_vecMax.x*=fScaleX;this->m_vecMin.y*=fScaleY;this->m_vecMax.y*=fScaleY;};


	// -----------------------------------------------------------------
	// rectangle test
	// -----------------------------------------------------------------

	/// Tests whether or not a point is inside the rectangle.
	/// @param vecTest Point/vector to be tested.
	/// @return Returns <i>true</i> if the point lies inside the
	/// rectangle otherwise <i>false</i>.
	inline bool		Inside(const CVector2& vecTest) const {return vecTest>=this->m_vecMin && vecTest<=this->m_vecMax;};

	/// Tests whether or not a rectangle is completely inside this rectangle.
	/// @param rcTest Rectangle to be tested.
	/// @return Returns <i>true</i> if the tested rectangle lies completely inside this
	/// rectangle otherwise <i>false</i>.
	inline bool		Inside(const CRect& rcTest) const {return this->Inside(rcTest.m_vecMin) && this->Inside(rcTest.m_vecMax);};

	/// Tests whether or not two rectangles overlap.
	/// @param rcTest Rectangle to be tested.
	/// @return Returns <i>true</i> if the two rectangles overlap,
	/// otherwise <i>false</i>.
	inline bool		Overlap(const CRect& rcTest) const {return this->Inside(rcTest.m_vecMin) || this->Inside(rcTest.m_vecMax) || rcTest.Inside(this->m_vecMin) || rcTest.Inside(this->m_vecMax);};

	/// Converts a point into local space of the rectangle.
	/// This means the position of the point with respect to the minimum
	/// coordinate of the rectangle
	/// @param vecConvert Point to be converted.
	inline void		ConvertToLocalSpace(CVector2& vecConvert) const {vecConvert-=this->m_vecMin;};


protected:

	// -----------------------------------------------------------------
	// private members of the rectangle
	// -----------------------------------------------------------------

	/// minimum coordinate of rectangle
	CVector2	m_vecMin;

	/// maximum coordinate of rectangle
	CVector2	m_vecMax;

};	// class CRect

};	// namespace pnpse

#endif // !defined(PNPSE_RECT_H_INCLUDED)
