/////////////////////////////////////////////////////////////////////////////////////
// pnpseVector2.h
// Definition of a 2 dimensional vector class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_VECTOR2_H_INCLUDED)
#define PNPSE_VECTOR2_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// base includes
#include "../utils.h"
#include <math.h>



// pnp sensory game engine
namespace pnpse	
{

/// Class to store 2 component (2D) vector.
class CVector2
{
public:

	// -----------------------------------------------------------------
	// construction of a vector
	// -----------------------------------------------------------------

	/// Constructs a vector without initialization.<br>
	/// Note: The vector is not initialized for efficiency reasons.
	inline	CVector2() 
		: x(0),y(0) {};

	/// Constructs and initializes a new vector.
	/// @param fX x component of new vector.
	/// @param fY y component of new vector.
	inline CVector2(const float& fX, const float& fY)
		: x(fX),y(fY) {};

	/// Constructs and initializes a new vector.
	/// @param pfCoords Pointer to an array of at least 2 real values.
	/// containing the coordinates of the vector.
	inline CVector2(const float* pfCoords)
		: x(pfCoords[0]),y(pfCoords[1]) {};


	// -----------------------------------------------------------------
	// Methods
	// -----------------------------------------------------------------

	/// Returns the vector's magnitude (length).
	/// @return magnitude (length) of vector.
	inline float	GetMagnitude() const	{return sqrt(this->GetMagnitudeSquared());};

	/// Returns the vector's squared magnitude (length).
	/// @return Squared magnitude (length) of vector.
	inline float	GetMagnitudeSquared() const	{return SQR(this->x)+SQR(this->y);};

	/// Normalizes the vector.
	inline void		MakeNormalized()
	{
		// get length
		float fLength=this->GetMagnitude();

		// devide all components by the length
		if (fLength!=0)
		{
			fLength=static_cast<float>(1)/fLength;
			(*this)*=fLength;
		}
	};

	/// Create a normalized version of this vector. 
	inline CVector2		Normalize() const
	{
		// create return vector
		CVector2 vecReturn(*this);
		vecReturn.MakeNormalized();

		// return result
		return vecReturn;
	}

	/// Computes the dot product with a second vector.
	/// @param vec Vector with which to compute the dot product.
	/// @return Returns the dot product between the two vectors.
	inline float		DotProduct(const CVector2& vec) const	{return this->x*vec.x+this->y*vec.y;};


	// -----------------------------------------------------------------
	// operators
	// -----------------------------------------------------------------

	/// Assignment operator.
	/// @param vec Vector to be assigned.
	/// @return Returns the assigned vector.
	inline CVector2&	operator = (const CVector2& vec) {this->x=vec.x;this->y=vec.y;return *this;};

	/// Assignment operator.
	/// @param prCoords Array of at least 2 real values to be assigned.
	/// @return Returns the assigned vector.
	inline CVector2&	operator = (float* pfCoords) {this->x=pfCoords[0];this->y=pfCoords[1];return *this;};

	/// Returns one element of the vector.
	/// @param idx Index of the vector's element to be returned.
	/// @return Value of the vector's element.
	inline float		operator [] (const int idx) const
	{
		// return value
		return *(&this->x+idx);
	};

	/// Returns one element of the vector.
	/// @param idx Index of the vector's element to be returned.
	/// @return The vector's element.
	inline float&		operator [] (const int idx)
	{
		// return value
		return *(&this->x+idx);
	};

	/// Compares equality.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the two vectors are equal.
	inline bool			operator == (const CVector2& vec) const {return this->x==vec.x && this->y==vec.y;};

	/// Compares greater.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the vector is greater.
	inline bool		operator > (const CVector2& vec) const {return this->x>vec.x && this->y>vec.y;};

	/// Compares less.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the vector is less.
	inline bool		operator < (const CVector2& vec) const {return this->x<vec.x && this->y<vec.y;};

	/// Compares greater or equal.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the vector is greater or equal.
	inline bool		operator >= (const CVector2& vec) const {return this->x>=vec.x && this->y>=vec.y;};

	/// Compares less or equal.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the vector is less or equal.
	inline bool		operator <= (const CVector2& vec) const {return this->x<=vec.x && this->y<=vec.y;};

	/// Compares inequality.
	/// @param vec Vector to be tested.
	/// @return Returns <i>true</i> or <i>false</i>, whether
	/// or not the two vectors are not equal.
	inline bool			operator != (const CVector2& vec) const {return this->x!=vec.x || this->y!=vec.y;};

	/// Adds two vectors.
	/// @param vec Vector to be added.
	inline CVector2&	operator += (const CVector2& vec) {this->x+=vec.x;this->y+=vec.y;return *this;};

	/// Adds two vectors.
	/// @param vec Vector to be added.
	/// @return Returns the two added vectors.
	inline CVector2		operator + (const CVector2& vec) const {return CVector2(this->x+vec.x,this->y+vec.y);};

	/// Subtracts two vectors.
	/// @param vec Vector to be subtracted.
	inline CVector2&	operator -= (const CVector2& vec) {this->x-=vec.x;this->y-=vec.y;return *this;};

	/// Subtracts two vectors.
	/// @param vec Vector to be subtracted.
	/// @return Returns the result of two subtracted vectors.
	inline CVector2		operator - (const CVector2& vec) const {return CVector2(this->x-vec.x,this->y-vec.y);};

	/// Multiplies two vectors.
	/// @param vec Vector to be multiplied.
	inline CVector2&	operator *= (const CVector2& vec) {this->x*=vec.x;this->y*=vec.y;return *this;};

	/// Multiplies two vectors.
	/// @param vec Vector to be multiplied.
	/// @return Returns the result of two multiplied vectors.
	inline CVector2		operator * (const CVector2& vec) const {return CVector2(this->x*vec.x,this->y*vec.y);};

	/// Scalar multiply a vector.
	/// @param scalar Scalar value which should be multiplied with the vector.
	inline CVector2&	operator *= (float scalar) {this->x*=scalar;this->y*=scalar;return *this;};

	/// Scalar multiply a vector.
	/// @param scalar Scalar value which should be multiplied with the vector.
	/// @return Returns the result of the multiplication.
	inline CVector2		operator * (float scalar) const {return CVector2(this->x*scalar,this->y*scalar);};

	/// Scalar multiply a vector.
	/// @param scalar Scalar value which should be multiplied with the vector.
	/// @param vec The vector to be multiplied.
	inline friend CVector2 operator * (float scalar, const CVector2& vec) {return CVector2(vec.x*scalar,vec.y*scalar);};

	/// Divides two vector.
	/// @param vec Vector to be divided.
	inline CVector2&	operator /= (const CVector2& vec) {this->x/=vec.x;this->y/=vec.y;return *this;};

	/// Divides two vectors.
	/// @param vec Vector to be divided.
	/// @return Returns the result of two divided vectors.
	inline CVector2		operator / (const CVector2& vec) const {return CVector2(this->x/vec.x,this->y/vec.y);};

	/// Scalar divide a vector.
	/// @param scalar Scalar value by which the vector should be divided.
	inline CVector2&	operator /= (float scalar) {this->x/=scalar;this->y/=scalar;return *this;};

	/// Scalar divide a vector.
	/// @param scalar Scalar value by which the vector should be divided.
	/// @return Returns the result of the division.
	inline CVector2		operator / (float scalar) const {return CVector2(this->x/scalar,this->y/scalar);};

	/// Scalar divide a vector.
	/// @param scalar Scalar value which should be divided by the vector.
	/// @param vec The vector to be divided.
	inline friend CVector2 operator / (float scalar, const CVector2& vec) {return CVector2(vec.x/scalar,vec.y/scalar);};

	/// Conversion operator.
	/// @return Returns a pointer to an array of real values
	/// for the vector's components
	inline operator	float*() const {return const_cast<float*>(&this->x);};


	// -----------------------------------------------------------------
	// public members of the vector
	// -----------------------------------------------------------------

	/// Vector's x component
	float	x;

	/// Vector's y component
	float	y;

};	// class CVector2


};	// namespace pnpse


#endif // !defined(PNPSE_VECTOR2_H_INCLUDED)