/////////////////////////////////////////////////////////////////////////////////////
// Utils.cpp
// various utility methods to support common plugin functionality.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  20. Nov. 2005 : Changed the way, the extension is removed from a file name =>
//                  Correct ogre xml mesh file extension ".mesh.xml" (Ralf Westphal)
//  10. Oct. 2006 : Achieved VS2005 compatibility, made compilation easier.
//  24. Jul. 2007 : Hack for ".mesh.xml"
//  19. Aug. 2007 : Added ReplaceForbiddenCharacters(), ToLower(), ToUpper()
//					AddTrailingBackslash(), AddTrailingSlash(),
//					ConvertToUnixFormat(), and ConvertToWindowsFormat()
//  13. Dec. 2007 : Added GetFileName(), RelativeToAbsolutePath(),
//					RemoveTrailingBackslash(), and RemoveTrailingSlash()
//  12. Sep. 2008 : Added ReplaceFileExtension()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "utils.h"

#include "windows.h"	// The Windows PlatformSDK has to be installed in order to compile PlugIns
#include <direct.h> 
#include <algorithm>

//#define LEGACY_DIR

#if !defined(LEGACY_DIR)
#include <Dbghelp.h>
#endif


/// extract the path from a string containing path and file name
std::string GetPath(const std::string& strFilePath)
{
	// extract path
	static std::string strFileName;
	static std::string strPath;
	RemoveFilePath(strFilePath,strFileName,strPath);

	// return path
	return strPath;
}

/// extract the file name from a string containing path and file name
std::string GetFileName(const std::string& strFilePath)
{
	// extract path
	static std::string strFileName;
	static std::string strPath;
	RemoveFilePath(strFilePath,strFileName,strPath);

	// return path
	return strFileName;
}

// creates the absolute path from a relative and a base path
std::string RelativeToAbsolutePath(const std::string& strBasePath,const std::string& strRelativePath)
{
	// return the number of folders to be stepped back
	bool bFoundLastPart=false;
	unsigned int nPart=0;
	unsigned int nRemoveChars=0;
	unsigned int nGoBack=0;
	while (!bFoundLastPart)
	{	
		// get that part of the relative path
		std::string strPart=GetPathPart(strRelativePath,nPart);
		if (strPart==".." || strPart==".")
		{
			// go back?
			if (strPart=="..")
			{
				nGoBack++;
			}

			// increment number of characters to be removed
			nRemoveChars+=static_cast<unsigned int>(strPart.size())+1;
		}
		else
		{
			bFoundLastPart=true;
		}

		// next part
		nPart++;
	}

	// now get the final relative part
	std::string strPart2=strRelativePath;
	if (nRemoveChars>0)
	{
		strPart2=strRelativePath.substr(nRemoveChars);
	}

	// go back a number of folders from the absolute base path
	std::string strPart1=strBasePath;	
	for (unsigned int nBack=0;nBack<nGoBack;++nBack)
	{
		RemoveTrailingBackslash(strPart1);
		strPart1=GetPath(strPart1);
	}

	// concatenate both parts
	AddTrailingBackslash(strPart1);
	return strPart1+strPart2;
}

/// extracts the file extension from a given filename
void RemoveFileExtension(const std::string& strSource,std::string& strFilename,std::string& strExtension)
{
	// first extract filename from path
	std::string strPath;
	RemoveFilePath(strSource,strFilename,strPath);

	// check for ogre ".mesh.xml" extension
	if (strSource.length()>9)
	{
		if (EqualNoCase(Right(strSource,9),".mesh.xml"))
		{
			// ogre xml mesh extension
			strExtension=Right(strSource,8);
			strFilename=Left(strSource,static_cast<int>(strSource.length())-9);
			return;
		}
	}

	// standard extension
	if (strFilename.length()>0)
	{
		for (int iLeft=0,iRight=(int)strFilename.length()-1;iRight>=0;iLeft++,iRight--)
		{
			if (strFilename.at(iRight)=='.')
			{
				// seperate extension and file name
				strExtension=strFilename.substr(minmax(static_cast<int>(strFilename.length())-iLeft,0,static_cast<int>(strFilename.length())-1),iLeft);
				strFilename=strFilename.substr(0,iRight);
				break;
			}
			else if (strFilename.at(iRight)=='\\')
			{
				// no extension found
				strFilename=strFilename;
				strExtension="";
				break;
			}
		}
	}
}

/// extracts the path from given filename including path.
void RemoveFilePath(const std::string& strSource,std::string& strFilename,std::string& strPath)
{
    char szPath[MAX_PATH];

    szPath[0] = '\0';
    int nLength = (int)strSource.length( );
    for (int i = nLength - 1; i >= 0; i--)
	{
        if (strSource.at(i) == '/' || strSource.at(i) == '\\')
        {
			int j=0;
            for (j = 0; j < i + 1; j++)
			{
                szPath[j] = strSource.at(j);
			}
            szPath[j] = '\0';
            break;
		}
    }

	// set return parameters
	strPath=szPath;
	strFilename=strSource.substr(strPath.length(),max(nLength-strPath.length(),0));
}

std::string ReplaceFileExtension(const std::string& strFileName,const std::string& strExtension)
{
	// remove old extension
	std::string strNewFileName;
	std::string strOldExtension;
	RemoveFileExtension(strFileName,strNewFileName,strOldExtension);

	// return new file name
	return GetPath(strFileName)+strNewFileName+"."+strExtension;
}

/// replace a string sequence within a given string with another string
int Replace(std::string* strWork,const std::string& strFind,const std::string& strReplace)
{
	// number of replacements
	int iReplaced=0;

	// find all strings
	std::string::size_type nPos=0;
	while ( (nPos=strWork->find(strFind,nPos))!=std::string::npos )
	{
		// replace string
		strWork->replace(nPos,strFind.size(),strReplace);
		nPos++;
		iReplaced++;
	}

	// return number of replacements
	return iReplaced;
}

/// trims whitespaces to the left of a given string.
size_t TrimLeft(std::string& strTrim)
{
	// walk from the beginning of the string to the first character not to be trimmed
	int iPos;
	for (iPos=0;iPos<(int)strTrim.length();iPos++)
	{
		switch (strTrim.at(iPos))
		{
		case ' ':
		case '\t':
			break;
		default:
			goto found;
			break;
		}
	}

found:

	// trim the string
	if (iPos>0)
	{
		strTrim=strTrim.substr(iPos,strTrim.length());
	}

	// return number of trimmed characters
	return iPos;
}

/// trims whitespaces to the right of a given string.
size_t TrimRight(std::string& strTrim)
{
	// walk from the end of the string to the first character not to be trimmed
	int iPos;
	for (iPos=static_cast<int>(strTrim.length())-1;iPos>=0;iPos--)
	{
		switch (strTrim.at(iPos))
		{
		case ' ':
		case '\t':
			break;
		default:
			goto found;
			break;
		}
	}

found:

	// trim the string
	int iTrimmed=0;
	if (iPos<0)
	{
		iTrimmed=static_cast<int>(strTrim.length());
		strTrim="";
	}
	else if (iPos<static_cast<int>(strTrim.length())-1)
	{
		iTrimmed=static_cast<int>(strTrim.length())-iPos-1;
		strTrim=strTrim.substr(0,iPos+1);
	}

	// return number of trimmed characters
	return iTrimmed;
}

/// trims whitespaces to the left and right of a given string.
size_t Trim(std::string& strTrim)
{
	// trim to both sides
	return TrimLeft(strTrim)+TrimRight(strTrim);
}

/// returns a number of characters from the right of a given string.
std::string Right(const std::string& str,unsigned int nCount)
{
	// check string length
	if (str.length()<nCount)
	{
		return str;
	}


	// return right of string
	return str.substr(max(str.length()-nCount,0),nCount);
}

/// returns a number of characters from the left of a given string.
std::string Left(const std::string& str,unsigned int nCount)
{
	// return left of string
	return str.substr(0,nCount);
}

/// Divides a string
bool Divide(const std::string& strSource,const std::string& strDivide,std::string& strLeft,std::string& strRight)
{
	// try to find dividing string
	const char* cDivide=strstr(strSource.c_str(),strDivide.c_str());
	if (cDivide!=NULL)
	{
		// found it => separate string
		int iPos=static_cast<int>(cDivide-strSource.c_str());
		strLeft=Left(strSource,iPos);
		strRight=Right(strSource,max(static_cast<int>(strSource.length())-iPos-1,0));

		// ok
		return true;
	}
	else
	{
		// not found
		return false;
	}
}

/// compare two strings ignoring their case
bool EqualNoCase(const std::string& str1,const std::string& str2)
{
	// compare the strings
	return (_stricmp(str1.c_str(),str2.c_str())==0);
}

/// converts an integer into a string.
std::string	ToString(int iValue)
{
	// convert string
	static char cTemp[100];
	_itoa(iValue,cTemp,10);

	// return string
	return std::string(cTemp);
}

/// converts a float into a string.
std::string	ToString(float fValue)
{
	// convert string
	static char cTemp[100];
	sprintf(cTemp,"%f",fValue);

	// return string
	return std::string(cTemp);
}

std::string ToLower(std::string& strConvert)
{
	// transform the string to lower cases
	transform(strConvert.begin(), strConvert.end(), strConvert.begin(), tolower);
	return strConvert;
}

std::string ToUpper(std::string& strConvert)
{
	// transform the string to upper cases
	transform(strConvert.begin(), strConvert.end(), strConvert.begin(), toupper);
	return strConvert;
}

bool AddTrailingBackslash(std::string& strWork)
{
	// test right most character
	if (!EqualNoCase(Right(strWork,1),"\\"))
	{
		// no backslash => add it
		strWork+="\\";
		return true;
	}
	else
	{
		// already ends with a backslash
		return false;
	}
}

bool AddTrailingSlash(std::string& strWork)
{
	// test right most character
	if (!EqualNoCase(Right(strWork,1),"/"))
	{
		// no slash => add it
		strWork+="/";
		return true;
	}
	else
	{
		// already ends with a slash
		return false;
	}
}

bool RemoveTrailingBackslash(std::string& strWork)
{
	// test right most character
	if (EqualNoCase(Right(strWork,1),"\\"))
	{
		// backslash => remove it
		strWork=Left(strWork,static_cast<int>(strWork.size())-1);
		return true;
	}
	else
	{
		// already ends with a backslash
		return false;
	}
}

bool RemoveTrailingSlash(std::string& strWork)
{
	// test right most character
	if (EqualNoCase(Right(strWork,1),"/"))
	{
		// backslash => remove it
		strWork=Left(strWork,static_cast<int>(strWork.size())-1);
		return true;
	}
	else
	{
		// already ends with a backslash
		return false;
	}
}

std::string	ConvertToUnixFormat(const std::string& strPath)
{
	// convert backslashes to slashes
	std::string strReturn(strPath);
	Replace(&strReturn,"\\","/");
	return strReturn;
}

std::string	ConvertToWindowsFormat(const std::string& strPath)
{
	// convert slashes to backslashes 
	std::string strReturn(strPath);
	Replace(&strReturn,"/","\\");
	return strReturn;
}

int ReplaceForbiddenCharacters(std::string* strWork,const std::string& strFind,const std::string& strReplace)
{
	// number of replaced characters
	int iReplaced=0;

	// go through the whole string
	if (strWork->length()>0)
	{
		for (int iPos=0;iPos<static_cast<int>(strWork->length());iPos++)
		{
			if (strFind.find(strWork->at(iPos))==std::string::npos)
			{
				// seperate extension and file name
				strWork->replace(iPos,1,Left(strReplace+"_",1));
				iReplaced++;
			}
		}
	}

	// return number of replaced characters
	return iReplaced;
}

/// check if directory exists
bool DirExists(const std::string& strSearch)
{
	// get current drive
	int iCurDrive=_getdrive();
	char path[_MAX_PATH];
	_getcwd(path, _MAX_PATH);
	std::string strCurDir=path;
	std::string strPath=strSearch;

	// check input
	if ( strSearch.substr(1,2)!=std::string(":\\") )
	{
		// no valid path
		return false;
	}

	// set drive
	std::transform(strPath.begin(),strPath.end(),strPath.begin(),toupper);  
	int iDestDrive=(int)(strPath.at(0)-'A'+1);
	if ( _chdrive(iDestDrive)!=0 )
	{
		// error
		return false;
	}
	// change into root directors
	_chdir("\\");

	// check all parts of the directory structure
	int iPart=1;
	std::string strActPath=GetPathPart(strPath,iPart);
	while ( strActPath.length()!=0 )
	{
		// test this part of the directory
		if (_chdir(strActPath.c_str())!=0)
		{
			// does not exist
			return false;
		}

		// check next path of the directory
		iPart++;
		strActPath=GetPathPart(strPath,iPart);
	}

	// restore current path and dir
	_chdrive(iCurDrive);
	_chdir(strCurDir.c_str());

	// OK
	return true;
}

/// creates directory
bool CreateDir(const std::string& strCreate)
{
#if !defined(LEGACY_DIR)
	// try to create that directory
	return MakeSureDirectoryPathExists(strCreate.c_str())!=0;
/*	int iResult=SHCreateDirectory(NULL,CA2W(strCreate.c_str()));
	return  iResult==ERROR_ALREADY_EXISTS || 
			iResult==ERROR_SUCCESS;*/
#else
	// get current drive
	int iCurDrive=_getdrive();
	char path[_MAX_PATH];
	_getcwd(path, _MAX_PATH);
	std::string strCurDir=path;
	std::string strPath=strCreate;

	// check input
	if ( strCreate.substr(1,2)!=std::string(":\\") )
	{
		// no valid path
		return false;
	}

	// set drive
	std::transform(strPath.begin(),strPath.end(),strPath.begin(),toupper);  
	int iDestDrive=(int)(strPath.at(0)-'A'+1);
	if ( _chdrive(iDestDrive)!=0 )
	{
		// error
		return false;
	}
	// change into root directors
	_chdir("\\");

	// check all parts of the directory structure
	int iPart=1;
	std::string strActPath=GetPathPart(strPath,iPart);
	while ( strActPath.length()!=0 )
	{
		// test this part of the directory
		if (_chdir(strActPath.c_str())!=0)
		{
			// directory does not exists => create it
			if (_mkdir(strActPath.c_str())!=0)
			{
				// Fehler
				return false;
			}
			else
			{
				// switch into this directory
				if (_chdir(strActPath.c_str())!=0)
				{
					// Fehler
					return false;
				}
			}
		}

		// test next part of path
		iPart++;
		strActPath=GetPathPart(strPath,iPart);
	}

	// restore current path and dir
	_chdrive(iCurDrive);
	_chdir(strCurDir.c_str());

	// OK
	return true;
#endif
}

std::string GetPathPart(const std::string& strPath,int iPart)
{
	int iPos=0;
	int iLastPos=0;

	// find first back slash
	iPos=static_cast<int>(strPath.find("\\",iLastPos));
	if (iPos!=0)
	{
		// found
		int iCurPart=0;
		while (iCurPart<=iPart && iPos>0)
		{
			if (iCurPart==iPart)
			{
				// this is the part we searched for
				return strPath.substr(iLastPos,iPos-iLastPos);
			}
			iLastPos=iPos+1;
			iCurPart++;
			iPos=static_cast<int>(strPath.find("\\",iLastPos));
			if (iPos==-1 && iLastPos<static_cast<int>(strPath.length()))
			{
				iPos=static_cast<int>(strPath.length());
			}
		}
		// not found
		return "";
	}
	else
	{
		// not found => no further parts exist
		if (iPart==0)
		{
			// the first one was the one to search for
			return strPath;
		}
		else
		{
			// not found
			return "";
		}
	}
}


void GlobalVectorTransform(PnPTC3DVec& vecTransform,float fScale,int iUpRotation,int iFlipAxis)
{
	// scale and flip values
	vecTransform.x*=fScale*((iFlipAxis==FLIP_XAXIS)?-1.f:1.f);
	vecTransform.y*=fScale*((iFlipAxis==FLIP_YAXIS)?-1.f:1.f);
	vecTransform.z*=fScale*((iFlipAxis==FLIP_ZAXIS)?-1.f:1.f);

	// perform up rotation
	switch (iUpRotation)
	{
	case UPROTATION_ROTX90:
		{
			float fTemp=vecTransform.z;
			vecTransform.z=vecTransform.y;
			vecTransform.y=fTemp*-1.f;
		}
		break;
	case UPROTATION_ROTX270:
		{
			float fTemp=vecTransform.z;
			vecTransform.z=vecTransform.y*-1.f;;
			vecTransform.y=fTemp;
		}
		break;
	case UPROTATION_ROTX180:
		{
			vecTransform.y*=-1.f;
			vecTransform.z*=-1.f;
		}
		break;
	}
}

bool TextureSupportsAlpha(const std::string& strTexture)
{
	// get file extension
	std::string strFileName;
	std::string strExtension=strTexture;
	if (strTexture.find(".")!=std::string::npos)
	{
		RemoveFileExtension(strTexture,strFileName,strExtension);
	}
	return EqualNoCase(strExtension,"gif") || EqualNoCase(strExtension,"png") || EqualNoCase(strExtension,"dds");
}
