/////////////////////////////////////////////////////////////////////////////////////
// Utils.h
// various utility methods to support common plugin functionality.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  29. Mai  2007 :	Added TextureSupportsAlpha() and GlobalVectorTransform()
//  19. Aug. 2007 : Added ReplaceForbiddenCharacters(), ToLower(), ToUpper()
//					AddTrailingBackslash(), AddTrailingSlash(),
//					ConvertToUnixFormat(), and ConvertToWindowsFormat()
//  13. Dec. 2007 : Added GetFileName(), RelativeToAbsolutePath(),
//					RemoveTrailingBackslash(), and RemoveTrailingSlash()
//  12. Sep. 2008 : Added ReplaceFileExtension()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_UTILS_H_INCLUDED)
#define PNPTC_UTILS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define _CRT_SECURE_NO_DEPRECATE 1

// ----------------------------------------------------------------
// global includes
// ----------------------------------------------------------------

#include <string>
#include "../../includes/IPnPTCPlugIn.h"



// ----------------------------------------------------------------
// global helper methods for file names and paths
// ----------------------------------------------------------------

/// extract the path from a string containing path and file name.
/// @param strFilePath filename including its path from which
/// the path should be extracted.
/// @return returns the path of the given file.
std::string GetPath(const std::string& strFilePath);

/// extract the file name from a string containing path and file name.
/// @param strFilePath filename including its path from which
/// the file name should be extracted.
/// @return returns the file name of the given path and file.
std::string GetFileName(const std::string& strFilePath);

/// Converts a relative path to an absolute path, given a base path 
/// @param strBasePath The base path, relative to which the relative
/// path should be applied.
/// @param strRelativePath The relative path to be resolved to
/// an absolute path.
/// @return Returns the absolute path derived from the
/// base path and the relative path.
std::string RelativeToAbsolutePath(const std::string& strBasePath,const std::string& strRelativePath);

/// extracts the file extension from a given filename.
/// @param strSource source file name including file extension.
/// @param strFilename (output) the file name without extension and "." is 
/// returned by this parameter.
/// @param strExtension (output) the file extension without "." is 
/// returned by this parameter.
void RemoveFileExtension(const std::string& strSource,std::string& strFilename,std::string& strExtension);

/// extracts the path from given filename including path.
/// @param strSource source file name including path.
/// @param strFilename (output) the file name without the path is returned by this parameter.
/// @param strPath (output) the path without "." is returned by this parameter.
void RemoveFilePath(const std::string& strSource,std::string& strFilename,std::string& strPath);

/// returns one part of a given directory structure string.
/// @param strPath complete directory structure string.
/// @param iPart part of the structure to be returned
/// @return returns the requested directory part if successfull
/// or an empty string in case of an error.
std::string GetPathPart(const std::string& strPath,int iPart);

/// Adds a trailing backslash "\" to a string, if it no already ends with a backslash.
/// @param strWork The string which should contain a trailing backslash.
/// @return Returns <i>true</i> or <i>false</i>, whether or not
/// a backslash was added to the string.
bool		AddTrailingBackslash(std::string& strWork);

/// Adds a trailing slash "/" to a string, if it no already ends with a slash.
/// @param strWork The string which should contain a trailing slash.
/// @return Returns <i>true</i> or <i>false</i>, whether or not
/// a slash was added to the string.
bool		AddTrailingSlash(std::string& strWork);

/// Removes a trailing backslash "\" from a string.
/// @param strWork The string from which  a trailing backslash should be removed.
/// @return Returns <i>true</i> or <i>false</i>, whether or not
/// a backslash was removed from the string.
bool		RemoveTrailingBackslash(std::string& strWork);

/// Removes a trailing slash "/" from a string.
/// @param strWork The string from which  a trailing slash should be removed.
/// @return Returns <i>true</i> or <i>false</i>, whether or not
/// a slash was removed from the string.
bool		RemoveTrailingSlash(std::string& strWork);

/// Converts a path to unix/linux format.
/// @param strPath The path/file name to be converted.
/// @return Returns a unix/linux conform path/file name string.
std::string	ConvertToUnixFormat(const std::string& strPath);

/// Converts a path to windows format.
/// @param strPath The path/file name to be converted.
/// @return Returns a windows conform path/file name string.
std::string	ConvertToWindowsFormat(const std::string& strPath);

/// Replaces the extension of a file name.
/// @param strFileName The name of the file (maybe including path),
/// whichs extension should be replaced.
/// @param strExtension The new extension of the file name.
/// @retur Returns the file name with the replaced extension.
std::string ReplaceFileExtension(const std::string& strFileName,const std::string& strExtension);


// ----------------------------------------------------------------
// global helper methods for directory management
// ----------------------------------------------------------------

/// checks whether or not a directory structure exists.
/// @param strSearch directory structure to be searched for.
/// @return returns <i>true</i> or <i>false</i>, whether
/// or not the directory already exists.
bool DirExists(const std::string& strSearch);

/// ensures that a given directy exists, by creating
/// the whole directory structure if necessary.
/// @param strCreate directory structure to be created.
/// @return returns <i>true</i> or <i>false</i>, whether
/// or not the directory could be created.
bool CreateDir(const std::string& strCreate);


// ----------------------------------------------------------------
// global helper methods for working with strings
// ----------------------------------------------------------------

/// replace a string sequence within a given string with another string.
/// @param strWork (in/out) this string will be searched and edited during
/// the execution of the method.
/// @param strFind string to search for.
/// @param strReplace string to replace the searched string.
/// @return Returns the number of string replacements.
int Replace(std::string* strWork,const std::string& strFind,const std::string& strReplace);

/// replace all characters in a string which are not within a second string of
/// all allowed characters.
/// @param strWork (in/out) this string will be searched and edited during
/// the execution of the method.
/// @param strAllowedCharacters The string of all allowed characters.
/// @param strReplace The string, with which all forbidden characters
/// are to be substantiated.
/// @return Returns the number of string replacements.
int ReplaceForbiddenCharacters(std::string* strWork,const std::string& strFind,const std::string& strReplace);

/// trims whitespaces to the left of a given string.
/// tab and ' ' are recognized as whitespaces.
/// @param strTrim (in/out) string to be trimmed.
/// @return returns the number of characters that have beend removed.
size_t TrimLeft(std::string& strTrim);

/// trims whitespaces to the right of a given string.
/// tab and ' ' are recognized as whitespaces.
/// @param strTrim (in/out) string to be trimmed.
/// @return returns the number of characters that have beend removed.
size_t TrimRight(std::string& strTrim);

/// trims whitespaces to the left and right of a given string.
/// tab and ' ' are recognized as whitespaces.
/// @param strTrim (in/out) string to be trimmed.
/// @return returns the number of characters that have beend removed.
size_t Trim(std::string& strTrim);

/// compares two strings if they are equal ignoring the case.
/// @param str1 first string to be compared.
/// @param str2 second string to be compared.
/// @return returns <i>true</i> if both strings are equal, otherwise
/// <i>false</i>.
bool	EqualNoCase(const std::string& str1,const std::string& str2);

/// converts an integer into a string.
/// @param iValue integer value to be converted.
/// @return returns the value as string.
std::string	ToString(int iValue);

/// converts a float into a string.
/// @param fValue integer value to be converted.
/// @return returns the value as string.
std::string	ToString(float fValue);

/// returns a number of characters from the right of a given string.
/// @param str source string.
/// @param nCount number of characters to be returned from the right.
/// @return returns the iCount number of characters from the
/// right of str.
std::string Right(const std::string& str,unsigned int nCount);

/// returns a number of characters from the left of a given string.
/// @param str source string.
/// @param nCount number of characters to be returned from the left.
/// @return returns the iCount number of characters from the
/// left of str.
std::string Left(const std::string& str,unsigned int nCount);

/// Divides a string into two sub string at the first
/// occurance of a given seperator string.
/// @param strSource The source string which should be divided.
/// @param strDivide The dividing string.
/// @param strLeft (output) The string left to the dividing string.
/// @param strRight (output) The string right to the dividing string.
/// @return Returns <i>true</i> or <i>false</i>, whether or not the
/// dividing string could be found. If it wasn't found, strLeft and
/// strRight will be unchanged.
bool		Divide(const std::string& strSource,const std::string& strDivide,std::string& strLeft,std::string& strRight);

/// Converts a string to lower cases.
/// @param strConvert The string to be converted.
/// This string will contain the lower case conversion,
/// when the function returns from execution.
/// @return Returns the lower case version of the string, 
/// so the method can be used as input for other methods.
std::string	ToLower(std::string& strConvert);

/// Converts a string to upper cases.
/// @param strConvert The string to be converted.
/// This string will contain the upper case conversion,
/// when the function returns from execution.
/// @return Returns the lower case version of the string, 
/// so the method can be used as input for other methods.
std::string	ToUpper(std::string& strConvert);


// ----------------------------------------------------------------
// 3D/Graphics methods
// ----------------------------------------------------------------


/// Affects a 3D vector by a given global transformation.
/// @param vecTransform The vector, which should be transformed.
/// @param fScale A scaling factor, which will be applied to all vertex
/// positions during export.
/// @param iUpRotation A rotation, which will be applied to all vertices
/// in order to correclty align the object's up axis. Possible values are
/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
/// @param iFlipAxis The axis along which all vertex positions are negated
/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
void		GlobalVectorTransform(PnPTC3DVec& vecTransform,float fScale,int iUpRotation,int iFlipAxis);

/// Tests, whether or not a given texture format supports alpha blending.
/// @param strTexture The name of the texture file to be tested.
/// @return Returns <i>true</i> if the texture file format supports
/// alpha blending, otherwise <i>false</i>.
bool		TextureSupportsAlpha(const std::string& strTexture);



// ----------------------------------------------------------------
// global helpers for pressure sensitive graphics tablets.
// ----------------------------------------------------------------


/// Modifies a given base parameter for a filter (for example) by
/// a pressure value which is specified by a pressure sensitive graphics tablet.
/// @param tBaseValue The base value which should be affected by the pressure value.
/// @param fPressure The pressure value.
/// @param tMinValue The minimum acceptable value.
/// @param tMaxValue The maximum acceptable value.
/// @return Returns the parameter value modified by the pressure.
/// The return value is in the range
/// [max(tBaseValue/2,tMinValue)..min(tBaseValue+tBaseValue/2,tMaxValue)]
template <class T>
T PressureMultiply(T tBaseValue,float fPressure,T tMinValue,T tMaxValue)
{
	// return value should be in the range [tBaseValue/2..tBaseValue+tBaseValue/2]
	return minmax(static_cast<T>(static_cast<float>(tBaseValue)/2.f+(minmax(fPressure,0.f,2.f)*tBaseValue)/2.f),tMinValue,tMaxValue);
}



// ----------------------------------------------------------------
// global defines
// ----------------------------------------------------------------


/// PI
#if !defined(PI)
#define PI	3.1415926535897932384626433832795
#endif

/// NULL
#if !defined(NULL)
#define NULL 0
#endif



/// minimum
template <typename T>
inline const T& minimum(const T& t1,const T& t2) {return (t1<t2)?t1:t2;};

/// maximum
template <typename T>
inline const T& maximum(const T& t1,const T& t2) {return (t1>t2)?t1:t2;};

/// min/max cap.
/// returns the value x capped to the range between min_value and max_value.
template <typename T>
inline const T& minmax(const T& x,const T& min_value,const T& max_value)	{return minimum(maximum(x,min_value),max_value);};

/// conversion from rad to degrees
template <typename T>
inline const T RAD2DEG(const T& rad) {return rad/static_cast<T>(PI)*static_cast<T>(180.);};

/// conversion from degrees to rad
template <typename T>
inline const T DEG2RAD(const T& deg) {return deg/static_cast<T>(180.)*static_cast<T>(PI);};

/// abs value.
template <typename T>
inline T ABS(const T& t) {return (t>=0)?t:-t;};

/// returns the sign of a value.
template <typename T>
inline T SGN(const T& t) {return (t>=0)?1:-1;};

/// compute the square of a value.
template <typename T>
inline T SQR(const T& t) {return t*t;};

/// swap two values
template <typename T>
inline void SWAP(T& t1,T& t2){T temp=t1;t1=t2;t2=temp;}



/// deletes an object created with new.
#if !defined(SAFE_DELETE)
#define SAFE_DELETE(p)		{ if(p) { delete (p);		(p)=NULL; } }
#endif

/// deletes an array of objects created with new[].
#if !defined(SAFE_DELETEARRAY)
#define SAFE_DELETEARRAY(p)	{ if(p) { delete[] (p);     (p)=NULL; } }
#endif

/// frees memory which has been allocated by malloc.
#if !defined(SAFE_FREE)
#define SAFE_FREE(p)		{ if(p) { free(p);          (p)=NULL; } }
#endif

/// releases reference counted objects like COM objects.
#if !defined(SAFE_RELEASE)
#define SAFE_RELEASE(p)		{ if(p) { (p)->Release();	(p)=NULL; } }
#endif


#endif // !defined(PNPTC_UTILS_H_INCLUDED)